// This is a TextMate grammar distributed by `starry-night`.
// This grammar is developed at
// <https://github.com/Alhadis/language-etc>
// and licensed `isc`.
// See <https://github.com/wooorm/starry-night> for more info.
/** @type {import('../lib/index.js').Grammar} */
const grammar = {
  extensions: ['.sfv'],
  names: ['simple-file-verification', 'sfv'],
  patterns: [
    {
      begin: '\\A(?=; Generated by RHash v\\d)',
      end: '(?=A)B',
      name: 'meta.rhash-output.sfv',
      patterns: [{include: '#rhash'}, {include: '#main'}]
    },
    {
      begin: '^(?=[a-fA-F0-9]+ \\*pocorgtfo\\d{2}\\.pdf\\s*$)',
      end: '(?=A)B',
      name: 'text.checksums',
      patterns: [{include: 'text.checksums'}]
    },
    {include: '#main'}
  ],
  repository: {
    checksum: {
      captures: {
        1: {name: 'string.quoted.double.filename.sfv'},
        2: {name: 'punctuation.definition.string.begin.sfv'},
        3: {name: 'punctuation.definition.string.end.sfv'},
        4: {name: 'string.unquoted.filename.sfv'},
        5: {name: 'constant.numeric.integer.int.hexadecimal.hex.crc32.sfv'}
      },
      match: '^\\s*(?:((")[^"]*("))|(\\S.*?))\\s+([a-fA-F0-9]{8})(?=\\s*$)',
      name: 'meta.crc32.checksum.sfv'
    },
    comment: {
      patterns: [
        {
          begin: '^\\s*(;)((!)SFV_COMMENT_START)[ \\t]*$',
          beginCaptures: {
            1: {name: 'punctuation.definition.comment.sfv'},
            2: {name: 'keyword.control.comment.begin.sfv'},
            3: {name: 'punctuation.definition.keyword.sfv'}
          },
          end: '^\\s*(;)((!)SFV_COMMENT_END)[ \\t]*$',
          endCaptures: {
            1: {name: 'punctuation.definition.comment.sfv'},
            2: {name: 'keyword.control.comment.end.sfv'},
            3: {name: 'punctuation.definition.keyword.sfv'}
          },
          name: 'comment.block.sfv'
        },
        {
          begin: '^\\s*(;)',
          beginCaptures: {1: {name: 'punctuation.definition.comment.sfv'}},
          end: '$',
          name: 'comment.line.semicolon.sfv'
        }
      ]
    },
    main: {
      patterns: [
        {include: '#comment'},
        {include: '#checksum'},
        {include: '#rhash'}
      ]
    },
    rhash: {
      captures: {
        1: {name: 'string.unquoted.filename.sfv'},
        2: {
          patterns: [
            {
              match: '\\b[a-zA-Z2-7]+\\b',
              name: 'constant.numeric.base32.rfc4648.sfv'
            },
            {
              match: '\\b[a-fA-F0-9]+\\b',
              name: 'constant.numeric.integer.int.hexadecimal.hex.sfv'
            }
          ]
        }
      },
      match:
        '(?x) ^ \\s* (\\S.*?(?<=\\S)) ((?: \\s+ (?: [a-fA-F0-9]{8} |   [a-fA-F0-9]{32} |   [a-fA-F0-9]{40} |   [a-fA-F0-9]{48} |   [a-fA-F0-9]{56} |   [a-fA-F0-9]{64} |   [a-fA-F0-9]{96} |   [a-fA-F0-9]{128} |   [a-zA-Z2-7]{32} |   [a-zA-Z2-7]{39} ) )+) (?=\\s*$)',
      name: 'meta.checksum-record.sfv'
    }
  },
  scopeName: 'source.sfv'
}

export default grammar
