// This is a TextMate grammar distributed by `starry-night`.
// This grammar is developed at
// <https://github.com/Alhadis/language-emacs-lisp>
// and licensed `isc`.
// See <https://github.com/wooorm/starry-night> for more info.
/** @type {import('../lib/index.js').Grammar} */
const grammar = {
  extensions: ['.el', '.emacs', '.emacs.desktop'],
  names: ['emacs-lisp', 'elisp', 'emacs'],
  patterns: [
    {
      begin: '\\A(#!)',
      beginCaptures: {
        1: {name: 'punctuation.definition.comment.hashbang.emacs.lisp'}
      },
      end: '$',
      name: 'comment.line.hashbang.emacs.lisp'
    },
    {include: '#main'}
  ],
  repository: {
    'archive-sources': {
      captures: {
        1: {name: 'support.language.constant.archive-source.emacs.lisp'}
      },
      match:
        '(?x)\\b(?<=[\\s()\\[]|^)\n(SC|gnu|marmalade|melpa-stable|melpa|org)\n(?=[\\s()]|$) \\b '
    },
    'arg-values': {
      patterns: [
        {
          match: '&(optional|rest)(?=\\s|\\))',
          name: 'constant.language.$1.arguments.emacs.lisp'
        }
      ]
    },
    autoload: {
      begin: '^(;;;###)(autoload)',
      beginCaptures: {
        1: {name: 'punctuation.definition.comment.emacs.lisp'},
        2: {name: 'storage.modifier.autoload.emacs.lisp'}
      },
      contentName: 'string.unquoted.other.emacs.lisp',
      end: '$',
      name: 'comment.line.semicolon.autoload.emacs.lisp'
    },
    binding: {
      match: '\\b(?<=[\\s()\\[]|^)(let\\*?|set[fq]?)(?=[\\s()]|$)',
      name: 'storage.binding.emacs.lisp'
    },
    boolean: {
      patterns: [
        {
          match: '\\b(?<=[\\s()\\[]|^)t(?=[\\s()]|$)\\b',
          name: 'constant.boolean.true.emacs.lisp'
        },
        {
          match: '\\b(?<=[\\s()\\[]|^)(nil)(?=[\\s()]|$)\\b',
          name: 'constant.language.nil.emacs.lisp'
        }
      ]
    },
    cask: {
      match:
        '\\b(?<=[\\s()\\[]|^)(?:files|source|development|depends-on|package-file|package-descriptor|package)(?=[\\s()]|$)\\b',
      name: 'support.function.emacs.lisp'
    },
    comment: {
      begin: ';',
      beginCaptures: {0: {name: 'punctuation.definition.comment.emacs.lisp'}},
      end: '$',
      name: 'comment.line.semicolon.emacs.lisp',
      patterns: [{include: '#modeline'}, {include: '#eldoc'}]
    },
    definition: {
      patterns: [
        {
          begin:
            '(\\()(?:(cl-(defun|defmacro|defsubst))|(defun|defmacro|defsubst))(?!-)\\b(?:\\s*(?![-+\\d])([-+=*/\\w~!@$%^&:<>{}?]+))?',
          beginCaptures: {
            1: {name: 'punctuation.section.expression.begin.emacs.lisp'},
            2: {name: 'storage.type.$3.function.cl-lib.emacs.lisp'},
            4: {name: 'storage.type.$4.function.emacs.lisp'},
            5: {name: 'entity.function.name.emacs.lisp'}
          },
          end: '\\)',
          endCaptures: {
            0: {name: 'punctuation.section.expression.end.emacs.lisp'}
          },
          name: 'meta.function.definition.emacs.lisp',
          patterns: [{include: '#defun-innards'}]
        },
        {
          match: '\\b(?<=[\\s()\\[]|^)defun(?=[\\s()]|$)',
          name: 'storage.type.function.emacs.lisp'
        },
        {
          begin:
            '(?x) (?<=\\s|^) (\\()\n(def(advice|class|const|custom|face|image|group|package|struct|subst|theme|type|var))\n(?:\\s+([-+=*/\\w~!@$%^&:<>{}?]+))?\n(?=[\\s()]|$)',
          beginCaptures: {
            1: {name: 'punctuation.section.expression.begin.emacs.lisp'},
            2: {name: 'storage.type.$3.emacs.lisp'},
            4: {name: 'entity.name.$3.emacs.lisp'}
          },
          end: '\\)',
          endCaptures: {
            0: {name: 'punctuation.section.expression.end.emacs.lisp'}
          },
          name: 'meta.$3.definition.emacs.lisp',
          patterns: [{include: '$self'}]
        },
        {
          match:
            '\\b(?<=[\\s()\\[]|^)(define-(?:condition|widget))(?=[\\s()]|$)\\b',
          name: 'storage.type.$1.emacs.lisp'
        }
      ]
    },
    'defun-innards': {
      patterns: [
        {
          begin: '\\G\\s*(\\()',
          beginCaptures: {
            0: {name: 'punctuation.section.expression.begin.emacs.lisp'}
          },
          end: '\\)',
          endCaptures: {
            0: {name: 'punctuation.section.expression.end.emacs.lisp'}
          },
          name: 'meta.argument-list.expression.emacs.lisp',
          patterns: [
            {include: '#arg-keywords'},
            {
              match: "(?![-+\\d:&'#])([-+=*/\\w~!@$%^&:<>{}?]+)",
              name: 'variable.parameter.emacs.lisp'
            },
            {include: '$self'}
          ]
        },
        {include: '$self'}
      ]
    },
    docesc: {
      patterns: [
        {
          match: '\\x5C{2}=',
          name: 'constant.escape.character.key-sequence.emacs.lisp'
        },
        {
          match: '\\x5C{2}+',
          name: 'constant.escape.character.suppress-link.emacs.lisp'
        }
      ]
    },
    dockey: {
      captures: {
        1: {name: 'punctuation.definition.reference.begin.emacs.lisp'},
        2: {name: 'constant.other.reference.link.emacs.lisp'},
        3: {name: 'punctuation.definition.reference.end.emacs.lisp'}
      },
      match: '(\\x5C{2}\\[)((?:[^\\s\\\\]|\\\\.)+)(\\])',
      name: 'variable.other.reference.key-sequence.emacs.lisp'
    },
    docmap: {
      patterns: [
        {
          captures: {
            1: {name: 'punctuation.definition.reference.begin.emacs.lisp'},
            2: {name: 'entity.name.tag.keymap.emacs.lisp'},
            3: {name: 'punctuation.definition.reference.end.emacs.lisp'}
          },
          match: '(\\x5C{2}{)((?:[^\\s\\\\]|\\\\.)+)(})',
          name: 'meta.keymap.summary.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'punctuation.definition.reference.begin.emacs.lisp'},
            2: {name: 'entity.name.tag.keymap.emacs.lisp'},
            3: {name: 'punctuation.definition.reference.end.emacs.lisp'}
          },
          match: '(\\x5C{2}<)((?:[^\\s\\\\]|\\\\.)+)(>)',
          name: 'meta.keymap.specifier.emacs.lisp'
        }
      ]
    },
    docvar: {
      captures: {
        1: {name: 'punctuation.definition.quote.begin.emacs.lisp'},
        2: {name: 'punctuation.definition.quote.end.emacs.lisp'}
      },
      match: "(`)[^\\s()]+(')",
      name: 'variable.other.literal.emacs.lisp'
    },
    eldoc: {
      patterns: [
        {include: '#docesc'},
        {include: '#docvar'},
        {include: '#dockey'},
        {include: '#docmap'}
      ]
    },
    escapes: {
      patterns: [
        {
          captures: {
            1: {name: 'punctuation.definition.codepoint.emacs.lisp'},
            2: {name: 'punctuation.definition.codepoint.emacs.lisp'}
          },
          match: '(\\?)\\\\u[A-Fa-f0-9]{4}|(\\?)\\\\U00[A-Fa-f0-9]{6}',
          name: 'constant.character.escape.hex.emacs.lisp'
        },
        {
          captures: {1: {name: 'punctuation.definition.codepoint.emacs.lisp'}},
          match: '(\\?)\\\\x[A-Fa-f0-9]+',
          name: 'constant.character.escape.hex.emacs.lisp'
        },
        {
          captures: {1: {name: 'punctuation.definition.codepoint.emacs.lisp'}},
          match: '(\\?)\\\\[0-7]{1,3}',
          name: 'constant.character.escape.octal.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'punctuation.definition.codepoint.emacs.lisp'},
            2: {name: 'punctuation.definition.backslash.emacs.lisp'}
          },
          match: '(\\?)(?:[^\\\\]|(\\\\).)',
          name: 'constant.numeric.codepoint.emacs.lisp'
        },
        {
          captures: {1: {name: 'punctuation.definition.backslash.emacs.lisp'}},
          match: '(\\\\).',
          name: 'constant.character.escape.emacs.lisp'
        }
      ]
    },
    expression: {
      patterns: [
        {
          begin: '\\(',
          beginCaptures: {
            0: {name: 'punctuation.section.expression.begin.emacs.lisp'}
          },
          end: '\\)',
          endCaptures: {
            0: {name: 'punctuation.section.expression.end.emacs.lisp'}
          },
          name: 'meta.expression.emacs.lisp',
          patterns: [{include: '$self'}]
        },
        {
          begin: "(\\')(\\()",
          beginCaptures: {
            1: {name: 'punctuation.definition.symbol.emacs.lisp'},
            2: {name: 'punctuation.section.quoted.expression.begin.emacs.lisp'}
          },
          end: '\\)',
          endCaptures: {
            0: {name: 'punctuation.section.quoted.expression.end.emacs.lisp'}
          },
          name: 'meta.quoted.expression.emacs.lisp',
          patterns: [{include: '$self'}]
        },
        {
          begin: '(\\`)(\\()',
          beginCaptures: {
            1: {name: 'punctuation.definition.symbol.emacs.lisp'},
            2: {
              name: 'punctuation.section.backquoted.expression.begin.emacs.lisp'
            }
          },
          end: '\\)',
          endCaptures: {
            0: {
              name: 'punctuation.section.backquoted.expression.end.emacs.lisp'
            }
          },
          name: 'meta.backquoted.expression.emacs.lisp',
          patterns: [{include: '$self'}]
        },
        {
          begin: '(,@)(\\()',
          beginCaptures: {
            1: {name: 'punctuation.definition.symbol.emacs.lisp'},
            2: {
              name: 'punctuation.section.interpolated.expression.begin.emacs.lisp'
            }
          },
          end: '\\)',
          endCaptures: {
            0: {
              name: 'punctuation.section.interpolated.expression.end.emacs.lisp'
            }
          },
          name: 'meta.interpolated.expression.emacs.lisp',
          patterns: [{include: '$self'}]
        }
      ]
    },
    'face-innards': {
      patterns: [
        {
          captures: {
            1: {name: 'punctuation.section.expression.begin.emacs.lisp'},
            2: {name: 'variable.language.display.type.emacs.lisp'},
            3: {name: 'support.constant.display.type.emacs.lisp'},
            4: {name: 'punctuation.section.expression.end.emacs.lisp'}
          },
          match: '(\\()(type)\\s+(graphic|x|pc|w32|tty)(\\))',
          name: 'meta.expression.display-type.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'punctuation.section.expression.begin.emacs.lisp'},
            2: {name: 'variable.language.display.class.emacs.lisp'},
            3: {name: 'support.constant.display.class.emacs.lisp'},
            4: {name: 'punctuation.section.expression.end.emacs.lisp'}
          },
          match: '(\\()(class)\\s+(color|grayscale|mono)(\\))',
          name: 'meta.expression.display-class.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'punctuation.section.expression.begin.emacs.lisp'},
            2: {name: 'variable.language.background-type.emacs.lisp'},
            3: {name: 'support.constant.background-type.emacs.lisp'},
            4: {name: 'punctuation.section.expression.end.emacs.lisp'}
          },
          match: '(\\()(background)\\s+(light|dark)(\\))',
          name: 'meta.expression.background-type.emacs.lisp'
        },
        {
          begin: '(\\()(min-colors|supports)(?=[\\s()]|$)',
          beginCaptures: {
            1: {name: 'punctuation.section.expression.begin.emacs.lisp'},
            2: {name: 'variable.language.display-prerequisite.emacs.lisp'}
          },
          end: '\\)',
          endCaptures: {
            0: {name: 'punctuation.section.expression.end.emacs.lisp'}
          },
          name: 'meta.expression.display-prerequisite.emacs.lisp',
          patterns: [{include: '$self'}]
        }
      ]
    },
    faces: {
      match:
        '(?x) \\b (?<=[\\s()\\[]|^)\n\t(?:Buffer-menu-buffer|Info-quoted|Info-title-1-face|Info-title-2-face|Info-title-3-face|Info-title-4-face|Man-overstrike|Man-reverse\n\t|Man-underline|antlr-default|antlr-font-lock-default-face|antlr-font-lock-keyword-face|antlr-font-lock-literal-face\n\t|antlr-font-lock-ruledef-face|antlr-font-lock-ruleref-face|antlr-font-lock-syntax-face|antlr-font-lock-tokendef-face\n\t|antlr-font-lock-tokenref-face|antlr-keyword|antlr-literal|antlr-ruledef|antlr-ruleref|antlr-syntax|antlr-tokendef|antlr-tokenref\n\t|apropos-keybinding|apropos-property|apropos-symbol|bat-label-face|bg:erc-color-face0|bg:erc-color-face1|bg:erc-color-face10\n\t|bg:erc-color-face11|bg:erc-color-face12|bg:erc-color-face13|bg:erc-color-face14|bg:erc-color-face15|bg:erc-color-face2\n\t|bg:erc-color-face3|bg:erc-color-face4|bg:erc-color-face5|bg:erc-color-face6|bg:erc-color-face7|bg:erc-color-face8|bg:erc-color-face9\n\t|bold-italic|bold|bookmark-menu-bookmark|bookmark-menu-heading|border|breakpoint-disabled|breakpoint-enabled|buffer-menu-buffer\n\t|button|c-annotation-face|calc-nonselected-face|calc-selected-face|calendar-month-header|calendar-today|calendar-weekday-header\n\t|calendar-weekend-header|change-log-acknowledgement-face|change-log-acknowledgement|change-log-acknowledgment\n\t|change-log-conditionals-face|change-log-conditionals|change-log-date-face|change-log-date|change-log-email-face|change-log-email\n\t|change-log-file-face|change-log-file|change-log-function-face|change-log-function|change-log-list-face|change-log-list\n\t|change-log-name-face|change-log-name|comint-highlight-input|comint-highlight-prompt|compare-windows|compilation-column-number\n\t|compilation-error|compilation-info|compilation-line-number|compilation-mode-line-exit|compilation-mode-line-fail\n\t|compilation-mode-line-run|compilation-warning|completions-annotations|completions-common-part|completions-first-difference\n\t|cperl-array-face|cperl-hash-face|cperl-nonoverridable-face|css-property|css-selector|cua-global-mark|cua-rectangle-noselect\n\t|cua-rectangle|cursor|custom-button-mouse|custom-button-pressed-unraised|custom-button-pressed|custom-button-unraised|custom-button\n\t|custom-changed|custom-comment-tag|custom-comment|custom-documentation|custom-face-tag|custom-group-subtitle|custom-group-tag-1\n\t|custom-group-tag|custom-invalid|custom-link|custom-modified|custom-rogue|custom-saved|custom-set|custom-state|custom-themed\n\t|custom-variable-button|custom-variable-tag|custom-visibility|cvs-filename-face|cvs-filename|cvs-handled-face|cvs-handled\n\t|cvs-header-face|cvs-header|cvs-marked-face|cvs-marked|cvs-msg-face|cvs-msg|cvs-need-action-face|cvs-need-action|cvs-unknown-face\n\t|cvs-unknown|default|diary-anniversary|diary-button|diary-time|diary|diff-added-face|diff-added|diff-changed-face|diff-changed\n\t|diff-context-face|diff-context|diff-file-header-face|diff-file-header|diff-function-face|diff-function|diff-header-face\n\t|diff-header|diff-hunk-header-face|diff-hunk-header|diff-index-face|diff-index|diff-indicator-added|diff-indicator-changed\n\t|diff-indicator-removed|diff-nonexistent-face|diff-nonexistent|diff-refine-added|diff-refine-change|diff-refine-changed\n\t|diff-refine-removed|diff-removed-face|diff-removed|dired-directory|dired-flagged|dired-header|dired-ignored|dired-mark\n\t|dired-marked|dired-perm-write|dired-symlink|dired-warning|ebrowse-default|ebrowse-file-name|ebrowse-member-attribute\n\t|ebrowse-member-class|ebrowse-progress|ebrowse-root-class|ebrowse-tree-mark|ediff-current-diff-A|ediff-current-diff-Ancestor\n\t|ediff-current-diff-B|ediff-current-diff-C|ediff-even-diff-A|ediff-even-diff-Ancestor|ediff-even-diff-B|ediff-even-diff-C\n\t|ediff-fine-diff-A|ediff-fine-diff-Ancestor|ediff-fine-diff-B|ediff-fine-diff-C|ediff-odd-diff-A|ediff-odd-diff-Ancestor\n\t|ediff-odd-diff-B|ediff-odd-diff-C|eieio-custom-slot-tag-face|eldoc-highlight-function-argument|epa-field-body\n\t|epa-field-name|epa-mark|epa-string|epa-validity-disabled|epa-validity-high|epa-validity-low|epa-validity-medium|erc-action-face\n\t|erc-bold-face|erc-button|erc-command-indicator-face|erc-current-nick-face|erc-dangerous-host-face|erc-default-face|erc-direct-msg-face\n\t|erc-error-face|erc-fool-face|erc-header-line|erc-input-face|erc-inverse-face|erc-keyword-face|erc-my-nick-face|erc-my-nick-prefix-face\n\t|erc-nick-default-face|erc-nick-msg-face|erc-nick-prefix-face|erc-notice-face|erc-pal-face|erc-prompt-face|erc-timestamp-face|erc-underline-face\n\t|error|ert-test-result-expected|ert-test-result-unexpected|escape-glyph|eww-form-checkbox|eww-form-file|eww-form-select|eww-form-submit\n\t|eww-form-text|eww-form-textarea|eww-invalid-certificate|eww-valid-certificate|excerpt|ffap|fg:erc-color-face0|fg:erc-color-face1\n\t|fg:erc-color-face10|fg:erc-color-face11|fg:erc-color-face12|fg:erc-color-face13|fg:erc-color-face14|fg:erc-color-face15|fg:erc-color-face2\n\t|fg:erc-color-face3|fg:erc-color-face4|fg:erc-color-face5|fg:erc-color-face6|fg:erc-color-face7|fg:erc-color-face8|fg:erc-color-face9\n\t|file-name-shadow|fixed-pitch|fixed|flymake-errline|flymake-warnline|flyspell-duplicate|flyspell-incorrect|font-lock-builtin-face\n\t|font-lock-comment-delimiter-face|font-lock-comment-face|font-lock-constant-face|font-lock-doc-face|font-lock-function-name-face\n\t|font-lock-keyword-face|font-lock-negation-char-face|font-lock-preprocessor-face|font-lock-regexp-grouping-backslash|font-lock-regexp-grouping-construct\n\t|font-lock-string-face|font-lock-type-face|font-lock-variable-name-face|font-lock-warning-face|fringe|glyphless-char|gnus-button\n\t|gnus-cite-1|gnus-cite-10|gnus-cite-11|gnus-cite-2|gnus-cite-3|gnus-cite-4|gnus-cite-5|gnus-cite-6|gnus-cite-7|gnus-cite-8|gnus-cite-9\n\t|gnus-cite-attribution-face|gnus-cite-attribution|gnus-cite-face-1|gnus-cite-face-10|gnus-cite-face-11|gnus-cite-face-2|gnus-cite-face-3\n\t|gnus-cite-face-4|gnus-cite-face-5|gnus-cite-face-6|gnus-cite-face-7|gnus-cite-face-8|gnus-cite-face-9|gnus-emphasis-bold-italic\n\t|gnus-emphasis-bold|gnus-emphasis-highlight-words|gnus-emphasis-italic|gnus-emphasis-strikethru|gnus-emphasis-underline-bold-italic\n\t|gnus-emphasis-underline-bold|gnus-emphasis-underline-italic|gnus-emphasis-underline|gnus-group-mail-1-empty-face|gnus-group-mail-1-empty\n\t|gnus-group-mail-1-face|gnus-group-mail-1|gnus-group-mail-2-empty-face|gnus-group-mail-2-empty|gnus-group-mail-2-face|gnus-group-mail-2\n\t|gnus-group-mail-3-empty-face|gnus-group-mail-3-empty|gnus-group-mail-3-face|gnus-group-mail-3|gnus-group-mail-low-empty-face\n\t|gnus-group-mail-low-empty|gnus-group-mail-low-face|gnus-group-mail-low|gnus-group-news-1-empty-face|gnus-group-news-1-empty|gnus-group-news-1-face\n\t|gnus-group-news-1|gnus-group-news-2-empty-face|gnus-group-news-2-empty|gnus-group-news-2-face|gnus-group-news-2|gnus-group-news-3-empty-face\n\t|gnus-group-news-3-empty|gnus-group-news-3-face|gnus-group-news-3|gnus-group-news-4-empty-face|gnus-group-news-4-empty|gnus-group-news-4-face\n\t|gnus-group-news-4|gnus-group-news-5-empty-face|gnus-group-news-5-empty|gnus-group-news-5-face|gnus-group-news-5|gnus-group-news-6-empty-face\n\t|gnus-group-news-6-empty|gnus-group-news-6-face|gnus-group-news-6|gnus-group-news-low-empty-face|gnus-group-news-low-empty|gnus-group-news-low-face\n\t|gnus-group-news-low|gnus-header-content-face|gnus-header-content|gnus-header-from-face|gnus-header-from|gnus-header-name-face\n\t|gnus-header-name|gnus-header-newsgroups-face|gnus-header-newsgroups|gnus-header-subject-face|gnus-header-subject|gnus-signature-face\n\t|gnus-signature|gnus-splash-face|gnus-splash|gnus-summary-cancelled-face|gnus-summary-cancelled|gnus-summary-high-ancient-face\n\t|gnus-summary-high-ancient|gnus-summary-high-read-face|gnus-summary-high-read|gnus-summary-high-ticked-face|gnus-summary-high-ticked\n\t|gnus-summary-high-undownloaded-face|gnus-summary-high-undownloaded|gnus-summary-high-unread-face|gnus-summary-high-unread|gnus-summary-low-ancient-face\n\t|gnus-summary-low-ancient|gnus-summary-low-read-face|gnus-summary-low-read|gnus-summary-low-ticked-face|gnus-summary-low-ticked\n\t|gnus-summary-low-undownloaded-face|gnus-summary-low-undownloaded|gnus-summary-low-unread-face|gnus-summary-low-unread|gnus-summary-normal-ancient-face\n\t|gnus-summary-normal-ancient|gnus-summary-normal-read-face|gnus-summary-normal-read|gnus-summary-normal-ticked-face|gnus-summary-normal-ticked\n\t|gnus-summary-normal-undownloaded-face|gnus-summary-normal-undownloaded|gnus-summary-normal-unread-face|gnus-summary-normal-unread\n\t|gnus-summary-selected-face|gnus-summary-selected|gomoku-O|gomoku-X|header-line|help-argument-name|hexl-address-region|hexl-ascii-region\n\t|hi-black-b|hi-black-hb|hi-blue-b|hi-blue|hi-green-b|hi-green|hi-pink|hi-red-b|hi-yellow|hide-ifdef-shadow|highlight-changes-delete-face\n\t|highlight-changes-delete|highlight-changes-face|highlight-changes|highlight|hl-line|holiday|icomplete-first-match|idlwave-help-link\n\t|idlwave-shell-bp|idlwave-shell-disabled-bp|idlwave-shell-electric-stop-line|idlwave-shell-pending-electric-stop|idlwave-shell-pending-stop\n\t|ido-first-match|ido-incomplete-regexp|ido-indicator|ido-only-match|ido-subdir|ido-virtual|info-header-node|info-header-xref|info-index-match\n\t|info-menu-5|info-menu-header|info-menu-star|info-node|info-title-1|info-title-2|info-title-3|info-title-4|info-xref|isearch-fail|isearch-lazy-highlight-face\n\t|isearch|iswitchb-current-match|iswitchb-invalid-regexp|iswitchb-single-match|iswitchb-virtual-matches|italic|landmark-font-lock-face-O\n\t|landmark-font-lock-face-X|lazy-highlight|ld-script-location-counter|link-visited|link|log-edit-header|log-edit-summary|log-edit-unknown-header\n\t|log-view-file-face|log-view-file|log-view-message-face|log-view-message|makefile-makepp-perl|makefile-shell|makefile-space-face\n\t|makefile-space|makefile-targets|match|menu|message-cited-text-face|message-cited-text|message-header-cc-face|message-header-cc|message-header-name-face\n\t|message-header-name|message-header-newsgroups-face|message-header-newsgroups|message-header-other-face|message-header-other|message-header-subject-face\n\t|message-header-subject|message-header-to-face|message-header-to|message-header-xheader-face|message-header-xheader|message-mml-face\n\t|message-mml|message-separator-face|message-separator|mh-folder-address|mh-folder-blacklisted|mh-folder-body|mh-folder-cur-msg-number\n\t|mh-folder-date|mh-folder-deleted|mh-folder-followup|mh-folder-msg-number|mh-folder-refiled|mh-folder-sent-to-me-hint|mh-folder-sent-to-me-sender\n\t|mh-folder-subject|mh-folder-tick|mh-folder-to|mh-folder-whitelisted|mh-letter-header-field|mh-search-folder|mh-show-cc|mh-show-date\n\t|mh-show-from|mh-show-header|mh-show-pgg-bad|mh-show-pgg-good|mh-show-pgg-unknown|mh-show-signature|mh-show-subject|mh-show-to|mh-speedbar-folder-with-unseen-messages\n\t|mh-speedbar-folder|mh-speedbar-selected-folder-with-unseen-messages|mh-speedbar-selected-folder|minibuffer-prompt|mm-command-output\n\t|mm-uu-extract|mode-line-buffer-id|mode-line-emphasis|mode-line-highlight|mode-line-inactive|mode-line|modeline-buffer-id|modeline-highlight\n\t|modeline-inactive|mouse|mpuz-solved|mpuz-text|mpuz-trivial|mpuz-unsolved|newsticker-date-face|newsticker-default-face|newsticker-enclosure-face\n\t|newsticker-extra-face|newsticker-feed-face|newsticker-immortal-item-face|newsticker-new-item-face|newsticker-obsolete-item-face\n\t|newsticker-old-item-face|newsticker-statistics-face|newsticker-treeview-face|newsticker-treeview-immortal-face|newsticker-treeview-new-face\n\t|newsticker-treeview-obsolete-face|newsticker-treeview-old-face|newsticker-treeview-selection-face|next-error|nobreak-space|nxml-attribute-colon\n\t|nxml-attribute-local-name|nxml-attribute-prefix|nxml-attribute-value-delimiter|nxml-attribute-value|nxml-cdata-section-CDATA\n\t|nxml-cdata-section-content|nxml-cdata-section-delimiter|nxml-char-ref-delimiter|nxml-char-ref-number|nxml-comment-content|nxml-comment-delimiter\n\t|nxml-delimited-data|nxml-delimiter|nxml-element-colon|nxml-element-local-name|nxml-element-prefix|nxml-entity-ref-delimiter|nxml-entity-ref-name\n\t|nxml-glyph|nxml-hash|nxml-heading|nxml-markup-declaration-delimiter|nxml-name|nxml-namespace-attribute-colon|nxml-namespace-attribute-prefix\n\t|nxml-namespace-attribute-value-delimiter|nxml-namespace-attribute-value|nxml-namespace-attribute-xmlns|nxml-outline-active-indicator\n\t|nxml-outline-ellipsis|nxml-outline-indicator|nxml-processing-instruction-content|nxml-processing-instruction-delimiter|nxml-processing-instruction-target\n\t|nxml-prolog-keyword|nxml-prolog-literal-content|nxml-prolog-literal-delimiter|nxml-ref|nxml-tag-delimiter|nxml-tag-slash|nxml-text\n\t|octave-function-comment-block|org-agenda-calendar-event|org-agenda-calendar-sexp|org-agenda-clocking|org-agenda-column-dateline\n\t|org-agenda-current-time|org-agenda-date-today|org-agenda-date-weekend|org-agenda-date|org-agenda-diary|org-agenda-dimmed-todo-face\n\t|org-agenda-done|org-agenda-filter-category|org-agenda-filter-regexp|org-agenda-filter-tags|org-agenda-restriction-lock|org-agenda-structure\n\t|org-archived|org-block-background|org-block-begin-line|org-block-end-line|org-block|org-checkbox-statistics-done|org-checkbox-statistics-todo\n\t|org-checkbox|org-clock-overlay|org-code|org-column-title|org-column|org-date-selected|org-date|org-default|org-document-info-keyword\n\t|org-document-info|org-document-title|org-done|org-drawer|org-ellipsis|org-footnote|org-formula|org-headline-done|org-hide|org-latex-and-related\n\t|org-level-1|org-level-2|org-level-3|org-level-4|org-level-5|org-level-6|org-level-7|org-level-8|org-link|org-list-dt|org-macro|org-meta-line\n\t|org-mode-line-clock-overrun|org-mode-line-clock|org-priority|org-property-value|org-quote|org-scheduled-previously|org-scheduled-today\n\t|org-scheduled|org-sexp-date|org-special-keyword|org-table|org-tag-group|org-tag|org-target|org-time-grid|org-todo|org-upcoming-deadline\n\t|org-verbatim|org-verse|org-warning|outline-1|outline-2|outline-3|outline-4|outline-5|outline-6|outline-7|outline-8|proced-mark|proced-marked\n\t|proced-sort-header|pulse-highlight-face|pulse-highlight-start-face|query-replace|rcirc-bright-nick|rcirc-dim-nick|rcirc-keyword\n\t|rcirc-my-nick|rcirc-nick-in-message-full-line|rcirc-nick-in-message|rcirc-other-nick|rcirc-prompt|rcirc-server-prefix|rcirc-server\n\t|rcirc-timestamp|rcirc-track-keyword|rcirc-track-nick|rcirc-url|reb-match-0|reb-match-1|reb-match-2|reb-match-3|rectangle-preview-face\n\t|region|rmail-header-name|rmail-highlight|rng-error|rst-adornment|rst-block|rst-comment|rst-definition|rst-directive|rst-emphasis1\n\t|rst-emphasis2|rst-external|rst-level-1|rst-level-2|rst-level-3|rst-level-4|rst-level-5|rst-level-6|rst-literal|rst-reference|rst-transition\n\t|ruler-mode-column-number|ruler-mode-comment-column|ruler-mode-current-column|ruler-mode-default|ruler-mode-fill-column|ruler-mode-fringes\n\t|ruler-mode-goal-column|ruler-mode-margins|ruler-mode-pad|ruler-mode-tab-stop|scroll-bar|secondary-selection|semantic-highlight-edits-face\n\t|semantic-highlight-func-current-tag-face|semantic-unmatched-syntax-face|senator-momentary-highlight-face|sgml-namespace|sh-escaped-newline\n\t|sh-heredoc-face|sh-heredoc|sh-quoted-exec|shadow|show-paren-match-face|show-paren-match|show-paren-mismatch-face|show-paren-mismatch\n\t|shr-link|shr-strike-through|smerge-base-face|smerge-base|smerge-markers-face|smerge-markers|smerge-mine-face|smerge-mine|smerge-other-face\n\t|smerge-other|smerge-refined-added|smerge-refined-change|smerge-refined-changed|smerge-refined-removed|speedbar-button-face|speedbar-directory-face\n\t|speedbar-file-face|speedbar-highlight-face|speedbar-selected-face|speedbar-separator-face|speedbar-tag-face|srecode-separator-face\n\t|strokes-char|subscript|success|superscript|table-cell|tcl-escaped-newline|term-bold|term-color-black|term-color-blue|term-color-cyan\n\t|term-color-green|term-color-magenta|term-color-red|term-color-white|term-color-yellow|term-underline|term|testcover-1value|testcover-nohits\n\t|tex-math-face|tex-math|tex-verbatim-face|tex-verbatim|texinfo-heading-face|texinfo-heading|tmm-inactive|todo-archived-only|todo-button\n\t|todo-category-string|todo-comment|todo-date|todo-diary-expired|todo-done-sep|todo-done|todo-key-prompt|todo-mark|todo-nondiary|todo-prefix-string\n\t|todo-search|todo-sorted-column|todo-time|todo-top-priority|tool-bar|tooltip|trailing-whitespace|tty-menu-disabled-face|tty-menu-enabled-face\n\t|tty-menu-selected-face|underline|variable-pitch|vc-conflict-state|vc-edited-state|vc-locally-added-state|vc-locked-state|vc-missing-state\n\t|vc-needs-update-state|vc-removed-state|vc-state-base-face|vc-up-to-date-state|vcursor|vera-font-lock-function|vera-font-lock-interface\n\t|vera-font-lock-number|verilog-font-lock-ams-face|verilog-font-lock-grouping-keywords-face|verilog-font-lock-p1800-face|verilog-font-lock-translate-off-face\n\t|vertical-border|vhdl-font-lock-attribute-face|vhdl-font-lock-directive-face|vhdl-font-lock-enumvalue-face|vhdl-font-lock-function-face\n\t|vhdl-font-lock-generic-\\/constant-face|vhdl-font-lock-prompt-face|vhdl-font-lock-reserved-words-face|vhdl-font-lock-translate-off-face\n\t|vhdl-font-lock-type-face|vhdl-font-lock-variable-face|vhdl-speedbar-architecture-face|vhdl-speedbar-architecture-selected-face\n\t|vhdl-speedbar-configuration-face|vhdl-speedbar-configuration-selected-face|vhdl-speedbar-entity-face|vhdl-speedbar-entity-selected-face\n\t|vhdl-speedbar-instantiation-face|vhdl-speedbar-instantiation-selected-face|vhdl-speedbar-library-face|vhdl-speedbar-package-face\n\t|vhdl-speedbar-package-selected-face|vhdl-speedbar-subprogram-face|viper-minibuffer-emacs|viper-minibuffer-insert|viper-minibuffer-vi\n\t|viper-replace-overlay|viper-search|warning|which-func|whitespace-big-indent|whitespace-empty|whitespace-hspace|whitespace-indentation\n\t|whitespace-line|whitespace-newline|whitespace-space-after-tab|whitespace-space-before-tab|whitespace-space|whitespace-tab|whitespace-trailing\n\t|widget-button-face|widget-button-pressed-face|widget-button-pressed|widget-button|widget-documentation-face|widget-documentation\n\t|widget-field-face|widget-field|widget-inactive-face|widget-inactive|widget-single-line-field-face|widget-single-line-field|window-divider-first-pixel\n\t|window-divider-last-pixel|window-divider|woman-addition-face|woman-addition|woman-bold-face|woman-bold|woman-italic-face|woman-italic\n\t|woman-unknown-face|woman-unknown)\n(?=[\\s()]|$)\\b',
      name: 'support.constant.face.emacs.lisp'
    },
    format: {
      begin: '\\G',
      contentName: 'string.quoted.double.emacs.lisp',
      end: '(?=")',
      patterns: [
        {
          captures: {
            1: {name: 'constant.other.placeholder.emacs.lisp'},
            2: {name: 'invalid.illegal.placeholder.emacs.lisp'}
          },
          match: '(%[%cdefgosSxX])|(%.)'
        },
        {include: '#string-innards'}
      ]
    },
    formatting: {
      begin: '(\\()(format|format-message|message|error)(?=\\s|$|")',
      beginCaptures: {
        1: {name: 'punctuation.section.expression.begin.emacs.lisp'},
        2: {name: 'support.function.$2.emacs.lisp'}
      },
      end: '\\)',
      endCaptures: {0: {name: 'punctuation.section.expression.end.emacs.lisp'}},
      name: 'meta.string-formatting.expression.emacs.lisp',
      patterns: [
        {
          begin: '\\G\\s*(")',
          beginCaptures: {
            1: {name: 'punctuation.definition.string.begin.emacs.lisp'}
          },
          end: '"',
          endCaptures: {
            0: {name: 'punctuation.definition.string.end.emacs.lisp'}
          },
          patterns: [{include: '#format'}]
        },
        {
          begin: '\\G\\s*$\\n?',
          end: '"|(?<!^)$|[\\s"](?=[^\\s"])',
          patterns: [
            {match: '^\\s*$\\n?'},
            {
              captures: {
                1: {name: 'punctuation.definition.string.begin.emacs.lisp'}
              },
              match: '(?:^|\\G)\\s*(")'
            },
            {
              begin: '(?<=")',
              end: '"',
              endCaptures: {
                0: {name: 'punctuation.definition.string.end.emacs.lisp'}
              },
              patterns: [{include: '#format'}]
            }
          ]
        },
        {include: '$self'}
      ]
    },
    functions: {
      match:
        '(?x)\\b(?<=[\\s()\\[]|^)\n(abs|append|apply|assoc|butlast|c[ad]{1,2}r|c[ad]r-safe|consp?|copy-alist|copy-tree\n|dolist|funcall|last|length|listp?|load|make-list|mapc|mapcar|max|min|member|nbutlast\n|nconc|nreverse|nth|nthcdr|null|pop|prin[1ct]|push|quote|rassoc|reverse|rplac[ad]\n|safe-length|setcar|setcdr)\n(?=[\\s()]|$)\\b',
      name: 'keyword.control.function.$1.emacs.lisp'
    },
    'key-notation': {
      patterns: [
        {
          match: '\\b(DEL|ESC|LFD|NUL|RET|SPC|TAB)\\b',
          name: 'constant.control-character.key.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'punctuation.definition.escape.backslash.emacs.lisp'}
          },
          match: '(\\\\)[0-7]{1,6}',
          name: 'constant.character.escape.octal.codepoint.key.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'punctuation.definition.escape.caret.emacs.lisp'}
          },
          match: '(\\^)\\S',
          name: 'constant.character.escape.caret.control.key.emacs.lisp'
        },
        {
          captures: {
            1: {
              name: 'punctuation.definition.double.angle.bracket.begin.emacs.lisp'
            },
            2: {
              name: 'punctuation.definition.double.angle.bracket.end.emacs.lisp'
            }
          },
          match: '(<<)[-A-Za-z0-9]+(>>)',
          name: 'constant.command-name.key.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'constant.numeric.integer.int.decimal.emacs.lisp'},
            2: {name: 'keyword.operator.arithmetic.multiply.emacs.lisp'}
          },
          match: '([0-9]+)(\\*)(?=[\\S])',
          name: 'meta.key-repetition.emacs.lisp'
        },
        {
          captures: {
            1: {patterns: [{include: '#key-notation-prefix'}]},
            2: {name: 'constant.character.key.emacs.lisp'}
          },
          match: '\\b(M-)(-?[0-9]+)\\b',
          name: 'meta.key-sequence.emacs.lisp'
        },
        {
          captures: {
            1: {patterns: [{include: '#key-notation-prefix'}]},
            2: {name: 'punctuation.definition.angle.bracket.begin.emacs.lisp'},
            3: {name: 'constant.control-character.key.emacs.lisp'},
            4: {name: 'punctuation.definition.angle.bracket.end.emacs.lisp'},
            5: {name: 'constant.control-character.key.emacs.lisp'},
            6: {name: 'invalid.illegal.bad-prefix.emacs.lisp'},
            7: {name: 'constant.character.key.emacs.lisp'}
          },
          match:
            '(?x)\n\\b((?:[MCSAHs]-)+)\n(?: (<)(DEL|ESC|LFD|NUL|RET|SPC|TAB)(>)\n|   (DEL|ESC|LFD|NUL|RET|SPC|TAB)\\b\n|   ([!-_a-z]{2,})\n|   ([!-_a-z])\n)?',
          name: 'meta.key-sequence.emacs.lisp'
        },
        {
          captures: {
            1: {
              patterns: [
                {
                  match: '<',
                  name: 'punctuation.definition.angle.bracket.begin.emacs.lisp'
                },
                {include: '#key-notation-prefix'}
              ]
            },
            2: {name: 'constant.function-key.emacs.lisp'},
            3: {name: 'punctuation.definition.angle.bracket.end.emacs.lisp'}
          },
          match: '([MCSAHs]-<|<[MCSAHs]-|<)([-A-Za-z0-9]+)(>)',
          name: 'meta.function-key.emacs.lisp'
        },
        {
          match: '(?<=\\s)(?![MCSAHs<>])[!-_a-z](?=\\s)',
          name: 'constant.character.key.emacs.lisp'
        }
      ]
    },
    'key-notation-prefix': {
      captures: {
        1: {name: 'constant.character.key.modifier.emacs.lisp'},
        2: {name: 'punctuation.separator.modifier.dash.emacs.lisp'}
      },
      match: '([MCSAHs])(-)'
    },
    keyword: {
      captures: {1: {name: 'punctuation.definition.keyword.emacs.lisp'}},
      match: '(?<=[\\s()\\[]|^)(:)[-+=*/\\w~!@$%^&:<>{}?]+',
      name: 'constant.keyword.emacs.lisp'
    },
    lambda: {
      begin: '(\\()(lambda|function)(?:\\s+|(?=[()]))',
      beginCaptures: {
        1: {name: 'punctuation.section.expression.begin.emacs.lisp'},
        2: {name: 'storage.type.lambda.function.emacs.lisp'}
      },
      end: '\\)',
      endCaptures: {0: {name: 'punctuation.section.expression.end.emacs.lisp'}},
      name: 'meta.lambda.expression.emacs.lisp',
      patterns: [{include: '#defun-innards'}]
    },
    loop: {
      begin: '(\\()(cl-loop)(?=[\\s()]|$)',
      beginCaptures: {
        1: {name: 'punctuation.section.expression.begin.emacs.lisp'},
        2: {name: 'support.function.cl-lib.emacs.lisp'}
      },
      end: '\\)',
      endCaptures: {0: {name: 'punctuation.section.expression.end.emacs.lisp'}},
      name: 'meta.cl-lib.loop.emacs.lisp',
      patterns: [
        {
          match:
            '(?x)(?<=[\\s()\\[]|^)\n(above|across|across-ref|always|and|append|as|below|by|collect|concat\n|count|do|each|finally|for|from|if|in|in-ref|initially|into|maximize\n|minimize|named|nconc|never|of|of-ref|on|repeat|return|sum|then|thereis\n|sum|to|unless|until|using|vconcat|when|while|with|(?:\n\t\n\tbeing    \\s+\n\t(?:the)? \\s+\n\t\n\t(?:element|hash-key|hash-value|key-code|key-binding\n\t|key-seq|overlay|interval|symbols|frame|window|buffer)\n\ts?\n\n))(?=[\\s()]|$)',
          name: 'keyword.control.emacs.lisp'
        },
        {include: '$self'}
      ]
    },
    main: {
      patterns: [
        {include: '#autoload'},
        {include: '#comment'},
        {include: '#lambda'},
        {include: '#loop'},
        {include: '#escapes'},
        {include: '#definition'},
        {include: '#formatting'},
        {include: '#face-innards'},
        {include: '#expression'},
        {include: '#operators'},
        {include: '#functions'},
        {include: '#binding'},
        {include: '#keyword'},
        {include: '#string'},
        {include: '#number'},
        {include: '#quote'},
        {include: '#symbols'},
        {include: '#vectors'},
        {include: '#arg-values'},
        {include: '#archive-sources'},
        {include: '#boolean'},
        {include: '#faces'},
        {include: '#cask'},
        {include: '#stdlib'}
      ]
    },
    modeline: {
      captures: {
        1: {name: 'punctuation.definition.modeline.begin.emacs.lisp'},
        2: {patterns: [{include: '#modeline-innards'}]},
        3: {name: 'punctuation.definition.modeline.end.emacs.lisp'}
      },
      match: '(-\\*-)(.*)(-\\*-)',
      name: 'meta.modeline.emacs.lisp'
    },
    'modeline-innards': {
      patterns: [
        {
          captures: {
            1: {name: 'variable.assignment.modeline.emacs.lisp'},
            2: {name: 'punctuation.separator.key-value.emacs.lisp'},
            3: {patterns: [{include: '#modeline-innards'}]}
          },
          match: '([^\\s:;]+)\\s*(:)\\s*([^;]*)',
          name: 'meta.modeline.variable.emacs.lisp'
        },
        {match: ';', name: 'punctuation.terminator.statement.emacs.lisp'},
        {match: ':', name: 'punctuation.separator.key-value.emacs.lisp'},
        {match: '\\S+', name: 'string.other.modeline.emacs.lisp'}
      ]
    },
    number: {
      patterns: [
        {
          captures: {1: {name: 'punctuation.definition.binary.emacs.lisp'}},
          match: '(?<=[\\s()\\[]|^)(#)[Bb][01]+',
          name: 'constant.numeric.integer.binary.emacs.lisp'
        },
        {
          captures: {1: {name: 'punctuation.definition.hex.emacs.lisp'}},
          match: '(?<=[\\s()\\[]|^)(#)[Xx][0-9A-Fa-f]+',
          name: 'constant.numeric.integer.hex.viml'
        },
        {
          match:
            '(?<=[\\s()\\[]|^)[-+]?\\d*\\.\\d+(?:[Ee][-+]?\\d+|[Ee]\\+(?:INF|NaN))?(?=[\\s()]|$)',
          name: 'constant.numeric.float.emacs.lisp'
        },
        {
          match:
            '(?<=[\\s()\\[]|^)[-+]?\\d+(?:[Ee][-+]?\\d+|[Ee]\\+(?:INF|NaN))?(?=[\\s()]|$)',
          name: 'constant.numeric.integer.emacs.lisp'
        }
      ]
    },
    operators: {
      patterns: [
        {
          match:
            '(?x)(?<=[()]|^)\n(and|catch|cond|condition-case(?:-unless-debug)?|dotimes|eql?\n|equal|if|not|or|pcase|prog[12n]|throw|unless|unwind-protect\n|when|while)\n(?=[\\s()]|$)',
          name: 'keyword.control.$1.emacs.lisp'
        },
        {
          match: '(?<=\\(|\\s|^)(interactive)(?=\\s|\\(|\\))',
          name: 'storage.modifier.interactive.function.emacs.lisp'
        },
        {
          match: '(?<=\\(|\\s|^)[-*+/%](?=\\s|\\)|$)',
          name: 'keyword.operator.numeric.emacs.lisp'
        },
        {
          match: '(?<=\\(|\\s|^)[/<>]=|[=<>](?=\\s|\\)|$)',
          name: 'keyword.operator.comparison.emacs.lisp'
        },
        {
          match: '(?<=\\s)\\.(?=\\s|$)',
          name: 'keyword.operator.pair-separator.emacs.lisp'
        }
      ]
    },
    quote: {
      patterns: [
        {
          captures: {
            1: {name: 'punctuation.definition.quote.emacs.lisp'},
            2: {patterns: [{include: '$self'}]}
          },
          match: "(')([-+=*/\\w~!@$%^&:<>{}?]+)",
          name: 'constant.other.symbol.emacs.lisp'
        }
      ]
    },
    stdlib: {
      patterns: [
        {
          match:
            '(?x)(?<=[()]|^)\n\t(`--pcase-macroexpander|Buffer-menu-unmark-all-buffers|Buffer-menu-unmark-all|Info-node-description|aa2u-mark-as-text|aa2u-mark-rectangle-as-text\n\t|aa2u-rectangle|aa2u|ada-find-file|ada-header|ada-mode|add-abbrev|add-change-log-entry-other-window|add-change-log-entry|add-dir-local-variable\n\t|add-file-local-variable-prop-line|add-file-local-variable|add-global-abbrev|add-log-current-defun|add-minor-mode|add-mode-abbrev|add-submenu\n\t|add-timeout|add-to-coding-system-list|add-to-list--anon-cmacro|add-variable-watcher|adoc-mode|advertised-undo|advice--add-function\n\t|advice--buffer-local|advice--called-interactively-skip|advice--car|advice--cd\\*r|advice--cdr|advice--defalias-fset|advice--interactive-form\n\t|advice--make-1|advice--make-docstring|advice--make-interactive-form|advice--make|advice--member-p|advice--normalize-place|advice--normalize\n\t|advice--props|advice--p|advice--remove-function|advice--set-buffer-local|advice--strip-macro|advice--subst-main|advice--symbol-function\n\t|advice--tweak|advice--where|after-insert-file-set-coding|aggressive-indent--extend-end-to-whole-sexps|aggressive-indent--indent-current-balanced-line\n\t|aggressive-indent--indent-if-changed|aggressive-indent--keep-track-of-changes|aggressive-indent--local-electric\n\t|aggressive-indent--proccess-changed-list-and-indent|aggressive-indent--run-user-hooks|aggressive-indent--softly-indent-defun\n\t|aggressive-indent--softly-indent-region-and-on|aggressive-indent-bug-report|aggressive-indent-global-mode|aggressive-indent-indent-defun\n\t|aggressive-indent-indent-region-and-on|aggressive-indent-mode-set-explicitly|aggressive-indent-mode|align-current|align-entire|align-highlight-rule\n\t|align-newline-and-indent|align-regexp|align-unhighlight-rule|align|alist-get|all-threads|allout-auto-activation-helper|allout-mode-p|allout-mode\n\t|allout-setup|allout-widgets-mode|allout-widgets-setup|alter-text-property|and-let\\*|ange-ftp-completion-hook-function|apache-mode\n\t|apropos-local-value|apropos-local-variable|arabic-shape-gstring|assoc-delete-all|auth-source--decode-octal-string|auth-source--symbol-keyword\n\t|auth-source-backend--anon-cmacro|auth-source-backend--eieio-childp|auth-source-backends-parser-file|auth-source-backends-parser-macos-keychain\n\t|auth-source-backends-parser-secrets|auth-source-json-check|auth-source-json-search|auth-source-pass-enable|auth-source-secrets-saver\n\t|auto-save-visited-mode|backtrace-frame--internal|backtrace-frames|backward-to-word|backward-word-strictly|battery-upower-prop|battery-upower\n\t|beginning-of-defun--in-emptyish-line-p|beginning-of-defun-comments|bf-help-describe-symbol|bf-help-mode|bf-help-setup|bignump|bison-mode\n\t|blink-cursor--rescan-frames|blink-cursor--should-blink|blink-cursor--start-idle-timer|blink-cursor--start-timer|bookmark-set-no-overwrite\n\t|brainfuck-mode|browse-url-conkeror|buffer-hash|bufferpos-to-filepos|byte-compile--function-signature|byte-compile--log-warning-for-byte-compile\n\t|byte-compile-cond-jump-table-info|byte-compile-cond-jump-table|byte-compile-cond-vars|byte-compile-define-symbol-prop|byte-compile-file-form-defvar-function\n\t|byte-compile-file-form-make-obsolete|byte-opt--arith-reduce|byte-opt--portable-numberp|byte-optimize-1-|byte-optimize-1\\+|byte-optimize-memq\n\t|c-or-c\\+\\+-mode|call-shell-region|cancel-debug-on-variable-change|cancel-debug-watch|capitalize-dwim|cconv--convert-funcbody|cconv--remap-llv\n\t|char-fold-to-regexp|char-from-name|checkdoc-file|checkdoc-package-keywords|cl--assertion-failed|cl--class-docstring--cmacro|cl--class-docstring\n\t|cl--class-index-table--cmacro|cl--class-index-table|cl--class-name--cmacro|cl--class-name|cl--class-p--cmacro|cl--class-parents--cmacro\n\t|cl--class-parents|cl--class-p|cl--class-slots--cmacro|cl--class-slots|cl--copy-slot-descriptor-1|cl--copy-slot-descriptor|cl--defstruct-predicate\n\t|cl--describe-class-slots|cl--describe-class-slot|cl--describe-class|cl--do-&aux|cl--find-class|cl--generic-arg-specializer\n\t|cl--generic-build-combined-method|cl--generic-cache-miss|cl--generic-class-parents|cl--generic-derived-specializers|cl--generic-describe\n\t|cl--generic-dispatches--cmacro|cl--generic-dispatches|cl--generic-fgrep|cl--generic-generalizer-name--cmacro|cl--generic-generalizer-name\n\t|cl--generic-generalizer-p--cmacro|cl--generic-generalizer-priority--cmacro|cl--generic-generalizer-priority|cl--generic-generalizer-p\n\t|cl--generic-generalizer-specializers-function--cmacro|cl--generic-generalizer-specializers-function|cl--generic-generalizer-tagcode-function--cmacro\n\t|cl--generic-generalizer-tagcode-function|cl--generic-get-dispatcher|cl--generic-isnot-nnm-p|cl--generic-lambda|cl--generic-load-hist-format\n\t|cl--generic-make--cmacro|cl--generic-make-defmethod-docstring|cl--generic-make-function|cl--generic-make-method--cmacro|cl--generic-make-method\n\t|cl--generic-make-next-function|cl--generic-make|cl--generic-member-method|cl--generic-method-documentation|cl--generic-method-files\n\t|cl--generic-method-function--cmacro|cl--generic-method-function|cl--generic-method-info|cl--generic-method-qualifiers--cmacro\n\t|cl--generic-method-qualifiers|cl--generic-method-specializers--cmacro|cl--generic-method-specializers|cl--generic-method-table--cmacro\n\t|cl--generic-method-table|cl--generic-method-uses-cnm--cmacro|cl--generic-method-uses-cnm|cl--generic-name--cmacro|cl--generic-name)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(cl--generic-no-next-method-function|cl--generic-options--cmacro|cl--generic-options|cl--generic-search-method|cl--generic-specializers-apply-to-type-p\n\t|cl--generic-split-args|cl--generic-standard-method-combination|cl--generic-struct-specializers|cl--generic-struct-tag|cl--generic-with-memoization\n\t|cl--generic|cl--make-random-state--cmacro|cl--make-random-state|cl--make-slot-descriptor--cmacro|cl--make-slot-descriptor|cl--make-slot-desc\n\t|cl--old-struct-type-of|cl--pcase-mutually-exclusive-p|cl--plist-remove|cl--print-table|cl--prog|cl--random-state-i--cmacro|cl--random-state-i\n\t|cl--random-state-j--cmacro|cl--random-state-j|cl--random-state-vec--cmacro|cl--random-state-vec|cl--slot-descriptor-initform--cmacro\n\t|cl--slot-descriptor-initform|cl--slot-descriptor-name--cmacro|cl--slot-descriptor-name|cl--slot-descriptor-props--cmacro|cl--slot-descriptor-props\n\t|cl--slot-descriptor-type--cmacro|cl--slot-descriptor-type|cl--struct-all-parents|cl--struct-cl--generic-method-p--cmacro|cl--struct-cl--generic-method-p\n\t|cl--struct-cl--generic-p--cmacro|cl--struct-cl--generic-p|cl--struct-class-children-sym--cmacro|cl--struct-class-children-sym\n\t|cl--struct-class-docstring--cmacro|cl--struct-class-docstring|cl--struct-class-index-table--cmacro|cl--struct-class-index-table\n\t|cl--struct-class-name--cmacro|cl--struct-class-named--cmacro|cl--struct-class-named|cl--struct-class-name|cl--struct-class-p--cmacro\n\t|cl--struct-class-parents--cmacro|cl--struct-class-parents|cl--struct-class-print--cmacro|cl--struct-class-print|cl--struct-class-p\n\t|cl--struct-class-slots--cmacro|cl--struct-class-slots|cl--struct-class-tag--cmacro|cl--struct-class-tag|cl--struct-class-type--cmacro|cl--struct-class-type\n\t|cl--struct-get-class|cl--struct-name-p|cl--struct-new-class--cmacro|cl--struct-new-class|cl--struct-register-child|cl-call-next-method|cl-defgeneric\n\t|cl-defmethod|cl-describe-type|cl-find-class|cl-find-method|cl-generic-all-functions|cl-generic-apply|cl-generic-call-method|cl-generic-combine-methods\n\t|cl-generic-current-method-specializers|cl-generic-define-context-rewriter|cl-generic-define-generalizer|cl-generic-define-method|cl-generic-define\n\t|cl-generic-ensure-function|cl-generic-function-options|cl-generic-generalizers|cl-generic-make-generalizer--cmacro|cl-generic-make-generalizer|cl-generic-p\n\t|cl-iter-defun|cl-method-qualifiers|cl-next-method-p|cl-no-applicable-method|cl-no-next-method|cl-no-primary-method|cl-old-struct-compat-mode\n\t|cl-prin1-to-string|cl-prin1|cl-print-expand-ellipsis|cl-print-object|cl-print-to-string-with-limit|cl-prog\\*|cl-prog|cl-random-state-p--cmacro\n\t|cl-slot-descriptor-p--cmacro|cl-slot-descriptor-p|cl-struct--pcase-macroexpander|cl-struct-define|cl-struct-p--cmacro|cl-struct-p\n\t|cl-struct-slot-value--inliner|cl-typep--inliner|clear-composition-cache|cmake-command-run|cmake-help-command|cmake-help-list-commands|cmake-help-module\n\t|cmake-help-property|cmake-help-variable|cmake-help|cmake-mode|coffee-mode|combine-change-calls-1|combine-change-calls|comment-line|comment-make-bol-ws\n\t|comment-quote-nested-default|comment-region-default-1|completion--category-override|completion-pcm--pattern-point-idx|condition-mutex|condition-name\n\t|condition-notify|condition-variable-p|condition-wait|conf-desktop-mode|conf-toml-mode|conf-toml-recognize-section|connection-local-set-profile-variables\n\t|connection-local-set-profiles|copy-cl--generic-generalizer|copy-cl--generic-method|copy-cl--generic|copy-from-above-command|copy-lisp-indent-state\n\t|copy-xref-elisp-location|copy-yas--exit|copy-yas--field|copy-yas--mirror|copy-yas--snippet|copy-yas--table|copy-yas--template|css-lookup-symbol\n\t|csv-mode|cuda-mode|current-thread|cursor-intangible-mode|cursor-sensor-mode|custom--should-apply-setting|debug-on-variable-change|debug-watch\n\t|default-font-width|define-symbol-prop|define-thing-chars|defined-colors-with-face-attributes|delete-selection-uses-region-p|describe-char-eldoc\n\t|describe-symbol|dir-locals--all-files|dir-locals-read-from-dir|dired--align-all-files|dired--need-align-p|dired-create-empty-file|dired-do-compress-to\n\t|dired-do-find-regexp-and-replace|dired-do-find-regexp|dired-mouse-find-file-other-frame|dired-mouse-find-file|dired-omit-mode|display-buffer--maybe-at-bottom\n\t|display-buffer--maybe-pop-up-frame|display-buffer--maybe-pop-up-window|display-buffer-in-child-frame|display-buffer-reuse-mode-window\n\t|display-buffer-use-some-frame|display-line-numbers-mode|dna-add-hooks|dna-isearch-forward|dna-mode|dna-reverse-complement-region|dockerfile-build-buffer\n\t|dockerfile-build-no-cache-buffer|dockerfile-mode|dolist-with-progress-reporter|dotenv-mode|downcase-dwim|dyalog-ediff-forward-word|dyalog-editor-connect\n\t|dyalog-fix-altgr-chars|dyalog-mode|dyalog-session-connect|easy-mmode--mode-docstring|eieio--add-new-slot|eieio--c3-candidate|eieio--c3-merge-lists\n\t|eieio--class-children--cmacro|eieio--class-class-allocation-values--cmacro|eieio--class-class-slots--cmacro|eieio--class-class-slots|eieio--class-constructor\n\t|eieio--class-default-object-cache--cmacro|eieio--class-docstring--cmacro|eieio--class-docstring|eieio--class-index-table--cmacro|eieio--class-index-table\n\t|eieio--class-initarg-tuples--cmacro|eieio--class-make--cmacro|eieio--class-make|eieio--class-method-invocation-order|eieio--class-name--cmacro\n\t|eieio--class-name|eieio--class-object|eieio--class-option-assoc|eieio--class-options--cmacro|eieio--class-option|eieio--class-p--cmacro)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(eieio--class-parents--cmacro|eieio--class-parents|eieio--class-precedence-bfs|eieio--class-precedence-c3|eieio--class-precedence-dfs\n\t|eieio--class-precedence-list|eieio--class-print-name|eieio--class-p|eieio--class-slot-initarg|eieio--class-slot-name-index|eieio--class-slots--cmacro\n\t|eieio--class-slots|eieio--class\\/struct-parents|eieio--generic-subclass-specializers|eieio--initarg-to-attribute|eieio--object-class-tag\n\t|eieio--pcase-macroexpander|eieio--perform-slot-validation-for-default|eieio--perform-slot-validation|eieio--slot-name-index|eieio--slot-override\n\t|eieio--validate-class-slot-value|eieio--validate-slot-value|eieio-change-class|eieio-class-slots|eieio-default-superclass--eieio-childp\n\t|eieio-defclass-internal|eieio-make-child-predicate|eieio-make-class-predicate|eieio-oref--anon-cmacro|eieio-pcase-slot-index-from-index-table\n\t|eieio-pcase-slot-index-table|eieio-slot-descriptor-name|eldoc--supported-p|eldoc-docstring-format-sym-doc|eldoc-mode-set-explicitly\n\t|electric-pair--balance-info|electric-pair--insert|electric-pair--inside-string-p|electric-pair--skip-whitespace|electric-pair--syntax-ppss\n\t|electric-pair--unbalanced-strings-p|electric-pair--with-uncached-syntax|electric-pair-conservative-inhibit|electric-pair-default-inhibit\n\t|electric-pair-default-skip-self|electric-pair-delete-pair|electric-pair-inhibit-if-helps-balance|electric-pair-local-mode\n\t|electric-pair-post-self-insert-function|electric-pair-skip-if-helps-balance|electric-pair-syntax-info|electric-pair-will-use-region\n\t|electric-quote-local-mode|electric-quote-mode|electric-quote-post-self-insert-function|elisp--font-lock-backslash|elisp--font-lock-flush-elisp-buffers\n\t|elisp--xref-backend|elisp--xref-make-xref|elisp-flymake--batch-compile-for-flymake|elisp-flymake--byte-compile-done|elisp-flymake-byte-compile\n\t|elisp-flymake-checkdoc|elisp-function-argstring|elisp-get-fnsym-args-string|elisp-get-var-docstring|elisp-load-path-roots|emacs-repository-version-git\n\t|enh-ruby-mode|epg-config--make-gpg-configuration|epg-config--make-gpgsm-configuration|epg-context-error-buffer--cmacro|epg-context-error-buffer\n\t|epg-find-configuration|erlang-compile|erlang-edoc-mode|erlang-find-tag-other-window|erlang-find-tag|erlang-mode|erlang-shell|erldoc-apropos\n\t|erldoc-browse-topic|erldoc-browse|erldoc-eldoc-function|etags--xref-backend|eval-expression-get-print-arguments\n\t|event-line-count|face-list-p|facemenu-set-charset|faces--attribute-at-point|faceup-clean-buffer|faceup-defexplainer|faceup-render-view-buffer\n\t|faceup-view-buffer|faceup-write-file|fic-mode|file-attribute-access-time|file-attribute-collect|file-attribute-device-number|file-attribute-group-id\n\t|file-attribute-inode-number|file-attribute-link-number|file-attribute-modes|file-attribute-modification-time|file-attribute-size\n\t|file-attribute-status-change-time|file-attribute-type|file-attribute-user-id|file-local-name|file-name-case-insensitive-p|file-name-quoted-p\n\t|file-name-quote|file-name-unquote|file-system-info|filepos-to-bufferpos--dos|filepos-to-bufferpos|files--ask-user-about-large-file\n\t|files--ensure-directory|files--force|files--make-magic-temp-file|files--message|files--name-absolute-system-p|files--splice-dirname-file\n\t|fill-polish-nobreak-p|find-function-on-key-other-frame|find-function-on-key-other-window|find-library-other-frame|find-library-other-window\n\t|fixnump|flymake-cc|flymake-diag-region|flymake-diagnostics|flymake-make-diagnostic|follow-scroll-down-window|follow-scroll-up-window\n\t|font-lock--remove-face-from-text-property|form-feed-mode|format-message|forth-block-mode|forth-eval-defun|forth-eval-last-expression-display-output\n\t|forth-eval-last-expression|forth-eval-region|forth-eval|forth-interaction-send|forth-kill|forth-load-file|forth-mode|forth-restart|forth-see\n\t|forth-switch-to-output-buffer|forth-switch-to-source-buffer|forth-words|fortune-message|forward-to-word|forward-word-strictly|frame--size-history\n\t|frame-after-make-frame|frame-ancestor-p|frame-creation-function|frame-edges|frame-focus-state|frame-geometry|frame-inner-height|frame-inner-width\n\t|frame-internal-border-width|frame-list-z-order|frame-monitor-attribute|frame-monitor-geometry|frame-monitor-workarea|frame-native-height\n\t|frame-native-width|frame-outer-height|frame-outer-width|frame-parent|frame-position|frame-restack|frame-size-changed-p|func-arity\n\t|generic--normalize-comments|generic-bracket-support|generic-mode-set-comments|generic-set-comment-syntax|generic-set-comment-vars\n\t|get-variable-watchers|gfm-mode|gfm-view-mode|ghc-core-create-core|ghc-core-mode|ghci-script-mode|git-commit--save-and-exit|git-commit-ack\n\t|git-commit-cc|git-commit-committer-email|git-commit-committer-name|git-commit-commit|git-commit-find-pseudo-header-position|git-commit-first-env-var\n\t|git-commit-font-lock-diff|git-commit-git-config-var|git-commit-insert-header-as-self|git-commit-insert-header|git-commit-mode|git-commit-reported\n\t|git-commit-review|git-commit-signoff|git-commit-test|git-define-git-commit-self|git-define-git-commit|gitattributes-mode--highlight-1st-field\n\t|gitattributes-mode-backward-field|gitattributes-mode-eldoc|gitattributes-mode-forward-field|gitattributes-mode-help|gitattributes-mode-menu\n\t|gitattributes-mode|gitconfig-indent-line|gitconfig-indentation-string|gitconfig-line-indented-p|gitconfig-mode|gitconfig-point-in-indentation-p\n\t|gitignore-mode|global-aggressive-indent-mode-check-buffers|global-aggressive-indent-mode-cmhh|global-aggressive-indent-mode-enable-in-buffers\n\t|global-aggressive-indent-mode|global-display-line-numbers-mode|global-eldoc-mode-check-buffers|global-eldoc-mode-cmhh|global-eldoc-mode-enable-in-buffers\n\t|glsl-mode|gnutls-asynchronous-parameters|gnutls-ciphers|gnutls-digests|gnutls-hash-digest|gnutls-hash-mac|gnutls-macs|gnutls-symmetric-decrypt\n\t|gnutls-symmetric-encrypt|go-download-play|go-mode|godoc|gofmt-before-save|gui-backend-get-selection|gui-backend-selection-exists-p\n\t|gui-backend-selection-owner-p|gui-backend-set-selection|gv-delay-error|gv-setter|gv-synthetic-place|hack-connection-local-variables-apply\n\t|handle-args-function|handle-move-frame|hash-table-empty-p|haskell-align-imports|haskell-c2hs-mode|haskell-cabal-get-dir|haskell-cabal-get-field\n\t|haskell-cabal-mode|haskell-cabal-visit-file|haskell-collapse-mode|haskell-compile|haskell-completions-completion-at-point|haskell-decl-scan-mode\n\t|haskell-describe|haskell-doc-current-info|haskell-doc-mode|haskell-doc-show-type|haskell-ds-create-imenu-index|haskell-forward-sexp|haskell-hayoo\n\t|haskell-hoogle-lookup-from-local|haskell-hoogle|haskell-indent-mode|haskell-indentation-mode|haskell-interactive-bring|haskell-interactive-kill\n\t|haskell-interactive-mode-echo|haskell-interactive-mode-reset-error|haskell-interactive-mode-return|haskell-interactive-mode-visit-error\n\t|haskell-interactive-switch|haskell-kill-session-process|haskell-menu|haskell-mode-after-save-handler|haskell-mode-find-uses|haskell-mode-generate-tags\n\t|haskell-mode-goto-loc|haskell-mode-jump-to-def-or-tag|haskell-mode-jump-to-def|haskell-mode-jump-to-tag|haskell-mode-show-type-at)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(haskell-mode-stylish-buffer|haskell-mode-tag-find|haskell-mode-view-news|haskell-mode|haskell-move-nested-left|haskell-move-nested-right\n\t|haskell-move-nested|haskell-navigate-imports-go|haskell-navigate-imports-return|haskell-navigate-imports|haskell-process-cabal-build\n\t|haskell-process-cabal-macros|haskell-process-cabal|haskell-process-cd|haskell-process-clear|haskell-process-do-info|haskell-process-do-type\n\t|haskell-process-interrupt|haskell-process-load-file|haskell-process-load-or-reload|haskell-process-minimal-imports|haskell-process-reload-devel-main\n\t|haskell-process-reload-file|haskell-process-reload|haskell-process-restart|haskell-process-show-repl-response|haskell-process-unignore|haskell-rgrep\n\t|haskell-session-all-modules|haskell-session-change-target|haskell-session-change|haskell-session-installed-modules|haskell-session-kill\n\t|haskell-session-maybe|haskell-session-process|haskell-session-project-modules|haskell-session|haskell-sort-imports|haskell-tab-indent-mode\n\t|haskell-version|hayoo|help--analyze-key|help--binding-undefined-p|help--docstring-quote|help--filter-info-list|help--load-prefixes|help--loaded-p\n\t|help--make-usage-docstring|help--make-usage|help--read-key-sequence|help--symbol-completion-table|help-definition-prefixes|help-fns--analyze-function\n\t|help-fns-function-description-header|help-fns-short-filename|highlight-uses-mode|hoogle|hyperspec-lookup|ibuffer-jump|ido-dired-other-frame\n\t|ido-dired-other-window|ido-display-buffer-other-frame|ido-find-alternate-file-other-window|if-let\\*|image-dired-minor-mode|image-mode-to-text\n\t|indent--default-inside-comment|indent--funcall-widened|indent-region-line-by-line|indent-relative-first-indent-point|inferior-erlang|inferior-lfe-mode\n\t|inferior-lfe|ini-mode|insert-directory-clean|insert-directory-wildcard-in-dir-p|interactive-haskell-mode|internal--compiler-macro-cXXr\n\t|internal--syntax-propertize|internal-auto-fill|internal-default-interrupt-process|internal-echo-keystrokes-prefix|internal-handle-focus-in\n\t|isearch--describe-regexp-mode|isearch--describe-word-mode|isearch--lax-regexp-function-p|isearch--momentary-message|isearch--yank-char-or-syntax\n\t|isearch-define-mode-toggle|isearch-lazy-highlight-start|isearch-string-propertize|isearch-toggle-char-fold|isearch-update-from-string-properties\n\t|isearch-xterm-paste|isearch-yank-symbol-or-char|jison-mode|jit-lock--run-functions|js-jsx-mode|js2-highlight-unused-variables-mode\n\t|js2-imenu-extras-mode|js2-imenu-extras-setup|js2-jsx-mode|js2-minor-mode|js2-mode|json--check-position|json--decode-utf-16-surrogates\n\t|json--plist-reverse|json--plist-to-alist|json--record-path|json-advance--inliner|json-path-to-position|json-peek--inliner|json-pop--inliner\n\t|json-pretty-print-buffer-ordered|json-pretty-print-ordered|json-readtable-dispatch|json-skip-whitespace--inliner|kill-current-buffer\n\t|kmacro-keyboard-macro-p|kmacro-p|kqueue-add-watch|kqueue-rm-watch|kqueue-valid-p|langdoc-call-fun|langdoc-define-help-mode|langdoc-if-let\n\t|langdoc-insert-link|langdoc-matched-strings|langdoc-while-let|lcms-cam02-ucs|lcms-cie-de2000|lcms-jab->jch|lcms-jch->jab|lcms-jch->xyz\n\t|lcms-temp->white-point|lcms-xyz->jch|lcms2-available-p|less-css-mode|let-when-compile|lfe-indent-function|lfe-mode|lgstring-remove-glyph\n\t|libxml-available-p|line-number-display-width|lisp--el-match-keyword|lisp--el-non-funcall-position-p|lisp-adaptive-fill|lisp-indent-calc-next\n\t|lisp-indent-initial-state|lisp-indent-region|lisp-indent-state-p--cmacro|lisp-indent-state-ppss--cmacro|lisp-indent-state-ppss-point--cmacro\n\t|lisp-indent-state-ppss-point|lisp-indent-state-ppss|lisp-indent-state-p|lisp-indent-state-stack--cmacro|lisp-indent-state-stack|lisp-ppss\n\t|list-timers|literate-haskell-mode|load-user-init-file|loadhist-unload-element|logcount|lread--substitute-object-in-subtree|macroexp-macroexpand\n\t|macroexp-parse-body|macrostep-c-mode-hook|macrostep-expand|macrostep-mode|major-mode-restore|major-mode-suspend|make-condition-variable\n\t|make-empty-file|make-finalizer|make-mutex|make-nearby-temp-file|make-pipe-process|make-process|make-record|make-temp-file-internal|make-thread\n\t|make-xref-elisp-location--cmacro|make-xref-elisp-location|make-yas--exit--cmacro|make-yas--exit|make-yas--field--cmacro|make-yas--field\n\t|make-yas--mirror--cmacro|make-yas--mirror|make-yas--snippet--cmacro|make-yas--snippet|make-yas--table--cmacro|make-yas--table|map--apply-alist\n\t|map--apply-array|map--apply-hash-table|map--do-alist|map--do-array|map--into-hash-table|map--make-pcase-bindings|map--make-pcase-patterns\n\t|map--pcase-macroexpander|map--put|map-apply|map-contains-key|map-copy|map-delete|map-do|map-elt|map-empty-p|map-every-p|map-filter|map-into\n\t|map-keys-apply|map-keys|map-length|map-let|map-merge-with|map-merge|map-nested-elt|map-pairs|map-put|map-remove|map-some|map-values-apply\n\t|map-values|mapbacktrace|mapp|mark-beginning-of-buffer|mark-end-of-buffer|markdown-live-preview-mode|markdown-mode|markdown-view-mode\n\t|mc-hide-unmatched-lines-mode|mc\\/add-cursor-on-click|mc\\/edit-beginnings-of-lines|mc\\/edit-ends-of-lines|mc\\/edit-lines|mc\\/insert-letters\n\t|mc\\/insert-numbers|mc\\/mark-all-dwim|mc\\/mark-all-in-region-regexp|mc\\/mark-all-in-region|mc\\/mark-all-like-this-dwim\n\t|mc\\/mark-all-like-this-in-defun|mc\\/mark-all-like-this|mc\\/mark-all-symbols-like-this-in-defun|mc\\/mark-all-symbols-like-this\n\t|mc\\/mark-all-words-like-this-in-defun|mc\\/mark-all-words-like-this|mc\\/mark-more-like-this-extended|mc\\/mark-next-like-this-word\n\t|mc\\/mark-next-like-this|mc\\/mark-next-lines|mc\\/mark-next-symbol-like-this|mc\\/mark-next-word-like-this|mc\\/mark-pop\n\t|mc\\/mark-previous-like-this-word|mc\\/mark-previous-like-this|mc\\/mark-previous-lines|mc\\/mark-previous-symbol-like-this\n\t|mc\\/mark-previous-word-like-this|mc\\/mark-sgml-tag-pair|mc\\/reverse-regions|mc\\/skip-to-next-like-this|mc\\/skip-to-previous-like-this\n\t|mc\\/sort-regions|mc\\/toggle-cursor-on-click|mc\\/unmark-next-like-this|mc\\/unmark-previous-like-this|mc\\/vertical-align-with-space\n\t|mc\\/vertical-align|menu-bar-bottom-and-right-window-divider|menu-bar-bottom-window-divider|menu-bar-display-line-numbers-mode\n\t|menu-bar-goto-uses-etags-p|menu-bar-no-window-divider|menu-bar-right-window-divider|menu-bar-window-divider-customize|mhtml-mode|midnight-mode\n\t|minibuffer-maybe-quote-filename|minibuffer-prompt-properties--setter|mm-images-in-region-p|mocha--get-callsite-name|mocha-attach-indium\n\t|mocha-check-debugger|mocha-compilation-filter|mocha-debug-at-point|mocha-debug-file|mocha-debug-project|mocha-debugger-get|mocha-debugger-name-p\n\t|mocha-debug|mocha-find-current-test|mocha-find-project-root|mocha-generate-command|mocha-list-of-strings-p|mocha-make-imenu-alist|mocha-opts-file\n\t|mocha-realgud:nodejs-attach|mocha-run|mocha-test-at-point|mocha-test-file|mocha-test-project|mocha-toggle-imenu-function|mocha-walk-up-to-it\n\t|mode-line-default-help-echo|module-function-p|module-load|mouse--click-1-maybe-follows-link|mouse-absolute-pixel-position|mouse-drag-and-drop-region\n\t|mouse-drag-bottom-edge|mouse-drag-bottom-left-corner|mouse-drag-bottom-right-corner|mouse-drag-frame|mouse-drag-left-edge|mouse-drag-right-edge\n\t|mouse-drag-top-edge|mouse-drag-top-left-corner|mouse-drag-top-right-corner|mouse-resize-frame|move-text--at-first-line-p)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(move-text--at-last-line-p|move-text--at-penultimate-line-p|move-text--last-line-is-just-newline|move-text--total-lines|move-text-default-bindings\n\t|move-text-down|move-text-line-down|move-text-line-up|move-text-region-down|move-text-region-up|move-text-region|move-text-up\n\t|move-to-window-group-line|mule--ucs-names-annotation|multiple-cursors-mode|mutex-lock|mutex-name|mutex-unlock|mutexp|nasm-mode|newlisp-mode\n\t|newlisp-show-repl|next-error-buffer-on-selected-frame|next-error-found|next-error-select-buffer|ninja-mode|obarray-get|obarray-make|obarray-map\n\t|obarray-put|obarray-remove|obarray-size|obarrayp|occur-regexp-descr|org-columns-insert-dblock|org-duration-from-minutes|org-duration-h:mm-only-p\n\t|org-duration-p|org-duration-set-regexps|org-duration-to-minutes|org-lint|package--activate-autoloads-and-load-path|package--add-to-compatibility-table\n\t|package--append-to-alist|package--autoloads-file-name|package--build-compatibility-table|package--check-signature-content\n\t|package--download-and-read-archives|package--find-non-dependencies|package--get-deps|package--incompatible-p|package--load-files-for-activation\n\t|package--newest-p|package--prettify-quick-help-key|package--print-help-section|package--quickstart-maybe-refresh|package--read-pkg-desc\n\t|package--removable-packages|package--remove-hidden|package--save-selected-packages|package--sort-by-dependence|package--sort-deps-in-alist\n\t|package--update-downloads-in-progress|package--update-selected-packages|package--used-elsewhere-p|package--user-installed-p|package--user-selected-p\n\t|package--with-response-buffer|package-activate-all|package-archive-priority|package-autoremove|package-delete-button-action|package-desc-priority-version\n\t|package-desc-priority|package-dir-info|package-install-selected-packages|package-menu--find-and-notify-upgrades|package-menu--list-to-prompt\n\t|package-menu--mark-or-notify-upgrades|package-menu--mark-upgrades-1|package-menu--partition-transaction|package-menu--perform-transaction\n\t|package-menu--populate-new-package-list|package-menu--post-refresh|package-menu--print-info-simple|package-menu--prompt-transaction-p\n\t|package-menu-hide-package|package-menu-mode-menu|package-menu-toggle-hiding|package-quickstart-refresh|package-reinstall|pcase--edebug-match-macro\n\t|pcase--make-docstring|pcase-lambda|pcomplete\\/find|perl-flymake|picolisp-mode|picolisp-repl-mode|picolisp-repl|pixel-scroll-mode\n\t|pos-visible-in-window-group-p|pov-mode|powershell-mode|powershell|prefix-command-preserve-state|prefix-command-update\n\t|prettify-symbols--post-command-hook|prettify-symbols-default-compose-p|print--preprocess|process-thread|prog-first-column|project-current\n\t|project-find-file|project-find-regexp|project-or-external-find-file|project-or-external-find-regexp|proper-list-p|provided-mode-derived-p\n\t|pulse-momentary-highlight-one-line|pulse-momentary-highlight-region|quelpa|query-replace--split-string|radix-tree--insert|radix-tree--lookup\n\t|radix-tree--prefixes|radix-tree--remove|radix-tree--subtree|radix-tree-count|radix-tree-from-map|radix-tree-insert|radix-tree-iter-mappings\n\t|radix-tree-iter-subtrees|radix-tree-leaf--pcase-macroexpander|radix-tree-lookup|radix-tree-prefixes|radix-tree-subtree|read-answer|read-multiple-choice\n\t|readable-foreground-color|recenter-window-group|recentf-mode|recode-file-name|recode-region|record-window-buffer|recordp|record|recover-file\n\t|recover-session-finish|recover-session|recover-this-file|rectangle-mark-mode|rectangle-number-lines|rectangular-region-mode|redirect-debugging-output\n\t|redisplay--pre-redisplay-functions|redisplay--update-region-highlight|redraw-modeline|refill-mode|reftex-all-document-files|reftex-citation\n\t|reftex-index-phrases-mode|reftex-isearch-minor-mode|reftex-mode|reftex-reset-scanning-information|regexp-builder|regexp-opt-group|region-active-p\n\t|region-bounds|region-modifiable-p|region-noncontiguous-p|register-ccl-program|register-code-conversion-map|register-definition-prefixes\n\t|register-describe-oneline|register-input-method|register-preview-default|register-preview|register-swap-out|register-to-point|register-val-describe\n\t|register-val-insert|register-val-jump-to|registerv--make--cmacro|registerv--make|registerv-data--cmacro|registerv-data|registerv-insert-func--cmacro\n\t|registerv-insert-func|registerv-jump-func--cmacro|registerv-jump-func|registerv-make|registerv-p--cmacro|registerv-print-func--cmacro\n\t|registerv-print-func|registerv-p|remember-clipboard|remember-diary-extract-entries|remember-notes|remember-other-frame|remember|remove-variable-watcher\n\t|remove-yank-excluded-properties|rename-uniquely|repeat-complex-command|repeat-matching-complex-command|repeat|replace--push-stack|replace-buffer-contents\n\t|replace-dehighlight|replace-eval-replacement|replace-highlight|replace-loop-through-replacements|replace-match-data|replace-match-maybe-edit\n\t|replace-match-string-symbols|replace-quote|replace-rectangle|replace-regexp|replace-search|replace-string|report-emacs-bug|report-errors\n\t|reporter-submit-bug-report|reposition-window|repunctuate-sentences|reset-language-environment|reset-this-command-lengths|resize-mini-window-internal\n\t|resize-temp-buffer-window|reveal-mode|reverse-region|revert-buffer--default|revert-buffer-insert-file-contents--default-function\n\t|revert-buffer-with-coding-system|rfc2104-hash|rfc822-goto-eoh|rfn-eshadow-setup-minibuffer|rfn-eshadow-sifn-equal|rfn-eshadow-update-overlay\n\t|rgrep|right-char|right-word|rlogin|rmail-input|rmail-mode|rmail-movemail-variant-p|rmail-output-as-seen|run-erlang|run-forth|run-haskell\n\t|run-lfe|run-newlisp|run-sml|rust-mode|rx--pcase-macroexpander|save-mark-and-excursion--restore|save-mark-and-excursion--save|save-mark-and-excursion\n\t|save-place-local-mode|save-place-mode|scad-mode|search-forward-help-for-help|secondary-selection-exist-p|secondary-selection-from-region\n\t|secondary-selection-to-region|secure-hash-algorithms|sed-mode|selected-window-group|seq--activate-font-lock-keywords|seq--elt-safe|seq--into-list\n\t|seq--into-string|seq--into-vector|seq--make-pcase-bindings|seq--make-pcase-patterns|seq--pcase-macroexpander|seq-contains|seq-difference\n\t|seq-do-indexed|seq-find|seq-group-by|seq-intersection|seq-into-sequence|seq-into|seq-let|seq-map-indexed|seq-mapcat|seq-mapn|seq-max|seq-min\n\t|seq-partition|seq-position|seq-random-elt|seq-set-equal-p|seq-some|seq-sort-by|seqp|set--this-command-keys|set-binary-mode|set-buffer-redisplay\n\t|set-mouse-absolute-pixel-position|set-process-thread|set-rectangular-region-anchor|set-window-group-start|shell-command--save-pos-or-erase\n\t|shell-command--set-point-after-cmd|shift-number-down|shift-number-up|slime-connect|slime-lisp-mode-hook|slime-mode|slime-scheme-mode-hook\n\t|slime-selector|slime-setup|slime|smerge-refine-regions|sml-cm-mode|sml-lex-mode|sml-mode|sml-run|sml-yacc-mode|snippet-mode|spice-mode\n\t|split-window-no-error|sql-mariadb|ssh-authorized-keys-mode|ssh-config-mode|ssh-known-hosts-mode|startup--setup-quote-display|string-distance\n\t|string-greaterp|string-version-lessp|string>|subr--with-wrapper-hook-no-warnings|switch-to-haskell|sxhash-eql|sxhash-equal|sxhash-eq|syntax-ppss--data)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(tabulated-list--col-local-max-widths|tabulated-list--get-sorter|tabulated-list-header-overlay-p|tabulated-list-line-number-width\n\t|tabulated-list-watch-line-number-width|tabulated-list-window-scroll-function|terminal-init-xterm|thing-at-point--beginning-of-sexp\n\t|thing-at-point--end-of-sexp|thing-at-point--read-from-whole-string|thread--blocker|thread-alive-p|thread-handle-event|thread-join\n\t|thread-last-error|thread-live-p|thread-name|thread-signal|thread-yield|threadp|tildify-mode|tildify-space|toml-mode\n\t|tramp-archive-autoload-file-name-regexp|tramp-register-archive-file-name-handler|tty-color-24bit|turn-on-haskell-decl-scan|turn-on-haskell-doc-mode\n\t|turn-on-haskell-doc|turn-on-haskell-indentation|turn-on-haskell-indent|turn-on-haskell-unicode-input-method|typescript-mode|uncomment-region-default-1\n\t|undo--wrap-and-run-primitive-undo|undo-amalgamate-change-group|undo-auto--add-boundary|undo-auto--boundaries|undo-auto--boundary-ensure-timer\n\t|undo-auto--boundary-timer|undo-auto--ensure-boundary|undo-auto--last-boundary-amalgamating-number|undo-auto--needs-boundary-p\n\t|undo-auto--undoable-change|undo-auto-amalgamate|universal-argument--description|universal-argument--preserve|upcase-char\n\t|upcase-dwim|url-asynchronous--cmacro|url-asynchronous|url-directory-files|url-domain|url-file-attributes|url-file-directory-p\n\t|url-file-executable-p|url-file-exists-p|url-file-handler-identity|url-file-name-all-completions|url-file-name-completion|url-file-symlink-p\n\t|url-file-truename|url-file-writable-p|url-handler-directory-file-name|url-handler-expand-file-name|url-handler-file-name-directory\n\t|url-handler-file-remote-p|url-handler-unhandled-file-name-directory|url-handlers-create-wrapper|url-handlers-set-buffer-mode|url-insert-buffer-contents\n\t|url-insert|url-run-real-handler|user-ptrp|userlock--ask-user-about-supersession-threat|vc-message-unresolved-conflicts|vc-print-branch-log|vc-push\n\t|vc-refresh-state|version-control-safe-local-p|vimrc-mode|wavefront-obj-mode|when-let\\*|window--adjust-process-windows\n\t|window--even-window-sizes|window--make-major-side-window-next-to|window--make-major-side-window|window--process-window-list\n\t|window--sides-check-failed|window--sides-check|window--sides-reverse-all|window--sides-reverse-frame|window--sides-reverse-on-frame-p\n\t|window--sides-reverse-side|window--sides-reverse|window--sides-verticalize-frame|window--sides-verticalize|window-absolute-body-pixel-edges\n\t|window-absolute-pixel-position|window-adjust-process-window-size-largest|window-adjust-process-window-size-smallest|window-adjust-process-window-size\n\t|window-body-edges|window-body-pixel-edges|window-divider-mode-apply|window-divider-mode|window-divider-width-valid-p|window-font-height|window-font-width\n\t|window-group-end|window-group-start|window-largest-empty-rectangle--disjoint-maximums|window-largest-empty-rectangle--maximums-1\n\t|window-largest-empty-rectangle--maximums|window-largest-empty-rectangle|window-lines-pixel-dimensions|window-main-window|window-max-chars-per-line\n\t|window-pixel-height-before-size-change|window-pixel-width-before-size-change|window-swap-states|window-system-initialization|window-toggle-side-windows\n\t|with-connection-local-profiles|with-mutex|x-load-color-file|xml-remove-comments|xref-backend-apropos|xref-backend-definitions\n\t|xref-backend-identifier-completion-table|xref-collect-matches|xref-elisp-location-file--cmacro|xref-elisp-location-file\n\t|xref-elisp-location-p--cmacro|xref-elisp-location-symbol--cmacro|xref-elisp-location-symbol|xref-elisp-location-type--cmacro\n\t|xref-elisp-location-type|xref-find-backend|xref-find-definitions-at-mouse|xref-make-elisp-location--cmacro|xref-marker-stack-empty-p\n\t|xterm--init-activate-get-selection|xterm--init-activate-set-selection|xterm--init-bracketed-paste-mode|xterm--init-focus-tracking\n\t|xterm--init-frame-title|xterm--init-modify-other-keys|xterm--pasted-text|xterm--push-map|xterm--query|xterm--read-event-for-query\n\t|xterm--report-background-handler|xterm--selection-char|xterm--suspend-tty-function|xterm--version-handler|xterm-maybe-set-dark-background-mode\n\t|xterm-paste|xterm-register-default-colors|xterm-rgb-convert-to-16bit|xterm-set-window-title-flag|xterm-set-window-title|xterm-translate-bracketed-paste\n\t|xterm-translate-focus-in|xterm-translate-focus-out|xterm-unset-window-title-flag|xwidget-webkit-browse-url|yaml-mode|yas--add-template\n\t|yas--advance-end-maybe|yas--advance-end-of-parents-maybe|yas--advance-start-maybe|yas--all-templates|yas--apply-transform|yas--auto-fill-wrapper\n\t|yas--auto-fill|yas--auto-next|yas--calculate-adjacencies|yas--calculate-group|yas--calculate-mirror-depth|yas--calculate-simple-fom-parentage\n\t|yas--check-commit-snippet|yas--collect-snippet-markers|yas--commit-snippet|yas--compute-major-mode-and-parents|yas--create-snippet-xrefs\n\t|yas--define-menu-1|yas--define-parents|yas--define-snippets-1|yas--define-snippets-2|yas--define|yas--delete-from-keymap|yas--delete-regions\n\t|yas--describe-pretty-table|yas--escape-string|yas--eval-condition|yas--eval-for-effect|yas--eval-for-string|yas--exit-marker--cmacro|yas--exit-marker\n\t|yas--exit-next--cmacro|yas--exit-next|yas--exit-p--cmacro|yas--exit-p|yas--expand-from-keymap-doc|yas--expand-from-trigger-key-doc\n\t|yas--expand-or-prompt-for-template|yas--expand-or-visit-from-menu|yas--fallback-translate-input|yas--fallback|yas--fetch|yas--field-contains-point-p\n\t|yas--field-end--cmacro|yas--field-end|yas--field-mirrors--cmacro|yas--field-mirrors|yas--field-modified-p--cmacro|yas--field-modified-p\n\t|yas--field-next--cmacro|yas--field-next|yas--field-number--cmacro|yas--field-number|yas--field-p--cmacro|yas--field-parent-field--cmacro\n\t|yas--field-parent-field|yas--field-parse-create|yas--field-probably-deleted-p|yas--field-p|yas--field-start--cmacro|yas--field-start\n\t|yas--field-text-for-display|yas--field-transform--cmacro|yas--field-transform|yas--field-update-display|yas--filter-templates-by-condition\n\t|yas--find-next-field|yas--finish-moving-snippets|yas--fom-end|yas--fom-next|yas--fom-parent-field|yas--fom-start|yas--format|yas--get-field-once\n\t|yas--get-snippet-tables|yas--get-template-by-uuid|yas--global-mode-reload-with-jit-maybe|yas--goto-saved-location|yas--guess-snippet-directories-1\n\t|yas--guess-snippet-directories|yas--indent-parse-create|yas--indent-region|yas--indent|yas--key-from-desc|yas--keybinding-beyond-yasnippet|yas--letenv\n\t|yas--load-directory-1|yas--load-directory-2|yas--load-pending-jits|yas--load-snippet-dirs|yas--load-yas-setup-file|yas--lookup-snippet-1\n\t|yas--make-control-overlay|yas--make-directory-maybe|yas--make-exit--cmacro|yas--make-exit|yas--make-field--cmacro|yas--make-field|yas--make-marker\n\t|yas--make-menu-binding|yas--make-mirror--cmacro|yas--make-mirror|yas--make-move-active-field-overlay|yas--make-move-field-protection-overlays\n\t|yas--make-snippet--cmacro|yas--make-snippet-table--cmacro|yas--make-snippet-table|yas--make-snippet|yas--make-template--cmacro|yas--make-template)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(yas--mark-this-and-children-modified|yas--markers-to-points|yas--maybe-clear-field-filter|yas--maybe-expand-from-keymap-filter\n\t|yas--maybe-expand-key-filter|yas--maybe-move-to-active-field|yas--menu-keymap-get-create|yas--message|yas--minor-mode-menu|yas--mirror-depth--cmacro\n\t|yas--mirror-depth|yas--mirror-end--cmacro|yas--mirror-end|yas--mirror-next--cmacro|yas--mirror-next|yas--mirror-p--cmacro|yas--mirror-parent-field--cmacro\n\t|yas--mirror-parent-field|yas--mirror-p|yas--mirror-start--cmacro|yas--mirror-start|yas--mirror-transform--cmacro|yas--mirror-transform\n\t|yas--mirror-update-display|yas--modes-to-activate|yas--move-to-field|yas--namehash-templates-alist|yas--on-buffer-kill|yas--on-field-overlay-modification\n\t|yas--on-protection-overlay-modification|yas--parse-template|yas--place-overlays|yas--points-to-markers|yas--post-command-handler\n\t|yas--prepare-snippets-for-move|yas--prompt-for-keys|yas--prompt-for-table|yas--prompt-for-template|yas--protect-escapes|yas--read-keybinding\n\t|yas--read-lisp|yas--read-table|yas--remove-misc-free-from-undo|yas--remove-template-by-uuid|yas--replace-all|yas--require-template-specific-condition-p\n\t|yas--restore-backquotes|yas--restore-escapes|yas--restore-marker-location|yas--restore-overlay-line-location|yas--restore-overlay-location|yas--safely-call-fun\n\t|yas--safely-run-hook|yas--save-backquotes|yas--save-restriction-and-widen|yas--scan-sexps|yas--schedule-jit|yas--show-menu-p|yas--simple-fom-create\n\t|yas--skip-and-clear-field-p|yas--skip-and-clear|yas--snapshot-marker-location|yas--snapshot-overlay-line-location|yas--snapshot-overlay-location\n\t|yas--snippet-active-field--cmacro|yas--snippet-active-field|yas--snippet-control-overlay--cmacro|yas--snippet-control-overlay|yas--snippet-create\n\t|yas--snippet-description-finish-runonce|yas--snippet-exit--cmacro|yas--snippet-exit|yas--snippet-expand-env--cmacro|yas--snippet-expand-env\n\t|yas--snippet-field-compare|yas--snippet-fields--cmacro|yas--snippet-fields|yas--snippet-find-field|yas--snippet-force-exit--cmacro|yas--snippet-force-exit\n\t|yas--snippet-id--cmacro|yas--snippet-id|yas--snippet-live-p|yas--snippet-map-markers|yas--snippet-next-id|yas--snippet-p--cmacro|yas--snippet-parse-create\n\t|yas--snippet-previous-active-field--cmacro|yas--snippet-previous-active-field|yas--snippet-p|yas--snippet-revive|yas--snippet-sort-fields\n\t|yas--snippets-at-point|yas--subdirs|yas--table-all-keys|yas--table-direct-keymap--cmacro|yas--table-direct-keymap|yas--table-get-create\n\t|yas--table-hash--cmacro|yas--table-hash|yas--table-mode|yas--table-name--cmacro|yas--table-name|yas--table-p--cmacro|yas--table-parents--cmacro\n\t|yas--table-parents|yas--table-p|yas--table-templates|yas--table-uuidhash--cmacro|yas--table-uuidhash|yas--take-care-of-redo|yas--template-can-expand-p\n\t|yas--template-condition--cmacro|yas--template-condition|yas--template-content--cmacro|yas--template-content|yas--template-expand-env--cmacro\n\t|yas--template-expand-env|yas--template-fine-group|yas--template-get-file|yas--template-group--cmacro|yas--template-group|yas--template-key--cmacro\n\t|yas--template-keybinding--cmacro|yas--template-keybinding|yas--template-key|yas--template-load-file--cmacro|yas--template-load-file\n\t|yas--template-menu-binding-pair--cmacro|yas--template-menu-binding-pair-get-create|yas--template-menu-binding-pair|yas--template-menu-managed-by-yas-define-menu\n\t|yas--template-name--cmacro|yas--template-name|yas--template-p--cmacro|yas--template-perm-group--cmacro|yas--template-perm-group|yas--template-pretty-list\n\t|yas--template-p|yas--template-save-file--cmacro|yas--template-save-file|yas--template-table--cmacro|yas--template-table|yas--template-uuid--cmacro\n\t|yas--template-uuid|yas--templates-for-key-at-point|yas--transform-mirror-parse-create|yas--undo-in-progress|yas--update-mirrors|yas--update-template-menu\n\t|yas--update-template|yas--visit-snippet-file-1|yas--warning|yas--watch-auto-fill|yas-abort-snippet|yas-about|yas-activate-extra-mode|yas-active-keys\n\t|yas-active-snippets|yas-auto-next|yas-choose-value|yas-compile-directory|yas-completing-prompt|yas-current-field|yas-deactivate-extra-mode\n\t|yas-default-from-field|yas-define-condition-cache|yas-define-menu|yas-define-snippets|yas-describe-table-by-namehash|yas-describe-tables\n\t|yas-direct-keymaps-reload|yas-dropdown-prompt|yas-escape-text|yas-exit-all-snippets|yas-exit-snippet|yas-expand-from-keymap|yas-expand-from-trigger-key\n\t|yas-expand-snippet|yas-expand|yas-field-value|yas-global-mode-check-buffers|yas-global-mode-cmhh|yas-global-mode-enable-in-buffers|yas-global-mode\n\t|yas-hippie-try-expand|yas-ido-prompt|yas-initialize|yas-insert-snippet|yas-inside-string|yas-key-to-value|yas-load-directory|yas-load-snippet-buffer-and-close\n\t|yas-load-snippet-buffer|yas-longest-key-from-whitespace|yas-lookup-snippet|yas-maybe-ido-prompt|yas-maybe-load-snippet-buffer|yas-minor-mode-on\n\t|yas-minor-mode-set-explicitly|yas-minor-mode|yas-new-snippet|yas-next-field-or-maybe-expand|yas-next-field-will-exit-p|yas-next-field|yas-no-prompt|yas-prev-field\n\t|yas-recompile-all|yas-reload-all|yas-selected-text|yas-shortest-key-until-whitespace|yas-skip-and-clear-field|yas-skip-and-clear-or-delete-char|yas-snippet-dirs\n\t|yas-snippet-mode-buffer-p|yas-substr|yas-text|yas-throw|yas-try-key-from-whitespace|yas-tryout-snippet|yas-unimplemented|yas-verify-value|yas-visit-snippet-file\n\t|yas-x-prompt|yas\\/abort-snippet|yas\\/about|yas\\/choose-value|yas\\/compile-directory|yas\\/completing-prompt|yas\\/default-from-field|yas\\/define-condition-cache\n\t|yas\\/define-menu|yas\\/define-snippets|yas\\/describe-tables|yas\\/direct-keymaps-reload|yas\\/dropdown-prompt|yas\\/exit-all-snippets|yas\\/exit-snippet\n\t|yas\\/expand-from-keymap|yas\\/expand-from-trigger-key|yas\\/expand-snippet|yas\\/expand|yas\\/field-value|yas\\/global-mode|yas\\/hippie-try-expand|yas\\/ido-prompt\n\t|yas\\/initialize|yas\\/insert-snippet|yas\\/inside-string|yas\\/key-to-value|yas\\/load-directory|yas\\/load-snippet-buffer|yas\\/minor-mode-on|yas\\/minor-mode\n\t|yas\\/new-snippet|yas\\/next-field-or-maybe-expand|yas\\/next-field|yas\\/no-prompt|yas\\/prev-field|yas\\/recompile-all|yas\\/reload-all|yas\\/selected-text\n\t|yas\\/skip-and-clear-or-delete-char|yas\\/snippet-dirs|yas\\/substr|yas\\/text|yas\\/throw|yas\\/tryout-snippet|yas\\/unimplemented|yas\\/verify-value\n\t|yas\\/visit-snippet-file|yas\\/x-prompt|yasnippet-unload-function|zap-up-to-char)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(abbrev-all-caps|abbrev-expand-function|abbrev-expansion|abbrev-file-name|abbrev-get|abbrev-insert|abbrev-map|abbrev-minor-mode-table-alist\n\t|abbrev-prefix-mark|abbrev-put|abbrev-start-location|abbrev-start-location-buffer|abbrev-symbol|abbrev-table-get|abbrev-table-name-list\n\t|abbrev-table-p|abbrev-table-put|abbreviate-file-name|abbrevs-changed|abort-recursive-edit|accept-change-group|accept-process-output\n\t|access-file|accessible-keymaps|acos|activate-change-group|activate-mark-hook|active-minibuffer-window|adaptive-fill-first-line-regexp\n\t|adaptive-fill-function|adaptive-fill-mode|adaptive-fill-regexp|add-face-text-property|add-function|add-hook|add-name-to-file\n\t|add-text-properties|add-to-history|add-to-invisibility-spec|add-to-list|add-to-ordered-list|adjust-window-trailing-edge|advice-add\n\t|advice-eval-interactive-spec|advice-function-mapc|advice-function-member-p|advice-mapc|advice-member-p|advice-remove|after-change-functions\n\t|after-change-major-mode-hook|after-find-file|after-init-hook|after-init-time|after-insert-file-functions|after-load-functions\n\t|after-make-frame-functions|after-revert-hook|after-save-hook|after-setting-font-hook|all-completions|append-to-file|apply-partially|apropos\n\t|aref|argv|arrayp|ascii-case-table|aset|ash|asin|ask-user-about-lock|ask-user-about-supersession-threat|assoc-default|assoc-string|assq\n\t|assq-delete-all|atan|atom|auto-coding-alist|auto-coding-functions|auto-coding-regexp-alist|auto-fill-chars|auto-fill-function|auto-hscroll-mode\n\t|auto-mode-alist|auto-raise-tool-bar-buttons|auto-resize-tool-bars|auto-save-default|auto-save-file-name-p|auto-save-hook|auto-save-interval\n\t|auto-save-list-file-name|auto-save-list-file-prefix|auto-save-mode|auto-save-timeout|auto-save-visited-file-name|auto-window-vscroll|autoload\n\t|autoload-do-load|autoloadp|back-to-indentation|backtrace|backtrace-debug|backtrace-frame|backup-buffer|backup-by-copying|backup-by-copying-when-linked\n\t|backup-by-copying-when-mismatch|backup-by-copying-when-privileged-mismatch|backup-directory-alist|backup-enable-predicate|backup-file-name-p\n\t|backup-inhibited|backward-button|backward-char|backward-delete-char-untabify|backward-delete-char-untabify-method|backward-list|backward-prefix-chars\n\t|backward-sexp|backward-to-indentation|backward-word|balance-windows|balance-windows-area|barf-if-buffer-read-only|base64-decode-region\n\t|base64-decode-string|base64-encode-region|base64-encode-string|batch-byte-compile|baud-rate|beep|before-change-functions|before-hack-local-variables-hook\n\t|before-init-hook|before-init-time|before-make-frame-hook|before-revert-hook|before-save-hook|beginning-of-buffer|beginning-of-defun\n\t|beginning-of-defun-function|beginning-of-line|bidi-display-reordering|bidi-paragraph-direction|bidi-string-mark-left-to-right\n\t|bindat-get-field|bindat-ip-to-string|bindat-length|bindat-pack|bindat-unpack|bitmap-spec-p|blink-cursor-alist|blink-matching-delay\n\t|blink-matching-open|blink-matching-paren|blink-matching-paren-distance|blink-paren-function|bobp|bolp|bool-vector-count-consecutive\n\t|bool-vector-count-population|bool-vector-exclusive-or|bool-vector-intersection|bool-vector-not|bool-vector-p|bool-vector-set-difference\n\t|bool-vector-subsetp|bool-vector-union|booleanp|boundp|buffer-access-fontified-property|buffer-access-fontify-functions|buffer-auto-save-file-format\n\t|buffer-auto-save-file-name|buffer-backed-up|buffer-base-buffer|buffer-chars-modified-tick|buffer-disable-undo|buffer-display-count|buffer-display-table\n\t|buffer-display-time|buffer-enable-undo|buffer-end|buffer-file-coding-system|buffer-file-format|buffer-file-name|buffer-file-number|buffer-file-truename\n\t|buffer-invisibility-spec|buffer-list|buffer-list-update-hook|buffer-live-p|buffer-local-value|buffer-local-variables|buffer-modified-p|buffer-modified-tick\n\t|buffer-name|buffer-name-history|buffer-narrowed-p|buffer-offer-save|buffer-quit-function|buffer-read-only|buffer-save-without-query|buffer-saved-size\n\t|buffer-size|buffer-stale-function|buffer-string|buffer-substring|buffer-substring-filters|buffer-substring-no-properties|buffer-swap-text|buffer-undo-list\n\t|bufferp|bury-buffer|button-activate|button-at|button-end|button-get|button-has-type-p|button-label|button-put|button-start|button-type|button-type-get\n\t|button-type-put|button-type-subtype-p|byte-boolean-vars|byte-code-function-p|byte-compile|byte-compile-dynamic|byte-compile-dynamic-docstrings\n\t|byte-compile-file|byte-recompile-directory|byte-to-position|byte-to-string|call-interactively|call-process|call-process-region|call-process-shell-command\n\t|called-interactively-p|cancel-change-group|cancel-debug-on-entry|cancel-timer|capitalize|capitalize-region|capitalize-word|case-fold-search|case-replace\n\t|case-table-p|category-docstring|category-set-mnemonics|category-table|category-table-p|ceiling|change-major-mode-after-body-hook|change-major-mode-hook\n\t|char-after|char-before|char-category-set|char-charset|char-code-property-description|char-displayable-p|char-equal|char-or-string-p|char-property-alias-alist\n\t|char-script-table|char-syntax|char-table-extra-slot|char-table-p|char-table-parent|char-table-range|char-table-subtype|char-to-string|char-width\n\t|char-width-table|characterp|charset-after|charset-list|charset-plist|charset-priority-list|charsetp|check-coding-system|check-coding-systems-region\n\t|checkdoc-minor-mode|cl|clear-abbrev-table|clear-image-cache|clear-string|clear-this-command-keys|clear-visited-file-modtime|clone-indirect-buffer\n\t|clrhash|coding-system-aliases|coding-system-change-eol-conversion|coding-system-change-text-conversion|coding-system-charset-list|coding-system-eol-type\n\t|coding-system-for-read|coding-system-for-write|coding-system-get|coding-system-list|coding-system-p|coding-system-priority-list|collapse-delayed-warnings\n\t|color-defined-p|color-gray-p|color-supported-p|color-values|combine-after-change-calls|combine-and-quote-strings|command-debug-status|command-error-function\n\t|command-execute|command-history|command-line|command-line-args|command-line-args-left|command-line-functions|command-line-processed|command-remapping\n\t|command-switch-alist|commandp|compare-buffer-substrings|compare-strings|compare-window-configurations|compile-defun|completing-read|completing-read-function\n\t|completion-at-point|completion-at-point-functions|completion-auto-help|completion-boundaries|completion-category-overrides|completion-extra-properties\n\t|completion-ignore-case|completion-ignored-extensions|completion-in-region|completion-regexp-list|completion-styles|completion-styles-alist\n\t|completion-table-case-fold|completion-table-dynamic|completion-table-in-turn|completion-table-merge|completion-table-subvert|completion-table-with-cache\n\t|completion-table-with-predicate|completion-table-with-quoting|completion-table-with-terminator|compute-motion|concat|cons-cells-consed|constrain-to-field\n\t|continue-process|controlling-tty-p|convert-standard-filename|coordinates-in-window-p|copy-abbrev-table|copy-category-table|copy-directory|copy-file\n\t|copy-hash-table|copy-keymap|copy-marker|copy-overlay|copy-region-as-kill|copy-sequence|copy-syntax-table|copysign|cos|count-lines|count-loop\n\t|count-screen-lines|count-words|create-file-buffer|create-fontset-from-fontset-spec|create-image|create-lockfiles|current-active-maps\n\t|current-bidi-paragraph-direction|current-buffer|current-case-table|current-column|current-fill-column|current-frame-configuration|current-global-map\n\t|current-idle-time|current-indentation|current-input-method|current-input-mode|current-justification|current-kill|current-left-margin|current-local-map\n\t|current-message|current-minor-mode-maps|current-prefix-arg|current-time|current-time-string|current-time-zone|current-window-configuration|current-word\n\t|cursor-in-echo-area|cursor-in-non-selected-windows|cursor-type|cust-print|custom-add-frequent-value|custom-initialize-delay|custom-known-themes\n\t|custom-reevaluate-setting|custom-set-faces|custom-set-variables|custom-theme-p|custom-theme-set-faces|custom-theme-set-variables\n\t|custom-unlispify-remove-prefixes|custom-variable-p|customize-package-emacs-version-alist|cygwin-convert-file-name-from-windows\n\t|cygwin-convert-file-name-to-windows|data-directory|date-leap-year-p|date-to-time|deactivate-mark|deactivate-mark-hook|debug|debug-ignored-errors\n\t|debug-on-entry|debug-on-error|debug-on-event|debug-on-message|debug-on-next-call|debug-on-quit|debug-on-signal|debugger|debugger-bury-or-kill|declare\n\t|declare-function|decode-char|decode-coding-inserted-region|decode-coding-region|decode-coding-string|decode-time|def-edebug-spec|defalias|default-boundp\n\t|default-directory|default-file-modes|default-frame-alist|default-input-method|default-justification|default-minibuffer-frame|default-process-coding-system\n\t|default-text-properties|default-value|define-abbrev|define-abbrev-table|define-alternatives|define-button-type|define-category|define-derived-mode\n\t|define-error|define-fringe-bitmap|define-generic-mode|define-globalized-minor-mode|define-hash-table-test|define-key|define-key-after|define-minor-mode\n\t|define-obsolete-face-alias|define-obsolete-function-alias|define-obsolete-variable-alias|define-package|define-prefix-command|defined-colors\n\t|defining-kbd-macro|defun-prompt-regexp|defvar-local|defvaralias|delay-mode-hooks|delayed-warnings-hook|delayed-warnings-list|delete\n\t|delete-and-extract-region|delete-auto-save-file-if-necessary|delete-auto-save-files|delete-backward-char|delete-blank-lines|delete-by-moving-to-trash\n\t|delete-char|delete-directory|delete-dups|delete-exited-processes|delete-field|delete-file|delete-frame|delete-frame-functions|delete-horizontal-space\n\t|delete-indentation|delete-minibuffer-contents|delete-old-versions|delete-other-windows|delete-overlay|delete-process|delete-region|delete-terminal\n\t|delete-terminal-functions|delete-to-left-margin|delete-trailing-whitespace|delete-window|delete-windows-on|delq|derived-mode-p|describe-bindings\n\t|describe-buffer-case-table|describe-categories|describe-current-display-table|describe-display-table|describe-mode|describe-prefix-bindings\n\t|describe-syntax|desktop-buffer-mode-handlers|desktop-save-buffer|destroy-fringe-bitmap|detect-coding-region|detect-coding-string|digit-argument\n\t|ding|dir-locals-class-alist|dir-locals-directory-cache|dir-locals-file|dir-locals-set-class-variables|dir-locals-set-directory-class|directory-file-name\n\t|directory-files|directory-files-and-attributes|dired-kept-versions|disable-command|disable-point-adjustment|disable-theme|disabled|disabled-command-function\n\t|disassemble|discard-input|display-backing-store|display-buffer|display-buffer-alist|display-buffer-at-bottom|display-buffer-base-action|display-buffer-below-selected\n\t|display-buffer-fallback-action|display-buffer-in-previous-window|display-buffer-no-window|display-buffer-overriding-action|display-buffer-pop-up-frame\n\t|display-buffer-pop-up-window|display-buffer-reuse-window|display-buffer-same-window|display-buffer-use-some-window|display-color-cells|display-color-p\n\t|display-completion-list|display-delayed-warnings|display-graphic-p|display-grayscale-p|display-images-p|display-message-or-buffer|display-mm-dimensions-alist\n\t|display-mm-height|display-mm-width|display-monitor-attributes-list|display-mouse-p|display-pixel-height|display-pixel-width|display-planes|display-popup-menus-p\n\t|display-save-under|display-screens|display-selections-p|display-supports-face-attributes-p|display-table-slot|display-visual-class|display-warning|dnd-protocol-alist\n\t|do-auto-save|doc-directory|documentation|documentation-property|dotimes-with-progress-reporter|double-click-fuzz|double-click-time|down-list|downcase|downcase-region\n\t|downcase-word|dump-emacs|dynamic-library-alist)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(easy-menu-define|easy-mmode-define-minor-mode|echo-area-clear-hook|echo-keystrokes|edebug|edebug-all-defs|edebug-all-forms\n\t|edebug-continue-kbd-macro|edebug-defun|edebug-display-freq-count|edebug-eval-macro-args|edebug-eval-top-level-form|edebug-global-break-condition\n\t|edebug-initial-mode|edebug-on-error|edebug-on-quit|edebug-print-circle|edebug-print-length|edebug-print-level|edebug-print-trace-after|edebug-print-trace-before\n\t|edebug-save-displayed-buffer-points|edebug-save-windows|edebug-set-global-break-condition|edebug-setup-hook|edebug-sit-for-seconds|edebug-temp-display-freq-count\n\t|edebug-test-coverage|edebug-trace|edebug-tracing|edebug-unwrap-results|edit-and-eval-command|electric-future-map|elt|emacs-build-time|emacs-init-time\n\t|emacs-lisp-docstring-fill-column|emacs-major-version|emacs-minor-version|emacs-pid|emacs-save-session-functions|emacs-session-restore|emacs-startup-hook\n\t|emacs-uptime|emacs-version|emulation-mode-map-alists|enable-command|enable-dir-local-variables|enable-local-eval|enable-local-variables\n\t|enable-multibyte-characters|enable-recursive-minibuffers|enable-theme|encode-char|encode-coding-region|encode-coding-string|encode-time|end-of-buffer\n\t|end-of-defun|end-of-defun-function|end-of-file|end-of-line|eobp|eolp|equal-including-properties|erase-buffer|error|error-conditions|error-message-string\n\t|esc-map|ESC-prefix|eval|eval-and-compile|eval-buffer|eval-current-buffer|eval-expression-debug-on-error|eval-expression-print-length\n\t|eval-expression-print-level|eval-minibuffer|eval-region|eval-when-compile|event-basic-type|event-click-count|event-convert-list|event-end|event-modifiers\n\t|event-start|eventp|ewoc-buffer|ewoc-collect|ewoc-create|ewoc-data|ewoc-delete|ewoc-enter-after|ewoc-enter-before|ewoc-enter-first|ewoc-enter-last\n\t|ewoc-filter|ewoc-get-hf|ewoc-goto-next|ewoc-goto-node|ewoc-goto-prev|ewoc-invalidate|ewoc-locate|ewoc-location|ewoc-map|ewoc-next|ewoc-nth|ewoc-prev\n\t|ewoc-refresh|ewoc-set-data|ewoc-set-hf|exec-directory|exec-path|exec-suffixes|executable-find|execute-extended-command|execute-kbd-macro|executing-kbd-macro\n\t|exit|exit-minibuffer|exit-recursive-edit|exp|expand-abbrev|expand-file-name|expt|extended-command-history|extra-keyboard-modifiers|face-all-attributes\n\t|face-attribute|face-attribute-relative-p|face-background|face-bold-p|face-differs-from-default-p|face-documentation|face-equal|face-font\n\t|face-font-family-alternatives|face-font-registry-alternatives|face-font-rescale-alist|face-font-selection-order|face-foreground|face-id|face-inverse-video-p\n\t|face-italic-p|face-list|face-name-history|face-remap-add-relative|face-remap-remove-relative|face-remap-reset-base|face-remap-set-base|face-remapping-alist\n\t|face-spec-set|face-stipple|face-underline-p|facemenu-keymap|facep|fboundp|fceiling|feature-unload-function|featurep|features|fetch-bytecode|ffloor\n\t|field-beginning|field-end|field-string|field-string-no-properties|file-accessible-directory-p|file-acl|file-already-exists|file-attributes|file-chase-links\n\t|file-coding-system-alist|file-directory-p|file-equal-p|file-error|file-executable-p|file-exists-p|file-expand-wildcards|file-extended-attributes\n\t|file-in-directory-p|file-local-copy|file-local-variables-alist|file-locked|file-locked-p|file-modes|file-modes-symbolic-to-number|file-name-absolute-p\n\t|file-name-all-completions|file-name-as-directory|file-name-base|file-name-coding-system|file-name-completion|file-name-directory|file-name-extension\n\t|file-name-handler-alist|file-name-history|file-name-nondirectory|file-name-sans-extension|file-name-sans-versions|file-newer-than-file-p|file-newest-backup\n\t|file-nlinks|file-notify-add-watch|file-notify-rm-watch|file-ownership-preserved-p|file-precious-flag|file-readable-p|file-regular-p|file-relative-name\n\t|file-remote-p|file-selinux-context|file-supersession|file-symlink-p|file-truename|file-writable-p|fill-column|fill-context-prefix\n\t|fill-forward-paragraph-function|fill-individual-paragraphs|fill-individual-varying-indent|fill-nobreak-predicate|fill-paragraph|fill-paragraph-function\n\t|fill-prefix|fill-region|fill-region-as-paragraph|fillarray|filter-buffer-substring|filter-buffer-substring-function|filter-buffer-substring-functions\n\t|find-auto-coding|find-backup-file-name|find-buffer-visiting|find-charset-region|find-charset-string|find-coding-systems-for-charsets|find-coding-systems-region\n\t|find-coding-systems-string|find-file|find-file-hook|find-file-literally|find-file-name-handler|find-file-noselect|find-file-not-found-functions|find-file-other-window\n\t|find-file-read-only|find-file-wildcards|find-font|find-image|find-operation-coding-system|first-change-hook|fit-frame-to-buffer|fit-frame-to-buffer-margins\n\t|fit-frame-to-buffer-sizes|fit-window-to-buffer|fit-window-to-buffer-horizontally|fixup-whitespace|float|float-e|float-output-format|float-pi|float-time\n\t|floatp|floats-consed|floor|fmakunbound|focus-follows-mouse|focus-in-hook|focus-out-hook|following-char|font-at|font-face-attributes|font-family-list|font-get\n\t|font-lock-add-keywords|font-lock-beginning-of-syntax-function|font-lock-builtin-face|font-lock-comment-delimiter-face|font-lock-comment-face\n\t|font-lock-constant-face|font-lock-defaults|font-lock-doc-face|font-lock-extend-after-change-region-function|font-lock-extra-managed-props\n\t|font-lock-fontify-buffer-function|font-lock-fontify-region-function|font-lock-function-name-face|font-lock-keyword-face|font-lock-keywords\n\t|font-lock-keywords-case-fold-search|font-lock-keywords-only|font-lock-mark-block-function|font-lock-multiline|font-lock-negation-char-face\n\t|font-lock-preprocessor-face|font-lock-remove-keywords|font-lock-string-face|font-lock-syntactic-face-function|font-lock-syntax-table|font-lock-type-face\n\t|font-lock-unfontify-buffer-function|font-lock-unfontify-region-function|font-lock-variable-name-face|font-lock-warning-face|font-put|font-spec\n\t|font-xlfd-name|fontification-functions|fontp|for|force-mode-line-update|force-window-update|format|format-alist|format-find-file|format-insert-file\n\t|format-mode-line|format-network-address|format-seconds|format-time-string|format-write-file|forward-button|forward-char|forward-comment|forward-line\n\t|forward-list|forward-sexp|forward-to-indentation|forward-word|frame-alpha-lower-limit|frame-auto-hide-function|frame-char-height|frame-char-width\n\t|frame-current-scroll-bars|frame-first-window|frame-height|frame-inherited-parameters|frame-list|frame-live-p|frame-monitor-attributes|frame-parameter\n\t|frame-parameters|frame-pixel-height|frame-pixel-width|frame-pointer-visible-p|frame-resize-pixelwise|frame-root-window|frame-selected-window|frame-terminal\n\t|frame-title-format|frame-visible-p|frame-width|framep|frexp|fringe-bitmaps-at-pos|fringe-cursor-alist|fringe-indicator-alist|fringes-outside-margins|fround\n\t|fset|ftp-login|ftruncate|function-get|functionp|fundamental-mode|fundamental-mode-abbrev-table|gap-position|gap-size|garbage-collect|garbage-collection-messages\n\t|gc-cons-percentage|gc-cons-threshold|gc-elapsed|gcs-done|generate-autoload-cookie|generate-new-buffer|generate-new-buffer-name|generated-autoload-file|get\n\t|get-buffer|get-buffer-create|get-buffer-process|get-buffer-window|get-buffer-window-list|get-byte|get-char-code-property|get-char-property\n\t|get-char-property-and-overlay|get-charset-property|get-device-terminal|get-file-buffer|get-internal-run-time|get-largest-window|get-load-suffixes\n\t|get-lru-window|get-pos-property|get-process|get-register|get-text-property|get-unused-category|get-window-with-predicate|getenv|gethash|global-abbrev-table\n\t|global-buffers-menu-map|global-disable-point-adjustment|global-key-binding|global-map|global-mode-string|global-set-key|global-unset-key|glyph-char|glyph-face\n\t|glyph-table|glyphless-char-display|glyphless-char-display-control|goto-char|goto-map|group-gid|group-real-gid|gv-define-expander|gv-define-setter\n\t|gv-define-simple-setter|gv-letplace|hack-dir-local-variables|hack-dir-local-variables-non-file-buffer|hack-local-variables|hack-local-variables-hook\n\t|handle-shift-selection|handle-switch-frame|hash-table-count|hash-table-p|hash-table-rehash-size|hash-table-rehash-threshold|hash-table-size|hash-table-test\n\t|hash-table-weakness|header-line-format|help-buffer|help-char|help-command|help-event-list|help-form|help-map|help-setup-xref|help-window-select\n\t|Helper-describe-bindings|Helper-help|Helper-help-map|history-add-new-input|history-delete-duplicates|history-length)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(icon-title-format|iconify-frame|identity|ignore|ignore-errors|ignore-window-parameters|ignored-local-variables|image-animate|image-animate-timer\n\t|image-cache-eviction-delay|image-current-frame|image-default-frame-delay|image-flush|image-format-suffixes|image-load-path|image-load-path-for-library\n\t|image-mask-p|image-minimum-frame-delay|image-multi-frame-p|image-show-frame|image-size|image-type-available-p|image-types|imagemagick-enabled-types\n\t|imagemagick-types|imagemagick-types-inhibit|imenu-add-to-menubar|imenu-case-fold-search|imenu-create-index-function|imenu-extract-index-name-function\n\t|imenu-generic-expression|imenu-prev-index-position-function|imenu-syntax-alist|inc|indent-according-to-mode|indent-code-rigidly|indent-for-tab-command\n\t|indent-line-function|indent-region|indent-region-function|indent-relative|indent-relative-maybe|indent-rigidly|indent-tabs-mode|indent-to|indent-to-left-margin\n\t|indicate-buffer-boundaries|indicate-empty-lines|indirect-function|indirect-variable|inhibit-default-init|inhibit-eol-conversion|inhibit-field-text-motion\n\t|inhibit-file-name-handlers|inhibit-file-name-operation|inhibit-iso-escape-detection|inhibit-local-variables-regexps|inhibit-modification-hooks|inhibit-null-byte-detection\n\t|inhibit-point-motion-hooks|inhibit-quit|inhibit-read-only|inhibit-splash-screen|inhibit-startup-echo-area-message|inhibit-startup-message|inhibit-startup-screen\n\t|inhibit-x-resources|init-file-user|initial-buffer-choice|initial-environment|initial-frame-alist|initial-major-mode|initial-scratch-message|initial-window-system\n\t|input-decode-map|input-method-alist|input-method-function|input-pending-p|insert|insert-abbrev-table-description|insert-and-inherit|insert-before-markers\n\t|insert-before-markers-and-inherit|insert-buffer|insert-buffer-substring|insert-buffer-substring-as-yank|insert-buffer-substring-no-properties|insert-button\n\t|insert-char|insert-default-directory|insert-directory|insert-directory-program|insert-file-contents|insert-file-contents-literally|insert-for-yank|insert-image\n\t|insert-register|insert-sliced-image|insert-text-button|installation-directory|integer-or-marker-p|integerp|interactive-form|intern|intern-soft|interpreter-mode-alist\n\t|interprogram-cut-function|interprogram-paste-function|interrupt-process|intervals-consed|invalid-function|invalid-read-syntax|invalid-regexp|invert-face\n\t|invisible-p|invocation-directory|invocation-name|isnan|jit-lock-register|jit-lock-unregister|just-one-space|justify-current-line|kbd|kbd-macro-termination-hook\n\t|kept-new-versions|kept-old-versions|key-binding|key-description|key-translation-map|keyboard-coding-system|keyboard-quit|keyboard-translate|keyboard-translate-table\n\t|keymap-parent|keymap-prompt|keymapp|keywordp|kill-all-local-variables|kill-append|kill-buffer|kill-buffer-hook|kill-buffer-query-functions|kill-emacs|kill-emacs-hook\n\t|kill-emacs-query-functions|kill-local-variable|kill-new|kill-process|kill-read-only-ok|kill-region|kill-ring|kill-ring-max|kill-ring-yank-pointer|kmacro-keymap\n\t|last-abbrev|last-abbrev-location|last-abbrev-text|last-buffer|last-coding-system-used|last-command|last-command-event|last-event-frame|last-input-event\n\t|last-kbd-macro|last-nonmenu-event|last-prefix-arg|last-repeatable-command|lax-plist-get|lax-plist-put|lazy-completion-table|ldexp|left-fringe-width|left-margin\n\t|left-margin-width|lexical-binding|libxml-parse-html-region|libxml-parse-xml-region|line-beginning-position|line-end-position|line-move-ignore-invisible\n\t|line-number-at-pos|line-prefix|line-spacing|lisp-mode-abbrev-table|list-buffers-directory|list-charset-chars|list-fonts|list-load-path-shadows|list-processes\n\t|list-system-processes|listify-key-sequence|ln|load-average|load-file|load-file-name|load-file-rep-suffixes|load-history|load-in-progress|load-library|load-path\n\t|load-prefer-newer|load-read-function|load-suffixes|load-theme|local-abbrev-table|local-function-key-map|local-key-binding|local-set-key|local-unset-key\n\t|local-variable-if-set-p|local-variable-p|locale-coding-system|locale-info|locate-file|locate-library|locate-user-emacs-file|lock-buffer|log|logand|logb|logior\n\t|lognot|logxor|looking-at|looking-at-p|looking-back|lookup-key|lower-frame|lsh|lwarn|macroexpand|macroexpand-all|macrop|magic-fallback-mode-alist|magic-mode-alist\n\t|mail-host-address|major-mode|make-abbrev-table|make-auto-save-file-name|make-backup-file-name|make-backup-file-name-function|make-backup-files|make-bool-vector|make-button\n\t|make-byte-code|make-category-set|make-category-table|make-char-table|make-composed-keymap|make-directory|make-display-table|make-frame|make-frame-invisible\n\t|make-frame-on-display|make-frame-visible|make-glyph-code|make-hash-table|make-help-screen|make-indirect-buffer|make-keymap|make-local-variable|make-marker\n\t|make-network-process|make-obsolete|make-obsolete-variable|make-overlay|make-progress-reporter|make-ring|make-serial-process|make-sparse-keymap|make-string\n\t|make-symbol|make-symbolic-link|make-syntax-table|make-temp-file|make-temp-name|make-text-button|make-translation-table|make-translation-table-from-alist\n\t|make-translation-table-from-vector|make-variable-buffer-local|make-vector|makehash|makunbound|map-char-table|map-charset-chars|map-keymap|map-y-or-n-p|mapatoms\n\t|mapconcat|maphash|mark|mark-active|mark-even-if-inactive|mark-marker|mark-ring|mark-ring-max|marker-buffer|marker-insertion-type|marker-position|markerp|match-beginning\n\t|match-data|match-end|match-string|match-string-no-properties|match-substitute-replacement|max-char|max-image-size|max-lisp-eval-depth|max-mini-window-height\n\t|max-specpdl-size|maximize-window|md5|member-ignore-case|memory-full|memory-limit|memory-use-counts|memq|memql|menu-bar-file-menu|menu-bar-final-items|menu-bar-help-menu\n\t|menu-bar-options-menu|menu-bar-tools-menu|menu-bar-update-hook|menu-item|menu-prompt-more-char|merge-face-attribute|message|message-box|message-log-max|message-or-box\n\t|message-truncate-lines|messages-buffer|meta-prefix-char|minibuffer-allow-text-properties|minibuffer-auto-raise|minibuffer-complete|minibuffer-complete-and-exit\n\t|minibuffer-complete-word|minibuffer-completion-confirm|minibuffer-completion-help|minibuffer-completion-predicate|minibuffer-completion-table|minibuffer-confirm-exit-commands\n\t|minibuffer-contents|minibuffer-contents-no-properties|minibuffer-depth|minibuffer-exit-hook|minibuffer-frame-alist|minibuffer-help-form|minibuffer-history\n\t|minibuffer-inactive-mode|minibuffer-local-completion-map|minibuffer-local-filename-completion-map|minibuffer-local-map|minibuffer-local-must-match-map\n\t|minibuffer-local-ns-map|minibuffer-local-shell-command-map|minibuffer-message|minibuffer-message-timeout|minibuffer-prompt|minibuffer-prompt-end\n\t|minibuffer-prompt-width|minibuffer-scroll-window|minibuffer-selected-window|minibuffer-setup-hook|minibuffer-window|minibuffer-window-active-p|minibufferp\n\t|minimize-window|minor-mode-alist|minor-mode-key-binding|minor-mode-list|minor-mode-map-alist|minor-mode-overriding-map-alist|misc-objects-consed|mkdir|mod\n\t|mode-line-buffer-identification|mode-line-client|mode-line-coding-system-map|mode-line-column-line-number-mode-map|mode-line-format|mode-line-frame-identification\n\t|mode-line-input-method-map|mode-line-modes|mode-line-modified|mode-line-mule-info|mode-line-position|mode-line-process|mode-line-remote|mode-name|mode-specific-map\n\t|modify-all-frames-parameters|modify-category-entry|modify-frame-parameters|modify-syntax-entry|momentary-string-display|most-negative-fixnum|most-positive-fixnum\n\t|mouse-1-click-follows-link|mouse-appearance-menu-map|mouse-leave-buffer-hook|mouse-movement-p|mouse-on-link-p|mouse-pixel-position|mouse-position|mouse-position-function\n\t|mouse-wheel-down-event|mouse-wheel-up-event|move-marker|move-overlay|move-point-visually|move-to-column|move-to-left-margin|move-to-window-line|movemail|mule-keymap\n\t|multi-query-replace-map|multibyte-char-to-unibyte|multibyte-string-p|multibyte-syntax-as-symbol|multiple-frames|narrow-map|narrow-to-page|narrow-to-region|natnump\n\t|negative-argument|network-coding-system-alist|network-interface-info|network-interface-list|newline|newline-and-indent|next-button|next-char-property-change\n\t|next-complete-history-element|next-frame|next-history-element|next-matching-history-element|next-overlay-change|next-property-change|next-screen-context-lines\n\t|next-single-char-property-change|next-single-property-change|next-window|nlistp|no-byte-compile|no-catch|no-redraw-on-reenter|noninteractive|noreturn\n\t|normal-auto-fill-function|normal-backup-enable-predicate|normal-mode|not-modified|notifications-close-notification|notifications-get-capabilities\n\t|notifications-get-server-information|notifications-notify|num-input-keys|num-nonmacro-input-events|number-or-marker-p|number-sequence|number-to-string\n\t|numberp|obarray|one-window-p|only-global-abbrevs|open-dribble-file|open-network-stream|open-paren-in-column-0-is-defun-start|open-termscript|other-buffer\n\t|other-window|other-window-scroll-buffer|overflow-newline-into-fringe|overlay-arrow-position|overlay-arrow-string|overlay-arrow-variable-list|overlay-buffer\n\t|overlay-end|overlay-get|overlay-properties|overlay-put|overlay-recenter|overlay-start|overlayp|overlays-at|overlays-in|overriding-local-map\n\t|overriding-local-map-menu-flag|overriding-terminal-local-map|overwrite-mode|package-archive-upload-base|package-archives|package-initialize|package-upload-buffer\n\t|package-upload-file|page-delimiter|paragraph-separate|paragraph-start|parse-colon-path|parse-partial-sexp|parse-sexp-ignore-comments|parse-sexp-lookup-properties\n\t|path-separator|perform-replace|play-sound|play-sound-file|play-sound-functions|plist-get|plist-member|plist-put|point|point-marker|point-max|point-max-marker\n\t|point-min|point-min-marker|pop-mark|pop-to-buffer|pop-up-frame-alist|pop-up-frame-function|pop-up-frames|pop-up-windows|pos-visible-in-window-p|position-bytes\n\t|posix-looking-at|posix-search-backward|posix-search-forward|posix-string-match|posn-actual-col-row|posn-area|posn-at-point|posn-at-x-y|posn-col-row|posn-image\n\t|posn-object|posn-object-width-height|posn-object-x-y|posn-point|posn-string|posn-timestamp|posn-window|posn-x-y|posnp|post-command-hook|post-gc-hook\n\t|post-self-insert-hook|pp|pre-command-hook|pre-redisplay-function|preceding-char|prefix-arg|prefix-help-command|prefix-numeric-value|preloaded-file-list\n\t|prepare-change-group|previous-button|previous-char-property-change|previous-complete-history-element|previous-frame|previous-history-element\n\t|previous-matching-history-element|previous-overlay-change|previous-property-change|previous-single-char-property-change|previous-single-property-change\n\t|previous-window|primitive-undo|prin1-to-string|print-circle|print-continuous-numbering|print-escape-multibyte|print-escape-newlines|print-escape-nonascii\n\t|print-gensym|print-length|print-level|print-number-table|print-quoted|printable-chars|process-adaptive-read-buffering|process-attributes|process-buffer\n\t|process-coding-system|process-coding-system-alist|process-command|process-connection-type|process-contact|process-datagram-address|process-environment\n\t|process-exit-status|process-file|process-file-shell-command|process-file-side-effects|process-filter|process-get|process-id|process-kill-buffer-query-function\n\t|process-lines|process-list|process-live-p|process-mark|process-name|process-plist|process-put|process-query-on-exit-flag|process-running-child-p|process-send-eof\n\t|process-send-region|process-send-string|process-sentinel|process-status|process-tty-name|process-type|processp|prog-mode|prog-mode-hook|progress-reporter-done\n\t|progress-reporter-force-update|progress-reporter-update|propertize|provide|provide-theme|pure-bytes-used|purecopy|purify-flag|push-button|push-mark|put\n\t|put-char-code-property|put-charset-property|put-image|put-text-property|puthash|query-replace-history|query-replace-map|quietly-read-abbrev-file|quit-flag\n\t|quit-process|quit-restore-window|quit-window|raise-frame|random|rassq|rassq-delete-all|re-builder|re-search-backward|re-search-forward|read|read-buffer\n\t|read-buffer-completion-ignore-case|read-buffer-function|read-char|read-char-choice|read-char-exclusive|read-circle|read-coding-system|read-color\n\t|read-command|read-directory-name|read-event|read-expression-history|read-file-modes|read-file-name|read-file-name-completion-ignore-case\n\t|read-file-name-function|read-from-minibuffer|read-from-string|read-input-method-name|read-kbd-macro|read-key|read-key-sequence|read-key-sequence-vector\n\t|read-minibuffer|read-no-blanks-input|read-non-nil-coding-system|read-only-mode|read-passwd|read-quoted-char|read-regexp|read-regexp-defaults-function\n\t|read-shell-command|read-string|read-variable|real-last-command|recent-auto-save-p|recent-keys|recenter|recenter-positions|recenter-redisplay\n\t|recenter-top-bottom|recursion-depth|recursive-edit|redirect-frame-focus|redisplay|redraw-display|redraw-frame|regexp-history|regexp-opt|regexp-opt-charset\n\t|regexp-opt-depth|regexp-quote|region-beginning|region-end|register-alist|register-read-with-preview|reindent-then-newline-and-indent|remhash\n\t|remote-file-name-inhibit-cache|remove|remove-from-invisibility-spec|remove-function|remove-hook|remove-images|remove-list-of-text-properties\n\t|remove-overlays|remove-text-properties|remq|rename-auto-save-file|rename-buffer|rename-file|replace-buffer-in-windows|replace-match|replace-re-search-function\n\t|replace-regexp-in-string|replace-search-function|require|require-final-newline|restore-buffer-modified-p|resume-tty|resume-tty-functions|revert-buffer\n\t|revert-buffer-function|revert-buffer-in-progress-p|revert-buffer-insert-file-contents-function|revert-without-query|right-fringe-width|right-margin-width\n\t|ring-bell-function|ring-copy|ring-elements|ring-empty-p|ring-insert|ring-insert-at-beginning|ring-length|ring-p|ring-ref|ring-remove|ring-size\n\t|risky-local-variable-p|rm|round|run-at-time|run-hook-with-args|run-hook-with-args-until-failure|run-hook-with-args-until-success|run-hooks\n\t|run-mode-hooks|run-with-idle-timer)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)\n\t(safe-local-eval-forms|safe-local-variable-p|safe-local-variable-values|same-window-buffer-names|same-window-p\n\t|same-window-regexps|save-abbrevs|save-buffer|save-buffer-coding-system|save-current-buffer|save-excursion|save-match-data|save-restriction\n\t|save-selected-window|save-some-buffers|save-window-excursion|scalable-fonts-allowed|scan-lists|scan-sexps|scroll-bar-event-ratio|scroll-bar-mode\n\t|scroll-bar-scale|scroll-bar-width|scroll-conservatively|scroll-down|scroll-down-aggressively|scroll-down-command|scroll-error-top-bottom|scroll-left\n\t|scroll-margin|scroll-other-window|scroll-preserve-screen-position|scroll-right|scroll-step|scroll-up|scroll-up-aggressively|scroll-up-command\n\t|search-backward|search-failed|search-forward|search-map|search-spaces-regexp|seconds-to-time|secure-hash|select-frame|select-frame-set-input-focus\n\t|select-safe-coding-system|select-safe-coding-system-accept-default-p|select-window|selected-frame|selected-window|selection-coding-system\n\t|selective-display|selective-display-ellipses|self-insert-and-exit|self-insert-command|send-string-to-terminal|sentence-end|sentence-end-double-space\n\t|sentence-end-without-period|sentence-end-without-space|sequencep|serial-process-configure|serial-term|set-advertised-calling-convention|set-auto-coding\n\t|set-auto-mode|set-buffer|set-buffer-auto-saved|set-buffer-major-mode|set-buffer-modified-p|set-buffer-multibyte|set-case-syntax|set-case-syntax-delims\n\t|set-case-syntax-pair|set-case-table|set-category-table|set-char-table-extra-slot|set-char-table-parent|set-char-table-range|set-charset-priority\n\t|set-coding-system-priority|set-default|set-default-file-modes|set-display-table-slot|set-face-attribute|set-face-background|set-face-bold|set-face-font\n\t|set-face-foreground|set-face-inverse-video|set-face-italic|set-face-stipple|set-face-underline|set-file-acl|set-file-extended-attributes|set-file-modes\n\t|set-file-selinux-context|set-file-times|set-fontset-font|set-frame-configuration|set-frame-height|set-frame-parameter|set-frame-position|set-frame-selected-window\n\t|set-frame-size|set-frame-width|set-fringe-bitmap-face|set-input-method|set-input-mode|set-keyboard-coding-system|set-keymap-parent|set-left-margin|set-mark|set-marker\n\t|set-marker-insertion-type|set-match-data|set-minibuffer-window|set-mouse-pixel-position|set-mouse-position|set-network-process-option|set-process-buffer\n\t|set-process-coding-system|set-process-datagram-address|set-process-filter|set-process-plist|set-process-query-on-exit-flag|set-process-sentinel\n\t|set-register|set-right-margin|set-standard-case-table|set-syntax-table|set-terminal-coding-system|set-terminal-parameter|set-text-properties|set-transient-map\n\t|set-visited-file-modtime|set-visited-file-name|set-window-buffer|set-window-combination-limit|set-window-configuration|set-window-dedicated-p|set-window-display-table\n\t|set-window-fringes|set-window-hscroll|set-window-margins|set-window-next-buffers|set-window-parameter|set-window-point|set-window-prev-buffers|set-window-scroll-bars\n\t|set-window-start|set-window-vscroll|setenv|setplist|setq-default|setq-local|shell-command-history|shell-command-to-string|shell-quote-argument|show-help-function\n\t|shr-insert-document|shrink-window-if-larger-than-buffer|signal|signal-process|sin|single-key-description|sit-for|site-run-file|skip-chars-backward|skip-chars-forward\n\t|skip-syntax-backward|skip-syntax-forward|sleep-for|small-temporary-file-directory|smie-bnf->prec2|smie-close-block|smie-config|smie-config-guess|smie-config-local\n\t|smie-config-save|smie-config-set-indent|smie-config-show-indent|smie-down-list|smie-merge-prec2s|smie-prec2->grammar|smie-precs->prec2|smie-rule-bolp|smie-rule-hanging-p\n\t|smie-rule-next-p|smie-rule-parent|smie-rule-parent-p|smie-rule-prev-p|smie-rule-separator|smie-rule-sibling-p|smie-setup|Snarf-documentation|sort|sort-columns|sort-fields\n\t|sort-fold-case|sort-lines|sort-numeric-base|sort-numeric-fields|sort-pages|sort-paragraphs|sort-regexp-fields|sort-subr|special-event-map|special-form-p|special-mode\n\t|special-variable-p|split-height-threshold|split-string|split-string-and-unquote|split-string-default-separators|split-width-threshold|split-window|split-window-below\n\t|split-window-keep-point|split-window-preferred-function|split-window-right|split-window-sensibly|sqrt|standard-case-table|standard-category-table|standard-display-table\n\t|standard-input|standard-output|standard-syntax-table|standard-translation-table-for-decode|standard-translation-table-for-encode|start-file-process\n\t|start-file-process-shell-command|start-process|start-process-shell-command|stop-process|store-match-data|store-substring|string|string-as-multibyte|string-as-unibyte\n\t|string-bytes|string-chars-consed|string-equal|string-lessp|string-match|string-match-p|string-or-null-p|string-prefix-p|string-suffix-p|string-to-char|string-to-int\n\t|string-to-multibyte|string-to-number|string-to-syntax|string-to-unibyte|string-width|string<|string=|stringp|strings-consed|subr-arity|subrp|subst-char-in-region\n\t|substitute-command-keys|substitute-in-file-name|substitute-key-definition|substring|substring-no-properties|suppress-keymap|suspend-emacs|suspend-frame|suspend-hook\n\t|suspend-resume-hook|suspend-tty|suspend-tty-functions|switch-to-buffer|switch-to-buffer-other-frame|switch-to-buffer-other-window|switch-to-buffer-preserve-window-point\n\t|switch-to-next-buffer|switch-to-prev-buffer|switch-to-visible-buffer|sxhash|symbol-file|symbol-function|symbol-name|symbol-plist|symbol-value|symbolp|symbols-consed\n\t|syntax-after|syntax-begin-function|syntax-class|syntax-ppss|syntax-ppss-flush-cache|syntax-ppss-toplevel-pos|syntax-propertize-extend-region-functions|syntax-propertize-function\n\t|syntax-table|syntax-table-p|system-configuration|system-groups|system-key-alist|system-messages-locale|system-name|system-time-locale|system-type|system-users|tab-always-indent\n\t|tab-stop-list|tab-to-tab-stop|tab-width|tabulated-list-entries|tabulated-list-format|tabulated-list-init-header|tabulated-list-mode|tabulated-list-print|tabulated-list-printer\n\t|tabulated-list-revert-hook|tabulated-list-sort-key|tan|temacs|temp-buffer-setup-hook|temp-buffer-show-function|temp-buffer-show-hook|temp-buffer-window-setup-hook\n\t|temp-buffer-window-show-hook|temporary-file-directory|term-file-prefix|terminal-coding-system|terminal-list|terminal-live-p|terminal-name|terminal-parameter\n\t|terminal-parameters|terpri|test-completion|testcover-mark-all|testcover-next-mark|testcover-start|text-char-description|text-mode|text-mode-abbrev-table\n\t|text-properties-at|text-property-any|text-property-default-nonsticky|text-property-not-all|thing-at-point|this-command|this-command-keys|this-command-keys-shift-translated\n\t|this-command-keys-vector|this-original-command|three-step-help|time-add|time-less-p|time-subtract|time-to-day-in-year|time-to-days|timer-max-repeats\n\t|toggle-enable-multibyte-characters|tool-bar-add-item|tool-bar-add-item-from-menu|tool-bar-border|tool-bar-button-margin|tool-bar-button-relief\n\t|tool-bar-local-item-from-menu|tool-bar-map|top-level|tq-close|tq-create|tq-enqueue|track-mouse|transient-mark-mode|translate-region|translation-table-for-input\n\t|transpose-regions|truncate|truncate-lines|truncate-partial-width-windows|truncate-string-to-width|try-completion|tty-color-alist|tty-color-approximate|tty-color-clear\n\t|tty-color-define|tty-color-translate|tty-erase-char|tty-setup-hook|tty-top-frame|type-of|unbury-buffer|undefined|underline-minimum-offset|undo-ask-before-discard\n\t|undo-boundary|undo-in-progress|undo-limit|undo-outer-limit|undo-strong-limit|unhandled-file-name-directory|unibyte-char-to-multibyte|unibyte-string|unicode-category-table\n\t|unintern|universal-argument|universal-argument-map|unload-feature|unload-feature-special-hooks|unlock-buffer|unread-command-events|unsafep|up-list|upcase|upcase-initials\n\t|upcase-region|upcase-word|update-directory-autoloads|update-file-autoloads|use-empty-active-region|use-global-map|use-hard-newlines|use-local-map|use-region-p\n\t|user-emacs-directory|user-error|user-full-name|user-init-file|user-login-name|user-mail-address|user-real-login-name|user-real-uid|user-uid|values|vc-mode\n\t|vc-prefix-map|vconcat|vector|vector-cells-consed|vectorp|verify-visited-file-modtime|version-control|vertical-motion|vertical-scroll-bar|view-register\n\t|visible-bell|visible-frame-list|visited-file-modtime|void-function|void-text-area-pointer|waiting-for-user-input-p|walk-windows|warn|warning-fill-prefix\n\t|warning-levels|warning-minimum-level|warning-minimum-log-level|warning-prefix-function|warning-series|warning-suppress-log-types|warning-suppress-types\n\t|warning-type-format|where-is-internal|while-no-input|wholenump|widen|window-absolute-pixel-edges|window-at|window-body-height|window-body-size\n\t|window-body-width|window-bottom-divider-width|window-buffer|window-child|window-combination-limit|window-combination-resize|window-combined-p\n\t|window-configuration-change-hook|window-configuration-frame|window-configuration-p|window-current-scroll-bars|window-dedicated-p|window-display-table\n\t|window-edges|window-end|window-frame|window-fringes|window-full-height-p|window-full-width-p|window-header-line-height|window-hscroll|window-in-direction\n\t|window-inside-absolute-pixel-edges|window-inside-edges|window-inside-pixel-edges|window-left-child|window-left-column|window-line-height|window-list\n\t|window-live-p|window-margins|window-min-height|window-min-size|window-min-width|window-minibuffer-p|window-mode-line-height|window-next-buffers\n\t|window-next-sibling|window-parameter|window-parameters|window-parent|window-persistent-parameters|window-pixel-edges|window-pixel-height\n\t|window-pixel-left|window-pixel-top|window-pixel-width|window-point|window-point-insertion-type|window-prev-buffers|window-prev-sibling\n\t|window-resizable|window-resize|window-resize-pixelwise|window-right-divider-width|window-scroll-bar-width|window-scroll-bars|window-scroll-functions\n\t|window-setup-hook|window-size-change-functions|window-size-fixed|window-start|window-state-get|window-state-put|window-system\n\t|window-system-initialization-alist|window-text-change-functions|window-text-pixel-size|window-top-child|window-top-line|window-total-height\n\t|window-total-size|window-total-width|window-tree|window-valid-p|window-vscroll|windowp|with-case-table|with-coding-priority|with-current-buffer\n\t|with-current-buffer-window|with-demoted-errors|with-eval-after-load|with-help-window|with-local-quit|with-no-warnings|with-output-to-string\n\t|with-output-to-temp-buffer|with-selected-window|with-syntax-table|with-temp-buffer|with-temp-buffer-window|with-temp-file|with-temp-message\n\t|with-timeout|word-search-backward|word-search-backward-lax|word-search-forward|word-search-forward-lax|word-search-regexp|words-include-escapes\n\t|wrap-prefix|write-abbrev-file|write-char|write-contents-functions|write-file|write-file-functions|write-region|write-region-annotate-functions\n\t|write-region-post-annotation-function|wrong-number-of-arguments|wrong-type-argument|x-alt-keysym|x-alternatives-map|x-bitmap-file-path\n\t|x-close-connection|x-color-defined-p|x-color-values|x-defined-colors|x-display-color-p|x-display-list|x-dnd-known-types|x-dnd-test-function\n\t|x-dnd-types-alist|x-family-fonts|x-get-resource|x-get-selection|x-hyper-keysym|x-list-fonts|x-meta-keysym|x-open-connection|x-parse-geometry\n\t|x-pointer-shape|x-popup-dialog|x-popup-menu|x-resource-class|x-resource-name|x-sensitive-text-pointer-shape|x-server-vendor|x-server-version\n\t|x-set-selection|x-setup-function-keys|x-super-keysym|y-or-n-p|y-or-n-p-with-timeout|yank|yank-excluded-properties|yank-handled-properties\n\t|yank-pop|yank-undo-function|yes-or-no-p|zerop|zlib-available-p|zlib-decompress-region)\n(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t mocha--other-js2-imenu-function|mocha-command|mocha-debug-port|mocha-debuggers|mocha-debugger|mocha-environment-variables|mocha-imenu-functions\n\t|mocha-options|mocha-project-test-directory|mocha-reporter|mocha-test-definition-nodes|mocha-which-node|node-error-regexp-alist|node-error-regexp)\n(?=[\\s()]|$)',
          name: 'support.variable.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\tdefine-modify-macro|define-setf-method|defsetf|eval-when-compile|flet|labels|lexical-let\\*?|\n\t\n\tcl-(?:acons|adjoin|assert|assoc|assoc-if|assoc-if-not|block|caddr|callf|callf2|case|ceiling|check-type|coerce\n\t|compiler-macroexpand|concatenate|copy-list|count|count-if|count-if-not|decf|declaim|declare|define-compiler-macro\n\t|defmacro|defstruct|defsubst|deftype|defun|delete|delete-duplicates|delete-if|delete-if-not|destructuring-bind\n\t|do\\*?|do-all-symbols|do-symbols|dolist|dotimes|ecase|endp|equalp|etypecase|eval-when|evenp|every|fill|find|find-if\n\t|find-if-not|first|flet|float-limits|floor|function|gcd|gensym|gentemp|getf?|incf|intersection|isqrt|labels\n\t|lcm|ldiff|letf\\*?|list\\*|list-length|load-time-value|locally|loop|macrolet|make-random-state|map|mapc|mapcan\n\t|mapcar|mapcon|mapl|maplist|member|member-if|member-if-not|merge|minusp|mismatch|mod|multiple-value-bind\n\t|multiple-value-setq|nintersection|notany|notevery|nset-difference|nset-exclusive-or|nsublis|nsubst|nsubst-if\n\t|nsubst-if-not|nsubstitute|nsubstitute-if|nsubstitute-if-not|nunion|oddp|pairlis|plusp|position|position-if\n\t|position-if-not|prettyexpand|proclaim|progv|psetf|psetq|pushnew|random|random-state-p|rassoc|rassoc-if|rassoc-if-not\n\t|reduce|remf?|remove|remove-duplicates|remove-if|remove-if-not|remprop|replace|rest|return|return-from|rotatef\n\t|round|search|set-difference|set-exclusive-or|shiftf|some|sort|stable-sort|sublis|subseq|subsetp|subst|subst-if\n\t|subst-if-not|substitute|substitute-if|substitute-if-not|symbol-macrolet|tagbody|tailp|the|tree-equal|truncate\n\t|typecase|typep|union)\n)(?=[\\s()]|$)',
          name: 'support.function.cl-lib.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t \\*table--cell-backward-kill-paragraph|\\*table--cell-backward-kill-sentence|\\*table--cell-backward-kill-sexp|\\*table--cell-backward-kill-word\n\t|\\*table--cell-backward-paragraph|\\*table--cell-backward-sentence|\\*table--cell-backward-word|\\*table--cell-beginning-of-buffer\n\t|\\*table--cell-beginning-of-line|\\*table--cell-center-line|\\*table--cell-center-paragraph|\\*table--cell-center-region|\\*table--cell-clipboard-yank\n\t|\\*table--cell-copy-region-as-kill|\\*table--cell-dabbrev-completion|\\*table--cell-dabbrev-expand|\\*table--cell-delete-backward-char\n\t|\\*table--cell-delete-char|\\*table--cell-delete-region|\\*table--cell-describe-bindings|\\*table--cell-describe-mode|\\*table--cell-end-of-buffer\n\t|\\*table--cell-end-of-line|\\*table--cell-fill-paragraph|\\*table--cell-forward-paragraph|\\*table--cell-forward-sentence|\\*table--cell-forward-word\n\t|\\*table--cell-insert|\\*table--cell-kill-line|\\*table--cell-kill-paragraph|\\*table--cell-kill-region|\\*table--cell-kill-ring-save\n\t|\\*table--cell-kill-sentence|\\*table--cell-kill-sexp|\\*table--cell-kill-word|\\*table--cell-move-beginning-of-line|\\*table--cell-move-end-of-line\n\t|\\*table--cell-newline-and-indent|\\*table--cell-newline|\\*table--cell-open-line|\\*table--cell-quoted-insert|\\*table--cell-self-insert-command\n\t|\\*table--cell-yank-clipboard-selection|\\*table--cell-yank|\\*table--present-cell-popup-menu|-cvs-create-fileinfo--cmacro|-cvs-create-fileinfo\n\t|-cvs-flags-make--cmacro|-cvs-flags-make|1\\+|1-|1value|2C-associate-buffer|2C-associated-buffer|2C-autoscroll|2C-command|2C-dissociate\n\t|2C-enlarge-window-horizontally|2C-merge|2C-mode|2C-newline|2C-other|2C-shrink-window-horizontally|2C-split|2C-toggle-autoscroll\n\t|2C-two-columns|5x5-bol|5x5-cell|5x5-copy-grid|5x5-crack-mutating-best|5x5-crack-mutating-current|5x5-crack-randomly|5x5-crack-xor-mutate\n\t|5x5-crack|5x5-defvar-local|5x5-down|5x5-draw-grid-end|5x5-draw-grid|5x5-eol|5x5-first|5x5-flip-cell|5x5-flip-current|5x5-grid-to-vec\n\t|5x5-grid-value|5x5-last|5x5-left|5x5-log-init|5x5-log|5x5-made-move|5x5-make-move|5x5-make-mutate-best|5x5-make-mutate-current|5x5-make-new-grid\n\t|5x5-make-random-grid|5x5-make-random-solution|5x5-make-xor-with-mutation|5x5-mode-menu|5x5-mode|5x5-mutate-solution|5x5-new-game\n\t|5x5-play-solution|5x5-position-cursor|5x5-quit-game|5x5-randomize|5x5-right|5x5-row-value|5x5-set-cell|5x5-solve-rotate-left|5x5-solve-rotate-right\n\t|5x5-solve-suggest|5x5-solver|5x5-up|5x5-vec-to-grid|5x5-xor|5x5-y-or-n-p|5x5|Buffer-menu--pretty-file-name|Buffer-menu--pretty-name\n\t|Buffer-menu--unmark|Buffer-menu-1-window|Buffer-menu-2-window|Buffer-menu-backup-unmark|Buffer-menu-beginning|Buffer-menu-buffer\n\t|Buffer-menu-bury|Buffer-menu-delete-backwards|Buffer-menu-delete|Buffer-menu-execute|Buffer-menu-info-node-description|Buffer-menu-isearch-buffers-regexp\n\t|Buffer-menu-isearch-buffers|Buffer-menu-mark|Buffer-menu-marked-buffers|Buffer-menu-mode|Buffer-menu-mouse-select|Buffer-menu-multi-occur\n\t|Buffer-menu-no-header|Buffer-menu-not-modified|Buffer-menu-other-window|Buffer-menu-save|Buffer-menu-select|Buffer-menu-sort|Buffer-menu-switch-other-window\n\t|Buffer-menu-this-window|Buffer-menu-toggle-files-only|Buffer-menu-toggle-read-only|Buffer-menu-unmark|Buffer-menu-view-other-window\n\t|Buffer-menu-view|Buffer-menu-visit-tags-table|Control-X-prefix|Custom-buffer-done|Custom-goto-parent|Custom-help|Custom-mode-menu\n\t|Custom-mode|Custom-newline|Custom-no-edit|Custom-reset-current|Custom-reset-saved|Custom-reset-standard|Custom-save|Custom-set|Electric-buffer-menu-exit\n\t|Electric-buffer-menu-mode-view-buffer|Electric-buffer-menu-mode|Electric-buffer-menu-mouse-select|Electric-buffer-menu-quit\n\t|Electric-buffer-menu-select|Electric-buffer-menu-undefined|Electric-command-history-redo-expression|Electric-command-loop|Electric-pop-up-window\n\t|Footnote-add-footnote|Footnote-assoc-index|Footnote-back-to-message|Footnote-current-regexp|Footnote-cycle-style|Footnote-delete-footnote\n\t|Footnote-english-lower|Footnote-english-upper|Footnote-goto-char-point-max|Footnote-goto-footnote|Footnote-index-to-string|Footnote-insert-footnote\n\t|Footnote-insert-numbered-footnote|Footnote-insert-pointer-marker|Footnote-insert-text-marker|Footnote-latin|Footnote-make-hole\n\t|Footnote-narrow-to-footnotes|Footnote-numeric|Footnote-refresh-footnotes|Footnote-renumber-footnotes|Footnote-renumber|Footnote-roman-common\n\t|Footnote-roman-lower|Footnote-roman-upper|Footnote-set-style|Footnote-sort|Footnote-style-p|Footnote-text-under-cursor|Footnote-under-cursor\n\t|Footnote-unicode|Info--search-loop|Info-apropos-find-file|Info-apropos-find-node|Info-apropos-matches|Info-apropos-toc-nodes|Info-backward-node\n\t|Info-bookmark-jump|Info-bookmark-make-record|Info-breadcrumbs|Info-build-node-completions-1|Info-build-node-completions|Info-cease-edit\n\t|Info-check-pointer|Info-clone-buffer|Info-complete-menu-item|Info-copy-current-node-name|Info-default-dirs|Info-desktop-buffer-misc-data\n\t|Info-dir-remove-duplicates|Info-directory-find-file|Info-directory-find-node|Info-directory-toc-nodes|Info-directory|Info-display-images-node\n\t|Info-edit-mode|Info-edit|Info-exit|Info-extract-menu-counting|Info-extract-menu-item|Info-extract-menu-node-name|Info-extract-pointer\n\t|Info-file-supports-index-cookies|Info-final-node|Info-find-emacs-command-nodes|Info-find-file|Info-find-in-tag-table-1|Info-find-in-tag-table\n\t|Info-find-index-name|Info-find-node-2|Info-find-node-in-buffer-1|Info-find-node-in-buffer|Info-find-node|Info-finder-find-file\n\t|Info-finder-find-node|Info-follow-nearest-node|Info-follow-reference|Info-following-node-name-re|Info-following-node-name|Info-fontify-node\n\t|Info-forward-node|Info-get-token|Info-goto-emacs-command-node|Info-goto-emacs-key-command-node|Info-goto-index|Info-goto-node\n\t|Info-help|Info-hide-cookies-node|Info-history-back|Info-history-find-file|Info-history-find-node|Info-history-forward|Info-history-toc-nodes\n\t|Info-history|Info-index-next|Info-index-node|Info-index-nodes|Info-index|Info-insert-dir|Info-install-speedbar-variables|Info-isearch-end\n\t|Info-isearch-filter|Info-isearch-pop-state|Info-isearch-push-state|Info-isearch-search|Info-isearch-start|Info-isearch-wrap|Info-kill-buffer\n\t|Info-last-menu-item|Info-last-preorder|Info-last|Info-menu-update|Info-menu|Info-mode-menu|Info-mode|Info-mouse-follow-link|Info-mouse-follow-nearest-node\n\t|Info-mouse-scroll-down|Info-mouse-scroll-up|Info-next-menu-item|Info-next-preorder|Info-next-reference-or-link|Info-next-reference\n\t|Info-next|Info-no-error|Info-node-at-bob-matching|Info-nth-menu-item|Info-on-current-buffer|Info-prev-reference-or-link|Info-prev-reference\n\t|Info-prev|Info-read-node-name-1|Info-read-node-name-2|Info-read-node-name|Info-read-subfile|Info-restore-desktop-buffer|Info-restore-point\n\t|Info-revert-buffer-function|Info-revert-find-node|Info-scroll-down|Info-scroll-up|Info-search-backward|Info-search-case-sensitively\n\t|Info-search-next|Info-search|Info-select-node|Info-set-mode-line|Info-speedbar-browser|Info-speedbar-buttons|Info-speedbar-expand-node\n\t|Info-speedbar-fetch-file-nodes|Info-speedbar-goto-node|Info-speedbar-hierarchy-buttons|Info-split-parameter-string|Info-split\n\t|Info-summary|Info-tagify|Info-toc-build|Info-toc-find-node|Info-toc-insert|Info-toc-nodes|Info-toc|Info-top-node|Info-try-follow-nearest-node\n\t|Info-undefined|Info-unescape-quotes|Info-up|Info-validate-node-name|Info-validate-tags-table|Info-validate|Info-virtual-call|Info-virtual-file-p\n\t|Info-virtual-fun|Info-virtual-index-find-node|Info-virtual-index|LaTeX-mode|Man-bgproc-filter|Man-bgproc-sentinel|Man-bookmark-jump\n\t|Man-bookmark-make-record|Man-build-man-command|Man-build-page-list|Man-build-references-alist|Man-build-section-alist|Man-cleanup-manpage\n\t|Man-completion-table|Man-default-bookmark-title|Man-default-man-entry|Man-find-section|Man-follow-manual-reference|Man-fontify-manpage\n\t|Man-getpage-in-background|Man-goto-page|Man-goto-section|Man-goto-see-also-section|Man-highlight-references|Man-highlight-references0\n\t|Man-init-defvars|Man-kill|Man-make-page-mode-string|Man-mode|Man-next-manpage|Man-next-section|Man-notify-when-ready|Man-page-from-arguments\n\t|Man-parse-man-k|Man-possibly-hyphenated-word|Man-previous-manpage|Man-previous-section|Man-quit|Man-softhyphen-to-minus|Man-start-calling\n\t|Man-strip-page-headers|Man-support-local-filenames|Man-translate-cleanup|Man-translate-references|Man-unindent|Man-update-manpage\n\t|Man-view-header-file|Man-xref-button-action|Math-anglep|Math-bignum-test|Math-equal-int|Math-equal|Math-integer-neg|Math-integer-negp\n\t|Math-integer-posp|Math-integerp|Math-lessp|Math-looks-negp|Math-messy-integerp|Math-natnum-lessp|Math-natnump|Math-negp|Math-num-integerp\n\t|Math-numberp|Math-objectp|Math-objvecp|Math-posp|Math-primp|Math-ratp|Math-realp|Math-scalarp|Math-vectorp|Math-zerop|TeX-mode|View-back-to-mark\n\t|View-exit-and-edit|View-exit|View-goto-line|View-goto-percent|View-kill-and-leave|View-leave|View-quit-all|View-quit|View-revert-buffer-scroll-page-forward\n\t|View-scroll-half-page-backward|View-scroll-half-page-forward|View-scroll-line-backward|View-scroll-line-forward|View-scroll-page-backward-set-page-size\n\t|View-scroll-page-backward|View-scroll-page-forward-set-page-size|View-scroll-page-forward|View-scroll-to-buffer-end|View-search-last-regexp-backward\n\t|View-search-last-regexp-forward|View-search-regexp-backward|View-search-regexp-forward|WoMan-find-buffer|WoMan-getpage-in-background\n\t|WoMan-log-1|WoMan-log-begin|WoMan-log-end|WoMan-log|WoMan-next-manpage|WoMan-previous-manpage|WoMan-warn-ignored|WoMan-warn|abbrev--active-tables\n\t|abbrev--before-point|abbrev--check-chars|abbrev--default-expand|abbrev--describe|abbrev--symbol|abbrev--write|abbrev-edit-save-buffer\n\t|abbrev-edit-save-to-file|abbrev-mode|abbrev-table-empty-p|abbrev-table-menu|abbrev-table-name|abort-if-file-too-large|about-emacs\n\t|accelerate-menu|accept-completion|acons|activate-input-method|activate-mark|activate-mode-local-bindings|ad--defalias-fset|ad--make-advised-docstring\n\t|ad-Advice-c-backward-sws|ad-Advice-c-beginning-of-macro|ad-Advice-c-forward-sws|ad-Advice-save-place-find-file-hook|ad-access-argument\n\t|ad-activate-advised-definition|ad-activate-all|ad-activate-internal|ad-activate-on|ad-activate-regexp|ad-activate|ad-add-advice\n\t|ad-advice-definition|ad-advice-enabled|ad-advice-name|ad-advice-p|ad-advice-position|ad-advice-protected|ad-advice-set-enabled\n\t|ad-advised-arglist|ad-advised-interactive-form|ad-arg-binding-field|ad-arglist|ad-assemble-advised-definition|ad-body-forms|ad-cache-id-verification-code\n\t|ad-class-p|ad-clear-advicefunname-definition|ad-clear-cache|ad-compile-function|ad-compiled-code|ad-compiled-p|ad-copy-advice-info\n\t|ad-deactivate-all|ad-deactivate-regexp|ad-deactivate|ad-definition-type|ad-disable-advice|ad-disable-regexp|ad-do-advised-functions\n\t|ad-docstring|ad-element-access|ad-enable-advice-internal|ad-enable-advice|ad-enable-regexp-internal|ad-enable-regexp|ad-find-advice\n\t|ad-find-some-advice|ad-get-advice-info-field|ad-get-advice-info-macro|ad-get-advice-info|ad-get-argument|ad-get-arguments|ad-get-cache-class-id\n\t|ad-get-cache-definition|ad-get-cache-id|ad-get-enabled-advices|ad-get-orig-definition|ad-has-any-advice|ad-has-enabled-advice\n\t|ad-has-proper-definition|ad-has-redefining-advice|ad-initialize-advice-info|ad-insert-argument-access-forms|ad-interactive-form\n\t|ad-is-active|ad-is-advised|ad-is-compilable|ad-lambda-expression|ad-lambda-p|ad-lambdafy|ad-list-access|ad-macrofy|ad-make-advice\n\t|ad-make-advicefunname|ad-make-advised-definition|ad-make-cache-id|ad-make-hook-form|ad-make-single-advice-docstring|ad-map-arglists\n\t|ad-name-p|ad-parse-arglist|ad-pop-advised-function|ad-position-p|ad-preactivate-advice|ad-pushnew-advised-function|ad-read-advice-class\n\t|ad-read-advice-name|ad-read-advice-specification|ad-read-advised-function|ad-read-regexp|ad-real-definition|ad-real-orig-definition\n\t|ad-recover-all|ad-recover-normality|ad-recover|ad-remove-advice|ad-retrieve-args-form|ad-set-advice-info-field|ad-set-advice-info\n\t|ad-set-argument|ad-set-arguments|ad-set-cache|ad-should-compile|ad-substitute-tree|ad-unadvise-all|ad-unadvise|ad-update-all|ad-update-regexp\n\t|ad-update|ad-verify-cache-class-id|ad-verify-cache-id|ad-with-originals|ada-activate-keys-for-case|ada-add-extensions|ada-adjust-case-buffer\n\t|ada-adjust-case-identifier|ada-adjust-case-interactive|ada-adjust-case-region|ada-adjust-case-skeleton|ada-adjust-case-substring\n\t|ada-adjust-case|ada-after-keyword-p|ada-array|ada-batch-reformat|ada-call-from-contextual-menu|ada-capitalize-word|ada-case-read-exceptions-from-file\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t ada-case-read-exceptions|ada-case|ada-change-prj|ada-check-current|ada-check-defun-name|ada-check-matching-start|ada-compile-application\n\t|ada-compile-current|ada-compile-goto-error|ada-compile-mouse-goto-error|ada-complete-identifier|ada-contextual-menu|ada-create-case-exception-substring\n\t|ada-create-case-exception|ada-create-keymap|ada-create-menu|ada-customize|ada-declare-block|ada-else|ada-elsif|ada-exception-block\n\t|ada-exception|ada-exit|ada-ff-other-window|ada-fill-comment-paragraph-justify|ada-fill-comment-paragraph-postfix|ada-fill-comment-paragraph\n\t|ada-find-any-references|ada-find-file|ada-find-local-references|ada-find-references|ada-find-src-file-in-dir|ada-for-loop|ada-format-paramlist\n\t|ada-function-spec|ada-gdb-application|ada-gen-treat-proc|ada-get-body-name|ada-get-current-indent|ada-get-indent-block-label|ada-get-indent-block-start\n\t|ada-get-indent-case|ada-get-indent-end|ada-get-indent-goto-label|ada-get-indent-if|ada-get-indent-loop|ada-get-indent-nochange\n\t|ada-get-indent-noindent|ada-get-indent-open-paren|ada-get-indent-paramlist|ada-get-indent-subprog|ada-get-indent-type|ada-get-indent-when\n\t|ada-gnat-style|ada-goto-decl-start|ada-goto-declaration-other-frame|ada-goto-declaration|ada-goto-matching-end|ada-goto-matching-start\n\t|ada-goto-next-non-ws|ada-goto-next-word|ada-goto-parent|ada-goto-previous-word|ada-goto-stmt-end|ada-goto-stmt-start|ada-header\n\t|ada-if|ada-in-comment-p|ada-in-decl-p|ada-in-numeric-literal-p|ada-in-open-paren-p|ada-in-paramlist-p|ada-in-string-or-comment-p\n\t|ada-in-string-p|ada-indent-current-function|ada-indent-current|ada-indent-newline-indent-conditional|ada-indent-newline-indent\n\t|ada-indent-on-previous-lines|ada-indent-region|ada-insert-paramlist|ada-justified-indent-current|ada-looking-at-semi-or|ada-looking-at-semi-private\n\t|ada-loop|ada-loose-case-word|ada-make-body-gnatstub|ada-make-body|ada-make-filename-from-adaname|ada-make-subprogram-body|ada-mode-menu\n\t|ada-mode-version|ada-mode|ada-move-to-end|ada-move-to-start|ada-narrow-to-defun|ada-next-package|ada-next-procedure|ada-no-auto-case\n\t|ada-other-file-name|ada-outline-level|ada-package-body|ada-package-spec|ada-point-and-xref|ada-popup-menu|ada-previous-package\n\t|ada-previous-procedure|ada-private|ada-prj-edit|ada-prj-new|ada-prj-save|ada-procedure-spec|ada-record|ada-region-selected|ada-remove-trailing-spaces\n\t|ada-reread-prj-file|ada-run-application|ada-save-exceptions-to-file|ada-scan-paramlist|ada-search-ignore-complex-boolean|ada-search-ignore-string-comment\n\t|ada-search-prev-end-stmt|ada-set-default-project-file|ada-set-main-compile-application|ada-set-point-accordingly|ada-show-current-main\n\t|ada-subprogram-body|ada-subtype|ada-tab-hard|ada-tab|ada-tabsize|ada-task-body|ada-task-spec|ada-type|ada-uncomment-region|ada-untab-hard\n\t|ada-untab|ada-use|ada-when|ada-which-function-are-we-in|ada-which-function|ada-while-loop|ada-with|ada-xref-goto-previous-reference\n\t|add-abbrev|add-change-log-entry-other-window|add-change-log-entry|add-completion-to-head|add-completion-to-tail-if-new|add-completion\n\t|add-completions-from-buffer|add-completions-from-c-buffer|add-completions-from-file|add-completions-from-lisp-buffer|add-completions-from-tags-table\n\t|add-dir-local-variable|add-file-local-variable-prop-line|add-file-local-variable|add-global-abbrev|add-log-current-defun|add-log-edit-next-comment\n\t|add-log-edit-prev-comment|add-log-file-name|add-log-iso8601-time-string|add-log-iso8601-time-zone|add-log-tcl-defun|add-minor-mode\n\t|add-mode-abbrev|add-new-page|add-permanent-completion|add-submenu|add-timeout|add-to-coding-system-list|add-to-list--anon-cmacro\n\t|addbib|adjoin|advertised-undo|advertised-widget-backward|advertised-xscheme-send-previous-expression|advice--add-function|advice--buffer-local\n\t|advice--called-interactively-skip|advice--car|advice--cd\\*r|advice--cdr|advice--defalias-fset|advice--interactive-form|advice--make-1\n\t|advice--make-docstring|advice--make-interactive-form|advice--make|advice--member-p|advice--normalize-place|advice--normalize|advice--p\n\t|advice--props|advice--remove-function|advice--set-buffer-local|advice--strip-macro|advice--subst-main|advice--symbol-function\n\t|advice--tweak|after-insert-file-set-coding|align--set-marker|align-adjust-col-for-rule|align-areas|align-column|align-current|align-entire\n\t|align-highlight-rule|align-match-tex-pattern|align-new-section-p|align-newline-and-indent|align-regexp|align-region|align-regions\n\t|align-set-vhdl-rules|align-unhighlight-rule|align|alist-get|allout-aberrant-container-p|allout-add-resumptions|allout-adjust-file-variable\n\t|allout-after-saves-handler|allout-annotate-hidden|allout-ascend-to-depth|allout-ascend|allout-auto-activation-helper|allout-auto-fill\n\t|allout-back-to-current-heading|allout-back-to-heading|allout-back-to-visible-text|allout-backward-current-level|allout-before-change-handler\n\t|allout-beginning-of-current-entry|allout-beginning-of-current-line|allout-beginning-of-level|allout-beginning-of-line|allout-body-modification-handler\n\t|allout-bullet-for-depth|allout-bullet-isearch|allout-called-interactively-p|allout-chart-exposure-contour-by-icon|allout-chart-siblings\n\t|allout-chart-subtree|allout-chart-to-reveal|allout-compose-and-institute-keymap|allout-copy-exposed-to-buffer|allout-copy-line-as-kill\n\t|allout-copy-topic-as-kill|allout-current-bullet-pos|allout-current-bullet|allout-current-decorated-p|allout-current-depth|allout-current-topic-collapsed-p\n\t|allout-deannotate-hidden|allout-decorate-item-and-context|allout-decorate-item-body|allout-decorate-item-cue|allout-decorate-item-guides\n\t|allout-decorate-item-icon|allout-decorate-item-span|allout-depth|allout-descend-to-depth|allout-distinctive-bullet|allout-do-doublecheck\n\t|allout-do-resumptions|allout-e-o-prefix-p|allout-elapsed-time-seconds|allout-encrypt-decrypted|allout-encrypt-string|allout-encrypted-topic-p\n\t|allout-encrypted-type-prefix|allout-end-of-current-heading|allout-end-of-current-line|allout-end-of-current-subtree|allout-end-of-entry\n\t|allout-end-of-heading|allout-end-of-level|allout-end-of-line|allout-end-of-prefix|allout-end-of-subtree|allout-expose-topic|allout-fetch-icon-image\n\t|allout-file-vars-section-data|allout-find-file-hook|allout-find-image|allout-flag-current-subtree|allout-flag-region|allout-flatten-exposed-to-buffer\n\t|allout-flatten|allout-format-quote|allout-forward-current-level|allout-frame-property|allout-get-body-text|allout-get-bullet|allout-get-configvar-values\n\t|allout-get-current-prefix|allout-get-invisibility-overlay|allout-get-item-widget|allout-get-or-create-item-widget|allout-get-or-create-parent-widget\n\t|allout-get-prefix-bullet|allout-goto-prefix-doublechecked|allout-goto-prefix|allout-graphics-modification-handler|allout-hidden-p\n\t|allout-hide-bodies|allout-hide-by-annotation|allout-hide-current-entry|allout-hide-current-leaves|allout-hide-current-subtree\n\t|allout-hide-region-body|allout-hotspot-key-handler|allout-indented-exposed-to-buffer|allout-infer-body-reindent|allout-infer-header-lead-and-primary-bullet\n\t|allout-infer-header-lead|allout-inhibit-auto-save-info-for-decryption|allout-init|allout-insert-latex-header|allout-insert-latex-trailer\n\t|allout-insert-listified|allout-institute-keymap|allout-isearch-end-handler|allout-item-actual-position|allout-item-element-span-is\n\t|allout-item-icon-key-handler|allout-item-location|allout-item-span|allout-kill-line|allout-kill-topic|allout-latex-verb-quote\n\t|allout-latex-verbatim-quote-curr-line|allout-latexify-exposed|allout-latexify-one-item|allout-lead-with-comment-string|allout-listify-exposed\n\t|allout-make-topic-prefix|allout-mark-active-p|allout-mark-marker|allout-mark-topic|allout-maybe-resume-auto-save-info-after-encryption\n\t|allout-minor-mode|allout-mode-map|allout-mode-p|allout-mode|allout-new-exposure|allout-new-item-widget|allout-next-heading|allout-next-sibling-leap\n\t|allout-next-sibling|allout-next-single-char-property-change|allout-next-topic-pending-encryption|allout-next-visible-heading\n\t|allout-number-siblings|allout-numbered-type-prefix|allout-old-expose-topic|allout-on-current-heading-p|allout-on-heading-p|allout-open-sibtopic\n\t|allout-open-subtopic|allout-open-supertopic|allout-open-topic|allout-overlay-insert-in-front-handler|allout-overlay-interior-modification-handler\n\t|allout-overlay-preparations|allout-parse-item-at-point|allout-post-command-business|allout-pre-command-business|allout-pre-next-prefix\n\t|allout-prefix-data|allout-previous-heading|allout-previous-sibling|allout-previous-single-char-property-change|allout-previous-visible-heading\n\t|allout-process-exposed|allout-range-overlaps|allout-rebullet-current-heading|allout-rebullet-heading|allout-rebullet-topic-grunt\n\t|allout-rebullet-topic|allout-recent-bullet|allout-recent-depth|allout-recent-prefix|allout-redecorate-item|allout-redecorate-visible-subtree\n\t|allout-region-active-p|allout-reindent-body|allout-renumber-to-depth|allout-reset-header-lead|allout-resolve-xref|allout-run-unit-tests\n\t|allout-select-safe-coding-system|allout-set-boundary-marker|allout-setup-menubar|allout-setup-text-properties|allout-setup|allout-shift-in\n\t|allout-shift-out|allout-show-all|allout-show-children|allout-show-current-branches|allout-show-current-entry|allout-show-current-subtree\n\t|allout-show-entry|allout-show-to-offshoot|allout-sibling-index|allout-snug-back|allout-solicit-alternate-bullet|allout-stringify-flat-index-indented\n\t|allout-stringify-flat-index-plain|allout-stringify-flat-index|allout-substring-no-properties|allout-test-range-overlaps|allout-test-resumptions\n\t|allout-tests-obliterate-variable|allout-this-or-next-heading|allout-toggle-current-subtree-encryption|allout-toggle-current-subtree-exposure\n\t|allout-toggle-subtree-encryption|allout-topic-flat-index|allout-unload-function|allout-unprotected|allout-up-current-level|allout-version\n\t|allout-widgetize-buffer|allout-widgets-additions-processor|allout-widgets-additions-recorder|allout-widgets-adjusting-message\n\t|allout-widgets-after-change-handler|allout-widgets-after-copy-or-kill-function|allout-widgets-after-undo-function|allout-widgets-before-change-handler\n\t|allout-widgets-changes-dispatcher|allout-widgets-copy-list|allout-widgets-count-buttons-in-region|allout-widgets-deletions-processor\n\t|allout-widgets-deletions-recorder|allout-widgets-exposure-change-processor|allout-widgets-exposure-change-recorder|allout-widgets-exposure-undo-processor\n\t|allout-widgets-exposure-undo-recorder|allout-widgets-hook-error-handler|allout-widgets-mode-disable|allout-widgets-mode-enable\n\t|allout-widgets-mode-off|allout-widgets-mode-on|allout-widgets-mode|allout-widgets-post-command-business|allout-widgets-pre-command-business\n\t|allout-widgets-prepopulate-buffer|allout-widgets-run-unit-tests|allout-widgets-setup|allout-widgets-shifts-processor|allout-widgets-shifts-recorder\n\t|allout-widgets-tally-string|allout-widgets-undecorate-item|allout-widgets-undecorate-region|allout-widgets-undecorate-text|allout-widgets-version\n\t|allout-write-contents-hook-handler|allout-yank-pop|allout-yank-processing|allout-yank|alter-text-property|ange-ftp-abbreviate-filename\n\t|ange-ftp-add-bs2000-host|ange-ftp-add-bs2000-posix-host|ange-ftp-add-cms-host|ange-ftp-add-dl-dir|ange-ftp-add-dumb-unix-host\n\t|ange-ftp-add-file-entry|ange-ftp-add-mts-host|ange-ftp-add-vms-host|ange-ftp-allow-child-lookup|ange-ftp-barf-if-not-directory\n\t|ange-ftp-barf-or-query-if-file-exists|ange-ftp-binary-file|ange-ftp-bs2000-cd-to-posix|ange-ftp-bs2000-host|ange-ftp-bs2000-posix-host\n\t|ange-ftp-call-chmod|ange-ftp-call-cont|ange-ftp-canonize-filename|ange-ftp-cd|ange-ftp-cf1|ange-ftp-cf2|ange-ftp-chase-symlinks\n\t|ange-ftp-cms-host|ange-ftp-cms-make-compressed-filename|ange-ftp-completion-hook-function|ange-ftp-compress|ange-ftp-copy-file-internal\n\t|ange-ftp-copy-file|ange-ftp-copy-files-async|ange-ftp-del-tmp-name|ange-ftp-delete-directory|ange-ftp-delete-file-entry|ange-ftp-delete-file\n\t|ange-ftp-directory-file-name|ange-ftp-directory-files-and-attributes|ange-ftp-directory-files|ange-ftp-dired-compress-file|ange-ftp-dired-uncache\n\t|ange-ftp-dl-parser|ange-ftp-dumb-unix-host|ange-ftp-error|ange-ftp-expand-dir|ange-ftp-expand-file-name|ange-ftp-expand-symlink\n\t|ange-ftp-file-attributes|ange-ftp-file-directory-p|ange-ftp-file-entry-not-ignored-p|ange-ftp-file-entry-p|ange-ftp-file-executable-p\n\t|ange-ftp-file-exists-p|ange-ftp-file-local-copy|ange-ftp-file-modtime|ange-ftp-file-name-all-completions|ange-ftp-file-name-as-directory\n\t|ange-ftp-file-name-completion-1|ange-ftp-file-name-completion|ange-ftp-file-name-directory|ange-ftp-file-name-nondirectory|ange-ftp-file-name-sans-versions\n)(?=[\\s()]|$)'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t ange-ftp-file-newer-than-file-p|ange-ftp-file-readable-p|ange-ftp-file-remote-p|ange-ftp-file-size|ange-ftp-file-symlink-p|ange-ftp-file-writable-p\n\t|ange-ftp-find-backup-file-name|ange-ftp-fix-dir-name-for-bs2000|ange-ftp-fix-dir-name-for-cms|ange-ftp-fix-dir-name-for-mts\n\t|ange-ftp-fix-dir-name-for-vms|ange-ftp-fix-name-for-bs2000|ange-ftp-fix-name-for-cms|ange-ftp-fix-name-for-mts|ange-ftp-fix-name-for-vms\n\t|ange-ftp-ftp-name-component|ange-ftp-ftp-name|ange-ftp-ftp-process-buffer|ange-ftp-generate-passwd-key|ange-ftp-generate-root-prefixes\n\t|ange-ftp-get-account|ange-ftp-get-file-entry|ange-ftp-get-file-part|ange-ftp-get-files|ange-ftp-get-host-with-passwd|ange-ftp-get-passwd\n\t|ange-ftp-get-process|ange-ftp-get-pwd|ange-ftp-get-user|ange-ftp-guess-hash-mark-size|ange-ftp-guess-host-type|ange-ftp-gwp-filter\n\t|ange-ftp-gwp-sentinel|ange-ftp-gwp-start|ange-ftp-hash-entry-exists-p|ange-ftp-hash-table-keys|ange-ftp-hook-function|ange-ftp-host-type\n\t|ange-ftp-ignore-errors-if-non-essential|ange-ftp-insert-directory|ange-ftp-insert-file-contents|ange-ftp-internal-add-file-entry\n\t|ange-ftp-internal-delete-file-entry|ange-ftp-kill-ftp-process|ange-ftp-load|ange-ftp-lookup-passwd|ange-ftp-ls-parser|ange-ftp-ls\n\t|ange-ftp-make-directory|ange-ftp-make-tmp-name|ange-ftp-message|ange-ftp-mts-host|ange-ftp-normal-login|ange-ftp-nslookup-host\n\t|ange-ftp-parse-bs2000-filename|ange-ftp-parse-bs2000-listing|ange-ftp-parse-cms-listing|ange-ftp-parse-dired-listing|ange-ftp-parse-filename\n\t|ange-ftp-parse-mts-listing|ange-ftp-parse-netrc-group|ange-ftp-parse-netrc-token|ange-ftp-parse-netrc|ange-ftp-parse-vms-filename\n\t|ange-ftp-parse-vms-listing|ange-ftp-passive-mode|ange-ftp-process-file|ange-ftp-process-filter|ange-ftp-process-handle-hash|ange-ftp-process-handle-line\n\t|ange-ftp-process-sentinel|ange-ftp-quote-string|ange-ftp-raw-send-cmd|ange-ftp-re-read-dir|ange-ftp-real-backup-buffer|ange-ftp-real-copy-file\n\t|ange-ftp-real-delete-directory|ange-ftp-real-delete-file|ange-ftp-real-directory-file-name|ange-ftp-real-directory-files-and-attributes\n\t|ange-ftp-real-directory-files|ange-ftp-real-expand-file-name|ange-ftp-real-file-attributes|ange-ftp-real-file-directory-p|ange-ftp-real-file-executable-p\n\t|ange-ftp-real-file-exists-p|ange-ftp-real-file-name-all-completions|ange-ftp-real-file-name-as-directory|ange-ftp-real-file-name-completion\n\t|ange-ftp-real-file-name-directory|ange-ftp-real-file-name-nondirectory|ange-ftp-real-file-name-sans-versions|ange-ftp-real-file-newer-than-file-p\n\t|ange-ftp-real-file-readable-p|ange-ftp-real-file-symlink-p|ange-ftp-real-file-writable-p|ange-ftp-real-find-backup-file-name\n\t|ange-ftp-real-insert-directory|ange-ftp-real-insert-file-contents|ange-ftp-real-load|ange-ftp-real-make-directory|ange-ftp-real-rename-file\n\t|ange-ftp-real-shell-command|ange-ftp-real-verify-visited-file-modtime|ange-ftp-real-write-region|ange-ftp-rename-file|ange-ftp-rename-local-to-remote\n\t|ange-ftp-rename-remote-to-local|ange-ftp-rename-remote-to-remote|ange-ftp-repaint-minibuffer|ange-ftp-replace-name-component\n\t|ange-ftp-reread-dir|ange-ftp-root-dir-p|ange-ftp-run-real-handler-orig|ange-ftp-run-real-handler|ange-ftp-send-cmd|ange-ftp-set-account\n\t|ange-ftp-set-ascii-mode|ange-ftp-set-binary-mode|ange-ftp-set-buffer-mode|ange-ftp-set-file-modes|ange-ftp-set-files|ange-ftp-set-passwd\n\t|ange-ftp-set-user|ange-ftp-set-xfer-size|ange-ftp-shell-command|ange-ftp-smart-login|ange-ftp-start-process|ange-ftp-switches-ok\n\t|ange-ftp-uncompress|ange-ftp-unhandled-file-name-directory|ange-ftp-use-gateway-p|ange-ftp-use-smart-gateway-p|ange-ftp-verify-visited-file-modtime\n\t|ange-ftp-vms-add-file-entry|ange-ftp-vms-delete-file-entry|ange-ftp-vms-file-name-as-directory|ange-ftp-vms-host|ange-ftp-vms-make-compressed-filename\n\t|ange-ftp-vms-sans-version|ange-ftp-wait-not-busy|ange-ftp-wipe-file-entries|ange-ftp-write-region|animate-birthday-present|animate-initialize\n\t|animate-place-char|animate-sequence|animate-step|animate-string|another-calc|ansi-color--find-face|ansi-color-apply-on-region|ansi-color-apply-overlay-face\n\t|ansi-color-apply-sequence|ansi-color-apply|ansi-color-filter-apply|ansi-color-filter-region|ansi-color-for-comint-mode-filter\n\t|ansi-color-for-comint-mode-off|ansi-color-for-comint-mode-on|ansi-color-freeze-overlay|ansi-color-get-face-1|ansi-color-make-color-map\n\t|ansi-color-make-extent|ansi-color-make-face|ansi-color-map-update|ansi-color-parse-sequence|ansi-color-process-output|ansi-color-set-extent-face\n\t|ansi-color-unfontify-region|ansi-term|antlr-beginning-of-body|antlr-beginning-of-rule|antlr-c\\+\\+-mode-extra|antlr-c-forward-sws\n\t|antlr-c-init-language-vars|antlr-default-directory|antlr-directory-dependencies|antlr-downcase-literals|antlr-electric-character\n\t|antlr-end-of-body|antlr-end-of-rule|antlr-file-dependencies|antlr-font-lock-keywords|antlr-grammar-tokens|antlr-hide-actions|antlr-imenu-create-index-function\n\t|antlr-indent-command|antlr-indent-line|antlr-insert-makefile-rules|antlr-insert-option-area|antlr-insert-option-do|antlr-insert-option-existing\n\t|antlr-insert-option-interactive|antlr-insert-option-space|antlr-insert-option|antlr-inside-rule-p|antlr-invalidate-context-cache\n\t|antlr-language-option-extra|antlr-language-option|antlr-makefile-insert-variable|antlr-mode-menu|antlr-mode|antlr-next-rule|antlr-option-kind\n\t|antlr-option-level|antlr-option-location|antlr-option-spec|antlr-options-menu-filter|antlr-outside-rule-p|antlr-re-search-forward\n\t|antlr-read-boolean|antlr-read-shell-command|antlr-read-value|antlr-run-tool-interactive|antlr-run-tool|antlr-search-backward|antlr-search-forward\n\t|antlr-set-tabs|antlr-show-makefile-rules|antlr-skip-exception-part|antlr-skip-file-prelude|antlr-skip-sexps|antlr-superclasses-glibs\n\t|antlr-syntactic-context|antlr-syntactic-grammar-depth|antlr-upcase-literals|antlr-upcase-p|antlr-version-string|antlr-with-displaying-help-buffer\n\t|antlr-with-syntax-table|append-next-kill|append-to-buffer|append-to-register|apply-macro-to-region-lines|apply-on-rectangle|appt-activate\n\t|appt-add|apropos-command|apropos-documentation-property|apropos-documentation|apropos-internal|apropos-library|apropos-read-pattern\n\t|apropos-user-option|apropos-value|apropos-variable|archive-\\*-expunge|archive-\\*-extract|archive-\\*-write-file-member|archive-7z-extract\n\t|archive-7z-summarize|archive-7z-write-file-member|archive-add-new-member|archive-alternate-display|archive-ar-extract|archive-ar-summarize\n\t|archive-arc-rename-entry|archive-arc-summarize|archive-calc-mode|archive-chgrp-entry|archive-chmod-entry|archive-chown-entry|archive-delete-local\n\t|archive-desummarize|archive-display-other-window|archive-dosdate|archive-dostime|archive-expunge|archive-extract-by-file|archive-extract-by-stdout\n\t|archive-extract-other-window|archive-extract|archive-file-name-handler|archive-find-type|archive-flag-deleted|archive-get-descr\n\t|archive-get-lineno|archive-get-marked|archive-int-to-mode|archive-l-e|archive-lzh-chgrp-entry|archive-lzh-chmod-entry|archive-lzh-chown-entry\n\t|archive-lzh-exe-extract|archive-lzh-exe-summarize|archive-lzh-extract|archive-lzh-ogm|archive-lzh-rename-entry|archive-lzh-resum\n\t|archive-lzh-summarize|archive-mark|archive-maybe-copy|archive-maybe-update|archive-mode-revert|archive-mode|archive-mouse-extract\n\t|archive-name|archive-next-line|archive-previous-line|archive-rar-exe-extract|archive-rar-exe-summarize|archive-rar-extract|archive-rar-summarize\n\t|archive-rename-entry|archive-resummarize|archive-set-buffer-as-visiting-file|archive-summarize-files|archive-summarize|archive-try-jka-compr\n\t|archive-undo|archive-unflag-backwards|archive-unflag|archive-unique-fname|archive-unixdate|archive-unixtime|archive-unmark-all-files\n\t|archive-view|archive-write-file-member|archive-write-file|archive-zip-chmod-entry|archive-zip-extract|archive-zip-summarize|archive-zip-write-file-member\n\t|archive-zoo-extract|archive-zoo-summarize|arp|array-backward-column|array-beginning-of-field|array-copy-backward|array-copy-column-backward\n\t|array-copy-column-forward|array-copy-down|array-copy-forward|array-copy-once-horizontally|array-copy-once-vertically|array-copy-row-down\n\t|array-copy-row-up|array-copy-to-cell|array-copy-to-column|array-copy-to-row|array-copy-up|array-current-column|array-current-row\n\t|array-cursor-in-array-range|array-display-local-variables|array-end-of-field|array-expand-rows|array-field-string|array-fill-rectangle\n\t|array-forward-column|array-goto-cell|array-make-template|array-maybe-scroll-horizontally|array-mode|array-move-one-column|array-move-one-row\n\t|array-move-to-cell|array-move-to-column|array-move-to-row|array-next-row|array-normalize-cursor|array-previous-row|array-reconfigure-rows\n\t|array-update-array-position|array-update-buffer-position|array-what-position|artist-2point-get-endpoint1|artist-2point-get-endpoint2\n\t|artist-2point-get-shapeinfo|artist-arrow-point-get-direction|artist-arrow-point-get-marker|artist-arrow-point-get-orig-char\n\t|artist-arrow-point-get-state|artist-arrow-point-set-state|artist-arrows|artist-backward-char|artist-calculate-new-char|artist-calculate-new-chars\n\t|artist-charlist-to-string|artist-clear-arrow-points|artist-clear-buffer|artist-compute-key-compl-table|artist-compute-line-char\n\t|artist-compute-popup-menu-table-sub|artist-compute-popup-menu-table|artist-compute-up-event-key|artist-coord-add-new-char|artist-coord-add-saved-char\n\t|artist-coord-get-new-char|artist-coord-get-saved-char|artist-coord-get-x|artist-coord-get-y|artist-coord-set-new-char|artist-coord-set-x\n\t|artist-coord-set-y|artist-coord-win-to-buf|artist-copy-generic|artist-copy-rect|artist-copy-square|artist-current-column|artist-current-line\n\t|artist-cut-rect|artist-cut-square|artist-direction-char|artist-direction-step-x|artist-direction-step-y|artist-do-nothing|artist-down-mouse-1\n\t|artist-down-mouse-3|artist-draw-circle|artist-draw-ellipse-general|artist-draw-ellipse-with-0-height|artist-draw-ellipse|artist-draw-line\n\t|artist-draw-rect|artist-draw-region-reset|artist-draw-region-trim-line-endings|artist-draw-sline|artist-draw-square|artist-eight-point\n\t|artist-ellipse-compute-fill-info|artist-ellipse-fill-info-add-center|artist-ellipse-generate-quadrant|artist-ellipse-mirror-quadrant\n\t|artist-ellipse-point-list-add-center|artist-ellipse-remove-0-fills|artist-endpoint-get-x|artist-endpoint-get-y|artist-erase-char\n\t|artist-erase-rect|artist-event-is-shifted|artist-fc-get-fn-from-symbol|artist-fc-get-fn|artist-fc-get-keyword|artist-fc-get-symbol\n\t|artist-fc-retrieve-from-symbol-sub|artist-fc-retrieve-from-symbol|artist-ff-get-rightmost-from-xy|artist-ff-is-bottommost-line\n\t|artist-ff-is-topmost-line|artist-ff-too-far-right|artist-figlet-choose-font|artist-figlet-get-extra-args|artist-figlet-get-font-list\n\t|artist-figlet-run|artist-figlet|artist-file-to-string|artist-fill-circle|artist-fill-ellipse|artist-fill-item-get-width|artist-fill-item-get-x\n\t|artist-fill-item-get-y|artist-fill-item-set-width|artist-fill-item-set-x|artist-fill-item-set-y|artist-fill-rect|artist-fill-square\n\t|artist-find-direction|artist-find-octant|artist-flood-fill|artist-forward-char|artist-funcall|artist-get-buffer-contents-at-xy\n\t|artist-get-char-at-xy-conv|artist-get-char-at-xy|artist-get-dfdx-init-coeff|artist-get-dfdy-init-coeff|artist-get-first-non-nil-op\n\t|artist-get-last-non-nil-op|artist-get-replacement-char|artist-get-x-step-q<0|artist-get-x-step-q>=0|artist-get-y-step-q<0|artist-get-y-step-q>=0\n\t|artist-go-get-arrow-pred-from-symbol|artist-go-get-arrow-pred|artist-go-get-arrow-set-fn-from-symbol|artist-go-get-arrow-set-fn\n\t|artist-go-get-desc|artist-go-get-draw-fn-from-symbol|artist-go-get-draw-fn|artist-go-get-draw-how-from-symbol|artist-go-get-draw-how\n\t|artist-go-get-exit-fn-from-symbol|artist-go-get-exit-fn|artist-go-get-fill-fn-from-symbol|artist-go-get-fill-fn|artist-go-get-fill-pred-from-symbol\n\t|artist-go-get-fill-pred|artist-go-get-init-fn-from-symbol|artist-go-get-init-fn|artist-go-get-interval-fn-from-symbol|artist-go-get-interval-fn\n\t|artist-go-get-keyword-from-symbol|artist-go-get-keyword|artist-go-get-mode-line-from-symbol|artist-go-get-mode-line|artist-go-get-prep-fill-fn-from-symbol\n\t|artist-go-get-prep-fill-fn|artist-go-get-shifted|artist-go-get-symbol-shift-sub|artist-go-get-symbol-shift|artist-go-get-symbol\n\t|artist-go-get-undraw-fn-from-symbol|artist-go-get-undraw-fn|artist-go-get-unshifted|artist-go-retrieve-from-symbol-sub|artist-go-retrieve-from-symbol\n\t|artist-intersection-char|artist-is-in-op-list-p|artist-key-do-continously-1point|artist-key-do-continously-2points|artist-key-do-continously-common\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t artist-key-do-continously-continously|artist-key-do-continously-poly|artist-key-draw-1point|artist-key-draw-2points|artist-key-draw-common\n\t|artist-key-draw-continously|artist-key-draw-poly|artist-key-set-point-1point|artist-key-set-point-2points|artist-key-set-point-common\n\t|artist-key-set-point-continously|artist-key-set-point-poly|artist-key-set-point|artist-key-undraw-1point|artist-key-undraw-2points\n\t|artist-key-undraw-common|artist-key-undraw-continously|artist-key-undraw-poly|artist-make-2point-object|artist-make-arrow-point\n\t|artist-make-endpoint|artist-make-prev-next-op-alist|artist-mn-get-items|artist-mn-get-title|artist-mode-exit|artist-mode-init\n\t|artist-mode-line-show-curr-operation|artist-mode-off|artist-mode|artist-modify-new-chars|artist-mouse-choose-operation|artist-mouse-draw-1point\n\t|artist-mouse-draw-2points|artist-mouse-draw-continously|artist-mouse-draw-poly|artist-move-to-xy|artist-mt-get-info-part|artist-mt-get-symbol-from-keyword-sub\n\t|artist-mt-get-symbol-from-keyword|artist-mt-get-tag|artist-new-coord|artist-new-fill-item|artist-next-line|artist-nil|artist-no-arrows\n\t|artist-no-rb-set-point1|artist-no-rb-set-point2|artist-no-rb-unset-point1|artist-no-rb-unset-point2|artist-no-rb-unset-points\n\t|artist-paste|artist-pen-line|artist-pen-reset-last-xy|artist-pen-set-arrow-points|artist-pen|artist-previous-line|artist-put-pixel\n\t|artist-rect-corners-squarify|artist-replace-char|artist-replace-chars|artist-replace-string|artist-save-chars-under-point-list\n\t|artist-save-chars-under-sline|artist-select-erase-char|artist-select-fill-char|artist-select-line-char|artist-select-next-op-in-list\n\t|artist-select-op-circle|artist-select-op-copy-rectangle|artist-select-op-copy-square|artist-select-op-cut-rectangle|artist-select-op-cut-square\n\t|artist-select-op-ellipse|artist-select-op-erase-char|artist-select-op-erase-rectangle|artist-select-op-flood-fill|artist-select-op-line\n\t|artist-select-op-paste|artist-select-op-pen-line|artist-select-op-poly-line|artist-select-op-rectangle|artist-select-op-spray-can\n\t|artist-select-op-spray-set-size|artist-select-op-square|artist-select-op-straight-line|artist-select-op-straight-poly-line|artist-select-op-text-overwrite\n\t|artist-select-op-text-see-thru|artist-select-op-vaporize-line|artist-select-op-vaporize-lines|artist-select-operation|artist-select-prev-op-in-list\n\t|artist-select-spray-chars|artist-set-arrow-points-for-2points|artist-set-arrow-points-for-poly|artist-set-pointer-shape|artist-shift-has-changed\n\t|artist-sline|artist-spray-clear-circle|artist-spray-get-interval|artist-spray-random-points|artist-spray-set-radius|artist-spray\n\t|artist-straight-calculate-length|artist-string-split|artist-string-to-charlist|artist-string-to-file|artist-submit-bug-report\n\t|artist-system|artist-t-if-fill-char-set|artist-t|artist-text-insert-common|artist-text-insert-overwrite|artist-text-insert-see-thru\n\t|artist-text-overwrite|artist-text-see-thru|artist-toggle-borderless-shapes|artist-toggle-first-arrow|artist-toggle-rubber-banding\n\t|artist-toggle-second-arrow|artist-toggle-trim-line-endings|artist-undraw-circle|artist-undraw-ellipse|artist-undraw-line|artist-undraw-rect\n\t|artist-undraw-sline|artist-undraw-square|artist-unintersection-char|artist-uniq|artist-update-display|artist-update-pointer-shape\n\t|artist-vap-find-endpoint|artist-vap-find-endpoints-horiz|artist-vap-find-endpoints-nwse|artist-vap-find-endpoints-swne|artist-vap-find-endpoints-vert\n\t|artist-vap-find-endpoints|artist-vap-group-in-pairs|artist-vaporize-by-endpoints|artist-vaporize-line|artist-vaporize-lines|asm-calculate-indentation\n\t|asm-colon|asm-comment|asm-indent-line|asm-mode|asm-newline|assert|assoc\\*|assoc-if-not|assoc-if|assoc-ignore-case|assoc-ignore-representation\n\t|async-shell-command|atomic-change-group|auth-source--aget|auth-source--aput-1|auth-source--aput|auth-source-backend-child-p|auth-source-backend-list-p\n\t|auth-source-backend-p|auth-source-backend-parse-parameters|auth-source-backend-parse|auth-source-backend|auth-source-current-line\n\t|auth-source-delete|auth-source-do-debug|auth-source-do-trivia|auth-source-do-warn|auth-source-ensure-strings|auth-source-epa-extract-gpg-token\n\t|auth-source-epa-make-gpg-token|auth-source-forget\\+|auth-source-forget-all-cached|auth-source-forget|auth-source-format-cache-entry\n\t|auth-source-format-prompt|auth-source-macos-keychain-create|auth-source-macos-keychain-result-append|auth-source-macos-keychain-search-items\n\t|auth-source-macos-keychain-search|auth-source-netrc-create|auth-source-netrc-element-or-first|auth-source-netrc-normalize|auth-source-netrc-parse-entries\n\t|auth-source-netrc-parse-next-interesting|auth-source-netrc-parse-one|auth-source-netrc-parse|auth-source-netrc-saver|auth-source-netrc-search\n\t|auth-source-pick-first-password|auth-source-plstore-create|auth-source-plstore-search|auth-source-read-char-choice|auth-source-recall\n\t|auth-source-remember|auth-source-remembered-p|auth-source-search-backends|auth-source-search-collection|auth-source-search|auth-source-secrets-create\n\t|auth-source-secrets-listify-pattern|auth-source-secrets-search|auth-source-specmatchp|auth-source-token-passphrase-callback-function\n\t|auth-source-user-and-password|auth-source-user-or-password|auto-coding-alist-lookup|auto-coding-regexp-alist-lookup|auto-compose-chars\n\t|auto-composition-mode|auto-compression-mode|auto-encryption-mode|auto-fill-mode|auto-image-file-mode|auto-insert-mode|auto-insert\n\t|auto-lower-mode|auto-raise-mode|auto-revert-active-p|auto-revert-buffers|auto-revert-handler|auto-revert-mode|auto-revert-notify-add-watch\n\t|auto-revert-notify-handler|auto-revert-notify-rm-watch|auto-revert-set-timer|auto-revert-tail-handler|auto-revert-tail-mode|autoarg-kp-digit-argument\n\t|autoarg-kp-mode|autoarg-mode|autoarg-terminate|autoconf-current-defun-function|autoconf-mode|autodoc-font-lock-keywords|autodoc-font-lock-line-markup\n\t|autoload-coding-system|autoload-rubric|avl-tree--check-node|avl-tree--check|avl-tree--cmpfun--cmacro|avl-tree--cmpfun|avl-tree--create--cmacro\n\t|avl-tree--create|avl-tree--del-balance|avl-tree--dir-to-sign|avl-tree--do-copy|avl-tree--do-del-internal|avl-tree--do-delete|avl-tree--do-enter\n\t|avl-tree--dummyroot--cmacro|avl-tree--dummyroot|avl-tree--enter-balance|avl-tree--mapc|avl-tree--node-balance--cmacro|avl-tree--node-balance\n\t|avl-tree--node-branch|avl-tree--node-create--cmacro|avl-tree--node-create|avl-tree--node-data--cmacro|avl-tree--node-data|avl-tree--node-left--cmacro\n\t|avl-tree--node-left|avl-tree--node-right--cmacro|avl-tree--node-right|avl-tree--root|avl-tree--sign-to-dir|avl-tree--stack-create\n\t|avl-tree--stack-p--cmacro|avl-tree--stack-p|avl-tree--stack-repopulate|avl-tree--stack-reverse--cmacro|avl-tree--stack-reverse\n\t|avl-tree--stack-store--cmacro|avl-tree--stack-store|avl-tree--switch-dir|avl-tree-clear|avl-tree-compare-function|avl-tree-copy\n\t|avl-tree-create|avl-tree-delete|avl-tree-empty|avl-tree-enter|avl-tree-first|avl-tree-flatten|avl-tree-last|avl-tree-map|avl-tree-mapc\n\t|avl-tree-mapcar|avl-tree-mapf|avl-tree-member-p|avl-tree-member|avl-tree-p--cmacro|avl-tree-p|avl-tree-size|avl-tree-stack-empty-p\n\t|avl-tree-stack-first|avl-tree-stack-p|avl-tree-stack-pop|avl-tree-stack|awk-mode|babel-as-string|background-color-at-point|backquote-delay-process\n\t|backquote-list\\*-function|backquote-list\\*-macro|backquote-list\\*|backquote-listify|backquote-process|backquote|backtrace--locals\n\t|backtrace-eval|backup-buffer-copy|backup-extract-version|backward-delete-char|backward-ifdef|backward-kill-paragraph|backward-kill-sentence\n\t|backward-kill-sexp|backward-kill-word|backward-page|backward-paragraph|backward-sentence|backward-text-line|backward-up-list|bad-package-check\n\t|balance-windows-1|balance-windows-2|balance-windows-area-adjust|basic-save-buffer-1|basic-save-buffer-2|basic-save-buffer|bat-cmd-help\n\t|bat-mode|bat-run-args|bat-run|bat-template|batch-byte-compile-file|batch-byte-compile-if-not-done|batch-byte-recompile-directory\n\t|batch-info-validate|batch-texinfo-format|batch-titdic-convert|batch-unrmail|batch-update-autoloads|battery-bsd-apm|battery-format\n\t|battery-linux-proc-acpi|battery-linux-proc-apm|battery-linux-sysfs|battery-pmset|battery-search-for-one-match-in-files|battery-update-handler\n\t|battery-update|battery|bb-bol|bb-done|bb-down|bb-eol|bb-goto|bb-init-board|bb-insert-board|bb-left|bb-outside-box|bb-place-ball|bb-right\n\t|bb-romp|bb-show-bogus-balls-2|bb-show-bogus-balls|bb-trace-ray-2|bb-trace-ray|bb-up|bb-update-board|beginning-of-buffer-other-window\n\t|beginning-of-defun-raw|beginning-of-icon-defun|beginning-of-line-text|beginning-of-sexp|beginning-of-thing|beginning-of-visual-line\n\t|benchmark-elapse|benchmark-run-compiled|benchmark-run|benchmark|bib-capitalize-title-region|bib-capitalize-title|bib-find-key|bib-mode\n\t|bibtex-Article|bibtex-Book|bibtex-BookInBook|bibtex-Booklet|bibtex-Collection|bibtex-InBook|bibtex-InCollection|bibtex-InProceedings\n\t|bibtex-InReference|bibtex-MVBook|bibtex-MVCollection|bibtex-MVProceedings|bibtex-MVReference|bibtex-Manual|bibtex-MastersThesis\n\t|bibtex-Misc|bibtex-Online|bibtex-Patent|bibtex-Periodical|bibtex-PhdThesis|bibtex-Preamble|bibtex-Proceedings|bibtex-Reference|bibtex-Report\n\t|bibtex-String|bibtex-SuppBook|bibtex-SuppCollection|bibtex-SuppPeriodical|bibtex-TechReport|bibtex-Thesis|bibtex-Unpublished|bibtex-autofill-entry\n\t|bibtex-autokey-abbrev|bibtex-autokey-demangle-name|bibtex-autokey-demangle-title|bibtex-autokey-get-field|bibtex-autokey-get-names\n\t|bibtex-autokey-get-title|bibtex-autokey-get-year|bibtex-beginning-first-field|bibtex-beginning-of-entry|bibtex-beginning-of-field\n\t|bibtex-beginning-of-first-entry|bibtex-button-action|bibtex-button|bibtex-clean-entry|bibtex-complete-crossref-cleanup|bibtex-complete-string-cleanup\n\t|bibtex-complete|bibtex-completion-at-point-function|bibtex-convert-alien|bibtex-copy-entry-as-kill|bibtex-copy-field-as-kill\n\t|bibtex-copy-summary-as-kill|bibtex-count-entries|bibtex-current-line|bibtex-delete-whitespace|bibtex-display-entries|bibtex-dist\n\t|bibtex-edit-menu|bibtex-empty-field|bibtex-enclosing-field|bibtex-end-of-entry|bibtex-end-of-field|bibtex-end-of-name-in-field\n\t|bibtex-end-of-string|bibtex-end-of-text-in-field|bibtex-end-of-text-in-string|bibtex-entry-alist|bibtex-entry-index|bibtex-entry-left-delimiter\n\t|bibtex-entry-right-delimiter|bibtex-entry-update|bibtex-entry|bibtex-field-left-delimiter|bibtex-field-list|bibtex-field-re-init\n\t|bibtex-field-right-delimiter|bibtex-fill-entry|bibtex-fill-field-bounds|bibtex-fill-field|bibtex-find-crossref|bibtex-find-entry\n\t|bibtex-find-text-internal|bibtex-find-text|bibtex-flash-head|bibtex-font-lock-cite|bibtex-font-lock-crossref|bibtex-font-lock-url\n\t|bibtex-format-entry|bibtex-generate-autokey|bibtex-global-key-alist|bibtex-goto-line|bibtex-init-sort-entry-class-alist|bibtex-initialize\n\t|bibtex-insert-kill|bibtex-ispell-abstract|bibtex-ispell-entry|bibtex-key-in-head|bibtex-kill-entry|bibtex-kill-field|bibtex-lessp\n\t|bibtex-make-field|bibtex-make-optional-field|bibtex-map-entries|bibtex-mark-entry|bibtex-mode|bibtex-move-outside-of-entry|bibtex-name-in-field\n\t|bibtex-narrow-to-entry|bibtex-next-field|bibtex-parse-association|bibtex-parse-buffers-stealthily|bibtex-parse-entry|bibtex-parse-field-name\n\t|bibtex-parse-field-string|bibtex-parse-field-text|bibtex-parse-field|bibtex-parse-keys|bibtex-parse-preamble|bibtex-parse-string-postfix\n\t|bibtex-parse-string-prefix|bibtex-parse-string|bibtex-parse-strings|bibtex-pop-next|bibtex-pop-previous|bibtex-pop|bibtex-prepare-new-entry\n\t|bibtex-print-help-message|bibtex-progress-message|bibtex-read-key|bibtex-read-string-key|bibtex-realign|bibtex-reference-key-in-string\n\t|bibtex-reformat|bibtex-remove-OPT-or-ALT|bibtex-remove-delimiters|bibtex-reposition-window|bibtex-search-backward-field|bibtex-search-crossref\n\t|bibtex-search-entries|bibtex-search-entry|bibtex-search-forward-field|bibtex-search-forward-string|bibtex-set-dialect|bibtex-skip-to-valid-entry\n\t|bibtex-sort-buffer|bibtex-start-of-field|bibtex-start-of-name-in-field|bibtex-start-of-text-in-field|bibtex-start-of-text-in-string\n\t|bibtex-string-files-init|bibtex-string=|bibtex-strings|bibtex-style-calculate-indentation|bibtex-style-indent-line|bibtex-style-mode\n\t|bibtex-summary|bibtex-text-in-field-bounds|bibtex-text-in-field|bibtex-text-in-string|bibtex-type-in-head|bibtex-url|bibtex-valid-entry\n\t|bibtex-validate-globally|bibtex-validate|bibtex-vec-incr|bibtex-vec-push|bibtex-yank-pop|bibtex-yank|bidi-find-overridden-directionality\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t bidi-resolved-levels|binary-overwrite-mode|bindat--length-group|bindat--pack-group|bindat--pack-item|bindat--pack-u16|bindat--pack-u16r\n\t|bindat--pack-u24|bindat--pack-u24r|bindat--pack-u32|bindat--pack-u32r|bindat--pack-u8|bindat--unpack-group|bindat--unpack-item\n\t|bindat--unpack-u16|bindat--unpack-u16r|bindat--unpack-u24|bindat--unpack-u24r|bindat--unpack-u32|bindat--unpack-u32r|bindat--unpack-u8\n\t|bindat-format-vector|bindat-vector-to-dec|bindat-vector-to-hex|bindings--define-key|binhex-char-int|binhex-char-map|binhex-decode-region-external\n\t|binhex-decode-region-internal|binhex-decode-region|binhex-header|binhex-insert-char|binhex-push-char|binhex-string-big-endian\n\t|binhex-string-little-endian|binhex-update-crc|binhex-verify-crc|blackbox-mode|blackbox-redefine-key|blackbox|blink-cursor-check\n\t|blink-cursor-end|blink-cursor-mode|blink-cursor-start|blink-cursor-suspend|blink-cursor-timer-function|blink-matching-check-mismatch\n\t|blink-paren-post-self-insert-function|block|bookmark--jump-via|bookmark-alist-from-buffer|bookmark-all-names|bookmark-bmenu-1-window\n\t|bookmark-bmenu-2-window|bookmark-bmenu-any-marks|bookmark-bmenu-backup-unmark|bookmark-bmenu-bookmark|bookmark-bmenu-delete-backwards\n\t|bookmark-bmenu-delete|bookmark-bmenu-edit-annotation|bookmark-bmenu-ensure-position|bookmark-bmenu-execute-deletions|bookmark-bmenu-filter-alist-by-regexp\n\t|bookmark-bmenu-goto-bookmark|bookmark-bmenu-hide-filenames|bookmark-bmenu-list|bookmark-bmenu-load|bookmark-bmenu-locate|bookmark-bmenu-mark\n\t|bookmark-bmenu-mode|bookmark-bmenu-other-window-with-mouse|bookmark-bmenu-other-window|bookmark-bmenu-relocate|bookmark-bmenu-rename\n\t|bookmark-bmenu-save|bookmark-bmenu-search|bookmark-bmenu-select|bookmark-bmenu-set-header|bookmark-bmenu-show-all-annotations\n\t|bookmark-bmenu-show-annotation|bookmark-bmenu-show-filenames|bookmark-bmenu-surreptitiously-rebuild-list|bookmark-bmenu-switch-other-window\n\t|bookmark-bmenu-this-window|bookmark-bmenu-toggle-filenames|bookmark-bmenu-unmark|bookmark-buffer-file-name|bookmark-buffer-name\n\t|bookmark-completing-read|bookmark-default-annotation-text|bookmark-default-handler|bookmark-delete|bookmark-edit-annotation-mode\n\t|bookmark-edit-annotation|bookmark-exit-hook-internal|bookmark-get-annotation|bookmark-get-bookmark-record|bookmark-get-bookmark\n\t|bookmark-get-filename|bookmark-get-front-context-string|bookmark-get-handler|bookmark-get-position|bookmark-get-rear-context-string\n\t|bookmark-grok-file-format-version|bookmark-handle-bookmark|bookmark-import-new-list|bookmark-insert-annotation|bookmark-insert-file-format-version-stamp\n\t|bookmark-insert-location|bookmark-insert|bookmark-jump-noselect|bookmark-jump-other-window|bookmark-jump|bookmark-kill-line|bookmark-load\n\t|bookmark-locate|bookmark-location|bookmark-make-record-default|bookmark-make-record|bookmark-map|bookmark-maybe-historicize-string\n\t|bookmark-maybe-load-default-file|bookmark-maybe-message|bookmark-maybe-rename|bookmark-maybe-sort-alist|bookmark-maybe-upgrade-file-format\n\t|bookmark-menu-popup-paned-menu|bookmark-name-from-full-record|bookmark-prop-get|bookmark-prop-set|bookmark-relocate|bookmark-rename\n\t|bookmark-save|bookmark-send-edited-annotation|bookmark-set-annotation|bookmark-set-filename|bookmark-set-front-context-string\n\t|bookmark-set-name|bookmark-set-position|bookmark-set-rear-context-string|bookmark-set|bookmark-show-all-annotations|bookmark-show-annotation\n\t|bookmark-store|bookmark-time-to-save-p|bookmark-unload-function|bookmark-upgrade-file-format-from-0|bookmark-upgrade-version-0-alist\n\t|bookmark-write-file|bookmark-write|bookmark-yank-word|bool-vector|bound-and-true-p|bounds-of-thing-at-point|bovinate|bovine-grammar-mode\n\t|browse-url-at-mouse|browse-url-at-point|browse-url-can-use-xdg-open|browse-url-cci|browse-url-chromium|browse-url-default-browser\n\t|browse-url-default-macosx-browser|browse-url-default-windows-browser|browse-url-delete-temp-file|browse-url-elinks-new-window\n\t|browse-url-elinks-sentinel|browse-url-elinks|browse-url-emacs-display|browse-url-emacs|browse-url-encode-url|browse-url-epiphany-sentinel\n\t|browse-url-epiphany|browse-url-file-url|browse-url-firefox-sentinel|browse-url-firefox|browse-url-galeon-sentinel|browse-url-galeon\n\t|browse-url-generic|browse-url-gnome-moz|browse-url-interactive-arg|browse-url-kde|browse-url-mail|browse-url-maybe-new-window\n\t|browse-url-mosaic|browse-url-mozilla-sentinel|browse-url-mozilla|browse-url-netscape-reload|browse-url-netscape-send|browse-url-netscape-sentinel\n\t|browse-url-netscape|browse-url-of-buffer|browse-url-of-dired-file|browse-url-of-file|browse-url-of-region|browse-url-process-environment\n\t|browse-url-text-emacs|browse-url-text-xterm|browse-url-url-at-point|browse-url-url-encode-chars|browse-url-w3-gnudoit|browse-url-w3\n\t|browse-url-xdg-open|browse-url|browse-web|bs--configuration-name-for-prefix-arg|bs--create-header-line|bs--current-buffer|bs--current-config-message\n\t|bs--down|bs--format-aux|bs--get-file-name|bs--get-marked-string|bs--get-mode-name|bs--get-modified-string|bs--get-name-length|bs--get-name\n\t|bs--get-readonly-string|bs--get-size-string|bs--get-value|bs--goto-current-buffer|bs--insert-one-entry|bs--make-header-match-string\n\t|bs--mark-unmark|bs--nth-wrapper|bs--redisplay|bs--remove-hooks|bs--restore-window-config|bs--set-toggle-to-show|bs--set-window-height\n\t|bs--show-config-message|bs--show-header|bs--show-with-configuration|bs--sort-by-filename|bs--sort-by-mode|bs--sort-by-name|bs--sort-by-size\n\t|bs--track-window-changes|bs--up|bs--update-current-line|bs-abort|bs-apply-sort-faces|bs-buffer-list|bs-buffer-sort|bs-bury-buffer\n\t|bs-clear-modified|bs-config--all-intern-last|bs-config--all|bs-config--files-and-scratch|bs-config--only-files|bs-config-clear\n\t|bs-customize|bs-cycle-next|bs-cycle-previous|bs-define-sort-function|bs-delete-backward|bs-delete|bs-down|bs-help|bs-kill|bs-mark-current\n\t|bs-message-without-log|bs-mode|bs-mouse-select-other-frame|bs-mouse-select|bs-next-buffer|bs-next-config-aux|bs-next-config|bs-previous-buffer\n\t|bs-refresh|bs-save|bs-select-in-one-window|bs-select-next-configuration|bs-select-other-frame|bs-select-other-window|bs-select\n\t|bs-set-configuration-and-refresh|bs-set-configuration|bs-set-current-buffer-to-show-always|bs-set-current-buffer-to-show-never\n\t|bs-show-in-buffer|bs-show-sorted|bs-show|bs-sort-buffer-interns-are-last|bs-tmp-select-other-window|bs-toggle-current-to-show\n\t|bs-toggle-readonly|bs-toggle-show-all|bs-unload-function|bs-unmark-current|bs-up|bs-view|bs-visit-tags-table|bs-visits-non-file\n\t|bubbles--char-at|bubbles--col|bubbles--colors|bubbles--compute-offsets|bubbles--count|bubbles--empty-char|bubbles--game-over|bubbles--goto\n\t|bubbles--grid-height|bubbles--grid-width|bubbles--initialize-faces|bubbles--initialize-images|bubbles--initialize|bubbles--mark-direct-neighbors\n\t|bubbles--mark-neighborhood|bubbles--neighborhood-available|bubbles--remove-overlays|bubbles--reset-score|bubbles--row|bubbles--set-faces\n\t|bubbles--shift-mode|bubbles--shift|bubbles--show-images|bubbles--show-scores|bubbles--update-faces-or-images|bubbles--update-neighborhood-score\n\t|bubbles--update-score|bubbles-customize|bubbles-mode|bubbles-plop|bubbles-quit|bubbles-save-settings|bubbles-set-game-difficult\n\t|bubbles-set-game-easy|bubbles-set-game-hard|bubbles-set-game-medium|bubbles-set-game-userdefined|bubbles-set-graphics-theme-ascii\n\t|bubbles-set-graphics-theme-balls|bubbles-set-graphics-theme-circles|bubbles-set-graphics-theme-diamonds|bubbles-set-graphics-theme-emacs\n\t|bubbles-set-graphics-theme-squares|bubbles-undo|bubbles|buffer-face-mode-invoke|buffer-face-mode|buffer-face-set|buffer-face-toggle\n\t|buffer-has-markers-at|buffer-menu-open|buffer-menu-other-window|buffer-menu|buffer-stale--default-function|buffer-substring--filter\n\t|buffer-substring-with-bidi-context|bug-reference-fontify|bug-reference-mode|bug-reference-prog-mode|bug-reference-push-button\n\t|bug-reference-set-overlay-properties|bug-reference-unfontify|build-mail-abbrevs|build-mail-aliases|bury-buffer-internal|butterfly\n\t|button--area-button-p|button--area-button-string|button-category-symbol|byte-code|byte-compile--declare-var|byte-compile--reify-function\n\t|byte-compile-abbreviate-file|byte-compile-and-folded|byte-compile-and-recursion|byte-compile-and|byte-compile-annotate-call-tree\n\t|byte-compile-arglist-signature-string|byte-compile-arglist-signature|byte-compile-arglist-signatures-congruent-p|byte-compile-arglist-vars\n\t|byte-compile-arglist-warn|byte-compile-associative|byte-compile-autoload|byte-compile-backward-char|byte-compile-backward-word\n\t|byte-compile-bind|byte-compile-body-do-effect|byte-compile-body|byte-compile-butlast|byte-compile-callargs-warn|byte-compile-catch\n\t|byte-compile-char-before|byte-compile-check-lambda-list|byte-compile-check-variable|byte-compile-cl-file-p|byte-compile-cl-warn\n\t|byte-compile-close-variables|byte-compile-concat|byte-compile-cond|byte-compile-condition-case--new|byte-compile-condition-case--old\n\t|byte-compile-condition-case|byte-compile-constant|byte-compile-constants-vector|byte-compile-defvar|byte-compile-delete-first\n\t|byte-compile-dest-file|byte-compile-disable-warning|byte-compile-discard|byte-compile-dynamic-variable-bind|byte-compile-dynamic-variable-op\n\t|byte-compile-enable-warning|byte-compile-eval-before-compile|byte-compile-eval|byte-compile-fdefinition|byte-compile-file-form-autoload\n\t|byte-compile-file-form-custom-declare-variable|byte-compile-file-form-defalias|byte-compile-file-form-define-abbrev-table|byte-compile-file-form-defmumble\n\t|byte-compile-file-form-defvar|byte-compile-file-form-eval|byte-compile-file-form-progn|byte-compile-file-form-require|byte-compile-file-form-with-no-warnings\n\t|byte-compile-file-form|byte-compile-find-bound-condition|byte-compile-find-cl-functions|byte-compile-fix-header|byte-compile-flush-pending\n\t|byte-compile-form-do-effect|byte-compile-form-make-variable-buffer-local|byte-compile-form|byte-compile-format-warn|byte-compile-from-buffer\n\t|byte-compile-fset|byte-compile-funcall|byte-compile-function-form|byte-compile-function-warn|byte-compile-get-closed-var|byte-compile-get-constant\n\t|byte-compile-goto-if|byte-compile-goto|byte-compile-if|byte-compile-indent-to|byte-compile-inline-expand|byte-compile-inline-lapcode\n\t|byte-compile-insert-header|byte-compile-insert|byte-compile-keep-pending|byte-compile-lambda-form|byte-compile-lambda|byte-compile-lapcode\n\t|byte-compile-let|byte-compile-list|byte-compile-log-1|byte-compile-log-file|byte-compile-log-lap-1|byte-compile-log-lap|byte-compile-log-warning\n\t|byte-compile-log|byte-compile-macroexpand-declare-function|byte-compile-make-args-desc|byte-compile-make-closure|byte-compile-make-lambda-lexenv\n\t|byte-compile-make-obsolete-variable|byte-compile-make-tag|byte-compile-make-variable-buffer-local|byte-compile-maybe-guarded\n\t|byte-compile-minus|byte-compile-nconc|byte-compile-negated|byte-compile-negation-optimizer|byte-compile-nilconstp|byte-compile-no-args\n\t|byte-compile-no-warnings|byte-compile-nogroup-warn|byte-compile-noop|byte-compile-normal-call|byte-compile-not-lexical-var-p\n\t|byte-compile-one-arg|byte-compile-one-or-two-args|byte-compile-or-recursion|byte-compile-or|byte-compile-out-tag|byte-compile-out-toplevel\n\t|byte-compile-out|byte-compile-output-as-comment|byte-compile-output-docform|byte-compile-output-file-form|byte-compile-preprocess\n\t|byte-compile-print-syms|byte-compile-prog1|byte-compile-prog2|byte-compile-progn|byte-compile-push-binding-init|byte-compile-push-bytecode-const2\n\t|byte-compile-push-bytecodes|byte-compile-push-constant|byte-compile-quo|byte-compile-quote|byte-compile-recurse-toplevel|byte-compile-refresh-preloaded\n\t|byte-compile-report-error|byte-compile-report-ops|byte-compile-save-current-buffer|byte-compile-save-excursion|byte-compile-save-restriction\n\t|byte-compile-set-default|byte-compile-set-symbol-position|byte-compile-setq-default|byte-compile-setq|byte-compile-sexp|byte-compile-stack-adjustment\n\t|byte-compile-stack-ref|byte-compile-stack-set|byte-compile-subr-wrong-args|byte-compile-three-args|byte-compile-top-level-body\n\t|byte-compile-top-level|byte-compile-toplevel-file-form|byte-compile-trueconstp|byte-compile-two-args|byte-compile-two-or-three-args\n\t|byte-compile-unbind|byte-compile-unfold-bcf|byte-compile-unfold-lambda|byte-compile-unwind-protect|byte-compile-variable-ref\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t byte-compile-variable-set|byte-compile-warn-about-unresolved-functions|byte-compile-warn-obsolete|byte-compile-warn|byte-compile-warning-enabled-p\n\t|byte-compile-warning-prefix|byte-compile-warning-series|byte-compile-while|byte-compile-zero-or-one-arg|byte-compiler-base-file-name\n\t|byte-decompile-bytecode-1|byte-decompile-bytecode|byte-defop-compiler-1|byte-defop-compiler|byte-defop|byte-extrude-byte-code-vectors\n\t|byte-force-recompile|byte-optimize-all-constp|byte-optimize-and|byte-optimize-apply|byte-optimize-approx-equal|byte-optimize-associative-math\n\t|byte-optimize-binary-predicate|byte-optimize-body|byte-optimize-cond|byte-optimize-delay-constants-math|byte-optimize-divide\n\t|byte-optimize-form-code-walker|byte-optimize-form|byte-optimize-funcall|byte-optimize-identity|byte-optimize-if|byte-optimize-inline-handler\n\t|byte-optimize-lapcode|byte-optimize-letX|byte-optimize-logmumble|byte-optimize-minus|byte-optimize-multiply|byte-optimize-nonassociative-math\n\t|byte-optimize-nth|byte-optimize-nthcdr|byte-optimize-or|byte-optimize-plus|byte-optimize-predicate|byte-optimize-quote|byte-optimize-set\n\t|byte-optimize-while|byte-recompile-file|byteorder|c\\+\\+-font-lock-keywords-2|c\\+\\+-font-lock-keywords-3|c\\+\\+-font-lock-keywords\n\t|c\\+\\+-mode|c--macroexpand-all|c-add-class-syntax|c-add-language|c-add-stmt-syntax|c-add-style|c-add-syntax|c-add-type|c-advise-fl-for-region\n\t|c-after-change-check-<>-operators|c-after-change|c-after-conditional|c-after-font-lock-init|c-after-special-operator-id|c-after-statement-terminator-p\n\t|c-append-backslashes-forward|c-append-lower-brace-pair-to-state-cache|c-append-syntax|c-append-to-state-cache|c-ascertain-following-literal\n\t|c-ascertain-preceding-literal|c-at-expression-start-p|c-at-macro-vsemi-p|c-at-statement-start-p|c-at-toplevel-p|c-at-vsemi-p|c-awk-menu\n\t|c-back-over-illiterals|c-back-over-member-initializer-braces|c-back-over-member-initializers|c-backslash-region|c-backward-<>-arglist\n\t|c-backward-colon-prefixed-type|c-backward-comments|c-backward-conditional|c-backward-into-nomenclature|c-backward-over-enum-header\n\t|c-backward-sexp|c-backward-single-comment|c-backward-sws|c-backward-syntactic-ws|c-backward-to-block-anchor|c-backward-to-decl-anchor\n\t|c-backward-to-nth-BOF-\\{|c-backward-token-1|c-backward-token-2|c-basic-common-init|c-before-change-check-<>-operators|c-before-change\n\t|c-before-hack-hook|c-beginning-of-current-token|c-beginning-of-decl-1|c-beginning-of-defun-1|c-beginning-of-defun|c-beginning-of-inheritance-list\n\t|c-beginning-of-macro|c-beginning-of-sentence-in-comment|c-beginning-of-sentence-in-string|c-beginning-of-statement-1|c-beginning-of-statement\n\t|c-beginning-of-syntax|c-benign-error|c-bind-special-erase-keys|c-block-in-arglist-dwim|c-bos-pop-state-and-retry|c-bos-pop-state\n\t|c-bos-push-state|c-bos-report-error|c-bos-restore-pos|c-bos-save-error-info|c-bos-save-pos|c-brace-anchor-point|c-brace-newlines\n\t|c-c\\+\\+-menu|c-c-menu|c-calc-comment-indent|c-calc-offset|c-calculate-state|c-change-set-fl-decl-start|c-cheap-inside-bracelist-p\n\t|c-check-type|c-clear-<-pair-props-if-match-after|c-clear-<-pair-props|c-clear-<>-pair-props|c-clear->-pair-props-if-match-before\n\t|c-clear->-pair-props|c-clear-c-type-property|c-clear-char-properties|c-clear-char-property-with-value-function|c-clear-char-property-with-value\n\t|c-clear-char-property|c-clear-cpp-delimiters|c-clear-found-types|c-collect-line-comments|c-comment-indent|c-comment-line-break-function\n\t|c-comment-out-cpps|c-common-init|c-compose-keywords-list|c-concat-separated|c-constant-symbol|c-context-line-break|c-context-open-line\n\t|c-context-set-fl-decl-start|c-count-cfss|c-cpp-define-name|c-crosses-statement-barrier-p|c-debug-add-face|c-debug-parse-state-double-cons\n\t|c-debug-parse-state|c-debug-put-decl-spot-faces|c-debug-remove-decl-spot-faces|c-debug-remove-face|c-debug-sws-msg|c-declaration-limits\n\t|c-declare-lang-variables|c-default-value-sentence-end|c-define-abbrev-table|c-define-lang-constant|c-defun-name|c-delete-and-extract-region\n\t|c-delete-backslashes-forward|c-delete-overlay|c-determine-\\+ve-limit|c-determine-limit-get-base|c-determine-limit|c-do-auto-fill\n\t|c-down-conditional-with-else|c-down-conditional|c-down-list-backward|c-down-list-forward|c-echo-parsing-error|c-electric-backspace\n\t|c-electric-brace|c-electric-colon|c-electric-continued-statement|c-electric-delete-forward|c-electric-delete|c-electric-indent-local-mode-hook\n\t|c-electric-indent-mode-hook|c-electric-lt-gt|c-electric-paren|c-electric-pound|c-electric-semi&comma|c-electric-slash|c-electric-star\n\t|c-end-of-current-token|c-end-of-decl-1|c-end-of-defun-1|c-end-of-defun|c-end-of-macro|c-end-of-sentence-in-comment|c-end-of-sentence-in-string\n\t|c-end-of-statement|c-evaluate-offset|c-extend-after-change-region|c-extend-font-lock-region-for-macros|c-extend-region-for-CPP\n\t|c-face-name-p|c-fdoc-shift-type-backward|c-fill-paragraph|c-find-assignment-for-mode|c-find-decl-prefix-search|c-find-decl-spots\n\t|c-find-invalid-doc-markup|c-fn-region-is-active-p|c-font-lock-<>-arglists|c-font-lock-c\\+\\+-new|c-font-lock-complex-decl-prepare\n\t|c-font-lock-declarations|c-font-lock-declarators|c-font-lock-doc-comments|c-font-lock-enclosing-decls|c-font-lock-enum-tail|c-font-lock-fontify-region\n\t|c-font-lock-init|c-font-lock-invalid-string|c-font-lock-keywords-2|c-font-lock-keywords-3|c-font-lock-keywords|c-font-lock-labels\n\t|c-font-lock-objc-method|c-font-lock-objc-methods|c-fontify-recorded-types-and-refs|c-fontify-types-and-refs|c-forward-<>-arglist-recur\n\t|c-forward-<>-arglist|c-forward-annotation|c-forward-comments|c-forward-conditional|c-forward-decl-or-cast-1|c-forward-id-comma-list\n\t|c-forward-into-nomenclature|c-forward-keyword-clause|c-forward-keyword-prefixed-id|c-forward-label|c-forward-name|c-forward-objc-directive\n\t|c-forward-over-cpp-define-id|c-forward-over-illiterals|c-forward-sexp|c-forward-single-comment|c-forward-sws|c-forward-syntactic-ws\n\t|c-forward-to-cpp-define-body|c-forward-to-nth-EOF-\\}|c-forward-token-1|c-forward-token-2|c-forward-type|c-get-cache-scan-pos\n\t|c-get-char-property|c-get-current-file|c-get-lang-constant|c-get-offset|c-get-style-variables|c-get-syntactic-indentation|c-gnu-impose-minimum\n\t|c-go-down-list-backward|c-go-down-list-forward|c-go-list-backward|c-go-list-forward|c-go-up-list-backward|c-go-up-list-forward\n\t|c-got-face-at|c-guess-accumulate-offset|c-guess-accumulate|c-guess-basic-syntax|c-guess-buffer-no-install|c-guess-buffer|c-guess-continued-construct\n\t|c-guess-current-offset|c-guess-dump-accumulator|c-guess-dump-guessed-style|c-guess-dump-guessed-values|c-guess-empty-line-p|c-guess-examine\n\t|c-guess-fill-prefix|c-guess-guess|c-guess-guessed-syntactic-symbols|c-guess-install|c-guess-make-basic-offset|c-guess-make-offsets-alist\n\t|c-guess-make-style|c-guess-merge-offsets-alists|c-guess-no-install|c-guess-region-no-install|c-guess-region|c-guess-reset-accumulator\n\t|c-guess-sort-accumulator|c-guess-style-name|c-guess-symbolize-integer|c-guess-symbolize-offsets-alist|c-guess-view-mark-guessed-entries\n\t|c-guess-view-reorder-offsets-alist-in-style|c-guess-view|c-guess|c-hungry-backspace|c-hungry-delete-backwards|c-hungry-delete-forward\n\t|c-hungry-delete|c-idl-menu|c-in-comment-line-prefix-p|c-in-function-trailer-p|c-in-gcc-asm-p|c-in-knr-argdecl|c-in-literal|c-in-method-def-p\n\t|c-indent-command|c-indent-defun|c-indent-exp|c-indent-line-or-region|c-indent-line|c-indent-multi-line-block|c-indent-new-comment-line\n\t|c-indent-one-line-block|c-indent-region|c-init-language-vars-for|c-initialize-builtin-style|c-initialize-cc-mode|c-inside-bracelist-p\n\t|c-int-to-char|c-intersect-lists|c-invalidate-find-decl-cache|c-invalidate-macro-cache|c-invalidate-state-cache-1|c-invalidate-state-cache\n\t|c-invalidate-sws-region-after|c-java-menu|c-just-after-func-arglist-p|c-keep-region-active|c-keyword-member|c-keyword-sym|c-lang-const\n\t|c-lang-defconst-eval-immediately|c-lang-defconst|c-lang-major-mode-is|c-langelem-2nd-pos|c-langelem-col|c-langelem-pos|c-langelem-sym\n\t|c-last-command-char|c-least-enclosing-brace|c-leave-cc-mode-mode|c-lineup-C-comments|c-lineup-ObjC-method-args-2|c-lineup-ObjC-method-args\n\t|c-lineup-ObjC-method-call-colons|c-lineup-ObjC-method-call|c-lineup-after-whitesmith-blocks|c-lineup-argcont-scan|c-lineup-argcont\n\t|c-lineup-arglist-close-under-paren|c-lineup-arglist-intro-after-paren|c-lineup-arglist-operators|c-lineup-arglist|c-lineup-assignments\n\t|c-lineup-cascaded-calls|c-lineup-close-paren|c-lineup-comment|c-lineup-cpp-define|c-lineup-dont-change|c-lineup-gcc-asm-reg|c-lineup-gnu-DEFUN-intro-cont\n\t|c-lineup-inexpr-block|c-lineup-java-inher|c-lineup-java-throws|c-lineup-knr-region-comment|c-lineup-math|c-lineup-multi-inher\n\t|c-lineup-respect-col-0|c-lineup-runin-statements|c-lineup-streamop|c-lineup-string-cont|c-lineup-template-args|c-lineup-topmost-intro-cont\n\t|c-lineup-whitesmith-in-block|c-list-found-types|c-literal-limits-fast|c-literal-limits|c-literal-type|c-looking-at-bos|c-looking-at-decl-block\n\t|c-looking-at-inexpr-block-backward|c-looking-at-inexpr-block|c-looking-at-non-alphnumspace|c-looking-at-special-brace-list|c-lookup-lists\n\t|c-macro-display-buffer|c-macro-expand|c-macro-expansion|c-macro-is-genuine-p|c-macro-vsemi-status-unknown-p|c-major-mode-is|c-make-bare-char-alt\n\t|c-make-font-lock-BO-decl-search-function|c-make-font-lock-context-search-function|c-make-font-lock-extra-types-blurb|c-make-font-lock-search-form\n\t|c-make-font-lock-search-function|c-make-inherited-keymap|c-make-inverse-face|c-make-keywords-re|c-make-macro-with-semi-re|c-make-styles-buffer-local\n\t|c-make-syntactic-matcher|c-mark-<-as-paren|c-mark->-as-paren|c-mark-function|c-mask-paragraph|c-mode-menu|c-mode-symbol|c-mode-var\n\t|c-mode|c-most-enclosing-brace|c-most-enclosing-decl-block|c-narrow-to-comment-innards|c-narrow-to-most-enclosing-decl-block|c-neutralize-CPP-line\n\t|c-neutralize-syntax-in-and-mark-CPP|c-newline-and-indent|c-next-single-property-change|c-objc-menu|c-on-identifier|c-one-line-string-p\n\t|c-outline-level|c-override-default-keywords|c-parse-state-1|c-parse-state-get-strategy|c-parse-state|c-partial-ws-p|c-pike-menu\n\t|c-point-syntax|c-point|c-populate-syntax-table|c-postprocess-file-styles|c-progress-fini|c-progress-init|c-progress-update|c-pull-open-brace\n\t|c-punctuation-in|c-put-c-type-property|c-put-char-property-fun|c-put-char-property|c-put-font-lock-face|c-put-font-lock-string-face\n\t|c-put-in-sws|c-put-is-sws|c-put-overlay|c-query-and-set-macro-start|c-query-macro-start|c-read-offset|c-real-parse-state|c-record-parse-state-state\n\t|c-record-ref-id|c-record-type-id|c-regexp-opt-depth|c-regexp-opt|c-region-is-active-p|c-remove-any-local-eval-or-mode-variables\n\t|c-remove-font-lock-face|c-remove-in-sws|c-remove-is-and-in-sws|c-remove-is-sws|c-remove-stale-state-cache-backwards|c-remove-stale-state-cache\n\t|c-renarrow-state-cache|c-replay-parse-state-state|c-restore-<->-as-parens|c-run-mode-hooks|c-safe-position|c-safe-scan-lists|c-safe\n\t|c-save-buffer-state|c-sc-parse-partial-sexp-no-category|c-sc-parse-partial-sexp|c-sc-scan-lists-no-category\\+1\\+1|c-sc-scan-lists-no-category\\+1-1\n\t|c-sc-scan-lists-no-category-1\\+1|c-sc-scan-lists-no-category-1-1|c-sc-scan-lists|c-scan-conditionals|c-scope-operator|c-search-backward-char-property\n\t|c-search-decl-header-end|c-search-forward-char-property|c-search-uplist-for-classkey|c-semi&comma-inside-parenlist|c-semi&comma-no-newlines-before-nonblanks\n\t|c-semi&comma-no-newlines-for-oneline-inliners|c-sentence-end|c-set-cpp-delimiters|c-set-fl-decl-start|c-set-offset|c-set-region-active\n\t|c-set-style-1|c-set-style|c-set-stylevar-fallback|c-setup-doc-comment-style|c-setup-filladapt|c-setup-paragraph-variables|c-shift-line-indentation\n\t|c-show-syntactic-information|c-simple-skip-symbol-backward|c-skip-comments-and-strings|c-skip-conditional|c-skip-ws-backward\n\t|c-skip-ws-forward|c-snug-1line-defun-close|c-snug-do-while|c-ssb-lit-begin|c-state-balance-parens-backwards|c-state-cache-after-top-paren\n\t|c-state-cache-init|c-state-cache-non-literal-place|c-state-cache-top-lparen|c-state-cache-top-paren|c-state-get-min-scan-pos\n\t|c-state-lit-beg|c-state-literal-at|c-state-mark-point-min-literal|c-state-maybe-marker|c-state-pp-to-literal|c-state-push-any-brace-pair\n\t|c-state-safe-place|c-state-semi-safe-place|c-submit-bug-report|c-subword-mode|c-suppress-<->-as-parens|c-syntactic-content|c-syntactic-end-of-macro\n\t|c-syntactic-information-on-region|c-syntactic-re-search-forward|c-syntactic-skip-backward|c-tentative-buffer-changes|c-tnt-chng-cleanup\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t c-tnt-chng-record-state|c-toggle-auto-hungry-state|c-toggle-auto-newline|c-toggle-auto-state|c-toggle-electric-state|c-toggle-hungry-state\n\t|c-toggle-parse-state-debug|c-toggle-syntactic-indentation|c-trim-found-types|c-try-one-liner|c-uncomment-out-cpps|c-unfind-coalesced-tokens\n\t|c-unfind-enclosing-token|c-unfind-type|c-unmark-<->-as-paren|c-up-conditional-with-else|c-up-conditional|c-up-list-backward|c-up-list-forward\n\t|c-update-modeline|c-valid-offset|c-version|c-vsemi-status-unknown-p|c-whack-state-after|c-whack-state-before|c-where-wrt-brace-construct\n\t|c-while-widening-to-decl-block|c-widen-to-enclosing-decl-scope|c-with-<->-as-parens-suppressed|c-with-all-but-one-cpps-commented-out\n\t|c-with-cpps-commented-out|c-with-syntax-table|caaaar|caaadr|caaar|caadar|caaddr|caadr|cadaar|cadadr|cadar|caddar|cadddr|caddr|cal-html-cursor-month\n\t|cal-html-cursor-year|cal-menu-context-mouse-menu|cal-menu-global-mouse-menu|cal-menu-holiday-window-suffix|cal-menu-set-date-title\n\t|cal-menu-x-popup-menu|cal-tex-cursor-day|cal-tex-cursor-filofax-2week|cal-tex-cursor-filofax-daily|cal-tex-cursor-filofax-week\n\t|cal-tex-cursor-filofax-year|cal-tex-cursor-month-landscape|cal-tex-cursor-month|cal-tex-cursor-week-iso|cal-tex-cursor-week-monday\n\t|cal-tex-cursor-week|cal-tex-cursor-week2-summary|cal-tex-cursor-week2|cal-tex-cursor-year-landscape|cal-tex-cursor-year|calc-alg-digit-entry\n\t|calc-alg-entry|calc-algebraic-entry|calc-align-stack-window|calc-auto-algebraic-entry|calc-big-or-small|calc-binary-op|calc-change-sign\n\t|calc-check-defines|calc-check-stack|calc-check-trail-aligned|calc-check-user-syntax|calc-clear-unread-commands|calc-count-lines\n\t|calc-create-buffer|calc-cursor-stack-index|calc-dispatch-help|calc-dispatch|calc-divide|calc-do-alg-entry|calc-do-calc-eval|calc-do-dispatch\n\t|calc-do-embedded-activate|calc-do-handle-whys|calc-do-quick-calc|calc-do-refresh|calc-do|calc-embedded-activate|calc-embedded|calc-enter-result\n\t|calc-enter|calc-eval|calc-get-stack-element|calc-grab-rectangle|calc-grab-region|calc-grab-sum-across|calc-grab-sum-down|calc-handle-whys\n\t|calc-help|calc-info-goto-node|calc-info-summary|calc-info|calc-inv|calc-keypad|calc-kill-stack-buffer|calc-last-args-stub|calc-left-divide\n\t|calc-match-user-syntax|calc-minibuffer-contains|calc-minibuffer-size|calc-minus|calc-missing-key|calc-mod|calc-mode-var-list-restore-default-values\n\t|calc-mode-var-list-restore-saved-values|calc-normalize|calc-num-prefix-name|calc-other-window|calc-over|calc-percent|calc-plus\n\t|calc-pop-above|calc-pop-push-list|calc-pop-push-record-list|calc-pop-stack|calc-pop|calc-power|calc-push-list|calc-quit|calc-read-key-sequence\n\t|calc-read-key|calc-record-list|calc-record-undo|calc-record-why|calc-record|calc-refresh|calc-renumber-stack|calc-report-bug|calc-roll-down-stack\n\t|calc-roll-down|calc-roll-up-stack|calc-roll-up|calc-same-interface|calc-select-buffer|calc-set-command-flag|calc-set-mode-line\n\t|calc-shift-Y-prefix-help|calc-slow-wrapper|calc-stack-size|calc-substack-height|calc-temp-minibuffer-message|calc-times|calc-top-list-n\n\t|calc-top-list|calc-top-n|calc-top|calc-trail-buffer|calc-trail-display|calc-trail-here|calc-transpose-lines|calc-tutorial|calc-unary-op\n\t|calc-undo|calc-unread-command|calc-user-invocation|calc-window-width|calc-with-default-simplification|calc-with-trail-buffer|calc-wrapper\n\t|calc-yank|calc|calcDigit-algebraic|calcDigit-backspace|calcDigit-edit|calcDigit-key|calcDigit-letter|calcDigit-nondigit|calcDigit-start\n\t|calcFunc-floor|calcFunc-inv|calcFunc-trunc|calculate-icon-indent|calculate-lisp-indent|calculate-tcl-indent|calculator-add-operators\n\t|calculator-backspace|calculator-clear-fragile|calculator-clear-saved|calculator-clear|calculator-close-paren|calculator-copy|calculator-dec\\/deg-mode\n\t|calculator-decimal|calculator-digit|calculator-displayer-next|calculator-displayer-prev|calculator-eng-display|calculator-enter\n\t|calculator-exp|calculator-expt|calculator-fact|calculator-funcall|calculator-get-display|calculator-get-register|calculator-groupize-number\n\t|calculator-help|calculator-last-input|calculator-menu|calculator-message|calculator-mode|calculator-need-3-lines|calculator-number-to-string\n\t|calculator-op-arity|calculator-op-or-exp|calculator-op-prec|calculator-op|calculator-open-paren|calculator-paste|calculator-push-curnum\n\t|calculator-put-value|calculator-quit|calculator-radix-input-mode|calculator-radix-mode|calculator-radix-output-mode|calculator-reduce-stack-once\n\t|calculator-reduce-stack|calculator-remove-zeros|calculator-repL|calculator-repR|calculator-reset|calculator-rotate-displayer-back\n\t|calculator-rotate-displayer|calculator-save-and-quit|calculator-save-on-list|calculator-saved-down|calculator-saved-move|calculator-saved-up\n\t|calculator-set-register|calculator-standard-displayer|calculator-string-to-number|calculator-truncate|calculator-update-display\n\t|calculator|calendar-abbrev-construct|calendar-absolute-from-gregorian|calendar-astro-date-string|calendar-astro-from-absolute\n\t|calendar-astro-goto-day-number|calendar-astro-print-day-number|calendar-astro-to-absolute|calendar-backward-day|calendar-backward-month\n\t|calendar-backward-week|calendar-backward-year|calendar-bahai-date-string|calendar-bahai-goto-date|calendar-bahai-mark-date-pattern\n\t|calendar-bahai-print-date|calendar-basic-setup|calendar-beginning-of-month|calendar-beginning-of-week|calendar-beginning-of-year\n\t|calendar-buffer-list|calendar-check-holidays|calendar-chinese-date-string|calendar-chinese-goto-date|calendar-chinese-print-date\n\t|calendar-column-to-segment|calendar-coptic-date-string|calendar-coptic-goto-date|calendar-coptic-print-date|calendar-count-days-region\n\t|calendar-current-date|calendar-cursor-holidays|calendar-cursor-to-date|calendar-cursor-to-nearest-date|calendar-cursor-to-visible-date\n\t|calendar-customized-p|calendar-date-compare|calendar-date-equal|calendar-date-is-valid-p|calendar-date-is-visible-p|calendar-date-string\n\t|calendar-day-header-construct|calendar-day-name|calendar-day-number|calendar-day-of-week|calendar-day-of-year-string|calendar-dayname-on-or-before\n\t|calendar-end-of-month|calendar-end-of-week|calendar-end-of-year|calendar-ensure-newline|calendar-ethiopic-date-string|calendar-ethiopic-goto-date\n\t|calendar-ethiopic-print-date|calendar-exchange-point-and-mark|calendar-exit|calendar-extract-day|calendar-extract-month|calendar-extract-year\n\t|calendar-forward-day|calendar-forward-month|calendar-forward-week|calendar-forward-year|calendar-frame-setup|calendar-french-date-string\n\t|calendar-french-goto-date|calendar-french-print-date|calendar-generate-month|calendar-generate-window|calendar-generate|calendar-goto-date\n\t|calendar-goto-day-of-year|calendar-goto-info-node|calendar-goto-today|calendar-gregorian-from-absolute|calendar-hebrew-date-string\n\t|calendar-hebrew-goto-date|calendar-hebrew-list-yahrzeits|calendar-hebrew-mark-date-pattern|calendar-hebrew-print-date|calendar-holiday-list\n\t|calendar-in-read-only-buffer|calendar-increment-month-cons|calendar-increment-month|calendar-insert-at-column|calendar-interval\n\t|calendar-islamic-date-string|calendar-islamic-goto-date|calendar-islamic-mark-date-pattern|calendar-islamic-print-date|calendar-iso-date-string\n\t|calendar-iso-from-absolute|calendar-iso-goto-date|calendar-iso-goto-week|calendar-iso-print-date|calendar-julian-date-string\n\t|calendar-julian-from-absolute|calendar-julian-goto-date|calendar-julian-print-date|calendar-last-day-of-month|calendar-leap-year-p\n\t|calendar-list-holidays|calendar-lunar-phases|calendar-make-alist|calendar-make-temp-face|calendar-mark-1|calendar-mark-complex\n\t|calendar-mark-date-pattern|calendar-mark-days-named|calendar-mark-holidays|calendar-mark-month|calendar-mark-today|calendar-mark-visible-date\n\t|calendar-mayan-date-string|calendar-mayan-goto-long-count-date|calendar-mayan-next-haab-date|calendar-mayan-next-round-date\n\t|calendar-mayan-next-tzolkin-date|calendar-mayan-previous-haab-date|calendar-mayan-previous-round-date|calendar-mayan-previous-tzolkin-date\n\t|calendar-mayan-print-date|calendar-mode-line-entry|calendar-mode|calendar-month-edges|calendar-month-name|calendar-mouse-view-diary-entries\n\t|calendar-mouse-view-other-diary-entries|calendar-move-to-column|calendar-nongregorian-visible-p|calendar-not-implemented|calendar-nth-named-absday\n\t|calendar-nth-named-day|calendar-other-dates|calendar-other-month|calendar-persian-date-string|calendar-persian-goto-date|calendar-persian-print-date\n\t|calendar-print-day-of-year|calendar-print-other-dates|calendar-read-date|calendar-read|calendar-recompute-layout-variables|calendar-redraw\n\t|calendar-scroll-left-three-months|calendar-scroll-left|calendar-scroll-right-three-months|calendar-scroll-right|calendar-scroll-toolkit-scroll\n\t|calendar-set-date-style|calendar-set-layout-variable|calendar-set-mark|calendar-set-mode-line|calendar-star-date|calendar-string-spread\n\t|calendar-sum|calendar-sunrise-sunset-month|calendar-sunrise-sunset|calendar-unmark|calendar-update-mode-line|calendar-week-end-day\n\t|calendar|call-last-kbd-macro|call-next-method|callf|callf2|cancel-edebug-on-entry|cancel-function-timers|cancel-kbd-macro-events\n\t|cancel-timer-internal|canlock-insert-header|canlock-verify|canonicalize-coding-system-name|canonically-space-region|capitalized-words-mode\n\t|car-less-than-car|case-table-get-table|case|cc-choose-style-for-mode|cc-eval-when-compile|cc-imenu-init|cc-imenu-java-build-type-args-regex\n\t|cc-imenu-objc-function|cc-imenu-objc-method-to-selector|cc-imenu-objc-remove-white-space|ccl-compile|ccl-dump|ccl-execute-on-string\n\t|ccl-execute-with-args|ccl-execute|ccl-program-p|cconv--analyze-function|cconv--analyze-use|cconv--convert-function|cconv--map-diff-elem\n\t|cconv--map-diff-set|cconv--map-diff|cconv--set-diff-map|cconv--set-diff|cconv-analyse-form|cconv-analyze-form|cconv-closure-convert\n\t|cconv-convert|cconv-warnings-only|cd-absolute|cd|cdaaar|cdaadr|cdaar|cdadar|cdaddr|cdadr|cddaar|cddadr|cddar|cdddar|cddddr|cdddr|cdl-get-file\n\t|cdl-put-region|cedet-version|ceiling\\*|center-line|center-paragraph|center-region|cfengine-auto-mode|cfengine-common-settings|cfengine-common-syntax\n\t|cfengine-fill-paragraph|cfengine-mode|cfengine2-beginning-of-defun|cfengine2-end-of-defun|cfengine2-indent-line|cfengine2-mode\n\t|cfengine2-outline-level|cfengine3--current-function|cfengine3-beginning-of-defun|cfengine3-clear-syntax-cache|cfengine3-completion-function\n\t|cfengine3-create-imenu-index|cfengine3-current-defun|cfengine3-documentation-function|cfengine3-end-of-defun|cfengine3-format-function-docstring\n\t|cfengine3-indent-line|cfengine3-make-syntax-cache|cfengine3-mode|change-class|change-log-beginning-of-defun|change-log-end-of-defun\n\t|change-log-fill-forward-paragraph|change-log-fill-parenthesized-list|change-log-find-file|change-log-get-method-definition-1\n\t|change-log-get-method-definition|change-log-goto-source-1|change-log-goto-source|change-log-indent|change-log-merge|change-log-mode\n\t|change-log-name|change-log-next-buffer|change-log-next-error|change-log-resolve-conflict|change-log-search-file-name|change-log-search-tag-name-1\n\t|change-log-search-tag-name|change-log-sortable-date-at|change-log-version-number-search|char-resolve-modifiers|char-valid-p|charset-bytes\n\t|charset-chars|charset-description|charset-dimension|charset-id-internal|charset-id|charset-info|charset-iso-final-char|charset-long-name\n\t|charset-short-name|chart-add-sequence|chart-axis-child-p|chart-axis-draw|chart-axis-list-p|chart-axis-names-child-p|chart-axis-names-list-p\n\t|chart-axis-names-p|chart-axis-names|chart-axis-p|chart-axis-range-child-p|chart-axis-range-list-p|chart-axis-range-p|chart-axis-range\n\t|chart-axis|chart-bar-child-p|chart-bar-list-p|chart-bar-p|chart-bar-quickie|chart-bar|chart-child-p|chart-deface-rectangle|chart-display-label\n\t|chart-draw-axis|chart-draw-data|chart-draw-line|chart-draw-title|chart-draw|chart-emacs-lists|chart-emacs-storage|chart-file-count\n\t|chart-goto-xy|chart-list-p|chart-mode|chart-new-buffer|chart-p|chart-rmail-from|chart-sequece-child-p|chart-sequece-list-p|chart-sequece-p\n\t|chart-sequece|chart-size-in-dir|chart-sort-matchlist|chart-sort|chart-space-usage|chart-test-it-all|chart-translate-namezone|chart-translate-xpos\n\t|chart-translate-ypos|chart-trim|chart-zap-chars|chart|check-ccl-program|check-completion-length|check-declare-directory|check-declare-errmsg\n\t|check-declare-file|check-declare-files|check-declare-locate|check-declare-scan|check-declare-sort|check-declare-verify|check-declare-warn\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t check-face|check-ispell-version|check-parens|check-type|checkdoc-autofix-ask-replace|checkdoc-buffer-label|checkdoc-char=|checkdoc-comments\n\t|checkdoc-continue|checkdoc-create-common-verbs-regexp|checkdoc-create-error|checkdoc-current-buffer|checkdoc-defun-info|checkdoc-defun\n\t|checkdoc-delete-overlay|checkdoc-display-status-buffer|checkdoc-error-end|checkdoc-error-start|checkdoc-error-text|checkdoc-error-unfixable\n\t|checkdoc-error|checkdoc-eval-current-buffer|checkdoc-eval-defun|checkdoc-file-comments-engine|checkdoc-in-example-string-p|checkdoc-in-sample-code-p\n\t|checkdoc-interactive-ispell-loop|checkdoc-interactive-loop|checkdoc-interactive|checkdoc-ispell-comments|checkdoc-ispell-continue\n\t|checkdoc-ispell-current-buffer|checkdoc-ispell-defun|checkdoc-ispell-docstring-engine|checkdoc-ispell-init|checkdoc-ispell-interactive\n\t|checkdoc-ispell-message-interactive|checkdoc-ispell-message-text|checkdoc-ispell-start|checkdoc-ispell|checkdoc-list-of-strings-p\n\t|checkdoc-make-overlay|checkdoc-message-interactive-ispell-loop|checkdoc-message-interactive|checkdoc-message-text-engine|checkdoc-message-text-next-string\n\t|checkdoc-message-text-search|checkdoc-message-text|checkdoc-mode-line-update|checkdoc-next-docstring|checkdoc-next-error|checkdoc-next-message-error\n\t|checkdoc-output-mode|checkdoc-outside-major-sexp|checkdoc-overlay-end|checkdoc-overlay-put|checkdoc-overlay-start|checkdoc-proper-noun-region-engine\n\t|checkdoc-recursive-edit|checkdoc-rogue-space-check-engine|checkdoc-rogue-spaces|checkdoc-run-hooks|checkdoc-sentencespace-region-engine\n\t|checkdoc-show-diagnostics|checkdoc-start-section|checkdoc-start|checkdoc-this-string-valid-engine|checkdoc-this-string-valid\n\t|checkdoc-y-or-n-p|checkdoc|child-of-class-p|chmod|choose-completion-delete-max-match|choose-completion-guess-base-position|choose-completion-string\n\t|choose-completion|cl--adjoin|cl--arglist-args|cl--block-throw--cmacro|cl--block-throw|cl--block-wrapper--cmacro|cl--block-wrapper\n\t|cl--check-key|cl--check-match|cl--check-test-nokey|cl--check-test|cl--compile-time-too|cl--compiler-macro-adjoin|cl--compiler-macro-assoc\n\t|cl--compiler-macro-cXXr|cl--compiler-macro-get|cl--compiler-macro-list\\*|cl--compiler-macro-member|cl--compiler-macro-typep\n\t|cl--compiling-file|cl--const-expr-p|cl--const-expr-val|cl--defalias|cl--defsubst-expand|cl--delete-duplicates|cl--do-arglist|cl--do-prettyprint\n\t|cl--do-proclaim|cl--do-remf|cl--do-subst|cl--expand-do-loop|cl--expr-contains-any|cl--expr-contains|cl--expr-depends-p|cl--finite-do\n\t|cl--function-convert|cl--gv-adapt|cl--labels-convert|cl--letf|cl--loop-build-ands|cl--loop-handle-accum|cl--loop-let|cl--loop-set-iterator-function\n\t|cl--macroexp-fboundp|cl--make-type-test|cl--make-usage-args|cl--make-usage-var|cl--map-intervals|cl--map-keymap-recursively|cl--map-overlays\n\t|cl--mapcar-many|cl--nsublis-rec|cl--parse-loop-clause|cl--parsing-keywords|cl--pass-args-to-cl-declare|cl--pop2|cl--position|cl--random-time\n\t|cl--safe-expr-p|cl--set-buffer-substring|cl--set-frame-visible-p|cl--set-getf|cl--set-substring|cl--simple-expr-p|cl--simple-exprs-p\n\t|cl--sm-macroexpand|cl--struct-epg-context-p--cmacro|cl--struct-epg-context-p|cl--struct-epg-data-p--cmacro|cl--struct-epg-data-p\n\t|cl--struct-epg-import-result-p--cmacro|cl--struct-epg-import-result-p|cl--struct-epg-import-status-p--cmacro|cl--struct-epg-import-status-p\n\t|cl--struct-epg-key-p--cmacro|cl--struct-epg-key-p|cl--struct-epg-key-signature-p--cmacro|cl--struct-epg-key-signature-p|cl--struct-epg-new-signature-p--cmacro\n\t|cl--struct-epg-new-signature-p|cl--struct-epg-sig-notation-p--cmacro|cl--struct-epg-sig-notation-p|cl--struct-epg-signature-p--cmacro\n\t|cl--struct-epg-signature-p|cl--struct-epg-sub-key-p--cmacro|cl--struct-epg-sub-key-p|cl--struct-epg-user-id-p--cmacro|cl--struct-epg-user-id-p\n\t|cl--sublis-rec|cl--sublis|cl--transform-lambda|cl--tree-equal-rec|cl--unused-var-p|cl--wrap-in-nil-block|cl-caaaar|cl-caaadr|cl-caaar\n\t|cl-caadar|cl-caaddr|cl-caadr|cl-cadaar|cl-cadadr|cl-cadar|cl-caddar|cl-cadddr|cl-cdaaar|cl-cdaadr|cl-cdaar|cl-cdadar|cl-cdaddr|cl-cdadr\n\t|cl-cddaar|cl-cddadr|cl-cddar|cl-cdddar|cl-cddddr|cl-cdddr|cl-clrhash|cl-copy-seq|cl-copy-tree|cl-digit-char-p|cl-eighth|cl-fifth|cl-flet\\*\n\t|cl-floatp-safe|cl-fourth|cl-fresh-line|cl-gethash|cl-hash-table-count|cl-hash-table-p|cl-maclisp-member|cl-macroexpand-all|cl-macroexpand\n\t|cl-make-hash-table|cl-map-extents|cl-map-intervals|cl-map-keymap-recursively|cl-map-keymap|cl-maphash|cl-multiple-value-apply|cl-multiple-value-call\n\t|cl-multiple-value-list|cl-ninth|cl-not-hash-table|cl-nreconc|cl-nth-value|cl-parse-integer|cl-prettyprint|cl-puthash|cl-remhash|cl-revappend\n\t|cl-second|cl-set-getf|cl-seventh|cl-signum|cl-sixth|cl-struct-sequence-type|cl-struct-setf-expander|cl-struct-slot-info|cl-struct-slot-offset\n\t|cl-struct-slot-value--cmacro|cl-struct-slot-value|cl-svref|cl-tenth|cl-third|cl-unload-function|cl-values-list|cl-values|class-abstract-p\n\t|class-children|class-constructor|class-direct-subclasses|class-direct-superclasses|class-method-invocation-order|class-name|class-of\n\t|class-option-assoc|class-option|class-p|class-parent|class-parents|class-precedence-list|class-slot-initarg|class-v|clean-buffer-list-delay\n\t|clean-buffer-list|clear-all-completions|clear-buffer-auto-save-failure|clear-charset-maps|clear-face-cache|clear-font-cache|clear-rectangle-line\n\t|clear-rectangle|clipboard-kill-region|clipboard-kill-ring-save|clipboard-yank|clone-buffer|clone-indirect-buffer-other-window\n\t|clone-process|clone|close-display-connection|close-font|close-rectangle|cmpl-coerce-string-case|cmpl-hours-since-origin|cmpl-merge-string-cases\n\t|cmpl-prefix-entry-head|cmpl-prefix-entry-tail|cmpl-string-case-type|coding-system-base|coding-system-category|coding-system-doc-string\n\t|coding-system-eol-type-mnemonic|coding-system-equal|coding-system-from-name|coding-system-lessp|coding-system-mnemonic|coding-system-plist\n\t|coding-system-post-read-conversion|coding-system-pre-write-conversion|coding-system-put|coding-system-translation-table-for-decode\n\t|coding-system-translation-table-for-encode|coding-system-type|coerce|color-cie-de2000|color-clamp|color-complement-hex|color-complement\n\t|color-darken-hsl|color-darken-name|color-desaturate-hsl|color-desaturate-name|color-distance|color-gradient|color-hsl-to-rgb|color-hue-to-rgb\n\t|color-lab-to-srgb|color-lab-to-xyz|color-lighten-hsl|color-lighten-name|color-name-to-rgb|color-rgb-to-hex|color-rgb-to-hsl|color-rgb-to-hsv\n\t|color-saturate-hsl|color-saturate-name|color-srgb-to-lab|color-srgb-to-xyz|color-xyz-to-lab|color-xyz-to-srgb|column-number-mode\n\t|combine-after-change-execute|comint--complete-file-name-data|comint--match-partial-filename|comint--requote-argument|comint--unquote&expand-filename\n\t|comint--unquote&requote-argument|comint--unquote-argument|comint-accumulate|comint-add-to-input-history|comint-adjust-point|comint-adjust-window-point\n\t|comint-after-pmark-p|comint-append-output-to-file|comint-args|comint-arguments|comint-backward-matching-input|comint-bol-or-process-mark\n\t|comint-bol|comint-c-a-p-replace-by-expanded-history|comint-carriage-motion|comint-check-proc|comint-check-source|comint-completion-at-point\n\t|comint-completion-file-name-table|comint-continue-subjob|comint-copy-old-input|comint-delchar-or-maybe-eof|comint-delete-input\n\t|comint-delete-output|comint-delim-arg|comint-directory|comint-dynamic-complete-as-filename|comint-dynamic-complete-filename|comint-dynamic-complete\n\t|comint-dynamic-list-completions|comint-dynamic-list-filename-completions|comint-dynamic-list-input-ring-select|comint-dynamic-list-input-ring\n\t|comint-dynamic-simple-complete|comint-exec-1|comint-exec|comint-extract-string|comint-filename-completion|comint-forward-matching-input\n\t|comint-get-next-from-history|comint-get-old-input-default|comint-get-source|comint-goto-input|comint-goto-process-mark|comint-history-isearch-backward-regexp\n\t|comint-history-isearch-backward|comint-history-isearch-end|comint-history-isearch-message|comint-history-isearch-pop-state|comint-history-isearch-push-state\n\t|comint-history-isearch-search|comint-history-isearch-setup|comint-history-isearch-wrap|comint-how-many-region|comint-insert-input\n\t|comint-insert-previous-argument|comint-interrupt-subjob|comint-kill-input|comint-kill-region|comint-kill-subjob|comint-kill-whole-line\n\t|comint-line-beginning-position|comint-magic-space|comint-match-partial-filename|comint-mode|comint-next-input|comint-next-matching-input-from-input\n\t|comint-next-matching-input|comint-next-prompt|comint-output-filter|comint-postoutput-scroll-to-bottom|comint-preinput-scroll-to-bottom\n\t|comint-previous-input-string|comint-previous-input|comint-previous-matching-input-from-input|comint-previous-matching-input-string-position\n\t|comint-previous-matching-input-string|comint-previous-matching-input|comint-previous-prompt|comint-proc-query|comint-quit-subjob\n\t|comint-quote-filename|comint-read-input-ring|comint-read-noecho|comint-redirect-cleanup|comint-redirect-filter|comint-redirect-preoutput-filter\n\t|comint-redirect-remove-redirection|comint-redirect-results-list-from-process|comint-redirect-results-list|comint-redirect-send-command-to-process\n\t|comint-redirect-send-command|comint-redirect-setup|comint-regexp-arg|comint-replace-by-expanded-filename|comint-replace-by-expanded-history-before-point\n\t|comint-replace-by-expanded-history|comint-restore-input|comint-run|comint-search-arg|comint-search-start|comint-send-eof|comint-send-input\n\t|comint-send-region|comint-send-string|comint-set-process-mark|comint-show-maximum-output|comint-show-output|comint-simple-send\n\t|comint-skip-input|comint-skip-prompt|comint-snapshot-last-prompt|comint-source-default|comint-stop-subjob|comint-strip-ctrl-m\n\t|comint-substitute-in-file-name|comint-truncate-buffer|comint-unquote-filename|comint-update-fence|comint-watch-for-password-prompt\n\t|comint-within-quotes|comint-word|comint-write-input-ring|comint-write-output|command-apropos|command-error-default-function|command-history-mode\n\t|command-history-repeat|command-line-1|command-line-normalize-file-name|comment-add|comment-beginning|comment-box|comment-choose-indent\n\t|comment-dwim|comment-enter-backward|comment-forward|comment-indent-default|comment-indent-new-line|comment-indent|comment-kill\n\t|comment-make-extra-lines|comment-normalize-vars|comment-only-p|comment-or-uncomment-region|comment-padleft|comment-padright|comment-quote-nested\n\t|comment-quote-re|comment-region-default|comment-region-internal|comment-region|comment-search-backward|comment-search-forward\n\t|comment-set-column|comment-string-reverse|comment-string-strip|comment-valid-prefix-p|comment-with-narrowing|common-lisp-indent-function\n\t|common-lisp-mode|compare-windows-dehighlight|compare-windows-get-next-window|compare-windows-get-recent-window|compare-windows-highlight\n\t|compare-windows-skip-whitespace|compare-windows-sync-default-function|compare-windows-sync-regexp|compare-windows|compilation--compat-error-properties\n\t|compilation--compat-parse-errors|compilation--ensure-parse|compilation--file-struct->file-spec|compilation--file-struct->formats\n\t|compilation--file-struct->loc-tree|compilation--flush-directory-cache|compilation--flush-file-structure|compilation--flush-parse\n\t|compilation--loc->col|compilation--loc->file-struct|compilation--loc->line|compilation--loc->marker|compilation--loc->visited\n\t|compilation--make-cdrloc|compilation--make-file-struct|compilation--make-message--cmacro|compilation--make-message|compilation--message->end-loc--cmacro\n\t|compilation--message->end-loc|compilation--message->loc--cmacro|compilation--message->loc|compilation--message->type--cmacro\n\t|compilation--message->type|compilation--message-p--cmacro|compilation--message-p|compilation--parse-region|compilation--previous-directory\n\t|compilation--put-prop|compilation--remove-properties|compilation--unsetup|compilation-auto-jump|compilation-buffer-internal-p\n\t|compilation-buffer-name|compilation-buffer-p|compilation-button-map|compilation-directory-properties|compilation-display-error\n\t|compilation-error-properties|compilation-face|compilation-fake-loc|compilation-filter|compilation-find-buffer|compilation-find-file\n\t|compilation-forget-errors|compilation-get-file-structure|compilation-goto-locus-delete-o|compilation-goto-locus|compilation-handle-exit\n\t|compilation-internal-error-properties|compilation-loop|compilation-minor-mode|compilation-mode-font-lock-keywords|compilation-mode\n\t|compilation-move-to-column|compilation-next-error-function|compilation-next-error|compilation-next-file|compilation-next-single-property-change\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t compilation-parse-errors|compilation-previous-error|compilation-previous-file|compilation-read-command|compilation-revert-buffer\n\t|compilation-sentinel|compilation-set-skip-threshold|compilation-set-window-height|compilation-set-window|compilation-setup|compilation-shell-minor-mode\n\t|compilation-start|compile-goto-error|compile-mouse-goto-error|compile|compiler-macroexpand|complete-in-turn|complete-symbol|complete-tag\n\t|complete-with-action|complete|completing-read-default|completing-read-multiple|completion--cache-all-sorted-completions|completion--capf-wrapper\n\t|completion--common-suffix|completion--complete-and-exit|completion--cycle-threshold|completion--do-completion|completion--done\n\t|completion--embedded-envvar-table|completion--field-metadata|completion--file-name-table|completion--flush-all-sorted-completions\n\t|completion--in-region-1|completion--in-region|completion--insert-strings|completion--make-envvar-table|completion--merge-suffix\n\t|completion--message|completion--metadata|completion--nth-completion|completion--post-self-insert|completion--replace|completion--sifn-requote\n\t|completion--some|completion--string-equal-p|completion--styles|completion--try-word-completion|completion--twq-all|completion--twq-try\n\t|completion-all-completions|completion-all-sorted-completions|completion-backup-filename|completion-basic--pattern|completion-basic-all-completions\n\t|completion-basic-try-completion|completion-before-command|completion-c-mode-hook|completion-complete-and-exit|completion-def-wrapper\n\t|completion-emacs21-all-completions|completion-emacs21-try-completion|completion-emacs22-all-completions|completion-emacs22-try-completion\n\t|completion-file-name-table|completion-find-file-hook|completion-help-at-point|completion-hilit-commonality|completion-in-region--postch\n\t|completion-in-region--single-word|completion-in-region-mode|completion-initialize|completion-initials-all-completions|completion-initials-expand\n\t|completion-initials-try-completion|completion-kill-region|completion-last-use-time|completion-lisp-mode-hook|completion-list-mode-finish\n\t|completion-list-mode|completion-metadata-get|completion-metadata|completion-mode|completion-num-uses|completion-pcm--all-completions\n\t|completion-pcm--filename-try-filter|completion-pcm--find-all-completions|completion-pcm--hilit-commonality|completion-pcm--merge-completions\n\t|completion-pcm--merge-try|completion-pcm--optimize-pattern|completion-pcm--pattern->regex|completion-pcm--pattern->string|completion-pcm--pattern-trivial-p\n\t|completion-pcm--prepare-delim-re|completion-pcm--string->pattern|completion-pcm-all-completions|completion-pcm-try-completion\n\t|completion-search-next|completion-search-peek|completion-search-reset-1|completion-search-reset|completion-setup-fortran-mode\n\t|completion-setup-function|completion-source|completion-string|completion-substring--all-completions|completion-substring-all-completions\n\t|completion-substring-try-completion|completion-table-with-context|completion-try-completion|compose-chars-after|compose-chars\n\t|compose-glyph-string-relative|compose-glyph-string|compose-gstring-for-dotted-circle|compose-gstring-for-graphic|compose-gstring-for-terminal\n\t|compose-gstring-for-variation-glyph|compose-last-chars|compose-mail-other-frame|compose-mail-other-window|compose-mail|compose-region-internal\n\t|compose-region|compose-string-internal|compose-string|composition-get-gstring|concatenate|condition-case-no-debug|conf-align-assignments\n\t|conf-colon-mode|conf-javaprop-mode|conf-mode-initialize|conf-mode-maybe|conf-mode|conf-outline-level|conf-ppd-mode|conf-quote-normal\n\t|conf-space-keywords|conf-space-mode-internal|conf-space-mode|conf-unix-mode|conf-windows-mode|conf-xdefaults-mode|confirm-nonexistent-file-or-buffer\n\t|constructor|convert-define-charset-argument|cookie-apropos|cookie-check-file|cookie-doctor|cookie-insert|cookie-read|cookie-shuffle-vector\n\t|cookie-snarf|cookie|cookie1|copy-case-table|copy-cvs-flags|copy-cvs-tag|copy-dir-locals-to-file-locals-prop-line|copy-dir-locals-to-file-locals\n\t|copy-ebrowse-bs|copy-ebrowse-cs|copy-ebrowse-hs|copy-ebrowse-ms|copy-ebrowse-position|copy-ebrowse-ts|copy-erc-channel-user|copy-erc-response\n\t|copy-erc-server-user|copy-ert--ewoc-entry|copy-ert--stats|copy-ert--test-execution-info|copy-ert-test-aborted-with-non-local-exit\n\t|copy-ert-test-failed|copy-ert-test-passed|copy-ert-test-quit|copy-ert-test-result-with-condition|copy-ert-test-result|copy-ert-test-skipped\n\t|copy-ert-test|copy-ewoc--node|copy-ewoc|copy-face|copy-file-locals-to-dir-locals|copy-flymake-ler|copy-gdb-handler|copy-gdb-table\n\t|copy-htmlize-fstruct|copy-js--js-handle|copy-js--pitem|copy-list|copy-package--bi-desc|copy-package-desc|copy-profiler-calltree\n\t|copy-profiler-profile|copy-rectangle-as-kill|copy-rectangle-to-register|copy-seq|copy-ses--locprn|copy-sgml-tag|copy-soap-array-type\n\t|copy-soap-basic-type|copy-soap-binding|copy-soap-bound-operation|copy-soap-element|copy-soap-message|copy-soap-namespace-link\n\t|copy-soap-namespace|copy-soap-operation|copy-soap-port-type|copy-soap-port|copy-soap-sequence-element|copy-soap-sequence-type\n\t|copy-soap-simple-type|copy-soap-wsdl|copy-tar-header|copy-to-buffer|copy-to-register|copy-url-queue|copyright-find-copyright|copyright-find-end\n\t|copyright-fix-years|copyright-limit|copyright-offset-too-large-p|copyright-re-search|copyright-start-point|copyright-update-directory\n\t|copyright-update-year|copyright-update|copyright|count-if-not|count-if|count-lines-page|count-lines-region|count-matches|count-text-lines\n\t|count-trailing-whitespace-region|count-windows|count-words--buffer-message|count-words--message|count-words-region|count|cperl-1\\+\n\t|cperl-1-|cperl-add-tags-recurse-noxs-fullpath|cperl-add-tags-recurse-noxs|cperl-add-tags-recurse|cperl-after-block-and-statement-beg\n\t|cperl-after-block-p|cperl-after-change-function|cperl-after-expr-p|cperl-after-label|cperl-after-sub-regexp|cperl-at-end-of-expr\n\t|cperl-backward-to-noncomment|cperl-backward-to-start-of-continued-exp|cperl-backward-to-start-of-expr|cperl-beautify-level|cperl-beautify-regexp-piece\n\t|cperl-beautify-regexp|cperl-beginning-of-property|cperl-block-p|cperl-build-manpage|cperl-cached-syntax-table|cperl-calculate-indent-within-comment\n\t|cperl-calculate-indent|cperl-check-syntax|cperl-choose-color|cperl-comment-indent|cperl-comment-region|cperl-commentify|cperl-contract-level\n\t|cperl-contract-levels|cperl-db|cperl-define-key|cperl-delay-update-hook|cperl-describe-perl-symbol|cperl-do-auto-fill|cperl-electric-backspace\n\t|cperl-electric-brace|cperl-electric-else|cperl-electric-keyword|cperl-electric-lbrace|cperl-electric-paren|cperl-electric-pod\n\t|cperl-electric-rparen|cperl-electric-semi|cperl-electric-terminator|cperl-emulate-lazy-lock|cperl-enable-font-lock|cperl-ensure-newlines\n\t|cperl-etags|cperl-facemenu-add-face-function|cperl-fill-paragraph|cperl-find-bad-style|cperl-find-pods-heres-region|cperl-find-pods-heres\n\t|cperl-find-sub-attrs|cperl-find-tags|cperl-fix-line-spacing|cperl-font-lock-fontify-region-function|cperl-font-lock-unfontify-region-function\n\t|cperl-fontify-syntaxically|cperl-fontify-update-bad|cperl-fontify-update|cperl-forward-group-in-re|cperl-forward-re|cperl-forward-to-end-of-expr\n\t|cperl-get-help-defer|cperl-get-help|cperl-get-here-doc-region|cperl-get-state|cperl-here-doc-spell|cperl-highlight-charclass|cperl-imenu--create-perl-index\n\t|cperl-imenu-addback|cperl-imenu-info-imenu-name|cperl-imenu-info-imenu-search|cperl-imenu-name-and-position|cperl-imenu-on-info\n\t|cperl-indent-command|cperl-indent-exp|cperl-indent-for-comment|cperl-indent-line|cperl-indent-region|cperl-info-buffer|cperl-info-on-command\n\t|cperl-info-on-current-command|cperl-init-faces-weak|cperl-init-faces|cperl-inside-parens-p|cperl-invert-if-unless-modifiers|cperl-invert-if-unless\n\t|cperl-lazy-hook|cperl-lazy-install|cperl-lazy-unstall|cperl-linefeed|cperl-lineup|cperl-list-fold|cperl-load-font-lock-keywords-1\n\t|cperl-load-font-lock-keywords-2|cperl-load-font-lock-keywords|cperl-look-at-leading-count|cperl-make-indent|cperl-make-regexp-x\n\t|cperl-map-pods-heres|cperl-mark-active|cperl-menu-to-keymap|cperl-menu|cperl-mode|cperl-modify-syntax-type|cperl-msb-fix|cperl-narrow-to-here-doc\n\t|cperl-next-bad-style|cperl-next-interpolated-REx-0|cperl-next-interpolated-REx-1|cperl-next-interpolated-REx|cperl-outline-level\n\t|cperl-perldoc-at-point|cperl-perldoc|cperl-pod-spell|cperl-pod-to-manpage|cperl-pod2man-build-command|cperl-postpone-fontification\n\t|cperl-protect-defun-start|cperl-ps-print-init|cperl-ps-print|cperl-put-do-not-fontify|cperl-putback-char|cperl-regext-to-level-start\n\t|cperl-select-this-pod-or-here-doc|cperl-set-style-back|cperl-set-style|cperl-setup-tmp-buf|cperl-sniff-for-indent|cperl-switch-to-doc-buffer\n\t|cperl-tags-hier-fill|cperl-tags-hier-init|cperl-tags-treeify|cperl-time-fontification|cperl-to-comment-or-eol|cperl-toggle-abbrev\n\t|cperl-toggle-auto-newline|cperl-toggle-autohelp|cperl-toggle-construct-fix|cperl-toggle-electric|cperl-toggle-set-debug-unwind\n\t|cperl-uncomment-region|cperl-unwind-to-safe|cperl-update-syntaxification|cperl-use-region-p|cperl-val|cperl-windowed-init|cperl-word-at-point-hard\n\t|cperl-word-at-point|cperl-write-tags|cperl-xsub-scan|cpp-choose-branch|cpp-choose-default-face|cpp-choose-face|cpp-choose-symbol\n\t|cpp-create-bg-face|cpp-edit-apply|cpp-edit-background|cpp-edit-false|cpp-edit-home|cpp-edit-known|cpp-edit-list-entry-get-or-create\n\t|cpp-edit-load|cpp-edit-mode|cpp-edit-reset|cpp-edit-save|cpp-edit-toggle-known|cpp-edit-toggle-unknown|cpp-edit-true|cpp-edit-unknown\n\t|cpp-edit-write|cpp-face-name|cpp-grow-overlay|cpp-highlight-buffer|cpp-make-button|cpp-make-known-overlay|cpp-make-overlay-hidden\n\t|cpp-make-overlay-read-only|cpp-make-overlay-sticky|cpp-make-unknown-overlay|cpp-parse-close|cpp-parse-edit|cpp-parse-error|cpp-parse-open\n\t|cpp-parse-reset|cpp-progress-message|cpp-push-button|cpp-signal-read-only|create-default-fontset|create-fontset-from-ascii-font\n\t|create-fontset-from-x-resource|create-glyph|crm--choose-completion-string|crm--collection-fn|crm--completion-command|crm--current-element\n\t|crm-complete-and-exit|crm-complete-word|crm-complete|crm-completion-help|crm-minibuffer-complete-and-exit|crm-minibuffer-complete\n\t|crm-minibuffer-completion-help|css--font-lock-keywords|css-current-defun-name|css-extract-keyword-list|css-extract-parse-val-grammar\n\t|css-extract-props-and-vals|css-fill-paragraph|css-mode|css-smie--backward-token|css-smie--forward-token|css-smie-rules|ctext-non-standard-encodings-table\n\t|ctext-post-read-conversion|ctext-pre-write-conversion|ctl-x-4-prefix|ctl-x-5-prefix|ctl-x-ctl-p-prefix|cua--M\\/H-key|cua--deactivate\n\t|cua--fallback|cua--filter-buffer-noprops|cua--init-keymaps|cua--keep-active|cua--post-command-handler-1|cua--post-command-handler\n\t|cua--pre-command-handler-1|cua--pre-command-handler|cua--prefix-arg|cua--prefix-copy-handler|cua--prefix-cut-handler|cua--prefix-override-handler\n\t|cua--prefix-override-replay|cua--prefix-override-timeout|cua--prefix-repeat-handler|cua--select-keymaps|cua--self-insert-char-p\n\t|cua--shift-control-c-prefix|cua--shift-control-prefix|cua--shift-control-x-prefix|cua--update-indications|cua-cancel|cua-copy-region\n\t|cua-cut-region|cua-debug|cua-delete-region|cua-exchange-point-and-mark|cua-help-for-region|cua-mode|cua-paste-pop|cua-paste|cua-pop-to-last-change\n\t|cua-rectangle-mark-mode|cua-scroll-down|cua-scroll-up|cua-selection-mode|cua-set-mark|cua-set-rectangle-mark|cua-toggle-global-mark\n\t|current-line|custom--frame-color-default|custom--initialize-widget-variables|custom--sort-vars-1|custom--sort-vars|custom-add-dependencies\n\t|custom-add-link|custom-add-load|custom-add-option|custom-add-package-version|custom-add-parent-links|custom-add-see-also|custom-add-to-group\n\t|custom-add-version|custom-autoload|custom-available-themes|custom-browse-face-tag-action|custom-browse-group-tag-action|custom-browse-insert-prefix\n\t|custom-browse-variable-tag-action|custom-browse-visibility-action|custom-buffer-create-internal|custom-buffer-create-other-window\n\t|custom-buffer-create|custom-check-theme|custom-command-apply|custom-comment-create|custom-comment-hide|custom-comment-invisible-p\n\t|custom-comment-show|custom-convert-widget|custom-current-group|custom-declare-face|custom-declare-group|custom-declare-theme|custom-declare-variable\n\t|custom-face-action|custom-face-attributes-get|custom-face-edit-activate|custom-face-edit-all|custom-face-edit-attribute-tag|custom-face-edit-convert-widget\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t custom-face-edit-deactivate|custom-face-edit-delete|custom-face-edit-fix-value|custom-face-edit-lisp|custom-face-edit-selected\n\t|custom-face-edit-value-create|custom-face-edit-value-visibility-action|custom-face-get-current-spec|custom-face-mark-to-reset-standard\n\t|custom-face-mark-to-save|custom-face-menu-create|custom-face-reset-saved|custom-face-reset-standard|custom-face-save-command\n\t|custom-face-save|custom-face-set|custom-face-standard-value|custom-face-state-set-and-redraw|custom-face-state-set|custom-face-state\n\t|custom-face-value-create|custom-face-widget-to-spec|custom-facep|custom-file|custom-filter-face-spec|custom-fix-face-spec|custom-get-fresh-buffer\n\t|custom-group-action|custom-group-link-action|custom-group-mark-to-reset-standard|custom-group-mark-to-save|custom-group-members\n\t|custom-group-menu-create|custom-group-of-mode|custom-group-reset-current|custom-group-reset-saved|custom-group-reset-standard\n\t|custom-group-sample-face-get|custom-group-save|custom-group-set|custom-group-state-set-and-redraw|custom-group-state-update|custom-group-value-create\n\t|custom-group-visibility-create|custom-guess-type|custom-handle-all-keywords|custom-handle-keyword|custom-hook-convert-widget\n\t|custom-initialize-changed|custom-initialize-default|custom-initialize-reset|custom-initialize-set|custom-load-symbol|custom-load-widget\n\t|custom-magic-reset|custom-magic-value-create|custom-make-theme-feature|custom-menu-create|custom-menu-filter|custom-mode|custom-note-var-changed\n\t|custom-notify|custom-post-filter-face-spec|custom-pre-filter-face-spec|custom-prefix-add|custom-prompt-customize-unsaved-options\n\t|custom-prompt-variable|custom-push-theme|custom-put-if-not|custom-quote|custom-redraw-magic|custom-redraw|custom-reset-faces|custom-reset-standard-save-and-update\n\t|custom-reset-variables|custom-reset|custom-save-all|custom-save-delete|custom-save-faces|custom-save-variables|custom-set-default\n\t|custom-set-minor-mode|custom-show|custom-sort-items|custom-split-regexp-maybe|custom-state-buffer-message|custom-tag-action|custom-tag-mouse-down-action\n\t|custom-theme--load-path|custom-theme-enabled-p|custom-theme-load-confirm|custom-theme-name-valid-p|custom-theme-recalc-face|custom-theme-recalc-variable\n\t|custom-theme-reset-faces|custom-theme-reset-variables|custom-theme-visit-theme|custom-toggle-hide-face|custom-toggle-hide-variable\n\t|custom-toggle-hide|custom-toggle-parent|custom-unlispify-menu-entry|custom-unlispify-tag-name|custom-unloaded-symbol-p|custom-unloaded-widget-p\n\t|custom-unsaved-options|custom-variable-action|custom-variable-backup-value|custom-variable-documentation|custom-variable-edit-lisp\n\t|custom-variable-edit|custom-variable-mark-to-reset-standard|custom-variable-mark-to-save|custom-variable-menu-create|custom-variable-prompt\n\t|custom-variable-reset-backup|custom-variable-reset-saved|custom-variable-reset-standard|custom-variable-save|custom-variable-set\n\t|custom-variable-standard-value|custom-variable-state-set-and-redraw|custom-variable-state-set|custom-variable-state|custom-variable-theme-value\n\t|custom-variable-type|custom-variable-value-create|customize-apropos-faces|customize-apropos-groups|customize-apropos-options\n\t|customize-apropos|customize-browse|customize-changed-options|customize-changed|customize-create-theme|customize-customized|customize-face-other-window\n\t|customize-face|customize-group-other-window|customize-group|customize-mark-as-set|customize-mark-to-save|customize-menu-create\n\t|customize-mode|customize-object|customize-option-other-window|customize-option|customize-package-emacs-version|customize-project\n\t|customize-push-and-save|customize-read-group|customize-rogue|customize-save-customized|customize-save-variable|customize-saved\n\t|customize-set-value|customize-set-variable|customize-target|customize-themes|customize-unsaved|customize-variable-other-window\n\t|customize-variable|customize-version-lessp|customize|cvs-add-branch-prefix|cvs-add-face|cvs-add-secondary-branch-prefix|cvs-addto-collection\n\t|cvs-append-to-ignore|cvs-append|cvs-applicable-p|cvs-buffer-check|cvs-buffer-p|cvs-bury-buffer|cvs-car|cvs-cdr|cvs-change-cvsroot\n\t|cvs-check-fileinfo|cvs-checkout|cvs-cleanup-collection|cvs-cleanup-removed|cvs-cmd-do|cvs-commit-filelist|cvs-commit-minor-wrap\n\t|cvs-create-fileinfo|cvs-defaults|cvs-diff-backup-extractor|cvs-dir-member-p|cvs-dired-noselect|cvs-do-commit|cvs-do-edit-log|cvs-do-match\n\t|cvs-do-removal|cvs-ediff-diff|cvs-ediff-exit-hook|cvs-ediff-merge|cvs-ediff-startup-hook|cvs-edit-log-filelist|cvs-edit-log-minor-wrap\n\t|cvs-edit-log-text-at-point|cvs-emerge-diff|cvs-emerge-merge|cvs-enabledp|cvs-every|cvs-examine|cvs-execute-single-file-list|cvs-execute-single-file\n\t|cvs-expand-dir-name|cvs-file-to-string|cvs-fileinfo->backup-file|cvs-fileinfo->base-rev--cmacro|cvs-fileinfo->base-rev|cvs-fileinfo->dir--cmacro\n\t|cvs-fileinfo->dir|cvs-fileinfo->file--cmacro|cvs-fileinfo->file|cvs-fileinfo->full-log--cmacro|cvs-fileinfo->full-log|cvs-fileinfo->full-name\n\t|cvs-fileinfo->full-path|cvs-fileinfo->head-rev--cmacro|cvs-fileinfo->head-rev|cvs-fileinfo->marked--cmacro|cvs-fileinfo->marked\n\t|cvs-fileinfo->merge--cmacro|cvs-fileinfo->merge|cvs-fileinfo->pp-name|cvs-fileinfo->subtype--cmacro|cvs-fileinfo->subtype|cvs-fileinfo->type--cmacro\n\t|cvs-fileinfo->type|cvs-fileinfo-from-entries|cvs-fileinfo-p--cmacro|cvs-fileinfo-p|cvs-fileinfo-pp|cvs-fileinfo-update|cvs-fileinfo<\n\t|cvs-find-modif|cvs-first|cvs-flags-defaults--cmacro|cvs-flags-defaults|cvs-flags-define|cvs-flags-desc--cmacro|cvs-flags-desc|cvs-flags-hist-sym--cmacro\n\t|cvs-flags-hist-sym|cvs-flags-p--cmacro|cvs-flags-p|cvs-flags-persist--cmacro|cvs-flags-persist|cvs-flags-qtypedesc--cmacro|cvs-flags-qtypedesc\n\t|cvs-flags-query|cvs-flags-set|cvs-get-buffer-create|cvs-get-cvsroot|cvs-get-marked|cvs-get-module|cvs-global-menu|cvs-header-msg\n\t|cvs-help|cvs-ignore-marks-p|cvs-insert-file|cvs-insert-strings|cvs-insert-visited-file|cvs-is-within-p|cvs-make-cvs-buffer|cvs-map\n\t|cvs-mark-buffer-changed|cvs-mark-fis-dead|cvs-match|cvs-menu|cvs-minor-mode|cvs-mode!|cvs-mode-acknowledge|cvs-mode-add-change-log-entry-other-window\n\t|cvs-mode-add|cvs-mode-byte-compile-files|cvs-mode-checkout|cvs-mode-commit-setup|cvs-mode-commit|cvs-mode-delete-lock|cvs-mode-diff-1\n\t|cvs-mode-diff-backup|cvs-mode-diff-head|cvs-mode-diff-map|cvs-mode-diff-repository|cvs-mode-diff-vendor|cvs-mode-diff-yesterday\n\t|cvs-mode-diff|cvs-mode-display-file|cvs-mode-do|cvs-mode-edit-log|cvs-mode-examine|cvs-mode-files|cvs-mode-find-file-other-window\n\t|cvs-mode-find-file|cvs-mode-force-command|cvs-mode-idiff-other|cvs-mode-idiff|cvs-mode-ignore|cvs-mode-imerge|cvs-mode-insert|cvs-mode-kill-buffers\n\t|cvs-mode-kill-process|cvs-mode-log|cvs-mode-map|cvs-mode-mark-all-files|cvs-mode-mark-get-modif|cvs-mode-mark-matching-files|cvs-mode-mark-on-state\n\t|cvs-mode-mark|cvs-mode-marked|cvs-mode-next-line|cvs-mode-previous-line|cvs-mode-quit|cvs-mode-remove-handled|cvs-mode-remove|cvs-mode-revert-buffer\n\t|cvs-mode-revert-to-rev|cvs-mode-run|cvs-mode-set-flags|cvs-mode-status|cvs-mode-tag|cvs-mode-toggle-mark|cvs-mode-toggle-marks\n\t|cvs-mode-tree|cvs-mode-undo|cvs-mode-unmark-all-files|cvs-mode-unmark-up|cvs-mode-unmark|cvs-mode-untag|cvs-mode-update|cvs-mode-view-file-other-window\n\t|cvs-mode-view-file|cvs-mode|cvs-mouse-toggle-mark|cvs-move-to-goal-column|cvs-or|cvs-parse-buffer|cvs-parse-commit|cvs-parse-merge\n\t|cvs-parse-msg|cvs-parse-process|cvs-parse-run-table|cvs-parse-status|cvs-parse-table|cvs-parsed-fileinfo|cvs-partition|cvs-pop-to-buffer-same-frame\n\t|cvs-prefix-define|cvs-prefix-get|cvs-prefix-make-local|cvs-prefix-set|cvs-prefix-sym|cvs-qtypedesc-complete--cmacro|cvs-qtypedesc-complete\n\t|cvs-qtypedesc-create--cmacro|cvs-qtypedesc-create|cvs-qtypedesc-hist-sym--cmacro|cvs-qtypedesc-hist-sym|cvs-qtypedesc-obj2str--cmacro\n\t|cvs-qtypedesc-obj2str|cvs-qtypedesc-p--cmacro|cvs-qtypedesc-p|cvs-qtypedesc-require--cmacro|cvs-qtypedesc-require|cvs-qtypedesc-str2obj--cmacro\n\t|cvs-qtypedesc-str2obj|cvs-query-directory|cvs-query-read|cvs-quickdir|cvs-reread-cvsrc|cvs-retrieve-revision|cvs-revert-if-needed\n\t|cvs-run-process|cvs-sentinel|cvs-set-branch-prefix|cvs-set-secondary-branch-prefix|cvs-status-current-file|cvs-status-current-tag\n\t|cvs-status-cvstrees|cvs-status-get-tags|cvs-status-minor-wrap|cvs-status-mode|cvs-status-next|cvs-status-prev|cvs-status-trees\n\t|cvs-status-vl-to-str|cvs-status|cvs-string-prefix-p|cvs-tag->name--cmacro|cvs-tag->name|cvs-tag->string|cvs-tag->type--cmacro|cvs-tag->type\n\t|cvs-tag->vlist--cmacro|cvs-tag->vlist|cvs-tag-compare-1|cvs-tag-compare|cvs-tag-lessp|cvs-tag-make--cmacro|cvs-tag-make-tag|cvs-tag-make\n\t|cvs-tag-merge|cvs-tag-p--cmacro|cvs-tag-p|cvs-tags->tree|cvs-tags-list|cvs-temp-buffer|cvs-tree-merge|cvs-tree-print|cvs-tree-tags-insert\n\t|cvs-union|cvs-update-filter|cvs-update-header|cvs-update|cvs-vc-command-advice|cwarn-font-lock-keywords|cwarn-font-lock-match-assignment-in-expression\n\t|cwarn-font-lock-match-dangerous-semicolon|cwarn-font-lock-match-reference|cwarn-font-lock-match|cwarn-inside-macro|cwarn-is-enabled\n\t|cwarn-mode-set-explicitly|cwarn-mode|cycle-spacing|cyrillic-encode-alternativnyj-char|cyrillic-encode-koi8-r-char|dabbrev--abbrev-at-point\n\t|dabbrev--find-all-expansions|dabbrev--find-expansion|dabbrev--goto-start-of-abbrev|dabbrev--ignore-buffer-p|dabbrev--ignore-case-p\n\t|dabbrev--make-friend-buffer-list|dabbrev--minibuffer-origin|dabbrev--reset-global-variables|dabbrev--safe-replace-match|dabbrev--same-major-mode-p\n\t|dabbrev--search|dabbrev--select-buffers|dabbrev--substitute-expansion|dabbrev--try-find|dabbrev-completion|dabbrev-expand|dabbrev-filter-elements\n\t|daemon-initialized|daemonp|data-debug-new-buffer|date-to-day|days-between|days-to-time|dbus--init-bus|dbus-byte-array-to-string\n\t|dbus-call-method-handler|dbus-check-event|dbus-escape-as-identifier|dbus-event-bus-name|dbus-event-interface-name|dbus-event-member-name\n\t|dbus-event-message-type|dbus-event-path-name|dbus-event-serial-number|dbus-event-service-name|dbus-get-all-managed-objects|dbus-get-all-properties\n\t|dbus-get-name-owner|dbus-get-property|dbus-get-unique-name|dbus-handle-bus-disconnect|dbus-handle-event|dbus-ignore-errors|dbus-init-bus\n\t|dbus-introspect-get-all-nodes|dbus-introspect-get-annotation-names|dbus-introspect-get-annotation|dbus-introspect-get-argument-names\n\t|dbus-introspect-get-argument|dbus-introspect-get-attribute|dbus-introspect-get-interface-names|dbus-introspect-get-interface\n\t|dbus-introspect-get-method-names|dbus-introspect-get-method|dbus-introspect-get-node-names|dbus-introspect-get-property-names\n\t|dbus-introspect-get-property|dbus-introspect-get-signal-names|dbus-introspect-get-signal|dbus-introspect-get-signature|dbus-introspect-xml\n\t|dbus-introspect|dbus-list-activatable-names|dbus-list-hash-table|dbus-list-known-names|dbus-list-names|dbus-list-queued-owners\n\t|dbus-managed-objects-handler|dbus-message-internal|dbus-method-error-internal|dbus-method-return-internal|dbus-notice-synchronous-call-errors\n\t|dbus-peer-handler|dbus-ping|dbus-property-handler|dbus-register-method|dbus-register-property|dbus-register-service|dbus-register-signal\n\t|dbus-set-property|dbus-setenv|dbus-string-to-byte-array|dbus-unescape-from-identifier|dbus-unregister-object|dbus-unregister-service\n\t|dbx|dcl-back-to-indentation-1|dcl-back-to-indentation|dcl-backward-command|dcl-beginning-of-command-p|dcl-beginning-of-command\n\t|dcl-beginning-of-statement|dcl-calc-command-indent-hang|dcl-calc-command-indent-multiple|dcl-calc-command-indent|dcl-calc-cont-indent-relative\n\t|dcl-calc-continuation-indent|dcl-command-p|dcl-delete-chars|dcl-delete-indentation|dcl-electric-character|dcl-end-of-command-p\n\t|dcl-end-of-command|dcl-end-of-statement|dcl-forward-command|dcl-get-line-type|dcl-guess-option-value|dcl-guess-option|dcl-imenu-create-index-function\n\t|dcl-indent-command-line|dcl-indent-command|dcl-indent-continuation-line|dcl-indent-line|dcl-indent-to|dcl-indentation-point|dcl-mode\n\t|dcl-option-value-basic|dcl-option-value-comment-line|dcl-option-value-margin-offset|dcl-option-value-offset|dcl-save-all-options\n\t|dcl-save-local-variable|dcl-save-mode|dcl-save-nondefault-options|dcl-save-option|dcl-set-option|dcl-show-line-type|dcl-split-line\n\t|dcl-tab|dcl-was-looking-at|deactivate-input-method|deactivate-mode-local-bindings|debug--function-list|debug--implement-debug-on-entry\n\t|debug-help-follow|debugger--backtrace-base|debugger--hide-locals|debugger--insert-locals|debugger--locals-visible-p|debugger--show-locals\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t debugger-continue|debugger-env-macro|debugger-eval-expression|debugger-frame-clear|debugger-frame-number|debugger-frame|debugger-jump\n\t|debugger-list-functions|debugger-make-xrefs|debugger-mode|debugger-record-expression|debugger-reenable|debugger-return-value|debugger-setup-buffer\n\t|debugger-step-through|debugger-toggle-locals|decf|decipher--analyze|decipher--digram-counts|decipher--digram-total|decipher-add-undo\n\t|decipher-adjacency-list|decipher-alphabet-keypress|decipher-analyze-buffer|decipher-analyze|decipher-complete-alphabet|decipher-copy-cons\n\t|decipher-digram-list|decipher-display-range|decipher-display-regexp|decipher-display-stats-buffer|decipher-frequency-count|decipher-get-undo\n\t|decipher-insert-frequency-counts|decipher-insert|decipher-keypress|decipher-last-command-char|decipher-loop-no-breaks|decipher-loop-with-breaks\n\t|decipher-make-checkpoint|decipher-mode|decipher-read-alphabet|decipher-restore-checkpoint|decipher-resync|decipher-set-map|decipher-show-alphabet\n\t|decipher-stats-buffer|decipher-stats-mode|decipher-undo|decipher|declaim|declare-ccl-program|declare-equiv-charset|decode-big5-char\n\t|decode-composition-components|decode-composition-rule|decode-hex-string|decode-hz-buffer|decode-hz-region|decode-sjis-char|decompose-region\n\t|decompose-string|decrease-left-margin|decrease-right-margin|def-gdb-auto-update-handler|def-gdb-auto-update-trigger|def-gdb-memory-format\n\t|def-gdb-memory-show-page|def-gdb-memory-unit|def-gdb-preempt-display-buffer|def-gdb-set-positive-number|def-gdb-thread-buffer-command\n\t|def-gdb-thread-buffer-gud-command|def-gdb-thread-buffer-simple-command|def-gdb-trigger-and-handler|default-command-history-filter\n\t|default-font-height|default-indent-new-line|default-line-height|default-toplevel-value|defcalcmodevar|defconst-mode-local|defcustom-c-stylevar\n\t|defcustom-mh|defezimage|defface-mh|defgeneric|defgroup-mh|defimage-speedbar|define-abbrevs|define-advice|define-auto-insert|define-ccl-program\n\t|define-char-code-property|define-charset-alias|define-charset-internal|define-charset|define-child-mode|define-coding-system-alias\n\t|define-coding-system-internal|define-coding-system|define-compilation-mode|define-compiler-macro|define-erc-module|define-erc-response-handler\n\t|define-global-abbrev|define-global-minor-mode|define-hmac-function|define-ibuffer-column|define-ibuffer-filter|define-ibuffer-op\n\t|define-ibuffer-sorter|define-inline|define-lex-analyzer|define-lex-block-analyzer|define-lex-block-type-analyzer|define-lex-keyword-type-analyzer\n\t|define-lex-regex-analyzer|define-lex-regex-type-analyzer|define-lex-sexp-type-analyzer|define-lex-simple-regex-analyzer|define-lex-string-type-analyzer\n\t|define-lex|define-mail-abbrev|define-mail-alias|define-mail-user-agent|define-mode-abbrev|define-mode-local-override|define-mode-overload-implementation\n\t|define-overload|define-overloadable-function|define-setf-expander|define-skeleton|define-translation-hash-table|define-translation-table\n\t|define-widget-keywords|defmacro-mh|defmath|defmethod|defun-cvs-mode|defun-gmm|defun-mh|defun-rcirc-command|defvar-mode-local|degrees-to-radians\n\t|dehexlify-buffer|delay-warning|delete\\*|delete-active-region|delete-all-overlays|delete-completion-window|delete-completion|delete-consecutive-dups\n\t|delete-dir-local-variable|delete-directory-internal|delete-duplicate-lines|delete-duplicates|delete-extract-rectangle-line|delete-extract-rectangle\n\t|delete-file-local-variable-prop-line|delete-file-local-variable|delete-forward-char|delete-frame-enabled-p|delete-if-not|delete-if\n\t|delete-instance|delete-matching-lines|delete-non-matching-lines|delete-other-frames|delete-other-windows-internal|delete-other-windows-vertically\n\t|delete-pair|delete-rectangle-line|delete-rectangle|delete-selection-helper|delete-selection-mode|delete-selection-pre-hook|delete-selection-repeat-replace-region\n\t|delete-side-window|delete-whitespace-rectangle-line|delete-whitespace-rectangle|delete-window-internal|delimit-columns-customize\n\t|delimit-columns-format|delimit-columns-rectangle-line|delimit-columns-rectangle-max|delimit-columns-rectangle|delimit-columns-region\n\t|delimit-columns-str|delphi-mode|delsel-unload-function|denato-region|derived-mode-abbrev-table-name|derived-mode-class|derived-mode-hook-name\n\t|derived-mode-init-mode-variables|derived-mode-make-docstring|derived-mode-map-name|derived-mode-merge-abbrev-tables|derived-mode-merge-keymaps\n\t|derived-mode-merge-syntax-tables|derived-mode-run-hooks|derived-mode-set-abbrev-table|derived-mode-set-keymap|derived-mode-set-syntax-table\n\t|derived-mode-setup-function-name|derived-mode-syntax-table-name|describe-bindings-internal|describe-buffer-bindings|describe-char-after\n\t|describe-char-categories|describe-char-display|describe-char-padded-string|describe-char-unicode-data|describe-char|describe-character-set\n\t|describe-chinese-environment-map|describe-coding-system|describe-copying|describe-current-coding-system-briefly|describe-current-coding-system\n\t|describe-current-input-method|describe-cyrillic-environment-map|describe-distribution|describe-european-environment-map|describe-face\n\t|describe-font|describe-fontset|describe-function-1|describe-function|describe-gnu-project|describe-indian-environment-map|describe-input-method\n\t|describe-key-briefly|describe-key|describe-language-environment|describe-minor-mode-completion-table-for-indicator|describe-minor-mode-completion-table-for-symbol\n\t|describe-minor-mode-from-indicator|describe-minor-mode-from-symbol|describe-minor-mode|describe-mode-local-bindings-in-mode\n\t|describe-mode-local-bindings|describe-no-warranty|describe-package-1|describe-package|describe-project|describe-property-list\n\t|describe-register-1|describe-specified-language-support|describe-text-category|describe-text-properties-1|describe-text-properties\n\t|describe-text-sexp|describe-text-widget|describe-theme|describe-variable-custom-version-info|describe-variable|describe-vector\n\t|desktop--check-dont-save|desktop--v2s|desktop-append-buffer-args|desktop-auto-save-cancel-timer|desktop-auto-save-disable|desktop-auto-save-enable\n\t|desktop-auto-save-set-timer|desktop-auto-save|desktop-buffer-info|desktop-buffer|desktop-change-dir|desktop-claim-lock|desktop-clear\n\t|desktop-create-buffer|desktop-file-name|desktop-full-file-name|desktop-full-lock-name|desktop-idle-create-buffers|desktop-kill\n\t|desktop-lazy-abort|desktop-lazy-complete|desktop-lazy-create-buffer|desktop-list\\*|desktop-load-default|desktop-load-file|desktop-outvar\n\t|desktop-owner|desktop-read|desktop-release-lock|desktop-remove|desktop-restore-file-buffer|desktop-restore-frameset|desktop-restoring-frameset-p\n\t|desktop-revert|desktop-save-buffer-p|desktop-save-frameset|desktop-save-in-desktop-dir|desktop-save-mode-off|desktop-save-mode\n\t|desktop-save|desktop-truncate|desktop-value-to-string|destructor|destructuring-bind|detect-coding-with-language-environment|detect-coding-with-priority\n\t|dframe-attached-frame|dframe-click|dframe-close-frame|dframe-current-frame|dframe-detach|dframe-double-click|dframe-frame-mode\n\t|dframe-frame-parameter|dframe-get-focus|dframe-hack-buffer-menu|dframe-handle-delete-frame|dframe-handle-iconify-frame|dframe-handle-make-frame-visible\n\t|dframe-help-echo|dframe-live-p|dframe-maybee-jump-to-attached-frame|dframe-message|dframe-mouse-event-p|dframe-mouse-hscroll|dframe-mouse-set-point\n\t|dframe-needed-height|dframe-popup-kludge|dframe-power-click|dframe-quick-mouse|dframe-reposition-frame-emacs|dframe-reposition-frame-xemacs\n\t|dframe-reposition-frame|dframe-select-attached-frame|dframe-set-timer-internal|dframe-set-timer|dframe-switch-buffer-attached-frame\n\t|dframe-temp-buffer-show-function|dframe-timer-fn|dframe-track-mouse-xemacs|dframe-track-mouse|dframe-update-keymap|dframe-with-attached-buffer\n\t|dframe-y-or-n-p|diary-add-to-list|diary-anniversary|diary-astro-day-number|diary-attrtype-convert|diary-bahai-date|diary-bahai-insert-entry\n\t|diary-bahai-insert-monthly-entry|diary-bahai-insert-yearly-entry|diary-bahai-list-entries|diary-bahai-mark-entries|diary-block\n\t|diary-check-diary-file|diary-chinese-anniversary|diary-chinese-date|diary-chinese-insert-anniversary-entry|diary-chinese-insert-entry\n\t|diary-chinese-insert-monthly-entry|diary-chinese-insert-yearly-entry|diary-chinese-list-entries|diary-chinese-mark-entries|diary-coptic-date\n\t|diary-cyclic|diary-date-display-form|diary-date|diary-day-of-year|diary-display-no-entries|diary-entry-compare|diary-entry-time\n\t|diary-ethiopic-date|diary-fancy-date-matcher|diary-fancy-date-pattern|diary-fancy-display-mode|diary-fancy-display|diary-fancy-font-lock-fontify-region-function\n\t|diary-float|diary-font-lock-date-forms|diary-font-lock-keywords-1|diary-font-lock-keywords|diary-font-lock-sexps|diary-french-date\n\t|diary-from-outlook-gnus|diary-from-outlook-internal|diary-from-outlook-rmail|diary-from-outlook|diary-goto-entry|diary-hebrew-birthday\n\t|diary-hebrew-date|diary-hebrew-insert-entry|diary-hebrew-insert-monthly-entry|diary-hebrew-insert-yearly-entry|diary-hebrew-list-entries\n\t|diary-hebrew-mark-entries|diary-hebrew-omer|diary-hebrew-parasha|diary-hebrew-rosh-hodesh|diary-hebrew-sabbath-candles|diary-hebrew-yahrzeit\n\t|diary-include-files|diary-include-other-diary-files|diary-insert-anniversary-entry|diary-insert-block-entry|diary-insert-cyclic-entry\n\t|diary-insert-entry-1|diary-insert-entry|diary-insert-monthly-entry|diary-insert-weekly-entry|diary-insert-yearly-entry|diary-islamic-date\n\t|diary-islamic-insert-entry|diary-islamic-insert-monthly-entry|diary-islamic-insert-yearly-entry|diary-islamic-list-entries|diary-islamic-mark-entries\n\t|diary-iso-date|diary-julian-date|diary-list-entries-1|diary-list-entries-2|diary-list-entries|diary-list-sexp-entries|diary-live-p\n\t|diary-lunar-phases|diary-mail-entries|diary-make-date|diary-make-entry|diary-mark-entries-1|diary-mark-entries|diary-mark-included-diary-files\n\t|diary-mark-sexp-entries|diary-mayan-date|diary-mode|diary-name-pattern|diary-ordinal-suffix|diary-outlook-format-1|diary-persian-date\n\t|diary-print-entries|diary-pull-attrs|diary-redraw-calendar|diary-remind|diary-set-header|diary-set-maybe-redraw|diary-sexp-entry\n\t|diary-show-all-entries|diary-simple-display|diary-sort-entries|diary-sunrise-sunset|diary-unhide-everything|diary-view-entries\n\t|diary-view-other-diary-entries|diary|diff-add-change-log-entries-other-window|diff-after-change-function|diff-apply-hunk|diff-auto-refine-mode\n\t|diff-backup|diff-beginning-of-file-and-junk|diff-beginning-of-file|diff-beginning-of-hunk|diff-bounds-of-file|diff-bounds-of-hunk\n\t|diff-buffer-with-file|diff-context->unified|diff-count-matches|diff-current-defun|diff-delete-empty-files|diff-delete-if-empty\n\t|diff-delete-trailing-whitespace|diff-ediff-patch|diff-end-of-file|diff-end-of-hunk|diff-file-kill|diff-file-local-copy|diff-file-next\n\t|diff-file-prev|diff-filename-drop-dir|diff-find-approx-text|diff-find-file-name|diff-find-source-location|diff-find-text|diff-fixup-modifs\n\t|diff-goto-source|diff-hunk-file-names|diff-hunk-kill|diff-hunk-next|diff-hunk-prev|diff-hunk-status-msg|diff-hunk-style|diff-hunk-text\n\t|diff-ignore-whitespace-hunk|diff-kill-applied-hunks|diff-kill-junk|diff-latest-backup-file|diff-make-unified|diff-merge-strings\n\t|diff-minor-mode|diff-mode-menu|diff-mode|diff-mouse-goto-source|diff-next-complex-hunk|diff-next-error|diff-no-select|diff-post-command-hook\n\t|diff-process-filter|diff-refine-hunk|diff-refine-preproc|diff-restrict-view|diff-reverse-direction|diff-sanity-check-context-hunk-half\n\t|diff-sanity-check-hunk|diff-sentinel|diff-setup-whitespace|diff-split-hunk|diff-splittable-p|diff-switches|diff-tell-file-name\n\t|diff-test-hunk|diff-undo|diff-unified->context|diff-unified-hunk-p|diff-write-contents-hooks|diff-xor|diff-yank-function|diff|dig-exit\n\t|dig-extract-rr|dig-invoke|dig-mode|dig-rr-get-pkix-cert|dig|digest-md5-challenge|digest-md5-digest-response|digest-md5-digest-uri\n\t|digest-md5-parse-digest-challenge|dir-locals-collect-mode-variables|dir-locals-collect-variables|dir-locals-find-file|dir-locals-get-class-variables\n\t|dir-locals-read-from-file|directory-files-recursively|directory-name-p|dired-add-file|dired-advertise|dired-advertised-find-file\n\t|dired-align-file|dired-alist-add-1|dired-at-point-prompter|dired-at-point|dired-backup-diff|dired-between-files|dired-buffer-stale-p\n\t|dired-buffers-for-dir|dired-build-subdir-alist|dired-change-marks|dired-check-switches|dired-clean-directory|dired-clean-up-after-deletion\n\t|dired-clear-alist|dired-compare-directories|dired-compress-file|dired-copy-file|dired-copy-filename-as-kill|dired-create-directory\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t dired-current-directory|dired-delete-entry|dired-delete-file|dired-desktop-buffer-misc-data|dired-diff|dired-directory-changed-p\n\t|dired-display-file|dired-dnd-do-ask-action|dired-dnd-handle-file|dired-dnd-handle-local-file|dired-dnd-popup-notice|dired-do-async-shell-command\n\t|dired-do-byte-compile|dired-do-chgrp|dired-do-chmod|dired-do-chown|dired-do-compress|dired-do-copy-regexp|dired-do-copy|dired-do-create-files-regexp\n\t|dired-do-delete|dired-do-flagged-delete|dired-do-hardlink-regexp|dired-do-hardlink|dired-do-isearch-regexp|dired-do-isearch|dired-do-kill-lines\n\t|dired-do-load|dired-do-print|dired-do-query-replace-regexp|dired-do-redisplay|dired-do-relsymlink|dired-do-rename-regexp|dired-do-rename\n\t|dired-do-search|dired-do-shell-command|dired-do-symlink-regexp|dired-do-symlink|dired-do-touch|dired-downcase|dired-file-marker\n\t|dired-file-name-at-point|dired-find-alternate-file|dired-find-buffer-nocreate|dired-find-file-other-window|dired-find-file|dired-flag-auto-save-files\n\t|dired-flag-backup-files|dired-flag-file-deletion|dired-flag-files-regexp|dired-flag-garbage-files|dired-format-columns-of-files\n\t|dired-fun-in-all-buffers|dired-get-file-for-visit|dired-get-filename|dired-get-marked-files|dired-get-subdir-max|dired-get-subdir-min\n\t|dired-get-subdir|dired-glob-regexp|dired-goto-file-1|dired-goto-file|dired-goto-next-file|dired-goto-next-nontrivial-file|dired-goto-subdir\n\t|dired-hide-all|dired-hide-details-mode|dired-hide-details-update-invisibility-spec|dired-hide-subdir|dired-in-this-tree|dired-initial-position\n\t|dired-insert-directory|dired-insert-old-subdirs|dired-insert-set-properties|dired-insert-subdir|dired-internal-do-deletions|dired-internal-noselect\n\t|dired-isearch-filenames-regexp|dired-isearch-filenames-setup|dired-isearch-filenames|dired-jump-other-window|dired-jump|dired-kill-subdir\n\t|dired-log-summary|dired-log|dired-make-absolute|dired-make-relative|dired-map-over-marks|dired-mark-directories|dired-mark-executables\n\t|dired-mark-files-containing-regexp|dired-mark-files-in-region|dired-mark-files-regexp|dired-mark-if|dired-mark-pop-up|dired-mark-prompt\n\t|dired-mark-remembered|dired-mark-subdir-files|dired-mark-symlinks|dired-mark|dired-marker-regexp|dired-maybe-insert-subdir|dired-mode\n\t|dired-mouse-find-file-other-window|dired-move-to-end-of-filename|dired-move-to-filename|dired-next-dirline|dired-next-line|dired-next-marked-file\n\t|dired-next-subdir|dired-normalize-subdir|dired-noselect|dired-other-frame|dired-other-window|dired-plural-s|dired-pop-to-buffer\n\t|dired-prev-dirline|dired-prev-marked-file|dired-prev-subdir|dired-previous-line|dired-query|dired-read-dir-and-switches|dired-read-regexp\n\t|dired-readin-insert|dired-readin|dired-relist-file|dired-remember-hidden|dired-remember-marks|dired-remove-file|dired-rename-file\n\t|dired-repeat-over-lines|dired-replace-in-string|dired-restore-desktop-buffer|dired-restore-positions|dired-revert|dired-run-shell-command\n\t|dired-safe-switches-p|dired-save-positions|dired-show-file-type|dired-sort-R-check|dired-sort-other|dired-sort-set-mode-line|dired-sort-set-modeline\n\t|dired-sort-toggle-or-edit|dired-sort-toggle|dired-string-replace-match|dired-subdir-index|dired-subdir-max|dired-summary|dired-switches-escape-p\n\t|dired-switches-recursive-p|dired-toggle-marks|dired-toggle-read-only|dired-tree-down|dired-tree-up|dired-unadvertise|dired-uncache\n\t|dired-undo|dired-unmark-all-files|dired-unmark-all-marks|dired-unmark-backward|dired-unmark|dired-up-directory|dired-upcase|dired-view-file\n\t|dired-why|dired|dirs|dirtrack-cygwin-directory-function|dirtrack-debug-message|dirtrack-debug-mode|dirtrack-debug-toggle|dirtrack-mode\n\t|dirtrack-toggle|dirtrack-windows-directory-function|dirtrack|disable-timeout|disassemble-1|disassemble-internal|disassemble-offset\n\t|display-about-screen|display-battery-mode|display-buffer--maybe-pop-up-frame-or-window|display-buffer--maybe-same-window|display-buffer--special-action\n\t|display-buffer-assq-regexp|display-buffer-in-atom-window|display-buffer-in-major-side-window|display-buffer-in-side-window|display-buffer-other-frame\n\t|display-buffer-record-window|display-call-tree|display-local-help|display-multi-font-p|display-multi-frame-p|display-splash-screen\n\t|display-startup-echo-area-message|display-startup-screen|display-table-print-array|display-time-mode|display-time-world|display-time\n\t|displaying-byte-compile-warnings|dissociated-press|dnd-get-local-file-name|dnd-get-local-file-uri|dnd-handle-one-url|dnd-insert-text\n\t|dnd-open-file|dnd-open-local-file|dnd-open-remote-url|dnd-unescape-uri|dns-get-txt-answer|dns-get|dns-inverse-get|dns-lookup-host\n\t|dns-make-network-process|dns-mode-menu|dns-mode-soa-increment-serial|dns-mode-soa-maybe-increment-serial|dns-mode|dns-query-cached\n\t|dns-query|dns-read-bytes|dns-read-int32|dns-read-name|dns-read-string-name|dns-read-txt|dns-read-type|dns-read|dns-servers-up-to-date-p\n\t|dns-set-servers|dns-write-bytes|dns-write-name|dns-write|dnsDomainIs|dnsResolve|do\\*|do-after-load-evaluation|do-all-symbols|do-auto-fill\n\t|do-symbols|do|doc\\$|doc\\/\\/|doc-file-to-info|doc-file-to-man|doc-view--current-cache-dir|doc-view-active-pages|doc-view-already-converted-p\n\t|doc-view-bookmark-jump|doc-view-bookmark-make-record|doc-view-buffer-message|doc-view-clear-cache|doc-view-clone-buffer-hook\n\t|doc-view-convert-current-doc|doc-view-current-cache-doc-pdf|doc-view-current-image|doc-view-current-info|doc-view-current-overlay\n\t|doc-view-current-page|doc-view-current-slice|doc-view-desktop-save-buffer|doc-view-dired-cache|doc-view-display|doc-view-djvu->tiff-converter-ddjvu\n\t|doc-view-doc->txt|doc-view-document->bitmap|doc-view-dvi->pdf|doc-view-enlarge|doc-view-fallback-mode|doc-view-first-page|doc-view-fit-height-to-window\n\t|doc-view-fit-page-to-window|doc-view-fit-width-to-window|doc-view-get-bounding-box|doc-view-goto-page|doc-view-guess-paper-size\n\t|doc-view-initiate-display|doc-view-insert-image|doc-view-intersection|doc-view-kill-proc-and-buffer|doc-view-kill-proc|doc-view-last-page-number\n\t|doc-view-last-page|doc-view-make-safe-dir|doc-view-menu|doc-view-minor-mode|doc-view-mode-maybe|doc-view-mode-p|doc-view-mode|doc-view-new-window-function\n\t|doc-view-next-line-or-next-page|doc-view-next-page|doc-view-odf->pdf-converter-soffice|doc-view-odf->pdf-converter-unoconv|doc-view-open-text\n\t|doc-view-pdf\\/ps->png|doc-view-pdf->png-converter-ghostscript|doc-view-pdf->png-converter-mupdf|doc-view-pdf->txt|doc-view-previous-line-or-previous-page\n\t|doc-view-previous-page|doc-view-ps->pdf|doc-view-ps->png-converter-ghostscript|doc-view-reconvert-doc|doc-view-reset-slice|doc-view-restore-desktop-buffer\n\t|doc-view-revert-buffer|doc-view-scale-adjust|doc-view-scale-bounding-box|doc-view-scale-reset|doc-view-scroll-down-or-previous-page\n\t|doc-view-scroll-up-or-next-page|doc-view-search-backward|doc-view-search-internal|doc-view-search-next-match|doc-view-search-no-of-matches\n\t|doc-view-search-previous-match|doc-view-search|doc-view-sentinel|doc-view-set-doc-type|doc-view-set-slice-from-bounding-box|doc-view-set-slice-using-mouse\n\t|doc-view-set-slice|doc-view-set-up-single-converter|doc-view-show-tooltip|doc-view-shrink|doc-view-sort|doc-view-start-process\n\t|doc-view-toggle-display|doctex-font-lock-\\^\\^A|doctex-font-lock-syntactic-face-function|doctex-mode|doctor-\\$|doctor-adjectivep\n\t|doctor-adverbp|doctor-alcohol|doctor-articlep|doctor-assm|doctor-build|doctor-chat|doctor-colorp|doctor-concat|doctor-conj|doctor-correct-spelling\n\t|doctor-death|doctor-def|doctor-define|doctor-defq|doctor-desire|doctor-desire1|doctor-doc|doctor-drug|doctor-eliza|doctor-family|doctor-fear\n\t|doctor-fix-2|doctor-fixup|doctor-forget|doctor-foul|doctor-getnoun|doctor-go|doctor-hate|doctor-hates|doctor-hates1|doctor-howdy|doctor-huh\n\t|doctor-love|doctor-loves|doctor-mach|doctor-make-string|doctor-math|doctor-meaning|doctor-mode|doctor-modifierp|doctor-mood|doctor-nmbrp\n\t|doctor-nounp|doctor-othermodifierp|doctor-plural|doctor-possess|doctor-possessivepronounp|doctor-prepp|doctor-pronounp|doctor-put-meaning\n\t|doctor-qloves|doctor-query|doctor-read-print|doctor-read-token|doctor-readin|doctor-remem|doctor-remember|doctor-replace|doctor-ret-or-read\n\t|doctor-rms|doctor-rthing|doctor-school|doctor-setprep|doctor-sexnoun|doctor-sexverb|doctor-short|doctor-shorten|doctor-sizep|doctor-sports\n\t|doctor-state|doctor-subjsearch|doctor-svo|doctor-symptoms|doctor-toke|doctor-txtype|doctor-type-symbol|doctor-type|doctor-verbp|doctor-vowelp\n\t|doctor-when|doctor-wherego|doctor-zippy|doctor|dom-add-child-before|dom-append-child|dom-attr|dom-attributes|dom-by-class|dom-by-id\n\t|dom-by-style|dom-by-tag|dom-child-by-tag|dom-children|dom-elements|dom-ensure-node|dom-node|dom-non-text-children|dom-parent|dom-pp\n\t|dom-set-attribute|dom-set-attributes|dom-tag|dom-text|dom-texts|dont-compile|double-column|double-mode|double-read-event|double-translate-key\n\t|down-ifdef|dsssl-mode|dunnet|dynamic-completion-mode|dynamic-completion-table|dynamic-setting-handle-config-changed-event|easy-menu-add-item\n\t|easy-menu-add|easy-menu-always-true-p|easy-menu-binding|easy-menu-change|easy-menu-convert-item-1|easy-menu-convert-item|easy-menu-create-menu\n\t|easy-menu-define-key|easy-menu-do-define|easy-menu-filter-return|easy-menu-get-map|easy-menu-intern|easy-menu-item-present-p|easy-menu-lookup-name\n\t|easy-menu-make-symbol|easy-menu-name-match|easy-menu-remove-item|easy-menu-remove|easy-menu-return-item|easy-mmode-define-global-mode\n\t|easy-mmode-define-keymap|easy-mmode-define-navigation|easy-mmode-define-syntax|easy-mmode-defmap|easy-mmode-defsyntax|easy-mmode-pretty-mode-name\n\t|easy-mmode-set-keymap-parents|ebnf-abn-initialize|ebnf-abn-parser|ebnf-adjust-empty|ebnf-adjust-width|ebnf-alternative-dimension\n\t|ebnf-alternative-width|ebnf-apply-style|ebnf-apply-style1|ebnf-begin-file|ebnf-begin-job|ebnf-begin-line|ebnf-bnf-initialize|ebnf-bnf-parser\n\t|ebnf-boolean|ebnf-buffer-substring|ebnf-check-style-values|ebnf-customize|ebnf-delete-style|ebnf-despool|ebnf-dimensions|ebnf-directory\n\t|ebnf-dtd-initialize|ebnf-dtd-parser|ebnf-dup-list|ebnf-ebx-initialize|ebnf-ebx-parser|ebnf-element-width|ebnf-eliminate-empty-rules\n\t|ebnf-empty-alternative|ebnf-end-of-string|ebnf-entry|ebnf-eop-horizontal|ebnf-eop-vertical|ebnf-eps-add-context|ebnf-eps-add-production\n\t|ebnf-eps-buffer|ebnf-eps-directory|ebnf-eps-file|ebnf-eps-filename|ebnf-eps-finish-and-write|ebnf-eps-footer-comment|ebnf-eps-footer\n\t|ebnf-eps-header-comment|ebnf-eps-header-footer-comment|ebnf-eps-header-footer-file|ebnf-eps-header-footer-p|ebnf-eps-header-footer-set\n\t|ebnf-eps-header-footer|ebnf-eps-header|ebnf-eps-output|ebnf-eps-production-list|ebnf-eps-region|ebnf-eps-remove-context|ebnf-eps-string\n\t|ebnf-eps-write-kill-temp|ebnf-except-dimension|ebnf-file|ebnf-find-style|ebnf-font-attributes|ebnf-font-background|ebnf-font-foreground\n\t|ebnf-font-height|ebnf-font-list|ebnf-font-name-select|ebnf-font-name|ebnf-font-select|ebnf-font-size|ebnf-font-width|ebnf-format-color\n\t|ebnf-format-float|ebnf-gen-terminal|ebnf-generate-alternative|ebnf-generate-empty|ebnf-generate-eps|ebnf-generate-except|ebnf-generate-non-terminal\n\t|ebnf-generate-one-or-more|ebnf-generate-optional|ebnf-generate-postscript|ebnf-generate-production|ebnf-generate-region|ebnf-generate-repeat\n\t|ebnf-generate-sequence|ebnf-generate-special|ebnf-generate-terminal|ebnf-generate-with-max-height|ebnf-generate-without-max-height\n\t|ebnf-generate-zero-or-more|ebnf-generate|ebnf-get-string|ebnf-horizontal-movement|ebnf-insert-ebnf-prologue|ebnf-insert-style\n\t|ebnf-iso-initialize|ebnf-iso-parser|ebnf-justify-list|ebnf-justify|ebnf-log-header|ebnf-log|ebnf-make-alternative|ebnf-make-dup-sequence\n\t|ebnf-make-empty|ebnf-make-except|ebnf-make-non-terminal|ebnf-make-one-or-more|ebnf-make-optional|ebnf-make-or-more1|ebnf-make-production\n\t|ebnf-make-repeat|ebnf-make-sequence|ebnf-make-special|ebnf-make-terminal|ebnf-make-terminal1|ebnf-make-zero-or-more|ebnf-max-width\n\t|ebnf-merge-style|ebnf-message-float|ebnf-message-info|ebnf-new-page|ebnf-newline|ebnf-node-action|ebnf-node-default|ebnf-node-dimension-func\n\t|ebnf-node-entry|ebnf-node-generation|ebnf-node-height|ebnf-node-kind|ebnf-node-list|ebnf-node-name|ebnf-node-production|ebnf-node-separator\n\t|ebnf-node-width-func|ebnf-node-width|ebnf-non-terminal-dimension|ebnf-one-or-more-dimension|ebnf-optimize|ebnf-optional-dimension\n\t|ebnf-otz-initialize|ebnf-parse-and-sort|ebnf-pop-style|ebnf-print-buffer|ebnf-print-directory|ebnf-print-file|ebnf-print-region\n\t|ebnf-production-dimension|ebnf-push-style|ebnf-range-regexp|ebnf-repeat-dimension|ebnf-reset-style|ebnf-sequence-dimension|ebnf-sequence-width\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t ebnf-setup|ebnf-shape-value|ebnf-sorter-ascending|ebnf-sorter-descending|ebnf-special-dimension|ebnf-spool-buffer|ebnf-spool-directory\n\t|ebnf-spool-file|ebnf-spool-region|ebnf-string|ebnf-syntax-buffer|ebnf-syntax-directory|ebnf-syntax-file|ebnf-syntax-region|ebnf-terminal-dimension\n\t|ebnf-terminal-dimension1|ebnf-token-alternative|ebnf-token-except|ebnf-token-optional|ebnf-token-repeat|ebnf-token-sequence|ebnf-trim-right\n\t|ebnf-vertical-movement|ebnf-yac-initialize|ebnf-yac-parser|ebnf-zero-or-more-dimension|ebrowse-back-in-position-stack|ebrowse-base-classes\n\t|ebrowse-browser-buffer-list|ebrowse-bs-file--cmacro|ebrowse-bs-file|ebrowse-bs-flags--cmacro|ebrowse-bs-flags|ebrowse-bs-name--cmacro\n\t|ebrowse-bs-name|ebrowse-bs-p--cmacro|ebrowse-bs-p|ebrowse-bs-pattern--cmacro|ebrowse-bs-pattern|ebrowse-bs-point--cmacro|ebrowse-bs-point\n\t|ebrowse-bs-scope--cmacro|ebrowse-bs-scope|ebrowse-buffer-p|ebrowse-build-tree-obarray|ebrowse-choose-from-browser-buffers|ebrowse-choose-tree\n\t|ebrowse-class-alist-for-member|ebrowse-class-declaration-regexp|ebrowse-class-in-tree|ebrowse-class-name-displayed-in-member-buffer\n\t|ebrowse-collapse-branch|ebrowse-collapse-fn|ebrowse-completing-read-value|ebrowse-const-p|ebrowse-create-tree-buffer|ebrowse-cs-file--cmacro\n\t|ebrowse-cs-file|ebrowse-cs-flags--cmacro|ebrowse-cs-flags|ebrowse-cs-name--cmacro|ebrowse-cs-name|ebrowse-cs-p--cmacro|ebrowse-cs-p\n\t|ebrowse-cs-pattern--cmacro|ebrowse-cs-pattern|ebrowse-cs-point--cmacro|ebrowse-cs-point|ebrowse-cs-scope--cmacro|ebrowse-cs-scope\n\t|ebrowse-cs-source-file--cmacro|ebrowse-cs-source-file|ebrowse-cyclic-display-next\\/previous-member-list|ebrowse-cyclic-successor-in-string-list\n\t|ebrowse-define-p|ebrowse-direct-base-classes|ebrowse-display-friends-member-list|ebrowse-display-function-member-list|ebrowse-display-member-buffer\n\t|ebrowse-display-member-list-for-accessor|ebrowse-display-next-member-list|ebrowse-display-previous-member-list|ebrowse-display-static-functions-member-list\n\t|ebrowse-display-static-variables-member-list|ebrowse-display-types-member-list|ebrowse-display-variables-member-list|ebrowse-displaying-friends\n\t|ebrowse-displaying-functions|ebrowse-displaying-static-functions|ebrowse-displaying-static-variables|ebrowse-displaying-types\n\t|ebrowse-displaying-variables|ebrowse-draw-file-member-info|ebrowse-draw-marks-fn|ebrowse-draw-member-attributes|ebrowse-draw-member-buffer-class-line\n\t|ebrowse-draw-member-long-fn|ebrowse-draw-member-regexp|ebrowse-draw-member-short-fn|ebrowse-draw-position-buffer|ebrowse-draw-tree-fn\n\t|ebrowse-electric-buffer-list|ebrowse-electric-choose-tree|ebrowse-electric-find-position|ebrowse-electric-get-buffer|ebrowse-electric-list-looper\n\t|ebrowse-electric-list-mode|ebrowse-electric-list-quit|ebrowse-electric-list-select|ebrowse-electric-list-undefined|ebrowse-electric-position-looper\n\t|ebrowse-electric-position-menu|ebrowse-electric-position-mode|ebrowse-electric-position-quit|ebrowse-electric-position-undefined\n\t|ebrowse-electric-select-position|ebrowse-electric-view-buffer|ebrowse-electric-view-position|ebrowse-every|ebrowse-expand-all\n\t|ebrowse-expand-branch|ebrowse-explicit-p|ebrowse-extern-c-p|ebrowse-files-list|ebrowse-files-table|ebrowse-fill-member-table|ebrowse-find-class-declaration\n\t|ebrowse-find-member-declaration|ebrowse-find-member-definition|ebrowse-find-pattern|ebrowse-find-source-file|ebrowse-for-all-trees\n\t|ebrowse-forward-in-position-stack|ebrowse-freeze-member-buffer|ebrowse-frozen-tree-buffer-name|ebrowse-function-declaration\\/definition-regexp\n\t|ebrowse-gather-statistics|ebrowse-globals-tree-p|ebrowse-goto-visible-member\\/all-member-lists|ebrowse-goto-visible-member\n\t|ebrowse-hack-electric-buffer-menu|ebrowse-hide-line|ebrowse-hs-command-line-options--cmacro|ebrowse-hs-command-line-options\n\t|ebrowse-hs-member-table--cmacro|ebrowse-hs-member-table|ebrowse-hs-p--cmacro|ebrowse-hs-p|ebrowse-hs-unused--cmacro|ebrowse-hs-unused\n\t|ebrowse-hs-version--cmacro|ebrowse-hs-version|ebrowse-ignoring-completion-case|ebrowse-inline-p|ebrowse-insert-supers|ebrowse-install-1-to-9-keys\n\t|ebrowse-kill-member-buffers-displaying|ebrowse-known-class-trees-buffer-list|ebrowse-list-of-matching-members|ebrowse-list-tree-buffers\n\t|ebrowse-mark-all-classes|ebrowse-marked-classes-p|ebrowse-member-bit-set-p|ebrowse-member-buffer-list|ebrowse-member-buffer-object-menu\n\t|ebrowse-member-buffer-p|ebrowse-member-class-name-object-menu|ebrowse-member-display-p|ebrowse-member-info-from-point|ebrowse-member-list-name\n\t|ebrowse-member-mode|ebrowse-member-mouse-2|ebrowse-member-mouse-3|ebrowse-member-name-object-menu|ebrowse-member-table|ebrowse-mouse-1-in-tree-buffer\n\t|ebrowse-mouse-2-in-tree-buffer|ebrowse-mouse-3-in-tree-buffer|ebrowse-mouse-find-member|ebrowse-move-in-position-stack|ebrowse-move-point-to-member\n\t|ebrowse-ms-definition-file--cmacro|ebrowse-ms-definition-file|ebrowse-ms-definition-pattern--cmacro|ebrowse-ms-definition-pattern\n\t|ebrowse-ms-definition-point--cmacro|ebrowse-ms-definition-point|ebrowse-ms-file--cmacro|ebrowse-ms-file|ebrowse-ms-flags--cmacro\n\t|ebrowse-ms-flags|ebrowse-ms-name--cmacro|ebrowse-ms-name|ebrowse-ms-p--cmacro|ebrowse-ms-p|ebrowse-ms-pattern--cmacro|ebrowse-ms-pattern\n\t|ebrowse-ms-point--cmacro|ebrowse-ms-point|ebrowse-ms-scope--cmacro|ebrowse-ms-scope|ebrowse-ms-visibility--cmacro|ebrowse-ms-visibility\n\t|ebrowse-mutable-p|ebrowse-name\\/accessor-alist-for-class-members|ebrowse-name\\/accessor-alist-for-visible-members|ebrowse-name\\/accessor-alist\n\t|ebrowse-on-class-name|ebrowse-on-member-name|ebrowse-output|ebrowse-pop\\/switch-to-member-buffer-for-same-tree|ebrowse-pop-from-member-to-tree-buffer\n\t|ebrowse-pop-to-browser-buffer|ebrowse-popup-menu|ebrowse-position-file-name--cmacro|ebrowse-position-file-name|ebrowse-position-info--cmacro\n\t|ebrowse-position-info|ebrowse-position-name|ebrowse-position-p--cmacro|ebrowse-position-p|ebrowse-position-point--cmacro|ebrowse-position-point\n\t|ebrowse-position-target--cmacro|ebrowse-position-target|ebrowse-position|ebrowse-pp-define-regexp|ebrowse-print-statistics-line\n\t|ebrowse-pure-virtual-p|ebrowse-push-position|ebrowse-qualified-class-name|ebrowse-read-class-name-and-go|ebrowse-read|ebrowse-redisplay-member-buffer\n\t|ebrowse-redraw-marks|ebrowse-redraw-tree|ebrowse-remove-all-member-filters|ebrowse-remove-class-and-kill-member-buffers|ebrowse-remove-class-at-point\n\t|ebrowse-rename-buffer|ebrowse-repeat-member-search|ebrowse-revert-tree-buffer-from-file|ebrowse-same-tree-member-buffer-list\n\t|ebrowse-save-class|ebrowse-save-selective|ebrowse-save-tree-as|ebrowse-save-tree|ebrowse-select-1st-to-9nth|ebrowse-set-face|ebrowse-set-mark-props\n\t|ebrowse-set-member-access-visibility|ebrowse-set-member-buffer-column-width|ebrowse-set-tree-indentation|ebrowse-show-displayed-class-in-tree\n\t|ebrowse-show-file-name-at-point|ebrowse-show-progress|ebrowse-some-member-table|ebrowse-some|ebrowse-sort-tree-list|ebrowse-statistics\n\t|ebrowse-switch-member-buffer-to-any-class|ebrowse-switch-member-buffer-to-base-class|ebrowse-switch-member-buffer-to-derived-class\n\t|ebrowse-switch-member-buffer-to-next-sibling-class|ebrowse-switch-member-buffer-to-other-class|ebrowse-switch-member-buffer-to-previous-sibling-class\n\t|ebrowse-switch-member-buffer-to-sibling-class|ebrowse-switch-to-next-member-buffer|ebrowse-symbol-regexp|ebrowse-tags-apropos\n\t|ebrowse-tags-choose-class|ebrowse-tags-complete-symbol|ebrowse-tags-display-member-buffer|ebrowse-tags-find-declaration-other-frame\n\t|ebrowse-tags-find-declaration-other-window|ebrowse-tags-find-declaration|ebrowse-tags-find-definition-other-frame|ebrowse-tags-find-definition-other-window\n\t|ebrowse-tags-find-definition|ebrowse-tags-list-members-in-file|ebrowse-tags-loop-continue|ebrowse-tags-next-file|ebrowse-tags-query-replace\n\t|ebrowse-tags-read-member\\+class-name|ebrowse-tags-read-name|ebrowse-tags-search-member-use|ebrowse-tags-search|ebrowse-tags-select\\/create-member-buffer\n\t|ebrowse-tags-view\\/find-member-decl\\/defn|ebrowse-tags-view-declaration-other-frame|ebrowse-tags-view-declaration-other-window\n\t|ebrowse-tags-view-declaration|ebrowse-tags-view-definition-other-frame|ebrowse-tags-view-definition-other-window|ebrowse-tags-view-definition\n\t|ebrowse-template-p|ebrowse-throw-list-p|ebrowse-toggle-base-class-display|ebrowse-toggle-const-member-filter|ebrowse-toggle-file-name-display\n\t|ebrowse-toggle-inline-member-filter|ebrowse-toggle-long-short-display|ebrowse-toggle-mark-at-point|ebrowse-toggle-member-attributes-display\n\t|ebrowse-toggle-private-member-filter|ebrowse-toggle-protected-member-filter|ebrowse-toggle-public-member-filter|ebrowse-toggle-pure-member-filter\n\t|ebrowse-toggle-regexp-display|ebrowse-toggle-virtual-member-filter|ebrowse-tree-at-point|ebrowse-tree-buffer-class-object-menu\n\t|ebrowse-tree-buffer-list|ebrowse-tree-buffer-object-menu|ebrowse-tree-buffer-p|ebrowse-tree-command:show-friends|ebrowse-tree-command:show-member-functions\n\t|ebrowse-tree-command:show-member-variables|ebrowse-tree-command:show-static-member-functions|ebrowse-tree-command:show-static-member-variables\n\t|ebrowse-tree-command:show-types|ebrowse-tree-mode|ebrowse-tree-obarray-as-alist|ebrowse-trim-string|ebrowse-ts-base-classes--cmacro\n\t|ebrowse-ts-base-classes|ebrowse-ts-class--cmacro|ebrowse-ts-class|ebrowse-ts-friends--cmacro|ebrowse-ts-friends|ebrowse-ts-mark--cmacro\n\t|ebrowse-ts-mark|ebrowse-ts-member-functions--cmacro|ebrowse-ts-member-functions|ebrowse-ts-member-variables--cmacro|ebrowse-ts-member-variables\n\t|ebrowse-ts-p--cmacro|ebrowse-ts-p|ebrowse-ts-static-functions--cmacro|ebrowse-ts-static-functions|ebrowse-ts-static-variables--cmacro\n\t|ebrowse-ts-static-variables|ebrowse-ts-subclasses--cmacro|ebrowse-ts-subclasses|ebrowse-ts-types--cmacro|ebrowse-ts-types|ebrowse-unhide-base-classes\n\t|ebrowse-update-member-buffer-mode-line|ebrowse-update-tree-buffer-mode-line|ebrowse-variable-declaration-regexp|ebrowse-view\\/find-class-declaration\n\t|ebrowse-view\\/find-file-and-search-pattern|ebrowse-view\\/find-member-declaration\\/definition|ebrowse-view\\/find-position\n\t|ebrowse-view-class-declaration|ebrowse-view-exit-fn|ebrowse-view-file-other-frame|ebrowse-view-member-declaration|ebrowse-view-member-definition\n\t|ebrowse-virtual-p|ebrowse-width-of-drawable-area|ebrowse-write-file-hook-fn|ebuffers|ebuffers3|ecase|ecomplete-display-matches\n\t|ecomplete-setup|ede--detect-ldf-predicate|ede--detect-ldf-root-predicate|ede--detect-ldf-rootonly-predicate|ede--detect-scan-directory-for-project-root\n\t|ede--detect-scan-directory-for-project|ede--detect-scan-directory-for-rootonly-project|ede--detect-stop-scan-p|ede--directory-project-add-description-to-hash\n\t|ede--directory-project-from-hash|ede--get-inode-dir-hash|ede--inode-for-dir|ede--inode-get-toplevel-open-project|ede--project-inode\n\t|ede--put-inode-dir-hash|ede-add-file|ede-add-project-autoload|ede-add-project-to-global-list|ede-add-subproject|ede-adebug-project-parent\n\t|ede-adebug-project-root|ede-adebug-project|ede-apply-object-keymap|ede-apply-preprocessor-map|ede-apply-project-local-variables\n\t|ede-apply-target-options|ede-auto-add-to-target|ede-auto-detect-in-dir|ede-auto-load-project|ede-buffer-belongs-to-project-p\n\t|ede-buffer-belongs-to-target-p|ede-buffer-documentation-files|ede-buffer-header-file|ede-buffer-mine|ede-buffer-object|ede-buffers\n\t|ede-build-forms-menu|ede-check-project-directory|ede-choose-object|ede-commit-local-variables|ede-compile-project|ede-compile-selected\n\t|ede-compile-target|ede-configuration-forms-menu|ede-convert-path|ede-cpp-root-project-child-p|ede-cpp-root-project-list-p|ede-cpp-root-project-p\n\t|ede-cpp-root-project|ede-create-tag-buttons|ede-current-project|ede-customize-current-target|ede-customize-forms-menu|ede-customize-project\n\t|ede-debug-target|ede-delete-project-from-global-list|ede-delete-target|ede-description|ede-detect-directory-for-project|ede-detect-qtest\n\t|ede-directory-get-open-project|ede-directory-get-toplevel-open-project|ede-directory-project-cons|ede-directory-project-p|ede-directory-safe-p\n\t|ede-dired-minor-mode|ede-dirmatch-installed|ede-do-dirmatch|ede-documentation-files|ede-documentation|ede-ecb-project-paths|ede-edit-file-target\n\t|ede-edit-web-page|ede-enable-generic-projects|ede-enable-locate-on-project|ede-expand-filename-impl-via-subproj|ede-expand-filename-impl\n\t|ede-expand-filename-local|ede-expand-filename|ede-file-find|ede-find-file|ede-find-nearest-file-line|ede-find-subproject-for-directory\n\t|ede-find-target|ede-flush-deleted-projects|ede-flush-directory-hash|ede-flush-project-hash|ede-get-locator-object|ede-global-list-sanity-check\n\t|ede-header-file|ede-html-documentation-files|ede-html-documentation|ede-ignore-file|ede-initialize-state-current-buffer|ede-invoke-method\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t ede-java-classpath|ede-linux-load|ede-load-cache|ede-load-project-file|ede-make-check-version|ede-make-dist|ede-make-project-local-variable\n\t|ede-map-all-subprojects|ede-map-any-target-p|ede-map-buffers|ede-map-project-buffers|ede-map-subprojects|ede-map-target-buffers\n\t|ede-map-targets|ede-menu-items-build|ede-menu-obj-of-class-p|ede-minor-mode|ede-name|ede-new-target-custom|ede-new-target|ede-new\n\t|ede-normalize-file\\/directory|ede-object-keybindings|ede-object-menu|ede-object-sourcecode|ede-parent-project|ede-preprocessor-map\n\t|ede-project-autoload-child-p|ede-project-autoload-dirmatch-child-p|ede-project-autoload-dirmatch-list-p|ede-project-autoload-dirmatch-p\n\t|ede-project-autoload-dirmatch|ede-project-autoload-list-p|ede-project-autoload-p|ede-project-autoload|ede-project-buffers|ede-project-child-p\n\t|ede-project-configurations-set|ede-project-directory-remove-hash|ede-project-forms-menu|ede-project-list-p|ede-project-p|ede-project-placeholder-child-p\n\t|ede-project-placeholder-list-p|ede-project-placeholder-p|ede-project-placeholder|ede-project-root-directory|ede-project-root\n\t|ede-project-sort-targets|ede-project|ede-remove-file|ede-rescan-toplevel|ede-reset-all-buffers|ede-run-target|ede-save-cache|ede-set-project-local-variable\n\t|ede-set-project-variables|ede-set|ede-singular-object|ede-source-paths|ede-sourcecode-child-p|ede-sourcecode-list-p|ede-sourcecode-p\n\t|ede-sourcecode|ede-speedbar-compile-file-project|ede-speedbar-compile-line|ede-speedbar-compile-project|ede-speedbar-edit-projectfile\n\t|ede-speedbar-file-setup|ede-speedbar-get-top-project-for-line|ede-speedbar-make-distribution|ede-speedbar-make-map|ede-speedbar-remove-file-from-target\n\t|ede-speedbar-toplevel-buttons|ede-speedbar|ede-subproject-p|ede-subproject-relative-path|ede-system-include-path|ede-tag-expand\n\t|ede-tag-find|ede-target-buffer-in-sourcelist|ede-target-buffers|ede-target-child-p|ede-target-forms-menu|ede-target-in-project-p\n\t|ede-target-list-p|ede-target-name|ede-target-p|ede-target-parent|ede-target-sourcecode|ede-target|ede-toplevel-project-or-nil|ede-toplevel-project\n\t|ede-toplevel|ede-turn-on-hook|ede-up-directory|ede-update-version|ede-upload-distribution|ede-upload-html-documentation|ede-vc-project-directory\n\t|ede-version|ede-want-any-auxiliary-files-p|ede-want-any-files-p|ede-want-any-source-files-p|ede-want-file-auxiliary-p|ede-want-file-p\n\t|ede-want-file-source-p|ede-web-browse-home|ede-with-projectfile|ede|edebug-&optional-wrapper|edebug-&rest-wrapper|edebug--called-interactively-skip\n\t|edebug--display|edebug--enter-trace|edebug--form-data-begin--cmacro|edebug--form-data-begin|edebug--form-data-end--cmacro|edebug--form-data-end\n\t|edebug--form-data-name--cmacro|edebug--form-data-name|edebug--make-form-data-entry--cmacro|edebug--make-form-data-entry|edebug--read\n\t|edebug--recursive-edit|edebug--require-cl-read|edebug--update-coverage|edebug-Continue-fast-mode|edebug-Go-nonstop-mode|edebug-Trace-fast-mode\n\t|edebug-`|edebug-adjust-window|edebug-after-offset|edebug-after|edebug-all-defuns|edebug-backtrace|edebug-basic-spec|edebug-before-offset\n\t|edebug-before|edebug-bounce-point|edebug-changing-windows|edebug-clear-coverage|edebug-clear-form-data-entry|edebug-clear-frequency-count\n\t|edebug-compute-previous-result|edebug-continue-mode|edebug-copy-cursor|edebug-create-eval-buffer|edebug-current-windows|edebug-cursor-expressions\n\t|edebug-cursor-offsets|edebug-debugger|edebug-defining-form|edebug-delete-eval-item|edebug-empty-cursor|edebug-enter|edebug-eval-defun\n\t|edebug-eval-display-list|edebug-eval-display|edebug-eval-expression|edebug-eval-last-sexp|edebug-eval-mode|edebug-eval-print-last-sexp\n\t|edebug-eval-redisplay|edebug-eval-result-list|edebug-eval|edebug-fast-after|edebug-fast-before|edebug-find-stop-point|edebug-form-data-symbol\n\t|edebug-form|edebug-format|edebug-forms|edebug-forward-sexp|edebug-get-displayed-buffer-points|edebug-get-form-data-entry|edebug-go-mode\n\t|edebug-goto-here|edebug-help|edebug-ignore-offset|edebug-inc-offset|edebug-initialize-offsets|edebug-install-read-eval-functions\n\t|edebug-instrument-callee|edebug-instrument-function|edebug-interactive-p-name|edebug-kill-buffer|edebug-lambda-list-keywordp\n\t|edebug-last-sexp|edebug-list-form-args|edebug-list-form|edebug-make-after-form|edebug-make-before-and-after-form|edebug-make-enter-wrapper\n\t|edebug-make-form-wrapper|edebug-make-top-form-data-entry|edebug-mark-marker|edebug-mark|edebug-match-&define|edebug-match-&key\n\t|edebug-match-¬|edebug-match-&optional|edebug-match-&or|edebug-match-&rest|edebug-match-arg|edebug-match-body|edebug-match-colon-name\n\t|edebug-match-def-body|edebug-match-def-form|edebug-match-form|edebug-match-function|edebug-match-gate|edebug-match-lambda-expr\n\t|edebug-match-list|edebug-match-name|edebug-match-nil|edebug-match-one-spec|edebug-match-place|edebug-match-sexp|edebug-match-specs\n\t|edebug-match-string|edebug-match-sublist|edebug-match-symbol|edebug-match|edebug-menu|edebug-message|edebug-mode|edebug-modify-breakpoint\n\t|edebug-move-cursor|edebug-new-cursor|edebug-next-breakpoint|edebug-next-mode|edebug-next-token-class|edebug-no-match|edebug-on-entry\n\t|edebug-outside-excursion|edebug-overlay-arrow|edebug-pop-to-buffer|edebug-previous-result|edebug-prin1-to-string|edebug-prin1\n\t|edebug-print|edebug-read-and-maybe-wrap-form|edebug-read-and-maybe-wrap-form1|edebug-read-backquote|edebug-read-comma|edebug-read-function\n\t|edebug-read-list|edebug-read-quote|edebug-read-sexp|edebug-read-storing-offsets|edebug-read-string|edebug-read-symbol|edebug-read-top-level-form\n\t|edebug-read-vector|edebug-report-error|edebug-restore-status|edebug-run-fast|edebug-run-slow|edebug-safe-eval|edebug-safe-prin1-to-string\n\t|edebug-set-breakpoint|edebug-set-buffer-points|edebug-set-conditional-breakpoint|edebug-set-cursor|edebug-set-form-data-entry\n\t|edebug-set-mode|edebug-set-windows|edebug-sexps|edebug-signal|edebug-skip-whitespace|edebug-slow-after|edebug-slow-before|edebug-sort-alist\n\t|edebug-spec-p|edebug-step-in|edebug-step-mode|edebug-step-out|edebug-step-through-mode|edebug-stop|edebug-store-after-offset|edebug-store-before-offset\n\t|edebug-storing-offsets|edebug-syntax-error|edebug-toggle-save-all-windows|edebug-toggle-save-selected-window|edebug-toggle-save-windows\n\t|edebug-toggle|edebug-top-element-required|edebug-top-element|edebug-top-level-nonstop|edebug-top-offset|edebug-trace-display|edebug-trace-mode\n\t|edebug-uninstall-read-eval-functions|edebug-unload-function|edebug-unset-breakpoint|edebug-unwrap\\*|edebug-unwrap|edebug-update-eval-list\n\t|edebug-var-status|edebug-view-outside|edebug-visit-eval-list|edebug-where|edebug-window-list|edebug-window-live-p|edebug-wrap-def-body\n\t|ediff-3way-comparison-job|ediff-3way-job|ediff-abbrev-jobname|ediff-abbreviate-file-name|ediff-activate-mark|ediff-add-slash-if-directory\n\t|ediff-add-to-history|ediff-ancestor-metajob|ediff-append-custom-diff|ediff-arrange-autosave-in-merge-jobs|ediff-background-face\n\t|ediff-backup|ediff-barf-if-not-control-buffer|ediff-buffer-live-p|ediff-buffer-type|ediff-buffers-internal|ediff-buffers|ediff-buffers3\n\t|ediff-bury-dir-diffs-buffer|ediff-calc-command-time|ediff-change-saved-variable|ediff-char-to-buftype|ediff-check-version|ediff-choose-syntax-table\n\t|ediff-choose-window-setup-function-automatically|ediff-cleanup-mess|ediff-cleanup-meta-buffer|ediff-clear-diff-vector|ediff-clear-fine-diff-vector\n\t|ediff-clear-fine-differences-in-one-buffer|ediff-clear-fine-differences|ediff-clone-buffer-for-current-diff-comparison|ediff-clone-buffer-for-region-comparison\n\t|ediff-clone-buffer-for-window-comparison|ediff-collect-custom-diffs|ediff-collect-diffs-metajob|ediff-color-display-p|ediff-combine-diffs\n\t|ediff-comparison-metajob3|ediff-compute-custom-diffs-maybe|ediff-compute-toolbar-width|ediff-convert-diffs-to-overlays|ediff-convert-fine-diffs-to-overlays\n\t|ediff-convert-standard-filename|ediff-copy-A-to-B|ediff-copy-A-to-C|ediff-copy-B-to-A|ediff-copy-B-to-C|ediff-copy-C-to-A|ediff-copy-C-to-B\n\t|ediff-copy-diff|ediff-copy-list|ediff-copy-to-buffer|ediff-current-file|ediff-customize|ediff-deactivate-mark|ediff-debug-info\n\t|ediff-default-suspend-function|ediff-defvar-local|ediff-delete-all-matches|ediff-delete-overlay|ediff-delete-temp-files|ediff-destroy-control-frame\n\t|ediff-device-type|ediff-diff-at-point|ediff-diff-to-diff|ediff-diff3-job|ediff-dir-diff-copy-file|ediff-directories-command|ediff-directories-internal\n\t|ediff-directories|ediff-directories3-command|ediff-directories3|ediff-directory-revisions-internal|ediff-directory-revisions\n\t|ediff-display-pixel-height|ediff-display-pixel-width|ediff-dispose-of-meta-buffer|ediff-dispose-of-variant-according-to-user\n\t|ediff-do-merge|ediff-documentation|ediff-draw-dir-diffs|ediff-empty-diff-region-p|ediff-empty-overlay-p|ediff-event-buffer|ediff-event-key\n\t|ediff-event-point|ediff-exec-process|ediff-extract-diffs|ediff-extract-diffs3|ediff-file-attributes|ediff-file-checked-in-p|ediff-file-checked-out-p\n\t|ediff-file-compressed-p|ediff-file-modtime|ediff-file-remote-p|ediff-file-size|ediff-filegroup-action|ediff-filename-magic-p|ediff-files-command\n\t|ediff-files-internal|ediff-files|ediff-files3|ediff-fill-leading-zero|ediff-find-file|ediff-focus-on-regexp-matches|ediff-format-bindings-of\n\t|ediff-format-date|ediff-forward-word|ediff-frame-char-height|ediff-frame-char-width|ediff-frame-has-dedicated-windows|ediff-frame-iconified-p\n\t|ediff-frame-unsplittable-p|ediff-get-buffer|ediff-get-combined-region|ediff-get-default-directory-name|ediff-get-default-file-name\n\t|ediff-get-diff-overlay-from-diff-record|ediff-get-diff-overlay|ediff-get-diff-posn|ediff-get-diff3-group|ediff-get-difference\n\t|ediff-get-directory-files-under-revision|ediff-get-file-eqstatus|ediff-get-fine-diff-vector-from-diff-record|ediff-get-fine-diff-vector\n\t|ediff-get-group-buffer|ediff-get-group-comparison-func|ediff-get-group-merge-autostore-dir|ediff-get-group-objA|ediff-get-group-objB\n\t|ediff-get-group-objC|ediff-get-group-regexp|ediff-get-lines-to-region-end|ediff-get-lines-to-region-start|ediff-get-meta-info\n\t|ediff-get-meta-overlay-at-pos|ediff-get-next-window|ediff-get-region-contents|ediff-get-region-size-coefficient|ediff-get-selected-buffers\n\t|ediff-get-session-activity-marker|ediff-get-session-buffer|ediff-get-session-number-at-pos|ediff-get-session-objA-name|ediff-get-session-objA\n\t|ediff-get-session-objB-name|ediff-get-session-objB|ediff-get-session-objC-name|ediff-get-session-objC|ediff-get-session-status\n\t|ediff-get-state-of-ancestor|ediff-get-state-of-diff|ediff-get-state-of-merge|ediff-get-symbol-from-alist|ediff-get-value-according-to-buffer-type\n\t|ediff-get-visible-buffer-window|ediff-get-window-by-clicking|ediff-good-frame-under-mouse|ediff-goto-word|ediff-has-face-support-p\n\t|ediff-has-gutter-support-p|ediff-has-toolbar-support-p|ediff-help-for-quick-help|ediff-help-message-line-length|ediff-hide-face\n\t|ediff-hide-marked-sessions|ediff-hide-regexp-matches|ediff-highlight-diff-in-one-buffer|ediff-highlight-diff|ediff-in-control-buffer-p\n\t|ediff-indent-help-message|ediff-inferior-compare-regions|ediff-insert-dirs-in-meta-buffer|ediff-insert-session-activity-marker-in-meta-buffer\n\t|ediff-insert-session-info-in-meta-buffer|ediff-insert-session-status-in-meta-buffer|ediff-install-fine-diff-if-necessary|ediff-intersect-directories\n\t|ediff-intersection|ediff-janitor|ediff-jump-to-difference-at-point|ediff-jump-to-difference|ediff-keep-window-config|ediff-key-press-event-p\n\t|ediff-kill-bottom-toolbar|ediff-kill-buffer-carefully|ediff-last-command-char|ediff-listable-file|ediff-load-version-control\n\t|ediff-looks-like-combined-merge|ediff-make-base-title|ediff-make-bottom-toolbar|ediff-make-bullet-proof-overlay|ediff-make-cloned-buffer\n\t|ediff-make-current-diff-overlay|ediff-make-diff2-buffer|ediff-make-empty-tmp-file|ediff-make-fine-diffs|ediff-make-frame-position\n\t|ediff-make-indirect-buffer|ediff-make-narrow-control-buffer-id|ediff-make-new-meta-list-element|ediff-make-new-meta-list-header\n\t|ediff-make-or-kill-fine-diffs|ediff-make-overlay|ediff-make-temp-file|ediff-make-wide-control-buffer-id|ediff-make-wide-display\n\t|ediff-mark-diff-as-space-only|ediff-mark-for-hiding-at-pos|ediff-mark-for-operation-at-pos|ediff-mark-if-equal|ediff-mark-session-for-hiding\n\t|ediff-mark-session-for-operation|ediff-maybe-checkout|ediff-maybe-save-and-delete-merge|ediff-member|ediff-merge-buffers-with-ancestor\n\t|ediff-merge-buffers|ediff-merge-changed-from-default-p|ediff-merge-command|ediff-merge-directories-command|ediff-merge-directories-with-ancestor-command\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t ediff-merge-directories-with-ancestor|ediff-merge-directories|ediff-merge-directory-revisions-with-ancestor|ediff-merge-directory-revisions\n\t|ediff-merge-files-with-ancestor|ediff-merge-files|ediff-merge-job|ediff-merge-metajob|ediff-merge-on-startup|ediff-merge-region-is-non-clash-to-skip\n\t|ediff-merge-region-is-non-clash|ediff-merge-revisions-with-ancestor|ediff-merge-revisions|ediff-merge-with-ancestor-command\n\t|ediff-merge-with-ancestor-job|ediff-merge-with-ancestor|ediff-merge|ediff-message-if-verbose|ediff-meta-insert-file-info1|ediff-meta-mark-equal-files\n\t|ediff-meta-mode|ediff-meta-session-p|ediff-meta-show-patch|ediff-metajob3|ediff-minibuffer-with-setup-hook|ediff-mode|ediff-mouse-event-p\n\t|ediff-move-overlay|ediff-multiframe-setup-p|ediff-narrow-control-frame-p|ediff-narrow-job|ediff-next-difference|ediff-next-meta-item\n\t|ediff-next-meta-item1|ediff-next-meta-overlay-start|ediff-no-fine-diffs-p|ediff-nonempty-string-p|ediff-nuke-selective-display\n\t|ediff-one-filegroup-metajob|ediff-operate-on-marked-sessions|ediff-operate-on-windows|ediff-other-buffer|ediff-overlay-buffer\n\t|ediff-overlay-end|ediff-overlay-get|ediff-overlay-put|ediff-overlay-start|ediff-overlayp|ediff-paint-background-regions-in-one-buffer\n\t|ediff-paint-background-regions|ediff-patch-buffer|ediff-patch-file-form-meta|ediff-patch-file-internal|ediff-patch-file|ediff-patch-job\n\t|ediff-patch-metajob|ediff-place-flags-in-buffer|ediff-place-flags-in-buffer1|ediff-pop-diff|ediff-position-region|ediff-prepare-error-list\n\t|ediff-prepare-meta-buffer|ediff-previous-difference|ediff-previous-meta-item|ediff-previous-meta-item1|ediff-previous-meta-overlay-start\n\t|ediff-print-diff-vector|ediff-problematic-session-p|ediff-process-filter|ediff-process-sentinel|ediff-profile|ediff-quit-meta-buffer\n\t|ediff-quit|ediff-re-merge|ediff-read-event|ediff-read-file-name|ediff-really-quit|ediff-recenter-ancestor|ediff-recenter-one-window\n\t|ediff-recenter|ediff-redraw-directory-group-buffer|ediff-redraw-registry-buffer|ediff-refresh-control-frame|ediff-refresh-mode-lines\n\t|ediff-region-help-echo|ediff-regions-internal|ediff-regions-linewise|ediff-regions-wordwise|ediff-registry-action|ediff-reload-keymap\n\t|ediff-remove-flags-from-buffer|ediff-replace-session-activity-marker-in-meta-buffer|ediff-replace-session-status-in-meta-buffer\n\t|ediff-reset-mouse|ediff-restore-diff-in-merge-buffer|ediff-restore-diff|ediff-restore-highlighting|ediff-restore-protected-variables\n\t|ediff-restore-variables|ediff-revert-buffers-then-recompute-diffs|ediff-revision-metajob|ediff-revision|ediff-safe-to-quit|ediff-same-contents\n\t|ediff-same-file-contents-lists|ediff-same-file-contents|ediff-save-buffer-in-file|ediff-save-buffer|ediff-save-diff-region|ediff-save-protected-variables\n\t|ediff-save-time|ediff-save-variables|ediff-scroll-horizontally|ediff-scroll-vertically|ediff-select-difference|ediff-select-lowest-window\n\t|ediff-set-actual-diff-options|ediff-set-diff-options|ediff-set-diff-overlays-in-one-buffer|ediff-set-difference|ediff-set-face-pixmap\n\t|ediff-set-file-eqstatus|ediff-set-fine-diff-properties-in-one-buffer|ediff-set-fine-diff-properties|ediff-set-fine-diff-vector\n\t|ediff-set-fine-overlays-for-combined-merge|ediff-set-fine-overlays-in-one-buffer|ediff-set-help-message|ediff-set-help-overlays\n\t|ediff-set-keys|ediff-set-merge-mode|ediff-set-meta-overlay|ediff-set-overlay-face|ediff-set-read-only-in-buf-A|ediff-set-session-status\n\t|ediff-set-state-of-all-diffs-in-all-buffers|ediff-set-state-of-diff-in-all-buffers|ediff-set-state-of-diff|ediff-set-state-of-merge\n\t|ediff-setup-control-buffer|ediff-setup-control-frame|ediff-setup-diff-regions|ediff-setup-diff-regions3|ediff-setup-fine-diff-regions\n\t|ediff-setup-keymap|ediff-setup-meta-map|ediff-setup-windows-default|ediff-setup-windows-multiframe-compare|ediff-setup-windows-multiframe-merge\n\t|ediff-setup-windows-multiframe|ediff-setup-windows-plain-compare|ediff-setup-windows-plain-merge|ediff-setup-windows-plain|ediff-setup-windows\n\t|ediff-setup|ediff-show-all-diffs|ediff-show-ancestor|ediff-show-current-session-meta-buffer|ediff-show-diff-output|ediff-show-dir-diffs\n\t|ediff-show-meta-buff-from-registry|ediff-show-meta-buffer|ediff-show-registry|ediff-shrink-window-C|ediff-skip-merge-region-if-changed-from-default-p\n\t|ediff-skip-unsuitable-frames|ediff-spy-after-mouse|ediff-status-info|ediff-strip-last-dir|ediff-strip-mode-line-format|ediff-submit-report\n\t|ediff-suspend|ediff-swap-buffers|ediff-test-save-region|ediff-toggle-autorefine|ediff-toggle-filename-truncation|ediff-toggle-help\n\t|ediff-toggle-hilit|ediff-toggle-ignore-case|ediff-toggle-multiframe|ediff-toggle-narrow-region|ediff-toggle-read-only|ediff-toggle-regexp-match\n\t|ediff-toggle-show-clashes-only|ediff-toggle-skip-changed-regions|ediff-toggle-skip-similar|ediff-toggle-split|ediff-toggle-use-toolbar\n\t|ediff-toggle-verbose-help-meta-buffer|ediff-toggle-wide-display|ediff-truncate-string-left|ediff-unhighlight-diff-in-one-buffer\n\t|ediff-unhighlight-diff|ediff-unhighlight-diffs-totally-in-one-buffer|ediff-unhighlight-diffs-totally|ediff-union|ediff-unique-buffer-name\n\t|ediff-unmark-all-for-hiding|ediff-unmark-all-for-operation|ediff-unselect-and-select-difference|ediff-unselect-difference|ediff-up-meta-hierarchy\n\t|ediff-update-diffs|ediff-update-markers-in-dir-meta-buffer|ediff-update-meta-buffer|ediff-update-registry|ediff-update-session-marker-in-dir-meta-buffer\n\t|ediff-use-toolbar-p|ediff-user-grabbed-mouse|ediff-valid-difference-p|ediff-verify-file-buffer|ediff-verify-file-merge-buffer\n\t|ediff-version|ediff-visible-region|ediff-whitespace-diff-region-p|ediff-window-display-p|ediff-window-ok-for-display|ediff-window-visible-p\n\t|ediff-windows-job|ediff-windows-linewise|ediff-windows-wordwise|ediff-windows|ediff-with-current-buffer|ediff-with-syntax-table\n\t|ediff-word-mode-job|ediff-wordify|ediff-write-merge-buffer-and-maybe-kill|ediff-xemacs-select-frame-hook|ediff|ediff3-files-command\n\t|ediff3|edir-merge-revisions-with-ancestor|edir-merge-revisions|edir-revisions|edirs-merge-with-ancestor|edirs-merge|edirs|edirs3\n\t|edit-abbrevs-mode|edit-abbrevs-redefine|edit-abbrevs|edit-bookmarks|edit-kbd-macro|edit-last-kbd-macro|edit-named-kbd-macro|edit-picture\n\t|edit-tab-stops-note-changes|edit-tab-stops|edmacro-finish-edit|edmacro-fix-menu-commands|edmacro-format-keys|edmacro-insert-key\n\t|edmacro-mode|edmacro-parse-keys|edmacro-sanitize-for-string|edt-advance|edt-append|edt-backup|edt-beginning-of-line|edt-bind-function-key-default\n\t|edt-bind-function-key|edt-bind-gold-key-default|edt-bind-gold-key|edt-bind-key-default|edt-bind-key|edt-bind-standard-key|edt-bottom-check\n\t|edt-bottom|edt-change-case|edt-change-direction|edt-character|edt-check-match|edt-check-prefix|edt-check-selection|edt-copy-rectangle\n\t|edt-copy|edt-current-line|edt-cut-or-copy|edt-cut-rectangle-insert-mode|edt-cut-rectangle-overstrike-mode|edt-cut-rectangle|edt-cut\n\t|edt-default-emulation-setup|edt-default-menu-bar-update-buffers|edt-define-key|edt-delete-character|edt-delete-entire-line|edt-delete-line\n\t|edt-delete-previous-character|edt-delete-to-beginning-of-line|edt-delete-to-beginning-of-word|edt-delete-to-end-of-line|edt-delete-word\n\t|edt-display-the-time|edt-duplicate-line|edt-duplicate-word|edt-electric-helpify|edt-electric-keypad-help|edt-electric-user-keypad-help\n\t|edt-eliminate-all-tabs|edt-emulation-off|edt-emulation-on|edt-end-of-line-backward|edt-end-of-line-forward|edt-end-of-line|edt-exit\n\t|edt-fill-region|edt-find-backward|edt-find-forward|edt-find-next-backward|edt-find-next-forward|edt-find-next|edt-find|edt-form-feed-insert\n\t|edt-goto-percentage|edt-indent-or-fill-region|edt-key-not-assigned|edt-keypad-help|edt-learn|edt-line-backward|edt-line-forward\n\t|edt-line-to-bottom-of-window|edt-line-to-middle-of-window|edt-line-to-top-of-window|edt-line|edt-load-keys|edt-lowercase|edt-mark-section-wisely\n\t|edt-match-beginning|edt-match-end|edt-next-line|edt-one-word-backward|edt-one-word-forward|edt-page-backward|edt-page-forward|edt-page\n\t|edt-paragraph-backward|edt-paragraph-forward|edt-paragraph|edt-paste-rectangle-insert-mode|edt-paste-rectangle-overstrike-mode\n\t|edt-paste-rectangle|edt-previous-line|edt-quit|edt-remember|edt-replace|edt-reset|edt-restore-key|edt-scroll-line|edt-scroll-window-backward-line\n\t|edt-scroll-window-backward|edt-scroll-window-forward-line|edt-scroll-window-forward|edt-scroll-window|edt-sect-backward|edt-sect-forward\n\t|edt-sect|edt-select-default-global-map|edt-select-mode|edt-select-user-global-map|edt-select|edt-sentence-backward|edt-sentence-forward\n\t|edt-sentence|edt-set-match|edt-set-screen-width-132|edt-set-screen-width-80|edt-set-scroll-margins|edt-setup-default-bindings\n\t|edt-show-match-markers|edt-split-window|edt-substitute|edt-switch-global-maps|edt-tab-insert|edt-toggle-capitalization-of-word\n\t|edt-toggle-select|edt-top-check|edt-top|edt-undelete-character|edt-undelete-line|edt-undelete-word|edt-unset-match|edt-uppercase\n\t|edt-user-emulation-setup|edt-user-menu-bar-update-buffers|edt-window-bottom|edt-window-top|edt-with-position|edt-word-backward\n\t|edt-word-forward|edt-word|edt-y-or-n-p|ehelp-command|eieio--check-type|eieio--class--unused-0|eieio--class-children|eieio--class-class-allocation-a\n\t|eieio--class-class-allocation-custom-group|eieio--class-class-allocation-custom-label|eieio--class-class-allocation-custom\n\t|eieio--class-class-allocation-doc|eieio--class-class-allocation-printer|eieio--class-class-allocation-protection|eieio--class-class-allocation-type\n\t|eieio--class-class-allocation-values|eieio--class-default-object-cache|eieio--class-initarg-tuples|eieio--class-options|eieio--class-parent\n\t|eieio--class-protection|eieio--class-public-a|eieio--class-public-custom-group|eieio--class-public-custom-label|eieio--class-public-custom\n\t|eieio--class-public-d|eieio--class-public-doc|eieio--class-public-printer|eieio--class-public-type|eieio--class-symbol-obarray\n\t|eieio--class-symbol|eieio--defalias|eieio--defgeneric-init-form|eieio--define-field-accessors|eieio--defmethod|eieio--object--unused-0\n\t|eieio--object-class|eieio--object-name|eieio--scoped-class|eieio--with-scoped-class|eieio-add-new-slot|eieio-attribute-to-initarg\n\t|eieio-barf-if-slot-unbound|eieio-browse|eieio-c3-candidate|eieio-c3-merge-lists|eieio-class-children-fast|eieio-class-children\n\t|eieio-class-name|eieio-class-parent|eieio-class-parents-fast|eieio-class-parents|eieio-class-precedence-bfs|eieio-class-precedence-c3\n\t|eieio-class-precedence-dfs|eieio-class-precedence-list|eieio-class-slot-name-index|eieio-class-un-autoload|eieio-copy-parents-into-subclass\n\t|eieio-custom-mode|eieio-custom-object-apply-reset|eieio-custom-toggle-hide|eieio-custom-toggle-parent|eieio-custom-widget-insert\n\t|eieio-customize-object-group|eieio-customize-object|eieio-default-eval-maybe|eieio-default-superclass-child-p|eieio-default-superclass-list-p\n\t|eieio-default-superclass-p|eieio-default-superclass|eieio-defclass-autoload|eieio-defclass|eieio-defgeneric-form-primary-only-one\n\t|eieio-defgeneric-form-primary-only|eieio-defgeneric-form|eieio-defgeneric-reset-generic-form-primary-only-one|eieio-defgeneric-reset-generic-form-primary-only\n\t|eieio-defgeneric-reset-generic-form|eieio-defgeneric|eieio-defmethod|eieio-done-customizing|eieio-edebug-prin1-to-string|eieio-eval-default-p\n\t|eieio-filter-slot-type|eieio-generic-call-primary-only|eieio-generic-call|eieio-generic-form|eieio-help-class|eieio-help-constructor\n\t|eieio-help-generic|eieio-initarg-to-attribute|eieio-instance-inheritor-child-p|eieio-instance-inheritor-list-p|eieio-instance-inheritor-p\n\t|eieio-instance-inheritor-slot-boundp|eieio-instance-inheritor|eieio-instance-tracker-child-p|eieio-instance-tracker-find|eieio-instance-tracker-list-p\n\t|eieio-instance-tracker-p|eieio-instance-tracker|eieio-list-prin1|eieio-named-child-p|eieio-named-list-p|eieio-named-p|eieio-named\n\t|eieio-object-abstract-to-value|eieio-object-class-name|eieio-object-class|eieio-object-match|eieio-object-name-string|eieio-object-name\n\t|eieio-object-p|eieio-object-set-name-string|eieio-object-value-create|eieio-object-value-get|eieio-object-value-to-abstract|eieio-oref-default\n\t|eieio-oref|eieio-oset-default|eieio-oset|eieio-override-prin1|eieio-perform-slot-validation-for-default|eieio-perform-slot-validation\n\t|eieio-persistent-child-p|eieio-persistent-convert-list-to-object|eieio-persistent-list-p|eieio-persistent-p|eieio-persistent-path-relative\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t eieio-persistent-read|eieio-persistent-save-interactive|eieio-persistent-save|eieio-persistent-slot-type-is-class-p|eieio-persistent-validate\\/fix-slot-value\n\t|eieio-persistent|eieio-read-customization-group|eieio-set-defaults|eieio-singleton-child-p|eieio-singleton-list-p|eieio-singleton-p\n\t|eieio-singleton|eieio-slot-name-index|eieio-slot-originating-class-p|eieio-slot-value-create|eieio-slot-value-get|eieio-specialized-key-to-generic-key\n\t|eieio-speedbar-buttons|eieio-speedbar-child-description|eieio-speedbar-child-make-tag-lines|eieio-speedbar-child-p|eieio-speedbar-create-engine\n\t|eieio-speedbar-create|eieio-speedbar-customize-line|eieio-speedbar-derive-line-path|eieio-speedbar-description|eieio-speedbar-directory-button-child-p\n\t|eieio-speedbar-directory-button-list-p|eieio-speedbar-directory-button-p|eieio-speedbar-directory-button|eieio-speedbar-expand\n\t|eieio-speedbar-file-button-child-p|eieio-speedbar-file-button-list-p|eieio-speedbar-file-button-p|eieio-speedbar-file-button\n\t|eieio-speedbar-find-nearest-object|eieio-speedbar-handle-click|eieio-speedbar-item-info|eieio-speedbar-line-path|eieio-speedbar-list-p\n\t|eieio-speedbar-make-map|eieio-speedbar-make-tag-line|eieio-speedbar-object-buttonname|eieio-speedbar-object-children|eieio-speedbar-object-click\n\t|eieio-speedbar-object-expand|eieio-speedbar-p|eieio-speedbar|eieio-unbind-method-implementations|eieio-validate-class-slot-value\n\t|eieio-validate-slot-value|eieio-version|eieio-widget-test-class-child-p|eieio-widget-test-class-list-p|eieio-widget-test-class-p\n\t|eieio-widget-test-class|eieiomt-add|eieiomt-install|eieiomt-method-list|eieiomt-next|eieiomt-sym-optimize|eighth|eldoc--message-command-p\n\t|eldoc-add-command-completions|eldoc-add-command|eldoc-display-message-no-interference-p|eldoc-display-message-p|eldoc-edit-message-commands\n\t|eldoc-message|eldoc-minibuffer-message|eldoc-mode|eldoc-pre-command-refresh-echo-area|eldoc-print-current-symbol-info|eldoc-remove-command-completions\n\t|eldoc-remove-command|eldoc-schedule-timer|electric--after-char-pos|electric--sort-post-self-insertion-hook|electric-apropos|electric-buffer-list\n\t|electric-buffer-menu-looper|electric-buffer-menu-mode|electric-buffer-update-highlight|electric-command-apropos|electric-describe-bindings\n\t|electric-describe-function|electric-describe-key|electric-describe-mode|electric-describe-syntax|electric-describe-variable|electric-help-command-loop\n\t|electric-help-ctrl-x-prefix|electric-help-execute-extended|electric-help-exit|electric-help-help|electric-help-mode|electric-help-retain\n\t|electric-help-undefined|electric-helpify|electric-icon-brace|electric-indent-just-newline|electric-indent-local-mode|electric-indent-mode\n\t|electric-indent-post-self-insert-function|electric-layout-mode|electric-layout-post-self-insert-function|electric-newline-and-maybe-indent\n\t|electric-nroff-mode|electric-nroff-newline|electric-pair-mode|electric-pascal-colon|electric-pascal-equal|electric-pascal-hash\n\t|electric-pascal-semi-or-dot|electric-pascal-tab|electric-pascal-terminate-line|electric-perl-terminator|electric-verilog-backward-sexp\n\t|electric-verilog-colon|electric-verilog-forward-sexp|electric-verilog-semi-with-comment|electric-verilog-semi|electric-verilog-tab\n\t|electric-verilog-terminate-and-indent|electric-verilog-terminate-line|electric-verilog-tick|electric-view-lossage|el-get[-\\w]*|elide-head-show\n\t|elide-head|elint-add-required-env|elint-check-cond-form|elint-check-condition-case-form|elint-check-conditional-form|elint-check-defalias-form\n\t|elint-check-defcustom-form|elint-check-defun-form|elint-check-defvar-form|elint-check-function-form|elint-check-let-form|elint-check-macro-form\n\t|elint-check-quote-form|elint-check-setq-form|elint-clear-log|elint-current-buffer|elint-defun|elint-directory|elint-display-log\n\t|elint-env-add-env|elint-env-add-func|elint-env-add-global-var|elint-env-add-macro|elint-env-add-var|elint-env-find-func|elint-env-find-var\n\t|elint-env-macro-env|elint-env-macrop|elint-error|elint-file|elint-find-args-in-code|elint-find-autoloaded-variables|elint-find-builtin-args\n\t|elint-find-builtins|elint-find-next-top-form|elint-form|elint-forms|elint-get-args|elint-get-log-buffer|elint-get-top-forms|elint-init-env\n\t|elint-init-form|elint-initialize|elint-log-message|elint-log|elint-make-env|elint-make-top-form|elint-match-args|elint-output|elint-put-function-args\n\t|elint-scan-doc-file|elint-set-mode-line|elint-top-form-form|elint-top-form-pos|elint-top-form|elint-unbound-variable|elint-update-env\n\t|elint-warning|elisp--beginning-of-sexp|elisp--byte-code-comment|elisp--company-doc-buffer|elisp--company-doc-string|elisp--company-location\n\t|elisp--current-symbol|elisp--docstring-first-line|elisp--docstring-format-sym-doc|elisp--eval-defun-1|elisp--eval-defun|elisp--eval-last-sexp-print-value\n\t|elisp--eval-last-sexp|elisp--expect-function-p|elisp--fnsym-in-current-sexp|elisp--form-quoted-p|elisp--function-argstring|elisp--get-fnsym-args-string\n\t|elisp--get-var-docstring|elisp--highlight-function-argument|elisp--last-data-store|elisp--local-variables-1|elisp--local-variables\n\t|elisp--preceding-sexp|elisp--xref-find-apropos|elisp--xref-find-definitions|elisp--xref-identifier-completion-table|elisp--xref-identifier-file\n\t|elisp-byte-code-mode|elisp-byte-code-syntax-propertize|elisp-completion-at-point|elisp-eldoc-documentation-function|elisp-index-search\n\t|elisp-last-sexp-toggle-display|elisp-xref-find|elp--instrumented-p|elp--make-wrapper|elp-elapsed-time|elp-instrument-function\n\t|elp-instrument-list|elp-instrument-package|elp-output-insert-symname|elp-output-result|elp-pack-number|elp-profilable-p|elp-reset-all\n\t|elp-reset-function|elp-reset-list|elp-restore-all|elp-restore-function|elp-restore-list|elp-results-jump-to-definition|elp-results\n\t|elp-set-master|elp-sort-by-average-time|elp-sort-by-call-count|elp-sort-by-total-time|elp-unload-function|elp-unset-master|emacs-bzr-get-version\n\t|emacs-bzr-version-bzr|emacs-bzr-version-dirstate|emacs-index-search|emacs-lisp-byte-compile-and-load|emacs-lisp-byte-compile\n\t|emacs-lisp-macroexpand|emacs-lisp-mode|emacs-lock--can-auto-unlock|emacs-lock--exit-locked-buffer|emacs-lock--kill-buffer-query-functions\n\t|emacs-lock--kill-emacs-hook|emacs-lock--kill-emacs-query-functions|emacs-lock--set-mode|emacs-lock-live-process-p|emacs-lock-mode\n\t|emacs-lock-unload-function|emacs-repository-get-version|emacs-session-filename|emacs-session-save|emerge-abort|emerge-auto-advance\n\t|emerge-buffers-with-ancestor|emerge-buffers|emerge-combine-versions-edit|emerge-combine-versions-internal|emerge-combine-versions-register\n\t|emerge-combine-versions|emerge-command-exit|emerge-compare-buffers|emerge-convert-diffs-to-markers|emerge-copy-as-kill-A|emerge-copy-as-kill-B\n\t|emerge-copy-modes|emerge-count-matches-string|emerge-default-A|emerge-default-B|emerge-define-key-if-possible|emerge-defvar-local\n\t|emerge-edit-mode|emerge-execute-line|emerge-extract-diffs|emerge-extract-diffs3|emerge-fast-mode|emerge-file-names|emerge-files-command\n\t|emerge-files-exit|emerge-files-internal|emerge-files-remote|emerge-files-with-ancestor-command|emerge-files-with-ancestor-internal\n\t|emerge-files-with-ancestor-remote|emerge-files-with-ancestor|emerge-files|emerge-find-difference-A|emerge-find-difference-B|emerge-find-difference-merge\n\t|emerge-find-difference|emerge-find-difference1|emerge-force-define-key|emerge-get-diff3-group|emerge-goto-line|emerge-handle-local-variables\n\t|emerge-hash-string-into-string|emerge-insert-A|emerge-insert-B|emerge-join-differences|emerge-jump-to-difference|emerge-line-number-in-buf\n\t|emerge-line-numbers|emerge-make-auto-save-file-name|emerge-make-diff-list|emerge-make-diff3-list|emerge-make-temp-file|emerge-mark-difference\n\t|emerge-merge-directories|emerge-mode|emerge-new-flags|emerge-next-difference|emerge-one-line-window|emerge-operate-on-windows\n\t|emerge-place-flags-in-buffer|emerge-place-flags-in-buffer1|emerge-position-region|emerge-prepare-error-list|emerge-previous-difference\n\t|emerge-protect-metachars|emerge-query-and-call|emerge-query-save-buffer|emerge-query-write-file|emerge-quit|emerge-read-file-name\n\t|emerge-really-quit|emerge-recenter|emerge-refresh-mode-line|emerge-remember-buffer-characteristics|emerge-remote-exit|emerge-remove-flags-in-buffer\n\t|emerge-restore-buffer-characteristics|emerge-restore-variables|emerge-revision-with-ancestor-internal|emerge-revisions-internal\n\t|emerge-revisions-with-ancestor|emerge-revisions|emerge-save-variables|emerge-scroll-down|emerge-scroll-left|emerge-scroll-reset\n\t|emerge-scroll-right|emerge-scroll-up|emerge-select-A-edit|emerge-select-A|emerge-select-B-edit|emerge-select-B|emerge-select-difference\n\t|emerge-select-prefer-Bs|emerge-select-version|emerge-set-combine-template|emerge-set-combine-versions-template|emerge-set-keys\n\t|emerge-set-merge-mode|emerge-setup-fixed-keymaps|emerge-setup-windows|emerge-setup-with-ancestor|emerge-setup|emerge-show-file-name\n\t|emerge-skip-prefers|emerge-split-difference|emerge-trim-difference|emerge-unique-buffer-name|emerge-unselect-and-select-difference\n\t|emerge-unselect-difference|emerge-unslashify-name|emerge-validate-difference|emerge-verify-file-buffer|emerge-write-and-delete\n\t|en\\/disable-command|enable-flow-control-on|enable-flow-control|encode-big5-char|encode-coding-char|encode-composition-components\n\t|encode-composition-rule|encode-hex-string|encode-hz-buffer|encode-hz-region|encode-sjis-char|encode-time-value|encoded-string-description\n\t|end-kbd-macro|end-of-buffer-other-window|end-of-icon-defun|end-of-paragraph-text|end-of-sexp|end-of-thing|end-of-visible-line|end-of-visual-line\n\t|endp|enlarge-window-horizontally|enlarge-window|enriched-after-change-major-mode|enriched-before-change-major-mode|enriched-decode-background\n\t|enriched-decode-display-prop|enriched-decode-foreground|enriched-decode|enriched-encode-other-face|enriched-encode|enriched-face-ans\n\t|enriched-get-file-width|enriched-handle-display-prop|enriched-insert-indentation|enriched-make-annotation|enriched-map-property-regions\n\t|enriched-mode-map|enriched-mode|enriched-next-annotation|enriched-remove-header|epa--decode-coding-string|epa--derived-mode-p\n\t|epa--encode-coding-string|epa--find-coding-system-for-mime-charset|epa--insert-keys|epa--key-list-revert-buffer|epa--key-widget-action\n\t|epa--key-widget-button-face-get|epa--key-widget-help-echo|epa--key-widget-value-create|epa--list-keys|epa--marked-keys|epa--read-signature-type\n\t|epa--select-keys|epa--select-safe-coding-system|epa--show-key|epa-decrypt-armor-in-region|epa-decrypt-file|epa-decrypt-region\n\t|epa-delete-keys|epa-dired-do-decrypt|epa-dired-do-encrypt|epa-dired-do-sign|epa-dired-do-verify|epa-display-error|epa-display-info\n\t|epa-display-verify-result|epa-encrypt-file|epa-encrypt-region|epa-exit-buffer|epa-export-keys|epa-file--file-name-regexp-set|epa-file-disable\n\t|epa-file-enable|epa-file-find-file-hook|epa-file-handler|epa-file-name-regexp-update|epa-global-mail-mode|epa-import-armor-in-region\n\t|epa-import-keys-region|epa-import-keys|epa-info-mode|epa-insert-keys|epa-key-list-mode|epa-key-mode|epa-list-keys|epa-list-secret-keys\n\t|epa-mail-decrypt|epa-mail-encrypt|epa-mail-import-keys|epa-mail-mode|epa-mail-sign|epa-mail-verify|epa-mark-key|epa-passphrase-callback-function\n\t|epa-progress-callback-function|epa-read-file-name|epa-select-keys|epa-sign-file|epa-sign-region|epa-unmark-key|epa-verify-cleartext-in-region\n\t|epa-verify-file|epa-verify-region|epatch-buffer|epatch|epg--args-from-sig-notations|epg--check-error-for-decrypt|epg--clear-string\n\t|epg--decode-coding-string|epg--decode-hexstring|epg--decode-percent-escape|epg--decode-quotedstring|epg--encode-coding-string\n\t|epg--gv-nreverse|epg--import-keys-1|epg--list-keys-1|epg--make-sub-key-1|epg--make-temp-file|epg--process-filter|epg--prompt-GET_BOOL-untrusted_key\\.override\n\t|epg--prompt-GET_BOOL|epg--start|epg--status-\\*SIG|epg--status-BADARMOR|epg--status-BADSIG|epg--status-DECRYPTION_FAILED|epg--status-DECRYPTION_OKAY\n\t|epg--status-DELETE_PROBLEM|epg--status-ENC_TO|epg--status-ERRSIG|epg--status-EXPKEYSIG|epg--status-EXPSIG|epg--status-GET_BOOL\n\t|epg--status-GET_HIDDEN|epg--status-GET_LINE|epg--status-GOODSIG|epg--status-IMPORTED|epg--status-IMPORT_OK|epg--status-IMPORT_PROBLEM\n\t|epg--status-IMPORT_RES|epg--status-INV_RECP|epg--status-INV_SGNR|epg--status-KEYEXPIRED|epg--status-KEYREVOKED|epg--status-KEY_CREATED\n\t|epg--status-KEY_NOT_CREATED|epg--status-NEED_PASSPHRASE|epg--status-NEED_PASSPHRASE_PIN|epg--status-NEED_PASSPHRASE_SYM|epg--status-NODATA\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            "(?x)(?<=[()]|^)(?:\n\t epg--status-NOTATION_DATA|epg--status-NOTATION_NAME|epg--status-NO_PUBKEY|epg--status-NO_RECP|epg--status-NO_SECKEY|epg--status-NO_SGNR\n\t|epg--status-POLICY_URL|epg--status-PROGRESS|epg--status-REVKEYSIG|epg--status-SIG_CREATED|epg--status-TRUST_FULLY|epg--status-TRUST_MARGINAL\n\t|epg--status-TRUST_NEVER|epg--status-TRUST_ULTIMATE|epg--status-TRUST_UNDEFINED|epg--status-UNEXPECTED|epg--status-USERID_HINT\n\t|epg--status-VALIDSIG|epg--time-from-seconds|epg-cancel|epg-check-configuration|epg-config--compare-version|epg-config--parse-version\n\t|epg-configuration|epg-context--make|epg-context-armor--cmacro|epg-context-armor|epg-context-cipher-algorithm--cmacro|epg-context-cipher-algorithm\n\t|epg-context-compress-algorithm--cmacro|epg-context-compress-algorithm|epg-context-digest-algorithm--cmacro|epg-context-digest-algorithm\n\t|epg-context-edit-callback--cmacro|epg-context-edit-callback|epg-context-error-output--cmacro|epg-context-error-output|epg-context-home-directory--cmacro\n\t|epg-context-home-directory|epg-context-include-certs--cmacro|epg-context-include-certs|epg-context-operation--cmacro|epg-context-operation\n\t|epg-context-output-file--cmacro|epg-context-output-file|epg-context-passphrase-callback--cmacro|epg-context-passphrase-callback\n\t|epg-context-pinentry-mode--cmacro|epg-context-pinentry-mode|epg-context-process--cmacro|epg-context-process|epg-context-program--cmacro\n\t|epg-context-program|epg-context-progress-callback--cmacro|epg-context-progress-callback|epg-context-protocol--cmacro|epg-context-protocol\n\t|epg-context-result--cmacro|epg-context-result-for|epg-context-result|epg-context-set-armor|epg-context-set-passphrase-callback\n\t|epg-context-set-progress-callback|epg-context-set-result-for|epg-context-set-signers|epg-context-set-textmode|epg-context-sig-notations--cmacro\n\t|epg-context-sig-notations|epg-context-signers--cmacro|epg-context-signers|epg-context-textmode--cmacro|epg-context-textmode|epg-data-file--cmacro\n\t|epg-data-file|epg-data-string--cmacro|epg-data-string|epg-decode-dn|epg-decrypt-file|epg-decrypt-string|epg-delete-keys|epg-delete-output-file\n\t|epg-dn-from-string|epg-edit-key|epg-encrypt-file|epg-encrypt-string|epg-error-to-string|epg-errors-to-string|epg-expand-group|epg-export-keys-to-file\n\t|epg-export-keys-to-string|epg-generate-key-from-file|epg-generate-key-from-string|epg-import-keys-from-file|epg-import-keys-from-server\n\t|epg-import-keys-from-string|epg-import-result-considered--cmacro|epg-import-result-considered|epg-import-result-imported--cmacro\n\t|epg-import-result-imported-rsa--cmacro|epg-import-result-imported-rsa|epg-import-result-imported|epg-import-result-imports--cmacro\n\t|epg-import-result-imports|epg-import-result-new-revocations--cmacro|epg-import-result-new-revocations|epg-import-result-new-signatures--cmacro\n\t|epg-import-result-new-signatures|epg-import-result-new-sub-keys--cmacro|epg-import-result-new-sub-keys|epg-import-result-new-user-ids--cmacro\n\t|epg-import-result-new-user-ids|epg-import-result-no-user-id--cmacro|epg-import-result-no-user-id|epg-import-result-not-imported--cmacro\n\t|epg-import-result-not-imported|epg-import-result-secret-imported--cmacro|epg-import-result-secret-imported|epg-import-result-secret-read--cmacro\n\t|epg-import-result-secret-read|epg-import-result-secret-unchanged--cmacro|epg-import-result-secret-unchanged|epg-import-result-to-string\n\t|epg-import-result-unchanged--cmacro|epg-import-result-unchanged|epg-import-status-fingerprint--cmacro|epg-import-status-fingerprint\n\t|epg-import-status-new--cmacro|epg-import-status-new|epg-import-status-reason--cmacro|epg-import-status-reason|epg-import-status-secret--cmacro\n\t|epg-import-status-secret|epg-import-status-signature--cmacro|epg-import-status-signature|epg-import-status-sub-key--cmacro|epg-import-status-sub-key\n\t|epg-import-status-user-id--cmacro|epg-import-status-user-id|epg-key-owner-trust--cmacro|epg-key-owner-trust|epg-key-signature-class--cmacro\n\t|epg-key-signature-class|epg-key-signature-creation-time--cmacro|epg-key-signature-creation-time|epg-key-signature-expiration-time--cmacro\n\t|epg-key-signature-expiration-time|epg-key-signature-exportable-p--cmacro|epg-key-signature-exportable-p|epg-key-signature-key-id--cmacro\n\t|epg-key-signature-key-id|epg-key-signature-pubkey-algorithm--cmacro|epg-key-signature-pubkey-algorithm|epg-key-signature-user-id--cmacro\n\t|epg-key-signature-user-id|epg-key-signature-validity--cmacro|epg-key-signature-validity|epg-key-sub-key-list--cmacro|epg-key-sub-key-list\n\t|epg-key-user-id-list--cmacro|epg-key-user-id-list|epg-list-keys|epg-make-context|epg-make-data-from-file--cmacro|epg-make-data-from-file\n\t|epg-make-data-from-string--cmacro|epg-make-data-from-string|epg-make-import-result--cmacro|epg-make-import-result|epg-make-import-status--cmacro\n\t|epg-make-import-status|epg-make-key--cmacro|epg-make-key-signature--cmacro|epg-make-key-signature|epg-make-key|epg-make-new-signature--cmacro\n\t|epg-make-new-signature|epg-make-sig-notation--cmacro|epg-make-sig-notation|epg-make-signature--cmacro|epg-make-signature|epg-make-sub-key--cmacro\n\t|epg-make-sub-key|epg-make-user-id--cmacro|epg-make-user-id|epg-new-signature-class--cmacro|epg-new-signature-class|epg-new-signature-creation-time--cmacro\n\t|epg-new-signature-creation-time|epg-new-signature-digest-algorithm--cmacro|epg-new-signature-digest-algorithm|epg-new-signature-fingerprint--cmacro\n\t|epg-new-signature-fingerprint|epg-new-signature-pubkey-algorithm--cmacro|epg-new-signature-pubkey-algorithm|epg-new-signature-to-string\n\t|epg-new-signature-type--cmacro|epg-new-signature-type|epg-passphrase-callback-function|epg-read-output|epg-receive-keys|epg-reset\n\t|epg-sig-notation-critical--cmacro|epg-sig-notation-critical|epg-sig-notation-human-readable--cmacro|epg-sig-notation-human-readable\n\t|epg-sig-notation-name--cmacro|epg-sig-notation-name|epg-sig-notation-value--cmacro|epg-sig-notation-value|epg-sign-file|epg-sign-keys\n\t|epg-sign-string|epg-signature-class--cmacro|epg-signature-class|epg-signature-creation-time--cmacro|epg-signature-creation-time\n\t|epg-signature-digest-algorithm--cmacro|epg-signature-digest-algorithm|epg-signature-expiration-time--cmacro|epg-signature-expiration-time\n\t|epg-signature-fingerprint--cmacro|epg-signature-fingerprint|epg-signature-key-id--cmacro|epg-signature-key-id|epg-signature-notations--cmacro\n\t|epg-signature-notations|epg-signature-pubkey-algorithm--cmacro|epg-signature-pubkey-algorithm|epg-signature-status--cmacro|epg-signature-status\n\t|epg-signature-to-string|epg-signature-validity--cmacro|epg-signature-validity|epg-signature-version--cmacro|epg-signature-version\n\t|epg-start-decrypt|epg-start-delete-keys|epg-start-edit-key|epg-start-encrypt|epg-start-export-keys|epg-start-generate-key|epg-start-import-keys\n\t|epg-start-receive-keys|epg-start-sign-keys|epg-start-sign|epg-start-verify|epg-sub-key-algorithm--cmacro|epg-sub-key-algorithm\n\t|epg-sub-key-capability--cmacro|epg-sub-key-capability|epg-sub-key-creation-time--cmacro|epg-sub-key-creation-time|epg-sub-key-expiration-time--cmacro\n\t|epg-sub-key-expiration-time|epg-sub-key-fingerprint--cmacro|epg-sub-key-fingerprint|epg-sub-key-id--cmacro|epg-sub-key-id|epg-sub-key-length--cmacro\n\t|epg-sub-key-length|epg-sub-key-secret-p--cmacro|epg-sub-key-secret-p|epg-sub-key-validity--cmacro|epg-sub-key-validity|epg-user-id-signature-list--cmacro\n\t|epg-user-id-signature-list|epg-user-id-string--cmacro|epg-user-id-string|epg-user-id-validity--cmacro|epg-user-id-validity|epg-verify-file\n\t|epg-verify-result-to-string|epg-verify-string|epg-wait-for-completion|epg-wait-for-status|equalp|erc-active-buffer|erc-add-dangerous-host\n\t|erc-add-default-channel|erc-add-entry-to-list|erc-add-fool|erc-add-keyword|erc-add-pal|erc-add-query|erc-add-scroll-to-bottom|erc-add-server-user\n\t|erc-add-timestamp|erc-add-to-input-ring|erc-all-buffer-names|erc-already-logged-in|erc-arrange-session-in-multiple-windows|erc-auto-query\n\t|erc-autoaway-mode|erc-autojoin-add|erc-autojoin-after-ident|erc-autojoin-channels-delayed|erc-autojoin-channels|erc-autojoin-disable\n\t|erc-autojoin-enable|erc-autojoin-mode|erc-autojoin-remove|erc-away-time|erc-banlist-finished|erc-banlist-store|erc-banlist-update\n\t|erc-beep-on-match|erc-beg-of-input-line|erc-bol|erc-browse-emacswiki-lisp|erc-browse-emacswiki|erc-buffer-filter|erc-buffer-list-with-nick\n\t|erc-buffer-list|erc-buffer-visible|erc-button-add-button|erc-button-add-buttons-1|erc-button-add-buttons|erc-button-add-face|erc-button-add-nickname-buttons\n\t|erc-button-beats-to-time|erc-button-click-button|erc-button-describe-symbol|erc-button-disable|erc-button-enable|erc-button-mode\n\t|erc-button-next-function|erc-button-next|erc-button-press-button|erc-button-previous|erc-button-remove-old-buttons|erc-button-setup\n\t|erc-call-hooks|erc-cancel-timer|erc-canonicalize-server-name|erc-capab-identify-mode|erc-change-user-nickname|erc-channel-begin-receiving-names\n\t|erc-channel-end-receiving-names|erc-channel-list|erc-channel-names|erc-channel-p|erc-channel-receive-names|erc-channel-user-admin--cmacro\n\t|erc-channel-user-admin-p|erc-channel-user-admin|erc-channel-user-halfop--cmacro|erc-channel-user-halfop-p|erc-channel-user-halfop\n\t|erc-channel-user-last-message-time--cmacro|erc-channel-user-last-message-time|erc-channel-user-op--cmacro|erc-channel-user-op-p\n\t|erc-channel-user-op|erc-channel-user-owner--cmacro|erc-channel-user-owner-p|erc-channel-user-owner|erc-channel-user-p--cmacro\n\t|erc-channel-user-p|erc-channel-user-voice--cmacro|erc-channel-user-voice-p|erc-channel-user-voice|erc-clear-input-ring|erc-client-info\n\t|erc-cmd-AMSG|erc-cmd-APPENDTOPIC|erc-cmd-AT|erc-cmd-AWAY|erc-cmd-BANLIST|erc-cmd-BL|erc-cmd-BYE|erc-cmd-CHANNEL|erc-cmd-CLEAR|erc-cmd-CLEARTOPIC\n\t|erc-cmd-COUNTRY|erc-cmd-CTCP|erc-cmd-DATE|erc-cmd-DCC|erc-cmd-DEOP|erc-cmd-DESCRIBE|erc-cmd-EXIT|erc-cmd-GAWAY|erc-cmd-GQ|erc-cmd-GQUIT\n\t|erc-cmd-H|erc-cmd-HELP|erc-cmd-IDLE|erc-cmd-IGNORE|erc-cmd-J|erc-cmd-JOIN|erc-cmd-KICK|erc-cmd-LASTLOG|erc-cmd-LEAVE|erc-cmd-LIST\n\t|erc-cmd-LOAD|erc-cmd-M|erc-cmd-MASSUNBAN|erc-cmd-ME'S|erc-cmd-ME|erc-cmd-MODE|erc-cmd-MSG|erc-cmd-MUB|erc-cmd-N|erc-cmd-NAMES|erc-cmd-NICK\n\t|erc-cmd-NOTICE|erc-cmd-NOTIFY|erc-cmd-OP|erc-cmd-OPS|erc-cmd-PART|erc-cmd-PING|erc-cmd-Q|erc-cmd-QUERY|erc-cmd-QUIT|erc-cmd-QUOTE\n\t|erc-cmd-RECONNECT|erc-cmd-SAY|erc-cmd-SERVER|erc-cmd-SET|erc-cmd-SIGNOFF|erc-cmd-SM|erc-cmd-SQUERY|erc-cmd-SV|erc-cmd-T|erc-cmd-TIME\n\t|erc-cmd-TOPIC|erc-cmd-UNIGNORE|erc-cmd-VAR|erc-cmd-VARIABLE|erc-cmd-WHOAMI|erc-cmd-WHOIS|erc-cmd-WHOLEFT|erc-cmd-WI|erc-cmd-WL|erc-cmd-default\n\t|erc-cmd-ezb|erc-coding-system-for-target|erc-command-indicator|erc-command-name|erc-command-no-process-p|erc-command-symbol|erc-complete-word-at-point\n\t|erc-complete-word|erc-completion-mode|erc-compute-full-name|erc-compute-nick|erc-compute-port|erc-compute-server|erc-connection-established\n\t|erc-controls-highlight|erc-controls-interpret|erc-controls-propertize|erc-controls-strip|erc-create-imenu-index|erc-ctcp-query-ACTION\n\t|erc-ctcp-query-CLIENTINFO|erc-ctcp-query-DCC|erc-ctcp-query-ECHO|erc-ctcp-query-FINGER|erc-ctcp-query-PING|erc-ctcp-query-TIME\n\t|erc-ctcp-query-USERINFO|erc-ctcp-query-VERSION|erc-ctcp-reply-CLIENTINFO|erc-ctcp-reply-ECHO|erc-ctcp-reply-FINGER|erc-ctcp-reply-PING\n\t|erc-ctcp-reply-TIME|erc-ctcp-reply-VERSION|erc-current-network|erc-current-nick-p|erc-current-nick|erc-current-time|erc-dcc-mode\n\t|erc-debug-missing-hooks|erc-decode-coding-string|erc-decode-parsed-server-response|erc-decode-string-from-target|erc-default-server-handler\n\t|erc-default-target|erc-define-catalog-entry|erc-define-catalog|erc-define-minor-mode|erc-delete-dangerous-host|erc-delete-default-channel\n\t|erc-delete-dups|erc-delete-fool|erc-delete-if|erc-delete-keyword|erc-delete-pal|erc-delete-query|erc-determine-network|erc-determine-parameters\n\t|erc-directory-writable-p|erc-display-command|erc-display-error-notice|erc-display-line-1|erc-display-line|erc-display-message-highlight\n\t|erc-display-message|erc-display-msg|erc-display-prompt|erc-display-server-message|erc-downcase|erc-echo-notice-in-active-buffer\n\t|erc-echo-notice-in-active-non-server-buffer|erc-echo-notice-in-default-buffer|erc-echo-notice-in-first-user-buffer|erc-echo-notice-in-minibuffer\n\t|erc-echo-notice-in-server-buffer|erc-echo-notice-in-target-buffer|erc-echo-notice-in-user-and-target-buffers|erc-echo-notice-in-user-buffers\n\t|erc-echo-timestamp|erc-emacs-time-to-erc-time|erc-encode-coding-string|erc-end-of-input-line|erc-ensure-channel-name|erc-error\n\t|erc-extract-command-from-line|erc-extract-nick|erc-ezb-add-session|erc-ezb-end-of-session-list|erc-ezb-get-login|erc-ezb-identify\n)(?=[\\s()]|$)",
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t erc-ezb-init-session-list|erc-ezb-initialize|erc-ezb-lookup-action|erc-ezb-notice-autodetect|erc-ezb-select-session|erc-ezb-select\n\t|erc-faces-in|erc-fill-disable|erc-fill-enable|erc-fill-mode|erc-fill-regarding-timestamp|erc-fill-static|erc-fill-variable|erc-fill\n\t|erc-find-file|erc-find-parsed-property|erc-find-script-file|erc-format-@nick|erc-format-away-status|erc-format-channel-modes|erc-format-lag-time\n\t|erc-format-message|erc-format-my-nick|erc-format-network|erc-format-nick|erc-format-privmessage|erc-format-target-and\\/or-network\n\t|erc-format-target-and\\/or-server|erc-format-target|erc-format-timestamp|erc-function-arglist|erc-generate-new-buffer-name|erc-get-arglist\n\t|erc-get-bg-color-face|erc-get-buffer-create|erc-get-buffer|erc-get-channel-mode-from-keypress|erc-get-channel-nickname-alist\n\t|erc-get-channel-nickname-list|erc-get-channel-user-list|erc-get-channel-user|erc-get-fg-color-face|erc-get-hook|erc-get-parsed-vector-nick\n\t|erc-get-parsed-vector-type|erc-get-parsed-vector|erc-get-server-nickname-alist|erc-get-server-nickname-list|erc-get-server-user\n\t|erc-get-user-mode-prefix|erc-get|erc-go-to-log-matches-buffer|erc-grab-region|erc-group-list|erc-handle-irc-url|erc-handle-login\n\t|erc-handle-parsed-server-response|erc-handle-unknown-server-response|erc-handle-user-status-change|erc-hide-current-message-p\n\t|erc-hide-fools|erc-hide-timestamps|erc-highlight-error|erc-highlight-notice|erc-identd-mode|erc-identd-start|erc-identd-stop|erc-ignored-reply-p\n\t|erc-ignored-user-p|erc-imenu-setup|erc-initialize-log-marker|erc-input-action|erc-input-message|erc-input-ring-setup|erc-insert-aligned\n\t|erc-insert-mode-command|erc-insert-timestamp-left-and-right|erc-insert-timestamp-left|erc-insert-timestamp-right|erc-invite-only-mode\n\t|erc-irccontrols-disable|erc-irccontrols-enable|erc-irccontrols-mode|erc-is-message-ctcp-and-not-action-p|erc-is-message-ctcp-p\n\t|erc-is-valid-nick-p|erc-ison-p|erc-iswitchb|erc-join-channel|erc-keep-place-disable|erc-keep-place-enable|erc-keep-place-mode|erc-keep-place\n\t|erc-kill-buffer-function|erc-kill-channel|erc-kill-input|erc-kill-query-buffers|erc-kill-server|erc-list-button|erc-list-disable\n\t|erc-list-enable|erc-list-handle-322|erc-list-insert-item|erc-list-install-322-handler|erc-list-join|erc-list-kill|erc-list-make-string\n\t|erc-list-match|erc-list-menu-mode|erc-list-menu-sort-by-column|erc-list-mode|erc-list-revert|erc-list|erc-load-irc-script-lines\n\t|erc-load-irc-script|erc-load-script|erc-log-aux|erc-log-irc-protocol|erc-log-matches-come-back|erc-log-matches-make-buffer|erc-log-matches\n\t|erc-log-mode|erc-log|erc-logging-enabled|erc-login|erc-lurker-cleanup|erc-lurker-initialize|erc-lurker-maybe-trim|erc-lurker-p|erc-lurker-update-status\n\t|erc-make-message-variable-name|erc-make-mode-line-buffer-name|erc-make-notice|erc-make-obsolete-variable|erc-make-obsolete|erc-make-read-only\n\t|erc-match-current-nick-p|erc-match-dangerous-host-p|erc-match-directed-at-fool-p|erc-match-disable|erc-match-enable|erc-match-fool-p\n\t|erc-match-keyword-p|erc-match-message|erc-match-mode|erc-match-pal-p|erc-member-if|erc-member-ignore-case|erc-menu-add|erc-menu-disable\n\t|erc-menu-enable|erc-menu-mode|erc-menu-remove|erc-menu|erc-message-english-PART|erc-message-target|erc-message-type-member|erc-message\n\t|erc-migrate-modules|erc-mode|erc-modes|erc-modified-channels-display|erc-modified-channels-object|erc-modified-channels-remove-buffer\n\t|erc-modified-channels-update|erc-move-to-prompt-disable|erc-move-to-prompt-enable|erc-move-to-prompt-mode|erc-move-to-prompt-setup\n\t|erc-move-to-prompt|erc-munge-invisibility-spec|erc-netsplit-JOIN|erc-netsplit-MODE|erc-netsplit-QUIT|erc-netsplit-disable|erc-netsplit-enable\n\t|erc-netsplit-install-message-catalogs|erc-netsplit-mode|erc-netsplit-timer|erc-network-name|erc-network|erc-networks-disable|erc-networks-enable\n\t|erc-networks-mode|erc-next-command|erc-nick-at-point|erc-nick-equal-p|erc-nick-popup|erc-nickname-in-use|erc-nickserv-identify-mode\n\t|erc-nickserv-identify|erc-noncommands-disable|erc-noncommands-enable|erc-noncommands-mode|erc-normalize-port|erc-notifications-mode\n\t|erc-notify-mode|erc-occur|erc-once-with-server-event|erc-open-server-buffer-p|erc-open-tls-stream|erc-open|erc-page-mode|erc-parse-modes\n\t|erc-parse-prefix|erc-parse-server-response|erc-parse-user|erc-part-from-channel|erc-part-reason-normal|erc-part-reason-various\n\t|erc-part-reason-zippy|erc-pcomplete-disable|erc-pcomplete-enable|erc-pcomplete-mode|erc-pcomplete|erc-pcompletions-at-point|erc-popup-input-buffer\n\t|erc-port-equal|erc-port-to-string|erc-ports-list|erc-previous-command|erc-process-away|erc-process-ctcp-query|erc-process-ctcp-reply\n\t|erc-process-input-line|erc-process-script-line|erc-process-sentinel-1|erc-process-sentinel-2|erc-process-sentinel|erc-prompt|erc-propertize\n\t|erc-put-text-properties|erc-put-text-property|erc-query-buffer-p|erc-query|erc-quit\\/part-reason-default|erc-quit-reason-normal\n\t|erc-quit-reason-various|erc-quit-reason-zippy|erc-quit-server|erc-readonly-disable|erc-readonly-enable|erc-readonly-mode|erc-remove-channel-member\n\t|erc-remove-channel-user|erc-remove-channel-users|erc-remove-current-channel-member|erc-remove-entry-from-list|erc-remove-if-not\n\t|erc-remove-server-user|erc-remove-text-properties-region|erc-remove-user|erc-replace-current-command|erc-replace-match-subexpression-in-string\n\t|erc-replace-mode|erc-replace-regexp-in-string|erc-response-p--cmacro|erc-response-p|erc-response\\.command--cmacro|erc-response\\.command-args--cmacro\n\t|erc-response\\.command-args|erc-response\\.command|erc-response\\.contents--cmacro|erc-response\\.contents|erc-response\\.sender--cmacro\n\t|erc-response\\.sender|erc-response\\.unparsed--cmacro|erc-response\\.unparsed|erc-restore-text-properties|erc-retrieve-catalog-entry\n\t|erc-ring-disable|erc-ring-enable|erc-ring-mode|erc-save-buffer-in-logs|erc-scroll-to-bottom|erc-scrolltobottom-disable|erc-scrolltobottom-enable\n\t|erc-scrolltobottom-mode|erc-sec-to-time|erc-seconds-to-string|erc-select-read-args|erc-select-startup-file|erc-select|erc-send-action\n\t|erc-send-command|erc-send-ctcp-message|erc-send-ctcp-notice|erc-send-current-line|erc-send-distinguish-noncommands|erc-send-input-line\n\t|erc-send-input|erc-send-line|erc-send-message|erc-server-001|erc-server-002|erc-server-003|erc-server-004|erc-server-005|erc-server-221\n\t|erc-server-250|erc-server-251|erc-server-252|erc-server-253|erc-server-254|erc-server-255|erc-server-256|erc-server-257|erc-server-258\n\t|erc-server-259|erc-server-265|erc-server-266|erc-server-275|erc-server-290|erc-server-301|erc-server-303|erc-server-305|erc-server-306\n\t|erc-server-307|erc-server-311|erc-server-312|erc-server-313|erc-server-314|erc-server-315|erc-server-317|erc-server-318|erc-server-319\n\t|erc-server-320|erc-server-321-message|erc-server-321|erc-server-322-message|erc-server-322|erc-server-323|erc-server-324|erc-server-328\n\t|erc-server-329|erc-server-330|erc-server-331|erc-server-332|erc-server-333|erc-server-341|erc-server-352|erc-server-353|erc-server-366\n\t|erc-server-367|erc-server-368|erc-server-369|erc-server-371|erc-server-372|erc-server-374|erc-server-375|erc-server-376|erc-server-377\n\t|erc-server-378|erc-server-379|erc-server-391|erc-server-401|erc-server-403|erc-server-404|erc-server-405|erc-server-406|erc-server-412\n\t|erc-server-421|erc-server-422|erc-server-431|erc-server-432|erc-server-433|erc-server-437|erc-server-442|erc-server-445|erc-server-446\n\t|erc-server-451|erc-server-461|erc-server-462|erc-server-463|erc-server-464|erc-server-465|erc-server-474|erc-server-475|erc-server-477\n\t|erc-server-481|erc-server-482|erc-server-483|erc-server-484|erc-server-485|erc-server-491|erc-server-501|erc-server-502|erc-server-671\n\t|erc-server-ERROR|erc-server-INVITE|erc-server-JOIN|erc-server-KICK|erc-server-MODE|erc-server-MOTD|erc-server-NICK|erc-server-NOTICE\n\t|erc-server-PART|erc-server-PING|erc-server-PONG|erc-server-PRIVMSG|erc-server-QUIT|erc-server-TOPIC|erc-server-WALLOPS|erc-server-buffer-live-p\n\t|erc-server-buffer-p|erc-server-buffer|erc-server-connect|erc-server-filter-function|erc-server-join-channel|erc-server-process-alive\n\t|erc-server-reconnect-p|erc-server-reconnect|erc-server-select|erc-server-send-ping|erc-server-send-queue|erc-server-send|erc-server-setup-periodical-ping\n\t|erc-server-user-buffers--cmacro|erc-server-user-buffers|erc-server-user-full-name--cmacro|erc-server-user-full-name|erc-server-user-host--cmacro\n\t|erc-server-user-host|erc-server-user-info--cmacro|erc-server-user-info|erc-server-user-login--cmacro|erc-server-user-login|erc-server-user-nickname--cmacro\n\t|erc-server-user-nickname|erc-server-user-p--cmacro|erc-server-user-p|erc-services-mode|erc-set-active-buffer|erc-set-channel-key\n\t|erc-set-channel-limit|erc-set-current-nick|erc-set-initial-user-mode|erc-set-modes|erc-set-network-name|erc-set-topic|erc-set-write-file-functions\n\t|erc-setup-buffer|erc-shorten-server-name|erc-show-timestamps|erc-smiley-disable|erc-smiley-enable|erc-smiley-mode|erc-smiley|erc-sort-channel-users-alphabetically\n\t|erc-sort-channel-users-by-activity|erc-sort-strings|erc-sound-mode|erc-speedbar-browser|erc-spelling-mode|erc-split-line|erc-split-multiline-safe\n\t|erc-ssl|erc-stamp-disable|erc-stamp-enable|erc-stamp-mode|erc-string-invisible-p|erc-string-no-properties|erc-string-to-emacs-time\n\t|erc-string-to-port|erc-subseq|erc-time-diff|erc-time-gt|erc-timestamp-mode|erc-timestamp-offset|erc-tls|erc-toggle-channel-mode\n\t|erc-toggle-ctcp-autoresponse|erc-toggle-debug-irc-protocol|erc-toggle-flood-control|erc-toggle-interpret-controls|erc-toggle-timestamps\n\t|erc-track-add-to-mode-line|erc-track-disable|erc-track-enable|erc-track-face-priority|erc-track-find-face|erc-track-get-active-buffer\n\t|erc-track-get-buffer-window|erc-track-minor-mode-maybe|erc-track-minor-mode|erc-track-mode|erc-track-modified-channels|erc-track-remove-from-mode-line\n\t|erc-track-shorten-names|erc-track-sort-by-activest|erc-track-sort-by-importance|erc-track-switch-buffer|erc-trim-string|erc-truncate-buffer-to-size\n\t|erc-truncate-buffer|erc-truncate-mode|erc-unique-channel-names|erc-unique-substring-1|erc-unique-substrings|erc-unmorse-disable\n\t|erc-unmorse-enable|erc-unmorse-mode|erc-unmorse|erc-unset-network-name|erc-upcase-first-word|erc-update-channel-key|erc-update-channel-limit\n\t|erc-update-channel-member|erc-update-channel-topic|erc-update-current-channel-member|erc-update-mode-line-buffer|erc-update-mode-line\n\t|erc-update-modes|erc-update-modules|erc-update-undo-list|erc-update-user-nick|erc-update-user|erc-user-input|erc-user-is-active\n\t|erc-user-spec|erc-version|erc-view-mode-enter|erc-wash-quit-reason|erc-window-configuration-change|erc-with-all-buffers-of-server\n\t|erc-with-buffer|erc-with-selected-window|erc-with-server-buffer|erc-xdcc-add-file|erc-xdcc-mode|erc|eregistry|erevision|ert--abbreviate-string\n\t|ert--activate-font-lock-keywords|ert--button-action-position|ert--ewoc-entry-expanded-p--cmacro|ert--ewoc-entry-expanded-p|ert--ewoc-entry-extended-printer-limits-p--cmacro\n\t|ert--ewoc-entry-extended-printer-limits-p|ert--ewoc-entry-hidden-p--cmacro|ert--ewoc-entry-hidden-p|ert--ewoc-entry-p--cmacro\n\t|ert--ewoc-entry-p|ert--ewoc-entry-test--cmacro|ert--ewoc-entry-test|ert--ewoc-position|ert--expand-should-1|ert--expand-should\n\t|ert--explain-equal-including-properties|ert--explain-equal-rec|ert--explain-equal|ert--explain-format-atom|ert--force-message-log-buffer-truncation\n\t|ert--format-time-iso8601|ert--insert-human-readable-selector|ert--insert-infos|ert--make-stats|ert--make-xrefs-region|ert--parse-keys-and-body\n\t|ert--plist-difference-explanation|ert--pp-with-indentation-and-newline|ert--print-backtrace|ert--print-test-for-ewoc|ert--proper-list-p\n\t|ert--record-backtrace|ert--remove-from-list|ert--results-expand-collapse-button-action|ert--results-font-lock-function|ert--results-format-expected-unexpected\n\t|ert--results-move|ert--results-progress-bar-button-action|ert--results-test-at-point-allow-redefinition|ert--results-test-at-point-no-redefinition\n\t|ert--results-test-node-at-point|ert--results-test-node-or-null-at-point|ert--results-update-after-test-redefinition|ert--results-update-ewoc-hf\n\t|ert--results-update-stats-display-maybe|ert--results-update-stats-display|ert--run-test-debugger|ert--run-test-internal|ert--setup-results-buffer\n\t|ert--should-error-handle-error|ert--signal-should-execution|ert--significant-plist-keys|ert--skip-unless|ert--special-operator-p\n\t|ert--stats-aborted-p--cmacro|ert--stats-aborted-p|ert--stats-current-test--cmacro|ert--stats-current-test|ert--stats-end-time--cmacro\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t ert--stats-end-time|ert--stats-failed-expected--cmacro|ert--stats-failed-expected|ert--stats-failed-unexpected--cmacro|ert--stats-failed-unexpected\n\t|ert--stats-next-redisplay--cmacro|ert--stats-next-redisplay|ert--stats-p--cmacro|ert--stats-p|ert--stats-passed-expected--cmacro\n\t|ert--stats-passed-expected|ert--stats-passed-unexpected--cmacro|ert--stats-passed-unexpected|ert--stats-selector--cmacro|ert--stats-selector\n\t|ert--stats-set-test-and-result|ert--stats-skipped--cmacro|ert--stats-skipped|ert--stats-start-time--cmacro|ert--stats-start-time\n\t|ert--stats-test-end-times--cmacro|ert--stats-test-end-times|ert--stats-test-key|ert--stats-test-map--cmacro|ert--stats-test-map\n\t|ert--stats-test-pos|ert--stats-test-results--cmacro|ert--stats-test-results|ert--stats-test-start-times--cmacro|ert--stats-test-start-times\n\t|ert--stats-tests--cmacro|ert--stats-tests|ert--string-first-line|ert--test-execution-info-ert-debug-on-error--cmacro|ert--test-execution-info-ert-debug-on-error\n\t|ert--test-execution-info-exit-continuation--cmacro|ert--test-execution-info-exit-continuation|ert--test-execution-info-next-debugger--cmacro\n\t|ert--test-execution-info-next-debugger|ert--test-execution-info-p--cmacro|ert--test-execution-info-p|ert--test-execution-info-result--cmacro\n\t|ert--test-execution-info-result|ert--test-execution-info-test--cmacro|ert--test-execution-info-test|ert--test-name-button-action\n\t|ert--tests-running-mode-line-indicator|ert--unload-function|ert-char-for-test-result|ert-deftest|ert-delete-all-tests|ert-delete-test\n\t|ert-describe-test|ert-equal-including-properties|ert-face-for-stats|ert-face-for-test-result|ert-fail|ert-find-test-other-window\n\t|ert-get-test|ert-info|ert-insert-test-name-button|ert-kill-all-test-buffers|ert-make-test-unbound|ert-pass|ert-read-test-name-at-point\n\t|ert-read-test-name|ert-results-describe-test-at-point|ert-results-find-test-at-point-other-window|ert-results-jump-between-summary-and-result\n\t|ert-results-mode-menu|ert-results-mode|ert-results-next-test|ert-results-pop-to-backtrace-for-test-at-point|ert-results-pop-to-messages-for-test-at-point\n\t|ert-results-pop-to-should-forms-for-test-at-point|ert-results-pop-to-timings|ert-results-previous-test|ert-results-rerun-all-tests\n\t|ert-results-rerun-test-at-point-debugging-errors|ert-results-rerun-test-at-point|ert-results-toggle-printer-limits-for-test-at-point\n\t|ert-run-or-rerun-test|ert-run-test|ert-run-tests-batch-and-exit|ert-run-tests-batch|ert-run-tests-interactively|ert-run-tests\n\t|ert-running-test|ert-select-tests|ert-set-test|ert-simple-view-mode|ert-skip|ert-stats-completed-expected|ert-stats-completed-unexpected\n\t|ert-stats-completed|ert-stats-skipped|ert-stats-total|ert-string-for-test-result|ert-summarize-tests-batch-and-exit|ert-test-aborted-with-non-local-exit-messages--cmacro\n\t|ert-test-aborted-with-non-local-exit-messages|ert-test-aborted-with-non-local-exit-p--cmacro|ert-test-aborted-with-non-local-exit-p\n\t|ert-test-aborted-with-non-local-exit-should-forms--cmacro|ert-test-aborted-with-non-local-exit-should-forms|ert-test-at-point\n\t|ert-test-body--cmacro|ert-test-body|ert-test-boundp|ert-test-documentation--cmacro|ert-test-documentation|ert-test-expected-result-type--cmacro\n\t|ert-test-expected-result-type|ert-test-failed-backtrace--cmacro|ert-test-failed-backtrace|ert-test-failed-condition--cmacro\n\t|ert-test-failed-condition|ert-test-failed-infos--cmacro|ert-test-failed-infos|ert-test-failed-messages--cmacro|ert-test-failed-messages\n\t|ert-test-failed-p--cmacro|ert-test-failed-p|ert-test-failed-should-forms--cmacro|ert-test-failed-should-forms|ert-test-most-recent-result--cmacro\n\t|ert-test-most-recent-result|ert-test-name--cmacro|ert-test-name|ert-test-p--cmacro|ert-test-p|ert-test-passed-messages--cmacro\n\t|ert-test-passed-messages|ert-test-passed-p--cmacro|ert-test-passed-p|ert-test-passed-should-forms--cmacro|ert-test-passed-should-forms\n\t|ert-test-quit-backtrace--cmacro|ert-test-quit-backtrace|ert-test-quit-condition--cmacro|ert-test-quit-condition|ert-test-quit-infos--cmacro\n\t|ert-test-quit-infos|ert-test-quit-messages--cmacro|ert-test-quit-messages|ert-test-quit-p--cmacro|ert-test-quit-p|ert-test-quit-should-forms--cmacro\n\t|ert-test-quit-should-forms|ert-test-result-expected-p|ert-test-result-messages--cmacro|ert-test-result-messages|ert-test-result-p--cmacro\n\t|ert-test-result-p|ert-test-result-should-forms--cmacro|ert-test-result-should-forms|ert-test-result-type-p|ert-test-result-with-condition-backtrace--cmacro\n\t|ert-test-result-with-condition-backtrace|ert-test-result-with-condition-condition--cmacro|ert-test-result-with-condition-condition\n\t|ert-test-result-with-condition-infos--cmacro|ert-test-result-with-condition-infos|ert-test-result-with-condition-messages--cmacro\n\t|ert-test-result-with-condition-messages|ert-test-result-with-condition-p--cmacro|ert-test-result-with-condition-p|ert-test-result-with-condition-should-forms--cmacro\n\t|ert-test-result-with-condition-should-forms|ert-test-skipped-backtrace--cmacro|ert-test-skipped-backtrace|ert-test-skipped-condition--cmacro\n\t|ert-test-skipped-condition|ert-test-skipped-infos--cmacro|ert-test-skipped-infos|ert-test-skipped-messages--cmacro|ert-test-skipped-messages\n\t|ert-test-skipped-p--cmacro|ert-test-skipped-p|ert-test-skipped-should-forms--cmacro|ert-test-skipped-should-forms|ert-test-tags--cmacro\n\t|ert-test-tags|ert|eshell\\/addpath|eshell\\/define|eshell\\/env|eshell\\/eshell-debug|eshell\\/exit|eshell\\/export|eshell\\/jobs\n\t|eshell\\/kill|eshell\\/setq|eshell\\/unset|eshell\\/wait|eshell\\/which|eshell--apply-redirections|eshell--do-opts|eshell--process-args\n\t|eshell--process-option|eshell--set-option|eshell-add-to-window-buffer-names|eshell-apply\\*|eshell-apply-indices|eshell-apply\n\t|eshell-applyn|eshell-arg-delimiter|eshell-arg-initialize|eshell-as-subcommand|eshell-backward-argument|eshell-begin-on-new-line\n\t|eshell-beginning-of-input|eshell-beginning-of-output|eshell-bol|eshell-buffered-print|eshell-clipboard-append|eshell-close-handles\n\t|eshell-close-target|eshell-cmd-initialize|eshell-command-finished|eshell-command-result|eshell-command-started|eshell-command-to-value\n\t|eshell-command|eshell-commands|eshell-complete-lisp-symbols|eshell-complete-variable-assignment|eshell-complete-variable-reference\n\t|eshell-condition-case|eshell-convert|eshell-copy-environment|eshell-copy-handles|eshell-copy-old-input|eshell-copy-tree|eshell-create-handles\n\t|eshell-current-ange-uids|eshell-debug-command|eshell-debug-show-parsed-args|eshell-directory-files-and-attributes|eshell-directory-files\n\t|eshell-do-command-to-value|eshell-do-eval|eshell-do-pipelines-synchronously|eshell-do-pipelines|eshell-do-subjob|eshell-end-of-output\n\t|eshell-environment-variables|eshell-envvar-names|eshell-error|eshell-errorn|eshell-escape-arg|eshell-eval\\*|eshell-eval-command\n\t|eshell-eval-using-options|eshell-eval|eshell-evaln|eshell-exec-lisp|eshell-execute-pipeline|eshell-exit-success-p|eshell-explicit-command\n\t|eshell-ext-initialize|eshell-external-command|eshell-file-attributes|eshell-find-alias-function|eshell-find-delimiter|eshell-find-interpreter\n\t|eshell-find-tag|eshell-finish-arg|eshell-flatten-and-stringify|eshell-flatten-list|eshell-flush|eshell-for|eshell-forward-argument\n\t|eshell-funcall\\*|eshell-funcall|eshell-funcalln|eshell-gather-process-output|eshell-get-old-input|eshell-get-target|eshell-get-variable\n\t|eshell-goto-input-start|eshell-group-id|eshell-group-name|eshell-handle-ansi-color|eshell-handle-control-codes|eshell-handle-local-variables\n\t|eshell-index-value|eshell-init-print-buffer|eshell-insert-buffer-name|eshell-insert-envvar|eshell-insert-process|eshell-insertion-filter\n\t|eshell-interactive-output-p|eshell-interactive-print|eshell-interactive-process|eshell-intercept-commands|eshell-interpolate-variable\n\t|eshell-interrupt-process|eshell-invoke-batch-file|eshell-invoke-directly|eshell-invokify-arg|eshell-io-initialize|eshell-kill-append\n\t|eshell-kill-buffer-function|eshell-kill-input|eshell-kill-new|eshell-kill-output|eshell-kill-process-function|eshell-kill-process\n\t|eshell-life-is-too-much|eshell-lisp-command\\*|eshell-lisp-command|eshell-looking-at-backslash-return|eshell-make-private-directory\n\t|eshell-manipulate|eshell-mark-output|eshell-mode|eshell-move-argument|eshell-named-command\\*|eshell-named-command|eshell-needs-pipe-p\n\t|eshell-no-command-conversion|eshell-operator|eshell-output-filter|eshell-output-object-to-target|eshell-output-object|eshell-parse-ange-ls\n\t|eshell-parse-argument|eshell-parse-arguments|eshell-parse-backslash|eshell-parse-colon-path|eshell-parse-command-input|eshell-parse-command\n\t|eshell-parse-delimiter|eshell-parse-double-quote|eshell-parse-indices|eshell-parse-lisp-argument|eshell-parse-literal-quote|eshell-parse-pipeline\n\t|eshell-parse-redirection|eshell-parse-special-reference|eshell-parse-subcommand-argument|eshell-parse-variable-ref|eshell-parse-variable\n\t|eshell-plain-command|eshell-postoutput-scroll-to-bottom|eshell-preinput-scroll-to-bottom|eshell-print|eshell-printable-size|eshell-printn\n\t|eshell-proc-initialize|eshell-process-identity|eshell-process-interact|eshell-processp|eshell-protect-handles|eshell-protect|eshell-push-command-mark\n\t|eshell-query-kill-processes|eshell-queue-input|eshell-quit-process|eshell-quote-argument|eshell-quote-backslash|eshell-read-group-names\n\t|eshell-read-host-names|eshell-read-hosts-file|eshell-read-hosts|eshell-read-passwd-file|eshell-read-passwd|eshell-read-process-name\n\t|eshell-read-user-names|eshell-record-process-object|eshell-redisplay|eshell-regexp-arg|eshell-remote-command|eshell-remove-from-window-buffer-names\n\t|eshell-remove-process-entry|eshell-repeat-argument|eshell-report-bug|eshell-reset-after-proc|eshell-reset|eshell-resolve-current-argument\n\t|eshell-resume-command|eshell-resume-eval|eshell-return-exits-minibuffer|eshell-rewrite-for-command|eshell-rewrite-if-command\n\t|eshell-rewrite-initial-subcommand|eshell-rewrite-named-command|eshell-rewrite-sexp-command|eshell-rewrite-while-command|eshell-round-robin-kill\n\t|eshell-run-output-filters|eshell-script-interpreter|eshell-search-path|eshell-self-insert-command|eshell-send-eof-to-process\n\t|eshell-send-input|eshell-send-invisible|eshell-sentinel|eshell-separate-commands|eshell-set-output-handle|eshell-show-maximum-output\n\t|eshell-show-output|eshell-show-usage|eshell-split-path|eshell-stringify-list|eshell-stringify|eshell-strip-redirections|eshell-structure-basic-command\n\t|eshell-subcommand-arg-values|eshell-subgroups|eshell-sublist|eshell-substring|eshell-to-flat-string|eshell-toggle-direct-send\n\t|eshell-trap-errors|eshell-truncate-buffer|eshell-under-windows-p|eshell-uniqify-list|eshell-unload-all-modules|eshell-unload-extension-modules\n\t|eshell-update-markers|eshell-user-id|eshell-user-name|eshell-using-module|eshell-var-initialize|eshell-variables-list|eshell-wait-for-process\n\t|eshell-watch-for-password-prompt|eshell-winnow-list|eshell-with-file-modes|eshell-with-private-file-modes|eshell|etags--xref-find-definitions\n\t|etags-file-of-tag|etags-goto-tag-location|etags-list-tags|etags-recognize-tags-table|etags-snarf-tag|etags-tags-apropos-additional\n\t|etags-tags-apropos|etags-tags-completion-table|etags-tags-included-tables|etags-tags-table-files|etags-verify-tags-table|etags-xref-find\n\t|ethio-composition-function|ethio-fidel-to-java-buffer|ethio-fidel-to-sera-buffer|ethio-fidel-to-sera-marker|ethio-fidel-to-sera-region\n\t|ethio-fidel-to-tex-buffer|ethio-find-file|ethio-input-special-character|ethio-insert-ethio-space|ethio-java-to-fidel-buffer|ethio-modify-vowel\n\t|ethio-replace-space|ethio-sera-to-fidel-buffer|ethio-sera-to-fidel-marker|ethio-sera-to-fidel-region|ethio-tex-to-fidel-buffer\n\t|ethio-write-file|etypecase|eudc-add-field-to-records|eudc-bookmark-current-server|eudc-bookmark-server|eudc-caar|eudc-cadr|eudc-cdaar\n\t|eudc-cdar|eudc-customize|eudc-default-set|eudc-display-generic-binary|eudc-display-jpeg-as-button|eudc-display-jpeg-inline|eudc-display-mail\n\t|eudc-display-records|eudc-display-sound|eudc-display-url|eudc-distribute-field-on-records|eudc-edit-hotlist|eudc-expand-inline\n\t|eudc-extract-n-word-formats|eudc-filter-duplicate-attributes|eudc-filter-partial-records|eudc-format-attribute-name-for-display\n\t|eudc-format-query|eudc-get-attribute-list|eudc-get-email|eudc-get-phone|eudc-insert-record-at-point-into-bbdb|eudc-install-menu\n\t|eudc-lax-plist-get|eudc-load-eudc|eudc-menu|eudc-mode|eudc-move-to-next-record|eudc-move-to-previous-record|eudc-plist-get|eudc-plist-member\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t eudc-print-attribute-value|eudc-print-record-field|eudc-process-form|eudc-protocol-local-variable-p|eudc-protocol-set|eudc-query-form\n\t|eudc-query|eudc-register-protocol|eudc-replace-in-string|eudc-save-options|eudc-select|eudc-server-local-variable-p|eudc-server-set\n\t|eudc-set-server|eudc-set|eudc-tools-menu|eudc-translate-attribute-list|eudc-translate-query|eudc-try-bbdb-insert|eudc-update-local-variables\n\t|eudc-update-variable|eudc-variable-default-value|eudc-variable-protocol-value|eudc-variable-server-value|eval-after-load--anon-cmacro\n\t|eval-after-load|eval-defun|eval-expression-print-format|eval-expression|eval-last-sexp|eval-next-after-load|eval-print-last-sexp\n\t|eval-sexp-add-defvars|eval-when|evenp|event-apply-alt-modifier|event-apply-control-modifier|event-apply-hyper-modifier|event-apply-meta-modifier\n\t|event-apply-modifier|event-apply-shift-modifier|event-apply-super-modifier|every|ewoc--adjust|ewoc--buffer--cmacro|ewoc--buffer\n\t|ewoc--create--cmacro|ewoc--create|ewoc--dll--cmacro|ewoc--dll|ewoc--filter-hf-nodes|ewoc--footer--cmacro|ewoc--footer|ewoc--header--cmacro\n\t|ewoc--header|ewoc--hf-pp--cmacro|ewoc--hf-pp|ewoc--insert-new-node|ewoc--last-node--cmacro|ewoc--last-node|ewoc--node-create--cmacro\n\t|ewoc--node-create|ewoc--node-data--cmacro|ewoc--node-data|ewoc--node-left--cmacro|ewoc--node-left|ewoc--node-next|ewoc--node-nth\n\t|ewoc--node-prev|ewoc--node-right--cmacro|ewoc--node-right|ewoc--node-start-marker--cmacro|ewoc--node-start-marker|ewoc--pretty-printer--cmacro\n\t|ewoc--pretty-printer|ewoc--refresh-node|ewoc--set-buffer-bind-dll-let\\*|ewoc--set-buffer-bind-dll|ewoc--wrap|ewoc-p--cmacro|ewoc-p\n\t|eww-add-bookmark|eww-back-url|eww-beginning-of-field|eww-beginning-of-text|eww-bookmark-browse|eww-bookmark-kill|eww-bookmark-mode\n\t|eww-bookmark-prepare|eww-bookmark-yank|eww-browse-url|eww-browse-with-external-browser|eww-buffer-kill|eww-buffer-select|eww-buffer-show-next\n\t|eww-buffer-show-previous|eww-buffer-show|eww-buffers-mode|eww-change-select|eww-copy-page-url|eww-current-url|eww-desktop-data-1\n\t|eww-desktop-history-duplicate|eww-desktop-misc-data|eww-detect-charset|eww-display-html|eww-display-image|eww-display-pdf|eww-display-raw\n\t|eww-download-callback|eww-download|eww-end-of-field|eww-end-of-text|eww-follow-link|eww-form-checkbox|eww-form-file|eww-form-submit\n\t|eww-form-text|eww-forward-url|eww-handle-link|eww-highest-readability|eww-history-browse|eww-history-mode|eww-input-value|eww-inputs\n\t|eww-links-at-point|eww-list-bookmarks|eww-list-buffers|eww-list-histories|eww-make-unique-file-name|eww-mode|eww-next-bookmark\n\t|eww-next-url|eww-open-file|eww-parse-headers|eww-previous-bookmark|eww-previous-url|eww-process-text-input|eww-read-bookmarks|eww-readable\n\t|eww-reload|eww-render|eww-restore-desktop|eww-restore-history|eww-same-page-p|eww-save-history|eww-score-readability|eww-search-words\n\t|eww-select-display|eww-select-file|eww-set-character-encoding|eww-setup-buffer|eww-size-text-inputs|eww-submit|eww-suggested-uris\n\t|eww-tag-a|eww-tag-body|eww-tag-form|eww-tag-input|eww-tag-link|eww-tag-select|eww-tag-textarea|eww-tag-title|eww-toggle-checkbox\n\t|eww-top-url|eww-up-url|eww-update-field|eww-update-header-line-format|eww-view-source|eww-write-bookmarks|eww|ex-args|ex-cd|ex-cmd-accepts-multiple-files-p\n\t|ex-cmd-assoc|ex-cmd-complete|ex-cmd-execute|ex-cmd-is-mashed-with-args|ex-cmd-is-one-letter|ex-cmd-not-yet|ex-cmd-obsolete|ex-cmd-read-exit\n\t|ex-command|ex-compile|ex-copy|ex-delete|ex-edit|ex-expand-filsyms|ex-find-file|ex-fixup-history|ex-get-inline-cmd-args|ex-global|ex-goto\n\t|ex-help|ex-line-no|ex-line-subr|ex-line|ex-map-read-args|ex-map|ex-mark|ex-next-related-buffer|ex-next|ex-preserve|ex-print-display-lines\n\t|ex-print|ex-put|ex-pwd|ex-quit|ex-read|ex-recover|ex-rewind|ex-search-address|ex-set-read-variable|ex-set-visited-file-name|ex-set\n\t|ex-shell|ex-show-vars|ex-source|ex-splice-args-in-1-letr-cmd|ex-substitute|ex-tag|ex-unmap-read-args|ex-unmap|ex-write-info|ex-write\n\t|ex-yank|exchange-dot-and-mark|exchange-point-and-mark|executable-chmod|executable-command-find-posix-p|executable-interpret|executable-make-buffer-file-executable-if-script-p\n\t|executable-self-display|executable-set-magic|execute-extended-command--shorter-1|execute-extended-command--shorter|exit-scheme-interaction-mode\n\t|exit-splash-screen|expand-abbrev-from-expand|expand-abbrev-hook|expand-add-abbrev|expand-add-abbrevs|expand-build-list|expand-build-marks\n\t|expand-c-for-skeleton|expand-clear-markers|expand-do-expansion|expand-in-literal|expand-jump-to-next-slot|expand-jump-to-previous-slot\n\t|expand-list-to-markers|expand-mail-aliases|expand-previous-word|expand-region-abbrevs|expand-skeleton-end-hook|external-debugging-output\n\t|extract-rectangle-line|extract-rectangle|ezimage-all-images|ezimage-image-association-dump|ezimage-image-dump|ezimage-image-over-string\n\t|ezimage-insert-image-button-maybe|ezimage-insert-over-text|f90-abbrev-help|f90-abbrev-start|f90-add-imenu-menu|f90-backslash-not-special\n\t|f90-beginning-of-block|f90-beginning-of-subprogram|f90-block-match|f90-break-line|f90-calculate-indent|f90-capitalize-keywords\n\t|f90-capitalize-region-keywords|f90-change-keywords|f90-comment-indent|f90-comment-region|f90-current-defun|f90-current-indentation\n\t|f90-do-auto-fill|f90-downcase-keywords|f90-downcase-region-keywords|f90-electric-insert|f90-end-of-block|f90-end-of-subprogram\n\t|f90-equal-symbols|f90-fill-region|f90-find-breakpoint|f90-font-lock-1|f90-font-lock-2|f90-font-lock-3|f90-font-lock-4|f90-font-lock-n\n\t|f90-get-correct-indent|f90-get-present-comment-type|f90-imenu-type-matcher|f90-in-comment|f90-in-string|f90-indent-line-no|f90-indent-line\n\t|f90-indent-new-line|f90-indent-region|f90-indent-subprogram|f90-indent-to|f90-insert-end|f90-join-lines|f90-line-continued|f90-looking-at-associate\n\t|f90-looking-at-critical|f90-looking-at-do|f90-looking-at-end-critical|f90-looking-at-if-then|f90-looking-at-program-block-end\n\t|f90-looking-at-program-block-start|f90-looking-at-select-case|f90-looking-at-type-like|f90-looking-at-where-or-forall|f90-mark-subprogram\n\t|f90-match-end|f90-menu|f90-mode|f90-next-block|f90-next-statement|f90-no-block-limit|f90-prepare-abbrev-list-buffer|f90-present-statement-cont\n\t|f90-previous-block|f90-previous-statement|f90-typedec-matcher|f90-typedef-matcher|f90-upcase-keywords|f90-upcase-region-keywords\n\t|f90-update-line|face-at-point|face-attr-construct|face-attr-match-p|face-attribute-merged-with|face-attribute-specified-or|face-attributes-as-vector\n\t|face-attrs-more-relative-p|face-background-pixmap|face-default-spec|face-descriptive-attribute-name|face-doc-string|face-name\n\t|face-nontrivial-p|face-read-integer|face-read-string|face-remap-order|face-set-after-frame-default|face-spec-choose|face-spec-match-p\n\t|face-spec-recalc|face-spec-reset-face|face-spec-set-2|face-spec-set-match-display|face-user-default-spec|face-valid-attribute-values\n\t|facemenu-active-faces|facemenu-add-face|facemenu-add-new-color|facemenu-add-new-face|facemenu-background-menu|facemenu-color-equal\n\t|facemenu-complete-face-list|facemenu-enable-faces-p|facemenu-face-menu|facemenu-foreground-menu|facemenu-indentation-menu|facemenu-iterate\n\t|facemenu-justification-menu|facemenu-menu|facemenu-post-self-insert-function|facemenu-read-color|facemenu-remove-all|facemenu-remove-face-props\n\t|facemenu-remove-special|facemenu-set-background|facemenu-set-bold-italic|facemenu-set-bold|facemenu-set-default|facemenu-set-face-from-menu\n\t|facemenu-set-face|facemenu-set-foreground|facemenu-set-intangible|facemenu-set-invisible|facemenu-set-italic|facemenu-set-read-only\n\t|facemenu-set-self-insert-face|facemenu-set-underline|facemenu-special-menu|facemenu-update|fancy-about-screen|fancy-splash-frame\n\t|fancy-splash-head|fancy-splash-image-file|fancy-splash-insert|fancy-startup-screen|fancy-startup-tail|feature-file|feature-symbols\n\t|feedmail-accume-n-nuke-header|feedmail-buffer-to-binmail|feedmail-buffer-to-sendmail|feedmail-buffer-to-smtp|feedmail-buffer-to-smtpmail\n\t|feedmail-confirm-addresses-hook-example|feedmail-create-queue-filename|feedmail-deduce-address-list|feedmail-default-date-generator\n\t|feedmail-default-message-id-generator|feedmail-default-x-mailer-generator|feedmail-dump-message-to-queue|feedmail-envelope-deducer\n\t|feedmail-fiddle-date|feedmail-fiddle-from|feedmail-fiddle-header|feedmail-fiddle-list-of-fiddle-plexes|feedmail-fiddle-list-of-spray-fiddle-plexes\n\t|feedmail-fiddle-message-id|feedmail-fiddle-sender|feedmail-fiddle-spray-address|feedmail-fiddle-x-mailer|feedmail-fill-this-one\n\t|feedmail-fill-to-cc-function|feedmail-find-eoh|feedmail-fqm-p|feedmail-give-it-to-buffer-eater|feedmail-look-at-queue-directory\n\t|feedmail-mail-send-hook-splitter|feedmail-message-action-draft-strong|feedmail-message-action-draft|feedmail-message-action-edit\n\t|feedmail-message-action-help-blat|feedmail-message-action-help|feedmail-message-action-queue-strong|feedmail-message-action-queue\n\t|feedmail-message-action-scroll-down|feedmail-message-action-scroll-up|feedmail-message-action-send-strong|feedmail-message-action-send\n\t|feedmail-message-action-toggle-spray|feedmail-one-last-look|feedmail-queue-express-to-draft|feedmail-queue-express-to-queue\n\t|feedmail-queue-reminder-brief|feedmail-queue-reminder-medium|feedmail-queue-reminder|feedmail-queue-runner-prompt|feedmail-queue-send-edit-prompt-inner\n\t|feedmail-queue-send-edit-prompt|feedmail-queue-subject-slug-maker|feedmail-rfc822-date|feedmail-rfc822-time-zone|feedmail-run-the-queue-global-prompt\n\t|feedmail-run-the-queue-no-prompts|feedmail-run-the-queue|feedmail-say-chatter|feedmail-say-debug|feedmail-scroll-buffer|feedmail-send-it-immediately-wrapper\n\t|feedmail-send-it-immediately|feedmail-send-it|feedmail-spray-via-bbdb|feedmail-tidy-up-slug|feedmail-vm-mail-mode|fetch-overload\n\t|ff-all-dirs-under|ff-basename|ff-cc-hh-converter|ff-find-file|ff-find-other-file|ff-find-related-file|ff-find-the-other-file|ff-get-file-name\n\t|ff-get-file|ff-get-other-file|ff-list-replace-env-vars|ff-mouse-find-other-file-other-window|ff-mouse-find-other-file|ff-other-file-name\n\t|ff-set-point-accordingly|ff-string-match|ff-switch-file|ff-switch-to-buffer|ff-treat-as-special|ff-upcase-p|ff-which-function-are-we-in\n\t|ffap--toggle-read-only|ffap-all-subdirs-loop|ffap-all-subdirs|ffap-alternate-file-other-window|ffap-alternate-file|ffap-at-mouse\n\t|ffap-bib|ffap-bindings|ffap-bug|ffap-c\\+\\+-mode|ffap-c-mode|ffap-completable|ffap-copy-string-as-kill|ffap-dired-other-frame|ffap-dired-other-window\n\t|ffap-dired|ffap-el-mode|ffap-el|ffap-event-buffer|ffap-file-at-point|ffap-file-exists-string|ffap-file-remote-p|ffap-file-suffix\n\t|ffap-fixup-machine|ffap-fixup-url|ffap-fortran-mode|ffap-gnus-hook|ffap-gnus-menu|ffap-gnus-next|ffap-gnus-wrapper|ffap-gopher-at-point\n\t|ffap-guess-file-name-at-point|ffap-guesser|ffap-highlight|ffap-home|ffap-host-to-filename|ffap-info-2|ffap-info-3|ffap-info|ffap-kpathsea-expand-path\n\t|ffap-latex-mode|ffap-lcd|ffap-list-directory|ffap-list-env|ffap-literally|ffap-locate-file|ffap-machine-at-point|ffap-machine-p\n\t|ffap-menu-ask|ffap-menu-cont|ffap-menu-rescan|ffap-menu|ffap-mouse-event|ffap-newsgroup-p|ffap-next-guess|ffap-next-url|ffap-next\n\t|ffap-other-frame|ffap-other-window|ffap-prompter|ffap-read-file-or-url-internal|ffap-read-file-or-url|ffap-read-only-other-frame\n\t|ffap-read-only-other-window|ffap-read-only|ffap-read-url-internal|ffap-reduce-path|ffap-replace-file-component|ffap-rfc|ffap-ro-mode-hook\n\t|ffap-string-around|ffap-string-at-point|ffap-submit-bug|ffap-symbol-value|ffap-tex-init|ffap-tex-mode|ffap-tex|ffap-url-at-point\n\t|ffap-url-p|ffap-url-unwrap-local|ffap-url-unwrap-remote|ffap-what-domain|ffap|field-at-pos|field-complete|fifth|file-attributes-lessp\n\t|file-cache--read-list|file-cache-add-directory-list|file-cache-add-directory-recursively|file-cache-add-directory-using-find\n\t|file-cache-add-directory-using-locate|file-cache-add-directory|file-cache-add-file-list|file-cache-add-file|file-cache-add-from-file-cache-buffer\n\t|file-cache-canonical-directory|file-cache-choose-completion|file-cache-clear-cache|file-cache-complete|file-cache-completion-setup-function\n\t|file-cache-debug-read-from-minibuffer|file-cache-delete-directory-list|file-cache-delete-directory|file-cache-delete-file-list\n\t|file-cache-delete-file-regexp|file-cache-delete-file|file-cache-directory-name|file-cache-display|file-cache-do-delete-directory\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t file-cache-file-name|file-cache-files-matching-internal|file-cache-files-matching|file-cache-minibuffer-complete|file-cache-mouse-choose-completion\n\t|file-dependents|file-loadhist-lookup|file-modes-char-to-right|file-modes-char-to-who|file-modes-rights-to-number|file-name-non-special\n\t|file-name-shadow-mode|file-notify--event-cookie|file-notify--event-file-name|file-notify--event-file1-name|file-notify-callback\n\t|file-notify-handle-event|file-of-tag|file-provides|file-requires|file-set-intersect|file-size-human-readable|file-tree-walk|filesets-add-buffer\n\t|filesets-alist-get|filesets-browse-dir|filesets-browser-name|filesets-build-dir-submenu-now|filesets-build-dir-submenu|filesets-build-ingroup-submenu\n\t|filesets-build-menu-maybe|filesets-build-menu-now|filesets-build-menu|filesets-build-submenu|filesets-close|filesets-cmd-get-args\n\t|filesets-cmd-get-def|filesets-cmd-get-fn|filesets-cmd-isearch-getargs|filesets-cmd-query-replace-getargs|filesets-cmd-query-replace-regexp-getargs\n\t|filesets-cmd-shell-command-getargs|filesets-cmd-shell-command|filesets-cmd-show-result|filesets-conditional-sort|filesets-convert-path-list\n\t|filesets-convert-patterns|filesets-customize|filesets-data-get-data|filesets-data-get-name|filesets-data-get|filesets-data-set-default\n\t|filesets-data-set|filesets-directory-files|filesets-edit|filesets-entry-get-dormant-flag|filesets-entry-get-file|filesets-entry-get-files\n\t|filesets-entry-get-filter-dirs-flag|filesets-entry-get-master|filesets-entry-get-open-fn|filesets-entry-get-pattern--dir|filesets-entry-get-pattern--pattern\n\t|filesets-entry-get-pattern|filesets-entry-get-save-fn|filesets-entry-get-tree-max-level|filesets-entry-get-tree|filesets-entry-get-verbosity\n\t|filesets-entry-mode|filesets-entry-set-files|filesets-error|filesets-eviewer-constraint-p|filesets-eviewer-get-props|filesets-exit\n\t|filesets-file-close|filesets-file-open|filesets-files-equalp|filesets-files-in-same-directory-p|filesets-filetype-get-prop|filesets-filetype-property\n\t|filesets-filter-dir-names|filesets-filter-list|filesets-find-file-using|filesets-find-file|filesets-find-or-display-file|filesets-get-cmd-menu\n\t|filesets-get-external-viewer-by-name|filesets-get-external-viewer|filesets-get-filelist|filesets-get-fileset-from-name|filesets-get-fileset-name\n\t|filesets-get-menu-epilog|filesets-get-quoted-selection|filesets-get-selection|filesets-get-shortcut|filesets-goto-homepage|filesets-info\n\t|filesets-ingroup-cache-get|filesets-ingroup-cache-put|filesets-ingroup-collect-build-menu|filesets-ingroup-collect-files|filesets-ingroup-collect-finder\n\t|filesets-ingroup-collect|filesets-ingroup-get-data|filesets-ingroup-get-pattern|filesets-ingroup-get-remdupl-p|filesets-init\n\t|filesets-member|filesets-menu-cache-file-load|filesets-menu-cache-file-save-maybe|filesets-menu-cache-file-save|filesets-message\n\t|filesets-open|filesets-ormap|filesets-quote|filesets-rebuild-this-submenu|filesets-remake-shortcut|filesets-remove-buffer|filesets-remove-from-ubl\n\t|filesets-reset-filename-on-change|filesets-reset-fileset|filesets-run-cmd--repl-fn|filesets-run-cmd|filesets-save-config|filesets-select-command\n\t|filesets-set-config|filesets-set-default!|filesets-set-default\\+|filesets-set-default|filesets-some|filesets-spawn-external-viewer\n\t|filesets-sublist|filesets-update-cleanup|filesets-update-pre010505|filesets-update|filesets-which-command-p|filesets-which-command\n\t|filesets-which-file|filesets-wrap-submenu|fill-comment-paragraph|fill-common-string-prefix|fill-delete-newlines|fill-delete-prefix\n\t|fill-find-break-point|fill-flowed-encode|fill-flowed|fill-forward-paragraph|fill-french-nobreak-p|fill-indent-to-left-margin|fill-individual-paragraphs-citation\n\t|fill-individual-paragraphs-prefix|fill-match-adaptive-prefix|fill-minibuffer-function|fill-move-to-break-point|fill-newline|fill-nobreak-p\n\t|fill-nonuniform-paragraphs|fill-single-char-nobreak-p|fill-single-word-nobreak-p|fill-text-properties-at|fill|filtered-frame-list\n\t|find-alternate-file-other-window|find-alternate-file|find-change-log|find-class|find-cmd|find-cmpl-prefix-entry|find-coding-systems-region-internal\n\t|find-composition-internal|find-composition|find-definition-noselect|find-dired-filter|find-dired-sentinel|find-dired|find-emacs-lisp-shadows\n\t|find-exact-completion|find-face-definition|find-file--read-only|find-file-at-point|find-file-existing|find-file-literally-at-point\n\t|find-file-noselect-1|find-file-other-frame|find-file-read-args|find-file-read-only-other-frame|find-file-read-only-other-window\n\t|find-function-C-source|find-function-advised-original|find-function-at-point|find-function-do-it|find-function-library|find-function-noselect\n\t|find-function-on-key|find-function-other-frame|find-function-other-window|find-function-read|find-function-search-for-symbol\n\t|find-function-setup-keys|find-function|find-grep-dired|find-grep|find-if-not|find-if|find-library--load-name|find-library-name|find-library-suffixes\n\t|find-library|find-lisp-debug-message|find-lisp-default-directory-predicate|find-lisp-default-file-predicate|find-lisp-file-predicate-is-directory\n\t|find-lisp-find-dired-filter|find-lisp-find-dired-insert-file|find-lisp-find-dired-internal|find-lisp-find-dired-subdirectories\n\t|find-lisp-find-dired|find-lisp-find-files-internal|find-lisp-find-files|find-lisp-format-time|find-lisp-format|find-lisp-insert-directory\n\t|find-lisp-object-file-name|find-lisp-time-index|find-multibyte-characters|find-name-dired|find-new-buffer-file-coding-system\n\t|find-tag-default-as-regexp|find-tag-default-as-symbol-regexp|find-tag-default-bounds|find-tag-default|find-tag-in-order|find-tag-interactive\n\t|find-tag-noselect|find-tag-other-frame|find-tag-other-window|find-tag-regexp|find-tag-tag|find-tag|find-variable-at-point|find-variable-noselect\n\t|find-variable-other-frame|find-variable-other-window|find-variable|find|finder-by-keyword|finder-commentary|finder-compile-keywords-make-dist\n\t|finder-compile-keywords|finder-current-item|finder-exit|finder-goto-xref|finder-insert-at-column|finder-list-keywords|finder-list-matches\n\t|finder-mode|finder-mouse-face-on-line|finder-mouse-select|finder-select|finder-summary|finder-unknown-keywords|finder-unload-function\n\t|finger|first-error|first|floatp-safe|floor\\*|flush-lines|flymake-add-buildfile-to-cache|flymake-add-err-info|flymake-add-line-err-info\n\t|flymake-add-project-include-dirs-to-cache|flymake-after-change-function|flymake-after-save-hook|flymake-can-syntax-check-file\n\t|flymake-check-include|flymake-check-patch-master-file-buffer|flymake-clear-buildfile-cache|flymake-clear-project-include-dirs-cache\n\t|flymake-compilation-is-running|flymake-compile|flymake-copy-buffer-to-temp-buffer|flymake-create-master-file|flymake-create-temp-inplace\n\t|flymake-create-temp-with-folder-structure|flymake-delete-own-overlays|flymake-delete-temp-directory|flymake-display-err-menu-for-current-line\n\t|flymake-display-warning|flymake-er-get-line-err-info-list|flymake-er-get-line|flymake-er-make-er|flymake-find-buffer-for-file\n\t|flymake-find-buildfile|flymake-find-err-info|flymake-find-file-hook|flymake-find-make-buildfile|flymake-find-possible-master-files\n\t|flymake-fix-file-name|flymake-fix-line-numbers|flymake-get-ant-cmdline|flymake-get-buildfile-from-cache|flymake-get-cleanup-function\n\t|flymake-get-err-count|flymake-get-file-name-mode-and-masks|flymake-get-first-err-line-no|flymake-get-full-nonpatched-file-name\n\t|flymake-get-full-patched-file-name|flymake-get-include-dirs-dot|flymake-get-include-dirs|flymake-get-init-function|flymake-get-last-err-line-no\n\t|flymake-get-line-err-count|flymake-get-make-cmdline|flymake-get-next-err-line-no|flymake-get-prev-err-line-no|flymake-get-project-include-dirs-from-cache\n\t|flymake-get-project-include-dirs-imp|flymake-get-project-include-dirs|flymake-get-real-file-name-function|flymake-get-real-file-name\n\t|flymake-get-syntax-check-program-args|flymake-get-system-include-dirs|flymake-get-tex-args|flymake-goto-file-and-line|flymake-goto-line\n\t|flymake-goto-next-error|flymake-goto-prev-error|flymake-highlight-err-lines|flymake-highlight-line|flymake-init-create-temp-buffer-copy\n\t|flymake-init-create-temp-source-and-master-buffer-copy|flymake-init-find-buildfile-dir|flymake-ins-after|flymake-kill-buffer-hook\n\t|flymake-kill-process|flymake-ler-file--cmacro|flymake-ler-file|flymake-ler-full-file--cmacro|flymake-ler-full-file|flymake-ler-line--cmacro\n\t|flymake-ler-line|flymake-ler-make-ler--cmacro|flymake-ler-make-ler|flymake-ler-p--cmacro|flymake-ler-p|flymake-ler-set-file|flymake-ler-set-full-file\n\t|flymake-ler-set-line|flymake-ler-text--cmacro|flymake-ler-text|flymake-ler-type--cmacro|flymake-ler-type|flymake-line-err-info-is-less-or-equal\n\t|flymake-log|flymake-make-overlay|flymake-master-cleanup|flymake-master-file-compare|flymake-master-make-header-init|flymake-master-make-init\n\t|flymake-master-tex-init|flymake-mode-off|flymake-mode-on|flymake-mode|flymake-on-timer-event|flymake-overlay-p|flymake-parse-err-lines\n\t|flymake-parse-line|flymake-parse-output-and-residual|flymake-parse-residual|flymake-patch-err-text|flymake-perl-init|flymake-php-init\n\t|flymake-popup-current-error-menu|flymake-post-syntax-check|flymake-process-filter|flymake-process-sentinel|flymake-read-file-to-temp-buffer\n\t|flymake-reformat-err-line-patterns-from-compile-el|flymake-region-has-flymake-overlays|flymake-replace-region|flymake-report-fatal-status\n\t|flymake-report-status|flymake-safe-delete-directory|flymake-safe-delete-file|flymake-same-files|flymake-save-buffer-in-file|flymake-set-at\n\t|flymake-simple-ant-java-init|flymake-simple-cleanup|flymake-simple-java-cleanup|flymake-simple-make-init-impl|flymake-simple-make-init\n\t|flymake-simple-make-java-init|flymake-simple-tex-init|flymake-skip-whitespace|flymake-split-output|flymake-start-syntax-check-process\n\t|flymake-start-syntax-check|flymake-stop-all-syntax-checks|flymake-xml-init|flyspell-abbrev-table|flyspell-accept-buffer-local-defs\n\t|flyspell-after-change-function|flyspell-ajust-cursor-point|flyspell-already-abbrevp|flyspell-auto-correct-previous-hook|flyspell-auto-correct-previous-word\n\t|flyspell-auto-correct-word|flyspell-buffer|flyspell-change-abbrev|flyspell-check-changed-word-p|flyspell-check-pre-word-p|flyspell-check-previous-highlighted-word\n\t|flyspell-check-region-doublons|flyspell-check-word-p|flyspell-correct-word-before-point|flyspell-correct-word|flyspell-debug-signal-changed-checked\n\t|flyspell-debug-signal-no-check|flyspell-debug-signal-pre-word-checked|flyspell-debug-signal-word-checked|flyspell-define-abbrev\n\t|flyspell-delay-command|flyspell-delay-commands|flyspell-delete-all-overlays|flyspell-delete-region-overlays|flyspell-deplacement-command\n\t|flyspell-deplacement-commands|flyspell-display-next-corrections|flyspell-do-correct|flyspell-emacs-popup|flyspell-external-point-words\n\t|flyspell-generic-progmode-verify|flyspell-get-casechars|flyspell-get-not-casechars|flyspell-get-word|flyspell-goto-next-error\n\t|flyspell-hack-local-variables-hook|flyspell-highlight-duplicate-region|flyspell-highlight-incorrect-region|flyspell-kill-ispell-hook\n\t|flyspell-large-region|flyspell-math-tex-command-p|flyspell-maybe-correct-doubling|flyspell-maybe-correct-transposition|flyspell-minibuffer-p\n\t|flyspell-mode-off|flyspell-mode-on|flyspell-mode|flyspell-notify-misspell|flyspell-overlay-p|flyspell-post-command-hook|flyspell-pre-command-hook\n\t|flyspell-process-localwords|flyspell-prog-mode|flyspell-properties-at-p|flyspell-region|flyspell-small-region|flyspell-tex-command-p\n\t|flyspell-unhighlight-at|flyspell-word-search-backward|flyspell-word-search-forward|flyspell-word|flyspell-xemacs-popup|focus-frame\n\t|foldout-exit-fold|foldout-mouse-goto-heading|foldout-mouse-hide-or-exit|foldout-mouse-show|foldout-mouse-swallow-events|foldout-mouse-zoom\n\t|foldout-update-mode-line|foldout-zoom-subtree|follow--window-sorter|follow-adjust-window|follow-align-compilation-windows|follow-all-followers\n\t|follow-avoid-tail-recenter|follow-cache-valid-p|follow-calc-win-end|follow-calc-win-start|follow-calculate-first-window-start-from-above\n\t|follow-calculate-first-window-start-from-below|follow-comint-scroll-to-bottom|follow-debug-message|follow-delete-other-windows-and-split\n\t|follow-end-of-buffer|follow-estimate-first-window-start|follow-find-file-hook|follow-first-window|follow-last-window|follow-maximize-region\n\t|follow-menu-filter|follow-mode|follow-mwheel-scroll|follow-next-window|follow-point-visible-all-windows-p|follow-pos-visible|follow-post-command-hook\n\t|follow-previous-window|follow-recenter|follow-redisplay|follow-redraw-after-event|follow-redraw|follow-scroll-bar-drag|follow-scroll-bar-scroll-down\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t follow-scroll-bar-scroll-up|follow-scroll-bar-toolkit-scroll|follow-scroll-down|follow-scroll-up|follow-select-if-end-visible\n\t|follow-select-if-visible-from-first|follow-select-if-visible|follow-split-followers|follow-switch-to-buffer-all|follow-switch-to-buffer\n\t|follow-switch-to-current-buffer-all|follow-update-window-start|follow-window-size-change|follow-windows-aligned-p|follow-windows-start-end\n\t|font-get-glyphs|font-get-system-font|font-get-system-normal-font|font-info|font-lock-after-change-function|font-lock-after-fontify-buffer\n\t|font-lock-after-unfontify-buffer|font-lock-append-text-property|font-lock-apply-highlight|font-lock-apply-syntactic-highlight\n\t|font-lock-change-mode|font-lock-choose-keywords|font-lock-compile-keyword|font-lock-compile-keywords|font-lock-default-fontify-buffer\n\t|font-lock-default-fontify-region|font-lock-default-function|font-lock-default-unfontify-buffer|font-lock-default-unfontify-region\n\t|font-lock-defontify|font-lock-ensure|font-lock-eval-keywords|font-lock-extend-jit-lock-region-after-change|font-lock-extend-region-multiline\n\t|font-lock-extend-region-wholelines|font-lock-fillin-text-property|font-lock-flush|font-lock-fontify-anchored-keywords|font-lock-fontify-block\n\t|font-lock-fontify-buffer|font-lock-fontify-keywords-region|font-lock-fontify-region|font-lock-fontify-syntactic-anchored-keywords\n\t|font-lock-fontify-syntactic-keywords-region|font-lock-fontify-syntactically-region|font-lock-initial-fontify|font-lock-match-c-style-declaration-item-and-skip-to-next\n\t|font-lock-match-meta-declaration-item-and-skip-to-next|font-lock-mode-internal|font-lock-mode-set-explicitly|font-lock-mode\n\t|font-lock-prepend-text-property|font-lock-refresh-defaults|font-lock-set-defaults|font-lock-specified-p|font-lock-turn-off-thing-lock\n\t|font-lock-turn-on-thing-lock|font-lock-unfontify-buffer|font-lock-unfontify-region|font-lock-update-removed-keyword-alist|font-lock-value-in-major-mode\n\t|font-match-p|font-menu-add-default|font-setting-change-default-font|font-shape-gstring|font-show-log|font-variation-glyphs|fontset-font\n\t|fontset-info|fontset-list|fontset-name-p|fontset-plain-name|footnote-mode|foreground-color-at-point|form-at-point|format-annotate-atomic-property-change\n\t|format-annotate-function|format-annotate-location|format-annotate-region|format-annotate-single-property-change|format-annotate-value\n\t|format-deannotate-region|format-decode-buffer|format-decode-region|format-decode-run-method|format-decode|format-delq-cons|format-encode-buffer\n\t|format-encode-region|format-encode-run-method|format-insert-annotations|format-kbd-macro|format-make-relatively-unique|format-proper-list-p\n\t|format-property-increment-region|format-read|format-reorder|format-replace-strings|format-spec-make|format-spec|format-subtract-regions\n\t|forms-find-file-other-window|forms-find-file|forms-mode|fortran-abbrev-help|fortran-abbrev-start|fortran-analyze-file-format|fortran-auto-fill-mode\n\t|fortran-auto-fill|fortran-beginning-do|fortran-beginning-if|fortran-beginning-of-block|fortran-beginning-of-subprogram|fortran-blink-match\n\t|fortran-blink-matching-do|fortran-blink-matching-if|fortran-break-line|fortran-calculate-indent|fortran-check-end-prog-re|fortran-check-for-matching-do\n\t|fortran-column-ruler|fortran-comment-indent|fortran-comment-region|fortran-current-defun|fortran-current-line-indentation|fortran-electric-line-number\n\t|fortran-end-do|fortran-end-if|fortran-end-of-block|fortran-end-of-subprogram|fortran-fill-paragraph|fortran-fill-statement|fortran-fill\n\t|fortran-find-comment-start-skip|fortran-gud-find-expr|fortran-hack-local-variables|fortran-indent-comment|fortran-indent-line\n\t|fortran-indent-new-line|fortran-indent-subprogram|fortran-indent-to-column|fortran-is-in-string-p|fortran-join-line|fortran-line-length\n\t|fortran-line-number-indented-correctly-p|fortran-looking-at-if-then|fortran-make-syntax-propertize-function|fortran-mark-do\n\t|fortran-mark-if|fortran-match-and-skip-declaration|fortran-menu|fortran-mode|fortran-next-statement|fortran-numerical-continuation-char\n\t|fortran-prepare-abbrev-list-buffer|fortran-previous-statement|fortran-remove-continuation|fortran-split-line|fortran-strip-sequence-nos\n\t|fortran-uncomment-region|fortran-window-create-momentarily|fortran-window-create|fortune-add-fortune|fortune-append|fortune-ask-file\n\t|fortune-compile|fortune-from-region|fortune-in-buffer|fortune-to-signature|fortune|forward-ifdef|forward-page|forward-paragraph\n\t|forward-point|forward-same-syntax|forward-sentence|forward-symbol|forward-text-line|forward-thing|forward-visible-line|forward-whitespace\n\t|fourth|frame-border-width|frame-bottom-divider-width|frame-can-run-window-configuration-change-hook|frame-char-size|frame-configuration-p\n\t|frame-configuration-to-register|frame-face-alist|frame-focus|frame-font-cache|frame-fringe-width|frame-geom-spec-cons|frame-geom-value-cons\n\t|frame-initialize|frame-notice-user-settings|frame-or-buffer-changed-p|frame-remove-geometry-params|frame-right-divider-width\n\t|frame-root-window-p|frame-scroll-bar-height|frame-scroll-bar-width|frame-set-background-mode|frame-terminal-default-bg-mode|frame-text-cols\n\t|frame-text-height|frame-text-lines|frame-text-width|frame-total-cols|frame-total-lines|frame-windows-min-size|framep-on-display\n\t|frames-on-display-list|frameset--find-frame-if|frameset--initial-params|frameset--jump-to-register|frameset--make--cmacro|frameset--make\n\t|frameset--minibufferless-last-p|frameset--print-register|frameset--prop-setter|frameset--record-minibuffer-relationships|frameset--restore-frame\n\t|frameset--reuse-frame|frameset--set-id|frameset-app--cmacro|frameset-app|frameset-cfg-id|frameset-compute-pos|frameset-copy|frameset-description--cmacro\n\t|frameset-description|frameset-filter-iconified|frameset-filter-minibuffer|frameset-filter-params|frameset-filter-sanitize-color\n\t|frameset-filter-shelve-param|frameset-filter-tty-to-GUI|frameset-filter-unshelve-param|frameset-frame-id-equal-p|frameset-frame-id\n\t|frameset-frame-with-id|frameset-keep-original-display-p|frameset-minibufferless-first-p|frameset-move-onscreen|frameset-name--cmacro\n\t|frameset-name|frameset-p--cmacro|frameset-p|frameset-prop|frameset-properties--cmacro|frameset-properties|frameset-restore|frameset-save\n\t|frameset-states--cmacro|frameset-states|frameset-switch-to-gui-p|frameset-switch-to-tty-p|frameset-timestamp--cmacro|frameset-timestamp\n\t|frameset-to-register|frameset-valid-p|frameset-version--cmacro|frameset-version|fringe--check-style|fringe-bitmap-p|fringe-columns\n\t|fringe-mode-initialize|fringe-mode|fringe-query-style|ftp-mode|ftp|full-calc-keypad|full-calc|funcall-interactively|function\\*|function-called-at-point\n\t|function-equal|function-overload-p|function-put|function|gamegrid-add-score-insecure|gamegrid-add-score-with-update-game-score-1\n\t|gamegrid-add-score-with-update-game-score|gamegrid-add-score|gamegrid-cell-offset|gamegrid-characterp|gamegrid-color|gamegrid-colorize-glyph\n\t|gamegrid-display-type|gamegrid-event-x|gamegrid-event-y|gamegrid-get-cell|gamegrid-init-buffer|gamegrid-init|gamegrid-initialize-display\n\t|gamegrid-kill-timer|gamegrid-make-color-tty-face|gamegrid-make-color-x-face|gamegrid-make-face|gamegrid-make-glyph|gamegrid-make-grid-x-face\n\t|gamegrid-make-image-from-vector|gamegrid-make-mono-tty-face|gamegrid-make-mono-x-face|gamegrid-match-spec-list|gamegrid-match-spec\n\t|gamegrid-set-cell|gamegrid-set-display-table|gamegrid-set-face|gamegrid-set-font|gamegrid-set-timer|gamegrid-setup-default-font\n\t|gamegrid-setup-face|gamegrid-start-timer|gametree-apply-layout|gametree-apply-register-layout|gametree-break-line-here|gametree-children-shown-p\n\t|gametree-compute-and-insert-score|gametree-compute-reduced-score|gametree-current-branch-depth|gametree-current-branch-ply|gametree-current-branch-score\n\t|gametree-current-layout|gametree-entry-shown-p|gametree-forward-line|gametree-hack-file-layout|gametree-insert-new-leaf|gametree-insert-score\n\t|gametree-layout-to-register|gametree-looking-at-ply|gametree-merge-line|gametree-mode|gametree-mouse-break-line-here|gametree-mouse-hide-subtree\n\t|gametree-mouse-show-children-and-entry|gametree-mouse-show-subtree|gametree-prettify-heading|gametree-restore-layout|gametree-save-and-hack-layout\n\t|gametree-save-layout|gametree-show-children-and-entry|gametree-transpose-following-leaves|gcd|gdb--check-interpreter|gdb--if-arrow\n\t|gdb-add-handler|gdb-add-subscriber|gdb-append-to-partial-output|gdb-bind-function-to-buffer|gdb-breakpoints-buffer-name|gdb-breakpoints-list-handler-custom\n\t|gdb-breakpoints-list-handler|gdb-breakpoints-mode|gdb-buffer-shows-main-thread-p|gdb-buffer-type|gdb-changed-registers-handler\n\t|gdb-check-target-async|gdb-clear-inferior-io|gdb-clear-partial-output|gdb-concat-output|gdb-console|gdb-continue-thread|gdb-control-all-threads\n\t|gdb-control-current-thread|gdb-create-define-alist|gdb-current-buffer-frame|gdb-current-buffer-rules|gdb-current-buffer-thread\n\t|gdb-current-context-buffer-name|gdb-current-context-command|gdb-current-context-mode-name|gdb-delchar-or-quit|gdb-delete-breakpoint\n\t|gdb-delete-frame-or-window|gdb-delete-handler|gdb-delete-subscriber|gdb-disassembly-buffer-name|gdb-disassembly-handler-custom\n\t|gdb-disassembly-handler|gdb-disassembly-mode|gdb-disassembly-place-breakpoints|gdb-display-breakpoints-buffer|gdb-display-buffer\n\t|gdb-display-disassembly-buffer|gdb-display-disassembly-for-thread|gdb-display-gdb-buffer|gdb-display-io-buffer|gdb-display-locals-buffer\n\t|gdb-display-locals-for-thread|gdb-display-memory-buffer|gdb-display-registers-buffer|gdb-display-registers-for-thread|gdb-display-source-buffer\n\t|gdb-display-stack-buffer|gdb-display-stack-for-thread|gdb-display-threads-buffer|gdb-done-or-error|gdb-done|gdb-edit-locals-value\n\t|gdb-edit-register-value|gdb-edit-value-handler|gdb-edit-value|gdb-emit-signal|gdb-enable-debug|gdb-error|gdb-find-file-hook|gdb-find-watch-expression\n\t|gdb-force-mode-line-update|gdb-frame-breakpoints-buffer|gdb-frame-disassembly-buffer|gdb-frame-disassembly-for-thread|gdb-frame-gdb-buffer\n\t|gdb-frame-handler|gdb-frame-io-buffer|gdb-frame-locals-buffer|gdb-frame-locals-for-thread|gdb-frame-location|gdb-frame-memory-buffer\n\t|gdb-frame-registers-buffer|gdb-frame-registers-for-thread|gdb-frame-stack-buffer|gdb-frame-stack-for-thread|gdb-frame-threads-buffer\n\t|gdb-frames-mode|gdb-gdb|gdb-get-buffer-create|gdb-get-buffer|gdb-get-changed-registers|gdb-get-handler-function|gdb-get-location\n\t|gdb-get-main-selected-frame|gdb-get-many-fields|gdb-get-prompt|gdb-get-source-file-list|gdb-get-source-file|gdb-get-subscribers\n\t|gdb-get-target-string|gdb-goto-breakpoint|gdb-gud-context-call|gdb-gud-context-command|gdb-handle-reply|gdb-handler-function--cmacro\n\t|gdb-handler-function|gdb-handler-p--cmacro|gdb-handler-p|gdb-handler-pending-trigger--cmacro|gdb-handler-pending-trigger|gdb-handler-token-number--cmacro\n\t|gdb-handler-token-number|gdb-ignored-notification|gdb-inferior-filter|gdb-inferior-io--init-proc|gdb-inferior-io-mode|gdb-inferior-io-name\n\t|gdb-inferior-io-sentinel|gdb-init-1|gdb-init-buffer|gdb-input|gdb-internals|gdb-interrupt-thread|gdb-invalidate-breakpoints|gdb-invalidate-disassembly\n\t|gdb-invalidate-frames|gdb-invalidate-locals|gdb-invalidate-memory|gdb-invalidate-registers|gdb-invalidate-threads|gdb-io-eof|gdb-io-interrupt\n\t|gdb-io-quit|gdb-io-stop|gdb-json-partial-output|gdb-json-read-buffer|gdb-json-string|gdb-jsonify-buffer|gdb-line-posns|gdb-locals-buffer-name\n\t|gdb-locals-handler-custom|gdb-locals-handler|gdb-locals-mode|gdb-make-header-line-mouse-map|gdb-many-windows|gdb-mark-line|gdb-memory-buffer-name\n\t|gdb-memory-column-width|gdb-memory-format-binary|gdb-memory-format-hexadecimal|gdb-memory-format-menu-1|gdb-memory-format-menu\n\t|gdb-memory-format-octal|gdb-memory-format-signed|gdb-memory-format-unsigned|gdb-memory-mode|gdb-memory-set-address-event|gdb-memory-set-address\n\t|gdb-memory-set-columns|gdb-memory-set-rows|gdb-memory-show-next-page|gdb-memory-show-previous-page|gdb-memory-unit-byte|gdb-memory-unit-giant\n\t|gdb-memory-unit-halfword|gdb-memory-unit-menu-1|gdb-memory-unit-menu|gdb-memory-unit-word|gdb-mi-quote|gdb-mouse-jump|gdb-mouse-set-clear-breakpoint\n\t|gdb-mouse-toggle-breakpoint-fringe|gdb-mouse-toggle-breakpoint-margin|gdb-mouse-until|gdb-non-stop-handler|gdb-pad-string|gdb-parent-mode\n\t|gdb-partial-output-name|gdb-pending-handler-p|gdb-place-breakpoints|gdb-preempt-existing-or-display-buffer|gdb-preemptively-display-disassembly-buffer\n\t|gdb-preemptively-display-locals-buffer|gdb-preemptively-display-registers-buffer|gdb-preemptively-display-stack-buffer|gdb-propertize-header\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t gdb-put-breakpoint-icon|gdb-put-string|gdb-read-memory-custom|gdb-read-memory-handler|gdb-register-names-handler|gdb-registers-buffer-name\n\t|gdb-registers-handler-custom|gdb-registers-handler|gdb-registers-mode|gdb-remove-all-pending-triggers|gdb-remove-breakpoint-icons\n\t|gdb-remove-strings|gdb-reset|gdb-restore-windows|gdb-resync|gdb-rules-buffer-mode|gdb-rules-name-maker|gdb-rules-update-trigger\n\t|gdb-running|gdb-script-beginning-of-defun|gdb-script-calculate-indentation|gdb-script-end-of-defun|gdb-script-font-lock-syntactic-face\n\t|gdb-script-indent-line|gdb-script-mode|gdb-script-skip-to-head|gdb-select-frame|gdb-select-thread|gdb-send|gdb-set-buffer-rules\n\t|gdb-set-window-buffer|gdb-setq-thread-number|gdb-setup-windows|gdb-shell|gdb-show-run-p|gdb-show-stop-p|gdb-speedbar-auto-raise\n\t|gdb-speedbar-expand-node|gdb-speedbar-timer-fn|gdb-speedbar-update|gdb-stack-buffer-name|gdb-stack-list-frames-custom|gdb-stack-list-frames-handler\n\t|gdb-starting|gdb-step-thread|gdb-stopped|gdb-strip-string-backslash|gdb-table-add-row|gdb-table-column-sizes--cmacro|gdb-table-column-sizes\n\t|gdb-table-p--cmacro|gdb-table-p|gdb-table-right-align--cmacro|gdb-table-right-align|gdb-table-row-properties--cmacro|gdb-table-row-properties\n\t|gdb-table-rows--cmacro|gdb-table-rows|gdb-table-string|gdb-thread-created|gdb-thread-exited|gdb-thread-list-handler-custom|gdb-thread-list-handler\n\t|gdb-thread-selected|gdb-threads-buffer-name|gdb-threads-mode|gdb-toggle-breakpoint|gdb-toggle-switch-when-another-stopped|gdb-tooltip-print-1\n\t|gdb-tooltip-print|gdb-update-buffer-name|gdb-update-gud-running|gdb-update|gdb-var-create-handler|gdb-var-delete-1|gdb-var-delete-children\n\t|gdb-var-delete|gdb-var-evaluate-expression-handler|gdb-var-list-children-handler|gdb-var-list-children|gdb-var-set-format|gdb-var-update-handler\n\t|gdb-var-update|gdb-wait-for-pending|gdb|gdbmi-bnf-async-record|gdbmi-bnf-console-stream-output|gdbmi-bnf-gdb-prompt|gdbmi-bnf-incomplete-record-result\n\t|gdbmi-bnf-init|gdbmi-bnf-log-stream-output|gdbmi-bnf-out-of-band-record|gdbmi-bnf-output|gdbmi-bnf-result-and-async-record-impl\n\t|gdbmi-bnf-result-record|gdbmi-bnf-skip-unrecognized|gdbmi-bnf-stream-record|gdbmi-bnf-target-stream-output|gdbmi-is-number|gdbmi-same-start\n\t|gdbmi-start-with|generate-fontset-menu|generic-char-p|generic-make-keywords-list|generic-mode-internal|generic-mode|generic-p|generic-primary-only-one-p\n\t|generic-primary-only-p|gensym|gentemp|get\\*|get-edebug-spec|get-file-char|get-free-disk-space|get-language-info|get-mode-local-parent\n\t|get-mru-window|get-next-valid-buffer|get-other-frame|get-scroll-bar-mode|get-unicode-property-internal|get-unused-iso-final-char\n\t|get-upcase-table|getenv-internal|getf|gfile-add-watch|gfile-rm-watch|glasses-change|glasses-convert-to-unreadable|glasses-custom-set\n\t|glasses-make-overlay|glasses-make-readable|glasses-make-unreadable|glasses-mode|glasses-overlay-p|glasses-parenthesis-exception-p\n\t|glasses-set-overlay-properties|global-auto-composition-mode|global-auto-revert-mode|global-cwarn-mode-check-buffers|global-cwarn-mode-cmhh\n\t|global-cwarn-mode-enable-in-buffers|global-cwarn-mode|global-ede-mode|global-eldoc-mode|global-font-lock-mode-check-buffers|global-font-lock-mode-cmhh\n\t|global-font-lock-mode-enable-in-buffers|global-font-lock-mode|global-hi-lock-mode-check-buffers|global-hi-lock-mode-cmhh|global-hi-lock-mode-enable-in-buffers\n\t|global-hi-lock-mode|global-highlight-changes-mode-check-buffers|global-highlight-changes-mode-cmhh|global-highlight-changes-mode-enable-in-buffers\n\t|global-highlight-changes-mode|global-highlight-changes|global-hl-line-highlight|global-hl-line-mode|global-hl-line-unhighlight-all\n\t|global-hl-line-unhighlight|global-linum-mode-check-buffers|global-linum-mode-cmhh|global-linum-mode-enable-in-buffers|global-linum-mode\n\t|global-prettify-symbols-mode-check-buffers|global-prettify-symbols-mode-cmhh|global-prettify-symbols-mode-enable-in-buffers\n\t|global-prettify-symbols-mode|global-reveal-mode|global-semantic-decoration-mode|global-semantic-highlight-edits-mode|global-semantic-highlight-func-mode\n\t|global-semantic-idle-completions-mode|global-semantic-idle-local-symbol-highlight-mode|global-semantic-idle-scheduler-mode\n\t|global-semantic-idle-summary-mode|global-semantic-mru-bookmark-mode|global-semantic-show-parser-state-mode|global-semantic-show-unmatched-syntax-mode\n\t|global-semantic-stickyfunc-mode|global-semanticdb-minor-mode|global-set-scheme-interaction-buffer|global-srecode-minor-mode\n\t|global-subword-mode|global-superword-mode|global-visual-line-mode-check-buffers|global-visual-line-mode-cmhh|global-visual-line-mode-enable-in-buffers\n\t|global-visual-line-mode|global-whitespace-mode|global-whitespace-newline-mode|global-whitespace-toggle-options|glyphless-set-char-table-range\n\t|gmm-called-interactively-p|gmm-customize-mode|gmm-error|gmm-format-time-string|gmm-image-load-path-for-library|gmm-image-search-load-path\n\t|gmm-labels|gmm-message|gmm-regexp-concat|gmm-tool-bar-from-list|gmm-widget-p|gmm-write-region|gnus--random-face-with-type|gnus-1\n\t|gnus-Folder-save-name|gnus-active|gnus-add-buffer|gnus-add-configuration|gnus-add-shutdown|gnus-add-text-properties-when|gnus-add-text-properties\n\t|gnus-add-to-sorted-list|gnus-agent-batch-fetch|gnus-agent-batch|gnus-agent-delete-group|gnus-agent-fetch-session|gnus-agent-find-parameter\n\t|gnus-agent-get-function|gnus-agent-get-undownloaded-list|gnus-agent-group-covered-p|gnus-agent-method-p|gnus-agent-possibly-alter-active\n\t|gnus-agent-possibly-save-gcc|gnus-agent-regenerate|gnus-agent-rename-group|gnus-agent-request-article|gnus-agent-retrieve-headers\n\t|gnus-agent-save-active|gnus-agent-save-group-info|gnus-agent-store-article|gnus-agentize|gnus-alist-pull|gnus-alive-p|gnus-and\n\t|gnus-annotation-in-region-p|gnus-apply-kill-file-internal|gnus-apply-kill-file|gnus-archive-server-wanted-p|gnus-article-date-lapsed\n\t|gnus-article-date-local|gnus-article-date-original|gnus-article-de-base64-unreadable|gnus-article-de-quoted-unreadable|gnus-article-decode-HZ\n\t|gnus-article-decode-encoded-words|gnus-article-delete-invisible-text|gnus-article-display-x-face|gnus-article-edit-article|gnus-article-edit-done\n\t|gnus-article-edit-mode|gnus-article-fill-cited-article|gnus-article-fill-cited-long-lines|gnus-article-hide-boring-headers|gnus-article-hide-citation-in-followups\n\t|gnus-article-hide-citation-maybe|gnus-article-hide-citation|gnus-article-hide-headers|gnus-article-hide-pem|gnus-article-hide-signature\n\t|gnus-article-highlight-citation|gnus-article-html|gnus-article-mail|gnus-article-mode|gnus-article-next-page|gnus-article-outlook-deuglify-article\n\t|gnus-article-outlook-repair-attribution|gnus-article-outlook-unwrap-lines|gnus-article-prepare-display|gnus-article-prepare\n\t|gnus-article-prev-page|gnus-article-read-summary-keys|gnus-article-remove-cr|gnus-article-remove-trailing-blank-lines|gnus-article-save\n\t|gnus-article-set-window-start|gnus-article-setup-buffer|gnus-article-strip-leading-blank-lines|gnus-article-treat-overstrike\n\t|gnus-article-unsplit-urls|gnus-article-wash-html|gnus-assq-delete-all|gnus-async-halt-prefetch|gnus-async-prefetch-article|gnus-async-prefetch-next\n\t|gnus-async-prefetch-remove-group|gnus-async-request-fetched-article|gnus-atomic-progn-assign|gnus-atomic-progn|gnus-atomic-setq\n\t|gnus-backlog-enter-article|gnus-backlog-remove-article|gnus-backlog-request-article|gnus-batch-kill|gnus-batch-score|gnus-binary-mode\n\t|gnus-bind-print-variables|gnus-blocked-images|gnus-bookmark-bmenu-list|gnus-bookmark-jump|gnus-bookmark-set|gnus-bound-and-true-p\n\t|gnus-boundp|gnus-browse-foreign-server|gnus-buffer-exists-p|gnus-buffer-live-p|gnus-buffers|gnus-bug|gnus-button-mailto|gnus-button-reply\n\t|gnus-byte-compile|gnus-cache-articles-in-group|gnus-cache-close|gnus-cache-delete-group|gnus-cache-enter-article|gnus-cache-enter-remove-article\n\t|gnus-cache-file-contents|gnus-cache-generate-active|gnus-cache-generate-nov-databases|gnus-cache-open|gnus-cache-possibly-alter-active\n\t|gnus-cache-possibly-enter-article|gnus-cache-possibly-remove-articles|gnus-cache-remove-article|gnus-cache-rename-group|gnus-cache-request-article\n\t|gnus-cache-retrieve-headers|gnus-cache-save-buffers|gnus-cache-update-article|gnus-cached-article-p|gnus-character-to-event|gnus-check-backend-function\n\t|gnus-check-reasonable-setup|gnus-completing-read|gnus-configure-windows|gnus-continuum-version|gnus-convert-article-to-rmail\n\t|gnus-convert-face-to-png|gnus-convert-gray-x-face-to-xpm|gnus-convert-image-to-gray-x-face|gnus-convert-png-to-face|gnus-copy-article-buffer\n\t|gnus-copy-file|gnus-copy-overlay|gnus-copy-sequence|gnus-create-hash-size|gnus-create-image|gnus-create-info-command|gnus-current-score-file-nondirectory\n\t|gnus-data-find|gnus-data-header|gnus-date-get-time|gnus-date-iso8601|gnus-dd-mmm|gnus-deactivate-mark|gnus-declare-backend|gnus-decode-newsgroups\n\t|gnus-define-group-parameter|gnus-define-keymap|gnus-define-keys-1|gnus-define-keys-safe|gnus-define-keys|gnus-delay-article|gnus-delay-initialize\n\t|gnus-delay-send-queue|gnus-delete-alist|gnus-delete-directory|gnus-delete-duplicates|gnus-delete-file|gnus-delete-first|gnus-delete-gnus-frame\n\t|gnus-delete-line|gnus-delete-overlay|gnus-demon-add-disconnection|gnus-demon-add-handler|gnus-demon-add-rescan|gnus-demon-add-scan-timestamps\n\t|gnus-demon-add-scanmail|gnus-demon-cancel|gnus-demon-init|gnus-demon-remove-handler|gnus-display-x-face-in-from|gnus-draft-mode\n\t|gnus-draft-reminder|gnus-dribble-enter|gnus-dribble-touch|gnus-dup-enter-articles|gnus-dup-suppress-articles|gnus-dup-unsuppress-article\n\t|gnus-edit-form|gnus-emacs-completing-read|gnus-emacs-version|gnus-ems-redefine|gnus-enter-server-buffer|gnus-ephemeral-group-p\n\t|gnus-error|gnus-eval-in-buffer-window|gnus-execute|gnus-expand-group-parameter|gnus-expand-group-parameters|gnus-expunge|gnus-extended-version\n\t|gnus-extent-detached-p|gnus-extent-start-open|gnus-extract-address-components|gnus-extract-references|gnus-face-from-file|gnus-faces-at\n\t|gnus-fetch-field|gnus-fetch-group-other-frame|gnus-fetch-group|gnus-fetch-original-field|gnus-file-newer-than|gnus-final-warning\n\t|gnus-find-method-for-group|gnus-find-subscribed-addresses|gnus-find-text-property-region|gnus-float-time|gnus-folder-save-name\n\t|gnus-frame-or-window-display-name|gnus-generate-new-group-name|gnus-get-buffer-create|gnus-get-buffer-window|gnus-get-display-table\n\t|gnus-get-info|gnus-get-text-property-excluding-characters-with-faces|gnus-getenv-nntpserver|gnus-gethash-safe|gnus-gethash|gnus-globalify-regexp\n\t|gnus-goto-char|gnus-goto-colon|gnus-graphic-display-p|gnus-grep-in-list|gnus-group-add-parameter|gnus-group-add-score|gnus-group-auto-expirable-p\n\t|gnus-group-customize|gnus-group-decoded-name|gnus-group-entry|gnus-group-fast-parameter|gnus-group-find-parameter|gnus-group-first-unread-group\n\t|gnus-group-foreign-p|gnus-group-full-name|gnus-group-get-new-news|gnus-group-get-parameter|gnus-group-group-name|gnus-group-guess-full-name-from-command-method\n\t|gnus-group-insert-group-line|gnus-group-iterate|gnus-group-list-groups|gnus-group-mail|gnus-group-make-help-group|gnus-group-method\n\t|gnus-group-name-charset|gnus-group-name-decode|gnus-group-name-to-method|gnus-group-native-p|gnus-group-news|gnus-group-parameter-value\n\t|gnus-group-position-point|gnus-group-post-news|gnus-group-prefixed-name|gnus-group-prefixed-p|gnus-group-quit-config|gnus-group-quit\n\t|gnus-group-read-only-p|gnus-group-real-name|gnus-group-real-prefix|gnus-group-remove-parameter|gnus-group-save-newsrc|gnus-group-secondary-p\n\t|gnus-group-send-queue|gnus-group-server|gnus-group-set-info|gnus-group-set-mode-line|gnus-group-set-parameter|gnus-group-setup-buffer\n\t|gnus-group-short-name|gnus-group-split-fancy|gnus-group-split-setup|gnus-group-split-update|gnus-group-split|gnus-group-startup-message\n\t|gnus-group-total-expirable-p|gnus-group-unread|gnus-group-update-group|gnus-groups-from-server|gnus-header-from|gnus-highlight-selected-tree\n\t|gnus-horizontal-recenter|gnus-html-prefetch-images|gnus-ido-completing-read|gnus-image-type-available-p|gnus-indent-rigidly|gnus-info-find-node\n\t|gnus-info-group|gnus-info-level|gnus-info-marks|gnus-info-method|gnus-info-params|gnus-info-rank|gnus-info-read|gnus-info-score\n\t|gnus-info-set-entry|gnus-info-set-group|gnus-info-set-level|gnus-info-set-marks|gnus-info-set-method|gnus-info-set-params|gnus-info-set-rank\n\t|gnus-info-set-read|gnus-info-set-score|gnus-insert-random-face-header|gnus-insert-random-x-face-header|gnus-interactive|gnus-intern-safe\n\t|gnus-intersection|gnus-invisible-p|gnus-iswitchb-completing-read|gnus-jog-cache|gnus-key-press-event-p|gnus-kill-all-overlays\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t gnus-kill-buffer|gnus-kill-ephemeral-group|gnus-kill-file-edit-file|gnus-kill-file-raise-followups-to-author|gnus-kill-save-kill-buffer\n\t|gnus-kill|gnus-list-debbugs|gnus-list-memq-of-list|gnus-list-of-read-articles|gnus-list-of-unread-articles|gnus-local-set-keys\n\t|gnus-mail-strip-quoted-names|gnus-mailing-list-insinuate|gnus-mailing-list-mode|gnus-make-directory|gnus-make-hashtable|gnus-make-local-hook\n\t|gnus-make-overlay|gnus-make-predicate-1|gnus-make-predicate|gnus-make-sort-function-1|gnus-make-sort-function|gnus-make-thread-indent-array\n\t|gnus-map-function|gnus-mapcar|gnus-mark-active-p|gnus-match-substitute-replacement|gnus-max-width-function|gnus-member-of-valid\n\t|gnus-merge|gnus-message-with-timestamp|gnus-message|gnus-method-ephemeral-p|gnus-method-equal|gnus-method-option-p|gnus-method-simplify\n\t|gnus-method-to-full-server-name|gnus-method-to-server-name|gnus-method-to-server|gnus-methods-equal-p|gnus-methods-sloppily-equal\n\t|gnus-methods-using|gnus-mime-view-all-parts|gnus-mode-line-buffer-identification|gnus-mode-string-quote|gnus-move-overlay|gnus-msg-mail\n\t|gnus-mule-max-width-function|gnus-multiple-choice|gnus-narrow-to-body|gnus-narrow-to-page|gnus-native-method-p|gnus-news-group-p\n\t|gnus-newsgroup-directory-form|gnus-newsgroup-kill-file|gnus-newsgroup-savable-name|gnus-newsrc-parse-options|gnus-next-char-property-change\n\t|gnus-no-server-1|gnus-no-server|gnus-not-ignore|gnus-notifications|gnus-offer-save-summaries|gnus-online|gnus-open-agent|gnus-open-server\n\t|gnus-or|gnus-other-frame|gnus-outlook-deuglify-article|gnus-output-to-mail|gnus-output-to-rmail|gnus-overlay-buffer|gnus-overlay-end\n\t|gnus-overlay-get|gnus-overlay-put|gnus-overlay-start|gnus-overlays-at|gnus-overlays-in|gnus-parameter-charset|gnus-parameter-ham-marks\n\t|gnus-parameter-ham-process-destination|gnus-parameter-ham-resend-to|gnus-parameter-large-newsgroup-initial|gnus-parameter-post-method\n\t|gnus-parameter-registry-ignore|gnus-parameter-spam-autodetect-methods|gnus-parameter-spam-autodetect|gnus-parameter-spam-contents\n\t|gnus-parameter-spam-marks|gnus-parameter-spam-process-destination|gnus-parameter-spam-process|gnus-parameter-spam-resend-to\n\t|gnus-parameter-subscribed|gnus-parameter-to-address|gnus-parameter-to-list|gnus-parameters-get-parameter|gnus-parent-id|gnus-parse-without-error\n\t|gnus-pick-mode|gnus-plugged|gnus-possibly-generate-tree|gnus-possibly-score-headers|gnus-post-news|gnus-pp-to-string|gnus-pp|gnus-previous-char-property-change\n\t|gnus-prin1-to-string|gnus-prin1|gnus-process-get|gnus-process-plist|gnus-process-put|gnus-put-display-table|gnus-put-image|gnus-put-overlay-excluding-newlines\n\t|gnus-put-text-property-excluding-characters-with-faces|gnus-put-text-property-excluding-newlines|gnus-put-text-property|gnus-random-face\n\t|gnus-random-x-face|gnus-range-add|gnus-read-event-char|gnus-read-group|gnus-read-init-file|gnus-read-method|gnus-read-shell-command\n\t|gnus-recursive-directory-files|gnus-redefine-select-method-widget|gnus-region-active-p|gnus-registry-handle-action|gnus-registry-initialize\n\t|gnus-registry-install-hooks|gnus-remassoc|gnus-remove-from-range|gnus-remove-if-not|gnus-remove-if|gnus-remove-image|gnus-remove-text-properties-when\n\t|gnus-remove-text-with-property|gnus-rename-file|gnus-replace-in-string|gnus-request-article-this-buffer|gnus-request-post|gnus-request-type\n\t|gnus-rescale-image|gnus-run-hook-with-args|gnus-run-hooks|gnus-run-mode-hooks|gnus-same-method-different-name|gnus-score-adaptive\n\t|gnus-score-advanced|gnus-score-close|gnus-score-customize|gnus-score-delta-default|gnus-score-file-name|gnus-score-find-trace\n\t|gnus-score-flush-cache|gnus-score-followup-article|gnus-score-followup-thread|gnus-score-headers|gnus-score-mode|gnus-score-save\n\t|gnus-secondary-method-p|gnus-seconds-month|gnus-seconds-today|gnus-seconds-year|gnus-select-frame-set-input-focus|gnus-select-lowest-window\n\t|gnus-server-add-address|gnus-server-equal|gnus-server-extend-method|gnus-server-get-method|gnus-server-server-name|gnus-server-set-info\n\t|gnus-server-status|gnus-server-string|gnus-server-to-method|gnus-servers-using-backend|gnus-set-active|gnus-set-file-modes|gnus-set-info\n\t|gnus-set-process-plist|gnus-set-process-query-on-exit-flag|gnus-set-sorted-intersection|gnus-set-window-start|gnus-set-work-buffer\n\t|gnus-sethash|gnus-short-group-name|gnus-shutdown|gnus-sieve-article-add-rule|gnus-sieve-generate|gnus-sieve-update|gnus-similar-server-opened\n\t|gnus-simplify-mode-line|gnus-slave-no-server|gnus-slave-unplugged|gnus-slave|gnus-sloppily-equal-method-parameters|gnus-sorted-complement\n\t|gnus-sorted-difference|gnus-sorted-intersection|gnus-sorted-ndifference|gnus-sorted-nintersection|gnus-sorted-nunion|gnus-sorted-range-intersection\n\t|gnus-sorted-union|gnus-splash-svg-color-symbols|gnus-splash|gnus-split-references|gnus-start-date-timer|gnus-stop-date-timer|gnus-string-equal\n\t|gnus-string-mark-left-to-right|gnus-string-match-p|gnus-string-or-1|gnus-string-or|gnus-string-prefix-p|gnus-string-remove-all-properties\n\t|gnus-string<|gnus-string>|gnus-strip-whitespace|gnus-subscribe-topics|gnus-summary-article-number|gnus-summary-bookmark-jump|gnus-summary-buffer-name\n\t|gnus-summary-cancel-article|gnus-summary-current-score|gnus-summary-exit|gnus-summary-followup-to-mail-with-original|gnus-summary-followup-to-mail\n\t|gnus-summary-followup-with-original|gnus-summary-followup|gnus-summary-increase-score|gnus-summary-insert-cached-articles|gnus-summary-insert-line\n\t|gnus-summary-last-subject|gnus-summary-line-format-spec|gnus-summary-lower-same-subject-and-select|gnus-summary-lower-same-subject\n\t|gnus-summary-lower-score|gnus-summary-lower-thread|gnus-summary-mail-forward|gnus-summary-mail-other-window|gnus-summary-news-other-window\n\t|gnus-summary-position-point|gnus-summary-post-forward|gnus-summary-post-news|gnus-summary-raise-same-subject-and-select|gnus-summary-raise-same-subject\n\t|gnus-summary-raise-score|gnus-summary-raise-thread|gnus-summary-read-group|gnus-summary-reply-with-original|gnus-summary-reply\n\t|gnus-summary-resend-bounced-mail|gnus-summary-resend-message|gnus-summary-save-article-folder|gnus-summary-save-article-vm|gnus-summary-save-in-folder\n\t|gnus-summary-save-in-vm|gnus-summary-score-map|gnus-summary-send-map|gnus-summary-set-agent-mark|gnus-summary-set-score|gnus-summary-skip-intangible\n\t|gnus-summary-supersede-article|gnus-summary-wide-reply-with-original|gnus-summary-wide-reply|gnus-suppress-keymap|gnus-symbolic-argument\n\t|gnus-sync-initialize|gnus-sync-install-hooks|gnus-time-iso8601|gnus-timer--function|gnus-tool-bar-update|gnus-topic-mode|gnus-topic-remove-group\n\t|gnus-topic-set-parameters|gnus-treat-article|gnus-treat-from-gravatar|gnus-treat-from-picon|gnus-treat-mail-gravatar|gnus-treat-mail-picon\n\t|gnus-treat-newsgroups-picon|gnus-tree-close|gnus-tree-open|gnus-try-warping-via-registry|gnus-turn-off-edit-menu|gnus-undo-mode\n\t|gnus-undo-register|gnus-union|gnus-unplugged|gnus-update-alist-soft|gnus-update-format|gnus-update-read-articles|gnus-url-unhex-string\n\t|gnus-url-unhex|gnus-use-long-file-name|gnus-user-format-function-D|gnus-user-format-function-d|gnus-uu-decode-binhex-view|gnus-uu-decode-binhex\n\t|gnus-uu-decode-save-view|gnus-uu-decode-save|gnus-uu-decode-unshar-and-save-view|gnus-uu-decode-unshar-and-save|gnus-uu-decode-unshar-view\n\t|gnus-uu-decode-unshar|gnus-uu-decode-uu-and-save-view|gnus-uu-decode-uu-and-save|gnus-uu-decode-uu-view|gnus-uu-decode-uu|gnus-uu-delete-work-dir\n\t|gnus-uu-digest-mail-forward|gnus-uu-digest-post-forward|gnus-uu-extract-map|gnus-uu-invert-processable|gnus-uu-mark-all|gnus-uu-mark-buffer\n\t|gnus-uu-mark-by-regexp|gnus-uu-mark-map|gnus-uu-mark-over|gnus-uu-mark-region|gnus-uu-mark-series|gnus-uu-mark-sparse|gnus-uu-mark-thread\n\t|gnus-uu-post-news|gnus-uu-unmark-thread|gnus-version|gnus-virtual-group-p|gnus-visual-p|gnus-window-edges|gnus-window-inside-pixel-edges\n\t|gnus-with-output-to-file|gnus-write-active-file|gnus-write-buffer|gnus-x-face-from-file|gnus-xmas-define|gnus-xmas-redefine|gnus-xmas-splash\n\t|gnus-y-or-n-p|gnus-yes-or-no-p|gnus|gnutls-available-p|gnutls-boot|gnutls-bye|gnutls-deinit|gnutls-error-fatalp|gnutls-error-string\n\t|gnutls-errorp|gnutls-get-initstage|gnutls-message-maybe|gnutls-negotiate|gnutls-peer-status-warning-describe|gnutls-peer-status\n\t|gomoku--intangible|gomoku-beginning-of-line|gomoku-check-filled-qtuple|gomoku-click|gomoku-crash-game|gomoku-cross-qtuple|gomoku-display-statistics\n\t|gomoku-emacs-plays|gomoku-end-of-line|gomoku-find-filled-qtuple|gomoku-goto-square|gomoku-goto-xy|gomoku-human-plays|gomoku-human-resigns\n\t|gomoku-human-takes-back|gomoku-index-to-x|gomoku-index-to-y|gomoku-init-board|gomoku-init-display|gomoku-init-score-table|gomoku-init-square-score\n\t|gomoku-max-height|gomoku-max-width|gomoku-mode|gomoku-mouse-play|gomoku-move-down|gomoku-move-ne|gomoku-move-nw|gomoku-move-se|gomoku-move-sw\n\t|gomoku-move-up|gomoku-nb-qtuples|gomoku-offer-a-draw|gomoku-play-move|gomoku-plot-square|gomoku-point-square|gomoku-point-y|gomoku-prompt-for-move\n\t|gomoku-prompt-for-other-game|gomoku-start-game|gomoku-strongest-square|gomoku-switch-to-window|gomoku-take-back|gomoku-terminate-game\n\t|gomoku-update-score-in-direction|gomoku-update-score-table|gomoku-xy-to-index|gomoku|goto-address-at-mouse|goto-address-at-point\n\t|goto-address-find-address-at-point|goto-address-fontify-region|goto-address-fontify|goto-address-mode|goto-address-prog-mode\n\t|goto-address-unfontify|goto-address|goto-history-element|goto-line|goto-next-locus|gpm-mouse-disable|gpm-mouse-enable|gpm-mouse-mode\n\t|gpm-mouse-start|gpm-mouse-stop|gravatar-retrieve-synchronously|gravatar-retrieve|grep-apply-setting|grep-compute-defaults|grep-default-command\n\t|grep-expand-template|grep-filter|grep-find|grep-mode|grep-probe|grep-process-setup|grep-read-files|grep-read-regexp|grep-tag-default\n\t|grep|gs-height-in-pt|gs-load-image|gs-options|gs-set-ghostview-colors-window-prop|gs-set-ghostview-window-prop|gs-width-in-pt|gud-backward-sexp\n\t|gud-basic-call|gud-call|gud-common-init|gud-dbx-marker-filter|gud-dbx-massage-args|gud-def|gud-dguxdbx-marker-filter|gud-display-frame\n\t|gud-display-line|gud-expansion-speedbar-buttons|gud-expr-compound-sep|gud-expr-compound|gud-file-name|gud-filter|gud-find-c-expr\n\t|gud-find-class|gud-find-expr|gud-find-file|gud-format-command|gud-forward-sexp|gud-gdb-completion-at-point|gud-gdb-completions-1\n\t|gud-gdb-completions|gud-gdb-fetch-lines-filter|gud-gdb-get-stackframe|gud-gdb-goto-stackframe|gud-gdb-marker-filter|gud-gdb-run-command-fetch-lines\n\t|gud-gdb|gud-gdbmi-completions|gud-gdbmi-fetch-lines-filter|gud-gdbmi-marker-filter|gud-goto-info|gud-guiler-marker-filter|gud-innermost-expr\n\t|gud-install-speedbar-variables|gud-irixdbx-marker-filter|gud-jdb-analyze-source|gud-jdb-build-class-source-alist-for-file|gud-jdb-build-class-source-alist\n\t|gud-jdb-build-source-files-list|gud-jdb-find-source-file|gud-jdb-find-source-using-classpath|gud-jdb-find-source|gud-jdb-marker-filter\n\t|gud-jdb-massage-args|gud-jdb-parse-classpath-string|gud-jdb-skip-block|gud-jdb-skip-character-literal|gud-jdb-skip-id-ish-thing\n\t|gud-jdb-skip-single-line-comment|gud-jdb-skip-string-literal|gud-jdb-skip-traditional-or-documentation-comment|gud-jdb-skip-whitespace-and-comments\n\t|gud-jdb-skip-whitespace|gud-kill-buffer-hook|gud-marker-filter|gud-mipsdbx-marker-filter|gud-mode|gud-next-expr|gud-pdb-marker-filter\n\t|gud-perldb-marker-filter|gud-perldb-massage-args|gud-prev-expr|gud-query-cmdline|gud-read-address|gud-refresh|gud-reset|gud-sdb-find-file\n\t|gud-sdb-marker-filter|gud-sentinel|gud-set-buffer|gud-speedbar-buttons|gud-speedbar-item-info|gud-stop-subjob|gud-symbol|gud-tool-bar-item-visible-no-fringe\n\t|gud-tooltip-activate-mouse-motions-if-enabled|gud-tooltip-activate-mouse-motions|gud-tooltip-change-major-mode|gud-tooltip-dereference\n\t|gud-tooltip-mode|gud-tooltip-mouse-motion|gud-tooltip-print-command|gud-tooltip-process-output|gud-tooltip-tips|gud-val|gud-watch\n\t|gud-xdb-marker-filter|gud-xdb-massage-args|gui--selection-value-internal|gui--valid-simple-selection-p|gui-call|gui-get-primary-selection\n\t|gui-get-selection|gui-method--name|gui-method-declare|gui-method-define|gui-method|gui-select-text|gui-selection-value|gui-set-selection\n\t|guiler|gv--defsetter|gv--defun-declaration|gv-deref|gv-get|gv-ref|hack-local-variables-apply|hack-local-variables-confirm|hack-local-variables-filter\n\t|hack-local-variables-prop-line|hack-one-local-variable--obsolete|hack-one-local-variable-constantp|hack-one-local-variable-eval-safep\n\t|hack-one-local-variable-quotep|hack-one-local-variable|handle-delete-frame|handle-focus-in|handle-focus-out|handle-save-session\n\t|handle-select-window|handwrite-10pt|handwrite-11pt|handwrite-12pt|handwrite-13pt|handwrite-insert-font|handwrite-insert-header\n\t|handwrite-insert-info|handwrite-insert-preamble|handwrite-set-pagenumber-off|handwrite-set-pagenumber-on|handwrite-set-pagenumber\n\t|handwrite|hangul-input-method-activate|hanoi-0|hanoi-goto-char|hanoi-insert-ring|hanoi-internal|hanoi-move-ring|hanoi-n|hanoi-pos-on-tower-p\n\t|hanoi-put-face|hanoi-ring-to-pos|hanoi-sit-for|hanoi-unix-64|hanoi-unix|hanoi|hash-table-keys|hash-table-values|hashcash-already-paid-p\n\t|hashcash-cancel-async|hashcash-check-payment|hashcash-generate-payment-async|hashcash-generate-payment|hashcash-insert-payment-async-2\n\t|hashcash-insert-payment-async|hashcash-insert-payment|hashcash-payment-required|hashcash-payment-to|hashcash-point-at-bol|hashcash-point-at-eol\n\t|hashcash-processes-running-p|hashcash-strip-quoted-names|hashcash-token-substring|hashcash-verify-payment|hashcash-version|hashcash-wait-async\n\t|hashcash-wait-or-cancel|he--all-buffers|he-buffer-member|he-capitalize-first|he-concat-directory-file-name|he-dabbrev-beg|he-dabbrev-kill-search\n\t|he-dabbrev-search|he-file-name-beg|he-init-string|he-kill-beg|he-line-beg|he-line-search-regexp|he-line-search|he-lisp-symbol-beg\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t he-list-beg|he-list-search|he-ordinary-case-p|he-reset-string|he-string-member|he-substitute-string|he-transfer-case|he-whole-kill-search\n\t|hebrew-font-get-precomposed|hebrew-shape-gstring|help--binding-locus|help--key-binding-keymap|help-C-file-name|help-add-fundoc-usage\n\t|help-at-pt-cancel-timer|help-at-pt-kbd-string|help-at-pt-maybe-display|help-at-pt-set-timer|help-at-pt-string|help-bookmark-jump\n\t|help-bookmark-make-record|help-button-action|help-describe-category-set|help-do-arg-highlight|help-do-xref|help-fns--autoloaded-p\n\t|help-fns--compiler-macro|help-fns--interactive-only|help-fns--key-bindings|help-fns--obsolete|help-fns--parent-mode|help-fns--signature\n\t|help-follow-mouse|help-follow-symbol|help-follow|help-for-help-internal-doc|help-for-help-internal|help-for-help|help-form-show\n\t|help-function-arglist|help-go-back|help-go-forward|help-highlight-arg|help-highlight-arguments|help-insert-string|help-insert-xref-button\n\t|help-key-description|help-make-usage|help-make-xrefs|help-mode-finish|help-mode-menu|help-mode-revert-buffer|help-mode-setup|help-mode\n\t|help-print-return-message|help-quit|help-split-fundoc|help-window-display-message|help-window-setup|help-with-tutorial-spec-language\n\t|help-with-tutorial|help-xref-button|help-xref-go-back|help-xref-go-forward|help-xref-interned|help-xref-on-pp|help|hexl-C-c-prefix\n\t|hexl-C-x-prefix|hexl-ESC-prefix|hexl-activate-ruler|hexl-address-to-marker|hexl-ascii-start-column|hexl-backward-char|hexl-backward-short\n\t|hexl-backward-word|hexl-beginning-of-1k-page|hexl-beginning-of-512b-page|hexl-beginning-of-buffer|hexl-beginning-of-line|hexl-char-after-point\n\t|hexl-current-address|hexl-end-of-1k-page|hexl-end-of-512b-page|hexl-end-of-buffer|hexl-end-of-line|hexl-find-file|hexl-follow-ascii-find\n\t|hexl-follow-ascii|hexl-follow-line|hexl-forward-char|hexl-forward-short|hexl-forward-word|hexl-goto-address|hexl-goto-hex-address\n\t|hexl-hex-char-to-integer|hexl-hex-string-to-integer|hexl-highlight-line-range|hexl-htoi|hexl-insert-char|hexl-insert-decimal-char\n\t|hexl-insert-hex-char|hexl-insert-hex-string|hexl-insert-multibyte-char|hexl-insert-octal-char|hexl-isearch-search-function|hexl-line-displen\n\t|hexl-maybe-dehexlify-buffer|hexl-menu|hexl-mode--minor-mode-p|hexl-mode--setq-local|hexl-mode-exit|hexl-mode-ruler|hexl-mode|hexl-next-line\n\t|hexl-oct-char-to-integer|hexl-octal-string-to-integer|hexl-options|hexl-previous-line|hexl-print-current-point-info|hexl-printable-character\n\t|hexl-quoted-insert|hexl-revert-buffer-function|hexl-rulerize|hexl-save-buffer|hexl-scroll-down|hexl-scroll-up|hexl-self-insert-command\n\t|hexlify-buffer|hfy-begin-span|hfy-bgcol|hfy-box-to-border-assoc|hfy-box-to-style|hfy-box|hfy-buffer|hfy-colour-vals|hfy-colour|hfy-combined-face-spec\n\t|hfy-compile-face-map|hfy-compile-stylesheet|hfy-copy-and-fontify-file|hfy-css-name|hfy-decor|hfy-default-footer|hfy-default-header\n\t|hfy-dirname|hfy-end-span|hfy-face-at|hfy-face-attr-for-class|hfy-face-or-def-to-name|hfy-face-resolve-face|hfy-face-to-css-default\n\t|hfy-face-to-style-i|hfy-face-to-style|hfy-fallback-colour-values|hfy-family|hfy-find-invisible-ranges|hfy-flatten-style|hfy-fontified-p\n\t|hfy-fontify-buffer|hfy-force-fontification|hfy-href-stub|hfy-href|hfy-html-dekludge-buffer|hfy-html-enkludge-buffer|hfy-html-quote\n\t|hfy-init-progn|hfy-initfile|hfy-interq|hfy-invisible-name|hfy-invisible|hfy-kludge-cperl-mode|hfy-link-style-string|hfy-link-style\n\t|hfy-list-files|hfy-load-tags-cache|hfy-lookup|hfy-make-directory|hfy-mark-tag-hrefs|hfy-mark-tag-names|hfy-mark-trailing-whitespace\n\t|hfy-merge-adjacent-spans|hfy-opt|hfy-overlay-props-at|hfy-parse-tags-buffer|hfy-prepare-index-i|hfy-prepare-index|hfy-prepare-tag-map\n\t|hfy-prop-invisible-p|hfy-relstub|hfy-save-buffer-state|hfy-save-initvar|hfy-save-kill-buffers|hfy-shell|hfy-size-to-int|hfy-size\n\t|hfy-slant|hfy-sprintf-stylesheet|hfy-subtract-maps|hfy-tags-for-file|hfy-text-p|hfy-triplet|hfy-unmark-trailing-whitespace|hfy-weight\n\t|hfy-which-etags|hfy-width|hfy-word-regex|hi-lock--hashcons|hi-lock--regexps-at-point|hi-lock-face-buffer|hi-lock-face-phrase-buffer\n\t|hi-lock-face-symbol-at-point|hi-lock-find-patterns|hi-lock-font-lock-hook|hi-lock-keyword->face|hi-lock-line-face-buffer|hi-lock-mode-set-explicitly\n\t|hi-lock-mode|hi-lock-process-phrase|hi-lock-read-face-name|hi-lock-regexp-okay|hi-lock-set-file-patterns|hi-lock-set-pattern|hi-lock-unface-buffer\n\t|hi-lock-unload-function|hi-lock-write-interactive-patterns|hide-body|hide-entry|hide-ifdef-block|hide-ifdef-define|hide-ifdef-guts\n\t|hide-ifdef-mode-menu|hide-ifdef-mode|hide-ifdef-region-internal|hide-ifdef-region|hide-ifdef-set-define-alist|hide-ifdef-toggle-outside-read-only\n\t|hide-ifdef-toggle-read-only|hide-ifdef-toggle-shadowing|hide-ifdef-undef|hide-ifdef-use-define-alist|hide-ifdefs|hide-leaves|hide-other\n\t|hide-region-body|hide-sublevels|hide-subtree|hif-add-new-defines|hif-after-revert-function|hif-and-expr|hif-and|hif-canonicalize-tokens\n\t|hif-canonicalize|hif-clear-all-ifdef-defined|hif-comma|hif-comp-expr|hif-compress-define-list|hif-conditional|hif-define-macro\n\t|hif-define-operator|hif-defined|hif-delimit|hif-divide|hif-end-of-line|hif-endif-to-ifdef|hif-eq-expr|hif-equal|hif-evaluate-macro\n\t|hif-evaluate-region|hif-expand-token-list|hif-expr|hif-exprlist|hif-factor|hif-find-any-ifX|hif-find-define|hif-find-ifdef-block\n\t|hif-find-next-relevant|hif-find-previous-relevant|hif-find-range|hif-flatten|hif-get-argument-list|hif-greater-equal|hif-greater\n\t|hif-hide-line|hif-if-valid-identifier-p|hif-ifdef-to-endif|hif-invoke|hif-less-equal|hif-less|hif-logand-expr|hif-logand|hif-logior-expr\n\t|hif-logior|hif-lognot|hif-logshift-expr|hif-logxor-expr|hif-logxor|hif-looking-at-elif|hif-looking-at-else|hif-looking-at-endif\n\t|hif-looking-at-ifX|hif-lookup|hif-macro-supply-arguments|hif-make-range|hif-math|hif-mathify-binop|hif-mathify|hif-merge-ifdef-region\n\t|hif-minus|hif-modulo|hif-muldiv-expr|hif-multiply|hif-nexttoken|hif-not|hif-notequal|hif-or-expr|hif-or|hif-parse-exp|hif-parse-macro-arglist\n\t|hif-place-macro-invocation|hif-plus|hif-possibly-hide|hif-range-elif|hif-range-else|hif-range-end|hif-range-start|hif-recurse-on\n\t|hif-set-var|hif-shiftleft|hif-shiftright|hif-show-all|hif-show-ifdef-region|hif-string-concatenation|hif-string-to-number|hif-stringify\n\t|hif-token-concat|hif-token-concatenation|hif-token-stringification|hif-tokenize|hif-undefine-symbol|highlight-changes-mode-set-explicitly\n\t|highlight-changes-mode-turn-on|highlight-changes-mode|highlight-changes-next-change|highlight-changes-previous-change|highlight-changes-remove-highlight\n\t|highlight-changes-rotate-faces|highlight-changes-visible-mode|highlight-compare-buffers|highlight-compare-with-file|highlight-lines-matching-regexp\n\t|highlight-markup-buffers|highlight-phrase|highlight-regexp|highlight-symbol-at-point|hilit-chg-bump-change|hilit-chg-clear|hilit-chg-cust-fix-changes-face-list\n\t|hilit-chg-desktop-restore|hilit-chg-display-changes|hilit-chg-fixup|hilit-chg-get-diff-info|hilit-chg-get-diff-list-hk|hilit-chg-hide-changes\n\t|hilit-chg-make-list|hilit-chg-make-ov|hilit-chg-map-changes|hilit-chg-set-face-on-change|hilit-chg-set|hilit-chg-unload-function\n\t|hilit-chg-update|hippie-expand|hl-line-highlight|hl-line-make-overlay|hl-line-mode|hl-line-move|hl-line-unhighlight|hl-line-unload-function\n\t|hmac-md5-96|hmac-md5|holiday-list|holidays|horizontal-scroll-bar-mode|horizontal-scroll-bars-available-p|how-many|hs-already-hidden-p\n\t|hs-c-like-adjust-block-beginning|hs-discard-overlays|hs-find-block-beginning|hs-forward-sexp|hs-grok-mode-type|hs-hide-all|hs-hide-block-at-point\n\t|hs-hide-block|hs-hide-comment-region|hs-hide-initial-comment-block|hs-hide-level-recursive|hs-hide-level|hs-inside-comment-p|hs-isearch-show-temporary\n\t|hs-isearch-show|hs-life-goes-on|hs-looking-at-block-start-p|hs-make-overlay|hs-minor-mode-menu|hs-minor-mode|hs-mouse-toggle-hiding\n\t|hs-overlay-at|hs-show-all|hs-show-block|hs-toggle-hiding|html-autoview-mode|html-checkboxes|html-current-defun-name|html-headline-1\n\t|html-headline-2|html-headline-3|html-headline-4|html-headline-5|html-headline-6|html-horizontal-rule|html-href-anchor|html-image\n\t|html-imenu-index|html-line|html-list-item|html-mode|html-name-anchor|html-ordered-list|html-paragraph|html-radio-buttons|html-unordered-list\n\t|html2text|htmlfontify-buffer|htmlfontify-copy-and-link-dir|htmlfontify-load-initfile|htmlfontify-load-rgb-file|htmlfontify-run-etags\n\t|htmlfontify-save-initfile|htmlfontify-string|htmlize-attrlist-to-fstruct|htmlize-buffer-1|htmlize-buffer-substring-no-invisible\n\t|htmlize-buffer|htmlize-color-to-rgb|htmlize-copy-attr-if-set|htmlize-css-insert-head|htmlize-css-insert-text|htmlize-css-specs\n\t|htmlize-defang-local-variables|htmlize-default-body-tag|htmlize-default-doctype|htmlize-despam-address|htmlize-ensure-fontified\n\t|htmlize-face-background|htmlize-face-color-internal|htmlize-face-emacs21-attr|htmlize-face-foreground|htmlize-face-list-p|htmlize-face-size\n\t|htmlize-face-specifies-property|htmlize-face-to-fstruct|htmlize-faces-at-point|htmlize-faces-in-buffer|htmlize-file|htmlize-font-body-tag\n\t|htmlize-font-insert-text|htmlize-fstruct-background--cmacro|htmlize-fstruct-background|htmlize-fstruct-boldp--cmacro|htmlize-fstruct-boldp\n\t|htmlize-fstruct-css-name--cmacro|htmlize-fstruct-css-name|htmlize-fstruct-foreground--cmacro|htmlize-fstruct-foreground|htmlize-fstruct-italicp--cmacro\n\t|htmlize-fstruct-italicp|htmlize-fstruct-overlinep--cmacro|htmlize-fstruct-overlinep|htmlize-fstruct-p--cmacro|htmlize-fstruct-p\n\t|htmlize-fstruct-size--cmacro|htmlize-fstruct-size|htmlize-fstruct-strikep--cmacro|htmlize-fstruct-strikep|htmlize-fstruct-underlinep--cmacro\n\t|htmlize-fstruct-underlinep|htmlize-get-color-rgb-hash|htmlize-inline-css-body-tag|htmlize-inline-css-insert-text|htmlize-locate-file\n\t|htmlize-make-face-map|htmlize-make-file-name|htmlize-make-hyperlinks|htmlize-many-files-dired|htmlize-many-files|htmlize-memoize\n\t|htmlize-merge-faces|htmlize-merge-size|htmlize-merge-two-faces|htmlize-method-function|htmlize-method|htmlize-next-change|htmlize-protect-string\n\t|htmlize-region-for-paste|htmlize-region|htmlize-trim-ellipsis|htmlize-unstringify-face|htmlize-untabify|htmlize-with-fontify-message\n\t|ibuffer-active-formats-name|ibuffer-add-saved-filters|ibuffer-add-to-tmp-hide|ibuffer-add-to-tmp-show|ibuffer-assert-ibuffer-mode\n\t|ibuffer-auto-mode|ibuffer-backward-filter-group|ibuffer-backward-line|ibuffer-backwards-next-marked|ibuffer-bs-show|ibuffer-buf-matches-predicates\n\t|ibuffer-buffer-file-name|ibuffer-buffer-name-face|ibuffer-buffer-names-with-mark|ibuffer-bury-buffer|ibuffer-check-formats|ibuffer-clear-filter-groups\n\t|ibuffer-clear-summary-columns|ibuffer-columnize-and-insert-list|ibuffer-compile-format|ibuffer-compile-make-eliding-form|ibuffer-compile-make-format-form\n\t|ibuffer-compile-make-substring-form|ibuffer-confirm-operation-on|ibuffer-copy-filename-as-kill|ibuffer-count-deletion-lines\n\t|ibuffer-count-marked-lines|ibuffer-current-buffer|ibuffer-current-buffers-with-marks|ibuffer-current-format|ibuffer-current-formats\n\t|ibuffer-current-mark|ibuffer-current-state-list|ibuffer-customize|ibuffer-decompose-filter-group|ibuffer-decompose-filter|ibuffer-delete-saved-filter-groups\n\t|ibuffer-delete-saved-filters|ibuffer-deletion-marked-buffer-names|ibuffer-diff-with-file|ibuffer-do-delete|ibuffer-do-eval|ibuffer-do-isearch-regexp\n\t|ibuffer-do-isearch|ibuffer-do-kill-lines|ibuffer-do-kill-on-deletion-marks|ibuffer-do-occur|ibuffer-do-print|ibuffer-do-query-replace-regexp\n\t|ibuffer-do-query-replace|ibuffer-do-rename-uniquely|ibuffer-do-replace-regexp|ibuffer-do-revert|ibuffer-do-save|ibuffer-do-shell-command-file\n\t|ibuffer-do-shell-command-pipe-replace|ibuffer-do-shell-command-pipe|ibuffer-do-sort-by-alphabetic|ibuffer-do-sort-by-filename\\/process\n\t|ibuffer-do-sort-by-major-mode|ibuffer-do-sort-by-mode-name|ibuffer-do-sort-by-recency|ibuffer-do-sort-by-size|ibuffer-do-toggle-modified\n\t|ibuffer-do-toggle-read-only|ibuffer-do-view-1|ibuffer-do-view-and-eval|ibuffer-do-view-horizontally|ibuffer-do-view-other-frame\n\t|ibuffer-do-view|ibuffer-exchange-filters|ibuffer-expand-format-entry|ibuffer-filter-buffers|ibuffer-filter-by-content|ibuffer-filter-by-derived-mode\n\t|ibuffer-filter-by-filename|ibuffer-filter-by-mode|ibuffer-filter-by-name|ibuffer-filter-by-predicate|ibuffer-filter-by-size-gt\n\t|ibuffer-filter-by-size-lt|ibuffer-filter-by-used-mode|ibuffer-filter-disable|ibuffer-filters-to-filter-group|ibuffer-find-file\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t ibuffer-format-column|ibuffer-forward-filter-group|ibuffer-forward-line|ibuffer-forward-next-marked|ibuffer-get-marked-buffers\n\t|ibuffer-included-in-filters-p|ibuffer-insert-buffer-line|ibuffer-insert-filter-group|ibuffer-interactive-filter-by-mode|ibuffer-invert-sorting\n\t|ibuffer-jump-to-buffer|ibuffer-jump-to-filter-group|ibuffer-kill-filter-group|ibuffer-kill-line|ibuffer-list-buffers|ibuffer-make-column-filename-and-process\n\t|ibuffer-make-column-filename|ibuffer-make-column-process|ibuffer-map-deletion-lines|ibuffer-map-lines-nomodify|ibuffer-map-lines\n\t|ibuffer-map-marked-lines|ibuffer-map-on-mark|ibuffer-mark-by-file-name-regexp|ibuffer-mark-by-mode-regexp|ibuffer-mark-by-mode\n\t|ibuffer-mark-by-name-regexp|ibuffer-mark-compressed-file-buffers|ibuffer-mark-dired-buffers|ibuffer-mark-dissociated-buffers\n\t|ibuffer-mark-for-delete-backwards|ibuffer-mark-for-delete|ibuffer-mark-forward|ibuffer-mark-help-buffers|ibuffer-mark-interactive\n\t|ibuffer-mark-modified-buffers|ibuffer-mark-old-buffers|ibuffer-mark-read-only-buffers|ibuffer-mark-special-buffers|ibuffer-mark-unsaved-buffers\n\t|ibuffer-marked-buffer-names|ibuffer-mode|ibuffer-mouse-filter-by-mode|ibuffer-mouse-popup-menu|ibuffer-mouse-toggle-filter-group\n\t|ibuffer-mouse-toggle-mark|ibuffer-mouse-visit-buffer|ibuffer-negate-filter|ibuffer-or-filter|ibuffer-other-window|ibuffer-pop-filter-group\n\t|ibuffer-pop-filter|ibuffer-recompile-formats|ibuffer-redisplay-current|ibuffer-redisplay-engine|ibuffer-redisplay|ibuffer-save-filter-groups\n\t|ibuffer-save-filters|ibuffer-set-filter-groups-by-mode|ibuffer-set-mark-1|ibuffer-set-mark|ibuffer-shrink-to-fit|ibuffer-skip-properties\n\t|ibuffer-sort-bufferlist|ibuffer-switch-format|ibuffer-switch-to-saved-filter-groups|ibuffer-switch-to-saved-filters|ibuffer-toggle-filter-group\n\t|ibuffer-toggle-marks|ibuffer-toggle-sorting-mode|ibuffer-unmark-all|ibuffer-unmark-backward|ibuffer-unmark-forward|ibuffer-update-format\n\t|ibuffer-update-title-and-summary|ibuffer-update|ibuffer-visible-p|ibuffer-visit-buffer-1-window|ibuffer-visit-buffer-other-frame\n\t|ibuffer-visit-buffer-other-window-noselect|ibuffer-visit-buffer-other-window|ibuffer-visit-buffer|ibuffer-visit-tags-table|ibuffer-yank-filter-group\n\t|ibuffer-yank|ibuffer|icalendar--add-decoded-times|icalendar--add-diary-entry|icalendar--all-events|icalendar--convert-all-timezones\n\t|icalendar--convert-anniversary-to-ical|icalendar--convert-block-to-ical|icalendar--convert-cyclic-to-ical|icalendar--convert-date-to-ical\n\t|icalendar--convert-float-to-ical|icalendar--convert-ical-to-diary|icalendar--convert-non-recurring-all-day-to-diary|icalendar--convert-non-recurring-not-all-day-to-diary\n\t|icalendar--convert-ordinary-to-ical|icalendar--convert-recurring-to-diary|icalendar--convert-sexp-to-ical|icalendar--convert-string-for-export\n\t|icalendar--convert-string-for-import|icalendar--convert-to-ical|icalendar--convert-tz-offset|icalendar--convert-weekly-to-ical\n\t|icalendar--convert-yearly-to-ical|icalendar--create-ical-alarm|icalendar--create-uid|icalendar--date-to-isodate|icalendar--datestring-to-isodate\n\t|icalendar--datetime-to-american-date|icalendar--datetime-to-colontime|icalendar--datetime-to-diary-date|icalendar--datetime-to-european-date\n\t|icalendar--datetime-to-iso-date|icalendar--datetime-to-noneuropean-date|icalendar--decode-isodatetime|icalendar--decode-isoduration\n\t|icalendar--diarytime-to-isotime|icalendar--dmsg|icalendar--do-create-ical-alarm|icalendar--find-time-zone|icalendar--format-ical-event\n\t|icalendar--get-children|icalendar--get-event-properties|icalendar--get-event-property-attributes|icalendar--get-event-property\n\t|icalendar--get-month-number|icalendar--get-unfolded-buffer|icalendar--get-weekday-abbrev|icalendar--get-weekday-number|icalendar--get-weekday-numbers\n\t|icalendar--parse-summary-and-rest|icalendar--parse-vtimezone|icalendar--read-element|icalendar--rris|icalendar--split-value|icalendar-convert-diary-to-ical\n\t|icalendar-export-file|icalendar-export-region|icalendar-extract-ical-from-buffer|icalendar-first-weekday-of-year|icalendar-import-buffer\n\t|icalendar-import-file|icalendar-import-format-sample|icomplete--completion-predicate|icomplete--completion-table|icomplete--field-beg\n\t|icomplete--field-end|icomplete--field-string|icomplete--in-region-setup|icomplete-backward-completions|icomplete-completions\n\t|icomplete-exhibit|icomplete-forward-completions|icomplete-minibuffer-setup|icomplete-mode|icomplete-post-command-hook|icomplete-pre-command-hook\n\t|icomplete-simple-completing-p|icomplete-tidy|icon-backward-to-noncomment|icon-backward-to-start-of-continued-exp|icon-backward-to-start-of-if\n\t|icon-comment-indent|icon-forward-sexp-function|icon-indent-command|icon-indent-line|icon-is-continuation-line|icon-is-continued-line\n\t|icon-mode|iconify-or-deiconify-frame|idl-font-lock-keywords-2|idl-font-lock-keywords-3|idl-font-lock-keywords|idl-mode|idlwave-action-and-binding\n\t|idlwave-active-rinfo-space|idlwave-add-file-link-selector|idlwave-after-successful-completion|idlwave-all-assq|idlwave-all-class-inherits\n\t|idlwave-all-class-tags|idlwave-all-method-classes|idlwave-all-method-keyword-classes|idlwave-any-syslib|idlwave-attach-class-tag-classes\n\t|idlwave-attach-classes|idlwave-attach-keyword-classes|idlwave-attach-method-classes|idlwave-auto-fill-mode|idlwave-auto-fill\n\t|idlwave-backward-block|idlwave-backward-up-block|idlwave-beginning-of-block|idlwave-beginning-of-statement|idlwave-beginning-of-subprogram\n\t|idlwave-best-rinfo-assoc|idlwave-best-rinfo-assq|idlwave-block-jump-out|idlwave-block-master|idlwave-calc-hanging-indent|idlwave-calculate-cont-indent\n\t|idlwave-calculate-indent|idlwave-calculate-paren-indent|idlwave-call-special|idlwave-case|idlwave-check-abbrev|idlwave-choose-completion\n\t|idlwave-choose|idlwave-class-alist|idlwave-class-file-or-buffer|idlwave-class-found-in|idlwave-class-info|idlwave-class-inherits\n\t|idlwave-class-or-superclass-with-tag|idlwave-class-tag-reset|idlwave-class-tags|idlwave-close-block|idlwave-code-abbrev|idlwave-command-hook\n\t|idlwave-comment-hook|idlwave-complete-class-structure-tag-help|idlwave-complete-class-structure-tag|idlwave-complete-class|idlwave-complete-filename\n\t|idlwave-complete-in-buffer|idlwave-complete-sysvar-help|idlwave-complete-sysvar-or-tag|idlwave-complete-sysvar-tag-help|idlwave-complete\n\t|idlwave-completing-read|idlwave-completion-fontify-classes|idlwave-concatenate-rinfo-lists|idlwave-context-help|idlwave-convert-xml-clean-routine-aliases\n\t|idlwave-convert-xml-clean-statement-aliases|idlwave-convert-xml-clean-sysvar-aliases|idlwave-convert-xml-system-routine-info\n\t|idlwave-count-eq|idlwave-count-memq|idlwave-count-outlawed-buffers|idlwave-create-customize-menu|idlwave-create-user-catalog-file\n\t|idlwave-current-indent|idlwave-current-routine-fullname|idlwave-current-routine|idlwave-current-statement-indent|idlwave-custom-ampersand-surround\n\t|idlwave-custom-ltgtr-surround|idlwave-customize|idlwave-debug-map|idlwave-default-choose-completion|idlwave-default-insert-timestamp\n\t|idlwave-define-abbrev|idlwave-delete-user-catalog-file|idlwave-determine-class|idlwave-display-calling-sequence|idlwave-display-completion-list-emacs\n\t|idlwave-display-completion-list-xemacs|idlwave-display-completion-list|idlwave-display-user-catalog-widget|idlwave-do-action\n\t|idlwave-do-context-help|idlwave-do-context-help1|idlwave-do-find-module|idlwave-do-kill-autoloaded-buffers|idlwave-do-mouse-completion-help\n\t|idlwave-doc-header|idlwave-doc-modification|idlwave-down-block|idlwave-downcase-safe|idlwave-edit-in-idlde|idlwave-elif|idlwave-end-of-block\n\t|idlwave-end-of-statement|idlwave-end-of-statement0|idlwave-end-of-subprogram|idlwave-entry-find-keyword|idlwave-entry-has-help\n\t|idlwave-entry-keywords|idlwave-expand-equal|idlwave-expand-keyword|idlwave-expand-lib-file-name|idlwave-expand-path|idlwave-expand-region-abbrevs\n\t|idlwave-explicit-class-listed|idlwave-fill-paragraph|idlwave-find-class-definition|idlwave-find-file-noselect|idlwave-find-inherited-class\n\t|idlwave-find-key|idlwave-find-module-this-file|idlwave-find-module|idlwave-find-struct-tag|idlwave-find-structure-definition\n\t|idlwave-fix-keywords|idlwave-fix-module-if-obj_new|idlwave-font-lock-fontify-region|idlwave-for|idlwave-forward-block|idlwave-function-menu\n\t|idlwave-function|idlwave-get-buffer-routine-info|idlwave-get-buffer-visiting|idlwave-get-routine-info-from-buffers|idlwave-goto-comment\n\t|idlwave-grep|idlwave-hard-tab|idlwave-has-help|idlwave-help-assistant-available|idlwave-help-assistant-close|idlwave-help-assistant-command\n\t|idlwave-help-assistant-help-with-topic|idlwave-help-assistant-open-link|idlwave-help-assistant-raise|idlwave-help-assistant-start\n\t|idlwave-help-check-locations|idlwave-help-diagnostics|idlwave-help-display-help-window|idlwave-help-error|idlwave-help-find-first-header\n\t|idlwave-help-find-header|idlwave-help-find-in-doc-header|idlwave-help-find-routine-definition|idlwave-help-fontify|idlwave-help-get-help-buffer\n\t|idlwave-help-get-special-help|idlwave-help-html-link|idlwave-help-menu|idlwave-help-mode|idlwave-help-quit|idlwave-help-return-to-calling-frame\n\t|idlwave-help-select-help-frame|idlwave-help-show-help-frame|idlwave-help-toggle-header-match-and-def|idlwave-help-toggle-header-top-and-def\n\t|idlwave-help-with-source|idlwave-highlight-linked-completions|idlwave-html-help-location|idlwave-if|idlwave-in-comment|idlwave-in-quote\n\t|idlwave-in-structure|idlwave-indent-and-action|idlwave-indent-left-margin|idlwave-indent-line|idlwave-indent-statement|idlwave-indent-subprogram\n\t|idlwave-indent-to|idlwave-info|idlwave-insert-source-location|idlwave-is-comment-line|idlwave-is-comment-or-empty-line|idlwave-is-continuation-line\n\t|idlwave-is-pointer-dereference|idlwave-keyboard-quit|idlwave-keyword-abbrev|idlwave-kill-autoloaded-buffers|idlwave-kill-buffer-update\n\t|idlwave-last-valid-char|idlwave-launch-idlhelp|idlwave-lib-p|idlwave-list-abbrevs|idlwave-list-all-load-path-shadows|idlwave-list-buffer-load-path-shadows\n\t|idlwave-list-load-path-shadows|idlwave-list-shell-load-path-shadows|idlwave-load-all-rinfo|idlwave-load-rinfo-next-step|idlwave-load-system-routine-info\n\t|idlwave-local-value|idlwave-locate-lib-file|idlwave-look-at|idlwave-make-force-complete-where-list|idlwave-make-full-name|idlwave-make-modified-completion-map-emacs\n\t|idlwave-make-modified-completion-map-xemacs|idlwave-make-one-key-alist|idlwave-make-space|idlwave-make-tags|idlwave-mark-block\n\t|idlwave-mark-doclib|idlwave-mark-statement|idlwave-mark-subprogram|idlwave-match-class-arrows|idlwave-members-only|idlwave-min-current-statement-indent\n\t|idlwave-mode-debug-menu|idlwave-mode-menu|idlwave-mode|idlwave-mouse-active-rinfo-right|idlwave-mouse-active-rinfo-shift|idlwave-mouse-active-rinfo\n\t|idlwave-mouse-choose-completion|idlwave-mouse-completion-help|idlwave-mouse-context-help|idlwave-new-buffer-update|idlwave-new-sintern-type\n\t|idlwave-newline|idlwave-next-statement|idlwave-nonmembers-only|idlwave-one-key-select|idlwave-online-help|idlwave-parse-definition\n\t|idlwave-path-alist-add-flag|idlwave-path-alist-remove-flag|idlwave-popup-select|idlwave-prepare-class-tag-completion|idlwave-prev-index-position\n\t|idlwave-previous-statement|idlwave-print-source|idlwave-procedure|idlwave-process-sysvars|idlwave-quit-help|idlwave-quoted|idlwave-read-paths\n\t|idlwave-recursive-directory-list|idlwave-region-active-p|idlwave-repeat|idlwave-replace-buffer-routine-info|idlwave-replace-string\n\t|idlwave-rescan-asynchronously|idlwave-rescan-catalog-directories|idlwave-reset-sintern-type|idlwave-reset-sintern|idlwave-resolve\n\t|idlwave-restore-wconf-after-completion|idlwave-revoke-license-to-kill|idlwave-rinfo-assoc|idlwave-rinfo-assq-any-class|idlwave-rinfo-assq\n\t|idlwave-rinfo-group-keywords|idlwave-rinfo-insert-keyword|idlwave-routine-entry-compare-twins|idlwave-routine-entry-compare\n\t|idlwave-routine-info|idlwave-routine-source-file|idlwave-routine-twin-compare|idlwave-routine-twins|idlwave-routines|idlwave-rw-case\n\t|idlwave-save-buffer-update|idlwave-save-routine-info|idlwave-scan-class-info|idlwave-scan-library-catalogs|idlwave-scan-user-lib-files\n\t|idlwave-scroll-completions|idlwave-selector|idlwave-set-local|idlwave-setup|idlwave-shell-break-here|idlwave-shell-compile-helper-routines\n\t|idlwave-shell-filter-sysvars|idlwave-shell-recenter-shell-window|idlwave-shell-run-region|idlwave-shell-save-and-run|idlwave-shell-send-command\n\t|idlwave-shell-show-commentary|idlwave-shell-update-routine-info|idlwave-shell|idlwave-shorten-syntax|idlwave-show-begin-check\n\t|idlwave-show-begin|idlwave-show-commentary|idlwave-show-matching-quote|idlwave-sintern-class-info|idlwave-sintern-class-tag|idlwave-sintern-class\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t idlwave-sintern-dir|idlwave-sintern-keyword-list|idlwave-sintern-keyword|idlwave-sintern-libname|idlwave-sintern-method|idlwave-sintern-rinfo-list\n\t|idlwave-sintern-routine-or-method|idlwave-sintern-routine|idlwave-sintern-set|idlwave-sintern-sysvar-alist|idlwave-sintern-sysvar\n\t|idlwave-sintern-sysvartag|idlwave-sintern|idlwave-skip-label-or-case|idlwave-skip-multi-commands|idlwave-skip-object|idlwave-special-lib-test\n\t|idlwave-split-line|idlwave-split-link-target|idlwave-split-menu-emacs|idlwave-split-menu-xemacs|idlwave-split-string|idlwave-start-load-rinfo-timer\n\t|idlwave-start-of-substatement|idlwave-statement-type|idlwave-struct-borders|idlwave-struct-inherits|idlwave-struct-tags|idlwave-study-twins\n\t|idlwave-substitute-link-target|idlwave-surround|idlwave-switch|idlwave-sys-dir|idlwave-syslib-p|idlwave-syslib-scanned-p|idlwave-sysvars-reset\n\t|idlwave-template|idlwave-this-word|idlwave-toggle-comment-region|idlwave-true-path-alist|idlwave-uniquify|idlwave-unit-name|idlwave-update-buffer-routine-info\n\t|idlwave-update-current-buffer-info|idlwave-update-routine-info|idlwave-user-catalog-command-hook|idlwave-what-function|idlwave-what-module-find-class\n\t|idlwave-what-module|idlwave-what-procedure|idlwave-where|idlwave-while|idlwave-widget-scan-user-lib-files|idlwave-with-special-syntax\n\t|idlwave-write-paths|idlwave-xml-create-class-method-lists|idlwave-xml-create-rinfo-list|idlwave-xml-create-sysvar-alist|idlwave-xml-system-routine-info-up-to-date\n\t|idlwave-xor|idna-to-ascii|ido-active|ido-add-virtual-buffers-to-list|ido-all-completions|ido-buffer-internal|ido-buffer-window-other-frame\n\t|ido-bury-buffer-at-head|ido-cache-ftp-valid|ido-cache-unc-valid|ido-choose-completion-string|ido-chop|ido-common-initialization\n\t|ido-complete-space|ido-complete|ido-completing-read|ido-completion-help|ido-completions|ido-copy-current-file-name|ido-copy-current-word\n\t|ido-delete-backward-updir|ido-delete-backward-word-updir|ido-delete-file-at-head|ido-directory-too-big-p|ido-dired|ido-display-buffer\n\t|ido-display-file|ido-edit-input|ido-enter-dired|ido-enter-find-file|ido-enter-insert-buffer|ido-enter-insert-file|ido-enter-switch-buffer\n\t|ido-everywhere|ido-exhibit|ido-existing-item-p|ido-exit-minibuffer|ido-expand-directory|ido-fallback-command|ido-file-extension-aux\n\t|ido-file-extension-lessp|ido-file-extension-order|ido-file-internal|ido-file-lessp|ido-file-name-all-completions-1|ido-file-name-all-completions\n\t|ido-final-slash|ido-find-alternate-file|ido-find-common-substring|ido-find-file-in-dir|ido-find-file-other-frame|ido-find-file-other-window\n\t|ido-find-file-read-only-other-frame|ido-find-file-read-only-other-window|ido-find-file-read-only|ido-find-file|ido-flatten-merged-list\n\t|ido-forget-work-directory|ido-fractionp|ido-get-buffers-in-frames|ido-get-bufname|ido-get-work-directory|ido-get-work-file|ido-ignore-item-p\n\t|ido-init-completion-maps|ido-initiate-auto-merge|ido-insert-buffer|ido-insert-file|ido-is-ftp-directory|ido-is-root-directory\n\t|ido-is-slow-ftp-host|ido-is-tramp-root|ido-is-unc-host|ido-is-unc-root|ido-kill-buffer-at-head|ido-kill-buffer|ido-kill-emacs-hook\n\t|ido-list-directory|ido-load-history|ido-local-file-exists-p|ido-magic-backward-char|ido-magic-delete-char|ido-magic-forward-char\n\t|ido-make-buffer-list-1|ido-make-buffer-list|ido-make-choice-list|ido-make-dir-list-1|ido-make-dir-list|ido-make-directory|ido-make-file-list-1\n\t|ido-make-file-list|ido-make-merged-file-list-1|ido-make-merged-file-list|ido-make-prompt|ido-makealist|ido-may-cache-directory\n\t|ido-merge-work-directories|ido-minibuffer-setup|ido-mode|ido-name|ido-next-match-dir|ido-next-match|ido-next-work-directory|ido-next-work-file\n\t|ido-no-final-slash|ido-nonreadable-directory-p|ido-pop-dir|ido-pp|ido-prev-match-dir|ido-prev-match|ido-prev-work-directory|ido-prev-work-file\n\t|ido-push-dir-first|ido-push-dir|ido-read-buffer|ido-read-directory-name|ido-read-file-name|ido-read-internal|ido-record-command\n\t|ido-record-work-directory|ido-record-work-file|ido-remove-cached-dir|ido-reread-directory|ido-restrict-to-matches|ido-save-history\n\t|ido-select-text|ido-set-common-completion|ido-set-current-directory|ido-set-current-home|ido-set-matches-1|ido-set-matches|ido-setup-completion-map\n\t|ido-sort-merged-list|ido-summary-buffers-to-end|ido-switch-buffer-other-frame|ido-switch-buffer-other-window|ido-switch-buffer\n\t|ido-take-first-match|ido-tidy|ido-time-stamp|ido-to-end|ido-toggle-case|ido-toggle-ignore|ido-toggle-literal|ido-toggle-prefix|ido-toggle-regexp\n\t|ido-toggle-trace|ido-toggle-vc|ido-toggle-virtual-buffers|ido-trace|ido-unc-hosts-net-view|ido-unc-hosts|ido-undo-merge-work-directory\n\t|ido-unload-function|ido-up-directory|ido-visit-buffer|ido-wash-history|ido-wide-find-dir-or-delete-dir|ido-wide-find-dir|ido-wide-find-dirs-or-files\n\t|ido-wide-find-file-or-pop-dir|ido-wide-find-file|ido-word-matching-substring|ido-write-file|ielm|ietf-drums-get-comment|ietf-drums-init\n\t|ietf-drums-make-address|ietf-drums-narrow-to-header|ietf-drums-parse-address|ietf-drums-parse-addresses|ietf-drums-parse-date\n\t|ietf-drums-quote-string|ietf-drums-remove-comments|ietf-drums-remove-whitespace|ietf-drums-strip|ietf-drums-token-to-list|ietf-drums-unfold-fws\n\t|if-let|ifconfig|iimage-mode-buffer|iimage-mode|iimage-modification-hook|iimage-recenter|image--set-speed|image-after-revert-hook\n\t|image-animate-get-speed|image-animate-set-speed|image-animate-timeout|image-animated-p|image-backward-hscroll|image-bob|image-bol\n\t|image-bookmark-jump|image-bookmark-make-record|image-decrease-speed|image-dired--with-db-file|image-dired-add-to-file-comment-list\n\t|image-dired-add-to-tag-file-list|image-dired-add-to-tag-file-lists|image-dired-associated-dired-buffer-window|image-dired-associated-dired-buffer\n\t|image-dired-backward-image|image-dired-comment-thumbnail|image-dired-copy-with-exif-file-name|image-dired-create-display-image-buffer\n\t|image-dired-create-gallery-lists|image-dired-create-thumb|image-dired-create-thumbnail-buffer|image-dired-create-thumbs|image-dired-define-display-image-mode-keymap\n\t|image-dired-define-thumbnail-mode-keymap|image-dired-delete-char|image-dired-delete-tag|image-dired-dir|image-dired-dired-after-readin-hook\n\t|image-dired-dired-comment-files|image-dired-dired-display-external|image-dired-dired-display-image|image-dired-dired-display-properties\n\t|image-dired-dired-edit-comment-and-tags|image-dired-dired-file-marked-p|image-dired-dired-next-line|image-dired-dired-previous-line\n\t|image-dired-dired-toggle-marked-thumbs|image-dired-dired-with-window-configuration|image-dired-display-current-image-full|image-dired-display-current-image-sized\n\t|image-dired-display-image-mode|image-dired-display-image|image-dired-display-next-thumbnail-original|image-dired-display-previous-thumbnail-original\n\t|image-dired-display-thumb-properties|image-dired-display-thumb|image-dired-display-thumbnail-original-image|image-dired-display-thumbs-append\n\t|image-dired-display-thumbs|image-dired-display-window-height|image-dired-display-window-width|image-dired-display-window|image-dired-flag-thumb-original-file\n\t|image-dired-format-properties-string|image-dired-forward-image|image-dired-gallery-generate|image-dired-get-buffer-window|image-dired-get-comment\n\t|image-dired-get-exif-data|image-dired-get-exif-file-name|image-dired-get-thumbnail-image|image-dired-hidden-p|image-dired-image-at-point-p\n\t|image-dired-insert-image|image-dired-insert-thumbnail|image-dired-jump-original-dired-buffer|image-dired-jump-thumbnail-buffer\n\t|image-dired-kill-buffer-and-window|image-dired-line-up-dynamic|image-dired-line-up-interactive|image-dired-line-up|image-dired-list-tags\n\t|image-dired-mark-and-display-next|image-dired-mark-tagged-files|image-dired-mark-thumb-original-file|image-dired-modify-mark-on-thumb-original-file\n\t|image-dired-mouse-display-image|image-dired-mouse-select-thumbnail|image-dired-mouse-toggle-mark|image-dired-next-line-and-display\n\t|image-dired-next-line|image-dired-original-file-name|image-dired-previous-line-and-display|image-dired-previous-line|image-dired-read-comment\n\t|image-dired-refresh-thumb|image-dired-remove-tag|image-dired-restore-window-configuration|image-dired-rotate-original-left|image-dired-rotate-original-right\n\t|image-dired-rotate-original|image-dired-rotate-thumbnail-left|image-dired-rotate-thumbnail-right|image-dired-rotate-thumbnail\n\t|image-dired-sane-db-file|image-dired-save-information-from-widgets|image-dired-set-exif-data|image-dired-setup-dired-keybindings\n\t|image-dired-show-all-from-dir|image-dired-slideshow-start|image-dired-slideshow-step|image-dired-slideshow-stop|image-dired-tag-files\n\t|image-dired-tag-thumbnail-remove|image-dired-tag-thumbnail|image-dired-thumb-name|image-dired-thumbnail-display-external|image-dired-thumbnail-mode\n\t|image-dired-thumbnail-set-image-description|image-dired-thumbnail-window|image-dired-toggle-append-browsing|image-dired-toggle-dired-display-properties\n\t|image-dired-toggle-mark-thumb-original-file|image-dired-toggle-movement-tracking|image-dired-track-original-file|image-dired-track-thumbnail\n\t|image-dired-unmark-thumb-original-file|image-dired-update-property|image-dired-window-height-pixels|image-dired-window-width-pixels\n\t|image-dired-write-comments|image-dired-write-tags|image-dired|image-display-size|image-eob|image-eol|image-extension-data|image-file-call-underlying\n\t|image-file-handler|image-file-name-regexp|image-file-yank-handler|image-forward-hscroll|image-get-display-property|image-goto-frame\n\t|image-increase-speed|image-jpeg-p|image-metadata|image-minor-mode|image-mode--images-in-directory|image-mode-as-text|image-mode-fit-frame\n\t|image-mode-maybe|image-mode-menu|image-mode-reapply-winprops|image-mode-setup-winprops|image-mode-window-get|image-mode-window-put\n\t|image-mode-winprops|image-mode|image-next-file|image-next-frame|image-next-line|image-previous-file|image-previous-frame|image-previous-line\n\t|image-refresh|image-reset-speed|image-reverse-speed|image-scroll-down|image-scroll-up|image-search-load-path|image-set-window-hscroll\n\t|image-set-window-vscroll|image-toggle-animation|image-toggle-display-image|image-toggle-display-text|image-toggle-display|image-transform-check-size\n\t|image-transform-fit-to-height|image-transform-fit-to-width|image-transform-fit-width|image-transform-properties|image-transform-reset\n\t|image-transform-set-rotation|image-transform-set-scale|image-transform-width|image-type-auto-detected-p|image-type-from-buffer\n\t|image-type-from-data|image-type-from-file-header|image-type-from-file-name|image-type|imagemagick-filter-types|imagemagick-register-types\n\t|imap-add-callback|imap-anonymous-auth|imap-anonymous-p|imap-arrival-filter|imap-authenticate|imap-body-lines|imap-capability|imap-close\n\t|imap-cram-md5-auth|imap-cram-md5-p|imap-current-mailbox-p-1|imap-current-mailbox-p|imap-current-mailbox|imap-current-message|imap-digest-md5-auth\n\t|imap-digest-md5-p|imap-disable-multibyte|imap-envelope-from|imap-error-text|imap-fetch-asynch|imap-fetch-safe|imap-fetch|imap-find-next-line\n\t|imap-forward|imap-gssapi-auth-p|imap-gssapi-auth|imap-gssapi-open|imap-gssapi-stream-p|imap-id|imap-interactive-login|imap-kerberos4-auth-p\n\t|imap-kerberos4-auth|imap-kerberos4-open|imap-kerberos4-stream-p|imap-list-to-message-set|imap-log|imap-login-auth|imap-login-p\n\t|imap-logout-wait|imap-logout|imap-mailbox-acl-delete|imap-mailbox-acl-get|imap-mailbox-acl-set|imap-mailbox-close|imap-mailbox-create-1\n\t|imap-mailbox-create|imap-mailbox-delete|imap-mailbox-examine-1|imap-mailbox-examine|imap-mailbox-expunge|imap-mailbox-get-1|imap-mailbox-get\n\t|imap-mailbox-list|imap-mailbox-lsub|imap-mailbox-map-1|imap-mailbox-map|imap-mailbox-put|imap-mailbox-rename|imap-mailbox-select-1\n\t|imap-mailbox-select|imap-mailbox-status-asynch|imap-mailbox-status|imap-mailbox-subscribe|imap-mailbox-unselect|imap-mailbox-unsubscribe\n\t|imap-message-append|imap-message-appenduid-1|imap-message-appenduid|imap-message-body|imap-message-copy|imap-message-copyuid-1\n\t|imap-message-copyuid|imap-message-envelope-bcc|imap-message-envelope-cc|imap-message-envelope-date|imap-message-envelope-from\n\t|imap-message-envelope-in-reply-to|imap-message-envelope-message-id|imap-message-envelope-reply-to|imap-message-envelope-sender\n\t|imap-message-envelope-subject|imap-message-envelope-to|imap-message-flag-permanent-p|imap-message-flags-add|imap-message-flags-del\n\t|imap-message-flags-set|imap-message-get|imap-message-map|imap-message-put|imap-namespace|imap-network-open|imap-network-p|imap-ok-p\n\t|imap-open-1|imap-open|imap-opened|imap-parse-acl|imap-parse-address-list|imap-parse-address|imap-parse-astring|imap-parse-body-ext\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t imap-parse-body-extension|imap-parse-body|imap-parse-data-list|imap-parse-envelope|imap-parse-fetch-body-section|imap-parse-fetch\n\t|imap-parse-flag-list|imap-parse-greeting|imap-parse-header-list|imap-parse-literal|imap-parse-mailbox|imap-parse-nil|imap-parse-nstring\n\t|imap-parse-number|imap-parse-resp-text-code|imap-parse-resp-text|imap-parse-response|imap-parse-status|imap-parse-string-list\n\t|imap-parse-string|imap-ping-server|imap-quote-specials|imap-range-to-message-set|imap-remassoc|imap-sasl-auth-p|imap-sasl-auth\n\t|imap-sasl-make-mechanisms|imap-search|imap-send-command-1|imap-send-command-wait|imap-send-command|imap-sentinel|imap-shell-open\n\t|imap-shell-p|imap-ssl-open|imap-ssl-p|imap-starttls-open|imap-starttls-p|imap-string-to-integer|imap-tls-open|imap-tls-p|imap-utf7-decode\n\t|imap-utf7-encode|imap-wait-for-tag|imenu--cleanup|imenu--completion-buffer|imenu--create-keymap|imenu--generic-function|imenu--in-alist\n\t|imenu--make-index-alist|imenu--menubar-select|imenu--mouse-menu|imenu--relative-position|imenu--sort-by-name|imenu--sort-by-position\n\t|imenu--split-menu|imenu--split-submenus|imenu--split|imenu--subalist-p|imenu--truncate-items|imenu-add-menubar-index|imenu-choose-buffer-index\n\t|imenu-default-create-index-function|imenu-default-goto-function|imenu-example--create-c-index|imenu-example--create-lisp-index\n\t|imenu-example--lisp-extract-index-name|imenu-example--name-and-position|imenu-find-default|imenu-progress-message|imenu-update-menubar\n\t|imenu|in-is13194-post-read-conversion|in-is13194-pre-write-conversion|in-string-p|inactivate-input-method|incf|increase-left-margin\n\t|increase-right-margin|increment-register|indent-accumulate-tab-stops|indent-for-comment|indent-icon-exp|indent-line-to|indent-new-comment-line\n\t|indent-next-tab-stop|indent-perl-exp|indent-pp-sexp|indent-rigidly--current-indentation|indent-rigidly--pop-undo|indent-rigidly-left-to-tab-stop\n\t|indent-rigidly-left|indent-rigidly-right-to-tab-stop|indent-rigidly-right|indent-sexp|indent-tcl-exp|indent-to-column|indented-text-mode\n\t|indian-2-column-to-ucs-region|indian-compose-regexp|indian-compose-region|indian-compose-string|indicate-copied-region|inferior-lisp-install-letter-bindings\n\t|inferior-lisp-menu|inferior-lisp-mode|inferior-lisp-proc|inferior-lisp|inferior-octave-check-process|inferior-octave-complete\n\t|inferior-octave-completion-at-point|inferior-octave-completion-table|inferior-octave-directory-tracker|inferior-octave-dynamic-list-input-ring\n\t|inferior-octave-mode|inferior-octave-output-digest|inferior-octave-process-live-p|inferior-octave-resync-dirs|inferior-octave-send-list-and-digest\n\t|inferior-octave-startup|inferior-octave-track-window-width-change|inferior-octave|inferior-python-mode|inferior-scheme-mode|inferior-tcl-mode\n\t|inferior-tcl-proc|inferior-tcl|info--manual-names|info--prettify-description|info-apropos|info-complete-file|info-complete-symbol\n\t|info-complete|info-display-manual|info-emacs-bug|info-emacs-manual|info-file-exists-p|info-finder|info-initialize|info-insert-file-contents-1\n\t|info-insert-file-contents|info-lookup->all-modes|info-lookup->cache|info-lookup->completions|info-lookup->doc-spec|info-lookup->ignore-case\n\t|info-lookup->initialized|info-lookup->mode-cache|info-lookup->mode-value|info-lookup->other-modes|info-lookup->parse-rule|info-lookup->refer-modes\n\t|info-lookup->regexp|info-lookup->topic-cache|info-lookup->topic-value|info-lookup-add-help\\*|info-lookup-add-help|info-lookup-change-mode\n\t|info-lookup-completions-at-point|info-lookup-file|info-lookup-guess-c-symbol|info-lookup-guess-custom-symbol|info-lookup-guess-default\\*\n\t|info-lookup-guess-default|info-lookup-interactive-arguments|info-lookup-make-completions|info-lookup-maybe-add-help|info-lookup-quick-all-modes\n\t|info-lookup-reset|info-lookup-select-mode|info-lookup-setup-mode|info-lookup-symbol|info-lookup|info-other-window|info-setup|info-standalone\n\t|info-xref-all-info-files|info-xref-check-all-custom|info-xref-check-all|info-xref-check-buffer|info-xref-check-list|info-xref-check-node\n\t|info-xref-check|info-xref-docstrings|info-xref-goto-node-p|info-xref-lock-file-p|info-xref-output-error|info-xref-output|info-xref-subfile-p\n\t|info-xref-with-file|info-xref-with-output|info|inhibit-local-variables-p|init-image-library|initialize-completions|initialize-instance\n\t|initialize-new-tags-table|inline|insert-abbrevs|insert-byte|insert-directory-adj-pos|insert-directory-safely|insert-file-1|insert-file-literally\n\t|insert-file|insert-for-yank-1|insert-image-file|insert-kbd-macro|insert-pair|insert-parentheses|insert-rectangle|insert-string|insert-tab\n\t|int-to-string|interactive-completion-string-reader|interactive-p|intern-safe|internal--after-save-selected-window|internal--after-with-selected-window\n\t|internal--before-save-selected-window|internal--before-with-selected-window|internal--build-binding-value-form|internal--build-binding\n\t|internal--build-bindings|internal--check-binding|internal--listify|internal--thread-argument|internal--track-mouse|internal-ange-ftp-mode\n\t|internal-char-font|internal-complete-buffer-except|internal-complete-buffer|internal-copy-lisp-face|internal-default-process-filter\n\t|internal-default-process-sentinel|internal-describe-syntax-value|internal-event-symbol-parse-modifiers|internal-face-x-get-resource\n\t|internal-get-lisp-face-attribute|internal-lisp-face-attribute-values|internal-lisp-face-empty-p|internal-lisp-face-equal-p|internal-lisp-face-p\n\t|internal-macroexpand-for-load|internal-make-lisp-face|internal-make-var-non-special|internal-merge-in-global-face|internal-pop-keymap\n\t|internal-push-keymap|internal-set-alternative-font-family-alist|internal-set-alternative-font-registry-alist|internal-set-font-selection-order\n\t|internal-set-lisp-face-attribute-from-resource|internal-set-lisp-face-attribute|internal-show-cursor-p|internal-show-cursor\n\t|internal-temp-output-buffer-show|internal-timer-start-idle|intersection|inverse-add-abbrev|inverse-add-global-abbrev|inverse-add-mode-abbrev\n\t|inversion-<|inversion-=|inversion-add-to-load-path|inversion-check-version|inversion-decode-version|inversion-download-package-ask\n\t|inversion-find-version|inversion-locate-package-files-and-split|inversion-locate-package-files|inversion-package-incompatibility-version\n\t|inversion-package-version|inversion-recode|inversion-release-to-number|inversion-require-emacs|inversion-require|inversion-reverse-test\n\t|inversion-test|ipconfig|irc|isInNet|isPlainHostName|isResolvable|isearch--get-state|isearch--set-state|isearch--state-barrier--cmacro\n\t|isearch--state-barrier|isearch--state-case-fold-search--cmacro|isearch--state-case-fold-search|isearch--state-error--cmacro\n\t|isearch--state-error|isearch--state-forward--cmacro|isearch--state-forward|isearch--state-message--cmacro|isearch--state-message\n\t|isearch--state-other-end--cmacro|isearch--state-other-end|isearch--state-p--cmacro|isearch--state-p|isearch--state-point--cmacro\n\t|isearch--state-point|isearch--state-pop-fun--cmacro|isearch--state-pop-fun|isearch--state-string--cmacro|isearch--state-string\n\t|isearch--state-success--cmacro|isearch--state-success|isearch--state-word--cmacro|isearch--state-word|isearch--state-wrapped--cmacro\n\t|isearch--state-wrapped|isearch-abort|isearch-back-into-window|isearch-backslash|isearch-backward-regexp|isearch-backward|isearch-cancel\n\t|isearch-char-by-name|isearch-clean-overlays|isearch-close-unnecessary-overlays|isearch-complete-edit|isearch-complete|isearch-complete1\n\t|isearch-dehighlight|isearch-del-char|isearch-delete-char|isearch-describe-bindings|isearch-describe-key|isearch-describe-mode\n\t|isearch-done|isearch-edit-string|isearch-exit|isearch-fail-pos|isearch-fallback|isearch-filter-visible|isearch-forward-exit-minibuffer\n\t|isearch-forward-regexp|isearch-forward-symbol-at-point|isearch-forward-symbol|isearch-forward-word|isearch-forward|isearch-help-for-help-internal-doc\n\t|isearch-help-for-help-internal|isearch-help-for-help|isearch-highlight-regexp|isearch-highlight|isearch-intersects-p|isearch-lazy-highlight-cleanup\n\t|isearch-lazy-highlight-new-loop|isearch-lazy-highlight-search|isearch-lazy-highlight-update|isearch-message-prefix|isearch-message-suffix\n\t|isearch-message|isearch-mode-help|isearch-mode|isearch-mouse-2|isearch-no-upper-case-p|isearch-nonincremental-exit-minibuffer\n\t|isearch-occur|isearch-open-necessary-overlays|isearch-open-overlay-temporary|isearch-pop-state|isearch-post-command-hook|isearch-pre-command-hook\n\t|isearch-printing-char|isearch-process-search-char|isearch-process-search-multibyte-characters|isearch-process-search-string\n\t|isearch-push-state|isearch-query-replace-regexp|isearch-query-replace|isearch-quote-char|isearch-range-invisible|isearch-repeat-backward\n\t|isearch-repeat-forward|isearch-repeat|isearch-resume|isearch-reverse-exit-minibuffer|isearch-ring-adjust|isearch-ring-adjust1\n\t|isearch-ring-advance|isearch-ring-retreat|isearch-search-and-update|isearch-search-fun-default|isearch-search-fun|isearch-search-string\n\t|isearch-search|isearch-string-out-of-window|isearch-symbol-regexp|isearch-text-char-description|isearch-toggle-case-fold|isearch-toggle-input-method\n\t|isearch-toggle-invisible|isearch-toggle-lax-whitespace|isearch-toggle-regexp|isearch-toggle-specified-input-method|isearch-toggle-symbol\n\t|isearch-toggle-word|isearch-unread|isearch-update-ring|isearch-update|isearch-yank-char-in-minibuffer|isearch-yank-char|isearch-yank-internal\n\t|isearch-yank-kill|isearch-yank-line|isearch-yank-pop|isearch-yank-string|isearch-yank-word-or-char|isearch-yank-word|isearch-yank-x-selection\n\t|isearchb-activate|isearchb-follow-char|isearchb-iswitchb|isearchb-set-keybindings|isearchb-stop|isearchb|iso-charset|iso-cvt-define-menu\n\t|iso-cvt-read-only|iso-cvt-write-only|iso-german|iso-gtex2iso|iso-iso2duden|iso-iso2gtex|iso-iso2sgml|iso-iso2tex|iso-sgml2iso|iso-spanish\n\t|iso-tex2iso|iso-transl-ctl-x-8-map|ispell-accept-buffer-local-defs|ispell-accept-output|ispell-add-per-file-word-list|ispell-aspell-add-aliases\n\t|ispell-aspell-find-dictionary|ispell-begin-skip-region-regexp|ispell-begin-skip-region|ispell-begin-tex-skip-regexp|ispell-buffer-local-dict\n\t|ispell-buffer-local-parsing|ispell-buffer-local-words|ispell-buffer-with-debug|ispell-buffer|ispell-call-process-region|ispell-call-process\n\t|ispell-change-dictionary|ispell-check-minver|ispell-check-version|ispell-command-loop|ispell-comments-and-strings|ispell-complete-word-interior-frag\n\t|ispell-complete-word|ispell-continue|ispell-create-debug-buffer|ispell-decode-string|ispell-display-buffer|ispell-filter|ispell-find-aspell-dictionaries\n\t|ispell-find-hunspell-dictionaries|ispell-get-aspell-config-value|ispell-get-casechars|ispell-get-coding-system|ispell-get-decoded-string\n\t|ispell-get-extended-character-mode|ispell-get-ispell-args|ispell-get-line|ispell-get-many-otherchars-p|ispell-get-not-casechars\n\t|ispell-get-otherchars|ispell-get-word|ispell-help|ispell-highlight-spelling-error-generic|ispell-highlight-spelling-error-overlay\n\t|ispell-highlight-spelling-error-xemacs|ispell-highlight-spelling-error|ispell-horiz-scroll|ispell-hunspell-fill-dictionary-entry\n\t|ispell-ignore-fcc|ispell-init-process|ispell-int-char|ispell-internal-change-dictionary|ispell-kill-ispell|ispell-looking-at|ispell-looking-back\n\t|ispell-lookup-words|ispell-menu-map|ispell-message|ispell-mime-multipartp|ispell-mime-skip-part|ispell-minor-check|ispell-minor-mode\n\t|ispell-non-empty-string|ispell-parse-hunspell-affix-file|ispell-parse-output|ispell-pdict-save|ispell-print-if-debug|ispell-process-line\n\t|ispell-process-status|ispell-region|ispell-send-replacement|ispell-send-string|ispell-set-spellchecker-params|ispell-show-choices\n\t|ispell-skip-region-list|ispell-skip-region|ispell-start-process|ispell-tex-arg-end|ispell-valid-dictionary-list|ispell-with-no-warnings\n\t|ispell-word|ispell|isqrt|iswitchb-buffer-other-frame|iswitchb-buffer-other-window|iswitchb-buffer|iswitchb-case|iswitchb-chop|iswitchb-complete\n\t|iswitchb-completion-help|iswitchb-completions|iswitchb-display-buffer|iswitchb-entryfn-p|iswitchb-exhibit|iswitchb-existing-buffer-p\n\t|iswitchb-exit-minibuffer|iswitchb-find-common-substring|iswitchb-find-file|iswitchb-get-buffers-in-frames|iswitchb-get-bufname\n\t|iswitchb-get-matched-buffers|iswitchb-ignore-buffername-p|iswitchb-init-XEmacs-trick|iswitchb-kill-buffer|iswitchb-make-buflist\n\t|iswitchb-makealist|iswitchb-minibuffer-setup|iswitchb-mode|iswitchb-next-match|iswitchb-output-completion|iswitchb-possible-new-buffer\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t iswitchb-post-command|iswitchb-pre-command|iswitchb-prev-match|iswitchb-read-buffer|iswitchb-rotate-list|iswitchb-select-buffer-text\n\t|iswitchb-set-common-completion|iswitchb-set-matches|iswitchb-summaries-to-end|iswitchb-tidy|iswitchb-to-end|iswitchb-toggle-case\n\t|iswitchb-toggle-ignore|iswitchb-toggle-regexp|iswitchb-visit-buffer|iswitchb-window-buffer-p|iswitchb-word-matching-substring\n\t|iswitchb-xemacs-backspacekey|iswitchb|iwconfig|japanese-hankaku-region|japanese-hankaku|japanese-hiragana-region|japanese-hiragana\n\t|japanese-katakana-region|japanese-katakana|japanese-zenkaku-region|japanese-zenkaku|java-font-lock-keywords-2|java-font-lock-keywords-3\n\t|java-font-lock-keywords|java-mode|javascript-mode|jdb|jit-lock--debug-fontify|jit-lock-after-change|jit-lock-context-fontify|jit-lock-debug-mode\n\t|jit-lock-deferred-fontify|jit-lock-fontify-now|jit-lock-force-redisplay|jit-lock-function|jit-lock-mode|jit-lock-refontify|jit-lock-stealth-chunk-start\n\t|jit-lock-stealth-fontify|jka-compr-build-file-regexp|jka-compr-byte-compiler-base-file-name|jka-compr-call-process|jka-compr-error\n\t|jka-compr-file-local-copy|jka-compr-get-compression-info|jka-compr-handler|jka-compr-info-can-append|jka-compr-info-compress-args\n\t|jka-compr-info-compress-message|jka-compr-info-compress-program|jka-compr-info-file-magic-bytes|jka-compr-info-regexp|jka-compr-info-strip-extension\n\t|jka-compr-info-uncompress-args|jka-compr-info-uncompress-message|jka-compr-info-uncompress-program|jka-compr-insert-file-contents\n\t|jka-compr-install|jka-compr-installed-p|jka-compr-load|jka-compr-make-temp-name|jka-compr-partial-uncompress|jka-compr-run-real-handler\n\t|jka-compr-set|jka-compr-uninstall|jka-compr-update|jka-compr-write-region|join-line|js--array-comp-indentation|js--backward-pstate\n\t|js--backward-syntactic-ws|js--backward-text-property|js--beginning-of-defun-flat|js--beginning-of-defun-nested|js--beginning-of-defun-raw\n\t|js--beginning-of-macro|js--class-decl-matcher|js--clear-stale-cache|js--continued-expression-p|js--ctrl-statement-indentation\n\t|js--debug|js--end-of-defun-flat|js--end-of-defun-nested|js--end-of-do-while-loop-p|js--ensure-cache--pop-if-ended|js--ensure-cache--update-parse\n\t|js--ensure-cache|js--flatten-list|js--flush-caches|js--forward-destructuring-spec|js--forward-expression|js--forward-function-decl\n\t|js--forward-pstate|js--forward-syntactic-ws|js--forward-text-property|js--function-prologue-beginning|js--get-all-known-symbols\n\t|js--get-c-offset|js--get-js-context|js--get-tabs|js--guess-eval-defun-info|js--guess-function-name|js--guess-symbol-at-point|js--imenu-create-index\n\t|js--imenu-to-flat|js--indent-in-array-comp|js--inside-dojo-class-list-p|js--inside-param-list-p|js--inside-pitem-p|js--js-add-resource-alias\n\t|js--js-content-window|js--js-create-instance|js--js-decode-retval|js--js-encode-value|js--js-enter-repl|js--js-eval|js--js-funcall\n\t|js--js-get-service|js--js-get|js--js-handle-expired-p|js--js-handle-id--cmacro|js--js-handle-id|js--js-handle-p--cmacro|js--js-handle-p\n\t|js--js-handle-process--cmacro|js--js-handle-process|js--js-leave-repl|js--js-list|js--js-new|js--js-not|js--js-put|js--js-qi|js--js-true\n\t|js--js-wait-for-eval-prompt|js--looking-at-operator-p|js--make-framework-matcher|js--make-merged-item|js--make-nsilocalfile|js--maybe-join\n\t|js--maybe-make-marker|js--multi-line-declaration-indentation|js--optimize-arglist|js--parse-state-at-point|js--pitem-add-child\n\t|js--pitem-b-end--cmacro|js--pitem-b-end|js--pitem-children--cmacro|js--pitem-children|js--pitem-format|js--pitem-goto-h-end|js--pitem-h-begin--cmacro\n\t|js--pitem-h-begin|js--pitem-name--cmacro|js--pitem-name|js--pitem-paren-depth--cmacro|js--pitem-paren-depth|js--pitem-strname\n\t|js--pitem-type--cmacro|js--pitem-type|js--pitems-to-imenu|js--proper-indentation|js--pstate-is-toplevel-defun|js--re-search-backward-inner\n\t|js--re-search-backward|js--re-search-forward-inner|js--re-search-forward|js--read-symbol|js--read-tab|js--regexp-opt-symbol|js--same-line\n\t|js--show-cache-at-point|js--splice-into-items|js--split-name|js--syntactic-context-from-pstate|js--syntax-begin-function|js--up-nearby-list\n\t|js--update-quick-match-re|js--variable-decl-matcher|js--wait-for-matching-output|js--which-func-joiner|js-beginning-of-defun\n\t|js-c-fill-paragraph|js-end-of-defun|js-eval-defun|js-eval|js-find-symbol|js-gc|js-indent-line|js-mode|js-set-js-context|js-syntactic-context\n\t|js-syntax-propertize-regexp|js-syntax-propertize|json--with-indentation|json-add-to-object|json-advance|json-alist-p|json-decode-char0\n\t|json-encode-alist|json-encode-array|json-encode-char|json-encode-char0|json-encode-hash-table|json-encode-key|json-encode-keyword\n\t|json-encode-list|json-encode-number|json-encode-plist|json-encode-string|json-encode|json-join|json-new-object|json-peek|json-plist-p\n\t|json-pop|json-pretty-print-buffer|json-pretty-print|json-read-array|json-read-escaped-char|json-read-file|json-read-from-string\n\t|json-read-keyword|json-read-number|json-read-object|json-read-string|json-read|json-skip-whitespace|jump-to-register|kbd-macro-query\n\t|keep-lines-read-args|keep-lines|kermit-clean-filter|kermit-clean-off|kermit-clean-on|kermit-default-cr|kermit-default-nl|kermit-esc\n\t|kermit-send-char|kermit-send-input-cr|keyboard-escape-quit|keymap--menu-item-binding|keymap--menu-item-with-binding|keymap--merge-bindings\n\t|keymap-canonicalize|keypad-setup|kill-all-abbrevs|kill-backward-chars|kill-backward-up-list|kill-buffer-and-window|kill-buffer-ask\n\t|kill-buffer-if-not-modified|kill-comment|kill-compilation|kill-completion|kill-emacs-save-completions|kill-find|kill-forward-chars\n\t|kill-grep|kill-line|kill-matching-buffers|kill-paragraph|kill-rectangle|kill-ring-save|kill-sentence|kill-sexp|kill-some-buffers\n\t|kill-this-buffer-enabled-p|kill-this-buffer|kill-visual-line|kill-whole-line|kill-word|kinsoku-longer|kinsoku-shorter|kinsoku|kkc-region\n\t|kmacro-add-counter|kmacro-bind-to-key|kmacro-call-macro|kmacro-call-ring-2nd-repeat|kmacro-call-ring-2nd|kmacro-cycle-ring-next\n\t|kmacro-cycle-ring-previous|kmacro-delete-ring-head|kmacro-display-counter|kmacro-display|kmacro-edit-lossage|kmacro-edit-macro-repeat\n\t|kmacro-edit-macro|kmacro-end-and-call-macro|kmacro-end-call-mouse|kmacro-end-macro|kmacro-end-or-call-macro-repeat|kmacro-end-or-call-macro\n\t|kmacro-exec-ring-item|kmacro-execute-from-register|kmacro-extract-lambda|kmacro-get-repeat-prefix|kmacro-insert-counter|kmacro-keyboard-quit\n\t|kmacro-lambda-form|kmacro-loop-setup-function|kmacro-name-last-macro|kmacro-pop-ring|kmacro-pop-ring1|kmacro-push-ring|kmacro-repeat-on-last-key\n\t|kmacro-ring-empty-p|kmacro-ring-head|kmacro-set-counter|kmacro-set-format|kmacro-split-ring-element|kmacro-start-macro-or-insert-counter\n\t|kmacro-start-macro|kmacro-step-edit-insert|kmacro-step-edit-macro|kmacro-step-edit-minibuf-setup|kmacro-step-edit-post-command\n\t|kmacro-step-edit-pre-command|kmacro-step-edit-prompt|kmacro-step-edit-query|kmacro-swap-ring|kmacro-to-register|kmacro-view-macro-repeat\n\t|kmacro-view-macro|kmacro-view-ring-2nd|lambda|landmark--distance|landmark--intangible|landmark-amble-robot|landmark-beginning-of-line\n\t|landmark-blackbox|landmark-calc-confidences|landmark-calc-current-smells|landmark-calc-distance-of-robot-from|landmark-calc-payoff\n\t|landmark-calc-smell-internal|landmark-check-filled-qtuple|landmark-click|landmark-confidence-for|landmark-crash-game|landmark-cross-qtuple\n\t|landmark-display-statistics|landmark-emacs-plays|landmark-end-of-line|landmark-f|landmark-find-filled-qtuple|landmark-fix-weights-for\n\t|landmark-flip-a-coin|landmark-goto-square|landmark-goto-xy|landmark-human-plays|landmark-human-resigns|landmark-human-takes-back\n\t|landmark-index-to-x|landmark-index-to-y|landmark-init-board|landmark-init-display|landmark-init-score-table|landmark-init-square-score\n\t|landmark-init|landmark-max-height|landmark-max-width|landmark-mode|landmark-mouse-play|landmark-move-down|landmark-move-ne|landmark-move-nw\n\t|landmark-move-se|landmark-move-sw|landmark-move-up|landmark-move|landmark-nb-qtuples|landmark-noise|landmark-nslify-wts-int|landmark-nslify-wts\n\t|landmark-offer-a-draw|landmark-play-move|landmark-plot-internal|landmark-plot-landmarks|landmark-plot-square|landmark-point-square\n\t|landmark-point-y|landmark-print-distance-int|landmark-print-distance|landmark-print-moves|landmark-print-smell-int|landmark-print-smell\n\t|landmark-print-w0-int|landmark-print-w0|landmark-print-wts-blackbox|landmark-print-wts-int|landmark-print-wts|landmark-print-y-s-noise-int\n\t|landmark-print-y-s-noise|landmark-prompt-for-move|landmark-prompt-for-other-game|landmark-random-move|landmark-randomize-weights-for\n\t|landmark-repeat|landmark-set-landmark-signal-strengths|landmark-start-game|landmark-start-robot|landmark-store-old-y_t|landmark-strongest-square\n\t|landmark-switch-to-window|landmark-take-back|landmark-terminate-game|landmark-test-run|landmark-update-naught-weights|landmark-update-normal-weights\n\t|landmark-update-score-in-direction|landmark-update-score-table|landmark-weights-debug|landmark-xy-to-index|landmark-y|landmark\n\t|lao-compose-region|lao-compose-string|lao-composition-function|lao-transcribe-roman-to-lao-string|lao-transcribe-single-roman-syllable-to-lao\n\t|last-nonminibuffer-frame|last-sexp-setup-props|latex-backward-sexp-1|latex-close-block|latex-complete-bibtex-keys|latex-complete-data\n\t|latex-complete-envnames|latex-complete-refkeys|latex-down-list|latex-electric-env-pair-mode|latex-env-before-change|latex-fill-nobreak-predicate\n\t|latex-find-indent|latex-forward-sexp-1|latex-forward-sexp|latex-imenu-create-index|latex-indent|latex-insert-block|latex-insert-item\n\t|latex-mode|latex-outline-level|latex-skip-close-parens|latex-split-block|latex-string-prefix-p|latex-syntax-after|latexenc-coding-system-to-inputenc\n\t|latexenc-find-file-coding-system|latexenc-inputenc-to-coding-system|latin1-display|lazy-highlight-cleanup|lcm|ld-script-mode|ldap-decode-address\n\t|ldap-decode-attribute|ldap-decode-boolean|ldap-decode-string|ldap-encode-address|ldap-encode-boolean|ldap-encode-country-string\n\t|ldap-encode-string|ldap-get-host-parameter|ldap-search-internal|ldap-search|ldiff|led-flash|led-off|led-on|led-update|left-char|left-word\n\t|let-alist--access-sexp|let-alist--deep-dot-search|let-alist--list-to-sexp|let-alist--remove-dot|let-alist|letf\\*|letf|letrec|lglyph-adjustment\n\t|lglyph-ascent|lglyph-char|lglyph-code|lglyph-copy|lglyph-descent|lglyph-from|lglyph-lbearing|lglyph-rbearing|lglyph-set-adjustment\n\t|lglyph-set-char|lglyph-set-code|lglyph-set-from-to|lglyph-set-width|lglyph-to|lglyph-width|lgrep|lgstring-char-len|lgstring-char\n\t|lgstring-font|lgstring-glyph-len|lgstring-glyph|lgstring-header|lgstring-insert-glyph|lgstring-set-glyph|lgstring-set-header|lgstring-set-id\n\t|lgstring-shaped-p|life-birth-char|life-birth-string|life-compute-neighbor-deltas|life-death-char|life-death-string|life-display-generation\n\t|life-expand-plane-if-needed|life-extinct-quit|life-grim-reaper|life-increment-generation|life-increment|life-insert-random-pattern\n\t|life-life-char|life-life-string|life-mode|life-not-void-regexp|life-setup|life-void-char|life-void-string|life|limit-index|line-move-1\n\t|line-move-finish|line-move-partial|line-move-to-column|line-move-visual|line-move|line-number-mode|line-pixel-height|line-substring-with-bidi-context\n\t|linum--face-width|linum-after-change|linum-after-scroll|linum-delete-overlays|linum-mode-set-explicitly|linum-mode|linum-on|linum-schedule\n\t|linum-unload-function|linum-update-current|linum-update-window|linum-update|lisp--match-hidden-arg|lisp-comment-indent|lisp-compile-defun-and-go\n\t|lisp-compile-defun|lisp-compile-file|lisp-compile-region-and-go|lisp-compile-region|lisp-compile-string|lisp-complete-symbol|lisp-completion-at-point\n\t|lisp-current-defun-name|lisp-describe-sym|lisp-do-defun|lisp-eval-defun-and-go|lisp-eval-defun|lisp-eval-form-and-next|lisp-eval-last-sexp\n\t|lisp-eval-paragraph|lisp-eval-region-and-go|lisp-eval-region|lisp-eval-string|lisp-fill-paragraph|lisp-find-tag-default|lisp-fn-called-at-pt\n\t|lisp-font-lock-syntactic-face-function|lisp-get-old-input|lisp-indent-defform|lisp-indent-function|lisp-indent-line|lisp-indent-specform\n\t|lisp-input-filter|lisp-interaction-mode|lisp-load-file|lisp-mode-auto-fill|lisp-mode-variables|lisp-mode|lisp-outline-level|lisp-show-arglist\n\t|lisp-show-function-documentation|lisp-show-variable-documentation|lisp-string-after-doc-keyword-p|lisp-string-in-doc-position-p\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t lisp-symprompt|lisp-var-at-pt|list\\*|list-abbrevs|list-all-completions-1|list-all-completions-by-hash-bucket-1|list-all-completions-by-hash-bucket\n\t|list-all-completions|list-at-point|list-bookmarks|list-buffers--refresh|list-buffers-noselect|list-buffers|list-character-sets\n\t|list-coding-categories|list-coding-systems|list-colors-display|list-colors-duplicates|list-colors-print|list-colors-redisplay\n\t|list-colors-sort-key|list-command-history|list-directory|list-dynamic-libraries|list-faces-display|list-fontsets|list-holidays\n\t|list-input-methods|list-length|list-matching-lines|list-packages|list-processes--refresh|list-registers|list-tags|lm-adapted-by\n\t|lm-authors|lm-code-mark|lm-code-start|lm-commentary-end|lm-commentary-mark|lm-commentary-start|lm-commentary|lm-copyright-mark|lm-crack-address\n\t|lm-crack-copyright|lm-creation-date|lm-get-header-re|lm-get-package-name|lm-header-multiline|lm-header|lm-history-mark|lm-history-start\n\t|lm-homepage|lm-insert-at-column|lm-keywords-finder-p|lm-keywords-list|lm-keywords|lm-last-modified-date|lm-maintainer|lm-report-bug\n\t|lm-section-end|lm-section-mark|lm-section-start|lm-summary|lm-synopsis|lm-verify|lm-version|lm-with-file|load-completions-from-file\n\t|load-history-filename-element|load-history-regexp|load-path-shadows-find|load-path-shadows-mode|load-path-shadows-same-file-or-nonexistent\n\t|load-save-place-alist-from-file|load-time-value|load-with-code-conversion|local-clear-scheme-interaction-buffer|local-set-scheme-interaction-buffer\n\t|locale-charset-match-p|locale-charset-to-coding-system|locale-name-match|locale-translate|locally|locate-completion-db-error|locate-completion-entry-retry\n\t|locate-completion-entry|locate-current-line-number|locate-default-make-command-line|locate-do-redisplay|locate-do-setup|locate-dominating-file\n\t|locate-file-completion-table|locate-file-completion|locate-file-internal|locate-filter-output|locate-find-directory-other-window\n\t|locate-find-directory|locate-get-dirname|locate-get-file-positions|locate-get-filename|locate-in-alternate-database|locate-insert-header\n\t|locate-main-listing-line-p|locate-mode|locate-mouse-view-file|locate-prompt-for-search-string|locate-set-properties|locate-tags\n\t|locate-update|locate-with-filter|locate-word-at-point|locate|log-edit--match-first-line|log-edit-add-field|log-edit-add-to-changelog\n\t|log-edit-beginning-of-line|log-edit-changelog-entries|log-edit-changelog-entry|log-edit-changelog-insert-entries|log-edit-changelog-ours-p\n\t|log-edit-changelog-paragraph|log-edit-changelog-subparagraph|log-edit-comment-search-backward|log-edit-comment-search-forward\n\t|log-edit-comment-to-change-log|log-edit-done|log-edit-empty-buffer-p|log-edit-extract-headers|log-edit-files|log-edit-font-lock-keywords\n\t|log-edit-goto-eoh|log-edit-hide-buf|log-edit-insert-changelog-entries|log-edit-insert-changelog|log-edit-insert-cvs-rcstemplate\n\t|log-edit-insert-cvs-template|log-edit-insert-filenames-without-changelog|log-edit-insert-filenames|log-edit-insert-message-template\n\t|log-edit-kill-buffer|log-edit-match-to-eoh|log-edit-menu|log-edit-mode-help|log-edit-mode|log-edit-narrow-changelog|log-edit-new-comment-index\n\t|log-edit-next-comment|log-edit-previous-comment|log-edit-remember-comment|log-edit-set-common-indentation|log-edit-set-header\n\t|log-edit-show-diff|log-edit-show-files|log-edit-toggle-header|log-edit|log-view-annotate-version|log-view-beginning-of-defun|log-view-current-entry\n\t|log-view-current-file|log-view-current-tag|log-view-diff-changeset|log-view-diff-common|log-view-diff|log-view-end-of-defun-1\n\t|log-view-end-of-defun|log-view-extract-comment|log-view-file-next|log-view-file-prev|log-view-find-revision|log-view-get-marked\n\t|log-view-goto-rev|log-view-inside-comment-p|log-view-minor-wrap|log-view-mode-menu|log-view-mode|log-view-modify-change-comment\n\t|log-view-msg-next|log-view-msg-prev|log-view-toggle-entry-display|log-view-toggle-mark-entry|log10|lookfor-dired|lookup-image-map\n\t|lookup-key-ignore-too-long|lookup-minor-mode-from-indicator|lookup-nested-alist|lookup-words|loop|lpr-buffer|lpr-customize|lpr-eval-switch\n\t|lpr-flatten-list-1|lpr-flatten-list|lpr-print-region|lpr-region|lpr-setup|lunar-phases|m2-begin-comment|m2-begin|m2-case|m2-compile\n\t|m2-definition|m2-else|m2-end-comment|m2-execute-monitor-command|m2-export|m2-for|m2-header|m2-if|m2-import|m2-link|m2-loop|m2-mode|m2-module\n\t|m2-or|m2-procedure|m2-record|m2-smie-backward-token|m2-smie-forward-token|m2-smie-refine-colon|m2-smie-refine-of|m2-smie-refine-semi\n\t|m2-smie-rules|m2-stdio|m2-toggle|m2-type|m2-until|m2-var|m2-visit|m2-while|m2-with|m4--quoted-p|m4-current-defun-name|m4-m4-buffer|m4-m4-region\n\t|m4-mode|macro-declaration-function|macroexp--accumulate|macroexp--all-clauses|macroexp--all-forms|macroexp--backtrace|macroexp--compiler-macro\n\t|macroexp--compiling-p|macroexp--cons|macroexp--const-symbol-p|macroexp--expand-all|macroexp--funcall-if-compiled|macroexp--maxsize\n\t|macroexp--obsolete-warning|macroexp--trim-backtrace-frame|macroexp--warn-and-return|macroexp-const-p|macroexp-copyable-p|macroexp-if\n\t|macroexp-let\\*|macroexp-let2\\*|macroexp-let2|macroexp-progn|macroexp-quote|macroexp-small-p|macroexp-unprogn|macroexpand-1|macrolet\n\t|mail-abbrev-complete-alias|mail-abbrev-end-of-buffer|mail-abbrev-expand-hook|mail-abbrev-expand-wrapper|mail-abbrev-in-expansion-header-p\n\t|mail-abbrev-insert-alias|mail-abbrev-make-syntax-table|mail-abbrev-next-line|mail-abbrevs-disable|mail-abbrevs-enable|mail-abbrevs-mode\n\t|mail-abbrevs-setup|mail-abbrevs-sync-aliases|mail-add-attachment|mail-add-payment-async|mail-add-payment|mail-attach-file|mail-bcc\n\t|mail-bury|mail-cc|mail-check-payment|mail-comma-list-regexp|mail-complete|mail-completion-at-point-function|mail-completion-expand\n\t|mail-content-type-get|mail-decode-encoded-address-region|mail-decode-encoded-address-string|mail-decode-encoded-word-region\n\t|mail-decode-encoded-word-string|mail-directory-process|mail-directory-stream|mail-directory|mail-do-fcc|mail-dont-reply-to|mail-dont-send\n\t|mail-encode-encoded-word-buffer|mail-encode-encoded-word-region|mail-encode-encoded-word-string|mail-encode-header|mail-envelope-from\n\t|mail-extract-address-components|mail-fcc|mail-fetch-field|mail-file-babyl-p|mail-fill-yanked-message|mail-get-names|mail-header-chars\n\t|mail-header-date|mail-header-encode-parameter|mail-header-end|mail-header-extra|mail-header-extract-no-properties|mail-header-extract\n\t|mail-header-field-value|mail-header-fold-field|mail-header-format|mail-header-from|mail-header-get-comment|mail-header-id|mail-header-lines\n\t|mail-header-make-address|mail-header-merge|mail-header-message-id|mail-header-narrow-to-field|mail-header-number|mail-header-parse-address\n\t|mail-header-parse-addresses|mail-header-parse-content-disposition|mail-header-parse-content-type|mail-header-parse-date|mail-header-parse\n\t|mail-header-references|mail-header-remove-comments|mail-header-remove-whitespace|mail-header-set-chars|mail-header-set-date|mail-header-set-extra\n\t|mail-header-set-from|mail-header-set-id|mail-header-set-lines|mail-header-set-message-id|mail-header-set-number|mail-header-set-references\n\t|mail-header-set-subject|mail-header-set-xref|mail-header-set|mail-header-strip|mail-header-subject|mail-header-unfold-field|mail-header-xref\n\t|mail-header|mail-hist-define-keys|mail-hist-enable|mail-hist-put-headers-into-history|mail-indent-citation|mail-insert-file|mail-insert-from-field\n\t|mail-mail-followup-to|mail-mail-reply-to|mail-mbox-from|mail-mode-auto-fill|mail-mode-fill-paragraph|mail-mode-flyspell-verify\n\t|mail-mode|mail-narrow-to-head|mail-other-frame|mail-other-window|mail-parse-comma-list|mail-position-on-field|mail-quote-printable-region\n\t|mail-quote-printable|mail-quote-string|mail-recover-1|mail-recover|mail-reply-to|mail-resolve-all-aliases-1|mail-resolve-all-aliases\n\t|mail-rfc822-date|mail-rfc822-time-zone|mail-send-and-exit|mail-send|mail-sendmail-delimit-header|mail-sendmail-undelimit-header\n\t|mail-sent-via|mail-sentto-newsgroups|mail-setup|mail-signature|mail-split-line|mail-string-delete|mail-strip-quoted-names|mail-subject\n\t|mail-text-start|mail-text|mail-to|mail-unquote-printable-hexdigit|mail-unquote-printable-region|mail-unquote-printable|mail-yank-clear-headers\n\t|mail-yank-original|mail-yank-region|mail|mailcap-add-mailcap-entry|mailcap-add|mailcap-command-p|mailcap-delete-duplicates|mailcap-extension-to-mime\n\t|mailcap-file-default-commands|mailcap-mailcap-entry-passes-test|mailcap-maybe-eval|mailcap-mime-info|mailcap-mime-types|mailcap-parse-mailcap-extras\n\t|mailcap-parse-mailcap|mailcap-parse-mailcaps|mailcap-parse-mimetype-file|mailcap-parse-mimetypes|mailcap-possible-viewers|mailcap-replace-in-string\n\t|mailcap-replace-regexp|mailcap-save-binary-file|mailcap-unescape-mime-test|mailcap-view-mime|mailcap-viewer-lessp|mailcap-viewer-passes-test\n\t|mailclient-encode-string-as-url|mailclient-gather-addresses|mailclient-send-it|mailclient-url-delim|mairix-build-search-list\n\t|mairix-call-mairix|mairix-edit-saved-searches-customize|mairix-edit-saved-searches|mairix-gnus-ephemeral-nndoc|mairix-gnus-fetch-field\n\t|mairix-insert-search-line|mairix-next-search|mairix-previous-search|mairix-replace-invalid-chars|mairix-rmail-display|mairix-rmail-fetch-field\n\t|mairix-save-search|mairix-search-from-this-article|mairix-search-thread-this-article|mairix-search|mairix-searches-mode|mairix-select-delete\n\t|mairix-select-edit|mairix-select-quit|mairix-select-save|mairix-select-search|mairix-sentinel-mairix-update-finished|mairix-show-folder\n\t|mairix-update-database|mairix-use-saved-search|mairix-vm-display|mairix-vm-fetch-field|mairix-widget-add|mairix-widget-build-editable-fields\n\t|mairix-widget-create-query|mairix-widget-get-values|mairix-widget-make-query-from-widgets|mairix-widget-save-search|mairix-widget-search-based-on-article\n\t|mairix-widget-search|mairix-widget-send-query|mairix-widget-toggle-activate|make-backup-file-name--default-function|make-backup-file-name-1\n\t|make-char-internal|make-char|make-cmpl-prefix-entry|make-coding-system|make-comint-in-buffer|make-comint|make-command-summary|make-completion\n\t|make-directory-internal|make-doctor-variables|make-ebrowse-bs--cmacro|make-ebrowse-bs|make-ebrowse-cs--cmacro|make-ebrowse-cs\n\t|make-ebrowse-hs--cmacro|make-ebrowse-hs|make-ebrowse-ms--cmacro|make-ebrowse-ms|make-ebrowse-position--cmacro|make-ebrowse-position\n\t|make-ebrowse-ts--cmacro|make-ebrowse-ts|make-empty-face|make-erc-channel-user--cmacro|make-erc-channel-user|make-erc-response--cmacro\n\t|make-erc-response|make-erc-server-user--cmacro|make-erc-server-user|make-ert--ewoc-entry--cmacro|make-ert--ewoc-entry|make-ert--stats--cmacro\n\t|make-ert--stats|make-ert--test-execution-info--cmacro|make-ert--test-execution-info|make-ert-test--cmacro|make-ert-test-aborted-with-non-local-exit--cmacro\n\t|make-ert-test-aborted-with-non-local-exit|make-ert-test-failed--cmacro|make-ert-test-failed|make-ert-test-passed--cmacro|make-ert-test-passed\n\t|make-ert-test-quit--cmacro|make-ert-test-quit|make-ert-test-result--cmacro|make-ert-test-result-with-condition--cmacro|make-ert-test-result-with-condition\n\t|make-ert-test-result|make-ert-test-skipped--cmacro|make-ert-test-skipped|make-ert-test|make-face-bold-italic|make-face-bold|make-face-italic\n\t|make-face-unbold|make-face-unitalic|make-face-x-resource-internal|make-face|make-flyspell-overlay|make-frame-command|make-frame-names-alist\n\t|make-full-mail-header|make-gdb-handler--cmacro|make-gdb-handler|make-gdb-table--cmacro|make-gdb-table|make-hippie-expand-function\n\t|make-htmlize-fstruct--cmacro|make-htmlize-fstruct|make-initial-minibuffer-frame|make-instance|make-js--js-handle--cmacro|make-js--js-handle\n\t|make-js--pitem--cmacro|make-js--pitem|make-mail-header|make-mode-line-mouse-map|make-obsolete-overload|make-package--ac-desc--cmacro\n\t|make-package--ac-desc|make-package--bi-desc--cmacro|make-package--bi-desc|make-random-state|make-ses--locprn--cmacro|make-ses--locprn\n\t|make-sgml-tag--cmacro|make-sgml-tag|make-soap-array-type--cmacro|make-soap-array-type|make-soap-basic-type--cmacro|make-soap-basic-type\n\t|make-soap-binding--cmacro|make-soap-binding|make-soap-bound-operation--cmacro|make-soap-bound-operation|make-soap-element--cmacro\n\t|make-soap-element|make-soap-message--cmacro|make-soap-message|make-soap-namespace--cmacro|make-soap-namespace-link--cmacro|make-soap-namespace-link\n\t|make-soap-namespace|make-soap-operation--cmacro|make-soap-operation|make-soap-port--cmacro|make-soap-port-type--cmacro|make-soap-port-type\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t make-soap-port|make-soap-sequence-element--cmacro|make-soap-sequence-element|make-soap-sequence-type--cmacro|make-soap-sequence-type\n\t|make-soap-simple-type--cmacro|make-soap-simple-type|make-soap-wsdl--cmacro|make-soap-wsdl|make-tar-header--cmacro|make-tar-header\n\t|make-term|make-terminal-frame|make-url-queue--cmacro|make-url-queue|make-variable-frame-local|makefile-add-log-defun|makefile-append-backslash\n\t|makefile-automake-mode|makefile-backslash-region|makefile-browse|makefile-browser-fill|makefile-browser-format-macro-line|makefile-browser-format-target-line\n\t|makefile-browser-get-state-for-line|makefile-browser-insert-continuation|makefile-browser-insert-selection-and-quit|makefile-browser-insert-selection\n\t|makefile-browser-next-line|makefile-browser-on-macro-line-p|makefile-browser-previous-line|makefile-browser-quit|makefile-browser-send-this-line-item\n\t|makefile-browser-set-state-for-line|makefile-browser-start-interaction|makefile-browser-this-line-macro-name|makefile-browser-this-line-target-name\n\t|makefile-browser-toggle-state-for-line|makefile-browser-toggle|makefile-bsdmake-mode|makefile-cleanup-continuations|makefile-complete\n\t|makefile-completions-at-point|makefile-create-up-to-date-overview|makefile-delete-backslash|makefile-do-macro-insertion|makefile-electric-colon\n\t|makefile-electric-dot|makefile-electric-equal|makefile-fill-paragraph|makefile-first-line-p|makefile-format-macro-ref|makefile-forward-after-target-colon\n\t|makefile-generate-temporary-filename|makefile-gmake-mode|makefile-imake-mode|makefile-insert-gmake-function|makefile-insert-macro-ref\n\t|makefile-insert-macro|makefile-insert-special-target|makefile-insert-target-ref|makefile-insert-target|makefile-last-line-p|makefile-make-font-lock-keywords\n\t|makefile-makepp-mode|makefile-match-action|makefile-match-dependency|makefile-match-function-end|makefile-mode|makefile-next-dependency\n\t|makefile-pickup-everything|makefile-pickup-filenames-as-targets|makefile-pickup-macros|makefile-pickup-targets|makefile-previous-dependency\n\t|makefile-prompt-for-gmake-funargs|makefile-query-by-make-minus-q|makefile-query-targets|makefile-remember-macro|makefile-remember-target\n\t|makefile-save-temporary|makefile-switch-to-browser|makefile-warn-continuations|makefile-warn-suspicious-lines|makeinfo-buffer\n\t|makeinfo-compilation-sentinel-buffer|makeinfo-compilation-sentinel-region|makeinfo-compile|makeinfo-current-node|makeinfo-next-error\n\t|makeinfo-recenter-compilation-buffer|makeinfo-region|man-follow|man|mantemp-insert-cxx-syntax|mantemp-make-mantemps-buffer|mantemp-make-mantemps-region\n\t|mantemp-make-mantemps|mantemp-remove-comments|mantemp-remove-memfuncs|mantemp-sort-and-unique-lines|manual-entry|map-keymap-internal\n\t|map-keymap-sorted|map-query-replace-regexp|map|mapcan|mapcar\\*|mapcon|mapl|maplist|mark-bib|mark-defun|mark-end-of-sentence|mark-icon-function\n\t|mark-page|mark-paragraph|mark-perl-function|mark-sexp|mark-whole-buffer|mark-word|master-mode|master-says-beginning-of-buffer|master-says-end-of-buffer\n\t|master-says-recenter|master-says-scroll-down|master-says-scroll-up|master-says|master-set-slave|master-show-slave|matching-paren\n\t|math-add-bignum|math-add-float|math-add|math-bignum-big|math-bignum|math-build-parse-table|math-check-complete|math-comp-concat\n\t|math-concat|math-constp|math-div-bignum-big|math-div-bignum-digit|math-div-bignum-part|math-div-bignum-try|math-div-bignum|math-div-float\n\t|math-div|math-div10-bignum|math-div2-bignum|math-div2|math-do-working|math-evenp|math-expr-ops|math-find-user-tokens|math-fixnatnump\n\t|math-fixnump|math-float|math-floatp|math-floor|math-format-bignum-decimal|math-format-bignum|math-format-flat-expr|math-format-number\n\t|math-format-stack-value|math-format-value|math-idivmod|math-imod|math-infinitep|math-ipow|math-looks-negp|math-make-float|math-match-substring\n\t|math-mod|math-mul-bignum-digit|math-mul-bignum|math-mul|math-neg|math-negp|math-normalize|math-numdigs|math-posp|math-pow|math-quotient\n\t|math-read-bignum|math-read-expr-list|math-read-exprs|math-read-if|math-read-number-simple|math-read-number|math-read-preprocess-string\n\t|math-read-radix-digit|math-read-token|math-reject-arg|math-remove-dashes|math-scale-int|math-scale-left-bignum|math-scale-left\n\t|math-scale-right-bignum|math-scale-right|math-scale-rounding|math-showing-full-precision|math-stack-value-offset|math-standard-ops-p\n\t|math-standard-ops|math-sub-bignum|math-sub-float|math-sub|math-trunc|math-with-extra-prec|math-working|math-zerop|md4-64|md4-F|md4-G\n\t|md4-H|md4-add|md4-and|md4-copy64|md4-make-step|md4-pack-int16|md4-pack-int32|md4-round1|md4-round2|md4-round3|md4-unpack-int16|md4-unpack-int32\n\t|md4|md5-binary|member\\*|member-if-not|member-if|memory-info|menu-bar-bookmark-map|menu-bar-buffer-vector|menu-bar-ediff-menu|menu-bar-ediff-merge-menu\n\t|menu-bar-ediff-misc-menu|menu-bar-enable-clipboard|menu-bar-epatch-menu|menu-bar-frame-for-menubar|menu-bar-handwrite-map|menu-bar-horizontal-scroll-bar\n\t|menu-bar-kill-ring-save|menu-bar-left-scroll-bar|menu-bar-make-mm-toggle|menu-bar-make-toggle|menu-bar-menu-at-x-y|menu-bar-menu-frame-live-and-visible-p\n\t|menu-bar-mode|menu-bar-next-tag-other-window|menu-bar-next-tag|menu-bar-no-horizontal-scroll-bar|menu-bar-no-scroll-bar|menu-bar-non-minibuffer-window-p\n\t|menu-bar-open|menu-bar-options-save|menu-bar-positive-p|menu-bar-read-lispintro|menu-bar-read-lispref|menu-bar-read-mail|menu-bar-right-scroll-bar\n\t|menu-bar-select-buffer|menu-bar-select-frame|menu-bar-select-yank|menu-bar-set-tool-bar-position|menu-bar-showhide-fringe-ind-box\n\t|menu-bar-showhide-fringe-ind-customize|menu-bar-showhide-fringe-ind-left|menu-bar-showhide-fringe-ind-mixed|menu-bar-showhide-fringe-ind-none\n\t|menu-bar-showhide-fringe-ind-right|menu-bar-showhide-fringe-menu-customize-disable|menu-bar-showhide-fringe-menu-customize-left\n\t|menu-bar-showhide-fringe-menu-customize-reset|menu-bar-showhide-fringe-menu-customize-right|menu-bar-showhide-fringe-menu-customize\n\t|menu-bar-showhide-tool-bar-menu-customize-disable|menu-bar-showhide-tool-bar-menu-customize-enable-bottom|menu-bar-showhide-tool-bar-menu-customize-enable-left\n\t|menu-bar-showhide-tool-bar-menu-customize-enable-right|menu-bar-showhide-tool-bar-menu-customize-enable-top|menu-bar-update-buffers-1\n\t|menu-bar-update-buffers|menu-bar-update-yank-menu|menu-find-file-existing|menu-or-popup-active-p|menu-set-font|mercury-mode|merge-coding-systems\n\t|merge-mail-abbrevs|merge|message--yank-original-internal|message-add-action|message-add-archive-header|message-add-header|message-alter-recipients-discard-bogus-full-name\n\t|message-beginning-of-line|message-bogus-recipient-p|message-bold-region|message-bounce|message-buffer-name|message-buffers|message-bury\n\t|message-caesar-buffer-body|message-caesar-region|message-cancel-news|message-canlock-generate|message-canlock-password|message-carefully-insert-headers\n\t|message-change-subject|message-check-element|message-check-news-body-syntax|message-check-news-header-syntax|message-check-news-syntax\n\t|message-check-recipients|message-check|message-checksum|message-cite-original-1|message-cite-original-without-signature|message-cite-original\n\t|message-cleanup-headers|message-clone-locals|message-completion-function|message-completion-in-region|message-cross-post-followup-to-header\n\t|message-cross-post-followup-to|message-cross-post-insert-note|message-default-send-mail-function|message-default-send-rename-function\n\t|message-delete-action|message-delete-line|message-delete-not-region|message-delete-overlay|message-disassociate-draft|message-display-abbrev\n\t|message-do-actions|message-do-auto-fill|message-do-fcc|message-do-send-housekeeping|message-dont-reply-to-names|message-dont-send\n\t|message-elide-region|message-encode-message-body|message-exchange-point-and-mark|message-expand-group|message-expand-name|message-fetch-field\n\t|message-fetch-reply-field|message-field-name|message-field-value|message-fill-field-address|message-fill-field-general|message-fill-field\n\t|message-fill-paragraph|message-fill-yanked-message|message-fix-before-sending|message-flatten-list|message-followup|message-font-lock-make-header-matcher\n\t|message-forward-make-body-digest-mime|message-forward-make-body-digest-plain|message-forward-make-body-digest|message-forward-make-body-mime\n\t|message-forward-make-body-mml|message-forward-make-body-plain|message-forward-make-body|message-forward-rmail-make-body|message-forward-subject-author-subject\n\t|message-forward-subject-fwd|message-forward-subject-name-subject|message-forward|message-generate-headers|message-generate-new-buffer-clone-locals\n\t|message-generate-unsubscribed-mail-followup-to|message-get-reply-headers|message-gnksa-enable-p|message-goto-bcc|message-goto-body\n\t|message-goto-cc|message-goto-distribution|message-goto-eoh|message-goto-fcc|message-goto-followup-to|message-goto-from|message-goto-keywords\n\t|message-goto-mail-followup-to|message-goto-newsgroups|message-goto-reply-to|message-goto-signature|message-goto-subject|message-goto-summary\n\t|message-goto-to|message-headers-to-generate|message-hide-header-p|message-hide-headers|message-idna-to-ascii-rhs-1|message-idna-to-ascii-rhs\n\t|message-in-body-p|message-indent-citation|message-info|message-insert-canlock|message-insert-citation-line|message-insert-courtesy-copy\n\t|message-insert-disposition-notification-to|message-insert-expires|message-insert-formatted-citation-line|message-insert-header\n\t|message-insert-headers|message-insert-importance-high|message-insert-importance-low|message-insert-newsgroups|message-insert-or-toggle-importance\n\t|message-insert-signature|message-insert-to|message-insert-wide-reply|message-insinuate-rmail|message-is-yours-p|message-kill-address\n\t|message-kill-all-overlays|message-kill-buffer|message-kill-to-signature|message-mail-alias-type-p|message-mail-file-mbox-p|message-mail-other-frame\n\t|message-mail-other-window|message-mail-p|message-mail-user-agent|message-mail|message-make-address|message-make-caesar-translation-table\n\t|message-make-date|message-make-distribution|message-make-domain|message-make-expires-date|message-make-expires|message-make-forward-subject\n\t|message-make-fqdn|message-make-from|message-make-html-message-with-image-files|message-make-in-reply-to|message-make-lines|message-make-mail-followup-to\n\t|message-make-message-id|message-make-organization|message-make-overlay|message-make-path|message-make-references|message-make-sender\n\t|message-make-tool-bar|message-mark-active-p|message-mark-insert-file|message-mark-inserted-region|message-mode-field-menu|message-mode-menu\n\t|message-mode|message-multi-smtp-send-mail|message-narrow-to-field|message-narrow-to-head-1|message-narrow-to-head|message-narrow-to-headers-or-head\n\t|message-narrow-to-headers|message-newline-and-reformat|message-news-other-frame|message-news-other-window|message-news-p|message-news\n\t|message-next-header|message-number-base36|message-options-get|message-options-set-recipient|message-options-set|message-output\n\t|message-overlay-put|message-pipe-buffer-body|message-point-in-header-p|message-pop-to-buffer|message-position-on-field|message-position-point\n\t|message-posting-charset|message-prune-recipients|message-put-addresses-in-ecomplete|message-read-from-minibuffer|message-recover\n\t|message-reduce-to-to-cc|message-remove-blank-cited-lines|message-remove-first-header|message-remove-header|message-remove-ignored-headers\n\t|message-rename-buffer|message-replace-header|message-reply|message-resend|message-send-and-exit|message-send-form-letter|message-send-mail-function\n\t|message-send-mail-partially|message-send-mail-with-mailclient|message-send-mail-with-mh|message-send-mail-with-qmail|message-send-mail-with-sendmail\n\t|message-send-mail|message-send-news|message-send-via-mail|message-send-via-news|message-send|message-sendmail-envelope-from|message-set-auto-save-file-name\n\t|message-setup-1|message-setup-fill-variables|message-setup-toolbar|message-setup|message-shorten-1|message-shorten-references\n\t|message-signed-or-encrypted-p|message-simplify-recipients|message-simplify-subject|message-skip-to-next-address|message-smtpmail-send-it\n\t|message-sort-headers-1|message-sort-headers|message-split-line|message-strip-forbidden-properties|message-strip-list-identifiers\n\t|message-strip-subject-encoded-words|message-strip-subject-re|message-strip-subject-trailing-was|message-subscribed-p|message-supersede\n\t|message-tab|message-talkative-question|message-tamago-not-in-use-p|message-text-with-property|message-to-list-only|message-tokenize-header\n\t|message-tool-bar-update|message-unbold-region|message-unique-id|message-unquote-tokens|message-use-alternative-email-as-from\n\t|message-user-mail-address|message-wash-subject|message-wide-reply|message-widen-reply|message-with-reply-buffer|message-y-or-n-p\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t message-yank-buffer|message-yank-original|messages-buffer-mode|meta-add-symbols|meta-beginning-of-defun|meta-car-string-lessp\n\t|meta-comment-defun|meta-comment-indent|meta-comment-region|meta-common-mode|meta-complete-symbol|meta-completions-at-point|meta-end-of-defun\n\t|meta-indent-buffer|meta-indent-calculate|meta-indent-current-indentation|meta-indent-current-nesting|meta-indent-defun|meta-indent-in-string-p\n\t|meta-indent-level-count|meta-indent-line|meta-indent-looking-at-code|meta-indent-previous-line|meta-indent-region|meta-indent-unfinished-line\n\t|meta-listify|meta-mark-active|meta-mark-defun|meta-mode-menu|meta-symbol-list|meta-uncomment-defun|meta-uncomment-region|metafont-mode\n\t|metamail-buffer|metamail-interpret-body|metamail-interpret-header|metamail-region|metapost-mode|mh-adaptive-cmd-note-flag-check\n\t|mh-add-missing-mime-version-header|mh-add-msgs-to-seq|mh-alias-address-to-alias|mh-alias-expand|mh-alias-for-from-p|mh-alias-grab-from-field\n\t|mh-alias-letter-expand-alias|mh-alias-minibuffer-confirm-address|mh-alias-reload-maybe|mh-assoc-string|mh-beginning-of-word|mh-bogofilter-blacklist\n\t|mh-bogofilter-whitelist|mh-buffer-data|mh-burst-digest|mh-cancel-timer|mh-catchup|mh-cl-flet|mh-clean-msg-header|mh-clear-sub-folders-cache\n\t|mh-coalesce-msg-list|mh-colors-available-p|mh-colors-in-use-p|mh-complete-word|mh-compose-forward|mh-compose-insertion|mh-copy-msg\n\t|mh-create-sequence-map|mh-customize|mh-decode-message-header|mh-decode-message-subject|mh-define-obsolete-variable-alias|mh-define-sequence\n\t|mh-defstruct|mh-delete-a-msg|mh-delete-line|mh-delete-msg-from-seq|mh-delete-msg-no-motion|mh-delete-msg|mh-delete-seq|mh-delete-subject-or-thread\n\t|mh-delete-subject|mh-destroy-postponed-handles|mh-display-color-cells|mh-display-completion-list|mh-display-emphasis|mh-display-msg\n\t|mh-display-smileys|mh-display-with-external-viewer|mh-do-at-event-location|mh-do-in-gnu-emacs|mh-do-in-xemacs|mh-edit-again|mh-ephem-message\n\t|mh-exchange-point-and-mark-preserving-active-mark|mh-exec-cmd-daemon|mh-exec-cmd-env-daemon|mh-exec-cmd-error|mh-exec-cmd-output\n\t|mh-exec-cmd-quiet|mh-exec-cmd|mh-exec-lib-cmd-output|mh-execute-commands|mh-expand-file-name|mh-extract-from-header-value|mh-extract-rejected-mail\n\t|mh-face-background|mh-face-data|mh-face-foreground|mh-file-command-p|mh-file-mime-type|mh-find-path|mh-find-seq|mh-first-msg|mh-folder-completion-function\n\t|mh-folder-from-address|mh-folder-inline-mime-part|mh-folder-list|mh-folder-mode|mh-folder-name-p|mh-folder-save-mime-part|mh-folder-speedbar-buttons\n\t|mh-folder-toggle-mime-part|mh-font-lock-add-keywords|mh-forward|mh-fully-kill-draft|mh-funcall-if-exists|mh-get-header-field|mh-get-msg-num\n\t|mh-gnus-article-highlight-citation|mh-goto-cur-msg|mh-goto-header-end|mh-goto-header-field|mh-goto-msg|mh-goto-next-button|mh-handle-process-error\n\t|mh-have-file-command|mh-header-display|mh-header-field-beginning|mh-header-field-end|mh-help|mh-identity-add-menu|mh-identity-handler-attribution-verb\n\t|mh-identity-handler-bottom|mh-identity-handler-gpg-identity|mh-identity-handler-signature|mh-identity-handler-top|mh-identity-insert-attribution-verb\n\t|mh-identity-make-menu-no-autoload|mh-identity-make-menu|mh-image-load-path-for-library|mh-image-search-load-path|mh-in-header-p\n\t|mh-in-show-buffer|mh-inc-folder|mh-inc-spool-make-no-autoload|mh-inc-spool-make|mh-index-add-to-sequence|mh-index-create-imenu-index\n\t|mh-index-create-sequences|mh-index-delete-folder-headers|mh-index-delete-from-sequence|mh-index-execute-commands|mh-index-group-by-folder\n\t|mh-index-insert-folder-headers|mh-index-new-messages|mh-index-next-folder|mh-index-previous-folder|mh-index-read-data|mh-index-sequenced-messages\n\t|mh-index-ticked-messages|mh-index-update-maps|mh-index-visit-folder|mh-insert-auto-fields|mh-insert-identity|mh-insert-signature\n\t|mh-interactive-range|mh-invalidate-show-buffer|mh-invisible-headers|mh-iterate-on-messages-in-region|mh-iterate-on-range|mh-junk-blacklist-disposition\n\t|mh-junk-blacklist|mh-junk-choose|mh-junk-process-blacklist|mh-junk-process-whitelist|mh-junk-whitelist|mh-kill-folder|mh-last-msg\n\t|mh-lessp|mh-letter-hide-all-skipped-fields|mh-letter-mode|mh-letter-next-header-field|mh-letter-skip-leading-whitespace-in-header-field\n\t|mh-letter-skipped-header-field-p|mh-letter-speedbar-buttons|mh-letter-toggle-header-field-display-button|mh-letter-toggle-header-field-display\n\t|mh-line-beginning-position|mh-line-end-position|mh-list-folders|mh-list-sequences|mh-list-to-string-1|mh-list-to-string|mh-logo-display\n\t|mh-macro-expansion-time-gnus-version|mh-mail-abbrev-make-syntax-table|mh-mail-header-end|mh-make-folder-mode-line|mh-make-local-hook\n\t|mh-make-local-vars|mh-make-obsolete-variable|mh-mapc|mh-mark-active-p|mh-match-string-no-properties|mh-maybe-show|mh-mh-compose-anon-ftp\n\t|mh-mh-compose-external-compressed-tar|mh-mh-compose-external-type|mh-mh-directive-present-p|mh-mh-to-mime-undo|mh-mh-to-mime\n\t|mh-mime-cleanup|mh-mime-display|mh-mime-save-parts|mh-mml-forward-message|mh-mml-secure-message-encrypt|mh-mml-secure-message-sign\n\t|mh-mml-secure-message-signencrypt|mh-mml-tag-present-p|mh-mml-to-mime|mh-mml-unsecure-message|mh-modify|mh-msg-filename|mh-msg-is-in-seq\n\t|mh-msg-num-width-to-column|mh-msg-num-width|mh-narrow-to-cc|mh-narrow-to-from|mh-narrow-to-range|mh-narrow-to-seq|mh-narrow-to-subject\n\t|mh-narrow-to-tick|mh-narrow-to-to|mh-new-draft-name|mh-next-button|mh-next-msg|mh-next-undeleted-msg|mh-next-unread-msg|mh-nmail\n\t|mh-notate-cur|mh-notate-deleted-and-refiled|mh-notate-user-sequences|mh-notate|mh-outstanding-commands-p|mh-pack-folder|mh-page-digest-backwards\n\t|mh-page-digest|mh-page-msg|mh-parse-flist-output-line|mh-pipe-msg|mh-position-on-field|mh-prefix-help|mh-prev-button|mh-previous-page\n\t|mh-previous-undeleted-msg|mh-previous-unread-msg|mh-print-msg|mh-process-daemon|mh-process-or-undo-commands|mh-profile-component-value\n\t|mh-profile-component|mh-prompt-for-folder|mh-prompt-for-refile-folder|mh-ps-print-msg-file|mh-ps-print-msg|mh-ps-print-toggle-color\n\t|mh-ps-print-toggle-faces|mh-put-msg-in-seq|mh-quit|mh-quote-for-shell|mh-quote-pick-expr|mh-range-to-msg-list|mh-read-address|mh-read-folder-sequences\n\t|mh-read-range|mh-read-seq-default|mh-recenter|mh-redistribute|mh-refile-a-msg|mh-refile-msg|mh-refile-or-write-again|mh-regenerate-headers\n\t|mh-remove-all-notation|mh-remove-cur-notation|mh-remove-from-sub-folders-cache|mh-replace-regexp-in-string|mh-replace-string\n\t|mh-reply|mh-require-cl|mh-require|mh-rescan-folder|mh-reset-threads-and-narrowing|mh-rmail|mh-run-time-gnus-version|mh-scan-folder\n\t|mh-scan-format-file-check|mh-scan-format|mh-scan-msg-number-regexp|mh-scan-msg-search-regexp|mh-search-from-end|mh-search-p|mh-search\n\t|mh-send-letter|mh-send|mh-seq-msgs|mh-seq-to-msgs|mh-set-cmd-note|mh-set-folder-modified-p|mh-set-help|mh-set-x-image-cache-directory\n\t|mh-show-addr|mh-show-buffer-message-number|mh-show-font-lock-keywords-with-cite|mh-show-font-lock-keywords|mh-show-mode|mh-show-preferred-alternative\n\t|mh-show-speedbar-buttons|mh-show-xface|mh-show|mh-showing-mode|mh-signature-separator-p|mh-smail-batch|mh-smail-other-window|mh-smail\n\t|mh-sort-folder|mh-spamassassin-blacklist|mh-spamassassin-identify-spammers|mh-spamassassin-whitelist|mh-spamprobe-blacklist|mh-spamprobe-whitelist\n\t|mh-speed-add-folder|mh-speed-flists-active-p|mh-speed-flists|mh-speed-invalidate-map|mh-start-of-uncleaned-message|mh-store-msg\n\t|mh-strip-package-version|mh-sub-folders|mh-test-completion|mh-thread-add-spaces|mh-thread-ancestor|mh-thread-delete|mh-thread-find-msg-subject\n\t|mh-thread-forget-message|mh-thread-generate|mh-thread-inc|mh-thread-next-sibling|mh-thread-parse-scan-line|mh-thread-previous-sibling\n\t|mh-thread-print-scan-lines|mh-thread-refile|mh-thread-update-scan-line-map|mh-toggle-mh-decode-mime-flag|mh-toggle-mime-buttons\n\t|mh-toggle-showing|mh-toggle-threads|mh-toggle-tick|mh-translate-range|mh-truncate-log-buffer|mh-undefine-sequence|mh-undo-folder\n\t|mh-undo|mh-update-sequences|mh-url-hexify-string|mh-user-agent-compose|mh-valid-seq-p|mh-valid-view-change-operation-p|mh-variant-gnu-mh-info\n\t|mh-variant-info|mh-variant-mh-info|mh-variant-nmh-info|mh-variant-p|mh-variant-set-variant|mh-variant-set|mh-variants|mh-version\n\t|mh-view-mode-enter|mh-visit-folder|mh-widen|mh-window-full-height-p|mh-write-file-functions|mh-write-msg-to-file|mh-xargs|mh-yank-cur-msg\n\t|midnight-buffer-display-time|midnight-delay-set|midnight-find|midnight-next|mime-to-mml|minibuf-eldef-setup-minibuffer|minibuf-eldef-update-minibuffer\n\t|minibuffer--bitset|minibuffer--double-dollars|minibuffer-avoid-prompt|minibuffer-completion-contents|minibuffer-default--in-prompt-regexps\n\t|minibuffer-default-add-completions|minibuffer-default-add-shell-commands|minibuffer-depth-indicate-mode|minibuffer-depth-setup\n\t|minibuffer-electric-default-mode|minibuffer-force-complete-and-exit|minibuffer-force-complete|minibuffer-frame-list|minibuffer-hide-completions\n\t|minibuffer-history-initialize|minibuffer-history-isearch-end|minibuffer-history-isearch-message|minibuffer-history-isearch-pop-state\n\t|minibuffer-history-isearch-push-state|minibuffer-history-isearch-search|minibuffer-history-isearch-setup|minibuffer-history-isearch-wrap\n\t|minibuffer-insert-file-name-at-point|minibuffer-keyboard-quit|minibuffer-with-setup-hook|minor-mode-menu-from-indicator|minusp\n\t|mismatch|mixal-debug|mixal-describe-operation-code|mixal-mode|mixal-run|mm-add-meta-html-tag|mm-alist-to-plist|mm-annotationp|mm-append-to-file\n\t|mm-archive-decoders|mm-archive-dissect-and-inline|mm-assoc-string-match|mm-attachment-override-p|mm-auto-mode-alist|mm-automatic-display-p\n\t|mm-automatic-external-display-p|mm-body-7-or-8|mm-body-encoding|mm-char-int|mm-char-or-char-int-p|mm-charset-after|mm-charset-to-coding-system\n\t|mm-codepage-setup|mm-coding-system-equal|mm-coding-system-list|mm-coding-system-p|mm-coding-system-to-mime-charset|mm-complicated-handles\n\t|mm-content-transfer-encoding|mm-convert-shr-links|mm-copy-to-buffer|mm-create-image-xemacs|mm-decode-body|mm-decode-coding-region\n\t|mm-decode-coding-string|mm-decode-content-transfer-encoding|mm-decode-string|mm-decompress-buffer|mm-default-file-encoding|mm-default-multibyte-p\n\t|mm-delete-duplicates|mm-destroy-part|mm-destroy-parts|mm-destroy-postponed-undisplay-list|mm-detect-coding-region|mm-detect-mime-charset-region\n\t|mm-disable-multibyte|mm-display-external|mm-display-inline|mm-display-part|mm-display-parts|mm-dissect-archive|mm-dissect-buffer\n\t|mm-dissect-multipart|mm-dissect-singlepart|mm-enable-multibyte|mm-encode-body|mm-encode-buffer|mm-encode-coding-region|mm-encode-coding-string\n\t|mm-encode-content-transfer-encoding|mm-enrich-utf-8-by-mule-ucs|mm-extern-cache-contents|mm-file-name-collapse-whitespace|mm-file-name-delete-control\n\t|mm-file-name-delete-gotchas|mm-file-name-delete-whitespace|mm-file-name-replace-whitespace|mm-file-name-trim-whitespace|mm-find-buffer-file-coding-system\n\t|mm-find-charset-region|mm-find-mime-charset-region|mm-find-part-by-type|mm-find-raw-part-by-type|mm-get-coding-system-list|mm-get-content-id\n\t|mm-get-image|mm-get-part|mm-guess-charset|mm-handle-buffer|mm-handle-cache|mm-handle-description|mm-handle-displayed-p|mm-handle-disposition\n\t|mm-handle-encoding|mm-handle-filename|mm-handle-id|mm-handle-media-subtype|mm-handle-media-supertype|mm-handle-media-type|mm-handle-multipart-ctl-parameter\n\t|mm-handle-multipart-from|mm-handle-multipart-original-buffer|mm-handle-set-cache|mm-handle-set-external-undisplayer|mm-handle-set-undisplayer\n\t|mm-handle-type|mm-handle-undisplayer|mm-image-fit-p|mm-image-load-path|mm-image-type-from-buffer|mm-inlinable-p|mm-inline-external-body\n\t|mm-inline-override-p|mm-inline-partial|mm-inlined-p|mm-insert-byte|mm-insert-file-contents|mm-insert-headers|mm-insert-inline|mm-insert-multipart-headers\n\t|mm-insert-part|mm-insert-rfc822-headers|mm-interactively-view-part|mm-iso-8859-x-to-15-region|mm-keep-viewer-alive-p|mm-line-number-at-pos\n\t|mm-long-lines-p|mm-mailcap-command|mm-make-handle|mm-make-temp-file|mm-merge-handles|mm-mime-charset|mm-mule-charset-to-mime-charset\n\t|mm-multibyte-char-to-unibyte|mm-multibyte-p|mm-multibyte-string-p|mm-multiple-handles|mm-pipe-part|mm-possibly-verify-or-decrypt\n\t|mm-preferred-alternative-precedence|mm-preferred-alternative|mm-preferred-coding-system|mm-qp-or-base64|mm-read-charset|mm-read-coding-system\n\t|mm-readable-p|mm-remove-part|mm-remove-parts|mm-replace-in-string|mm-safer-encoding|mm-save-part-to-file|mm-save-part|mm-set-buffer-file-coding-system\n\t|mm-set-buffer-multibyte|mm-set-handle-multipart-parameter|mm-setup-codepage-ibm|mm-setup-codepage-iso-8859|mm-shr|mm-sort-coding-systems-predicate\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t mm-special-display-p|mm-string-as-multibyte|mm-string-as-unibyte|mm-string-make-unibyte|mm-string-to-multibyte|mm-subst-char-in-string\n\t|mm-substring-no-properties|mm-temp-files-delete|mm-ucs-to-char|mm-url-decode-entities-nbsp|mm-url-decode-entities-string|mm-url-decode-entities\n\t|mm-url-encode-multipart-form-data|mm-url-encode-www-form-urlencoded|mm-url-form-encode-xwfu|mm-url-insert-file-contents-external\n\t|mm-url-insert-file-contents|mm-url-insert|mm-url-load-url|mm-url-remove-markup|mm-uu-dissect-text-parts|mm-uu-dissect|mm-valid-and-fit-image-p\n\t|mm-valid-image-format-p|mm-view-pkcs7|mm-with-multibyte-buffer|mm-with-part|mm-with-unibyte-buffer|mm-with-unibyte-current-buffer\n\t|mm-write-region|mm-xemacs-find-mime-charset-1|mm-xemacs-find-mime-charset|mml-attach-buffer|mml-attach-external|mml-attach-file\n\t|mml-buffer-substring-no-properties-except-hard-newlines|mml-compute-boundary-1|mml-compute-boundary|mml-content-disposition\n\t|mml-destroy-buffers|mml-dnd-attach-file|mml-expand-html-into-multipart-related|mml-generate-mime-1|mml-generate-mime|mml-generate-new-buffer\n\t|mml-insert-buffer|mml-insert-empty-tag|mml-insert-mime-headers|mml-insert-mime|mml-insert-mml-markup|mml-insert-multipart|mml-insert-parameter-string\n\t|mml-insert-parameter|mml-insert-part|mml-insert-tag|mml-make-boundary|mml-menu|mml-minibuffer-read-description|mml-minibuffer-read-disposition\n\t|mml-minibuffer-read-file|mml-minibuffer-read-type|mml-mode|mml-parameter-string|mml-parse-1|mml-parse-file-name|mml-parse-singlepart-with-multiple-charsets\n\t|mml-parse|mml-pgp-encrypt-buffer|mml-pgp-sign-buffer|mml-pgpauto-encrypt-buffer|mml-pgpauto-sign-buffer|mml-pgpmime-encrypt-buffer\n\t|mml-pgpmime-sign-buffer|mml-preview-insert-mail-followup-to|mml-preview|mml-quote-region|mml-read-part|mml-read-tag|mml-secure-encrypt-pgp\n\t|mml-secure-encrypt-pgpmime|mml-secure-encrypt-smime|mml-secure-encrypt|mml-secure-message-encrypt-pgp|mml-secure-message-encrypt-pgpauto\n\t|mml-secure-message-encrypt-pgpmime|mml-secure-message-encrypt-smime|mml-secure-message-encrypt|mml-secure-message-sign-encrypt\n\t|mml-secure-message-sign-pgp|mml-secure-message-sign-pgpauto|mml-secure-message-sign-pgpmime|mml-secure-message-sign-smime|mml-secure-message-sign\n\t|mml-secure-message|mml-secure-part|mml-secure-sign-pgp|mml-secure-sign-pgpauto|mml-secure-sign-pgpmime|mml-secure-sign-smime|mml-secure-sign\n\t|mml-signencrypt-style|mml-smime-encrypt-buffer|mml-smime-encrypt-query|mml-smime-encrypt|mml-smime-sign-buffer|mml-smime-sign-query\n\t|mml-smime-sign|mml-smime-verify-test|mml-smime-verify|mml-to-mime|mml-tweak-externalize-attachments|mml-tweak-part|mml-unsecure-message\n\t|mml-validate|mml1991-encrypt|mml1991-sign|mml2015-decrypt-test|mml2015-decrypt|mml2015-encrypt|mml2015-self-encrypt|mml2015-sign\n\t|mml2015-verify-test|mml2015-verify|mod\\*|mode-line-bury-buffer|mode-line-change-eol|mode-line-eol-desc|mode-line-frame-control\n\t|mode-line-minor-mode-help|mode-line-modified-help-echo|mode-line-mule-info-help-echo|mode-line-next-buffer|mode-line-other-buffer\n\t|mode-line-previous-buffer|mode-line-read-only-help-echo|mode-line-toggle-modified|mode-line-toggle-read-only|mode-line-unbury-buffer\n\t|mode-line-widen|mode-local--expand-overrides|mode-local--overload-body|mode-local--override|mode-local-augment-function-help\n\t|mode-local-bind|mode-local-describe-bindings-1|mode-local-describe-bindings-2|mode-local-equivalent-mode-p|mode-local-initialized-p\n\t|mode-local-map-file-buffers|mode-local-map-mode-buffers|mode-local-on-major-mode-change|mode-local-post-major-mode-change|mode-local-print-binding\n\t|mode-local-print-bindings|mode-local-read-function|mode-local-setup-edebug-specs|mode-local-symbol-value|mode-local-symbol|mode-local-use-bindings-p\n\t|mode-local-value|mode-specific-command-prefix|modify-coding-system-alist|modify-face|modula-2-mode|morse-region|mouse--down-1-maybe-follows-link\n\t|mouse--drag-set-mark-and-point|mouse--strip-first-event|mouse-appearance-menu|mouse-autoselect-window-cancel|mouse-autoselect-window-select\n\t|mouse-autoselect-window-start|mouse-avoidance-banish-destination|mouse-avoidance-banish-mouse|mouse-avoidance-banish|mouse-avoidance-delta\n\t|mouse-avoidance-exile|mouse-avoidance-fancy|mouse-avoidance-ignore-p|mouse-avoidance-mode|mouse-avoidance-nudge-mouse|mouse-avoidance-point-position\n\t|mouse-avoidance-random-shape|mouse-avoidance-set-mouse-position|mouse-avoidance-set-pointer-shape|mouse-avoidance-too-close-p\n\t|mouse-buffer-menu-alist|mouse-buffer-menu-keymap|mouse-buffer-menu-map|mouse-buffer-menu-split|mouse-buffer-menu|mouse-choose-completion\n\t|mouse-copy-work-around-drag-bug|mouse-delete-other-windows|mouse-delete-window|mouse-drag-drag|mouse-drag-events-are-point-events-p\n\t|mouse-drag-header-line|mouse-drag-line|mouse-drag-mode-line|mouse-drag-region|mouse-drag-repeatedly-safe-scroll|mouse-drag-safe-scroll\n\t|mouse-drag-scroll-delta|mouse-drag-secondary-moving|mouse-drag-secondary-pasting|mouse-drag-secondary|mouse-drag-should-do-col-scrolling\n\t|mouse-drag-throw|mouse-drag-track|mouse-drag-vertical-line|mouse-event-p|mouse-fixup-help-message|mouse-kill-preserving-secondary\n\t|mouse-kill-ring-save|mouse-kill-secondary|mouse-kill|mouse-major-mode-menu|mouse-menu-bar-map|mouse-menu-major-mode-map|mouse-menu-non-singleton\n\t|mouse-minibuffer-check|mouse-minor-mode-menu|mouse-popup-menubar-stuff|mouse-popup-menubar|mouse-posn-property|mouse-region-match\n\t|mouse-save-then-kill-delete-region|mouse-save-then-kill|mouse-scroll-subr|mouse-secondary-save-then-kill|mouse-select-buffer\n\t|mouse-select-font|mouse-select-window|mouse-set-font|mouse-set-mark-fast|mouse-set-mark|mouse-set-point|mouse-set-region-1|mouse-set-region\n\t|mouse-set-secondary|mouse-skip-word|mouse-split-window-horizontally|mouse-split-window-vertically|mouse-start-end|mouse-start-secondary\n\t|mouse-tear-off-window|mouse-undouble-last-event|mouse-wheel-change-button|mouse-wheel-mode|mouse-yank-at-click|mouse-yank-primary\n\t|mouse-yank-secondary|move-beginning-of-line|move-end-of-line|move-file-to-trash|move-past-close-and-reindent|move-to-column-untabify\n\t|move-to-tab-stop|move-to-window-line-top-bottom|mpc--debug|mpc--faster-stop|mpc--faster-toggle-refresh|mpc--faster-toggle|mpc--faster\n\t|mpc--proc-alist-to-alists|mpc--proc-connect|mpc--proc-filter|mpc--proc-quote-string|mpc--songduration|mpc--status-callback|mpc--status-idle-timer-run\n\t|mpc--status-idle-timer-start|mpc--status-idle-timer-stop|mpc--status-timer-run|mpc--status-timer-start|mpc--status-timer-stop\n\t|mpc--status-timers-refresh|mpc-assq-all|mpc-cmd-add|mpc-cmd-clear|mpc-cmd-delete|mpc-cmd-find|mpc-cmd-flush|mpc-cmd-list|mpc-cmd-move\n\t|mpc-cmd-pause|mpc-cmd-play|mpc-cmd-special-tag-p|mpc-cmd-status|mpc-cmd-stop|mpc-cmd-tagtypes|mpc-cmd-update|mpc-compare-strings\n\t|mpc-constraints-get-current|mpc-constraints-pop|mpc-constraints-push|mpc-constraints-restore|mpc-constraints-tag-lookup|mpc-current-refresh\n\t|mpc-data-directory|mpc-drag-n-drop|mpc-event-set-point|mpc-ffwd|mpc-file-local-copy|mpc-format|mpc-intersection|mpc-mode-menu|mpc-mode\n\t|mpc-next|mpc-pause|mpc-play-at-point|mpc-play|mpc-playlist-add|mpc-playlist-create|mpc-playlist-delete|mpc-playlist-destroy|mpc-playlist-rename\n\t|mpc-playlist|mpc-prev|mpc-proc-buf-to-alist|mpc-proc-buf-to-alists|mpc-proc-buffer|mpc-proc-check|mpc-proc-cmd-list-ok|mpc-proc-cmd-list\n\t|mpc-proc-cmd-to-alist|mpc-proc-cmd|mpc-proc-sync|mpc-proc-tag-string-to-sym|mpc-proc|mpc-quit|mpc-reorder|mpc-resume|mpc-rewind|mpc-ring-make\n\t|mpc-ring-pop|mpc-ring-push|mpc-secs-to-time|mpc-select-extend|mpc-select-get-selection|mpc-select-make-overlay|mpc-select-restore\n\t|mpc-select-save|mpc-select-toggle|mpc-select|mpc-selection-refresh|mpc-separator|mpc-songpointer-context|mpc-songpointer-refresh-hairy\n\t|mpc-songpointer-refresh|mpc-songpointer-score|mpc-songpointer-set|mpc-songs-buf|mpc-songs-hashcons|mpc-songs-jump-to|mpc-songs-kill-search\n\t|mpc-songs-mode|mpc-songs-refresh|mpc-songs-search|mpc-songs-selection|mpc-sort|mpc-status-buffer-refresh|mpc-status-buffer-show\n\t|mpc-status-mode|mpc-status-refresh|mpc-status-stop|mpc-stop|mpc-string-prefix-p|mpc-tagbrowser-all-p|mpc-tagbrowser-all-select\n\t|mpc-tagbrowser-buf|mpc-tagbrowser-dir-mode|mpc-tagbrowser-dir-toggle|mpc-tagbrowser-mode|mpc-tagbrowser-refresh|mpc-tagbrowser-tag-name\n\t|mpc-tagbrowser|mpc-tempfiles-add|mpc-tempfiles-clean|mpc-union|mpc-update|mpc-updated-db|mpc-volume-mouse-set|mpc-volume-refresh\n\t|mpc-volume-widget|mpc|mpuz-ask-for-try|mpuz-build-random-perm|mpuz-check-all-solved|mpuz-close-game|mpuz-create-buffer|mpuz-digit-solved-p\n\t|mpuz-ding|mpuz-get-buffer|mpuz-mode|mpuz-offer-abort|mpuz-paint-board|mpuz-paint-digit|mpuz-paint-errors|mpuz-paint-number|mpuz-paint-statistics\n\t|mpuz-put-number-on-board|mpuz-random-puzzle|mpuz-show-solution|mpuz-solve|mpuz-start-new-game|mpuz-switch-to-window|mpuz-to-digit\n\t|mpuz-to-letter|mpuz-try-letter|mpuz-try-proposal|mpuz|msb--add-separators|msb--add-to-menu|msb--aggregate-alist|msb--choose-file-menu\n\t|msb--choose-menu|msb--collect|msb--create-buffer-menu-2|msb--create-buffer-menu|msb--create-function-info|msb--create-sort-item\n\t|msb--dired-directory|msb--format-title|msb--init-file-alist|msb--make-keymap-menu|msb--mode-menu-cond|msb--most-recently-used-menu\n\t|msb--split-menus-2|msb--split-menus|msb--strip-dir|msb--toggle-menu-type|msb-alon-item-handler|msb-custom-set|msb-dired-item-handler\n\t|msb-invisible-buffer-p|msb-item-handler|msb-menu-bar-update-buffers|msb-mode|msb-sort-by-directory|msb-sort-by-name|msb-unload-function\n\t|msb|mspools-get-folder-from-spool|mspools-get-spool-files|mspools-get-spool-name|mspools-help|mspools-mode|mspools-quit|mspools-revert-buffer\n\t|mspools-set-vm-spool-files|mspools-show-again|mspools-show|mspools-size-folder|mspools-visit-spool|mule-diag|multi-isearch-buffers-regexp\n\t|multi-isearch-buffers|multi-isearch-end|multi-isearch-files-regexp|multi-isearch-files|multi-isearch-next-buffer-from-list|multi-isearch-next-file-buffer-from-list\n\t|multi-isearch-pop-state|multi-isearch-push-state|multi-isearch-read-buffers|multi-isearch-read-files|multi-isearch-read-matching-buffers\n\t|multi-isearch-read-matching-files|multi-isearch-search-fun|multi-isearch-setup|multi-isearch-wrap|multi-occur-in-matching-buffers\n\t|multi-occur|multiple-value-apply|multiple-value-bind|multiple-value-call|multiple-value-list|multiple-value-setq|mwheel-event-button\n\t|mwheel-event-window|mwheel-filter-click-events|mwheel-inhibit-click-timeout|mwheel-install|mwheel-scroll|name-last-kbd-macro|narrow-to-defun\n\t|nato-region|nested-alist-p|net-utils--revert-function|net-utils-machine-at-point|net-utils-mode|net-utils-remove-ctrl-m-filter\n\t|net-utils-run-program|net-utils-run-simple|net-utils-url-at-point|netrc-credentials|netrc-find-service-name|netrc-get|netrc-machine-user-or-password\n\t|netrc-machine|netrc-parse-services|netrc-parse|netrc-port-equal|netstat|network-connection-mode-setup|network-connection-mode|network-connection-reconnect\n\t|network-connection-to-service|network-connection|network-service-connection|network-stream-certificate|network-stream-command\n\t|network-stream-get-response|network-stream-open-plain|network-stream-open-shell|network-stream-open-starttls|network-stream-open-tls\n\t|new-fontset|new-frame|new-mode-local-bindings|newline-cache-check|newsticker--age|newsticker--buffer-beginning-of-feed|newsticker--buffer-beginning-of-item\n\t|newsticker--buffer-do-insert-text|newsticker--buffer-end-of-feed|newsticker--buffer-end-of-item|newsticker--buffer-get-feed-title-at-point\n\t|newsticker--buffer-get-item-title-at-point|newsticker--buffer-goto|newsticker--buffer-hideshow|newsticker--buffer-insert-all-items\n\t|newsticker--buffer-insert-item|newsticker--buffer-make-item-completely-visible|newsticker--buffer-redraw|newsticker--buffer-set-faces\n\t|newsticker--buffer-set-invisibility|newsticker--buffer-set-uptodate|newsticker--buffer-statistics|newsticker--cache-add|newsticker--cache-contains\n\t|newsticker--cache-dir|newsticker--cache-get-feed|newsticker--cache-item-compare-by-position|newsticker--cache-item-compare-by-time\n\t|newsticker--cache-item-compare-by-title|newsticker--cache-mark-expired|newsticker--cache-read-feed|newsticker--cache-read-version1\n\t|newsticker--cache-read|newsticker--cache-remove|newsticker--cache-replace-age|newsticker--cache-save-feed|newsticker--cache-save-version1\n\t|newsticker--cache-save|newsticker--cache-set-preformatted-contents|newsticker--cache-set-preformatted-title|newsticker--cache-sort\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t newsticker--cache-update|newsticker--count-grouped-feeds|newsticker--count-groups|newsticker--debug-msg|newsticker--decode-iso8601-date\n\t|newsticker--decode-rfc822-date|newsticker--desc|newsticker--display-jump|newsticker--display-scroll|newsticker--display-tick\n\t|newsticker--do-forget-preformatted|newsticker--do-mark-item-at-point-as-read|newsticker--do-print-extra-element|newsticker--do-run-auto-mark-filter\n\t|newsticker--do-xml-workarounds|newsticker--echo-area-clean-p|newsticker--enclosure|newsticker--extra|newsticker--forget-preformatted\n\t|newsticker--get-group-names|newsticker--get-icon-url-atom-1\\.0|newsticker--get-logo-url-atom-0\\.3|newsticker--get-logo-url-atom-1\\.0\n\t|newsticker--get-logo-url-rss-0\\.91|newsticker--get-logo-url-rss-0\\.92|newsticker--get-logo-url-rss-1\\.0|newsticker--get-logo-url-rss-2\\.0\n\t|newsticker--get-news-by-funcall|newsticker--get-news-by-url-callback|newsticker--get-news-by-url|newsticker--get-news-by-wget\n\t|newsticker--group-all-groups|newsticker--group-do-find-group|newsticker--group-do-get-group|newsticker--group-do-rename-group\n\t|newsticker--group-find-parent-group|newsticker--group-get-feeds|newsticker--group-get-group|newsticker--group-get-subgroups\n\t|newsticker--group-manage-orphan-feeds|newsticker--group-names|newsticker--group-remove-obsolete-feeds|newsticker--group-shift\n\t|newsticker--guid-to-string|newsticker--guid|newsticker--icon-read|newsticker--icons-dir|newsticker--image-download-by-url-callback\n\t|newsticker--image-download-by-url|newsticker--image-download-by-wget|newsticker--image-get|newsticker--image-read|newsticker--image-remove\n\t|newsticker--image-save|newsticker--image-sentinel|newsticker--images-dir|newsticker--imenu-create-index|newsticker--imenu-goto\n\t|newsticker--insert-enclosure|newsticker--insert-image|newsticker--link|newsticker--lists-intersect-p|newsticker--opml-import-outlines\n\t|newsticker--parse-atom-0\\.3|newsticker--parse-atom-1\\.0|newsticker--parse-generic-feed|newsticker--parse-generic-items|newsticker--parse-rss-0\\.91\n\t|newsticker--parse-rss-0\\.92|newsticker--parse-rss-1\\.0|newsticker--parse-rss-2\\.0|newsticker--pos|newsticker--preformatted-contents\n\t|newsticker--preformatted-title|newsticker--print-extra-elements|newsticker--process-auto-mark-filter-match|newsticker--real-feed-name\n\t|newsticker--remove-whitespace|newsticker--run-auto-mark-filter|newsticker--sentinel-work|newsticker--sentinel|newsticker--set-customvar-buffer\n\t|newsticker--set-customvar-formatting|newsticker--set-customvar-retrieval|newsticker--set-customvar-sorting|newsticker--set-customvar-ticker\n\t|newsticker--set-face-properties|newsticker--splicer|newsticker--start-feed|newsticker--stat-num-items-for-group|newsticker--stat-num-items-total\n\t|newsticker--stat-num-items|newsticker--stop-feed|newsticker--ticker-text-remove|newsticker--ticker-text-setup|newsticker--time\n\t|newsticker--title|newsticker--tree-widget-icon-create|newsticker--treeview-activate-node|newsticker--treeview-buffer-init|newsticker--treeview-count-node-items\n\t|newsticker--treeview-do-get-node-by-id|newsticker--treeview-do-get-node-of-feed|newsticker--treeview-first-feed|newsticker--treeview-frame-init\n\t|newsticker--treeview-get-current-node|newsticker--treeview-get-feed-vfeed|newsticker--treeview-get-first-child|newsticker--treeview-get-id\n\t|newsticker--treeview-get-last-child|newsticker--treeview-get-next-sibling|newsticker--treeview-get-next-uncle|newsticker--treeview-get-node-by-id\n\t|newsticker--treeview-get-node-of-feed|newsticker--treeview-get-other-tree|newsticker--treeview-get-prev-sibling|newsticker--treeview-get-prev-uncle\n\t|newsticker--treeview-get-second-child|newsticker--treeview-get-selected-item|newsticker--treeview-ids-eq|newsticker--treeview-item-buffer\n\t|newsticker--treeview-item-show-text|newsticker--treeview-item-show|newsticker--treeview-item-update|newsticker--treeview-item-window\n\t|newsticker--treeview-list-add-item|newsticker--treeview-list-all-items|newsticker--treeview-list-buffer|newsticker--treeview-list-clear-highlight\n\t|newsticker--treeview-list-clear|newsticker--treeview-list-compare-item-by-age-reverse|newsticker--treeview-list-compare-item-by-age\n\t|newsticker--treeview-list-compare-item-by-time-reverse|newsticker--treeview-list-compare-item-by-time|newsticker--treeview-list-compare-item-by-title-reverse\n\t|newsticker--treeview-list-compare-item-by-title|newsticker--treeview-list-feed-items|newsticker--treeview-list-highlight-start\n\t|newsticker--treeview-list-immortal-items|newsticker--treeview-list-items-v|newsticker--treeview-list-items-with-age-callback\n\t|newsticker--treeview-list-items-with-age|newsticker--treeview-list-items|newsticker--treeview-list-new-items|newsticker--treeview-list-obsolete-items\n\t|newsticker--treeview-list-select|newsticker--treeview-list-sort-by-column|newsticker--treeview-list-sort-items|newsticker--treeview-list-update-faces\n\t|newsticker--treeview-list-update-highlight|newsticker--treeview-list-update|newsticker--treeview-list-window|newsticker--treeview-load\n\t|newsticker--treeview-mark-item|newsticker--treeview-nodes-eq|newsticker--treeview-propertize-tag|newsticker--treeview-render-text\n\t|newsticker--treeview-restore-layout|newsticker--treeview-set-current-node|newsticker--treeview-tree-buffer|newsticker--treeview-tree-do-update-tags\n\t|newsticker--treeview-tree-expand-status|newsticker--treeview-tree-expand|newsticker--treeview-tree-get-tag|newsticker--treeview-tree-open-menu\n\t|newsticker--treeview-tree-update-highlight|newsticker--treeview-tree-update-tag|newsticker--treeview-tree-update-tags|newsticker--treeview-tree-update\n\t|newsticker--treeview-tree-window|newsticker--treeview-unfold-node|newsticker--treeview-virtual-feed-p|newsticker--treeview-window-init\n\t|newsticker--unxml-attribute|newsticker--unxml-node|newsticker--unxml|newsticker--update-process-ids|newsticker-add-url|newsticker-browse-url-item\n\t|newsticker-browse-url|newsticker-buffer-force-update|newsticker-buffer-update|newsticker-close-buffer|newsticker-customize|newsticker-download-enclosures\n\t|newsticker-download-images|newsticker-get-all-news|newsticker-get-news-at-point|newsticker-get-news|newsticker-group-add-group\n\t|newsticker-group-delete-group|newsticker-group-move-feed|newsticker-group-rename-group|newsticker-group-shift-feed-down|newsticker-group-shift-feed-up\n\t|newsticker-group-shift-group-down|newsticker-group-shift-group-up|newsticker-handle-url|newsticker-hide-all-desc|newsticker-hide-entry\n\t|newsticker-hide-extra|newsticker-hide-feed-desc|newsticker-hide-new-item-desc|newsticker-hide-old-item-desc|newsticker-hide-old-items\n\t|newsticker-htmlr-render|newsticker-item-not-immortal-p|newsticker-item-not-old-p|newsticker-mark-all-items-as-read|newsticker-mark-all-items-at-point-as-read-and-redraw\n\t|newsticker-mark-all-items-at-point-as-read|newsticker-mark-all-items-of-feed-as-read|newsticker-mark-item-at-point-as-immortal\n\t|newsticker-mark-item-at-point-as-read|newsticker-mode|newsticker-mouse-browse-url|newsticker-new-item-functions-sample|newsticker-next-feed-available-p\n\t|newsticker-next-feed|newsticker-next-item-available-p|newsticker-next-item-same-feed|newsticker-next-item|newsticker-next-new-item\n\t|newsticker-opml-export|newsticker-opml-import|newsticker-plainview|newsticker-previous-feed-available-p|newsticker-previous-feed\n\t|newsticker-previous-item-available-p|newsticker-previous-item|newsticker-previous-new-item|newsticker-retrieve-random-message\n\t|newsticker-running-p|newsticker-save-item|newsticker-set-auto-narrow-to-feed|newsticker-set-auto-narrow-to-item|newsticker-show-all-desc\n\t|newsticker-show-entry|newsticker-show-extra|newsticker-show-feed-desc|newsticker-show-new-item-desc|newsticker-show-news|newsticker-show-old-item-desc\n\t|newsticker-show-old-items|newsticker-start-ticker|newsticker-start|newsticker-stop-ticker|newsticker-stop|newsticker-ticker-running-p\n\t|newsticker-toggle-auto-narrow-to-feed|newsticker-toggle-auto-narrow-to-item|newsticker-treeview-browse-url-item|newsticker-treeview-browse-url\n\t|newsticker-treeview-get-news|newsticker-treeview-item-mode|newsticker-treeview-jump|newsticker-treeview-list-make-sort-button\n\t|newsticker-treeview-list-mode|newsticker-treeview-mark-item-old|newsticker-treeview-mark-list-items-old|newsticker-treeview-mode\n\t|newsticker-treeview-mouse-browse-url|newsticker-treeview-next-feed|newsticker-treeview-next-item|newsticker-treeview-next-new-or-immortal-item\n\t|newsticker-treeview-next-page|newsticker-treeview-prev-feed|newsticker-treeview-prev-item|newsticker-treeview-prev-new-or-immortal-item\n\t|newsticker-treeview-quit|newsticker-treeview-save-item|newsticker-treeview-save|newsticker-treeview-scroll-item|newsticker-treeview-show-item\n\t|newsticker-treeview-toggle-item-immortal|newsticker-treeview-tree-click|newsticker-treeview-tree-do-click|newsticker-treeview-update\n\t|newsticker-treeview|newsticker-w3m-show-inline-images|next-buffer|next-cdabbrev|next-completion|next-error-buffer-p|next-error-find-buffer\n\t|next-error-follow-minor-mode|next-error-follow-mode-post-command-hook|next-error-internal|next-error-no-select|next-error|next-file\n\t|next-ifdef|next-line-or-history-element|next-line|next-logical-line|next-match|next-method-p|next-multiframe-window|next-page|next-read-file-uses-dialog-p\n\t|nintersection|ninth|nndiary-generate-nov-databases|nndoc-add-type|nndraft-request-associate-buffer|nndraft-request-expire-articles\n\t|nnfolder-generate-active-file|nnheader-accept-process-output|nnheader-article-p|nnheader-article-to-file-alist|nnheader-be-verbose\n\t|nnheader-cancel-function-timers|nnheader-cancel-timer|nnheader-concat|nnheader-directory-articles|nnheader-directory-files-safe\n\t|nnheader-directory-files|nnheader-directory-regular-files|nnheader-fake-message-id-p|nnheader-file-error|nnheader-file-size|nnheader-file-to-group\n\t|nnheader-file-to-number|nnheader-find-etc-directory|nnheader-find-file-noselect|nnheader-find-nov-line|nnheader-fold-continuation-lines\n\t|nnheader-generate-fake-message-id|nnheader-get-lines-and-char|nnheader-get-report-string|nnheader-get-report|nnheader-group-pathname\n\t|nnheader-header-value|nnheader-init-server-buffer|nnheader-insert-article-line|nnheader-insert-buffer-substring|nnheader-insert-file-contents\n\t|nnheader-insert-head|nnheader-insert-header|nnheader-insert-nov-file|nnheader-insert-nov|nnheader-insert-references|nnheader-insert\n\t|nnheader-message-maybe|nnheader-message|nnheader-ms-strip-cr|nnheader-narrow-to-headers|nnheader-nov-delete-outside-range|nnheader-nov-field\n\t|nnheader-nov-parse-extra|nnheader-nov-read-integer|nnheader-nov-read-message-id|nnheader-nov-skip-field|nnheader-parse-head|nnheader-parse-naked-head\n\t|nnheader-parse-nov|nnheader-parse-overview-file|nnheader-re-read-dir|nnheader-remove-body|nnheader-remove-cr-followed-by-lf|nnheader-replace-chars-in-string\n\t|nnheader-replace-duplicate-chars-in-string|nnheader-replace-header|nnheader-replace-regexp|nnheader-replace-string|nnheader-report\n\t|nnheader-set-temp-buffer|nnheader-skeleton-replace|nnheader-strip-cr|nnheader-translate-file-chars|nnheader-update-marks-actions\n\t|nnheader-write-overview-file|nnmail-article-group|nnmail-message-id|nnmail-split-fancy|nnml-generate-nov-databases|nnvirtual-catchup-group\n\t|nnvirtual-convert-headers|nnvirtual-find-group-art|no-applicable-method|no-next-method|nonincremental-re-search-backward|nonincremental-re-search-forward\n\t|nonincremental-repeat-search-backward|nonincremental-repeat-search-forward|nonincremental-search-backward|nonincremental-search-forward\n\t|normal-about-screen|normal-erase-is-backspace-mode|normal-erase-is-backspace-setup-frame|normal-mouse-startup-screen|normal-no-mouse-startup-screen\n\t|normal-splash-screen|normal-top-level-add-subdirs-to-load-path|normal-top-level-add-to-load-path|normal-top-level|notany|notevery\n\t|notifications-on-action-signal|notifications-on-closed-signal|nreconc|nroff-backward-text-line|nroff-comment-indent|nroff-count-text-lines\n\t|nroff-electric-mode|nroff-electric-newline|nroff-forward-text-line|nroff-insert-comment-function|nroff-mode|nroff-outline-level\n\t|nroff-view|nset-difference|nset-exclusive-or|nslookup-host|nslookup-mode|nslookup|nsm-certificate-part|nsm-check-certificate|nsm-check-plain-connection\n\t|nsm-check-protocol|nsm-check-tls-connection|nsm-fingerprint-ok-p|nsm-fingerprint|nsm-format-certificate|nsm-host-settings|nsm-id\n\t|nsm-level|nsm-new-fingerprint-ok-p|nsm-parse-subject|nsm-query-user|nsm-query|nsm-read-settings|nsm-remove-permanent-setting|nsm-remove-temporary-setting\n\t|nsm-save-host|nsm-verify-connection|nsm-warnings-ok-p|nsm-write-settings|nsublis|nsubst-if-not|nsubst-if|nsubst|nsubstitute-if-not\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t nsubstitute-if|nsubstitute|nth-value|ntlm-ascii2unicode|ntlm-build-auth-request|ntlm-build-auth-response|ntlm-get-password-hashes\n\t|ntlm-md4hash|ntlm-smb-des-e-p16|ntlm-smb-des-e-p24|ntlm-smb-dohash|ntlm-smb-hash|ntlm-smb-owf-encrypt|ntlm-smb-passwd-hash|ntlm-smb-str-to-key\n\t|ntlm-string-lshift|ntlm-string-permute|ntlm-string-xor|ntlm-unicode2ascii|nullify-allout-prefix-data|number-at-point|number-to-register\n\t|nunion|nxml-enable-unicode-char-name-sets|nxml-glyph-display-string|nxml-mode|obj-of-class-p|objc-font-lock-keywords-2|objc-font-lock-keywords-3\n\t|objc-font-lock-keywords|objc-mode|object-add-to-list|object-assoc-list-safe|object-assoc-list|object-assoc|object-class-fast|object-class-name\n\t|object-class|object-name-string|object-name|object-of-class-p|object-p|object-print|object-remove-from-list|object-set-name-string\n\t|object-slots|object-write|occur-1|occur-accumulate-lines|occur-after-change-function|occur-cease-edit|occur-context-lines|occur-edit-mode\n\t|occur-engine-add-prefix|occur-engine-line|occur-engine|occur-find-match|occur-mode-display-occurrence|occur-mode-find-occurrence\n\t|occur-mode-goto-occurrence-other-window|occur-mode-goto-occurrence|occur-mode-mouse-goto|occur-mode|occur-next-error|occur-next\n\t|occur-prev|occur-read-primary-args|occur-rename-buffer|occur-revert-function|occur|octave--indent-new-comment-line|octave-add-log-current-defun\n\t|octave-beginning-of-defun|octave-beginning-of-line|octave-complete-symbol|octave-completing-read|octave-completion-at-point|octave-eldoc-function-signatures\n\t|octave-eldoc-function|octave-end-of-line|octave-eval-print-last-sexp|octave-fill-paragraph|octave-find-definition-default-filename\n\t|octave-find-definition|octave-font-lock-texinfo-comment|octave-function-file-comment|octave-function-file-p|octave-goto-function-definition\n\t|octave-help-mode|octave-help|octave-hide-process-buffer|octave-in-comment-p|octave-in-string-or-comment-p|octave-in-string-p|octave-indent-comment\n\t|octave-indent-defun|octave-indent-new-comment-line|octave-insert-defun|octave-kill-process|octave-lookfor|octave-looking-at-kw\n\t|octave-mark-block|octave-maybe-insert-continuation-string|octave-mode-menu|octave-mode|octave-next-code-line|octave-previous-code-line\n\t|octave-send-block|octave-send-buffer|octave-send-defun|octave-send-line|octave-send-region|octave-show-process-buffer|octave-skip-comment-forward\n\t|octave-smie-backward-token|octave-smie-forward-token|octave-smie-rules|octave-source-directories|octave-source-file|octave-submit-bug-report\n\t|octave-sync-function-file-names|octave-syntax-propertize-function|octave-syntax-propertize-sqs|octave-update-function-file-comment\n\t|oddp|opascal-block-start|opascal-char-token-at|opascal-charset-token-at|opascal-column-of|opascal-comment-block-end|opascal-comment-block-start\n\t|opascal-comment-content-start|opascal-comment-indent-of|opascal-composite-type-start|opascal-corrected-indentation|opascal-current-token\n\t|opascal-debug-goto-next-token|opascal-debug-goto-point|opascal-debug-goto-previous-token|opascal-debug-log|opascal-debug-show-current-string\n\t|opascal-debug-show-current-token|opascal-debug-token-string|opascal-debug-tokenize-buffer|opascal-debug-tokenize-region|opascal-debug-tokenize-window\n\t|opascal-else-start|opascal-enclosing-indent-of|opascal-ensure-buffer|opascal-explicit-token-at|opascal-fill-comment|opascal-find-current-body\n\t|opascal-find-current-def|opascal-find-current-xdef|opascal-find-unit-file|opascal-find-unit-in-directory|opascal-find-unit|opascal-group-end\n\t|opascal-group-start|opascal-in-token|opascal-indent-line|opascal-indent-of|opascal-is-block-after-expr-statement|opascal-is-directory\n\t|opascal-is-file|opascal-is-literal-end|opascal-is-simple-class-type|opascal-is-use-clause-end|opascal-is|opascal-line-indent-of\n\t|opascal-literal-end-pattern|opascal-literal-kind|opascal-literal-start-pattern|opascal-literal-stop-pattern|opascal-literal-token-at\n\t|opascal-log-msg|opascal-looking-at-string|opascal-match-token|opascal-mode|opascal-new-comment-line|opascal-next-line-start|opascal-next-token\n\t|opascal-next-visible-token|opascal-on-first-comment-line|opascal-open-group-indent|opascal-point-token-at|opascal-previous-indent-of\n\t|opascal-previous-token|opascal-progress-done|opascal-progress-start|opascal-save-excursion|opascal-search-directory|opascal-section-indent-of\n\t|opascal-set-token-end|opascal-set-token-kind|opascal-set-token-start|opascal-space-token-at|opascal-step-progress|opascal-stmt-line-indent-of\n\t|opascal-string-of|opascal-tab|opascal-token-at|opascal-token-end|opascal-token-kind|opascal-token-of|opascal-token-start|opascal-token-string\n\t|opascal-word-token-at|open-font|open-gnutls-stream|open-line|open-protocol-stream|open-rectangle-line|open-rectangle|open-tls-stream\n\t|operate-on-rectangle|optimize-char-table|oref-default|oref|org-2ft|org-N-empty-lines-before-current|org-activate-angle-links|org-activate-bracket-links\n\t|org-activate-code|org-activate-dates|org-activate-footnote-links|org-activate-mark|org-activate-plain-links|org-activate-tags\n\t|org-activate-target-links|org-adaptive-fill-function|org-add-angle-brackets|org-add-archive-files|org-add-hook|org-add-link-props\n\t|org-add-link-type|org-add-log-note|org-add-log-setup|org-add-note|org-add-planning-info|org-add-prop-inherited|org-add-props|org-advertized-archive-subtree\n\t|org-agenda-check-for-timestamp-as-reason-to-ignore-todo-item|org-agenda-columns|org-agenda-file-p|org-agenda-file-to-front|org-agenda-files\n\t|org-agenda-list-stuck-projects|org-agenda-list|org-agenda-prepare-buffers|org-agenda-set-restriction-lock|org-agenda-to-appt\n\t|org-agenda|org-align-all-tags|org-align-tags-here|org-all-targets|org-apply-on-list|org-apps-regexp-alist|org-archive-subtree-default-with-confirmation\n\t|org-archive-subtree-default|org-archive-subtree|org-archive-to-archive-sibling|org-ascii-export-as-ascii|org-ascii-export-to-ascii\n\t|org-ascii-publish-to-ascii|org-ascii-publish-to-latin1|org-ascii-publish-to-utf8|org-assign-fast-keys|org-at-TBLFM-p|org-at-block-p\n\t|org-at-clock-log-p|org-at-comment-p|org-at-date-range-p|org-at-drawer-p|org-at-heading-or-item-p|org-at-heading-p|org-at-item-bullet-p\n\t|org-at-item-checkbox-p|org-at-item-counter-p|org-at-item-description-p|org-at-item-p|org-at-item-timer-p|org-at-property-p|org-at-regexp-p\n\t|org-at-table-hline-p|org-at-table-p|org-at-table\\.el-p|org-at-target-p|org-at-timestamp-p|org-attach|org-auto-fill-function|org-auto-repeat-maybe\n\t|org-babel--shell-command-on-region|org-babel-active-location-p|org-babel-balanced-split|org-babel-check-confirm-evaluate|org-babel-check-evaluate\n\t|org-babel-check-src-block|org-babel-chomp|org-babel-combine-header-arg-lists|org-babel-comint-buffer-livep|org-babel-comint-eval-invisibly-and-wait-for-file\n\t|org-babel-comint-in-buffer|org-babel-comint-input-command|org-babel-comint-wait-for-output|org-babel-comint-with-output|org-babel-confirm-evaluate\n\t|org-babel-current-result-hash|org-babel-del-hlines|org-babel-demarcate-block|org-babel-describe-bindings|org-babel-detangle|org-babel-disassemble-tables\n\t|org-babel-do-in-edit-buffer|org-babel-do-key-sequence-in-edit-buffer|org-babel-do-load-languages|org-babel-edit-distance|org-babel-enter-header-arg-w-completion\n\t|org-babel-eval-error-notify|org-babel-eval-read-file|org-babel-eval-wipe-error-buffer|org-babel-eval|org-babel-examplize-region\n\t|org-babel-execute-buffer|org-babel-execute-maybe|org-babel-execute-safely-maybe|org-babel-execute-src-block-maybe|org-babel-execute-src-block\n\t|org-babel-execute-subtree|org-babel-execute:emacs-lisp|org-babel-exp-code|org-babel-exp-do-export|org-babel-exp-get-export-buffer\n\t|org-babel-exp-in-export-file|org-babel-exp-process-buffer|org-babel-exp-results|org-babel-exp-src-block|org-babel-expand-body:emacs-lisp\n\t|org-babel-expand-body:generic|org-babel-expand-noweb-references|org-babel-expand-src-block-maybe|org-babel-expand-src-block\n\t|org-babel-find-file-noselect-refresh|org-babel-find-named-block|org-babel-find-named-result|org-babel-format-result|org-babel-get-colnames\n\t|org-babel-get-header|org-babel-get-inline-src-block-matches|org-babel-get-lob-one-liner-matches|org-babel-get-rownames|org-babel-get-src-block-info\n\t|org-babel-goto-named-result|org-babel-goto-named-src-block|org-babel-goto-src-block-head|org-babel-hash-at-point|org-babel-header-arg-expand\n\t|org-babel-hide-all-hashes|org-babel-hide-hash|org-babel-hide-result-toggle-maybe|org-babel-hide-result-toggle|org-babel-import-elisp-from-file\n\t|org-babel-in-example-or-verbatim|org-babel-initiate-session|org-babel-insert-header-arg|org-babel-insert-result|org-babel-join-splits-near-ch\n\t|org-babel-load-file|org-babel-load-in-session-maybe|org-babel-load-in-session|org-babel-lob-execute-maybe|org-babel-lob-execute\n\t|org-babel-lob-get-info|org-babel-lob-ingest|org-babel-local-file-name|org-babel-map-call-lines|org-babel-map-executables|org-babel-map-inline-src-blocks\n\t|org-babel-map-src-blocks|org-babel-mark-block|org-babel-merge-params|org-babel-named-data-regexp-for-name|org-babel-named-src-block-regexp-for-name\n\t|org-babel-next-src-block|org-babel-noweb-p|org-babel-noweb-wrap|org-babel-number-p|org-babel-open-src-block-result|org-babel-params-from-properties\n\t|org-babel-parse-header-arguments|org-babel-parse-inline-src-block-match|org-babel-parse-multiple-vars|org-babel-parse-src-block-match\n\t|org-babel-pick-name|org-babel-pop-to-session-maybe|org-babel-pop-to-session|org-babel-previous-src-block|org-babel-process-file-name\n\t|org-babel-process-params|org-babel-put-colnames|org-babel-put-rownames|org-babel-read-link|org-babel-read-list|org-babel-read-result\n\t|org-babel-read-table|org-babel-read|org-babel-reassemble-table|org-babel-ref-at-ref-p|org-babel-ref-goto-headline-id|org-babel-ref-headline-body\n\t|org-babel-ref-index-list|org-babel-ref-parse|org-babel-ref-resolve|org-babel-ref-split-args|org-babel-remove-result|org-babel-remove-temporary-directory\n\t|org-babel-result-cond|org-babel-result-end|org-babel-result-hide-all|org-babel-result-hide-spec|org-babel-result-names|org-babel-result-to-file\n\t|org-babel-script-escape|org-babel-set-current-result-hash|org-babel-sha1-hash|org-babel-show-result-all|org-babel-spec-to-string\n\t|org-babel-speed-command-activate|org-babel-speed-command-hook|org-babel-src-block-names|org-babel-string-read|org-babel-switch-to-session-with-code\n\t|org-babel-switch-to-session|org-babel-table-truncate-at-newline|org-babel-tangle-clean|org-babel-tangle-collect-blocks|org-babel-tangle-comment-links\n\t|org-babel-tangle-file|org-babel-tangle-jump-to-org|org-babel-tangle-publish|org-babel-tangle-single-block|org-babel-tangle|org-babel-temp-file\n\t|org-babel-tramp-handle-call-process-region|org-babel-trim|org-babel-update-block-body|org-babel-view-src-block-info|org-babel-when-in-src-block\n\t|org-babel-where-is-src-block-head|org-babel-where-is-src-block-result|org-babel-with-temp-filebuffer|org-back-over-empty-lines\n\t|org-back-to-heading|org-backward-element|org-backward-heading-same-level|org-backward-paragraph|org-backward-sentence|org-base-buffer\n\t|org-batch-agenda-csv|org-batch-agenda|org-batch-store-agenda-views|org-bbdb-anniversaries|org-beamer-export-as-latex|org-beamer-export-to-latex\n\t|org-beamer-export-to-pdf|org-beamer-insert-options-template|org-beamer-mode|org-beamer-publish-to-latex|org-beamer-publish-to-pdf\n\t|org-beamer-select-environment|org-before-change-function|org-before-first-heading-p|org-beginning-of-dblock|org-beginning-of-item-list\n\t|org-beginning-of-item|org-beginning-of-line|org-between-regexps-p|org-block-map|org-block-todo-from-checkboxes|org-block-todo-from-children-or-siblings-or-parent\n\t|org-bookmark-jump-unhide|org-bound-and-true-p|org-buffer-list|org-buffer-narrowed-p|org-buffer-property-keys|org-cached-entry-get\n\t|org-calendar-goto-agenda|org-calendar-holiday|org-calendar-select-mouse|org-calendar-select|org-call-for-shift-select|org-call-with-arg\n\t|org-called-interactively-p|org-capture-import-remember-templates|org-capture-string|org-capture|org-cdlatex-math-modify|org-cdlatex-mode\n\t|org-cdlatex-underscore-caret|org-change-tag-in-region|org-char-to-string|org-check-after-date|org-check-agenda-file|org-check-and-save-marker\n\t|org-check-before-date|org-check-before-invisible-edit|org-check-dates-range|org-check-deadlines|org-check-external-command|org-check-for-hidden\n\t|org-check-running-clock|org-check-version|org-clean-visibility-after-subtree-move|org-clock-cancel|org-clock-display|org-clock-get-clocktable\n\t|org-clock-goto|org-clock-in-last|org-clock-in|org-clock-is-active|org-clock-out|org-clock-persistence-insinuate|org-clock-remove-overlays\n\t|org-clock-report|org-clock-sum|org-clock-update-time-maybe|org-clocktable-shift|org-clocktable-try-shift|org-clone-local-variables\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t org-clone-subtree-with-time-shift|org-closest-date|org-columns-compute|org-columns-get-format-and-top-level|org-columns-number-to-string\n\t|org-columns-remove-overlays|org-columns|org-combine-plists|org-command-at-point|org-comment-line-break-function|org-comment-or-uncomment-region\n\t|org-compatible-face|org-complete-expand-structure-template|org-completing-read-no-i|org-completing-read|org-compute-latex-and-related-regexp\n\t|org-compute-property-at-point|org-content|org-context-p|org-context|org-contextualize-keys|org-contextualize-validate-key|org-convert-to-odd-levels\n\t|org-convert-to-oddeven-levels|org-copy-face|org-copy-special|org-copy-subtree|org-copy-visible|org-copy|org-count-lines|org-count\n\t|org-create-customize-menu|org-create-dblock|org-create-formula--latex-header|org-create-formula-image-with-dvipng|org-create-formula-image-with-imagemagick\n\t|org-create-formula-image|org-create-math-formula|org-create-multibrace-regexp|org-ctrl-c-ctrl-c|org-ctrl-c-minus|org-ctrl-c-ret\n\t|org-ctrl-c-star|org-current-effective-time|org-current-level|org-current-line-string|org-current-line|org-current-time|org-cursor-to-region-beginning\n\t|org-customize|org-cut-special|org-cut-subtree|org-cycle-agenda-files|org-cycle-hide-archived-subtrees|org-cycle-hide-drawers|org-cycle-hide-inline-tasks\n\t|org-cycle-internal-global|org-cycle-internal-local|org-cycle-item-indentation|org-cycle-level|org-cycle-list-bullet|org-cycle-show-empty-lines\n\t|org-cycle|org-date-from-calendar|org-date-to-gregorian|org-datetree-find-date-create|org-days-to-iso-week|org-days-to-time|org-dblock-update\n\t|org-dblock-write:clocktable|org-dblock-write:columnview|org-deadline-close|org-deadline|org-decompose-region|org-default-apps\n\t|org-defkey|org-defvaralias|org-delete-all|org-delete-backward-char|org-delete-char|org-delete-directory|org-delete-property-globally\n\t|org-delete-property|org-demote-subtree|org-demote|org-detach-overlay|org-diary-sexp-entry|org-diary-to-ical-string|org-diary|org-display-custom-time\n\t|org-display-inline-images|org-display-inline-modification-hook|org-display-inline-remove-overlay|org-display-outline-path|org-display-warning\n\t|org-do-demote|org-do-emphasis-faces|org-do-latex-and-related|org-do-occur|org-do-promote|org-do-remove-indentation|org-do-sort\n\t|org-do-wrap|org-down-element|org-drag-element-backward|org-drag-element-forward|org-drag-line-backward|org-drag-line-forward|org-duration-string-to-minutes\n\t|org-dvipng-color-format|org-dvipng-color|org-edit-agenda-file-list|org-edit-fixed-width-region|org-edit-special|org-edit-src-abort\n\t|org-edit-src-code|org-edit-src-continue|org-edit-src-exit|org-edit-src-find-buffer|org-edit-src-find-region-and-lang|org-edit-src-get-indentation\n\t|org-edit-src-get-label-format|org-edit-src-get-lang|org-edit-src-save|org-element-at-point|org-element-context|org-element-interpret-data\n\t|org-email-link-description|org-emphasize|org-end-of-item-list|org-end-of-item|org-end-of-line|org-end-of-meta-data-and-drawers\n\t|org-end-of-subtree|org-entities-create-table|org-entities-help|org-entity-get-representation|org-entity-get|org-entity-latex-math-p\n\t|org-entry-add-to-multivalued-property|org-entry-beginning-position|org-entry-blocked-p|org-entry-delete|org-entry-end-position\n\t|org-entry-get-multivalued-property|org-entry-get-with-inheritance|org-entry-get|org-entry-is-done-p|org-entry-is-todo-p|org-entry-member-in-multivalued-property\n\t|org-entry-properties|org-entry-protect-space|org-entry-put-multivalued-property|org-entry-put|org-entry-remove-from-multivalued-property\n\t|org-entry-restore-space|org-escape-code-in-region|org-escape-code-in-string|org-eval-in-calendar|org-eval-in-environment|org-eval\n\t|org-evaluate-time-range|org-every|org-export-as|org-export-dispatch|org-export-insert-default-template|org-export-replace-region-by\n\t|org-export-string-as|org-export-to-buffer|org-export-to-file|org-extract-attributes|org-extract-log-state-settings|org-face-from-face-or-color\n\t|org-fast-tag-insert|org-fast-tag-selection|org-fast-tag-show-exit|org-fast-todo-selection|org-feed-goto-inbox|org-feed-show-raw-feed\n\t|org-feed-update-all|org-feed-update|org-file-apps-entry-match-against-dlink-p|org-file-complete-link|org-file-contents|org-file-equal-p\n\t|org-file-image-p|org-file-menu-entry|org-file-remote-p|org-files-list|org-fill-line-break-nobreak-p|org-fill-paragraph-with-timestamp-nobreak-p\n\t|org-fill-paragraph|org-fill-template|org-find-base-buffer-visiting|org-find-dblock|org-find-entry-with-id|org-find-exact-heading-in-directory\n\t|org-find-exact-headline-in-buffer|org-find-file-at-mouse|org-find-if|org-find-invisible-foreground|org-find-invisible|org-find-library-dir\n\t|org-find-olp|org-find-overlays|org-find-text-property-in-string|org-find-visible|org-first-headline-recenter|org-first-sibling-p\n\t|org-fit-window-to-buffer|org-fix-decoded-time|org-fix-indentation|org-fix-position-after-promote|org-fix-tags-on-the-fly|org-fixup-indentation\n\t|org-fixup-message-id-for-http|org-flag-drawer|org-flag-heading|org-flag-subtree|org-float-time|org-floor\\*|org-follow-timestamp-link\n\t|org-font-lock-add-priority-faces|org-font-lock-add-tag-faces|org-font-lock-ensure|org-font-lock-hook|org-fontify-entities|org-fontify-like-in-org-mode\n\t|org-fontify-meta-lines-and-blocks-1|org-fontify-meta-lines-and-blocks|org-footnote-action|org-footnote-all-labels|org-footnote-at-definition-p\n\t|org-footnote-at-reference-p|org-footnote-auto-adjust-maybe|org-footnote-create-definition|org-footnote-delete-definitions|org-footnote-delete-references\n\t|org-footnote-delete|org-footnote-get-definition|org-footnote-get-next-reference|org-footnote-goto-definition|org-footnote-goto-local-insertion-point\n\t|org-footnote-goto-previous-reference|org-footnote-in-valid-context-p|org-footnote-new|org-footnote-next-reference-or-definition\n\t|org-footnote-normalize-label|org-footnote-normalize|org-footnote-renumber-fn:N|org-footnote-unique-label|org-force-cycle-archived\n\t|org-force-self-insert|org-format-latex-as-mathml|org-format-latex-mathml-available-p|org-format-latex|org-format-outline-path\n\t|org-format-seconds|org-forward-element|org-forward-heading-same-level|org-forward-paragraph|org-forward-sentence|org-get-agenda-file-buffer\n\t|org-get-alist-option|org-get-at-bol|org-get-buffer-for-internal-link|org-get-buffer-tags|org-get-category|org-get-checkbox-statistics-face\n\t|org-get-compact-tod|org-get-cursor-date|org-get-date-from-calendar|org-get-deadline-time|org-get-entry|org-get-export-keywords\n\t|org-get-heading|org-get-indentation|org-get-indirect-buffer|org-get-last-sibling|org-get-level-face|org-get-limited-outline-regexp\n\t|org-get-local-tags-at|org-get-local-tags|org-get-local-variables|org-get-location|org-get-next-sibling|org-get-org-file|org-get-outline-path\n\t|org-get-packages-alist|org-get-previous-line-level|org-get-priority|org-get-property-block|org-get-repeat|org-get-scheduled-time\n\t|org-get-string-indentation|org-get-tag-face|org-get-tags-at|org-get-tags-string|org-get-tags|org-get-todo-face|org-get-todo-sequence-head\n\t|org-get-todo-state|org-get-valid-level|org-get-wdays|org-get-x-clipboard-compat|org-get-x-clipboard|org-git-version|org-global-cycle\n\t|org-global-tags-completion-table|org-goto-calendar|org-goto-first-child|org-goto-left|org-goto-line|org-goto-local-auto-isearch\n\t|org-goto-local-search-headings|org-goto-map|org-goto-marker-or-bmk|org-goto-quit|org-goto-ret|org-goto-right|org-goto-sibling|org-goto\n\t|org-heading-components|org-hh:mm-string-to-minutes|org-hidden-tree-error|org-hide-archived-subtrees|org-hide-block-all|org-hide-block-toggle-all\n\t|org-hide-block-toggle-maybe|org-hide-block-toggle|org-hide-wide-columns|org-highlight-new-match|org-hours-to-clocksum-string\n\t|org-html-convert-region-to-html|org-html-export-as-html|org-html-export-to-html|org-html-htmlize-generate-css|org-html-publish-to-html\n\t|org-icalendar-combine-agenda-files|org-icalendar-export-agenda-files|org-icalendar-export-to-ics|org-icompleting-read|org-id-copy\n\t|org-id-find-id-file|org-id-find|org-id-get-create|org-id-get-with-outline-drilling|org-id-get-with-outline-path-completion|org-id-get\n\t|org-id-goto|org-id-new|org-id-store-link|org-id-update-id-locations|org-ido-switchb|org-image-file-name-regexp|org-imenu-get-tree\n\t|org-imenu-new-marker|org-in-block-p|org-in-clocktable-p|org-in-commented-line|org-in-drawer-p|org-in-fixed-width-region-p|org-in-indented-comment-line\n\t|org-in-invisibility-spec-p|org-in-item-p|org-in-regexp|org-in-src-block-p|org-in-subtree-not-table-p|org-in-verbatim-emphasis\n\t|org-inc-effort|org-indent-block|org-indent-drawer|org-indent-item-tree|org-indent-item|org-indent-line-to|org-indent-line|org-indent-mode\n\t|org-indent-region|org-indent-to-column|org-info|org-inhibit-invisibility|org-insert-all-links|org-insert-columns-dblock|org-insert-comment\n\t|org-insert-drawer|org-insert-heading-after-current|org-insert-heading-respect-content|org-insert-heading|org-insert-item|org-insert-link-global\n\t|org-insert-link|org-insert-property-drawer|org-insert-subheading|org-insert-time-stamp|org-insert-todo-heading-respect-content\n\t|org-insert-todo-heading|org-insert-todo-subheading|org-inside-LaTeX-fragment-p|org-inside-latex-macro-p|org-install-agenda-files-menu\n\t|org-invisible-p2|org-irc-store-link|org-iread-file-name|org-isearch-end|org-isearch-post-command|org-iswitchb-completing-read\n\t|org-iswitchb|org-item-beginning-re|org-item-re|org-key|org-kill-is-subtree-p|org-kill-line|org-kill-new|org-kill-note-or-show-branches\n\t|org-last|org-latex-color-format|org-latex-color|org-latex-convert-region-to-latex|org-latex-export-as-latex|org-latex-export-to-latex\n\t|org-latex-export-to-pdf|org-latex-packages-to-string|org-latex-publish-to-latex|org-latex-publish-to-pdf|org-let|org-let2|org-level-increment\n\t|org-link-display-format|org-link-escape|org-link-expand-abbrev|org-link-fontify-links-to-this-file|org-link-prettify|org-link-search\n\t|org-link-try-special-completion|org-link-unescape-compound|org-link-unescape-single-byte-sequence|org-link-unescape|org-list-at-regexp-after-bullet-p\n\t|org-list-bullet-string|org-list-context|org-list-delete-item|org-list-get-all-items|org-list-get-bottom-point|org-list-get-bullet\n\t|org-list-get-checkbox|org-list-get-children|org-list-get-counter|org-list-get-first-item|org-list-get-ind|org-list-get-item-begin\n\t|org-list-get-item-end-before-blank|org-list-get-item-end|org-list-get-item-number|org-list-get-last-item|org-list-get-list-begin\n\t|org-list-get-list-end|org-list-get-list-type|org-list-get-next-item|org-list-get-nth|org-list-get-parent|org-list-get-prev-item\n\t|org-list-get-subtree|org-list-get-tag|org-list-get-top-point|org-list-has-child-p|org-list-in-valid-context-p|org-list-inc-bullet-maybe\n\t|org-list-indent-item-generic|org-list-insert-item|org-list-insert-radio-list|org-list-item-body-column|org-list-item-trim-br\n\t|org-list-make-subtree|org-list-parents-alist|org-list-prevs-alist|org-list-repair|org-list-search-backward|org-list-search-forward\n\t|org-list-search-generic|org-list-send-item|org-list-send-list|org-list-separating-blank-lines-number|org-list-set-bullet|org-list-set-checkbox\n\t|org-list-set-ind|org-list-set-item-visibility|org-list-set-nth|org-list-struct-apply-struct|org-list-struct-assoc-end|org-list-struct-fix-box\n\t|org-list-struct-fix-bul|org-list-struct-fix-ind|org-list-struct-fix-item-end|org-list-struct-indent|org-list-struct-outdent|org-list-swap-items\n\t|org-list-to-generic|org-list-to-html|org-list-to-latex|org-list-to-subtree|org-list-to-texinfo|org-list-use-alpha-bul-p|org-list-write-struct\n\t|org-load-modules-maybe|org-load-noerror-mustsuffix|org-local-logging|org-log-into-drawer|org-looking-at-p|org-looking-back|org-macro--collect-macros\n\t|org-macro-expand|org-macro-initialize-templates|org-macro-replace-all|org-make-link-regexps|org-make-link-string|org-make-options-regexp\n\t|org-make-org-heading-search-string|org-make-parameter-alist|org-make-tags-matcher|org-make-target-link-regexp|org-make-tdiff-string\n\t|org-map-dblocks|org-map-entries|org-map-region|org-map-tree|org-mark-element|org-mark-ring-goto|org-mark-ring-push|org-mark-subtree\n\t|org-match-any-p|org-match-line|org-match-sparse-tree|org-match-string-no-properties|org-matcher-time|org-maybe-intangible|org-md-convert-region-to-md\n\t|org-md-export-as-markdown|org-md-export-to-markdown|org-meta-return|org-metadown|org-metaleft|org-metaright|org-metaup|org-minutes-to-clocksum-string\n\t|org-minutes-to-hh:mm-string|org-mobile-pull|org-mobile-push|org-mode-flyspell-verify|org-mode-restart|org-mode|org-modifier-cursor-error\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t org-modify-ts-extra|org-move-item-down|org-move-item-up|org-move-subtree-down|org-move-subtree-up|org-move-to-column|org-narrow-to-block\n\t|org-narrow-to-element|org-narrow-to-subtree|org-next-block|org-next-item|org-next-link|org-no-popups|org-no-properties|org-no-read-only\n\t|org-no-warnings|org-normalize-color|org-not-nil|org-notes-order-reversed-p|org-number-sequence|org-occur-in-agenda-files|org-occur-link-in-agenda-files\n\t|org-occur-next-match|org-occur|org-odt-convert|org-odt-export-as-odf-and-open|org-odt-export-as-odf|org-odt-export-to-odt|org-offer-links-in-entry\n\t|org-olpath-completing-read|org-on-heading-p|org-on-target-p|org-op-to-function|org-open-at-mouse|org-open-at-point-global|org-open-at-point\n\t|org-open-file-with-emacs|org-open-file-with-system|org-open-file|org-open-line|org-open-link-from-string|org-optimize-window-after-visibility-change\n\t|org-order-calendar-date-args|org-org-export-as-org|org-org-export-to-org|org-org-menu|org-org-publish-to-org|org-outdent-item-tree\n\t|org-outdent-item|org-outline-level|org-outline-overlay-data|org-overlay-before-string|org-overlay-display|org-overview|org-parse-arguments\n\t|org-parse-time-string|org-paste-special|org-paste-subtree|org-pcomplete-case-double|org-pcomplete-initial|org-plist-delete|org-plot\\/gnuplot\n\t|org-point-at-end-of-empty-headline|org-point-in-group|org-pop-to-buffer-same-window|org-pos-in-match-range|org-prepare-dblock\n\t|org-preserve-lc|org-preview-latex-fragment|org-previous-block|org-previous-item|org-previous-line-empty-p|org-previous-link|org-print-speed-command\n\t|org-priority-down|org-priority-up|org-priority|org-promote-subtree|org-promote|org-propertize|org-property-action|org-property-get-allowed-values\n\t|org-property-inherit-p|org-property-next-allowed-value|org-property-or-variable-value|org-property-previous-allowed-value|org-property-values\n\t|org-protect-slash|org-publish-all|org-publish-current-file|org-publish-current-project|org-publish-project|org-publish|org-quote-csv-field\n\t|org-quote-vert|org-raise-scripts|org-re-property|org-re-timestamp|org-re|org-read-agenda-file-list|org-read-date-analyze|org-read-date-display\n\t|org-read-date-get-relative|org-read-date|org-read-property-name|org-read-property-value|org-rear-nonsticky-at|org-recenter-calendar\n\t|org-redisplay-inline-images|org-reduce|org-reduced-level|org-refile--get-location|org-refile-cache-check-set|org-refile-cache-clear\n\t|org-refile-cache-get|org-refile-cache-put|org-refile-check-position|org-refile-get-location|org-refile-get-targets|org-refile-goto-last-stored\n\t|org-refile-marker|org-refile-new-child|org-refile|org-refresh-category-properties|org-refresh-properties|org-reftex-citation|org-region-active-p\n\t|org-reinstall-markers-in-region|org-release-buffers|org-release|org-reload|org-remap|org-remove-angle-brackets|org-remove-double-quotes\n\t|org-remove-empty-drawer-at|org-remove-empty-overlays-at|org-remove-file|org-remove-flyspell-overlays-in|org-remove-font-lock-display-properties\n\t|org-remove-from-invisibility-spec|org-remove-if-not|org-remove-if|org-remove-indentation|org-remove-inline-images|org-remove-keyword-keys\n\t|org-remove-latex-fragment-image-overlays|org-remove-occur-highlights|org-remove-tabs|org-remove-timestamp-with-keyword|org-remove-uninherited-tags\n\t|org-replace-escapes|org-replace-match-keep-properties|org-require-autoloaded-modules|org-reset-checkbox-state-subtree|org-resolve-clocks\n\t|org-restart-font-lock|org-return-indent|org-return|org-reveal|org-reverse-string|org-revert-all-org-buffers|org-run-like-in-org-mode\n\t|org-save-all-org-buffers|org-save-markers-in-region|org-save-outline-visibility|org-sbe|org-scan-tags|org-schedule|org-search-not-self\n\t|org-search-view|org-select-frame-set-input-focus|org-self-insert-command|org-set-current-tags-overlay|org-set-effort|org-set-emph-re\n\t|org-set-font-lock-defaults|org-set-frame-title|org-set-local|org-set-modules|org-set-outline-overlay-data|org-set-packages-alist\n\t|org-set-property-and-value|org-set-property-function|org-set-property|org-set-regexps-and-options-for-tags|org-set-regexps-and-options\n\t|org-set-startup-visibility|org-set-tag-faces|org-set-tags-command|org-set-tags-to|org-set-tags|org-set-transient-map|org-set-visibility-according-to-property\n\t|org-setup-comments-handling|org-setup-filling|org-shiftcontroldown|org-shiftcontrolleft|org-shiftcontrolright|org-shiftcontrolup\n\t|org-shiftdown|org-shiftleft|org-shiftmetadown|org-shiftmetaleft|org-shiftmetaright|org-shiftmetaup|org-shiftright|org-shiftselect-error\n\t|org-shifttab|org-shiftup|org-shorten-string|org-show-block-all|org-show-context|org-show-empty-lines-in-parent|org-show-entry|org-show-hidden-entry\n\t|org-show-priority|org-show-siblings|org-show-subtree|org-show-todo-tree|org-skip-over-state-notes|org-skip-whitespace|org-small-year-to-year\n\t|org-some|org-sort-entries|org-sort-list|org-sort-remove-invisible|org-sort|org-sparse-tree|org-speed-command-activate|org-speed-command-default-hook\n\t|org-speed-command-help|org-speed-move-safe|org-speedbar-set-agenda-restriction|org-splice-latex-header|org-split-string|org-src-associate-babel-session\n\t|org-src-babel-configure-edit-buffer|org-src-construct-edit-buffer-name|org-src-do-at-code-block|org-src-do-key-sequence-at-code-block\n\t|org-src-edit-buffer-p|org-src-font-lock-fontify-block|org-src-fontify-block|org-src-fontify-buffer|org-src-get-lang-mode|org-src-in-org-buffer\n\t|org-src-mode-configure-edit-buffer|org-src-mode|org-src-native-tab-command-maybe|org-src-switch-to-buffer|org-src-tangle|org-store-agenda-views\n\t|org-store-link-props|org-store-link|org-store-log-note|org-store-new-agenda-file-list|org-string-match-p|org-string-nw-p|org-string-width\n\t|org-string<=|org-string<>|org-string>|org-string>=|org-sublist|org-submit-bug-report|org-substitute-posix-classes|org-subtree-end-visible-p\n\t|org-switch-to-buffer-other-window|org-switchb|org-table-align|org-table-begin|org-table-blank-field|org-table-convert-region|org-table-convert\n\t|org-table-copy-down|org-table-copy-region|org-table-create-or-convert-from-region|org-table-create-with-table\\.el|org-table-create\n\t|org-table-current-dline|org-table-cut-region|org-table-delete-column|org-table-edit-field|org-table-edit-formulas|org-table-end\n\t|org-table-eval-formula|org-table-export|org-table-field-info|org-table-get-stored-formulas|org-table-goto-column|org-table-hline-and-move\n\t|org-table-import|org-table-insert-column|org-table-insert-hline|org-table-insert-row|org-table-iterate-buffer-tables|org-table-iterate\n\t|org-table-justify-field-maybe|org-table-kill-row|org-table-map-tables|org-table-maybe-eval-formula|org-table-maybe-recalculate-line\n\t|org-table-move-column-left|org-table-move-column-right|org-table-move-column|org-table-move-row-down|org-table-move-row-up|org-table-move-row\n\t|org-table-next-field|org-table-next-row|org-table-p|org-table-paste-rectangle|org-table-previous-field|org-table-recalculate-buffer-tables\n\t|org-table-recalculate|org-table-recognize-table\\.el|org-table-rotate-recalc-marks|org-table-set-constants|org-table-sort-lines\n\t|org-table-sum|org-table-to-lisp|org-table-toggle-coordinate-overlays|org-table-toggle-formula-debugger|org-table-wrap-region\n\t|org-tag-inherit-p|org-tags-completion-function|org-tags-expand|org-tags-sparse-tree|org-tags-view|org-tbl-menu|org-texinfo-convert-region-to-texinfo\n\t|org-texinfo-publish-to-texinfo|org-thing-at-point|org-time-from-absolute|org-time-stamp-format|org-time-stamp-inactive|org-time-stamp-to-now\n\t|org-time-stamp|org-time-string-to-absolute|org-time-string-to-seconds|org-time-string-to-time|org-time-today|org-time<|org-time<=\n\t|org-time<>|org-time=|org-time>|org-time>=|org-timer-change-times-in-region|org-timer-item|org-timer-set-timer|org-timer-start|org-timer\n\t|org-timestamp-change|org-timestamp-down-day|org-timestamp-down|org-timestamp-format|org-timestamp-has-time-p|org-timestamp-split-range\n\t|org-timestamp-translate|org-timestamp-up-day|org-timestamp-up|org-today|org-todo-list|org-todo-trigger-tag-changes|org-todo-yesterday\n\t|org-todo|org-toggle-archive-tag|org-toggle-checkbox|org-toggle-comment|org-toggle-custom-properties-visibility|org-toggle-fixed-width-section\n\t|org-toggle-heading|org-toggle-inline-images|org-toggle-item|org-toggle-link-display|org-toggle-ordered-property|org-toggle-pretty-entities\n\t|org-toggle-sticky-agenda|org-toggle-tag|org-toggle-tags-groups|org-toggle-time-stamp-overlays|org-toggle-timestamp-type|org-tr-level\n\t|org-translate-link-from-planner|org-translate-link|org-translate-time|org-transpose-element|org-transpose-words|org-tree-to-indirect-buffer\n\t|org-trim|org-truely-invisible-p|org-try-cdlatex-tab|org-try-structure-completion|org-unescape-code-in-region|org-unescape-code-in-string\n\t|org-unfontify-region|org-unindent-buffer|org-uniquify-alist|org-uniquify|org-unlogged-message|org-unmodified|org-up-element|org-up-heading-all\n\t|org-up-heading-safe|org-update-all-dblocks|org-update-checkbox-count-maybe|org-update-checkbox-count|org-update-dblock|org-update-parent-todo-statistics\n\t|org-update-property-plist|org-update-radio-target-regexp|org-update-statistics-cookies|org-uuidgen-p|org-version-check|org-version\n\t|org-with-gensyms|org-with-limited-levels|org-with-point-at|org-with-remote-undo|org-with-silent-modifications|org-with-wide-buffer\n\t|org-without-partial-completion|org-wrap|org-xemacs-without-invisibility|org-xor|org-yank-folding-would-swallow-text|org-yank-generic\n\t|org-yank|org<>|orgstruct\\+\\+-mode|orgstruct-error|orgstruct-make-binding|orgstruct-mode|orgstruct-setup|orgtbl-mode|orgtbl-to-csv\n\t|orgtbl-to-generic|orgtbl-to-html|orgtbl-to-latex|orgtbl-to-orgtbl|orgtbl-to-texinfo|orgtbl-to-tsv|oset-default|oset|other-frame|other-window-for-scrolling\n\t|outline-back-to-heading|outline-backward-same-level|outline-demote|outline-end-of-heading|outline-end-of-subtree|outline-flag-region\n\t|outline-flag-subtree|outline-font-lock-face|outline-forward-same-level|outline-get-last-sibling|outline-get-next-sibling|outline-head-from-level\n\t|outline-headers-as-kill|outline-insert-heading|outline-invent-heading|outline-invisible-p|outline-isearch-open-invisible|outline-level\n\t|outline-map-region|outline-mark-subtree|outline-minor-mode|outline-mode|outline-move-subtree-down|outline-move-subtree-up|outline-next-heading\n\t|outline-next-preface|outline-next-visible-heading|outline-on-heading-p|outline-previous-heading|outline-previous-visible-heading\n\t|outline-promote|outline-reveal-toggle-invisible|outline-show-heading|outline-toggle-children|outline-up-heading|outlineify-sticky\n\t|outlinify-sticky|overlay-lists|overload-docstring-extension|overload-obsoleted-by|overload-that-obsolete|package--ac-desc-extras--cmacro\n\t|package--ac-desc-extras|package--ac-desc-kind--cmacro|package--ac-desc-kind|package--ac-desc-reqs--cmacro|package--ac-desc-reqs\n\t|package--ac-desc-summary--cmacro|package--ac-desc-summary|package--ac-desc-version--cmacro|package--ac-desc-version|package--add-to-archive-contents\n\t|package--alist-to-plist-args|package--archive-file-exists-p|package--bi-desc-reqs--cmacro|package--bi-desc-reqs|package--bi-desc-summary--cmacro\n\t|package--bi-desc-summary|package--bi-desc-version--cmacro|package--bi-desc-version|package--check-signature|package--compile\n\t|package--description-file|package--display-verify-error|package--download-one-archive|package--from-builtin|package--has-keyword-p\n\t|package--list-loaded-files|package--make-autoloads-and-stuff|package--mapc|package--prepare-dependencies|package--push|package--read-archive-file\n\t|package--with-work-buffer|package--write-file-no-coding|package-activate-1|package-activate|package-all-keywords|package-archive-base\n\t|package-autoload-ensure-default-file|package-buffer-info|package-built-in-p|package-compute-transaction|package-delete|package-desc--keywords\n\t|package-desc-archive--cmacro|package-desc-archive|package-desc-create--cmacro|package-desc-create|package-desc-dir--cmacro|package-desc-dir\n\t|package-desc-extras--cmacro|package-desc-extras|package-desc-from-define|package-desc-full-name|package-desc-kind--cmacro|package-desc-kind\n\t|package-desc-name--cmacro|package-desc-name|package-desc-p--cmacro|package-desc-p|package-desc-reqs--cmacro|package-desc-reqs\n\t|package-desc-signed--cmacro|package-desc-signed|package-desc-status|package-desc-suffix|package-desc-summary--cmacro|package-desc-summary\n\t|package-desc-version--cmacro|package-desc-version|package-disabled-p|package-download-transaction|package-generate-autoloads\n\t|package-generate-description-file|package-import-keyring|package-install-button-action|package-install-file|package-install-from-archive\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t package-install-from-buffer|package-install|package-installed-p|package-keyword-button-action|package-list-packages-no-fetch\n\t|package-list-packages|package-load-all-descriptors|package-load-descriptor|package-make-ac-desc--cmacro|package-make-ac-desc\n\t|package-make-builtin--cmacro|package-make-builtin|package-make-button|package-menu--archive-predicate|package-menu--description-predicate\n\t|package-menu--find-upgrades|package-menu--generate|package-menu--name-predicate|package-menu--print-info|package-menu--refresh\n\t|package-menu--status-predicate|package-menu--version-predicate|package-menu-backup-unmark|package-menu-describe-package|package-menu-execute\n\t|package-menu-filter|package-menu-get-status|package-menu-mark-delete|package-menu-mark-install|package-menu-mark-obsolete-for-deletion\n\t|package-menu-mark-unmark|package-menu-mark-upgrades|package-menu-mode|package-menu-quick-help|package-menu-refresh|package-menu-view-commentary\n\t|package-process-define-package|package-read-all-archive-contents|package-read-archive-contents|package-read-from-string|package-refresh-contents\n\t|package-show-package-list|package-strip-rcs-id|package-tar-file-info|package-unpack|package-untar-buffer|package-version-join\n\t|pages-copy-header-and-position|pages-directory-address-mode|pages-directory-for-addresses|pages-directory-goto-with-mouse|pages-directory-goto\n\t|pages-directory-mode|pages-directory|pairlis|paragraph-indent-minor-mode|paragraph-indent-text-mode|parse-iso8601-time-string\n\t|parse-time-string-chars|parse-time-string|parse-time-tokenize|pascal-beg-of-defun|pascal-build-defun-re|pascal-calculate-indent\n\t|pascal-capitalize-keywords|pascal-change-keywords|pascal-comment-area|pascal-comp-defun|pascal-complete-word|pascal-completion\n\t|pascal-completions-at-point|pascal-declaration-beg|pascal-declaration-end|pascal-downcase-keywords|pascal-end-of-defun|pascal-end-of-statement\n\t|pascal-func-completion|pascal-get-completion-decl|pascal-get-default-symbol|pascal-get-lineup-indent|pascal-goto-defun|pascal-hide-other-defuns\n\t|pascal-indent-case|pascal-indent-command|pascal-indent-comment|pascal-indent-declaration|pascal-indent-level|pascal-indent-line\n\t|pascal-indent-paramlist|pascal-insert-block|pascal-keyword-completion|pascal-mark-defun|pascal-mode|pascal-outline-change|pascal-outline-goto-defun\n\t|pascal-outline-mode|pascal-outline-next-defun|pascal-outline-prev-defun|pascal-outline|pascal-set-auto-comments|pascal-show-all\n\t|pascal-show-completions|pascal-star-comment|pascal-string-diff|pascal-type-completion|pascal-uncomment-area|pascal-upcase-keywords\n\t|pascal-var-completion|pascal-within-string|password-cache-add|password-cache-remove|password-in-cache-p|password-read-and-add\n\t|password-read-from-cache|password-read|password-reset|pcase--and|pcase--app-subst-match|pcase--app-subst-rest|pcase--eval|pcase--expand\n\t|pcase--fgrep|pcase--flip|pcase--funcall|pcase--if|pcase--let\\*|pcase--macroexpand|pcase--mark-used|pcase--match|pcase--mutually-exclusive-p\n\t|pcase--self-quoting-p|pcase--small-branch-p|pcase--split-equal|pcase--split-match|pcase--split-member|pcase--split-pred|pcase--split-rest\n\t|pcase--trivial-upat-p|pcase--u|pcase--u1|pcase-codegen|pcase-defmacro|pcase-dolist|pcase-exhaustive|pcase-let\\*|pcase-let|pcomplete\\/ack-grep\n\t|pcomplete\\/ack|pcomplete\\/ag|pcomplete\\/bzip2|pcomplete\\/cd|pcomplete\\/chgrp|pcomplete\\/chown|pcomplete\\/cvs|pcomplete\\/erc-mode\\/CLEARTOPIC\n\t|pcomplete\\/erc-mode\\/CTCP|pcomplete\\/erc-mode\\/DCC|pcomplete\\/erc-mode\\/DEOP|pcomplete\\/erc-mode\\/DESCRIBE|pcomplete\\/erc-mode\\/IDLE\n\t|pcomplete\\/erc-mode\\/KICK|pcomplete\\/erc-mode\\/LEAVE|pcomplete\\/erc-mode\\/LOAD|pcomplete\\/erc-mode\\/ME|pcomplete\\/erc-mode\\/MODE\n\t|pcomplete\\/erc-mode\\/MSG|pcomplete\\/erc-mode\\/NAMES|pcomplete\\/erc-mode\\/NOTICE|pcomplete\\/erc-mode\\/NOTIFY|pcomplete\\/erc-mode\\/OP\n\t|pcomplete\\/erc-mode\\/PART|pcomplete\\/erc-mode\\/QUERY|pcomplete\\/erc-mode\\/SAY|pcomplete\\/erc-mode\\/SOUND|pcomplete\\/erc-mode\\/TOPIC\n\t|pcomplete\\/erc-mode\\/UNIGNORE|pcomplete\\/erc-mode\\/WHOIS|pcomplete\\/erc-mode\\/complete-command|pcomplete\\/eshell-mode\\/eshell-debug\n\t|pcomplete\\/eshell-mode\\/export|pcomplete\\/eshell-mode\\/setq|pcomplete\\/eshell-mode\\/unset|pcomplete\\/gdb|pcomplete\\/gzip\n\t|pcomplete\\/kill|pcomplete\\/make|pcomplete\\/mount|pcomplete\\/org-mode\\/block-option\\/clocktable|pcomplete\\/org-mode\\/block-option\\/src\n\t|pcomplete\\/org-mode\\/drawer|pcomplete\\/org-mode\\/file-option\\/author|pcomplete\\/org-mode\\/file-option\\/bind|pcomplete\\/org-mode\\/file-option\\/date\n\t|pcomplete\\/org-mode\\/file-option\\/email|pcomplete\\/org-mode\\/file-option\\/exclude_tags|pcomplete\\/org-mode\\/file-option\\/filetags\n\t|pcomplete\\/org-mode\\/file-option\\/infojs_opt|pcomplete\\/org-mode\\/file-option\\/language|pcomplete\\/org-mode\\/file-option\\/options\n\t|pcomplete\\/org-mode\\/file-option\\/priorities|pcomplete\\/org-mode\\/file-option\\/select_tags|pcomplete\\/org-mode\\/file-option\\/startup\n\t|pcomplete\\/org-mode\\/file-option\\/tags|pcomplete\\/org-mode\\/file-option\\/title|pcomplete\\/org-mode\\/file-option|pcomplete\\/org-mode\\/link\n\t|pcomplete\\/org-mode\\/prop|pcomplete\\/org-mode\\/searchhead|pcomplete\\/org-mode\\/tag|pcomplete\\/org-mode\\/tex|pcomplete\\/org-mode\\/todo\n\t|pcomplete\\/pushd|pcomplete\\/rm|pcomplete\\/rmdir|pcomplete\\/rpm|pcomplete\\/scp|pcomplete\\/ssh|pcomplete\\/tar|pcomplete\\/time\n\t|pcomplete\\/tlmgr|pcomplete\\/umount|pcomplete\\/which|pcomplete\\/xargs|pcomplete--common-suffix|pcomplete--entries|pcomplete--help\n\t|pcomplete--here|pcomplete--test|pcomplete-actual-arg|pcomplete-all-entries|pcomplete-arg|pcomplete-begin|pcomplete-comint-setup\n\t|pcomplete-command-name|pcomplete-completions-at-point|pcomplete-completions|pcomplete-continue|pcomplete-dirs-or-entries|pcomplete-dirs\n\t|pcomplete-do-complete|pcomplete-entries|pcomplete-erc-all-nicks|pcomplete-erc-channels|pcomplete-erc-command-name|pcomplete-erc-commands\n\t|pcomplete-erc-nicks|pcomplete-erc-not-ops|pcomplete-erc-ops|pcomplete-erc-parse-arguments|pcomplete-erc-setup|pcomplete-event-matches-key-specifier-p\n\t|pcomplete-executables|pcomplete-expand-and-complete|pcomplete-expand|pcomplete-find-completion-function|pcomplete-help|pcomplete-here\\*\n\t|pcomplete-here|pcomplete-insert-entry|pcomplete-list|pcomplete-match-beginning|pcomplete-match-end|pcomplete-match-string|pcomplete-match\n\t|pcomplete-next-arg|pcomplete-opt|pcomplete-parse-arguments|pcomplete-parse-buffer-arguments|pcomplete-parse-comint-arguments\n\t|pcomplete-process-result|pcomplete-quote-argument|pcomplete-read-event|pcomplete-restore-windows|pcomplete-reverse|pcomplete-shell-setup\n\t|pcomplete-show-completions|pcomplete-std-complete|pcomplete-stub|pcomplete-test|pcomplete-uniqify-list|pcomplete-unquote-argument\n\t|pcomplete|pdb|pending-delete-mode|perl-backward-to-noncomment|perl-backward-to-start-of-continued-exp|perl-beginning-of-function\n\t|perl-calculate-indent|perl-comment-indent|perl-continuation-line-p|perl-current-defun-name|perl-electric-noindent-p|perl-electric-terminator\n\t|perl-end-of-function|perl-font-lock-syntactic-face-function|perl-hanging-paren-p|perl-indent-command|perl-indent-exp|perl-indent-line\n\t|perl-indent-new-calculate|perl-mark-function|perl-mode|perl-outline-level|perl-quote-syntax-table|perl-syntax-propertize-function\n\t|perl-syntax-propertize-special-constructs|perldb|picture-backward-clear-column|picture-backward-column|picture-beginning-of-line\n\t|picture-clear-column|picture-clear-line|picture-clear-rectangle-to-register|picture-clear-rectangle|picture-current-line|picture-delete-char\n\t|picture-draw-rectangle|picture-duplicate-line|picture-end-of-line|picture-forward-column|picture-insert-rectangle|picture-insert\n\t|picture-mode-exit|picture-mode|picture-motion-reverse|picture-motion|picture-mouse-set-point|picture-move-down|picture-move-up\n\t|picture-move|picture-movement-down|picture-movement-left|picture-movement-ne|picture-movement-nw|picture-movement-right|picture-movement-se\n\t|picture-movement-sw|picture-movement-up|picture-newline|picture-open-line|picture-replace-match|picture-self-insert|picture-set-motion\n\t|picture-set-tab-stops|picture-snarf-rectangle|picture-tab-search|picture-tab|picture-update-desired-column|picture-yank-at-click\n\t|picture-yank-rectangle-from-register|picture-yank-rectangle|pike-font-lock-keywords-2|pike-font-lock-keywords-3|pike-font-lock-keywords\n\t|pike-mode|ping|plain-TeX-mode|plain-tex-mode|play-sound-internal|plstore-delete|plstore-find|plstore-get-file|plstore-mode|plstore-open\n\t|plstore-put|plstore-save|plusp|po-find-charset|po-find-file-coding-system-guts|po-find-file-coding-system|point-at-bol|point-at-eol\n\t|point-to-register|pong-display-options|pong-init-buffer|pong-init|pong-move-down|pong-move-left|pong-move-right|pong-move-up|pong-pause\n\t|pong-quit|pong-resume|pong-update-bat|pong-update-game|pong-update-score|pong|pop-global-mark|pop-tag-mark|pop-to-buffer-same-window\n\t|pop-to-mark-command|pop3-movemail|popup-menu-normalize-position|popup-menu|position-if-not|position-if|position|posn-set-point|post-read-decode-hz\n\t|pp-buffer|pp-display-expression|pp-eval-expression|pp-eval-last-sexp|pp-last-sexp|pp-macroexpand-expression|pp-macroexpand-last-sexp\n\t|pp-to-string|pr-alist-custom-set|pr-article-date|pr-auto-mode-p|pr-call-process|pr-choice-alist|pr-command|pr-complete-alist|pr-create-interface\n\t|pr-customize|pr-delete-file-if-exists|pr-delete-file|pr-despool-preview|pr-despool-print|pr-despool-ps-print|pr-despool-using-ghostscript\n\t|pr-do-update-menus|pr-dosify-file-name|pr-eval-alist|pr-eval-local-alist|pr-eval-setting-alist|pr-even-or-odd-pages|pr-expand-file-name\n\t|pr-file-list|pr-find-buffer-visiting|pr-find-command|pr-get-symbol|pr-global-menubar|pr-gnus-lpr|pr-gnus-print|pr-help|pr-i-directory\n\t|pr-i-ps-send|pr-insert-button|pr-insert-checkbox|pr-insert-italic|pr-insert-menu|pr-insert-radio-button|pr-insert-section-1|pr-insert-section-2\n\t|pr-insert-section-3|pr-insert-section-4|pr-insert-section-5|pr-insert-section-6|pr-insert-section-7|pr-insert-toggle|pr-interactive-dir-args\n\t|pr-interactive-dir|pr-interactive-n-up-file|pr-interactive-n-up-inout|pr-interactive-n-up|pr-interactive-ps-dir-args|pr-interactive-regexp\n\t|pr-interface-directory|pr-interface-help|pr-interface-infile|pr-interface-outfile|pr-interface-preview|pr-interface-printify|pr-interface-ps-print\n\t|pr-interface-ps|pr-interface-quit|pr-interface-save|pr-interface-txt-print|pr-interface|pr-keep-region-active|pr-kill-help|pr-kill-local-variable\n\t|pr-local-variable|pr-lpr-message-from-summary|pr-menu-alist|pr-menu-bind|pr-menu-char-height|pr-menu-char-width|pr-menu-create\n\t|pr-menu-get-item|pr-menu-index|pr-menu-lock|pr-menu-lookup|pr-menu-position|pr-menu-set-item-name|pr-menu-set-ps-title|pr-menu-set-txt-title\n\t|pr-menu-set-utility-title|pr-mh-current-message|pr-mh-lpr-1|pr-mh-lpr-2|pr-mh-print-1|pr-mh-print-2|pr-mode-alist-p|pr-mode-lpr\n\t|pr-mode-print|pr-path-command|pr-printify-buffer|pr-printify-directory|pr-printify-region|pr-prompt-gs|pr-prompt-region|pr-prompt\n\t|pr-ps-buffer-preview|pr-ps-buffer-print|pr-ps-buffer-ps-print|pr-ps-buffer-using-ghostscript|pr-ps-directory-preview|pr-ps-directory-print\n\t|pr-ps-directory-ps-print|pr-ps-directory-using-ghostscript|pr-ps-fast-fire|pr-ps-file-list|pr-ps-file-preview|pr-ps-file-print\n\t|pr-ps-file-ps-print|pr-ps-file-up-preview|pr-ps-file-up-ps-print|pr-ps-file-using-ghostscript|pr-ps-file|pr-ps-infile-preprint\n\t|pr-ps-message-from-summary|pr-ps-mode-preview|pr-ps-mode-print|pr-ps-mode-ps-print|pr-ps-mode-using-ghostscript|pr-ps-mode|pr-ps-name-custom-set\n\t|pr-ps-name|pr-ps-outfile-preprint|pr-ps-preview|pr-ps-print|pr-ps-region-preview|pr-ps-region-print|pr-ps-region-ps-print|pr-ps-region-using-ghostscript\n\t|pr-ps-set-printer|pr-ps-set-utility|pr-ps-using-ghostscript|pr-ps-utility-args|pr-ps-utility-custom-set|pr-ps-utility-process\n\t|pr-ps-utility|pr-read-string|pr-region-active-p|pr-region-active-string|pr-region-active-symbol|pr-remove-nil-from-list|pr-rmail-lpr\n\t|pr-rmail-print|pr-save-file-modes|pr-set-dir-args|pr-set-keymap-name|pr-set-keymap-parents|pr-set-n-up-and-filename|pr-set-outfilename\n\t|pr-set-ps-dir-args|pr-setup|pr-show-lpr-setup|pr-show-pr-setup|pr-show-ps-setup|pr-show-setup|pr-standard-file-name|pr-switches-string\n\t|pr-switches|pr-text2ps|pr-toggle-duplex-menu|pr-toggle-duplex|pr-toggle-faces-menu|pr-toggle-faces|pr-toggle-file-duplex-menu|pr-toggle-file-duplex\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t pr-toggle-file-landscape-menu|pr-toggle-file-landscape|pr-toggle-file-tumble-menu|pr-toggle-file-tumble|pr-toggle-ghostscript-menu\n\t|pr-toggle-ghostscript|pr-toggle-header-frame-menu|pr-toggle-header-frame|pr-toggle-header-menu|pr-toggle-header|pr-toggle-landscape-menu\n\t|pr-toggle-landscape|pr-toggle-line-menu|pr-toggle-line|pr-toggle-lock-menu|pr-toggle-lock|pr-toggle-mode-menu|pr-toggle-mode|pr-toggle-region-menu\n\t|pr-toggle-region|pr-toggle-spool-menu|pr-toggle-spool|pr-toggle-tumble-menu|pr-toggle-tumble|pr-toggle-upside-down-menu|pr-toggle-upside-down\n\t|pr-toggle-zebra-menu|pr-toggle-zebra|pr-toggle|pr-txt-buffer|pr-txt-directory|pr-txt-fast-fire|pr-txt-mode|pr-txt-name-custom-set\n\t|pr-txt-name|pr-txt-print|pr-txt-region|pr-txt-set-printer|pr-unixify-file-name|pr-update-checkbox|pr-update-menus|pr-update-mode-line\n\t|pr-update-radio-button|pr-update-var|pr-using-ghostscript-p|pr-visible-p|pr-vm-lpr|pr-vm-print|pr-widget-field-action|pre-write-encode-hz\n\t|preceding-sexp|prefer-coding-system|prepare-abbrev-list-buffer|prepend-to-buffer|prepend-to-register|prettify-symbols--compose-symbol\n\t|prettify-symbols--make-keywords|prettify-symbols-mode-set-explicitly|prettify-symbols-mode|previous-buffer|previous-completion\n\t|previous-error-no-select|previous-error|previous-ifdef|previous-line-or-history-element|previous-line|previous-logical-line|previous-multiframe-window\n\t|previous-page|prin1-char|princ-list|print-buffer|print-help-return-message|print-region-1|print-region-new-buffer|print-region|printify-region\n\t|proced-<|proced-auto-update-timer|proced-children-alist|proced-children-pids|proced-do-mark-all|proced-do-mark|proced-filter-children\n\t|proced-filter-interactive|proced-filter-parents|proced-filter|proced-format-args|proced-format-interactive|proced-format-start\n\t|proced-format-time|proced-format-tree|proced-format-ttname|proced-format|proced-header-line|proced-help|proced-insert-mark|proced-log-summary\n\t|proced-log|proced-mark-all|proced-mark-children|proced-mark-parents|proced-mark-process-alist|proced-mark|proced-marked-processes\n\t|proced-marker-regexp|proced-menu|proced-mode|proced-move-to-goal-column|proced-omit-process|proced-omit-processes|proced-pid-at-point\n\t|proced-process-attributes|proced-process-tree-internal|proced-process-tree|proced-refine|proced-renice|proced-revert|proced-send-signal\n\t|proced-sort-header|proced-sort-interactive|proced-sort-p|proced-sort-pcpu|proced-sort-pid|proced-sort-pmem|proced-sort-start|proced-sort-time\n\t|proced-sort-user|proced-sort|proced-string-lessp|proced-success-message|proced-time-lessp|proced-toggle-auto-update|proced-toggle-marks\n\t|proced-toggle-tree|proced-tree-insert|proced-tree|proced-undo|proced-unmark-all|proced-unmark-backward|proced-unmark|proced-update\n\t|proced-why|proced-with-processes-buffer|proced-xor|proced|process-filter-multibyte-p|process-inherit-coding-system-flag|process-kill-without-query\n\t|process-menu-delete-process|process-menu-mode|process-menu-visit-buffer|proclaim|produce-allout-mode-menubar-entries|profiler-calltree-build-1\n\t|profiler-calltree-build-unified|profiler-calltree-build|profiler-calltree-children--cmacro|profiler-calltree-children|profiler-calltree-compute-percentages\n\t|profiler-calltree-count--cmacro|profiler-calltree-count-percent--cmacro|profiler-calltree-count-percent|profiler-calltree-count\n\t|profiler-calltree-count<|profiler-calltree-count>|profiler-calltree-depth|profiler-calltree-entry--cmacro|profiler-calltree-entry\n\t|profiler-calltree-find|profiler-calltree-leaf-p|profiler-calltree-p--cmacro|profiler-calltree-p|profiler-calltree-parent--cmacro\n\t|profiler-calltree-parent|profiler-calltree-sort|profiler-calltree-walk|profiler-compare-logs|profiler-compare-profiles|profiler-cpu-log\n\t|profiler-cpu-profile|profiler-cpu-running-p|profiler-cpu-start|profiler-cpu-stop|profiler-ensure-string|profiler-find-profile-other-frame\n\t|profiler-find-profile-other-window|profiler-find-profile|profiler-fixup-backtrace|profiler-fixup-entry|profiler-fixup-log|profiler-fixup-profile\n\t|profiler-format-entry|profiler-format-number|profiler-format-percent|profiler-format|profiler-make-calltree--cmacro|profiler-make-calltree\n\t|profiler-make-profile--cmacro|profiler-make-profile|profiler-memory-log|profiler-memory-profile|profiler-memory-running-p|profiler-memory-start\n\t|profiler-memory-stop|profiler-profile-diff-p--cmacro|profiler-profile-diff-p|profiler-profile-log--cmacro|profiler-profile-log\n\t|profiler-profile-tag--cmacro|profiler-profile-tag|profiler-profile-timestamp--cmacro|profiler-profile-timestamp|profiler-profile-type--cmacro\n\t|profiler-profile-type|profiler-profile-version--cmacro|profiler-profile-version|profiler-read-profile|profiler-report-ascending-sort\n\t|profiler-report-calltree-at-point|profiler-report-collapse-entry|profiler-report-compare-profile|profiler-report-cpu|profiler-report-descending-sort\n\t|profiler-report-describe-entry|profiler-report-expand-entry|profiler-report-find-entry|profiler-report-header-line-format|profiler-report-insert-calltree-children\n\t|profiler-report-insert-calltree|profiler-report-line-format|profiler-report-make-buffer-name|profiler-report-make-entry-part\n\t|profiler-report-make-name-part|profiler-report-memory|profiler-report-menu|profiler-report-mode|profiler-report-move-to-entry\n\t|profiler-report-next-entry|profiler-report-previous-entry|profiler-report-profile-other-frame|profiler-report-profile-other-window\n\t|profiler-report-profile|profiler-report-render-calltree-1|profiler-report-render-calltree|profiler-report-render-reversed-calltree\n\t|profiler-report-rerender-calltree|profiler-report-setup-buffer-1|profiler-report-setup-buffer|profiler-report-toggle-entry|profiler-report-write-profile\n\t|profiler-report|profiler-reset|profiler-running-p|profiler-start|profiler-stop|profiler-write-profile|prog-indent-sexp|progress-reporter-do-update\n\t|progv|project-add-file|project-compile-project|project-compile-target|project-debug-target|project-delete-target|project-dist-files\n\t|project-edit-file-target|project-interactive-select-target|project-make-dist|project-new-target-custom|project-new-target|project-remove-file\n\t|project-rescan|project-run-target|prolog-Info-follow-nearest-node|prolog-atleast-version|prolog-atom-under-point|prolog-beginning-of-clause\n\t|prolog-beginning-of-predicate|prolog-bsts|prolog-buffer-module|prolog-build-info-alist|prolog-build-prolog-command|prolog-clause-end\n\t|prolog-clause-info|prolog-clause-start|prolog-comment-limits|prolog-compile-buffer|prolog-compile-file|prolog-compile-predicate\n\t|prolog-compile-region|prolog-compile-string|prolog-consult-buffer|prolog-consult-compile-buffer|prolog-consult-compile-file|prolog-consult-compile-filter\n\t|prolog-consult-compile-predicate|prolog-consult-compile-region|prolog-consult-compile|prolog-consult-file|prolog-consult-predicate\n\t|prolog-consult-region|prolog-consult-string|prolog-debug-off|prolog-debug-on|prolog-disable-sicstus-sd|prolog-do-auto-fill|prolog-edit-menu-insert-move\n\t|prolog-edit-menu-runtime|prolog-electric--colon|prolog-electric--dash|prolog-electric--dot|prolog-electric--if-then-else|prolog-electric--underscore\n\t|prolog-enable-sicstus-sd|prolog-end-of-clause|prolog-end-of-predicate|prolog-ensure-process|prolog-face-name-p|prolog-fill-paragraph\n\t|prolog-find-documentation|prolog-find-term|prolog-find-unmatched-paren|prolog-find-value-by-system|prolog-font-lock-keywords\n\t|prolog-font-lock-object-matcher|prolog-get-predspec|prolog-goto-predicate-info|prolog-goto-prolog-process-buffer|prolog-guess-fill-prefix\n\t|prolog-help-apropos|prolog-help-info|prolog-help-on-predicate|prolog-help-online|prolog-in-object|prolog-indent-buffer|prolog-indent-predicate\n\t|prolog-inferior-buffer|prolog-inferior-guess-flavor|prolog-inferior-menu-all|prolog-inferior-menu|prolog-inferior-mode|prolog-inferior-self-insert-command\n\t|prolog-input-filter|prolog-insert-module-modeline|prolog-insert-next-clause|prolog-insert-predicate-template|prolog-insert-predspec\n\t|prolog-mark-clause|prolog-mark-predicate|prolog-menu-help|prolog-menu|prolog-mode-keybindings-common|prolog-mode-keybindings-edit\n\t|prolog-mode-keybindings-inferior|prolog-mode-variables|prolog-mode-version|prolog-mode|prolog-old-process-buffer|prolog-old-process-file\n\t|prolog-old-process-predicate|prolog-old-process-region|prolog-paren-balance|prolog-parse-sicstus-compilation-errors|prolog-post-self-insert\n\t|prolog-pred-end|prolog-pred-start|prolog-process-insert-string|prolog-program-name|prolog-program-switches|prolog-prompt-regexp\n\t|prolog-read-predicate|prolog-replace-in-string|prolog-smie-backward-token|prolog-smie-forward-token|prolog-smie-rules|prolog-temporary-file\n\t|prolog-toggle-sicstus-sd|prolog-trace-off|prolog-trace-on|prolog-uncomment-region|prolog-variables-to-anonymous|prolog-view-predspec\n\t|prolog-zip-off|prolog-zip-on|prompt-for-change-log-name|propertized-buffer-identification|prune-directory-list|ps-alist-position\n\t|ps-avg-char-width|ps-background-image|ps-background-pages|ps-background-text|ps-background|ps-basic-plot-str|ps-basic-plot-string\n\t|ps-basic-plot-whitespace|ps-begin-file|ps-begin-job|ps-begin-page|ps-boolean-capitalized|ps-boolean-constant|ps-build-reference-face-lists\n\t|ps-color-device|ps-color-scale|ps-color-values|ps-comment-string|ps-continue-line|ps-control-character|ps-count-lines-preprint\n\t|ps-count-lines|ps-del|ps-despool|ps-do-despool|ps-end-job|ps-end-page|ps-end-sheet|ps-extend-face-list|ps-extend-face|ps-extension-bit\n\t|ps-face-attribute-list|ps-face-attributes|ps-face-background-color-p|ps-face-background-name|ps-face-background|ps-face-bold-p\n\t|ps-face-box-p|ps-face-color-p|ps-face-extract-color|ps-face-foreground-color-p|ps-face-foreground-name|ps-face-italic-p|ps-face-overline-p\n\t|ps-face-strikeout-p|ps-face-underlined-p|ps-find-wrappoint|ps-float-format|ps-flush-output|ps-font-alist|ps-font-lock-face-attributes\n\t|ps-font-number|ps-font|ps-fonts|ps-format-color|ps-frame-parameter|ps-generate-header-line|ps-generate-header|ps-generate-postscript-with-faces\n\t|ps-generate-postscript-with-faces1|ps-generate-postscript|ps-generate|ps-get-boundingbox|ps-get-buffer-name|ps-get-font-size|ps-get-page-dimensions\n\t|ps-get-size|ps-get|ps-header-dirpart|ps-header-page|ps-header-sheet|ps-init-output-queue|ps-insert-file|ps-insert-string|ps-kill-emacs-check\n\t|ps-line-height|ps-line-lengths-internal|ps-line-lengths|ps-lookup|ps-map-face|ps-mark-active-p|ps-message-log-max|ps-mode--syntax-propertize-special\n\t|ps-mode-RE|ps-mode-backward-delete-char|ps-mode-center|ps-mode-comment-out-region|ps-mode-epsf-rich|ps-mode-epsf-sparse|ps-mode-heapsort\n\t|ps-mode-latin-extended|ps-mode-main|ps-mode-octal-buffer|ps-mode-octal-region|ps-mode-other-newline|ps-mode-print-buffer|ps-mode-print-region\n\t|ps-mode-right|ps-mode-show-version|ps-mode-smie-rules|ps-mode-submit-bug-report|ps-mode-syntax-propertize|ps-mode-target-column\n\t|ps-mode-uncomment-region|ps-mode|ps-mule-begin-job|ps-mule-end-job|ps-mule-initialize|ps-n-up-columns|ps-n-up-end|ps-n-up-filling\n\t|ps-n-up-landscape|ps-n-up-lines|ps-n-up-missing|ps-n-up-printing|ps-n-up-repeat|ps-n-up-xcolumn|ps-n-up-xline|ps-n-up-xstart|ps-n-up-ycolumn\n\t|ps-n-up-yline|ps-n-up-ystart|ps-nb-pages-buffer|ps-nb-pages-region|ps-nb-pages|ps-next-line|ps-next-page|ps-output-boolean|ps-output-frame-properties\n\t|ps-output-prologue|ps-output-string-prim|ps-output-string|ps-output|ps-page-dimensions-get-height|ps-page-dimensions-get-media\n\t|ps-page-dimensions-get-width|ps-page-number|ps-plot-region|ps-plot-string|ps-plot-with-face|ps-plot|ps-print-buffer-with-faces\n\t|ps-print-buffer|ps-print-customize|ps-print-ensure-fontified|ps-print-page-p|ps-print-preprint-region|ps-print-preprint|ps-print-quote\n\t|ps-print-region-with-faces|ps-print-region|ps-print-sheet-p|ps-print-with-faces|ps-print-without-faces|ps-printing-region|ps-prologue-file\n\t|ps-put|ps-remove-duplicates|ps-restore-selected-pages|ps-rgb-color|ps-run-boundingbox|ps-run-buffer|ps-run-cleanup|ps-run-clear\n\t|ps-run-goto-error|ps-run-kill|ps-run-make-tmp-filename|ps-run-mode|ps-run-mouse-goto-error|ps-run-quit|ps-run-region|ps-run-running\n\t|ps-run-send-string|ps-run-start|ps-screen-to-bit-face|ps-select-font|ps-selected-pages|ps-set-bg|ps-set-color|ps-set-face-attribute\n\t|ps-set-face-bold|ps-set-face-italic|ps-set-face-underline|ps-set-font|ps-setup|ps-size-scale|ps-skip-newline|ps-space-width|ps-spool-buffer-with-faces\n\t|ps-spool-buffer|ps-spool-region-with-faces|ps-spool-region|ps-spool-with-faces|ps-spool-without-faces|ps-time-stamp-hh:mm:ss|ps-time-stamp-iso8601\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t ps-time-stamp-locale-default|ps-time-stamp-mon-dd-yyyy|ps-time-stamp-yyyy-mm-dd|ps-title-line-height|ps-value-string|ps-value\n\t|psetf|psetq|push-mark-command|pushnew|put-unicode-property-internal|pwd|python-check|python-comint-output-filter-function|python-comint-postoutput-scroll-to-bottom\n\t|python-completion-at-point|python-completion-complete-at-point|python-define-auxiliary-skeleton|python-docstring-at-p|python-eldoc--get-doc-at-point\n\t|python-eldoc-at-point|python-eldoc-function|python-electric-pair-string-delimiter|python-ffap-module-path|python-fill-comment\n\t|python-fill-decorator|python-fill-paragraph|python-fill-paren|python-fill-string|python-font-lock-syntactic-face-function|python-imenu--build-tree\n\t|python-imenu--put-parent|python-imenu-create-flat-index|python-imenu-create-index|python-imenu-format-item-label|python-imenu-format-parent-item-jump-label\n\t|python-imenu-format-parent-item-label|python-indent-calculate-indentation|python-indent-calculate-levels|python-indent-context\n\t|python-indent-dedent-line-backspace|python-indent-dedent-line|python-indent-guess-indent-offset|python-indent-line-function\n\t|python-indent-line|python-indent-post-self-insert-function|python-indent-region|python-indent-shift-left|python-indent-shift-right\n\t|python-indent-toggle-levels|python-info-assignment-continuation-line-p|python-info-beginning-of-backslash|python-info-beginning-of-block-p\n\t|python-info-beginning-of-statement-p|python-info-block-continuation-line-p|python-info-closing-block-message|python-info-closing-block\n\t|python-info-continuation-line-p|python-info-current-defun|python-info-current-line-comment-p|python-info-current-line-empty-p\n\t|python-info-current-symbol|python-info-dedenter-opening-block-message|python-info-dedenter-opening-block-position|python-info-dedenter-opening-block-positions\n\t|python-info-dedenter-statement-p|python-info-encoding-from-cookie|python-info-encoding|python-info-end-of-block-p|python-info-end-of-statement-p\n\t|python-info-line-ends-backslash-p|python-info-looking-at-beginning-of-defun|python-info-ppss-comment-or-string-p|python-info-ppss-context-type\n\t|python-info-ppss-context|python-info-statement-ends-block-p|python-info-statement-starts-block-p|python-menu|python-mode|python-nav--beginning-of-defun\n\t|python-nav--forward-defun|python-nav--forward-sexp|python-nav--lisp-forward-sexp-safe|python-nav--lisp-forward-sexp|python-nav--syntactically\n\t|python-nav--up-list|python-nav-backward-block|python-nav-backward-defun|python-nav-backward-sexp-safe|python-nav-backward-sexp\n\t|python-nav-backward-statement|python-nav-backward-up-list|python-nav-beginning-of-block|python-nav-beginning-of-defun|python-nav-beginning-of-statement\n\t|python-nav-end-of-block|python-nav-end-of-defun|python-nav-end-of-statement|python-nav-forward-block|python-nav-forward-defun\n\t|python-nav-forward-sexp-safe|python-nav-forward-sexp|python-nav-forward-statement|python-nav-if-name-main|python-nav-up-list\n\t|python-pdbtrack-comint-output-filter-function|python-pdbtrack-set-tracked-buffer|python-proc|python-send-receive|python-send-string\n\t|python-shell--save-temp-file|python-shell-accept-process-output|python-shell-buffer-substring|python-shell-calculate-command\n\t|python-shell-calculate-exec-path|python-shell-calculate-process-environment|python-shell-calculate-pythonpath|python-shell-comint-end-of-output-p\n\t|python-shell-completion-at-point|python-shell-completion-complete-at-point|python-shell-completion-complete-or-indent|python-shell-completion-get-completions\n\t|python-shell-font-lock-cleanup-buffer|python-shell-font-lock-comint-output-filter-function|python-shell-font-lock-get-or-create-buffer\n\t|python-shell-font-lock-kill-buffer|python-shell-font-lock-post-command-hook|python-shell-font-lock-toggle|python-shell-font-lock-turn-off\n\t|python-shell-font-lock-turn-on|python-shell-font-lock-with-font-lock-buffer|python-shell-get-buffer|python-shell-get-or-create-process\n\t|python-shell-get-process-name|python-shell-get-process|python-shell-internal-get-or-create-process|python-shell-internal-get-process-name\n\t|python-shell-internal-send-string|python-shell-make-comint|python-shell-output-filter|python-shell-package-enable|python-shell-parse-command\n\t|python-shell-prompt-detect|python-shell-prompt-set-calculated-regexps|python-shell-prompt-validate-regexps|python-shell-send-buffer\n\t|python-shell-send-defun|python-shell-send-file|python-shell-send-region|python-shell-send-setup-code|python-shell-send-string-no-output\n\t|python-shell-send-string|python-shell-switch-to-shell|python-shell-with-shell-buffer|python-skeleton--else|python-skeleton--except\n\t|python-skeleton--finally|python-skeleton-add-menu-items|python-skeleton-class|python-skeleton-def|python-skeleton-define|python-skeleton-for\n\t|python-skeleton-if|python-skeleton-import|python-skeleton-try|python-skeleton-while|python-syntax-comment-or-string-p|python-syntax-context-type\n\t|python-syntax-context|python-syntax-count-quotes|python-syntax-stringify|python-util-clone-local-variables|python-util-comint-last-prompt\n\t|python-util-forward-comment|python-util-goto-line|python-util-list-directories|python-util-list-files|python-util-list-packages\n\t|python-util-popn|python-util-strip-string|python-util-text-properties-replace-name|python-util-valid-regexp-p|quail-define-package\n\t|quail-define-rules|quail-defrule-internal|quail-defrule|quail-install-decode-map|quail-install-map|quail-set-keyboard-layout|quail-show-keyboard-layout\n\t|quail-title|quail-update-leim-list-file|quail-use-package|query-dig|query-font|query-fontset|query-replace-compile-replacement\n\t|query-replace-descr|query-replace-read-args|query-replace-read-from|query-replace-read-to|query-replace-regexp-eval|query-replace-regexp\n\t|query-replace|quick-calc|quickurl-add-url|quickurl-ask|quickurl-browse-url-ask|quickurl-browse-url|quickurl-edit-urls|quickurl-find-url\n\t|quickurl-grab-url|quickurl-insert|quickurl-list-add-url|quickurl-list-insert-lookup|quickurl-list-insert-naked-url|quickurl-list-insert-url\n\t|quickurl-list-insert-with-desc|quickurl-list-insert-with-lookup|quickurl-list-insert|quickurl-list-make-inserter|quickurl-list-mode\n\t|quickurl-list-mouse-select|quickurl-list-populate-buffer|quickurl-list-quit|quickurl-list|quickurl-load-urls|quickurl-make-url\n\t|quickurl-read|quickurl-save-urls|quickurl-url-comment|quickurl-url-commented-p|quickurl-url-description|quickurl-url-keyword|quickurl-url-url\n\t|quickurl|quit-windows-on|quoted-insert|quoted-printable-decode-region|quoted-printable-decode-string|quoted-printable-encode-region\n\t|r2b-barf-output|r2b-capitalize-title-region|r2b-capitalize-title|r2b-clear-variables|r2b-convert-buffer|r2b-convert-month|r2b-convert-record\n\t|r2b-get-field|r2b-help|r2b-isa-proceedings|r2b-isa-university|r2b-match|r2b-moveq|r2b-put-field|r2b-require|r2b-reset|r2b-set-match\n\t|r2b-snarf-input|r2b-trace|r2b-warning|radians-to-degrees|raise-sexp|random\\*|random-state-p|rassoc\\*|rassoc-if-not|rassoc-if|rcirc--connection-open-p\n\t|rcirc-abbreviate|rcirc-activity-string|rcirc-add-face|rcirc-add-or-remove|rcirc-any-buffer|rcirc-authenticate|rcirc-browse-url\n\t|rcirc-buffer-nick|rcirc-buffer-process|rcirc-change-major-mode-hook|rcirc-channel-nicks|rcirc-channel-p|rcirc-check-auth-status\n\t|rcirc-clean-up-buffer|rcirc-clear-activity|rcirc-clear-unread|rcirc-cmd-bright|rcirc-cmd-ctcp|rcirc-cmd-dim|rcirc-cmd-ignore|rcirc-cmd-invite\n\t|rcirc-cmd-join|rcirc-cmd-keyword|rcirc-cmd-kick|rcirc-cmd-list|rcirc-cmd-me|rcirc-cmd-mode|rcirc-cmd-msg|rcirc-cmd-names|rcirc-cmd-nick\n\t|rcirc-cmd-oper|rcirc-cmd-part|rcirc-cmd-query|rcirc-cmd-quit|rcirc-cmd-quote|rcirc-cmd-reconnect|rcirc-cmd-topic|rcirc-cmd-whois\n\t|rcirc-complete|rcirc-completion-at-point|rcirc-condition-filter|rcirc-connect|rcirc-ctcp-sender-PING|rcirc-debug|rcirc-delete-process\n\t|rcirc-disconnect-buffer|rcirc-edit-multiline|rcirc-elapsed-lines|rcirc-facify|rcirc-fill-paragraph|rcirc-filter|rcirc-float-time\n\t|rcirc-format-response-string|rcirc-generate-log-filename|rcirc-generate-new-buffer-name|rcirc-get-buffer-create|rcirc-get-buffer\n\t|rcirc-get-temp-buffer-create|rcirc-handler-001|rcirc-handler-301|rcirc-handler-317|rcirc-handler-332|rcirc-handler-333|rcirc-handler-353\n\t|rcirc-handler-366|rcirc-handler-433|rcirc-handler-477|rcirc-handler-CTCP-response|rcirc-handler-CTCP|rcirc-handler-ERROR|rcirc-handler-INVITE\n\t|rcirc-handler-JOIN|rcirc-handler-KICK|rcirc-handler-MODE|rcirc-handler-NICK|rcirc-handler-NOTICE|rcirc-handler-PART-or-KICK|rcirc-handler-PART\n\t|rcirc-handler-PING|rcirc-handler-PONG|rcirc-handler-PRIVMSG|rcirc-handler-QUIT|rcirc-handler-TOPIC|rcirc-handler-WALLOPS|rcirc-handler-ctcp-ACTION\n\t|rcirc-handler-ctcp-KEEPALIVE|rcirc-handler-ctcp-TIME|rcirc-handler-ctcp-VERSION|rcirc-handler-generic|rcirc-ignore-update-automatic\n\t|rcirc-insert-next-input|rcirc-insert-prev-input|rcirc-join-channels-post-auth|rcirc-join-channels|rcirc-jump-to-first-unread-line\n\t|rcirc-keepalive|rcirc-kill-buffer-hook|rcirc-last-line|rcirc-last-quit-line|rcirc-log-write|rcirc-log|rcirc-looking-at-input|rcirc-make-trees\n\t|rcirc-markup-attributes|rcirc-markup-bright-nicks|rcirc-markup-fill|rcirc-markup-keywords|rcirc-markup-my-nick|rcirc-markup-timestamp\n\t|rcirc-markup-urls|rcirc-maybe-remember-nick-quit|rcirc-mode|rcirc-multiline-minor-cancel|rcirc-multiline-minor-mode|rcirc-multiline-minor-submit\n\t|rcirc-next-active-buffer|rcirc-nick-channels|rcirc-nick-remove|rcirc-nick|rcirc-nickname<|rcirc-non-irc-buffer|rcirc-omit-mode\n\t|rcirc-prev-input-string|rcirc-print|rcirc-process-command|rcirc-process-input-line|rcirc-process-list|rcirc-process-message|rcirc-process-server-response-1\n\t|rcirc-process-server-response|rcirc-prompt-for-encryption|rcirc-put-nick-channel|rcirc-rebuild-tree|rcirc-record-activity|rcirc-remove-nick-channel\n\t|rcirc-reschedule-timeout|rcirc-send-ctcp|rcirc-send-input|rcirc-send-message|rcirc-send-privmsg|rcirc-send-string|rcirc-sentinel\n\t|rcirc-server-name|rcirc-set-changed|rcirc-short-buffer-name|rcirc-sort-nicknames-join|rcirc-split-activity|rcirc-split-message\n\t|rcirc-switch-to-server-buffer|rcirc-target-buffer|rcirc-toggle-ignore-buffer-activity|rcirc-toggle-low-priority|rcirc-track-minor-mode\n\t|rcirc-update-activity-string|rcirc-update-prompt|rcirc-update-short-buffer-names|rcirc-user-nick|rcirc-view-log-file|rcirc-visible-buffers\n\t|rcirc-window-configuration-change-1|rcirc-window-configuration-change|rcirc|re-builder-unload-function|re-search-backward-lax-whitespace\n\t|re-search-forward-lax-whitespace|read--expression|read-abbrev-file|read-all-face-attributes|read-buffer-file-coding-system|read-buffer-to-switch\n\t|read-char-by-name|read-charset|read-cookie|read-envvar-name|read-extended-command|read-face-and-attribute|read-face-attribute|read-face-font\n\t|read-face-name|read-feature|read-file-name--defaults|read-file-name-default|read-file-name-internal|read-from-whole-string|read-hiragana-string\n\t|read-input|read-language-name|read-multilingual-string|read-number|read-regexp-suggestions|reb-assert-buffer-in-window|reb-auto-update\n\t|reb-change-syntax|reb-change-target-buffer|reb-color-display-p|reb-cook-regexp|reb-copy|reb-count-subexps|reb-delete-overlays|reb-display-subexp\n\t|reb-do-update|reb-empty-regexp|reb-enter-subexp-mode|reb-force-update|reb-initialize-buffer|reb-insert-regexp|reb-kill-buffer|reb-lisp-mode\n\t|reb-lisp-syntax-p|reb-mode-buffer-p|reb-mode-common|reb-mode|reb-next-match|reb-prev-match|reb-quit-subexp-mode|reb-quit|reb-read-regexp\n\t|reb-show-subexp|reb-target-binding|reb-toggle-case|reb-update-modestring|reb-update-overlays|reb-update-regexp|rebuild-mail-abbrevs\n\t|recentf-add-file|recentf-apply-filename-handlers|recentf-apply-menu-filter|recentf-arrange-by-dir|recentf-arrange-by-mode|recentf-arrange-by-rule\n\t|recentf-auto-cleanup|recentf-build-mode-rules|recentf-cancel-dialog|recentf-cleanup|recentf-dialog-goto-first|recentf-dialog-mode\n\t|recentf-dialog|recentf-digit-shortcut-command-name|recentf-dir-rule|recentf-directory-compare|recentf-dump-variable|recentf-edit-list-select\n\t|recentf-edit-list-validate|recentf-edit-list|recentf-elements|recentf-enabled-p|recentf-expand-file-name|recentf-file-name-nondir\n\t|recentf-filter-changer-select|recentf-filter-changer|recentf-hide-menu|recentf-include-p|recentf-indirect-mode-rule|recentf-keep-default-predicate\n\t|recentf-keep-p|recentf-load-list|recentf-make-default-menu-element|recentf-make-menu-element|recentf-make-menu-item|recentf-make-menu-items\n\t|recentf-match-rule|recentf-menu-bar|recentf-menu-customization-changed|recentf-menu-element-item|recentf-menu-element-value|recentf-menu-elements\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t rmail-output-body-to-file|rmail-output-to-rmail-buffer|rmail-output|rmail-parse-url|rmail-perm-variables|rmail-pop-to-buffer|rmail-previous-labeled-message\n\t|rmail-previous-message|rmail-previous-same-subject|rmail-previous-undeleted-message|rmail-probe|rmail-quit|rmail-read-label|rmail-redecode-body\n\t|rmail-reply|rmail-require-mime-maybe|rmail-resend|rmail-restore-desktop-buffer|rmail-retry-failure|rmail-revert|rmail-search-backwards\n\t|rmail-search-message|rmail-search|rmail-select-summary|rmail-set-attribute-1|rmail-set-attribute|rmail-set-header-1|rmail-set-header\n\t|rmail-set-message-counters-counter|rmail-set-message-counters|rmail-set-message-deleted-p|rmail-set-remote-password|rmail-show-message-1\n\t|rmail-show-message|rmail-simplified-subject-regexp|rmail-simplified-subject|rmail-sort-by-author|rmail-sort-by-correspondent\n\t|rmail-sort-by-date|rmail-sort-by-labels|rmail-sort-by-lines|rmail-sort-by-recipient|rmail-sort-by-subject|rmail-speedbar-button\n\t|rmail-speedbar-buttons|rmail-speedbar-find-file|rmail-speedbar-move-message-to-folder-on-line|rmail-speedbar-move-message|rmail-start-mail\n\t|rmail-summary-by-labels|rmail-summary-by-recipients|rmail-summary-by-regexp|rmail-summary-by-senders|rmail-summary-by-topic|rmail-summary-displayed\n\t|rmail-summary-exists|rmail-summary|rmail-swap-buffers-maybe|rmail-swap-buffers|rmail-toggle-header|rmail-undelete-previous-message\n\t|rmail-unfontify-buffer-function|rmail-unknown-mail-followup-to|rmail-unrmail-new-mail-maybe|rmail-unrmail-new-mail|rmail-update-summary\n\t|rmail-variables|rmail-view-buffer-kill-buffer-hook|rmail-what-message|rmail-widen-to-current-msgbeg|rmail-widen|rmail-write-region-annotate\n\t|rmail-yank-current-message|rmail|rng-c-load-schema|rng-nxml-mode-init|rng-validate-mode|rng-xsd-compile|robin-define-package|robin-modify-package\n\t|robin-use-package|rot13-other-window|rot13-region|rot13-string|rot13|rotate-yank-pointer|rotatef|round\\*|route|rsh|rst-minor-mode\n\t|rst-mode|ruby--at-indentation-p|ruby--detect-encoding|ruby--electric-indent-p|ruby--encoding-comment-required-p|ruby--insert-coding-comment\n\t|ruby--inverse-string-quote|ruby--string-region|ruby-accurate-end-of-block|ruby-add-log-current-method|ruby-backward-sexp|ruby-beginning-of-block\n\t|ruby-beginning-of-defun|ruby-beginning-of-indent|ruby-block-contains-point|ruby-brace-to-do-end|ruby-calculate-indent|ruby-current-indentation\n\t|ruby-deep-indent-paren-p|ruby-do-end-to-brace|ruby-end-of-block|ruby-end-of-defun|ruby-expr-beg|ruby-forward-sexp|ruby-forward-string\n\t|ruby-here-doc-end-match|ruby-imenu-create-index-in-block|ruby-imenu-create-index|ruby-in-ppss-context-p|ruby-indent-exp|ruby-indent-line\n\t|ruby-indent-size|ruby-indent-to|ruby-match-expression-expansion|ruby-mode-menu|ruby-mode-set-encoding|ruby-mode-variables|ruby-mode\n\t|ruby-move-to-block|ruby-parse-partial|ruby-parse-region|ruby-singleton-class-p|ruby-smie--args-separator-p|ruby-smie--at-dot-call\n\t|ruby-smie--backward-token|ruby-smie--bosp|ruby-smie--closing-pipe-p|ruby-smie--forward-token|ruby-smie--implicit-semi-p|ruby-smie--indent-to-stmt-p\n\t|ruby-smie--indent-to-stmt|ruby-smie--opening-pipe-p|ruby-smie--redundant-do-p|ruby-smie-rules|ruby-special-char-p|ruby-string-at-point-p\n\t|ruby-syntax-enclosing-percent-literal|ruby-syntax-expansion-allowed-p|ruby-syntax-propertize-expansion|ruby-syntax-propertize-expansions\n\t|ruby-syntax-propertize-function|ruby-syntax-propertize-heredoc|ruby-syntax-propertize-percent-literal|ruby-toggle-block|ruby-toggle-string-quotes\n\t|ruler--save-header-line-format|ruler-mode-character-validate|ruler-mode-full-window-width|ruler-mode-mouse-add-tab-stop|ruler-mode-mouse-del-tab-stop\n\t|ruler-mode-mouse-drag-any-column-iteration|ruler-mode-mouse-drag-any-column|ruler-mode-mouse-grab-any-column|ruler-mode-mouse-set-left-margin\n\t|ruler-mode-mouse-set-right-margin|ruler-mode-ruler|ruler-mode-space|ruler-mode-toggle-show-tab-stops|ruler-mode-window-col|ruler-mode\n\t|run-dig|run-hook-wrapped|run-lisp|run-network-program|run-octave|run-prolog|run-python-internal|run-python|run-scheme|run-tcl|run-window-configuration-change-hook\n\t|run-window-scroll-functions|run-with-timer|rx-\\*\\*|rx-=|rx->=|rx-and|rx-any-condense-range|rx-any-delete-from-range|rx-any|rx-anything\n\t|rx-atomic-p|rx-backref|rx-category|rx-check-any-string|rx-check-any|rx-check-backref|rx-check-category|rx-check-not|rx-check|rx-eval\n\t|rx-form|rx-greedy|rx-group-if|rx-info|rx-kleene|rx-not-char|rx-not-syntax|rx-not|rx-or|rx-regexp|rx-repeat|rx-submatch-n|rx-submatch\n\t|rx-syntax|rx-to-string|rx-trans-forms|rx|rzgrep|safe-date-to-time|same-class-fast-p|same-class-p|sanitize-coding-system-list|sasl-anonymous-response\n\t|sasl-client-mechanism|sasl-client-name|sasl-client-properties|sasl-client-property|sasl-client-server|sasl-client-service|sasl-client-set-properties\n\t|sasl-client-set-property|sasl-error|sasl-find-mechanism|sasl-login-response-1|sasl-login-response-2|sasl-make-client|sasl-make-mechanism\n\t|sasl-mechanism-name|sasl-mechanism-steps|sasl-next-step|sasl-plain-response|sasl-read-passphrase|sasl-step-data|sasl-step-set-data\n\t|sasl-unique-id-function|sasl-unique-id-number-base36|sasl-unique-id|save-buffers-kill-emacs|save-buffers-kill-terminal|save-completions-to-file\n\t|save-place-alist-to-file|save-place-dired-hook|save-place-find-file-hook|save-place-forget-unreadable-files|save-place-kill-emacs-hook\n\t|save-place-to-alist|save-places-to-alist|savehist-autosave|savehist-install|savehist-load|savehist-minibuffer-hook|savehist-mode\n\t|savehist-printable|savehist-save|savehist-trim-history|savehist-uninstall|sc-S-cite-region-limit|sc-S-mail-header-nuke-list|sc-S-mail-nuke-mail-headers\n\t|sc-S-preferred-attribution-list|sc-S-preferred-header-style|sc-T-auto-fill-region|sc-T-confirm-always|sc-T-describe|sc-T-downcase\n\t|sc-T-electric-circular|sc-T-electric-references|sc-T-fixup-whitespace|sc-T-mail-nuke-blank-lines|sc-T-nested-citation|sc-T-use-only-preferences\n\t|sc-add-citation-level|sc-ask|sc-attribs-!-addresses|sc-attribs-%@-addresses|sc-attribs-<>-addresses|sc-attribs-chop-address|sc-attribs-chop-namestring\n\t|sc-attribs-emailname|sc-attribs-extract-namestring|sc-attribs-filter-namelist|sc-attribs-strip-initials|sc-cite-coerce-cited-line\n\t|sc-cite-coerce-dumb-citer|sc-cite-line|sc-cite-original|sc-cite-regexp|sc-cite-region|sc-describe|sc-electric-mode|sc-eref-abort\n\t|sc-eref-exit|sc-eref-goto|sc-eref-insert-selected|sc-eref-jump|sc-eref-next|sc-eref-prev|sc-eref-setn|sc-eref-show|sc-fill-if-different\n\t|sc-get-address|sc-guess-attribution|sc-guess-nesting|sc-hdr|sc-header-attributed-writes|sc-header-author-writes|sc-header-inarticle-writes\n\t|sc-header-on-said|sc-header-regarding-adds|sc-header-verbose|sc-insert-citation|sc-insert-reference|sc-mail-append-field|sc-mail-build-nuke-frame\n\t|sc-mail-check-from|sc-mail-cleanup-blank-lines|sc-mail-error-in-mail-field|sc-mail-fetch-field|sc-mail-field-query|sc-mail-field\n\t|sc-mail-nuke-continuation-line|sc-mail-nuke-header-line|sc-mail-nuke-line|sc-mail-process-headers|sc-make-citation|sc-minor-mode\n\t|sc-name-substring|sc-no-blank-line-or-header|sc-no-header|sc-open-line|sc-raw-mode-toggle|sc-recite-line|sc-recite-region|sc-scan-info-alist\n\t|sc-select-attribution|sc-set-variable|sc-setup-filladapt|sc-setvar-symbol|sc-toggle-fn|sc-toggle-symbol|sc-toggle-var|sc-uncite-line\n\t|sc-uncite-region|sc-valid-index-p|sc-whofrom|scan-buf-move-to-region|scan-buf-next-region|scan-buf-previous-region|scheme-compile-definition-and-go\n\t|scheme-compile-definition|scheme-compile-file|scheme-compile-region-and-go|scheme-compile-region|scheme-debugger-mode-commands\n\t|scheme-debugger-mode-initialize|scheme-debugger-mode|scheme-debugger-self-insert|scheme-expand-current-form|scheme-form-at-point\n\t|scheme-get-old-input|scheme-get-process|scheme-indent-function|scheme-input-filter|scheme-interaction-mode-commands|scheme-interaction-mode-initialize\n\t|scheme-interaction-mode|scheme-interactively-start-process|scheme-let-indent|scheme-load-file|scheme-mode-commands|scheme-mode-variables\n\t|scheme-mode|scheme-proc|scheme-send-definition-and-go|scheme-send-definition|scheme-send-last-sexp|scheme-send-region-and-go|scheme-send-region\n\t|scheme-start-file|scheme-syntax-propertize-sexp-comment|scheme-syntax-propertize|scheme-trace-procedure|scroll-all-beginning-of-buffer-all\n\t|scroll-all-check-to-scroll|scroll-all-end-of-buffer-all|scroll-all-function-all|scroll-all-mode|scroll-all-page-down-all|scroll-all-page-up-all\n\t|scroll-all-scroll-down-all|scroll-all-scroll-up-all|scroll-bar-columns|scroll-bar-drag-1|scroll-bar-drag-position|scroll-bar-drag\n\t|scroll-bar-horizontal-drag-1|scroll-bar-horizontal-drag|scroll-bar-lines|scroll-bar-maybe-set-window-start|scroll-bar-scroll-down\n\t|scroll-bar-scroll-up|scroll-bar-set-window-start|scroll-bar-toolkit-horizontal-scroll|scroll-bar-toolkit-scroll|scroll-down-line\n\t|scroll-lock-mode|scroll-other-window-down|scroll-up-line|scss-mode|scss-smie--not-interpolation-p|sdb|search-backward-lax-whitespace\n\t|search-backward-regexp|search-emacs-glossary|search-forward-lax-whitespace|search-forward-regexp|search-pages|search-unencodable-char\n\t|search|second|seconds-to-string|secrets-close-session|secrets-collection-handler|secrets-collection-path|secrets-create-collection\n\t|secrets-create-item|secrets-delete-alias|secrets-delete-collection|secrets-delete-item|secrets-empty-path|secrets-expand-collection\n\t|secrets-expand-item|secrets-get-alias|secrets-get-attribute|secrets-get-attributes|secrets-get-collection-properties|secrets-get-collection-property\n\t|secrets-get-collections|secrets-get-item-properties|secrets-get-item-property|secrets-get-items|secrets-get-secret|secrets-item-path\n\t|secrets-list-collections|secrets-list-items|secrets-mode|secrets-open-session|secrets-prompt-handler|secrets-prompt|secrets-search-items\n\t|secrets-set-alias|secrets-show-collections|secrets-show-secrets|secrets-tree-widget-after-toggle-function|secrets-tree-widget-show-password\n\t|secrets-unlock-collection|secure-hash|select-frame-by-name|select-frame-set-input-focus|select-frame|select-message-coding-system\n\t|select-safe-coding-system-interactively|select-safe-coding-system|select-scheme|select-tags-table-mode|select-tags-table-quit\n\t|select-tags-table-select|select-tags-table|select-window|selected-frame|selected-window|self-insert-and-exit|self-insert-command\n\t|semantic--set-buffer-cache|semantic--tag-attributes-cdr|semantic--tag-copy-properties|semantic--tag-deep-copy-attributes|semantic--tag-deep-copy-tag-list\n\t|semantic--tag-deep-copy-value|semantic--tag-expand|semantic--tag-expanded-p|semantic--tag-find-parent-by-name|semantic--tag-get-property\n\t|semantic--tag-link-cache-to-buffer|semantic--tag-link-list-to-buffer|semantic--tag-link-to-buffer|semantic--tag-overlay-cdr\n\t|semantic--tag-properties-cdr|semantic--tag-put-property-no-side-effect|semantic--tag-put-property|semantic--tag-run-hooks|semantic--tag-set-overlay\n\t|semantic--tag-unlink-cache-from-buffer|semantic--tag-unlink-from-buffer|semantic--tag-unlink-list-from-buffer|semantic--umatched-syntax-needs-refresh-p\n\t|semantic-active-p|semantic-add-label|semantic-add-minor-mode|semantic-add-system-include|semantic-alias-obsolete|semantic-analyze-completion-at-point-function\n\t|semantic-analyze-current-context|semantic-analyze-current-tag|semantic-analyze-nolongprefix-completion-at-point-function|semantic-analyze-notc-completion-at-point-function\n\t|semantic-analyze-possible-completions|semantic-analyze-proto-impl-toggle|semantic-analyze-type-constants|semantic-assert-valid-token\n\t|semantic-bovinate-from-nonterminal-full|semantic-bovinate-from-nonterminal|semantic-bovinate-region-until-error|semantic-bovinate-stream\n\t|semantic-bovinate-toplevel|semantic-buffer-local-value|semantic-c-add-preprocessor-symbol|semantic-cache-data-post-command-hook\n\t|semantic-cache-data-to-buffer|semantic-calculate-scope|semantic-change-function|semantic-clean-token-of-unmatched-syntax|semantic-clean-unmatched-syntax-in-buffer\n\t|semantic-clean-unmatched-syntax-in-region|semantic-clear-parser-warnings|semantic-clear-toplevel-cache|semantic-clear-unmatched-syntax-cache\n\t|semantic-comment-lexer|semantic-complete-analyze-and-replace|semantic-complete-analyze-inline-idle|semantic-complete-analyze-inline\n\t|semantic-complete-inline-project|semantic-complete-jump-local-members|semantic-complete-jump-local|semantic-complete-jump|semantic-complete-self-insert\n\t|semantic-complete-symbol|semantic-create-imenu-index|semantic-create-tag-proxy|semantic-ctxt-current-mode|semantic-current-tag-parent\n\t|semantic-current-tag|semantic-customize-system-include-path|semantic-debug|semantic-decoration-include-visit|semantic-decoration-unparsed-include-do-reset\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t semantic-default-c-setup|semantic-default-elisp-setup|semantic-default-html-setup|semantic-default-make-setup|semantic-default-scheme-setup\n\t|semantic-default-texi-setup|semantic-delete-overlay-maybe|semantic-dependency-tag-file|semantic-describe-buffer-var-helper|semantic-describe-buffer\n\t|semantic-describe-tag|semantic-desktop-ignore-this-minor-mode|semantic-documentation-for-tag|semantic-dump-parser-warnings|semantic-edits-incremental-parser\n\t|semantic-elapsed-time|semantic-equivalent-tag-p|semantic-error-if-unparsed|semantic-event-window|semantic-exit-on-input|semantic-fetch-available-tags\n\t|semantic-fetch-tags-fast|semantic-fetch-tags|semantic-file-tag-table|semantic-file-token-stream|semantic-find-file-noselect|semantic-find-first-tag-by-name\n\t|semantic-find-tag-by-overlay-in-region|semantic-find-tag-by-overlay-next|semantic-find-tag-by-overlay-prev|semantic-find-tag-by-overlay\n\t|semantic-find-tag-for-completion|semantic-find-tag-parent-by-overlay|semantic-find-tags-by-scope-protection|semantic-find-tags-included\n\t|semantic-flatten-tags-table|semantic-flex-buffer|semantic-flex-end|semantic-flex-keyword-get|semantic-flex-keyword-p|semantic-flex-keyword-put\n\t|semantic-flex-keywords|semantic-flex-list|semantic-flex-make-keyword-table|semantic-flex-map-keywords|semantic-flex-start|semantic-flex-text\n\t|semantic-flex|semantic-force-refresh|semantic-foreign-tag-check|semantic-foreign-tag-invalid|semantic-foreign-tag-p|semantic-foreign-tag\n\t|semantic-format-tag-concise-prototype|semantic-format-tag-name|semantic-format-tag-prototype|semantic-format-tag-summarize|semantic-fw-add-edebug-spec\n\t|semantic-gcc-setup|semantic-get-cache-data|semantic-go-to-tag|semantic-highlight-edits-mode|semantic-highlight-edits-new-change-hook-fcn\n\t|semantic-highlight-func-highlight-current-tag|semantic-highlight-func-menu|semantic-highlight-func-mode|semantic-highlight-func-popup-menu\n\t|semantic-ia-complete-symbol-menu|semantic-ia-complete-symbol|semantic-ia-complete-tip|semantic-ia-describe-class|semantic-ia-fast-jump\n\t|semantic-ia-fast-mouse-jump|semantic-ia-show-doc|semantic-ia-show-summary|semantic-ia-show-variants|semantic-idle-completions-mode\n\t|semantic-idle-scheduler-mode|semantic-idle-summary-mode|semantic-insert-foreign-tag-change-log-mode|semantic-insert-foreign-tag-default\n\t|semantic-insert-foreign-tag-log-edit-mode|semantic-insert-foreign-tag|semantic-install-function-overrides|semantic-lex-beginning-of-line\n\t|semantic-lex-buffer|semantic-lex-catch-errors|semantic-lex-charquote|semantic-lex-close-paren|semantic-lex-comments-as-whitespace\n\t|semantic-lex-comments|semantic-lex-debug-break|semantic-lex-debug|semantic-lex-default-action|semantic-lex-end-block|semantic-lex-expand-block-specs\n\t|semantic-lex-highlight-token|semantic-lex-ignore-comments|semantic-lex-ignore-newline|semantic-lex-ignore-whitespace|semantic-lex-init\n\t|semantic-lex-keyword-get|semantic-lex-keyword-invalid|semantic-lex-keyword-p|semantic-lex-keyword-put|semantic-lex-keyword-set\n\t|semantic-lex-keyword-symbol|semantic-lex-keyword-value|semantic-lex-keywords|semantic-lex-list|semantic-lex-make-keyword-table\n\t|semantic-lex-make-type-table|semantic-lex-map-keywords|semantic-lex-map-symbols|semantic-lex-map-types|semantic-lex-newline-as-whitespace\n\t|semantic-lex-newline|semantic-lex-number|semantic-lex-one-token|semantic-lex-open-paren|semantic-lex-paren-or-list|semantic-lex-preset-default-types\n\t|semantic-lex-punctuation-type|semantic-lex-punctuation|semantic-lex-push-token|semantic-lex-spp-table-write-slot-value|semantic-lex-start-block\n\t|semantic-lex-string|semantic-lex-symbol-or-keyword|semantic-lex-test|semantic-lex-token-bounds|semantic-lex-token-class|semantic-lex-token-end\n\t|semantic-lex-token-p|semantic-lex-token-start|semantic-lex-token-text|semantic-lex-token-with-text-p|semantic-lex-token-without-text-p\n\t|semantic-lex-token|semantic-lex-type-get|semantic-lex-type-invalid|semantic-lex-type-p|semantic-lex-type-put|semantic-lex-type-set\n\t|semantic-lex-type-symbol|semantic-lex-type-value|semantic-lex-types|semantic-lex-unterminated-syntax-detected|semantic-lex-unterminated-syntax-protection\n\t|semantic-lex-whitespace|semantic-lex|semantic-make-local-hook|semantic-make-overlay|semantic-map-buffers|semantic-map-mode-buffers\n\t|semantic-menu-item|semantic-mode-line-update|semantic-mode|semantic-narrow-to-tag|semantic-new-buffer-fcn|semantic-next-unmatched-syntax\n\t|semantic-obtain-foreign-tag|semantic-overlay-buffer|semantic-overlay-delete|semantic-overlay-end|semantic-overlay-get|semantic-overlay-lists\n\t|semantic-overlay-live-p|semantic-overlay-move|semantic-overlay-next-change|semantic-overlay-p|semantic-overlay-previous-change\n\t|semantic-overlay-properties|semantic-overlay-put|semantic-overlay-start|semantic-overlays-at|semantic-overlays-in|semantic-overload-symbol-from-function\n\t|semantic-parse-changes-default|semantic-parse-changes|semantic-parse-region-default|semantic-parse-region|semantic-parse-stream-default\n\t|semantic-parse-stream|semantic-parse-tree-needs-rebuild-p|semantic-parse-tree-needs-update-p|semantic-parse-tree-set-needs-rebuild\n\t|semantic-parse-tree-set-needs-update|semantic-parse-tree-set-up-to-date|semantic-parse-tree-unparseable-p|semantic-parse-tree-unparseable\n\t|semantic-parse-tree-up-to-date-p|semantic-parser-working-message|semantic-popup-menu|semantic-push-parser-warning|semantic-read-event\n\t|semantic-read-function|semantic-read-symbol|semantic-read-type|semantic-read-variable|semantic-refresh-tags-safe|semantic-remove-system-include\n\t|semantic-repeat-parse-whole-stream|semantic-require-version|semantic-reset-system-include|semantic-run-mode-hooks|semantic-safe\n\t|semantic-sanity-check|semantic-set-unmatched-syntax-cache|semantic-show-label|semantic-show-parser-state-auto-marker|semantic-show-parser-state-marker\n\t|semantic-show-parser-state-mode|semantic-show-unmatched-lex-tokens-fetch|semantic-show-unmatched-syntax-mode|semantic-show-unmatched-syntax-next\n\t|semantic-show-unmatched-syntax|semantic-showing-unmatched-syntax-p|semantic-simple-lexer|semantic-something-to-stream|semantic-something-to-tag-table\n\t|semantic-speedbar-analysis|semantic-stickyfunc-fetch-stickyline|semantic-stickyfunc-menu|semantic-stickyfunc-mode|semantic-stickyfunc-popup-menu\n\t|semantic-stickyfunc-tag-to-stick|semantic-subst-char-in-string|semantic-symref-find-file-references-by-name|semantic-symref-find-references-by-name\n\t|semantic-symref-find-tags-by-completion|semantic-symref-find-tags-by-name|semantic-symref-find-tags-by-regexp|semantic-symref-find-text\n\t|semantic-symref-regexp|semantic-symref-symbol|semantic-symref-tool-cscope-child-p|semantic-symref-tool-cscope-list-p|semantic-symref-tool-cscope-p\n\t|semantic-symref-tool-cscope|semantic-symref-tool-global-child-p|semantic-symref-tool-global-list-p|semantic-symref-tool-global-p\n\t|semantic-symref-tool-global|semantic-symref-tool-grep-child-p|semantic-symref-tool-grep-list-p|semantic-symref-tool-grep-p|semantic-symref-tool-grep\n\t|semantic-symref-tool-idutils-child-p|semantic-symref-tool-idutils-list-p|semantic-symref-tool-idutils-p|semantic-symref-tool-idutils\n\t|semantic-symref|semantic-tag-add-hook|semantic-tag-alias-class|semantic-tag-alias-definition|semantic-tag-attributes|semantic-tag-bounds\n\t|semantic-tag-buffer|semantic-tag-children-compatibility|semantic-tag-class|semantic-tag-clone|semantic-tag-code-detail|semantic-tag-components-default\n\t|semantic-tag-components-with-overlays-default|semantic-tag-components-with-overlays|semantic-tag-components|semantic-tag-copy\n\t|semantic-tag-deep-copy-one-tag|semantic-tag-docstring|semantic-tag-end|semantic-tag-external-member-parent|semantic-tag-faux-p\n\t|semantic-tag-file-name|semantic-tag-function-arguments|semantic-tag-function-constructor-p|semantic-tag-function-destructor-p\n\t|semantic-tag-function-parent|semantic-tag-function-throws|semantic-tag-get-attribute|semantic-tag-in-buffer-p|semantic-tag-include-filename-default\n\t|semantic-tag-include-filename|semantic-tag-include-system-p|semantic-tag-make-assoc-list|semantic-tag-make-plist|semantic-tag-mode\n\t|semantic-tag-modifiers|semantic-tag-name|semantic-tag-named-parent|semantic-tag-new-alias|semantic-tag-new-code|semantic-tag-new-function\n\t|semantic-tag-new-include|semantic-tag-new-package|semantic-tag-new-type|semantic-tag-new-variable|semantic-tag-of-class-p|semantic-tag-of-type-p\n\t|semantic-tag-overlay|semantic-tag-p|semantic-tag-properties|semantic-tag-prototype-p|semantic-tag-put-attribute-no-side-effect\n\t|semantic-tag-put-attribute|semantic-tag-remove-hook|semantic-tag-resolve-proxy|semantic-tag-set-bounds|semantic-tag-set-faux\n\t|semantic-tag-set-name|semantic-tag-set-proxy|semantic-tag-similar-with-subtags-p|semantic-tag-start|semantic-tag-type-compound-p\n\t|semantic-tag-type-interfaces|semantic-tag-type-members|semantic-tag-type-superclass-protection|semantic-tag-type-superclasses\n\t|semantic-tag-type|semantic-tag-variable-constant-p|semantic-tag-variable-default|semantic-tag-with-position-p|semantic-tag-write-list-slot-value\n\t|semantic-tag|semantic-test-data-cache|semantic-throw-on-input|semantic-toggle-minor-mode-globally|semantic-token-type-parent\n\t|semantic-unmatched-syntax-overlay-p|semantic-unmatched-syntax-tokens|semantic-varalias-obsolete|semantic-with-buffer-narrowed-to-current-tag\n\t|semantic-with-buffer-narrowed-to-tag|semanticdb-database-typecache-child-p|semanticdb-database-typecache-list-p|semanticdb-database-typecache-p\n\t|semanticdb-database-typecache|semanticdb-enable-gnu-global-databases|semanticdb-file-table-object|semanticdb-find-adebug-lost-includes\n\t|semanticdb-find-result-length|semanticdb-find-result-nth-in-buffer|semanticdb-find-result-nth|semanticdb-find-table-for-include\n\t|semanticdb-find-tags-by-class|semanticdb-find-tags-by-name-regexp|semanticdb-find-tags-by-name|semanticdb-find-tags-for-completion\n\t|semanticdb-find-test-translate-path|semanticdb-find-translate-path|semanticdb-minor-mode-p|semanticdb-project-database-file-child-p\n\t|semanticdb-project-database-file-list-p|semanticdb-project-database-file-p|semanticdb-project-database-file|semanticdb-strip-find-results\n\t|semanticdb-typecache-child-p|semanticdb-typecache-find|semanticdb-typecache-list-p|semanticdb-typecache-p|semanticdb-typecache\n\t|semanticdb-without-unloaded-file-searches|senator-copy-tag-to-register|senator-copy-tag|senator-go-to-up-reference|senator-kill-tag\n\t|senator-next-tag|senator-previous-tag|senator-transpose-tags-down|senator-transpose-tags-up|senator-yank-tag|send-invisible|send-process-next-char\n\t|send-region|send-string|sendmail-query-once|sendmail-query-user-about-smtp|sendmail-send-it|sendmail-sync-aliases|sendmail-user-agent-compose\n\t|sentence-at-point|seq--count-successive|seq--drop-list|seq--drop-while-list|seq--take-list|seq--take-while-list|seq-concatenate\n\t|seq-contains-p|seq-copy|seq-count|seq-do|seq-doseq|seq-drop-while|seq-drop|seq-each|seq-elt|seq-empty-p|seq-every-p|seq-filter|seq-length\n\t|seq-map|seq-reduce|seq-remove|seq-reverse|seq-some-p|seq-sort|seq-subseq|seq-take-while|seq-take|seq-uniq|serial-mode-line-config-menu-1\n\t|serial-mode-line-config-menu|serial-mode-line-speed-menu-1|serial-mode-line-speed-menu|serial-nice-speed-history|serial-port-is-file-p\n\t|serial-read-name|serial-read-speed|serial-speed|serial-supported-or-barf|serial-update-config-menu|serial-update-speed-menu|server--on-display-p\n\t|server-add-client|server-buffer-done|server-clients-with|server-create-tty-frame|server-create-window-system-frame|server-delete-client\n\t|server-done|server-edit|server-ensure-safe-dir|server-eval-and-print|server-eval-at|server-execute-continuation|server-execute\n\t|server-force-delete|server-force-stop|server-generate-key|server-get-auth-key|server-goto-line-column|server-goto-toplevel|server-handle-delete-frame\n\t|server-handle-suspend-tty|server-kill-buffer|server-kill-emacs-query-function|server-log|server-mode|server-process-filter|server-quote-arg\n\t|server-reply-print|server-return-error|server-running-p|server-save-buffers-kill-terminal|server-select-display|server-send-string\n\t|server-sentinel|server-start|server-switch-buffer|server-temp-file-p|server-unload-function|server-unquote-arg|server-unselect-display\n\t|server-visit-files|server-with-environment|ses\\+|ses--advice-copy-region-as-kill|ses--advice-yank|ses--cell|ses--clean-!|ses--clean-_\n\t|ses--letref|ses--local-printer|ses--locprn-compiled--cmacro|ses--locprn-compiled|ses--locprn-def--cmacro|ses--locprn-def|ses--locprn-local-printer-list--cmacro\n\t|ses--locprn-local-printer-list|ses--locprn-number--cmacro|ses--locprn-number|ses--locprn-p--cmacro|ses--locprn-p|ses--metaprogramming\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t ses--time-check|ses-adjust-print-width|ses-append-row-jump-first-column|ses-aset-with-undo|ses-average|ses-begin-change|ses-calculate-cell\n\t|ses-call-printer|ses-cell--formula--cmacro|ses-cell--formula|ses-cell--printer--cmacro|ses-cell--printer|ses-cell--properties--cmacro\n\t|ses-cell--properties|ses-cell--references--cmacro|ses-cell--references|ses-cell--symbol--cmacro|ses-cell--symbol|ses-cell-formula\n\t|ses-cell-p|ses-cell-printer|ses-cell-property-pop|ses-cell-property|ses-cell-references|ses-cell-set-formula|ses-cell-symbol|ses-cell-value\n\t|ses-center-span|ses-center|ses-check-curcell|ses-cleanup|ses-clear-cell-backward|ses-clear-cell-forward|ses-clear-cell|ses-col-printer\n\t|ses-col-width|ses-column-letter|ses-column-printers|ses-column-widths|ses-command-hook|ses-copy-region-helper|ses-copy-region|ses-create-cell-symbol\n\t|ses-create-cell-variable-range|ses-create-cell-variable|ses-create-header-string|ses-dashfill-span|ses-dashfill|ses-decode-cell-symbol\n\t|ses-default-printer|ses-define-local-printer|ses-delete-blanks|ses-delete-column|ses-delete-line|ses-delete-row|ses-destroy-cell-variable-range\n\t|ses-dorange|ses-edit-cell|ses-end-of-line|ses-export-keymap|ses-export-tab|ses-export-tsf|ses-export-tsv|ses-file-format-extend-parameter-list\n\t|ses-formula-record|ses-formula-references|ses-forward-or-insert|ses-get-cell|ses-goto-data|ses-goto-print|ses-header-line-menu\n\t|ses-header-row|ses-in-print-area|ses-initialize-Dijkstra-attempt|ses-insert-column|ses-insert-range-click|ses-insert-range|ses-insert-row\n\t|ses-insert-ses-range-click|ses-insert-ses-range|ses-is-cell-sym-p|ses-jump-safe|ses-jump|ses-kill-override|ses-load|ses-local-printer-compile\n\t|ses-make-cell--cmacro|ses-make-cell|ses-make-local-printer-info|ses-mark-column|ses-mark-row|ses-menu|ses-mode-print-map|ses-mode\n\t|ses-print-cell-new-width|ses-print-cell|ses-printer-record|ses-printer-validate|ses-range|ses-read-cell-printer|ses-read-cell|ses-read-column-printer\n\t|ses-read-default-printer|ses-read-printer|ses-read-symbol|ses-recalculate-all|ses-recalculate-cell|ses-reconstruct-all|ses-refresh-local-printer\n\t|ses-relocate-all|ses-relocate-formula|ses-relocate-range|ses-relocate-symbol|ses-rename-cell|ses-renarrow-buffer|ses-repair-cell-reference-all\n\t|ses-replace-name-in-formula|ses-reprint-all|ses-reset-header-string|ses-safe-formula|ses-safe-printer|ses-select|ses-set-cell|ses-set-column-width\n\t|ses-set-curcell|ses-set-header-row|ses-set-localvars|ses-set-parameter|ses-set-with-undo|ses-setter-with-undo|ses-setup|ses-sort-column-click\n\t|ses-sort-column|ses-sym-rowcol|ses-tildefill-span|ses-truncate-cell|ses-unload-function|ses-unsafe|ses-unset-header-row|ses-update-cells\n\t|ses-vector-delete|ses-vector-insert|ses-warn-unsafe|ses-widen|ses-write-cells|ses-yank-cells|ses-yank-one|ses-yank-pop|ses-yank-resize\n\t|ses-yank-tsf|set-allout-regexp|set-auto-mode-0|set-auto-mode-1|set-background-color|set-border-color|set-buffer-file-coding-system\n\t|set-buffer-process-coding-system|set-cdabbrev-buffer|set-charset-plist|set-clipboard-coding-system|set-cmpl-prefix-entry-head\n\t|set-cmpl-prefix-entry-tail|set-coding-priority|set-comment-column|set-completion-last-use-time|set-completion-num-uses|set-completion-string\n\t|set-cursor-color|set-default-coding-systems|set-default-font|set-default-toplevel-value|set-difference|set-display-table-and-terminal-coding-system\n\t|set-downcase-syntax|set-exclusive-or|set-face-attribute-from-resource|set-face-attributes-from-resources|set-face-background-pixmap\n\t|set-face-bold-p|set-face-doc-string|set-face-documentation|set-face-inverse-video-p|set-face-italic-p|set-face-underline-p|set-file-name-coding-system\n\t|set-fill-column|set-fill-prefix|set-font-encoding|set-foreground-color|set-frame-font|set-frame-name|set-fringe-mode-1|set-fringe-mode\n\t|set-fringe-style|set-goal-column|set-hard-newline-properties|set-input-interrupt-mode|set-input-meta-mode|set-justification-center\n\t|set-justification-full|set-justification-left|set-justification-none|set-justification-right|set-justification|set-keyboard-coding-system-internal\n\t|set-language-environment-charset|set-language-environment-coding-systems|set-language-environment-input-method|set-language-environment-nonascii-translation\n\t|set-language-environment-unibyte|set-language-environment|set-language-info-alist|set-language-info-internal|set-language-info\n\t|set-locale-environment|set-mark-command|set-mode-local-parent|set-mouse-color|set-nested-alist|set-next-selection-coding-system\n\t|set-output-flow-control|set-page-delimiter|set-process-filter-multibyte|set-process-inherit-coding-system-flag|set-process-window-size\n\t|set-quit-char|set-rcirc-decode-coding-system|set-rcirc-encode-coding-system|set-rmail-inbox-list|set-safe-terminal-coding-system-internal\n\t|set-scroll-bar-mode|set-selection-coding-system|set-selective-display|set-slot-value|set-temporary-overlay-map|set-terminal-coding-system-internal\n\t|set-time-zone-rule|set-upcase-syntax|set-variable|set-viper-state-in-major-mode|set-window-buffer-start-and-point|set-window-dot\n\t|set-window-new-normal|set-window-new-pixel|set-window-new-total|set-window-redisplay-end-trigger|set-window-text-height|set-woman-file-regexp\n\t|setenv-internal|setq-mode-local|setup-chinese-environment-map|setup-cyrillic-environment-map|setup-default-fontset|setup-ethiopic-environment-internal\n\t|setup-european-environment-map|setup-indian-environment-map|setup-japanese-environment-internal|setup-korean-environment-internal\n\t|setup-specified-language-environment|seventh|sexp-at-point|sgml-at-indentation-p|sgml-attributes|sgml-auto-attributes|sgml-beginning-of-tag\n\t|sgml-calculate-indent|sgml-close-tag|sgml-comment-indent-new-line|sgml-comment-indent|sgml-delete-tag|sgml-electric-tag-pair-before-change-function\n\t|sgml-electric-tag-pair-flush-overlays|sgml-electric-tag-pair-mode|sgml-empty-tag-p|sgml-fill-nobreak|sgml-get-context|sgml-guess-indent\n\t|sgml-html-meta-auto-coding-function|sgml-indent-line|sgml-lexical-context|sgml-looking-back-at|sgml-make-syntax-table|sgml-make-tag--cmacro\n\t|sgml-make-tag|sgml-maybe-end-tag|sgml-maybe-name-self|sgml-mode-facemenu-add-face-function|sgml-mode-flyspell-verify|sgml-mode\n\t|sgml-name-8bit-mode|sgml-name-char|sgml-name-self|sgml-namify-char|sgml-parse-dtd|sgml-parse-tag-backward|sgml-parse-tag-name|sgml-point-entered\n\t|sgml-pretty-print|sgml-quote|sgml-show-context|sgml-skip-tag-backward|sgml-skip-tag-forward|sgml-slash-matching|sgml-slash|sgml-tag-end--cmacro\n\t|sgml-tag-end|sgml-tag-help|sgml-tag-name--cmacro|sgml-tag-name|sgml-tag-p--cmacro|sgml-tag-p|sgml-tag-start--cmacro|sgml-tag-start\n\t|sgml-tag-text-p|sgml-tag-type--cmacro|sgml-tag-type|sgml-tag|sgml-tags-invisible|sgml-unclosed-tag-p|sgml-validate|sgml-value|sgml-xml-auto-coding-function\n\t|sgml-xml-guess|sh--cmd-completion-table|sh--inside-noncommand-expression|sh--maybe-here-document|sh--vars-before-point|sh-add-completer\n\t|sh-add|sh-after-hack-local-variables|sh-append-backslash|sh-append|sh-assignment|sh-backslash-region|sh-basic-indent-line|sh-beginning-of-command\n\t|sh-blink|sh-calculate-indent|sh-canonicalize-shell|sh-case|sh-cd-here|sh-check-rule|sh-completion-at-point-function|sh-current-defun-name\n\t|sh-debug|sh-delete-backslash|sh-electric-here-document-mode|sh-end-of-command|sh-execute-region|sh-feature|sh-find-prev-matching\n\t|sh-find-prev-switch|sh-font-lock-backslash-quote|sh-font-lock-keywords-1|sh-font-lock-keywords-2|sh-font-lock-keywords|sh-font-lock-open-heredoc\n\t|sh-font-lock-paren|sh-font-lock-quoted-subshell|sh-font-lock-syntactic-face-function|sh-for|sh-function|sh-get-indent-info|sh-get-indent-var-for-line\n\t|sh-get-kw|sh-get-word|sh-goto-match-for-done|sh-goto-matching-case|sh-goto-matching-if|sh-guess-basic-offset|sh-handle-after-case-label\n\t|sh-handle-prev-case-alt-end|sh-handle-prev-case|sh-handle-prev-do|sh-handle-prev-done|sh-handle-prev-else|sh-handle-prev-esac\n\t|sh-handle-prev-fi|sh-handle-prev-if|sh-handle-prev-open|sh-handle-prev-rc-case|sh-handle-prev-then|sh-handle-this-close|sh-handle-this-do\n\t|sh-handle-this-done|sh-handle-this-else|sh-handle-this-esac|sh-handle-this-fi|sh-handle-this-rc-case|sh-handle-this-then|sh-help-string-for-variable\n\t|sh-if|sh-in-comment-or-string|sh-indent-line|sh-indexed-loop|sh-is-quoted-p|sh-learn-buffer-indent|sh-learn-line-indent|sh-load-style\n\t|sh-make-vars-local|sh-mark-init|sh-mark-line|sh-maybe-here-document|sh-mkword-regexpr|sh-mode-syntax-table|sh-mode|sh-modify|sh-must-support-indent\n\t|sh-name-style|sh-prev-line|sh-prev-stmt|sh-prev-thing|sh-quoted-p|sh-read-variable|sh-remember-variable|sh-repeat|sh-reset-indent-vars-to-global-values\n\t|sh-safe-forward-sexp|sh-save-styles-to-buffer|sh-select|sh-send-line-or-region-and-step|sh-send-text|sh-set-indent|sh-set-shell\n\t|sh-set-var-value|sh-shell-initialize-variables|sh-shell-process|sh-show-indent|sh-show-shell|sh-smie--continuation-start-indent\n\t|sh-smie--default-backward-token|sh-smie--default-forward-token|sh-smie--keyword-p|sh-smie--looking-back-at-continuation-p|sh-smie--newline-semi-p\n\t|sh-smie--rc-after-special-arg-p|sh-smie--rc-newline-semi-p|sh-smie--sh-keyword-in-p|sh-smie--sh-keyword-p|sh-smie-rc-backward-token\n\t|sh-smie-rc-forward-token|sh-smie-rc-rules|sh-smie-sh-backward-token|sh-smie-sh-forward-token|sh-smie-sh-rules|sh-syntax-propertize-function\n\t|sh-syntax-propertize-here-doc|sh-this-is-a-continuation|sh-tmp-file|sh-until|sh-var-value|sh-while-getopts|sh-while|sha1|shadow-add-to-todo\n\t|shadow-cancel|shadow-cluster-name|shadow-cluster-primary|shadow-cluster-regexp|shadow-contract-file-name|shadow-copy-file|shadow-copy-files\n\t|shadow-define-cluster|shadow-define-literal-group|shadow-define-regexp-group|shadow-expand-cluster-in-file-name|shadow-expand-file-name\n\t|shadow-file-match|shadow-find|shadow-get-cluster|shadow-get-user|shadow-initialize|shadow-insert-var|shadow-invalidate-hashtable\n\t|shadow-local-file|shadow-make-cluster|shadow-make-fullname|shadow-make-group|shadow-parse-fullname|shadow-parse-name|shadow-read-files\n\t|shadow-read-site|shadow-regexp-superquote|shadow-remove-from-todo|shadow-replace-name-component|shadow-same-site|shadow-save-buffers-kill-emacs\n\t|shadow-save-todo-file|shadow-set-cluster|shadow-shadows-of-1|shadow-shadows-of|shadow-shadows|shadow-site-cluster|shadow-site-match\n\t|shadow-site-primary|shadow-suffix|shadow-union|shadow-write-info-file|shadow-write-todo-file|shadowfile-unload-function|shared-initialize\n\t|shell--command-completion-data|shell--parse-pcomplete-arguments|shell--requote-argument|shell--unquote&requote-argument|shell--unquote-argument\n\t|shell-apply-ansi-color|shell-backward-command|shell-c-a-p-replace-by-expanded-directory|shell-cd|shell-command-completion-function\n\t|shell-command-completion|shell-command-on-region|shell-command-sentinel|shell-command|shell-completion-vars|shell-copy-environment-variable\n\t|shell-directory-tracker|shell-dirstack-message|shell-dirtrack-mode|shell-dirtrack-toggle|shell-dynamic-complete-command|shell-dynamic-complete-environment-variable\n\t|shell-dynamic-complete-filename|shell-environment-variable-completion|shell-extract-num|shell-filename-completion|shell-filter-ctrl-a-ctrl-b\n\t|shell-forward-command|shell-match-partial-variable|shell-mode|shell-prefixed-directory-name|shell-process-cd|shell-process-popd\n\t|shell-process-pushd|shell-quote-wildcard-pattern|shell-reapply-ansi-color|shell-replace-by-expanded-directory|shell-resync-dirs\n\t|shell-script-mode|shell-snarf-envar|shell-strip-ctrl-m|shell-unquote-argument|shell-write-history-on-exit|shell|shiftf|should-error\n\t|should-not|should|show-all|show-branches|show-buffer|show-children|show-entry|show-ifdef-block|show-ifdefs|show-paren--categorize-paren\n\t|show-paren--default|show-paren--locate-near-paren|show-paren--unescaped-p|show-paren-function|show-paren-mode|show-subtree|shr--extract-best-source\n\t|shr--get-media-pref|shr-add-font|shr-browse-image|shr-browse-url|shr-buffer-width|shr-char-breakable-p--inliner|shr-char-breakable-p\n\t|shr-char-kinsoku-bol-p--inliner|shr-char-kinsoku-bol-p|shr-char-kinsoku-eol-p--inliner|shr-char-kinsoku-eol-p|shr-char-nospace-p--inliner\n\t|shr-char-nospace-p|shr-color->hexadecimal|shr-color-check|shr-color-hsl-to-rgb-fractions|shr-color-hue-to-rgb|shr-color-relative-to-absolute\n\t|shr-color-set-minimum-interval|shr-color-visible|shr-colorize-region|shr-column-specs|shr-copy-url|shr-count|shr-descend|shr-dom-print\n\t|shr-dom-to-xml|shr-encode-url|shr-ensure-newline|shr-ensure-paragraph|shr-expand-newlines|shr-expand-url|shr-find-fill-point|shr-fold-text\n\t|shr-fontize-dom|shr-generic|shr-get-image-data|shr-heading|shr-image-displayer|shr-image-fetched|shr-image-from-data|shr-indent\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t shr-insert-image|shr-insert-table-ruler|shr-insert-table|shr-insert|shr-make-table-1|shr-make-table|shr-max-columns|shr-mouse-browse-url\n\t|shr-next-link|shr-parse-base|shr-parse-image-data|shr-parse-style|shr-previous-link|shr-previous-newline-padding-width|shr-pro-rate-columns\n\t|shr-put-image|shr-remove-trailing-whitespace|shr-render-buffer|shr-render-region|shr-render-td|shr-rescale-image|shr-save-contents\n\t|shr-show-alt-text|shr-store-contents|shr-table-widths|shr-tag-a|shr-tag-audio|shr-tag-b|shr-tag-base|shr-tag-blockquote|shr-tag-body\n\t|shr-tag-br|shr-tag-comment|shr-tag-dd|shr-tag-del|shr-tag-div|shr-tag-dl|shr-tag-dt|shr-tag-em|shr-tag-font|shr-tag-h1|shr-tag-h2|shr-tag-h3\n\t|shr-tag-h4|shr-tag-h5|shr-tag-h6|shr-tag-hr|shr-tag-i|shr-tag-img|shr-tag-label|shr-tag-li|shr-tag-object|shr-tag-ol|shr-tag-p|shr-tag-pre\n\t|shr-tag-s|shr-tag-script|shr-tag-span|shr-tag-strong|shr-tag-style|shr-tag-sub|shr-tag-sup|shr-tag-svg|shr-tag-table-1|shr-tag-table\n\t|shr-tag-title|shr-tag-u|shr-tag-ul|shr-tag-video|shr-urlify|shr-zoom-image|shrink-window-horizontally|shrink-window|shuffle-vector\n\t|sieve-manage|sieve-mode|sieve-upload-and-bury|sieve-upload-and-kill|sieve-upload|signum|simula-backward-up-level|simula-calculate-indent\n\t|simula-context|simula-electric-keyword|simula-electric-label|simula-expand-keyword|simula-expand-stdproc|simula-find-do-match\n\t|simula-find-if|simula-find-inspect|simula-forward-down-level|simula-forward-up-level|simula-goto-definition|simula-indent-command\n\t|simula-indent-exp|simula-indent-line|simula-inside-parens|simula-install-standard-abbrevs|simula-mode|simula-next-statement|simula-popup-menu\n\t|simula-previous-statement|simula-search-backward|simula-search-forward|simula-skip-comment-backward|simula-skip-comment-forward\n\t|simula-submit-bug-report|sixth|size-indication-mode|skeleton-insert|skeleton-internal-1|skeleton-internal-list|skeleton-pair-insert-maybe\n\t|skeleton-proxy-new|skeleton-read|skip-line-prefix|slitex-mode|slot-boundp|slot-exists-p|slot-makeunbound|slot-missing|slot-unbound\n\t|slot-value|smbclient-list-shares|smbclient-mode|smbclient|smerge--get-marker|smerge-apply-resolution-patch|smerge-auto-combine\n\t|smerge-auto-leave|smerge-batch-resolve|smerge-check|smerge-combine-with-next|smerge-conflict-overlay|smerge-context-menu|smerge-diff-base-mine\n\t|smerge-diff-base-other|smerge-diff-mine-other|smerge-diff|smerge-ediff|smerge-ensure-match|smerge-find-conflict|smerge-get-current\n\t|smerge-keep-all|smerge-keep-base|smerge-keep-current|smerge-keep-mine|smerge-keep-n|smerge-keep-other|smerge-kill-current|smerge-makeup-conflict\n\t|smerge-match-conflict|smerge-mode-menu|smerge-mode|smerge-next|smerge-popup-context-menu|smerge-prev|smerge-refine-chopup-region\n\t|smerge-refine-forward|smerge-refine-highlight-change|smerge-refine-subst|smerge-refine|smerge-remove-props|smerge-resolve--extract-comment\n\t|smerge-resolve--normalize|smerge-resolve-all|smerge-resolve|smerge-start-session|smerge-swap|smie--associative-p|smie--matching-block-data\n\t|smie--next-indent-change|smie--opener\\/closer-at-point|smie-auto-fill|smie-backward-sexp-command|smie-backward-sexp|smie-blink-matching-check\n\t|smie-blink-matching-open|smie-bnf--classify|smie-bnf--closer-alist|smie-bnf--set-class|smie-config--advice|smie-config--get-trace\n\t|smie-config--guess-1|smie-config--guess-value|smie-config--guess|smie-config--mode-hook|smie-config--setter|smie-debug--describe-cycle\n\t|smie-debug--prec2-cycle|smie-default-backward-token|smie-default-forward-token|smie-edebug|smie-forward-sexp-command|smie-forward-sexp\n\t|smie-indent--bolp-1|smie-indent--bolp|smie-indent--hanging-p|smie-indent--offset|smie-indent--parent|smie-indent--rule-1|smie-indent--rule\n\t|smie-indent--separator-outdent|smie-indent-after-keyword|smie-indent-backward-token|smie-indent-bob|smie-indent-calculate|smie-indent-close\n\t|smie-indent-comment-close|smie-indent-comment-continue|smie-indent-comment-inside|smie-indent-comment|smie-indent-exps|smie-indent-fixindent\n\t|smie-indent-forward-token|smie-indent-inside-string|smie-indent-keyword|smie-indent-line|smie-indent-virtual|smie-next-sexp|smie-op-left\n\t|smie-op-right|smie-set-prec2tab|smiley-buffer|smiley-region|smtpmail-command-or-throw|smtpmail-cred-cert|smtpmail-cred-key|smtpmail-cred-passwd\n\t|smtpmail-cred-port|smtpmail-cred-server|smtpmail-cred-user|smtpmail-deduce-address-list|smtpmail-do-bcc|smtpmail-find-credentials\n\t|smtpmail-fqdn|smtpmail-intersection|smtpmail-maybe-append-domain|smtpmail-ok-p|smtpmail-process-filter|smtpmail-query-smtp-server\n\t|smtpmail-read-response|smtpmail-response-code|smtpmail-response-text|smtpmail-send-command|smtpmail-send-data-1|smtpmail-send-data\n\t|smtpmail-send-it|smtpmail-send-queued-mail|smtpmail-try-auth-method|smtpmail-try-auth-methods|smtpmail-user-mail-address|smtpmail-via-smtp\n\t|snake-active-p|snake-display-options|snake-end-game|snake-final-x-velocity|snake-final-y-velocity|snake-init-buffer|snake-mode\n\t|snake-move-down|snake-move-left|snake-move-right|snake-move-up|snake-pause-game|snake-reset-game|snake-start-game|snake-update-game\n\t|snake-update-score|snake-update-velocity|snake|snarf-spooks|snmp-calculate-indent|snmp-common-mode|snmp-completing-read|snmp-indent-line\n\t|snmp-mode-imenu-create-index|snmp-mode|snmpv2-mode|soap-array-type-element-type--cmacro|soap-array-type-element-type|soap-array-type-name--cmacro\n\t|soap-array-type-name|soap-array-type-namespace-tag--cmacro|soap-array-type-namespace-tag|soap-array-type-p--cmacro|soap-array-type-p\n\t|soap-basic-type-kind--cmacro|soap-basic-type-kind|soap-basic-type-name--cmacro|soap-basic-type-name|soap-basic-type-namespace-tag--cmacro\n\t|soap-basic-type-namespace-tag|soap-basic-type-p--cmacro|soap-basic-type-p|soap-binding-name--cmacro|soap-binding-name|soap-binding-namespace-tag--cmacro\n\t|soap-binding-namespace-tag|soap-binding-operations--cmacro|soap-binding-operations|soap-binding-p--cmacro|soap-binding-p|soap-binding-port-type--cmacro\n\t|soap-binding-port-type|soap-bound-operation-operation--cmacro|soap-bound-operation-operation|soap-bound-operation-p--cmacro\n\t|soap-bound-operation-p|soap-bound-operation-soap-action--cmacro|soap-bound-operation-soap-action|soap-bound-operation-use--cmacro\n\t|soap-bound-operation-use|soap-create-envelope|soap-decode-any-type|soap-decode-array-type|soap-decode-array|soap-decode-basic-type\n\t|soap-decode-sequence-type|soap-decode-type|soap-default-soapenc-types|soap-default-xsd-types|soap-element-fq-name|soap-element-name--cmacro\n\t|soap-element-name|soap-element-namespace-tag--cmacro|soap-element-namespace-tag|soap-element-p--cmacro|soap-element-p|soap-encode-array-type\n\t|soap-encode-basic-type|soap-encode-body|soap-encode-sequence-type|soap-encode-simple-type|soap-encode-value|soap-extract-xmlns\n\t|soap-get-target-namespace|soap-invoke|soap-l2fq|soap-l2wk|soap-load-wsdl-from-url|soap-load-wsdl|soap-message-name--cmacro|soap-message-name\n\t|soap-message-namespace-tag--cmacro|soap-message-namespace-tag|soap-message-p--cmacro|soap-message-p|soap-message-parts--cmacro\n\t|soap-message-parts|soap-namespace-elements--cmacro|soap-namespace-elements|soap-namespace-get|soap-namespace-link-name--cmacro\n\t|soap-namespace-link-name|soap-namespace-link-namespace-tag--cmacro|soap-namespace-link-namespace-tag|soap-namespace-link-p--cmacro\n\t|soap-namespace-link-p|soap-namespace-link-target--cmacro|soap-namespace-link-target|soap-namespace-name--cmacro|soap-namespace-name\n\t|soap-namespace-p--cmacro|soap-namespace-p|soap-namespace-put-link|soap-namespace-put|soap-operation-faults--cmacro|soap-operation-faults\n\t|soap-operation-input--cmacro|soap-operation-input|soap-operation-name--cmacro|soap-operation-name|soap-operation-namespace-tag--cmacro\n\t|soap-operation-namespace-tag|soap-operation-output--cmacro|soap-operation-output|soap-operation-p--cmacro|soap-operation-p|soap-operation-parameter-order--cmacro\n\t|soap-operation-parameter-order|soap-parse-binding|soap-parse-complex-type-complex-content|soap-parse-complex-type-sequence|soap-parse-complex-type\n\t|soap-parse-envelope|soap-parse-message|soap-parse-operation|soap-parse-port-type|soap-parse-response|soap-parse-schema-element\n\t|soap-parse-schema|soap-parse-sequence|soap-parse-simple-type|soap-parse-wsdl|soap-port-binding--cmacro|soap-port-binding|soap-port-name--cmacro\n\t|soap-port-name|soap-port-namespace-tag--cmacro|soap-port-namespace-tag|soap-port-p--cmacro|soap-port-p|soap-port-service-url--cmacro\n\t|soap-port-service-url|soap-port-type-name--cmacro|soap-port-type-name|soap-port-type-namespace-tag--cmacro|soap-port-type-namespace-tag\n\t|soap-port-type-operations--cmacro|soap-port-type-operations|soap-port-type-p--cmacro|soap-port-type-p|soap-resolve-references-for-array-type\n\t|soap-resolve-references-for-binding|soap-resolve-references-for-element|soap-resolve-references-for-message|soap-resolve-references-for-operation\n\t|soap-resolve-references-for-port|soap-resolve-references-for-sequence-type|soap-resolve-references-for-simple-type|soap-sequence-element-multiple\\?--cmacro\n\t|soap-sequence-element-multiple\\?|soap-sequence-element-name--cmacro|soap-sequence-element-name|soap-sequence-element-nillable\\?--cmacro\n\t|soap-sequence-element-nillable\\?|soap-sequence-element-p--cmacro|soap-sequence-element-p|soap-sequence-element-type--cmacro\n\t|soap-sequence-element-type|soap-sequence-type-elements--cmacro|soap-sequence-type-elements|soap-sequence-type-name--cmacro|soap-sequence-type-name\n\t|soap-sequence-type-namespace-tag--cmacro|soap-sequence-type-namespace-tag|soap-sequence-type-p--cmacro|soap-sequence-type-p\n\t|soap-sequence-type-parent--cmacro|soap-sequence-type-parent|soap-simple-type-enumeration--cmacro|soap-simple-type-enumeration\n\t|soap-simple-type-kind--cmacro|soap-simple-type-kind|soap-simple-type-name--cmacro|soap-simple-type-name|soap-simple-type-namespace-tag--cmacro\n\t|soap-simple-type-namespace-tag|soap-simple-type-p--cmacro|soap-simple-type-p|soap-type-p|soap-warning|soap-with-local-xmlns|soap-wk2l\n\t|soap-wsdl-add-alias|soap-wsdl-add-namespace|soap-wsdl-alias-table--cmacro|soap-wsdl-alias-table|soap-wsdl-find-namespace|soap-wsdl-get\n\t|soap-wsdl-namespaces--cmacro|soap-wsdl-namespaces|soap-wsdl-origin--cmacro|soap-wsdl-origin|soap-wsdl-p--cmacro|soap-wsdl-p|soap-wsdl-ports--cmacro\n\t|soap-wsdl-ports|soap-wsdl-resolve-references|soap-xml-get-attribute-or-nil1|soap-xml-get-children1|socks-build-auth-list|socks-chap-auth\n\t|socks-cram-auth|socks-filter|socks-find-route|socks-find-services-entry|socks-gssapi-auth|socks-nslookup-host|socks-open-connection\n\t|socks-open-network-stream|socks-original-open-network-stream|socks-parse-services|socks-register-authentication-method|socks-send-command\n\t|socks-split-string|socks-unregister-authentication-method|socks-username\\/password-auth-filter|socks-username\\/password-auth\n\t|socks-wait-for-state-change|solicit-char-in-string|solitaire-build-mode-line|solitaire-center-point|solitaire-check|solitaire-current-line\n\t|solitaire-do-check|solitaire-down|solitaire-insert-board|solitaire-left|solitaire-mode|solitaire-move-down|solitaire-move-left\n\t|solitaire-move-right|solitaire-move-up|solitaire-move|solitaire-possible-move|solitaire-right|solitaire-solve|solitaire-undo|solitaire-up\n\t|solitaire|some-window|some|sort\\*|sort-build-lists|sort-charsets|sort-coding-systems|sort-fields-1|sort-pages-buffer|sort-pages-in-region\n\t|sort-regexp-fields-next-record|sort-reorder-buffer|sort-skip-fields|soundex|spaces-string|spam-initialize|spam-report-agentize\n\t|spam-report-deagentize|spam-report-process-queue|spam-report-url-ping-mm-url|spam-report-url-to-file|special-display-p|special-display-popup-frame\n\t|speedbar-add-expansion-list|speedbar-add-ignored-directory-regexp|speedbar-add-ignored-path-regexp|speedbar-add-indicator|speedbar-add-localized-speedbar-support\n\t|speedbar-add-mode-functions-list|speedbar-add-supported-extension|speedbar-backward-list|speedbar-buffer-buttons-engine|speedbar-buffer-buttons-temp\n\t|speedbar-buffer-buttons|speedbar-buffer-click|speedbar-buffer-kill-buffer|speedbar-buffer-revert-buffer|speedbar-buffers-item-info\n\t|speedbar-buffers-line-directory|speedbar-buffers-line-path|speedbar-buffers-tail-notes|speedbar-center-buffer-smartly|speedbar-change-expand-button-char\n\t|speedbar-change-initial-expansion-list|speedbar-check-obj-this-line|speedbar-check-objects|speedbar-check-read-only|speedbar-check-vc-this-line\n\t|speedbar-check-vc|speedbar-clear-current-file|speedbar-click|speedbar-contract-line-descendants|speedbar-contract-line|speedbar-create-directory\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t speedbar-create-tag-hierarchy|speedbar-current-frame|speedbar-customize|speedbar-default-directory-list|speedbar-delete-overlay\n\t|speedbar-delete-subblock|speedbar-dir-follow|speedbar-directory-buttons-follow|speedbar-directory-buttons|speedbar-directory-line\n\t|speedbar-dired|speedbar-disable-update|speedbar-do-function-pointer|speedbar-edit-line|speedbar-enable-update|speedbar-expand-line-descendants\n\t|speedbar-expand-line|speedbar-extension-list-to-regex|speedbar-extract-one-symbol|speedbar-fetch-dynamic-etags|speedbar-fetch-dynamic-imenu\n\t|speedbar-fetch-dynamic-tags|speedbar-fetch-replacement-function|speedbar-file-lists|speedbar-files-item-info|speedbar-files-line-directory\n\t|speedbar-find-file-in-frame|speedbar-find-file|speedbar-find-selected-file|speedbar-flush-expand-line|speedbar-forward-list|speedbar-frame-mode\n\t|speedbar-frame-reposition-smartly|speedbar-frame-width|speedbar-generic-item-info|speedbar-generic-list-group-p|speedbar-generic-list-positioned-group-p\n\t|speedbar-generic-list-tag-p|speedbar-get-focus|speedbar-goto-this-file|speedbar-handle-delete-frame|speedbar-highlight-one-tag-line\n\t|speedbar-image-dump|speedbar-initial-expansion-list|speedbar-initial-keymap|speedbar-initial-menu|speedbar-initial-stealthy-functions\n\t|speedbar-insert-button|speedbar-insert-etags-list|speedbar-insert-files-at-point|speedbar-insert-generic-list|speedbar-insert-image-button-maybe\n\t|speedbar-insert-imenu-list|speedbar-insert-separator|speedbar-item-byte-compile|speedbar-item-copy|speedbar-item-delete|speedbar-item-info-file-helper\n\t|speedbar-item-info-tag-helper|speedbar-item-info|speedbar-item-load|speedbar-item-object-delete|speedbar-item-rename|speedbar-line-directory\n\t|speedbar-line-file|speedbar-line-path|speedbar-line-text|speedbar-line-token|speedbar-make-button|speedbar-make-overlay|speedbar-make-specialized-keymap\n\t|speedbar-make-tag-line|speedbar-maybe-add-localized-support|speedbar-maybee-jump-to-attached-frame|speedbar-message|speedbar-mode-line-update\n\t|speedbar-mode|speedbar-mouse-item-info|speedbar-navigate-list|speedbar-next|speedbar-overlay-put|speedbar-parse-c-or-c\\+\\+tag\n\t|speedbar-parse-tex-string|speedbar-path-line|speedbar-position-cursor-on-line|speedbar-prefix-group-tag-hierarchy|speedbar-prev\n\t|speedbar-recenter-to-top|speedbar-recenter|speedbar-reconfigure-keymaps|speedbar-refresh|speedbar-remove-localized-speedbar-support\n\t|speedbar-reset-scanners|speedbar-restricted-move|speedbar-restricted-next|speedbar-restricted-prev|speedbar-scroll-down|speedbar-scroll-up\n\t|speedbar-select-attached-frame|speedbar-set-mode-line-format|speedbar-set-timer|speedbar-show-info-under-mouse|speedbar-simple-group-tag-hierarchy\n\t|speedbar-sort-tag-hierarchy|speedbar-stealthy-updates|speedbar-tag-expand|speedbar-tag-file|speedbar-tag-find|speedbar-this-file-in-vc\n\t|speedbar-timer-fn|speedbar-toggle-etags|speedbar-toggle-images|speedbar-toggle-line-expansion|speedbar-toggle-show-all-files\n\t|speedbar-toggle-sorting|speedbar-toggle-updates|speedbar-track-mouse|speedbar-trim-words-tag-hierarchy|speedbar-try-completion\n\t|speedbar-unhighlight-one-tag-line|speedbar-up-directory|speedbar-update-contents|speedbar-update-current-file|speedbar-update-directory-contents\n\t|speedbar-update-localized-contents|speedbar-update-special-contents|speedbar-vc-check-dir-p|speedbar-with-attached-buffer|speedbar-with-writable\n\t|speedbar-y-or-n-p|speedbar|split-char|split-line|split-window-horizontally|split-window-internal|split-window-vertically|spook|sql--completion-table\n\t|sql--make-help-docstring|sql--oracle-show-reserved-words|sql-accumulate-and-indent|sql-add-product-keywords|sql-add-product|sql-beginning-of-statement\n\t|sql-buffer-live-p|sql-build-completions-1|sql-build-completions|sql-comint-db2|sql-comint-informix|sql-comint-ingres|sql-comint-interbase\n\t|sql-comint-linter|sql-comint-ms|sql-comint-mysql|sql-comint-oracle|sql-comint-postgres|sql-comint-solid|sql-comint-sqlite|sql-comint-sybase\n\t|sql-comint-vertica|sql-comint|sql-connect|sql-connection-menu-filter|sql-copy-column|sql-db2|sql-default-value|sql-del-product|sql-end-of-statement\n\t|sql-ends-with-prompt-re|sql-escape-newlines-filter|sql-execute-feature|sql-execute|sql-find-sqli-buffer|sql-font-lock-keywords-builder\n\t|sql-for-each-login|sql-get-login-ext|sql-get-login|sql-get-product-feature|sql-help-list-products|sql-help|sql-highlight-ansi-keywords\n\t|sql-highlight-db2-keywords|sql-highlight-informix-keywords|sql-highlight-ingres-keywords|sql-highlight-interbase-keywords|sql-highlight-linter-keywords\n\t|sql-highlight-ms-keywords|sql-highlight-mysql-keywords|sql-highlight-oracle-keywords|sql-highlight-postgres-keywords|sql-highlight-product\n\t|sql-highlight-solid-keywords|sql-highlight-sqlite-keywords|sql-highlight-sybase-keywords|sql-highlight-vertica-keywords|sql-informix\n\t|sql-ingres|sql-input-sender|sql-interactive-mode-menu|sql-interactive-mode|sql-interactive-remove-continuation-prompt|sql-interbase\n\t|sql-linter|sql-list-all|sql-list-table|sql-magic-go|sql-magic-semicolon|sql-make-alternate-buffer-name|sql-mode-menu|sql-mode|sql-ms\n\t|sql-mysql|sql-oracle-completion-object|sql-oracle-list-all|sql-oracle-list-table|sql-oracle-restore-settings|sql-oracle-save-settings\n\t|sql-oracle|sql-placeholders-filter|sql-postgres-completion-object|sql-postgres|sql-product-font-lock-syntax-alist|sql-product-font-lock\n\t|sql-product-interactive|sql-product-syntax-table|sql-read-connection|sql-read-product|sql-read-table-name|sql-redirect-one|sql-redirect-value\n\t|sql-redirect|sql-regexp-abbrev-list|sql-regexp-abbrev|sql-remove-tabs-filter|sql-rename-buffer|sql-save-connection|sql-send-buffer\n\t|sql-send-line-and-next|sql-send-magic-terminator|sql-send-paragraph|sql-send-region|sql-send-string|sql-set-product-feature|sql-set-product\n\t|sql-set-sqli-buffer-generally|sql-set-sqli-buffer|sql-show-sqli-buffer|sql-solid|sql-sqlite-completion-object|sql-sqlite|sql-starts-with-prompt-re\n\t|sql-statement-regexp|sql-stop|sql-str-literal|sql-sybase|sql-toggle-pop-to-buffer-after-send-region|sql-vertica|squeeze-bidi-context-1\n\t|squeeze-bidi-context|srecode-compile-templates|srecode-document-insert-comment|srecode-document-insert-function-comment|srecode-document-insert-group-comments\n\t|srecode-document-insert-variable-one-line-comment|srecode-get-maps|srecode-insert-getset|srecode-insert-prototype-expansion\n\t|srecode-insert|srecode-minor-mode|srecode-semantic-handle-:c|srecode-semantic-handle-:cpp|srecode-semantic-handle-:el-custom\n\t|srecode-semantic-handle-:el|srecode-semantic-handle-:java|srecode-semantic-handle-:srt|srecode-semantic-handle-:texi|srecode-semantic-handle-:texitag\n\t|srecode-template-mode|srecode-template-setup-parser|srt-mode|stable-sort|standard-class|standard-display-8bit|standard-display-ascii\n\t|standard-display-cyrillic-translit|standard-display-default|standard-display-european-internal|standard-display-european|standard-display-g1\n\t|standard-display-graphic|standard-display-underline|start-kbd-macro|start-of-paragraph-text|start-scheme|starttls-any-program-available\n\t|starttls-available-p|starttls-negotiate-gnutls|starttls-negotiate|starttls-open-stream-gnutls|starttls-open-stream|starttls-set-process-query-on-exit-flag\n\t|startup-echo-area-message|straight-use-package|store-kbd-macro-event|string-blank-p|string-collate-equalp|string-collate-lessp|string-empty-p|string-insert-rectangle\n\t|string-join|string-make-multibyte|string-make-unibyte|string-rectangle-line|string-rectangle|string-remove-prefix|string-remove-suffix\n\t|string-reverse|string-to-list|string-to-vector|string-trim-left|string-trim-right|string-trim|strokes-alphabetic-lessp|strokes-button-press-event-p\n\t|strokes-button-release-event-p|strokes-click-p|strokes-compose-complex-stroke|strokes-decode-buffer|strokes-define-stroke|strokes-describe-stroke\n\t|strokes-distance-squared|strokes-do-complex-stroke|strokes-do-stroke|strokes-eliminate-consecutive-redundancies|strokes-encode-buffer\n\t|strokes-event-closest-point-1|strokes-event-closest-point|strokes-execute-stroke|strokes-fill-current-buffer-with-whitespace\n\t|strokes-fill-stroke|strokes-get-grid-position|strokes-get-stroke-extent|strokes-global-set-stroke-string|strokes-global-set-stroke\n\t|strokes-help|strokes-lift-p|strokes-list-strokes|strokes-load-user-strokes|strokes-match-stroke|strokes-mode|strokes-mouse-event-p\n\t|strokes-prompt-user-save-strokes|strokes-rate-stroke|strokes-read-complex-stroke|strokes-read-stroke|strokes-remassoc|strokes-renormalize-to-grid\n\t|strokes-report-bug|strokes-square|strokes-toggle-strokes-buffer|strokes-unload-function|strokes-unset-last-stroke|strokes-update-window-configuration\n\t|strokes-window-configuration-changed-p|strokes-xpm-char-bit-p|strokes-xpm-char-on-p|strokes-xpm-decode-char|strokes-xpm-encode-length-as-string\n\t|strokes-xpm-for-compressed-string|strokes-xpm-for-stroke|strokes-xpm-to-compressed-string|studlify-buffer|studlify-region|studlify-word\n\t|sublis|subr-name|subregexp-context-p|subseq|subsetp|subst-char-in-string|subst-if-not|subst-if|subst|substitute-env-in-file-name|substitute-env-vars\n\t|substitute-if-not|substitute-if|substitute-key-definition-key|substitute|subtract-time|subword-mode|sunrise-sunset|superword-mode\n\t|suspicious-object|svref|switch-to-completions|switch-to-lisp|switch-to-prolog|switch-to-scheme|switch-to-tcl|symbol-at-point|symbol-before-point-for-complete\n\t|symbol-before-point|symbol-macrolet|symbol-under-or-before-point|symbol-under-point|syntax-ppss-after-change-function|syntax-ppss-context\n\t|syntax-ppss-debug|syntax-ppss-depth|syntax-ppss-stats|syntax-propertize--shift-groups|syntax-propertize-multiline|syntax-propertize-precompile-rules\n\t|syntax-propertize-rules|syntax-propertize-via-font-lock|syntax-propertize-wholelines|syntax-propertize|t-mouse-mode|tabify|table--at-cell-p\n\t|table--buffer-substring-and-trim|table--cancel-timer|table--cell-blank-str|table--cell-can-span-p|table--cell-can-split-horizontally-p\n\t|table--cell-can-split-vertically-p|table--cell-horizontal-char-p|table--cell-insert-char|table--cell-list-to-coord-list|table--cell-to-coord\n\t|table--char-in-str-at-column|table--copy-coordinate|table--create-growing-space-below|table--current-line|table--detect-cell-alignment\n\t|table--editable-cell-p|table--fill-region-strictly|table--fill-region|table--find-row-column|table--finish-delayed-tasks|table--generate-source-cell-contents\n\t|table--generate-source-cells-in-a-row|table--generate-source-epilogue|table--generate-source-prologue|table--generate-source-scan-lines\n\t|table--generate-source-scan-rows|table--get-cell-justify-property|table--get-cell-valign-property|table--get-coordinate|table--get-last-command\n\t|table--get-property|table--goto-coordinate|table--horizontal-cell-list|table--horizontally-shift-above-and-below|table--insert-rectangle\n\t|table--justify-cell-contents|table--line-column-position|table--log|table--make-cell-map|table--measure-max-width|table--min-coord-list\n\t|table--multiply-string|table--offset-coordinate|table--point-entered-cell-function|table--point-in-cell-p|table--point-left-cell-function\n\t|table--probe-cell-left-up|table--probe-cell-right-bottom|table--probe-cell|table--put-cell-content-property|table--put-cell-face-property\n\t|table--put-cell-indicator-property|table--put-cell-justify-property|table--put-cell-keymap-property|table--put-cell-line-property\n\t|table--put-cell-point-entered\\/left-property|table--put-cell-property|table--put-cell-rear-nonsticky|table--put-cell-valign-property\n\t|table--put-property|table--query-justification|table--read-from-minibuffer|table--region-in-cell-p|table--remove-blank-lines\n\t|table--remove-cell-properties|table--remove-eol-spaces|table--row-column-insertion-point-p|table--set-timer|table--spacify-frame\n\t|table--str-index-at-column|table--string-to-number-list|table--test-cell-list|table--transcoord-cache-to-table|table--transcoord-table-to-cache\n\t|table--uniform-list-p|table--untabify-line|table--untabify|table--update-cell-face|table--update-cell-heightened|table--update-cell-widened\n\t|table--update-cell|table--valign|table--vertical-cell-list|table--warn-incompatibility|table-backward-cell|table-capture|table-delete-column\n\t|table-delete-row|table-fixed-width-mode|table-forward-cell|table-function|table-generate-source|table-get-source-info|table-global-menu-map\n\t|table-goto-bottom-left-corner|table-goto-bottom-right-corner|table-goto-top-left-corner|table-goto-top-right-corner|table-heighten-cell\n\t|table-insert-column|table-insert-row-column|table-insert-row|table-insert-sequence|table-insert|table-justify-cell|table-justify-column\n\t|table-justify-row|table-justify|table-narrow-cell|table-put-source-info|table-query-dimension|table-recognize-cell|table-recognize-region\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t table-recognize-table|table-recognize|table-release|table-shorten-cell|table-span-cell|table-split-cell-horizontally|table-split-cell-vertically\n\t|table-split-cell|table-unrecognize-cell|table-unrecognize-region|table-unrecognize-table|table-unrecognize|table-widen-cell|table-with-cache-buffer\n\t|tabulated-list--column-number|tabulated-list--sort-by-column-name|tabulated-list-col-sort|tabulated-list-delete-entry|tabulated-list-entry-size->\n\t|tabulated-list-get-entry|tabulated-list-get-id|tabulated-list-print-col|tabulated-list-print-entry|tabulated-list-print-fake-header\n\t|tabulated-list-put-tag|tabulated-list-revert|tabulated-list-set-col|tabulated-list-sort|tag-any-match-p|tag-exact-file-name-match-p\n\t|tag-exact-match-p|tag-file-name-match-p|tag-find-file-of-tag-noselect|tag-find-file-of-tag|tag-implicit-name-match-p|tag-partial-file-name-match-p\n\t|tag-re-match-p|tag-symbol-match-p|tag-word-match-p|tags-apropos|tags-complete-tags-table-file|tags-completion-at-point-function\n\t|tags-completion-table|tags-expand-table-name|tags-included-tables|tags-lazy-completion-table|tags-loop-continue|tags-loop-eval\n\t|tags-next-table|tags-query-replace|tags-recognize-empty-tags-table|tags-reset-tags-tables|tags-search|tags-table-check-computed-list\n\t|tags-table-extend-computed-list|tags-table-files|tags-table-including|tags-table-list-member|tags-table-mode|tags-verify-table\n\t|tags-with-face|tai-viet-composition-function|tailp|talk-add-display|talk-connect|talk-disconnect|talk-handle-delete-frame|talk-split-up-frame\n\t|talk-update-buffers|talk|tar--check-descriptor|tar--extract|tar-alter-one-field|tar-change-major-mode-hook|tar-chgrp-entry|tar-chmod-entry\n\t|tar-chown-entry|tar-clear-modification-flags|tar-clip-time-string|tar-copy|tar-current-descriptor|tar-data-swapped-p|tar-display-other-window\n\t|tar-expunge-internal|tar-expunge|tar-extract-other-window|tar-extract|tar-file-name-handler|tar-flag-deleted|tar-get-descriptor\n\t|tar-get-file-descriptor|tar-grind-file-mode|tar-header-block-check-checksum|tar-header-block-checksum|tar-header-block-summarize\n\t|tar-header-block-tokenize|tar-header-checksum--cmacro|tar-header-checksum|tar-header-data-end|tar-header-data-start--cmacro|tar-header-data-start\n\t|tar-header-date--cmacro|tar-header-date|tar-header-dmaj--cmacro|tar-header-dmaj|tar-header-dmin--cmacro|tar-header-dmin|tar-header-gid--cmacro\n\t|tar-header-gid|tar-header-gname--cmacro|tar-header-gname|tar-header-header-start--cmacro|tar-header-header-start|tar-header-link-name--cmacro\n\t|tar-header-link-name|tar-header-link-type--cmacro|tar-header-link-type|tar-header-magic--cmacro|tar-header-magic|tar-header-mode--cmacro\n\t|tar-header-mode|tar-header-name--cmacro|tar-header-name|tar-header-p--cmacro|tar-header-p|tar-header-size--cmacro|tar-header-size\n\t|tar-header-uid--cmacro|tar-header-uid|tar-header-uname--cmacro|tar-header-uname|tar-mode-kill-buffer-hook|tar-mode-revert|tar-mode\n\t|tar-mouse-extract|tar-next-line|tar-octal-time|tar-pad-to-blocksize|tar-parse-octal-integer-safe|tar-parse-octal-integer|tar-parse-octal-long-integer\n\t|tar-previous-line|tar-read-file-name|tar-rename-entry|tar-roundup-512|tar-subfile-mode|tar-subfile-save-buffer|tar-summarize-buffer\n\t|tar-swap-data|tar-unflag-backwards|tar-unflag|tar-untar-buffer|tar-view|tar-write-region-annotate|tcl-add-log-defun|tcl-auto-fill-mode\n\t|tcl-beginning-of-defun|tcl-calculate-indent|tcl-comment-indent|tcl-current-word|tcl-electric-brace|tcl-electric-char|tcl-electric-hash\n\t|tcl-end-of-defun|tcl-eval-defun|tcl-eval-region|tcl-figure-type|tcl-files-alist|tcl-filter|tcl-guess-application|tcl-hairy-scan-for-comment\n\t|tcl-hashify-buffer|tcl-help-on-word|tcl-help-snarf-commands|tcl-in-comment|tcl-indent-command|tcl-indent-exp|tcl-indent-for-comment\n\t|tcl-indent-line|tcl-load-file|tcl-mark-defun|tcl-mark|tcl-mode-menu|tcl-mode|tcl-outline-level|tcl-popup-menu|tcl-quote|tcl-real-command-p\n\t|tcl-real-comment-p|tcl-reread-help-files|tcl-restart-with-file|tcl-send-region|tcl-send-string|tcl-set-font-lock-keywords|tcl-set-proc-regexp\n\t|tcl-uncomment-region|tcl-word-no-props|tear-off-window|telnet-c-z|telnet-check-software-type-initialize|telnet-filter|telnet-initial-filter\n\t|telnet-interrupt-subjob|telnet-mode|telnet-send-input|telnet-simple-send|telnet|temp-buffer-resize-mode|temp-buffer-window-setup\n\t|temp-buffer-window-show|tempo-add-tag|tempo-backward-mark|tempo-build-collection|tempo-complete-tag|tempo-define-template|tempo-display-completions\n\t|tempo-expand-if-complete|tempo-find-match-string|tempo-forget-insertions|tempo-forward-mark|tempo-insert-mark|tempo-insert-named\n\t|tempo-insert-prompt-compat|tempo-insert-prompt|tempo-insert-template|tempo-insert|tempo-invalidate-collection|tempo-is-user-element\n\t|tempo-lookup-named|tempo-process-and-insert-string|tempo-save-named|tempo-template-dcl-f\\$context|tempo-template-dcl-f\\$csid\n\t|tempo-template-dcl-f\\$cvsi|tempo-template-dcl-f\\$cvtime|tempo-template-dcl-f\\$cvui|tempo-template-dcl-f\\$device|tempo-template-dcl-f\\$directory\n\t|tempo-template-dcl-f\\$edit|tempo-template-dcl-f\\$element|tempo-template-dcl-f\\$environment|tempo-template-dcl-f\\$extract\n\t|tempo-template-dcl-f\\$fao|tempo-template-dcl-f\\$file_attributes|tempo-template-dcl-f\\$getdvi|tempo-template-dcl-f\\$getjpi\n\t|tempo-template-dcl-f\\$getqui|tempo-template-dcl-f\\$getsyi|tempo-template-dcl-f\\$identifier|tempo-template-dcl-f\\$integer\n\t|tempo-template-dcl-f\\$length|tempo-template-dcl-f\\$locate|tempo-template-dcl-f\\$message|tempo-template-dcl-f\\$mode|tempo-template-dcl-f\\$parse\n\t|tempo-template-dcl-f\\$pid|tempo-template-dcl-f\\$privilege|tempo-template-dcl-f\\$process|tempo-template-dcl-f\\$search|tempo-template-dcl-f\\$setprv\n\t|tempo-template-dcl-f\\$string|tempo-template-dcl-f\\$time|tempo-template-dcl-f\\$trnlnm|tempo-template-dcl-f\\$type|tempo-template-dcl-f\\$user\n\t|tempo-template-dcl-f\\$verify|tempo-template-snmp-object-type|tempo-template-snmp-table-type|tempo-template-snmpv2-object-type\n\t|tempo-template-snmpv2-table-type|tempo-template-snmpv2-textual-convention|tempo-use-tag-list|tenth|term-adjust-current-row-cache\n\t|term-after-pmark-p|term-ansi-make-term|term-ansi-reset|term-args|term-arguments|term-backward-matching-input|term-bol|term-buffer-vertical-motion\n\t|term-char-mode|term-check-kill-echo-list|term-check-proc|term-check-size|term-check-source|term-command-hook|term-continue-subjob\n\t|term-copy-old-input|term-current-column|term-current-row|term-delchar-or-maybe-eof|term-delete-chars|term-delete-lines|term-delim-arg\n\t|term-directory|term-display-buffer-line|term-display-line|term-down|term-dynamic-complete-as-filename|term-dynamic-complete-filename\n\t|term-dynamic-complete|term-dynamic-list-completions|term-dynamic-list-filename-completions|term-dynamic-list-input-ring|term-dynamic-simple-complete\n\t|term-emulate-terminal|term-erase-in-display|term-erase-in-line|term-exec-1|term-exec|term-extract-string|term-forward-matching-input\n\t|term-get-old-input-default|term-get-source|term-goto-home|term-goto|term-handle-ansi-escape|term-handle-ansi-terminal-messages\n\t|term-handle-colors-array|term-handle-deferred-scroll|term-handle-exit|term-handle-scroll|term-handling-pager|term-horizontal-column\n\t|term-how-many-region|term-in-char-mode|term-in-line-mode|term-insert-char|term-insert-lines|term-insert-spaces|term-interrupt-subjob\n\t|term-kill-input|term-kill-output|term-kill-subjob|term-line-mode|term-magic-space|term-match-partial-filename|term-mode|term-mouse-paste\n\t|term-move-columns|term-next-input|term-next-matching-input-from-input|term-next-matching-input|term-next-prompt|term-pager-back-line\n\t|term-pager-back-page|term-pager-bob|term-pager-continue|term-pager-disable|term-pager-discard|term-pager-enable|term-pager-enabled\n\t|term-pager-eob|term-pager-help|term-pager-line|term-pager-menu|term-pager-page|term-pager-toggle|term-paste|term-previous-input-string\n\t|term-previous-input|term-previous-matching-input-from-input|term-previous-matching-input-string-position|term-previous-matching-input-string\n\t|term-previous-matching-input|term-previous-prompt|term-proc-query|term-process-pager|term-quit-subjob|term-read-input-ring|term-read-noecho\n\t|term-regexp-arg|term-replace-by-expanded-filename|term-replace-by-expanded-history-before-point|term-replace-by-expanded-history\n\t|term-reset-size|term-reset-terminal|term-search-arg|term-search-start|term-send-backspace|term-send-del|term-send-down|term-send-end\n\t|term-send-eof|term-send-home|term-send-input|term-send-insert|term-send-invisible|term-send-left|term-send-next|term-send-prior\n\t|term-send-raw-meta|term-send-raw-string|term-send-raw|term-send-region|term-send-right|term-send-string|term-send-up|term-sentinel\n\t|term-set-escape-char|term-set-scroll-region|term-show-maximum-output|term-show-output|term-signals-menu|term-simple-send|term-skip-prompt\n\t|term-source-default|term-start-line-column|term-start-output-log|term-stop-output-log|term-stop-subjob|term-terminal-menu|term-terminal-pos\n\t|term-unwrap-line|term-update-mode-line|term-using-alternate-sub-buffer|term-vertical-motion|term-window-width|term-within-quotes\n\t|term-word|term-write-input-ring|term|testcover-1value|testcover-after|testcover-end|testcover-enter|testcover-mark|testcover-read\n\t|testcover-reinstrument-compose|testcover-reinstrument-list|testcover-reinstrument|testcover-this-defun|testcover-unmark-all|tetris-active-p\n\t|tetris-default-update-speed-function|tetris-display-options|tetris-draw-border-p|tetris-draw-next-shape|tetris-draw-score|tetris-draw-shape\n\t|tetris-end-game|tetris-erase-shape|tetris-full-row|tetris-get-shape-cell|tetris-get-tick-period|tetris-init-buffer|tetris-mode\n\t|tetris-move-bottom|tetris-move-left|tetris-move-right|tetris-new-shape|tetris-pause-game|tetris-reset-game|tetris-rotate-next|tetris-rotate-prev\n\t|tetris-shape-done|tetris-shape-rotations|tetris-shape-width|tetris-shift-down|tetris-shift-row|tetris-start-game|tetris-test-shape\n\t|tetris-update-game|tetris-update-score|tetris|tex-alt-print|tex-append|tex-bibtex-file|tex-buffer|tex-categorize-whitespace|tex-close-latex-block\n\t|tex-cmd-doc-view|tex-command-active-p|tex-command-executable|tex-common-initialization|tex-compile-default|tex-compile|tex-count-words\n\t|tex-current-defun-name|tex-define-common-keys|tex-delete-last-temp-files|tex-display-shell|tex-env-mark|tex-executable-exists-p\n\t|tex-expand-files|tex-facemenu-add-face-function|tex-feed-input|tex-file|tex-font-lock-append-prop|tex-font-lock-match-suscript\n\t|tex-font-lock-suscript|tex-font-lock-syntactic-face-function|tex-font-lock-unfontify-region|tex-font-lock-verb|tex-format-cmd\n\t|tex-generate-zap-file-name|tex-goto-last-unclosed-latex-block|tex-guess-main-file|tex-guess-mode|tex-insert-braces|tex-insert-quote\n\t|tex-kill-job|tex-last-unended-begin|tex-last-unended-eparen|tex-latex-block|tex-main-file|tex-mode-flyspell-verify|tex-mode-internal\n\t|tex-mode|tex-next-unmatched-end|tex-next-unmatched-eparen|tex-old-error-file-name|tex-print|tex-recenter-output-buffer|tex-region-header\n\t|tex-region|tex-search-noncomment|tex-send-command|tex-send-tex-command|tex-set-buffer-directory|tex-shell-buf-no-error|tex-shell-buf\n\t|tex-shell-proc|tex-shell-running|tex-shell-sentinel|tex-shell|tex-show-print-queue|tex-start-shell|tex-start-tex|tex-string-prefix-p\n\t|tex-summarize-command|tex-suscript-height|tex-terminate-paragraph|tex-uptodate-p|tex-validate-buffer|tex-validate-region|tex-view\n\t|texi2info|texinfmt-version|texinfo-alias|texinfo-all-menus-update|texinfo-alphaenumerate-item|texinfo-alphaenumerate|texinfo-anchor\n\t|texinfo-append-refill|texinfo-capsenumerate-item|texinfo-capsenumerate|texinfo-check-for-node-name|texinfo-clean-up-node-line\n\t|texinfo-clear|texinfo-clone-environment|texinfo-copy-menu-title|texinfo-copy-menu|texinfo-copy-next-section-title|texinfo-copy-node-name\n\t|texinfo-copy-section-title|texinfo-copying|texinfo-current-defun-name|texinfo-define-common-keys|texinfo-define-info-enclosure\n\t|texinfo-delete-existing-pointers|texinfo-delete-from-print-queue|texinfo-delete-old-menu|texinfo-description|texinfo-discard-command-and-arg\n\t|texinfo-discard-command|texinfo-discard-line-with-args|texinfo-discard-line|texinfo-do-flushright|texinfo-do-itemize|texinfo-end-alphaenumerate\n\t|texinfo-end-capsenumerate|texinfo-end-defun|texinfo-end-direntry|texinfo-end-enumerate|texinfo-end-example|texinfo-end-flushleft\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t texinfo-end-flushright|texinfo-end-ftable|texinfo-end-indextable|texinfo-end-itemize|texinfo-end-multitable|texinfo-end-table\n\t|texinfo-end-vtable|texinfo-enumerate-item|texinfo-enumerate|texinfo-every-node-update|texinfo-filter|texinfo-find-higher-level-node\n\t|texinfo-find-lower-level-node|texinfo-find-pointer|texinfo-footnotestyle|texinfo-format-\\.|texinfo-format-:|texinfo-format-French-OE-ligature\n\t|texinfo-format-French-oe-ligature|texinfo-format-German-sharp-S|texinfo-format-Latin-Scandinavian-AE|texinfo-format-Latin-Scandinavian-ae\n\t|texinfo-format-Polish-suppressed-L|texinfo-format-Polish-suppressed-l-lower-case|texinfo-format-Scandinavian-A-with-circle\n\t|texinfo-format-Scandinavian-O-with-slash|texinfo-format-Scandinavian-a-with-circle|texinfo-format-Scandinavian-o-with-slash-lower-case\n\t|texinfo-format-TeX|texinfo-format-begin-end|texinfo-format-begin|texinfo-format-breve-accent|texinfo-format-buffer-1|texinfo-format-buffer\n\t|texinfo-format-bullet|texinfo-format-cedilla-accent|texinfo-format-center|texinfo-format-chapter-1|texinfo-format-chapter|texinfo-format-cindex\n\t|texinfo-format-code|texinfo-format-convert|texinfo-format-copyright|texinfo-format-ctrl|texinfo-format-defcv|texinfo-format-deffn\n\t|texinfo-format-defindex|texinfo-format-defivar|texinfo-format-defmethod|texinfo-format-defn|texinfo-format-defop|texinfo-format-deftypefn\n\t|texinfo-format-deftypefun|texinfo-format-defun-1|texinfo-format-defun|texinfo-format-defunx|texinfo-format-dircategory|texinfo-format-direntry\n\t|texinfo-format-documentdescription|texinfo-format-dotless|texinfo-format-dots|texinfo-format-email|texinfo-format-emph|texinfo-format-end-node\n\t|texinfo-format-end|texinfo-format-enddots|texinfo-format-equiv|texinfo-format-error|texinfo-format-example|texinfo-format-exdent\n\t|texinfo-format-expand-region|texinfo-format-expansion|texinfo-format-findex|texinfo-format-flushleft|texinfo-format-flushright\n\t|texinfo-format-footnote|texinfo-format-hacek-accent|texinfo-format-html|texinfo-format-ifeq|texinfo-format-ifhtml|texinfo-format-ifnotinfo\n\t|texinfo-format-ifplaintext|texinfo-format-iftex|texinfo-format-ifxml|texinfo-format-ignore|texinfo-format-image|texinfo-format-inforef\n\t|texinfo-format-kbd|texinfo-format-key|texinfo-format-kindex|texinfo-format-long-Hungarian-umlaut|texinfo-format-menu|texinfo-format-minus\n\t|texinfo-format-node|texinfo-format-noop|texinfo-format-option|texinfo-format-overdot-accent|texinfo-format-paragraph-break|texinfo-format-parse-args\n\t|texinfo-format-parse-defun-args|texinfo-format-parse-line-args|texinfo-format-pindex|texinfo-format-point|texinfo-format-pounds\n\t|texinfo-format-print|texinfo-format-printindex|texinfo-format-pxref|texinfo-format-refill|texinfo-format-region|texinfo-format-result\n\t|texinfo-format-ring-accent|texinfo-format-scan|texinfo-format-section|texinfo-format-sectionpad|texinfo-format-separate-node\n\t|texinfo-format-setfilename|texinfo-format-soft-hyphen|texinfo-format-sp|texinfo-format-specialized-defun|texinfo-format-subsection\n\t|texinfo-format-subsubsection|texinfo-format-synindex|texinfo-format-tex|texinfo-format-tie-after-accent|texinfo-format-timestamp\n\t|texinfo-format-tindex|texinfo-format-titlepage|texinfo-format-titlespec|texinfo-format-today|texinfo-format-underbar-accent|texinfo-format-underdot-accent\n\t|texinfo-format-upside-down-exclamation-mark|texinfo-format-upside-down-question-mark|texinfo-format-uref|texinfo-format-var\n\t|texinfo-format-verb|texinfo-format-vindex|texinfo-format-xml|texinfo-format-xref|texinfo-ftable-item|texinfo-ftable|texinfo-hierarchic-level\n\t|texinfo-if-clear|texinfo-if-set|texinfo-incorporate-descriptions|texinfo-incorporate-menu-entry-names|texinfo-indent-menu-description\n\t|texinfo-index-defcv|texinfo-index-deffn|texinfo-index-defivar|texinfo-index-defmethod|texinfo-index-defop|texinfo-index-deftypefn\n\t|texinfo-index-defun|texinfo-index|texinfo-indextable-item|texinfo-indextable|texinfo-insert-@code|texinfo-insert-@dfn|texinfo-insert-@email\n\t|texinfo-insert-@emph|texinfo-insert-@end|texinfo-insert-@example|texinfo-insert-@file|texinfo-insert-@item|texinfo-insert-@kbd\n\t|texinfo-insert-@node|texinfo-insert-@noindent|texinfo-insert-@quotation|texinfo-insert-@samp|texinfo-insert-@strong|texinfo-insert-@table\n\t|texinfo-insert-@uref|texinfo-insert-@url|texinfo-insert-@var|texinfo-insert-block|texinfo-insert-braces|texinfo-insert-master-menu-list\n\t|texinfo-insert-menu|texinfo-insert-node-lines|texinfo-insert-pointer|texinfo-insert-quote|texinfo-insertcopying|texinfo-inside-env-p\n\t|texinfo-inside-macro-p|texinfo-item|texinfo-itemize-item|texinfo-itemize|texinfo-last-unended-begin|texinfo-locate-menu-p|texinfo-make-menu-list\n\t|texinfo-make-menu|texinfo-make-one-menu|texinfo-master-menu-list|texinfo-master-menu|texinfo-menu-copy-old-description|texinfo-menu-end\n\t|texinfo-menu-first-node|texinfo-menu-indent-description|texinfo-menu-locate-entry-p|texinfo-mode-flyspell-verify|texinfo-mode-menu\n\t|texinfo-mode|texinfo-multi-file-included-list|texinfo-multi-file-master-menu-list|texinfo-multi-file-update|texinfo-multi-files-insert-main-menu\n\t|texinfo-multiple-files-update|texinfo-multitable-extract-row|texinfo-multitable-item|texinfo-multitable-widths|texinfo-multitable\n\t|texinfo-next-unmatched-end|texinfo-noindent|texinfo-old-menu-p|texinfo-optional-braces-discard|texinfo-paragraphindent|texinfo-parse-arg-discard\n\t|texinfo-parse-expanded-arg|texinfo-parse-line-arg|texinfo-pointer-name|texinfo-pop-stack|texinfo-print-index|texinfo-push-stack\n\t|texinfo-quit-job|texinfo-raise-lower-sections|texinfo-sequential-node-update|texinfo-sequentially-find-pointer|texinfo-sequentially-insert-pointer\n\t|texinfo-sequentially-update-the-node|texinfo-set|texinfo-show-structure|texinfo-sort-region|texinfo-sort-startkeyfun|texinfo-specific-section-type\n\t|texinfo-start-menu-description|texinfo-table-item|texinfo-table|texinfo-tex-buffer|texinfo-tex-print|texinfo-tex-region|texinfo-tex-view\n\t|texinfo-texindex|texinfo-top-pointer-case|texinfo-unsupported|texinfo-update-menu-region-beginning|texinfo-update-menu-region-end\n\t|texinfo-update-node|texinfo-update-the-node|texinfo-value|texinfo-vtable-item|texinfo-vtable|text-clone--maintain|text-clone-create\n\t|text-mode-hook-identify|text-scale-adjust|text-scale-decrease|text-scale-increase|text-scale-mode|text-scale-set|thai-compose-buffer\n\t|thai-compose-region|thai-compose-string|thai-composition-function|the|thing-at-point--bounds-of-markedup-url|thing-at-point--bounds-of-well-formed-url\n\t|thing-at-point-bounds-of-list-at-point|thing-at-point-bounds-of-url-at-point|thing-at-point-looking-at|thing-at-point-newsgroup-p\n\t|thing-at-point-url-at-point|third|this-major-mode-requires-vi-state|this-single-command-keys|this-single-command-raw-keys|thread-first\n\t|thread-last|thumbs-backward-char|thumbs-backward-line|thumbs-call-convert|thumbs-call-setroot-command|thumbs-cleanup-thumbsdir\n\t|thumbs-current-image|thumbs-delete-images|thumbs-dired-setroot|thumbs-dired-show-marked|thumbs-dired-show|thumbs-dired|thumbs-display-thumbs-buffer\n\t|thumbs-do-thumbs-insertion|thumbs-emboss-image|thumbs-enlarge-image|thumbs-file-alist|thumbs-file-list|thumbs-file-size|thumbs-find-image-at-point-other-window\n\t|thumbs-find-image-at-point|thumbs-find-image|thumbs-find-thumb|thumbs-forward-char|thumbs-forward-line|thumbs-image-type|thumbs-insert-image\n\t|thumbs-insert-thumb|thumbs-kill-buffer|thumbs-make-thumb|thumbs-mark|thumbs-mode|thumbs-modify-image|thumbs-monochrome-image|thumbs-mouse-find-image\n\t|thumbs-negate-image|thumbs-new-image-size|thumbs-next-image|thumbs-previous-image|thumbs-redraw-buffer|thumbs-rename-images|thumbs-resize-image-1\n\t|thumbs-resize-image|thumbs-rotate-left|thumbs-rotate-right|thumbs-save-current-image|thumbs-set-image-at-point-to-root-window\n\t|thumbs-set-root|thumbs-show-from-dir|thumbs-show-image-num|thumbs-show-more-images|thumbs-show-name|thumbs-show-thumbs-list|thumbs-shrink-image\n\t|thumbs-temp-dir|thumbs-temp-file|thumbs-thumbname|thumbs-thumbsdir|thumbs-unmark|thumbs-view-image-mode|thumbs|tibetan-char-p|tibetan-compose-buffer\n\t|tibetan-compose-region|tibetan-compose-string|tibetan-decompose-buffer|tibetan-decompose-region|tibetan-decompose-string|tibetan-post-read-conversion\n\t|tibetan-pre-write-canonicalize-for-unicode|tibetan-pre-write-conversion|tibetan-tibetan-to-transcription|tibetan-transcription-to-tibetan\n\t|tildify--deprecated-ignore-evironments|tildify--find-env|tildify--foreach-region|tildify--pick-alist-entry|tildify-buffer|tildify-foreach-ignore-environments\n\t|tildify-region|tildify-tildify|time-date--day-in-year|time-since|time-stamp-conv-warn|time-stamp-do-number|time-stamp-fconcat|time-stamp-mail-host-name\n\t|time-stamp-once|time-stamp-string-preprocess|time-stamp-string|time-stamp-toggle-active|time-stamp|time-to-number-of-days|time-to-seconds\n\t|timeclock-ask-for-project|timeclock-ask-for-reason|timeclock-change|timeclock-completing-read|timeclock-current-debt|timeclock-currently-in-p\n\t|timeclock-day-alist|timeclock-day-base|timeclock-day-begin|timeclock-day-break|timeclock-day-debt|timeclock-day-end|timeclock-day-length\n\t|timeclock-day-list-begin|timeclock-day-list-break|timeclock-day-list-debt|timeclock-day-list-end|timeclock-day-list-length|timeclock-day-list-projects\n\t|timeclock-day-list-required|timeclock-day-list-span|timeclock-day-list-template|timeclock-day-list|timeclock-day-projects|timeclock-day-required\n\t|timeclock-day-span|timeclock-entry-begin|timeclock-entry-comment|timeclock-entry-end|timeclock-entry-length|timeclock-entry-list-begin\n\t|timeclock-entry-list-break|timeclock-entry-list-end|timeclock-entry-list-length|timeclock-entry-list-projects|timeclock-entry-list-span\n\t|timeclock-entry-project|timeclock-find-discrep|timeclock-generate-report|timeclock-in|timeclock-last-period|timeclock-log-data\n\t|timeclock-log|timeclock-make-hours-explicit|timeclock-mean|timeclock-mode-line-display|timeclock-modeline-display|timeclock-out\n\t|timeclock-project-alist|timeclock-query-out|timeclock-read-moment|timeclock-reread-log|timeclock-seconds-to-string|timeclock-seconds-to-time\n\t|timeclock-status-string|timeclock-time-to-date|timeclock-time-to-seconds|timeclock-update-mode-line|timeclock-update-modeline\n\t|timeclock-visit-timelog|timeclock-when-to-leave-string|timeclock-when-to-leave|timeclock-workday-elapsed-string|timeclock-workday-elapsed\n\t|timeclock-workday-remaining-string|timeclock-workday-remaining|timeout-event-p|timep|timer--activate|timer--args--cmacro|timer--args\n\t|timer--check|timer--function--cmacro|timer--function|timer--high-seconds--cmacro|timer--high-seconds|timer--idle-delay--cmacro\n\t|timer--idle-delay|timer--low-seconds--cmacro|timer--low-seconds|timer--psecs--cmacro|timer--psecs|timer--repeat-delay--cmacro\n\t|timer--repeat-delay|timer--time-less-p|timer--time-setter|timer--time|timer--triggered--cmacro|timer--triggered|timer--usecs--cmacro\n\t|timer--usecs|timer-activate-when-idle|timer-activate|timer-create--cmacro|timer-create|timer-duration|timer-event-handler|timer-inc-time\n\t|timer-next-integral-multiple-of-time|timer-relative-time|timer-set-function|timer-set-idle-time|timer-set-time-with-usecs|timer-set-time\n\t|timer-until|timerp|timezone-absolute-from-gregorian|timezone-day-number|timezone-fix-time|timezone-last-day-of-month|timezone-leap-year-p\n\t|timezone-make-arpa-date|timezone-make-date-arpa-standard|timezone-make-date-sortable|timezone-make-sortable-date|timezone-make-time-string\n\t|timezone-parse-date|timezone-parse-time|timezone-time-from-absolute|timezone-time-zone-from-absolute|timezone-zone-to-minute\n\t|titdic-convert|tls-certificate-information|tmm--completion-table|tmm-add-one-shortcut|tmm-add-prompt|tmm-add-shortcuts|tmm-completion-delete-prompt\n\t|tmm-define-keys|tmm-get-keybind|tmm-get-keymap|tmm-goto-completions|tmm-menubar-mouse|tmm-menubar|tmm-prompt|tmm-remove-inactive-mouse-face\n\t|tmm-shortcut|todo--user-error-if-marked-done-item|todo-absolute-file-name|todo-add-category|todo-add-file|todo-adjusted-category-label-length\n\t|todo-archive-done-item|todo-archive-mode|todo-backward-category|todo-backward-item|todo-categories-mode|todo-category-completions\n\t|todo-category-number|todo-category-select|todo-category-string-matcher-1|todo-category-string-matcher-2|todo-check-file|todo-check-filtered-items-file\n\t|todo-check-format|todo-choose-archive|todo-clear-matches|todo-comment-string-matcher|todo-convert-legacy-date-time|todo-convert-legacy-files\n\t|todo-current-category|todo-date-string-matcher|todo-delete-category|todo-delete-file|todo-delete-item|todo-desktop-save-buffer\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t todo-diary-expired-matcher|todo-diary-goto-entry|todo-diary-item-p|todo-diary-nonmarking-matcher|todo-display-categories|todo-display-sorted\n\t|todo-done-item-p|todo-done-item-section-p|todo-done-separator|todo-done-string-matcher|todo-edit-category-diary-inclusion|todo-edit-category-diary-nonmarking\n\t|todo-edit-file|todo-edit-item--diary-inclusion|todo-edit-item--header|todo-edit-item--next-key|todo-edit-item--text|todo-edit-item\n\t|todo-edit-mode|todo-edit-quit|todo-files|todo-filter-diary-items-multifile|todo-filter-diary-items|todo-filter-items-1|todo-filter-items-filename\n\t|todo-filter-items|todo-filter-regexp-items-multifile|todo-filter-regexp-items|todo-filter-top-priorities-multifile|todo-filter-top-priorities\n\t|todo-filtered-items-mode|todo-find-archive|todo-find-filtered-items-file|todo-find-item|todo-forward-category|todo-forward-item\n\t|todo-get-count|todo-get-overlay|todo-go-to-source-item|todo-indent|todo-insert-category-line|todo-insert-item--apply-args|todo-insert-item--argsleft\n\t|todo-insert-item--basic|todo-insert-item--keyof|todo-insert-item--next-param|todo-insert-item--this-key|todo-insert-item-from-calendar\n\t|todo-insert-item|todo-insert-sort-button|todo-insert-with-overlays|todo-item-done|todo-item-end|todo-item-start|todo-item-string\n\t|todo-item-undone|todo-jump-to-archive-category|todo-jump-to-category|todo-label-to-key|todo-longest-category-name-length|todo-lower-category\n\t|todo-lower-item-priority|todo-make-categories-list|todo-mark-category|todo-marked-item-p|todo-menu|todo-merge-category|todo-mode-external-set\n\t|todo-mode-line-control|todo-mode|todo-modes-set-1|todo-modes-set-2|todo-modes-set-3|todo-move-category|todo-move-item|todo-multiple-filter-files\n\t|todo-next-button|todo-next-item|todo-nondiary-marker-matcher|todo-padded-string|todo-prefix-overlays|todo-previous-button|todo-previous-item\n\t|todo-print-buffer-to-file|todo-print-buffer|todo-quit|todo-raise-category|todo-raise-item-priority|todo-read-category|todo-read-date\n\t|todo-read-dayname|todo-read-file-name|todo-read-time|todo-reevaluate-category-completions-files-defcustom|todo-reevaluate-default-file-defcustom\n\t|todo-reevaluate-filelist-defcustoms|todo-reevaluate-filter-files-defcustom|todo-remove-item|todo-rename-category|todo-rename-file\n\t|todo-repair-categories-sexp|todo-reset-and-enable-done-separator|todo-reset-comment-string|todo-reset-done-separator-string\n\t|todo-reset-done-separator|todo-reset-done-string|todo-reset-global-current-todo-file|todo-reset-highlight-item|todo-reset-nondiary-marker\n\t|todo-reset-prefix|todo-restore-desktop-buffer|todo-revert-buffer|todo-save-filtered-items-buffer|todo-save|todo-search|todo-set-categories\n\t|todo-set-category-number|todo-set-date-from-calendar|todo-set-item-priority|todo-set-show-current-file|todo-set-top-priorities-in-category\n\t|todo-set-top-priorities-in-file|todo-set-top-priorities|todo-short-file-name|todo-show-categories-table|todo-show-current-file\n\t|todo-show|todo-sort-categories-alphabetically-or-numerically|todo-sort-categories-by-archived|todo-sort-categories-by-diary\n\t|todo-sort-categories-by-done|todo-sort-categories-by-todo|todo-sort|todo-time-string-matcher|todo-toggle-item-header|todo-toggle-item-highlighting\n\t|todo-toggle-mark-item|todo-toggle-prefix-numbers|todo-toggle-view-done-items|todo-toggle-view-done-only|todo-total-item-counts\n\t|todo-unarchive-items|todo-unmark-category|todo-update-buffer-list|todo-update-categories-display|todo-update-categories-sexp\n\t|todo-update-count|todo-validate-name|todo-y-or-n-p|toggle-auto-composition|toggle-case-fold-search|toggle-debug-on-error|toggle-debug-on-quit\n\t|toggle-emacs-lock|toggle-frame-fullscreen|toggle-frame-maximized|toggle-horizontal-scroll-bar|toggle-indicate-empty-lines|toggle-input-method\n\t|toggle-menu-bar-mode-from-frame|toggle-read-only|toggle-rot13-mode|toggle-save-place-globally|toggle-save-place|toggle-scroll-bar\n\t|toggle-text-mode-auto-fill|toggle-tool-bar-mode-from-frame|toggle-truncate-lines|toggle-uniquify-buffer-names|toggle-use-system-font\n\t|toggle-viper-mode|toggle-word-wrap|tool-bar--image-expression|tool-bar-get-system-style|tool-bar-height|tool-bar-lines-needed\n\t|tool-bar-local-item|tool-bar-make-keymap-1|tool-bar-make-keymap|tool-bar-mode|tool-bar-pixel-width|tool-bar-setup|tooltip-cancel-delayed-tip\n\t|tooltip-delay|tooltip-event-buffer|tooltip-expr-to-print|tooltip-gud-toggle-dereference|tooltip-help-tips|tooltip-hide|tooltip-identifier-from-point\n\t|tooltip-mode|tooltip-process-prompt-regexp|tooltip-set-param|tooltip-show-help-non-mode|tooltip-show-help|tooltip-show|tooltip-start-delayed-tip\n\t|tooltip-strip-prompt|tooltip-timeout|tq-buffer|tq-filter|tq-process-buffer|tq-process|tq-queue-add|tq-queue-empty|tq-queue-head-closure\n\t|tq-queue-head-fn|tq-queue-head-question|tq-queue-head-regexp|tq-queue-pop|tq-queue|trace--display-buffer|trace--read-args|trace-entry-message\n\t|trace-exit-message|trace-function-background|trace-function-foreground|trace-function-internal|trace-function|trace-is-traced\n\t|trace-make-advice|trace-values|traceroute|tramp-accept-process-output|tramp-action-login|tramp-action-out-of-band|tramp-action-password\n\t|tramp-action-permission-denied|tramp-action-process-alive|tramp-action-succeed|tramp-action-terminal|tramp-action-yesno|tramp-action-yn\n\t|tramp-adb-file-name-handler|tramp-adb-file-name-p|tramp-adb-parse-device-names|tramp-autoload-file-name-handler|tramp-backtrace\n\t|tramp-buffer-name|tramp-bug|tramp-cache-print|tramp-call-process|tramp-check-cached-permissions|tramp-check-for-regexp|tramp-check-proper-method-and-host\n\t|tramp-cleanup-all-buffers|tramp-cleanup-all-connections|tramp-cleanup-connection|tramp-cleanup-this-connection|tramp-clear-passwd\n\t|tramp-compat-coding-system-change-eol-conversion|tramp-compat-condition-case-unless-debug|tramp-compat-copy-directory|tramp-compat-copy-file\n\t|tramp-compat-decimal-to-octal|tramp-compat-delete-directory|tramp-compat-delete-file|tramp-compat-file-attributes|tramp-compat-font-lock-add-keywords\n\t|tramp-compat-funcall|tramp-compat-load|tramp-compat-make-temp-file|tramp-compat-most-positive-fixnum|tramp-compat-number-sequence\n\t|tramp-compat-octal-to-decimal|tramp-compat-process-get|tramp-compat-process-put|tramp-compat-process-running-p|tramp-compat-replace-regexp-in-string\n\t|tramp-compat-set-process-query-on-exit-flag|tramp-compat-split-string|tramp-compat-temporary-file-directory|tramp-compat-with-temp-message\n\t|tramp-completion-dissect-file-name|tramp-completion-dissect-file-name1|tramp-completion-file-name-handler|tramp-completion-handle-file-name-all-completions\n\t|tramp-completion-handle-file-name-completion|tramp-completion-make-tramp-file-name|tramp-completion-mode-p|tramp-completion-run-real-handler\n\t|tramp-condition-case-unless-debug|tramp-connectable-p|tramp-connection-property-p|tramp-debug-buffer-name|tramp-debug-message\n\t|tramp-debug-outline-level|tramp-default-file-modes|tramp-delete-temp-file-function|tramp-dissect-file-name|tramp-drop-volume-letter\n\t|tramp-equal-remote|tramp-error-with-buffer|tramp-error|tramp-eshell-directory-change|tramp-exists-file-name-handler|tramp-file-mode-from-int\n\t|tramp-file-mode-permissions|tramp-file-name-domain|tramp-file-name-for-operation|tramp-file-name-handler|tramp-file-name-hop\n\t|tramp-file-name-host|tramp-file-name-localname|tramp-file-name-method|tramp-file-name-p|tramp-file-name-port|tramp-file-name-real-host\n\t|tramp-file-name-real-user|tramp-file-name-user|tramp-find-file-name-coding-system-alist|tramp-find-foreign-file-name-handler\n\t|tramp-find-host|tramp-find-method|tramp-find-user|tramp-flush-connection-property|tramp-flush-directory-property|tramp-flush-file-property\n\t|tramp-ftp-enable-ange-ftp|tramp-ftp-file-name-handler|tramp-ftp-file-name-p|tramp-get-buffer|tramp-get-completion-function|tramp-get-completion-methods\n\t|tramp-get-completion-user-host|tramp-get-connection-buffer|tramp-get-connection-name|tramp-get-connection-process|tramp-get-connection-property\n\t|tramp-get-debug-buffer|tramp-get-device|tramp-get-file-property|tramp-get-inode|tramp-get-local-gid|tramp-get-local-uid|tramp-get-method-parameter\n\t|tramp-get-remote-tmpdir|tramp-gvfs-file-name-handler|tramp-gvfs-file-name-p|tramp-gw-open-connection|tramp-handle-directory-file-name\n\t|tramp-handle-directory-files-and-attributes|tramp-handle-directory-files|tramp-handle-dired-uncache|tramp-handle-file-accessible-directory-p\n\t|tramp-handle-file-exists-p|tramp-handle-file-modes|tramp-handle-file-name-as-directory|tramp-handle-file-name-completion|tramp-handle-file-name-directory\n\t|tramp-handle-file-name-nondirectory|tramp-handle-file-newer-than-file-p|tramp-handle-file-notify-add-watch|tramp-handle-file-notify-rm-watch\n\t|tramp-handle-file-regular-p|tramp-handle-file-remote-p|tramp-handle-file-symlink-p|tramp-handle-find-backup-file-name|tramp-handle-insert-directory\n\t|tramp-handle-insert-file-contents|tramp-handle-load|tramp-handle-make-auto-save-file-name|tramp-handle-make-symbolic-link|tramp-handle-set-visited-file-modtime\n\t|tramp-handle-shell-command|tramp-handle-substitute-in-file-name|tramp-handle-unhandled-file-name-directory|tramp-handle-verify-visited-file-modtime\n\t|tramp-list-connections|tramp-local-host-p|tramp-make-tramp-file-name|tramp-make-tramp-temp-file|tramp-message|tramp-mode-string-to-int\n\t|tramp-parse-connection-properties|tramp-parse-file|tramp-parse-group|tramp-parse-hosts-group|tramp-parse-hosts|tramp-parse-netrc-group\n\t|tramp-parse-netrc|tramp-parse-passwd-group|tramp-parse-passwd|tramp-parse-putty-group|tramp-parse-putty|tramp-parse-rhosts-group\n\t|tramp-parse-rhosts|tramp-parse-sconfig-group|tramp-parse-sconfig|tramp-parse-shostkeys-sknownhosts|tramp-parse-shostkeys|tramp-parse-shosts-group\n\t|tramp-parse-shosts|tramp-parse-sknownhosts|tramp-process-actions|tramp-process-one-action|tramp-progress-reporter-update|tramp-read-passwd\n\t|tramp-register-autoload-file-name-handlers|tramp-register-file-name-handlers|tramp-replace-environment-variables|tramp-rfn-eshadow-setup-minibuffer\n\t|tramp-rfn-eshadow-update-overlay|tramp-run-real-handler|tramp-send-string|tramp-set-auto-save-file-modes|tramp-set-completion-function\n\t|tramp-set-connection-property|tramp-set-file-property|tramp-sh-file-name-handler|tramp-shell-quote-argument|tramp-smb-file-name-handler\n\t|tramp-smb-file-name-p|tramp-subst-strs-in-string|tramp-time-diff|tramp-tramp-file-p|tramp-unload-file-name-handlers|tramp-unload-tramp\n\t|tramp-user-error|tramp-uuencode-region|tramp-version|tramp-wait-for-regexp|transform-make-coding-system-args|translate-region-internal\n\t|transpose-chars|transpose-lines|transpose-paragraphs|transpose-sentences|transpose-sexps|transpose-subr-1|transpose-subr|transpose-words\n\t|tree-equal|tree-widget--locate-sub-directory|tree-widget-action|tree-widget-button-click|tree-widget-children-value-save|tree-widget-convert-widget\n\t|tree-widget-create-image|tree-widget-expander-p|tree-widget-find-image|tree-widget-help-echo|tree-widget-icon-action|tree-widget-icon-create\n\t|tree-widget-icon-help-echo|tree-widget-image-formats|tree-widget-image-properties|tree-widget-keep|tree-widget-leaf-node-icon-p\n\t|tree-widget-lookup-image|tree-widget-node|tree-widget-p|tree-widget-set-image-properties|tree-widget-set-parent-theme|tree-widget-set-theme\n\t|tree-widget-theme-name|tree-widget-themes-path|tree-widget-use-image-p|tree-widget-value-create|truncate\\*|truncated-partial-width-window-p\n\t|try-complete-file-name-partially|try-complete-file-name|try-complete-lisp-symbol-partially|try-complete-lisp-symbol|try-expand-all-abbrevs\n\t|try-expand-dabbrev-all-buffers|try-expand-dabbrev-from-kill|try-expand-dabbrev-visible|try-expand-dabbrev|try-expand-line-all-buffers\n\t|try-expand-line|try-expand-list-all-buffers|try-expand-list|try-expand-whole-kill|tty-color-by-index|tty-color-canonicalize|tty-color-desc\n\t|tty-color-gray-shades|tty-color-off-gray-diag|tty-color-standard-values|tty-color-values|tty-create-frame-with-faces|tty-display-color-cells\n\t|tty-display-color-p|tty-find-type|tty-handle-args|tty-handle-reverse-video|tty-modify-color-alist|tty-no-underline|tty-register-default-colors\n\t|tty-run-terminal-initialization|tty-set-up-initial-frame-faces|tty-suppress-bold-inverse-default-colors|tty-type|tumme|turkish-case-conversion-disable\n\t|turkish-case-conversion-enable|turn-off-auto-fill|turn-off-flyspell|turn-off-follow-mode|turn-off-hideshow|turn-off-iimage-mode\n\t|turn-off-xterm-mouse-tracking-on-terminal|turn-on-auto-fill|turn-on-auto-revert-mode|turn-on-auto-revert-tail-mode|turn-on-cwarn-mode-if-enabled\n\t|turn-on-cwarn-mode|turn-on-eldoc-mode|turn-on-flyspell|turn-on-follow-mode|turn-on-font-lock-if-desired|turn-on-font-lock|turn-on-gnus-dired-mode\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t turn-on-gnus-mailing-list-mode|turn-on-hi-lock-if-enabled|turn-on-iimage-mode|turn-on-org-cdlatex|turn-on-orgstruct\\+\\+|turn-on-orgstruct\n\t|turn-on-orgtbl|turn-on-prettify-symbols-mode|turn-on-reftex|turn-on-visual-line-mode|turn-on-xterm-mouse-tracking-on-terminal\n\t|type-break-alarm|type-break-cancel-function-timers|type-break-cancel-schedule|type-break-cancel-time-warning-schedule|type-break-catch-up-event\n\t|type-break-check-keystroke-warning|type-break-check-post-command-hook|type-break-check|type-break-choose-file|type-break-demo-boring\n\t|type-break-demo-hanoi|type-break-demo-life|type-break-do-query|type-break-file-keystroke-count|type-break-file-time|type-break-force-mode-line-update\n\t|type-break-format-time|type-break-get-previous-count|type-break-get-previous-time|type-break-guesstimate-keystroke-threshold\n\t|type-break-keystroke-reset|type-break-keystroke-warning|type-break-mode-line-countdown-or-break|type-break-mode-line-message-mode\n\t|type-break-mode|type-break-noninteractive-query|type-break-query-mode|type-break-query|type-break-run-at-time|type-break-run-tb-post-command-hook\n\t|type-break-schedule|type-break-statistics|type-break-time-difference|type-break-time-stamp|type-break-time-sum|type-break-time-warning-alarm\n\t|type-break-time-warning-schedule|type-break-time-warning|type-break|typecase|typep|uce-insert-ranting|uce-reply-to-uce|ucs-input-activate\n\t|ucs-insert|ucs-names|ucs-normalize-HFS-NFC-region|ucs-normalize-HFS-NFC-string|ucs-normalize-HFS-NFD-region|ucs-normalize-HFS-NFD-string\n\t|ucs-normalize-NFC-region|ucs-normalize-NFC-string|ucs-normalize-NFD-region|ucs-normalize-NFD-string|ucs-normalize-NFKC-region\n\t|ucs-normalize-NFKC-string|ucs-normalize-NFKD-region|ucs-normalize-NFKD-string|uncomment-region-default|uncomment-region|uncompface\n\t|underline-region|undigestify-rmail-message|undo-adjust-beg-end|undo-adjust-elt|undo-adjust-pos|undo-copy-list-1|undo-copy-list\n\t|undo-delta|undo-elt-crosses-region|undo-elt-in-region|undo-make-selective-list|undo-more|undo-only|undo-outer-limit-truncate|undo-start\n\t|undo|unencodable-char-position|unexpand-abbrev|unfocus-frame|unforward-rmail-message|unhighlight-regexp|unicode-property-table-internal\n\t|unify-8859-on-decoding-mode|unify-8859-on-encoding-mode|unify-charset|union|uniquify--create-file-buffer-advice|uniquify--rename-buffer-advice\n\t|uniquify-buffer-base-name|uniquify-buffer-file-name|uniquify-get-proposed-name|uniquify-item-base--cmacro|uniquify-item-base\n\t|uniquify-item-buffer--cmacro|uniquify-item-buffer|uniquify-item-dirname--cmacro|uniquify-item-dirname|uniquify-item-greaterp\n\t|uniquify-item-p--cmacro|uniquify-item-p|uniquify-item-proposed--cmacro|uniquify-item-proposed|uniquify-kill-buffer-function|uniquify-make-item--cmacro\n\t|uniquify-make-item|uniquify-maybe-rerationalize-w\\/o-cb|uniquify-rationalize-a-list|uniquify-rationalize-conflicting-sublist\n\t|uniquify-rationalize-file-buffer-names|uniquify-rationalize|uniquify-rename-buffer|uniquify-rerationalize-w\\/o-cb|uniquify-unload-function\n\t|universal-argument--mode|universal-argument-more|universal-coding-system-argument|unix-sync|unjustify-current-line|unjustify-region\n\t|unload--set-major-mode|unmorse-region|unmsys--file-name|unread-bib|unrecord-window-buffer|unrmail|unsafep-function|unsafep-let|unsafep-progn\n\t|unsafep-variable|untabify-backward|untabify|untrace-all|untrace-function|ununderline-region|up-ifdef|upcase-initials-region|update-glyphless-char-display\n\t|update-leim-list-file|url--allowed-chars|url-attributes--cmacro|url-attributes|url-auth-registered|url-auth-user-prompt|url-basepath\n\t|url-basic-auth|url-bit-for-url|url-build-query-string|url-cache-create-filename|url-cache-extract|url-cache-prune-cache|url-cid\n\t|url-completion-function|url-cookie-clean-up|url-cookie-create--cmacro|url-cookie-create|url-cookie-delete|url-cookie-domain--cmacro\n\t|url-cookie-domain|url-cookie-expired-p|url-cookie-expires--cmacro|url-cookie-expires|url-cookie-generate-header-lines|url-cookie-handle-set-cookie\n\t|url-cookie-host-can-set-p|url-cookie-list|url-cookie-localpart--cmacro|url-cookie-localpart|url-cookie-mode|url-cookie-name--cmacro\n\t|url-cookie-name|url-cookie-p--cmacro|url-cookie-p|url-cookie-parse-file|url-cookie-quit|url-cookie-retrieve|url-cookie-secure--cmacro\n\t|url-cookie-secure|url-cookie-setup-save-timer|url-cookie-store|url-cookie-value--cmacro|url-cookie-value|url-cookie-write-file\n\t|url-copy-file|url-data|url-dav-request|url-dav-supported-p|url-dav-vc-registered|url-debug|url-default-expander|url-default-find-proxy-for-url\n\t|url-device-type|url-digest-auth-create-key|url-digest-auth|url-display-percentage|url-do-auth-source-search|url-do-setup|url-domsuf-cookie-allowed-p\n\t|url-domsuf-parse-file|url-eat-trailing-space|url-encode-url|url-expand-file-name|url-expander-remove-relative-links|url-extract-mime-headers\n\t|url-file-directory|url-file-extension|url-file-handler|url-file-local-copy|url-file-nondirectory|url-file|url-filename--cmacro\n\t|url-filename|url-find-proxy-for-url|url-fullness--cmacro|url-fullness|url-gateway-nslookup-host|url-gc-dead-buffers|url-generate-unique-filename\n\t|url-generic-emulator-loader|url-generic-parse-url|url-get-authentication|url-get-normalized-date|url-get-url-at-point|url-handle-content-transfer-encoding\n\t|url-handler-mode|url-have-visited-url|url-hexify-string|url-history-parse-history|url-history-save-history|url-history-setup-save-timer\n\t|url-history-update-url|url-host--cmacro|url-host|url-http-activate-callback|url-http-async-sentinel|url-http-chunked-encoding-after-change-function\n\t|url-http-clean-headers|url-http-content-length-after-change-function|url-http-create-request|url-http-debug|url-http-end-of-document-sentinel\n\t|url-http-expand-file-name|url-http-file-attributes|url-http-file-exists-p|url-http-file-readable-p|url-http-find-free-connection\n\t|url-http-generic-filter|url-http-handle-authentication|url-http-handle-cookies|url-http-head-file-attributes|url-http-head|url-http-idle-sentinel\n\t|url-http-mark-connection-as-busy|url-http-mark-connection-as-free|url-http-options|url-http-parse-headers|url-http-parse-response\n\t|url-http-simple-after-change-function|url-http-symbol-value-in-buffer|url-http-user-agent-string|url-http-wait-for-headers-change-function\n\t|url-http|url-https-create-secure-wrapper|url-https-expand-file-name|url-https-file-attributes|url-https-file-exists-p|url-https-file-readable-p\n\t|url-https|url-identity-expander|url-info|url-insert-entities-in-string|url-insert-file-contents|url-irc|url-is-cached|url-lazy-message\n\t|url-ldap|url-mail|url-mailto|url-make-private-file|url-man|url-mark-buffer-as-dead|url-mime-charset-string|url-mm-callback|url-mm-url\n\t|url-news|url-normalize-url|url-ns-prefs|url-ns-user-pref|url-open-rlogin|url-open-stream|url-open-telnet|url-p--cmacro|url-p|url-parse-args\n\t|url-parse-make-urlobj--cmacro|url-parse-make-urlobj|url-parse-query-string|url-password--cmacro|url-password-for-url|url-password\n\t|url-path-and-query|url-percentage|url-port-if-non-default|url-port|url-portspec--cmacro|url-portspec|url-pretty-length|url-proxy\n\t|url-queue-buffer--cmacro|url-queue-buffer|url-queue-callback--cmacro|url-queue-callback-function|url-queue-callback|url-queue-cbargs--cmacro\n\t|url-queue-cbargs|url-queue-inhibit-cookiesp--cmacro|url-queue-inhibit-cookiesp|url-queue-kill-job|url-queue-p--cmacro|url-queue-p\n\t|url-queue-pre-triggered--cmacro|url-queue-pre-triggered|url-queue-prune-old-entries|url-queue-remove-jobs-from-host|url-queue-retrieve\n\t|url-queue-run-queue|url-queue-setup-runners|url-queue-silentp--cmacro|url-queue-silentp|url-queue-start-retrieve|url-queue-start-time--cmacro\n\t|url-queue-start-time|url-queue-url--cmacro|url-queue-url|url-recreate-url-attributes|url-recreate-url|url-register-auth-scheme\n\t|url-retrieve-internal|url-retrieve-synchronously|url-retrieve|url-rlogin|url-scheme-default-loader|url-scheme-get-property|url-scheme-register-proxy\n\t|url-set-mime-charset-string|url-setup-privacy-info|url-silent--cmacro|url-silent|url-snews|url-store-in-cache|url-strip-leading-spaces\n\t|url-target--cmacro|url-target|url-telnet|url-tn3270|url-tramp-file-handler|url-truncate-url-for-viewing|url-type--cmacro|url-type\n\t|url-unhex-string|url-unhex|url-use-cookies--cmacro|url-use-cookies|url-user--cmacro|url-user-for-url|url-user|url-view-url|url-wait-for-string\n\t|url-warn|use-cjk-char-width-table|use-completion-backward-under|use-completion-backward|use-completion-before-point|use-completion-before-separator\n\t|use-completion-minibuffer-separator|use-completion-under-or-before-point|use-completion-under-point|use-default-char-width-table\n\t|use-fancy-splash-screens-p|use-package|user-original-login-name|user-variable-p|utf-7-imap-post-read-conversion|utf-7-imap-pre-write-conversion\n\t|utf-7-post-read-conversion|utf-7-pre-write-conversion|utf7-decode|utf7-encode|uudecode-char-int|uudecode-decode-region-external\n\t|uudecode-decode-region-internal|uudecode-decode-region|uudecode-string-to-multibyte|values-list|variable-at-point|variable-binding-locus\n\t|variable-pitch-mode|vc--add-line|vc--process-sentinel|vc--read-lines|vc--remove-regexp|vc-after-save|vc-annotate|vc-backend-for-registration\n\t|vc-backend-subdirectory-name|vc-backend|vc-before-save|vc-branch-p|vc-branch-part|vc-buffer-context|vc-buffer-sync|vc-bzr-registered\n\t|vc-call-backend|vc-call|vc-check-headers|vc-check-master-templates|vc-checkin|vc-checkout-model|vc-checkout|vc-clear-context|vc-coding-system-for-diff\n\t|vc-comment-search-forward|vc-comment-search-reverse|vc-comment-to-change-log|vc-compatible-state|vc-compilation-mode|vc-context-matches-p\n\t|vc-create-repo|vc-create-tag|vc-cvs-after-dir-status|vc-cvs-annotate-command|vc-cvs-annotate-current-time|vc-cvs-annotate-extract-revision-at-line\n\t|vc-cvs-annotate-process-filter|vc-cvs-annotate-time|vc-cvs-append-to-ignore|vc-cvs-check-headers|vc-cvs-checkin|vc-cvs-checkout-model\n\t|vc-cvs-checkout|vc-cvs-command|vc-cvs-comment-history|vc-cvs-could-register|vc-cvs-create-tag|vc-cvs-delete-file|vc-cvs-diff|vc-cvs-dir-extra-headers\n\t|vc-cvs-dir-status-files|vc-cvs-dir-status-heuristic|vc-cvs-file-to-string|vc-cvs-find-admin-dir|vc-cvs-find-revision|vc-cvs-get-entries\n\t|vc-cvs-ignore|vc-cvs-make-version-backups-p|vc-cvs-merge-file|vc-cvs-merge-news|vc-cvs-merge|vc-cvs-mode-line-string|vc-cvs-modify-change-comment\n\t|vc-cvs-next-revision|vc-cvs-parse-entry|vc-cvs-parse-root|vc-cvs-parse-status|vc-cvs-parse-sticky-tag|vc-cvs-parse-uhp|vc-cvs-previous-revision\n\t|vc-cvs-print-log|vc-cvs-register|vc-cvs-registered|vc-cvs-repository-hostname|vc-cvs-responsible-p|vc-cvs-retrieve-tag|vc-cvs-revert\n\t|vc-cvs-revision-completion-table|vc-cvs-revision-granularity|vc-cvs-revision-table|vc-cvs-state-heuristic|vc-cvs-state|vc-cvs-stay-local-p\n\t|vc-cvs-update-changelog|vc-cvs-valid-revision-number-p|vc-cvs-valid-symbolic-tag-name-p|vc-cvs-working-revision|vc-deduce-backend\n\t|vc-deduce-fileset|vc-default-check-headers|vc-default-comment-history|vc-default-dir-status-files|vc-default-extra-menu|vc-default-find-file-hook\n\t|vc-default-find-revision|vc-default-ignore-completion-table|vc-default-ignore|vc-default-log-edit-mode|vc-default-log-view-mode\n\t|vc-default-make-version-backups-p|vc-default-mark-resolved|vc-default-mode-line-string|vc-default-receive-file|vc-default-registered\n\t|vc-default-rename-file|vc-default-responsible-p|vc-default-retrieve-tag|vc-default-revert|vc-default-revision-completion-table\n\t|vc-default-show-log-entry|vc-default-working-revision|vc-delete-automatic-version-backups|vc-delete-file|vc-delistify|vc-diff-build-argument-list-internal\n\t|vc-diff-finish|vc-diff-internal|vc-diff-switches-list|vc-diff|vc-dir-mode|vc-dir|vc-dired-deduce-fileset|vc-dispatcher-browsing\n\t|vc-do-async-command|vc-do-command|vc-ediff|vc-editable-p|vc-ensure-vc-buffer|vc-error-occurred|vc-exec-after|vc-expand-dirs|vc-file-clearprops\n\t|vc-file-getprop|vc-file-setprop|vc-file-tree-walk-internal|vc-file-tree-walk|vc-find-backend-function|vc-find-conflicted-file\n\t|vc-find-file-hook|vc-find-position-by-context|vc-find-revision|vc-find-root|vc-finish-logentry|vc-follow-link|vc-git-registered\n\t|vc-hg-registered|vc-ignore|vc-incoming-outgoing-internal|vc-insert-file|vc-insert-headers|vc-kill-buffer-hook|vc-log-edit|vc-log-incoming\n\t|vc-log-internal-common|vc-log-outgoing|vc-make-backend-sym|vc-make-version-backup|vc-mark-resolved|vc-maybe-resolve-conflicts\n\t|vc-menu-map-filter|vc-menu-map|vc-merge|vc-mode-line|vc-modify-change-comment|vc-mtn-registered|vc-next-action|vc-next-comment|vc-parse-buffer\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t vc-position-context|vc-possible-master|vc-previous-comment|vc-print-log-internal|vc-print-log-setup-buttons|vc-print-log|vc-print-root-log\n\t|vc-process-filter|vc-pull|vc-rcs-registered|vc-read-backend|vc-read-revision|vc-region-history|vc-register-with|vc-register|vc-registered\n\t|vc-rename-file|vc-resolve-conflicts|vc-responsible-backend|vc-restore-buffer-context|vc-resynch-buffer|vc-resynch-buffers-in-directory\n\t|vc-resynch-window|vc-retrieve-tag|vc-revert-buffer-internal|vc-revert-buffer|vc-revert-file|vc-revert|vc-revision-other-window\n\t|vc-rollback|vc-root-diff|vc-root-dir|vc-run-delayed|vc-sccs-registered|vc-sccs-search-project-dir|vc-set-async-update|vc-set-mode-line-busy-indicator\n\t|vc-setup-buffer|vc-src-registered|vc-start-logentry|vc-state-refresh|vc-state|vc-steal-lock|vc-string-prefix-p|vc-svn-registered\n\t|vc-switch-backend|vc-switches|vc-tag-precondition|vc-toggle-read-only|vc-transfer-file|vc-up-to-date-p|vc-update-change-log|vc-update\n\t|vc-user-login-name|vc-version-backup-file-name|vc-version-backup-file|vc-version-diff|vc-version-ediff|vc-workfile-version|vc-working-revision\n\t|vcursor-backward-char|vcursor-backward-word|vcursor-beginning-of-buffer|vcursor-beginning-of-line|vcursor-bind-keys|vcursor-check\n\t|vcursor-compare-windows|vcursor-copy-line|vcursor-copy-word|vcursor-copy|vcursor-cs-binding|vcursor-disable|vcursor-end-of-buffer\n\t|vcursor-end-of-line|vcursor-execute-command|vcursor-execute-key|vcursor-find-window|vcursor-forward-char|vcursor-forward-word\n\t|vcursor-get-char-count|vcursor-goto|vcursor-insert|vcursor-isearch-backward|vcursor-isearch-forward|vcursor-locate|vcursor-map\n\t|vcursor-move|vcursor-next-line|vcursor-other-window|vcursor-post-command|vcursor-previous-line|vcursor-relative-move|vcursor-scroll-down\n\t|vcursor-scroll-up|vcursor-swap-point|vcursor-toggle-copy|vcursor-toggle-vcursor-map|vcursor-use-vcursor-map|vcursor-window-funcall\n\t|vector-or-char-table-p|vendor-specific-keysyms|vera-add-syntax|vera-backward-same-indent|vera-backward-statement|vera-backward-syntactic-ws\n\t|vera-beginning-of-statement|vera-beginning-of-substatement|vera-comment-uncomment-region|vera-corresponding-begin|vera-corresponding-if\n\t|vera-customize|vera-electric-closing-brace|vera-electric-opening-brace|vera-electric-pound|vera-electric-return|vera-electric-slash\n\t|vera-electric-space|vera-electric-star|vera-electric-tab|vera-evaluate-offset|vera-expand-abbrev|vera-font-lock-match-item|vera-fontify-buffer\n\t|vera-forward-same-indent|vera-forward-statement|vera-forward-syntactic-ws|vera-get-offset|vera-guess-basic-syntax|vera-in-literal\n\t|vera-indent-block-closing|vera-indent-buffer|vera-indent-line|vera-indent-region|vera-langelem-col|vera-lineup-C-comments|vera-lineup-comment\n\t|vera-mode-menu|vera-mode|vera-point|vera-prepare-search|vera-re-search-backward|vera-re-search-forward|vera-skip-backward-literal\n\t|vera-skip-forward-literal|vera-submit-bug-report|vera-try-expand-abbrev|vera-version|verify-xscheme-buffer|verilog-add-list-unique\n\t|verilog-alw-get-inputs|verilog-alw-get-outputs-delayed|verilog-alw-get-outputs-immediate|verilog-alw-get-temps|verilog-alw-get-uses-delayed\n\t|verilog-alw-new|verilog-at-close-constraint-p|verilog-at-close-struct-p|verilog-at-constraint-p|verilog-at-struct-mv-p|verilog-at-struct-p\n\t|verilog-auto-arg-ports|verilog-auto-arg|verilog-auto-ascii-enum|verilog-auto-assign-modport|verilog-auto-inout-comp|verilog-auto-inout-in\n\t|verilog-auto-inout-modport|verilog-auto-inout-module|verilog-auto-inout-param|verilog-auto-inout|verilog-auto-input|verilog-auto-insert-last\n\t|verilog-auto-insert-lisp|verilog-auto-inst-first|verilog-auto-inst-param|verilog-auto-inst-port-list|verilog-auto-inst-port-map\n\t|verilog-auto-inst-port|verilog-auto-inst|verilog-auto-logic-setup|verilog-auto-logic|verilog-auto-output-every|verilog-auto-output\n\t|verilog-auto-re-search-do|verilog-auto-read-locals|verilog-auto-reeval-locals|verilog-auto-reg-input|verilog-auto-reg|verilog-auto-reset\n\t|verilog-auto-save-check|verilog-auto-save-compile|verilog-auto-sense-sigs|verilog-auto-sense|verilog-auto-star-safe|verilog-auto-star\n\t|verilog-auto-template-lint|verilog-auto-templated-rel|verilog-auto-tieoff|verilog-auto-undef|verilog-auto-unused|verilog-auto-wire\n\t|verilog-auto|verilog-back-to-start-translate-off|verilog-backward-case-item|verilog-backward-open-bracket|verilog-backward-open-paren\n\t|verilog-backward-sexp|verilog-backward-syntactic-ws-quick|verilog-backward-syntactic-ws|verilog-backward-token|verilog-backward-up-list\n\t|verilog-backward-ws&directives|verilog-batch-auto|verilog-batch-delete-auto|verilog-batch-delete-trailing-whitespace|verilog-batch-diff-auto\n\t|verilog-batch-error-wrapper|verilog-batch-execute-func|verilog-batch-indent|verilog-batch-inject-auto|verilog-beg-of-defun-quick\n\t|verilog-beg-of-defun|verilog-beg-of-statement-1|verilog-beg-of-statement|verilog-booleanp|verilog-build-defun-re|verilog-calc-1\n\t|verilog-calculate-indent-directive|verilog-calculate-indent|verilog-case-indent-level|verilog-clog2|verilog-colorize-include-files-buffer\n\t|verilog-comment-depth|verilog-comment-indent|verilog-comment-region|verilog-comp-defun|verilog-complete-word|verilog-completion-response\n\t|verilog-completion|verilog-continued-line-1|verilog-continued-line|verilog-current-flags|verilog-current-indent-level|verilog-customize\n\t|verilog-declaration-beg|verilog-declaration-end|verilog-decls-append|verilog-decls-get-assigns|verilog-decls-get-consts|verilog-decls-get-gparams\n\t|verilog-decls-get-inouts|verilog-decls-get-inputs|verilog-decls-get-interfaces|verilog-decls-get-iovars|verilog-decls-get-modports\n\t|verilog-decls-get-outputs|verilog-decls-get-ports|verilog-decls-get-signals|verilog-decls-get-vars|verilog-decls-new|verilog-decls-princ\n\t|verilog-define-abbrev|verilog-delete-auto-star-all|verilog-delete-auto-star-implicit|verilog-delete-auto|verilog-delete-autos-lined\n\t|verilog-delete-empty-auto-pair|verilog-delete-to-paren|verilog-delete-trailing-whitespace|verilog-diff-auto|verilog-diff-buffers-p\n\t|verilog-diff-file-with-buffer|verilog-diff-report|verilog-dir-file-exists-p|verilog-dir-files|verilog-do-indent|verilog-easy-menu-filter\n\t|verilog-end-of-defun|verilog-end-of-statement|verilog-end-translate-off|verilog-enum-ascii|verilog-error-regexp-add-emacs|verilog-expand-command\n\t|verilog-expand-dirnames|verilog-expand-vector-internal|verilog-expand-vector|verilog-faq|verilog-font-customize|verilog-font-lock-match-item\n\t|verilog-forward-close-paren|verilog-forward-or-insert-line|verilog-forward-sexp-cmt|verilog-forward-sexp-function|verilog-forward-sexp-ign-cmt\n\t|verilog-forward-sexp|verilog-forward-syntactic-ws|verilog-forward-ws&directives|verilog-func-completion|verilog-generate-numbers\n\t|verilog-get-completion-decl|verilog-get-default-symbol|verilog-get-end-of-defun|verilog-get-expr|verilog-get-lineup-indent-2\n\t|verilog-get-lineup-indent|verilog-getopt-file|verilog-getopt-flags|verilog-getopt|verilog-goto-defun-file|verilog-goto-defun|verilog-header\n\t|verilog-highlight-buffer|verilog-highlight-region|verilog-in-attribute-p|verilog-in-case-region-p|verilog-in-comment-or-string-p\n\t|verilog-in-comment-p|verilog-in-coverage-p|verilog-in-directive-p|verilog-in-escaped-name-p|verilog-in-fork-region-p|verilog-in-generate-region-p\n\t|verilog-in-parameter-p|verilog-in-paren-count|verilog-in-paren-quick|verilog-in-paren|verilog-in-parenthesis-p|verilog-in-slash-comment-p\n\t|verilog-in-star-comment-p|verilog-in-struct-nested-p|verilog-in-struct-p|verilog-indent-buffer|verilog-indent-comment|verilog-indent-declaration\n\t|verilog-indent-line-relative|verilog-indent-line|verilog-inject-arg|verilog-inject-auto|verilog-inject-inst|verilog-inject-sense\n\t|verilog-insert-1|verilog-insert-block|verilog-insert-date|verilog-insert-definition|verilog-insert-indent|verilog-insert-indices\n\t|verilog-insert-last-command-event|verilog-insert-one-definition|verilog-insert-year|verilog-insert|verilog-inside-comment-or-string-p\n\t|verilog-is-number|verilog-just-one-space|verilog-keyword-completion|verilog-kill-existing-comment|verilog-label-be|verilog-leap-to-case-head\n\t|verilog-leap-to-head|verilog-library-filenames|verilog-lint-off|verilog-linter-name|verilog-load-file-at-mouse|verilog-load-file-at-point\n\t|verilog-make-width-expression|verilog-mark-defun|verilog-match-translate-off|verilog-menu|verilog-mode|verilog-modi-cache-add-gparams\n\t|verilog-modi-cache-add-inouts|verilog-modi-cache-add-inputs|verilog-modi-cache-add-outputs|verilog-modi-cache-add-vars|verilog-modi-cache-add\n\t|verilog-modi-cache-results|verilog-modi-current-get|verilog-modi-current|verilog-modi-file-or-buffer|verilog-modi-filename|verilog-modi-get-decls\n\t|verilog-modi-get-point|verilog-modi-get-sub-decls|verilog-modi-get-type|verilog-modi-goto|verilog-modi-lookup|verilog-modi-modport-lookup-one\n\t|verilog-modi-modport-lookup|verilog-modi-name|verilog-modi-new|verilog-modify-compile-command|verilog-modport-clockings-add|verilog-modport-clockings\n\t|verilog-modport-decls-set|verilog-modport-decls|verilog-modport-name|verilog-modport-new|verilog-modport-princ|verilog-module-filenames\n\t|verilog-module-inside-filename-p|verilog-more-comment|verilog-one-line|verilog-parenthesis-depth|verilog-point-text|verilog-preprocess\n\t|verilog-preserve-dir-cache|verilog-preserve-modi-cache|verilog-pretty-declarations-auto|verilog-pretty-declarations|verilog-pretty-expr\n\t|verilog-re-search-backward-quick|verilog-re-search-backward-substr|verilog-re-search-backward|verilog-re-search-forward-quick\n\t|verilog-re-search-forward-substr|verilog-re-search-forward|verilog-read-always-signals-recurse|verilog-read-always-signals|verilog-read-arg-pins\n\t|verilog-read-auto-constants|verilog-read-auto-lisp-present|verilog-read-auto-lisp|verilog-read-auto-params|verilog-read-auto-template-hit\n\t|verilog-read-auto-template-middle|verilog-read-auto-template|verilog-read-decls|verilog-read-defines|verilog-read-includes|verilog-read-inst-backward-name\n\t|verilog-read-inst-module-matcher|verilog-read-inst-module|verilog-read-inst-name|verilog-read-inst-param-value|verilog-read-inst-pins\n\t|verilog-read-instants|verilog-read-module-name|verilog-read-signals|verilog-read-sub-decls-expr|verilog-read-sub-decls-gate|verilog-read-sub-decls-line\n\t|verilog-read-sub-decls-sig|verilog-read-sub-decls|verilog-regexp-opt|verilog-regexp-words|verilog-repair-close-comma|verilog-repair-open-comma\n\t|verilog-run-hooks|verilog-save-buffer-state|verilog-save-font-mods|verilog-save-no-change-functions|verilog-save-scan-cache|verilog-scan-and-debug\n\t|verilog-scan-cache-flush|verilog-scan-cache-ok-p|verilog-scan-debug|verilog-scan-region|verilog-scan|verilog-set-auto-endcomments\n\t|verilog-set-compile-command|verilog-set-define|verilog-show-completions|verilog-showscopes|verilog-sig-bits|verilog-sig-comment\n\t|verilog-sig-enum|verilog-sig-memory|verilog-sig-modport|verilog-sig-multidim-string|verilog-sig-multidim|verilog-sig-name|verilog-sig-new\n\t|verilog-sig-signed|verilog-sig-tieoff|verilog-sig-type-set|verilog-sig-type|verilog-sig-width|verilog-signals-combine-bus|verilog-signals-edit-wire-reg\n\t|verilog-signals-from-signame|verilog-signals-in|verilog-signals-matching-dir-re|verilog-signals-matching-enum|verilog-signals-matching-regexp\n\t|verilog-signals-memory|verilog-signals-not-in|verilog-signals-not-matching-regexp|verilog-signals-not-params|verilog-signals-princ\n\t|verilog-signals-sort-compare|verilog-signals-with|verilog-simplify-range-expression|verilog-sk-always|verilog-sk-assign|verilog-sk-begin\n\t|verilog-sk-case|verilog-sk-casex|verilog-sk-casez|verilog-sk-comment|verilog-sk-datadef|verilog-sk-def-reg|verilog-sk-define-signal\n\t|verilog-sk-else-if|verilog-sk-for|verilog-sk-fork|verilog-sk-function|verilog-sk-generate|verilog-sk-header-tmpl|verilog-sk-header\n\t|verilog-sk-if|verilog-sk-initial|verilog-sk-inout|verilog-sk-input|verilog-sk-module|verilog-sk-output|verilog-sk-ovm-class|verilog-sk-primitive\n\t|verilog-sk-prompt-clock|verilog-sk-prompt-condition|verilog-sk-prompt-inc|verilog-sk-prompt-init|verilog-sk-prompt-lsb|verilog-sk-prompt-msb\n\t|verilog-sk-prompt-name|verilog-sk-prompt-output|verilog-sk-prompt-reset|verilog-sk-prompt-state-selector|verilog-sk-prompt-width\n\t|verilog-sk-reg|verilog-sk-repeat|verilog-sk-specify|verilog-sk-state-machine|verilog-sk-task|verilog-sk-uvm-component|verilog-sk-uvm-object\n\t|verilog-sk-while|verilog-sk-wire|verilog-skip-backward-comment-or-string|verilog-skip-backward-comments|verilog-skip-forward-comment-or-string\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t verilog-skip-forward-comment-p|verilog-star-comment|verilog-start-translate-off|verilog-stmt-menu|verilog-string-diff|verilog-string-match-fold\n\t|verilog-string-remove-spaces|verilog-string-replace-matches|verilog-strip-comments|verilog-subdecls-get-inouts|verilog-subdecls-get-inputs\n\t|verilog-subdecls-get-interfaced|verilog-subdecls-get-interfaces|verilog-subdecls-get-outputs|verilog-subdecls-new|verilog-submit-bug-report\n\t|verilog-surelint-off|verilog-symbol-detick-denumber|verilog-symbol-detick-text|verilog-symbol-detick|verilog-syntax-ppss|verilog-typedef-name-p\n\t|verilog-uncomment-region|verilog-var-completion|verilog-verilint-off|verilog-version|verilog-wai|verilog-warn-error|verilog-warn\n\t|verilog-within-string|verilog-within-translate-off|version-list-<|version-list-<=|version-list-=|version-list-not-zero|version-to-list\n\t|version|version<|version<=|version=|vhdl-abbrev-list-init|vhdl-activate-customizations|vhdl-add-modified-file|vhdl-add-source-files-menu\n\t|vhdl-add-syntax|vhdl-adelete|vhdl-aget|vhdl-align-buffer|vhdl-align-declarations|vhdl-align-group|vhdl-align-inline-comment-buffer\n\t|vhdl-align-inline-comment-group|vhdl-align-inline-comment-region-1|vhdl-align-inline-comment-region|vhdl-align-list|vhdl-align-region-1\n\t|vhdl-align-region-2|vhdl-align-region-groups|vhdl-align-region|vhdl-align-same-indent|vhdl-aput-delete-if-nil|vhdl-aput|vhdl-auto-load-project\n\t|vhdl-back-to-indentation|vhdl-backward-same-indent|vhdl-backward-sexp|vhdl-backward-skip-label|vhdl-backward-syntactic-ws|vhdl-backward-to-block\n\t|vhdl-backward-up-list|vhdl-beautify-buffer|vhdl-beautify-region|vhdl-begin-p|vhdl-beginning-of-block|vhdl-beginning-of-defun|vhdl-beginning-of-libunit\n\t|vhdl-beginning-of-macro|vhdl-beginning-of-statement-1|vhdl-beginning-of-statement|vhdl-case-alternative-p|vhdl-case-keyword|vhdl-case-word\n\t|vhdl-character-to-event|vhdl-comment-append-inline|vhdl-comment-block|vhdl-comment-display-line|vhdl-comment-display|vhdl-comment-indent\n\t|vhdl-comment-insert-inline|vhdl-comment-insert|vhdl-comment-kill-inline-region|vhdl-comment-kill-region|vhdl-comment-uncomment-line\n\t|vhdl-comment-uncomment-region|vhdl-compile-directory|vhdl-compile-init|vhdl-compile-print-file-name|vhdl-compile|vhdl-compose-components-package\n\t|vhdl-compose-configuration-architecture|vhdl-compose-configuration|vhdl-compose-insert-generic|vhdl-compose-insert-port|vhdl-compose-insert-signal\n\t|vhdl-compose-new-component|vhdl-compose-place-component|vhdl-compose-wire-components|vhdl-corresponding-begin|vhdl-corresponding-defun\n\t|vhdl-corresponding-end|vhdl-corresponding-mid|vhdl-create-mode-menu|vhdl-current-line|vhdl-custom-set|vhdl-customize|vhdl-decision-query\n\t|vhdl-default-directory|vhdl-defun-p|vhdl-delete-indentation|vhdl-delete|vhdl-directory-files|vhdl-do-group|vhdl-do-list|vhdl-do-same-indent\n\t|vhdl-doc-mode|vhdl-doc-variable|vhdl-duplicate-project|vhdl-electric-close-bracket|vhdl-electric-comma|vhdl-electric-dash|vhdl-electric-equal\n\t|vhdl-electric-mode|vhdl-electric-open-bracket|vhdl-electric-period|vhdl-electric-quote|vhdl-electric-return|vhdl-electric-semicolon\n\t|vhdl-electric-space|vhdl-electric-tab|vhdl-end-of-block|vhdl-end-of-defun|vhdl-end-of-leader|vhdl-end-of-statement|vhdl-end-p|vhdl-end-translate-off\n\t|vhdl-error-regexp-add-emacs|vhdl-expand-abbrev|vhdl-expand-paren|vhdl-export-project|vhdl-fill-group|vhdl-fill-list|vhdl-fill-region\n\t|vhdl-fill-same-indent|vhdl-first-word|vhdl-fix-case-buffer|vhdl-fix-case-region-1|vhdl-fix-case-region|vhdl-fix-case-word|vhdl-fix-clause-buffer\n\t|vhdl-fix-clause|vhdl-fix-statement-buffer|vhdl-fix-statement-region|vhdl-fixup-whitespace-buffer|vhdl-fixup-whitespace-region\n\t|vhdl-font-lock-init|vhdl-font-lock-match-item|vhdl-fontify-buffer|vhdl-forward-comment|vhdl-forward-same-indent|vhdl-forward-sexp\n\t|vhdl-forward-skip-label|vhdl-forward-syntactic-ws|vhdl-function-name|vhdl-generate-makefile-1|vhdl-generate-makefile|vhdl-get-block-state\n\t|vhdl-get-compile-options|vhdl-get-components-package-name|vhdl-get-end-of-unit|vhdl-get-hierarchy|vhdl-get-instantiations|vhdl-get-library-unit\n\t|vhdl-get-make-options|vhdl-get-offset|vhdl-get-packages|vhdl-get-source-files|vhdl-get-subdirs|vhdl-get-syntactic-context|vhdl-get-visible-signals\n\t|vhdl-goto-marker|vhdl-has-syntax|vhdl-he-list-beg|vhdl-hideshow-init|vhdl-hooked-abbrev|vhdl-hs-forward-sexp-func|vhdl-hs-minor-mode\n\t|vhdl-import-project|vhdl-in-argument-list-p|vhdl-in-comment-p|vhdl-in-extended-identifier-p|vhdl-in-literal|vhdl-in-quote-p|vhdl-in-string-p\n\t|vhdl-indent-buffer|vhdl-indent-group|vhdl-indent-line|vhdl-indent-region|vhdl-indent-sexp|vhdl-index-menu-init|vhdl-insert-file-contents\n\t|vhdl-insert-keyword|vhdl-insert-string-or-file|vhdl-keep-region-active|vhdl-last-word|vhdl-libunit-p|vhdl-line-copy|vhdl-line-expand\n\t|vhdl-line-kill-entire|vhdl-line-kill|vhdl-line-open|vhdl-line-transpose-next|vhdl-line-transpose-previous|vhdl-line-yank|vhdl-lineup-arglist-intro\n\t|vhdl-lineup-arglist|vhdl-lineup-comment|vhdl-lineup-statement-cont|vhdl-load-cache|vhdl-make|vhdl-makefile-name|vhdl-mark-defun\n\t|vhdl-match-string-downcase|vhdl-match-translate-off|vhdl-max-marker|vhdl-menu-split|vhdl-minibuffer-tab|vhdl-mode-abbrev-table-init\n\t|vhdl-mode-map-init|vhdl-mode|vhdl-model-defun|vhdl-model-example-model|vhdl-model-insert|vhdl-model-map-init|vhdl-parse-group-comment\n\t|vhdl-parse-string|vhdl-paste-group-comment|vhdl-point|vhdl-port-copy|vhdl-port-flatten|vhdl-port-paste-component|vhdl-port-paste-constants\n\t|vhdl-port-paste-context-clause|vhdl-port-paste-declaration|vhdl-port-paste-entity|vhdl-port-paste-generic-map|vhdl-port-paste-generic\n\t|vhdl-port-paste-initializations|vhdl-port-paste-instance|vhdl-port-paste-port-map|vhdl-port-paste-port|vhdl-port-paste-signals\n\t|vhdl-port-paste-testbench|vhdl-port-reverse-direction|vhdl-prepare-search-1|vhdl-prepare-search-2|vhdl-print-warnings|vhdl-process-command-line-option\n\t|vhdl-project-p|vhdl-ps-print-init|vhdl-ps-print-settings|vhdl-re-search-backward|vhdl-re-search-forward|vhdl-read-offset|vhdl-regress-line\n\t|vhdl-remove-trailing-spaces-region|vhdl-remove-trailing-spaces|vhdl-replace-string|vhdl-require-hierarchy-info|vhdl-resolve-env-variable\n\t|vhdl-resolve-paths|vhdl-run-when-idle|vhdl-safe|vhdl-save-cache|vhdl-save-caches|vhdl-scan-context-clause|vhdl-scan-directory-contents\n\t|vhdl-scan-project-contents|vhdl-sequential-statement-p|vhdl-set-compiler|vhdl-set-default-project|vhdl-set-offset|vhdl-set-project\n\t|vhdl-set-style|vhdl-show-messages|vhdl-show-syntactic-information|vhdl-skip-case-alternative|vhdl-sort-alist|vhdl-speedbar-check-unit\n\t|vhdl-speedbar-configuration|vhdl-speedbar-contract-all|vhdl-speedbar-contract-level|vhdl-speedbar-dired|vhdl-speedbar-display-directory\n\t|vhdl-speedbar-display-projects|vhdl-speedbar-expand-all|vhdl-speedbar-expand-architecture|vhdl-speedbar-expand-config|vhdl-speedbar-expand-dirs\n\t|vhdl-speedbar-expand-entity|vhdl-speedbar-expand-package|vhdl-speedbar-expand-project|vhdl-speedbar-expand-units|vhdl-speedbar-find-file\n\t|vhdl-speedbar-generate-makefile|vhdl-speedbar-goto-this-unit|vhdl-speedbar-higher-text|vhdl-speedbar-initialize|vhdl-speedbar-insert-dir-hierarchy\n\t|vhdl-speedbar-insert-dirs|vhdl-speedbar-insert-hierarchy|vhdl-speedbar-insert-project-hierarchy|vhdl-speedbar-insert-projects\n\t|vhdl-speedbar-insert-subpackages|vhdl-speedbar-item-info|vhdl-speedbar-line-key|vhdl-speedbar-line-project|vhdl-speedbar-line-text\n\t|vhdl-speedbar-make-design|vhdl-speedbar-make-inst-line|vhdl-speedbar-make-pack-line|vhdl-speedbar-make-subpack-line|vhdl-speedbar-make-subprogram-line\n\t|vhdl-speedbar-make-title-line|vhdl-speedbar-place-component|vhdl-speedbar-port-copy|vhdl-speedbar-refresh|vhdl-speedbar-rescan-hierarchy\n\t|vhdl-speedbar-select-mra|vhdl-speedbar-set-depth|vhdl-speedbar-update-current-project|vhdl-speedbar-update-current-unit|vhdl-speedbar-update-units\n\t|vhdl-speedbar|vhdl-standard-p|vhdl-start-translate-off|vhdl-statement-p|vhdl-statistics-buffer|vhdl-stutter-mode|vhdl-submit-bug-report\n\t|vhdl-subprog-copy|vhdl-subprog-flatten|vhdl-subprog-paste-body|vhdl-subprog-paste-call|vhdl-subprog-paste-declaration|vhdl-subprog-paste-specification\n\t|vhdl-template-alias-hook|vhdl-template-alias|vhdl-template-and-hook|vhdl-template-architecture-hook|vhdl-template-architecture\n\t|vhdl-template-argument-list|vhdl-template-array|vhdl-template-assert-hook|vhdl-template-assert|vhdl-template-attribute-decl|vhdl-template-attribute-hook\n\t|vhdl-template-attribute-spec|vhdl-template-attribute|vhdl-template-bare-loop-hook|vhdl-template-bare-loop|vhdl-template-begin-end\n\t|vhdl-template-block-configuration|vhdl-template-block-hook|vhdl-template-block|vhdl-template-break-hook|vhdl-template-break|vhdl-template-case-hook\n\t|vhdl-template-case-is|vhdl-template-case-use|vhdl-template-case|vhdl-template-clocked-wait|vhdl-template-component-conf|vhdl-template-component-decl\n\t|vhdl-template-component-hook|vhdl-template-component-inst|vhdl-template-component|vhdl-template-conditional-signal-asst-hook\n\t|vhdl-template-conditional-signal-asst|vhdl-template-configuration-decl|vhdl-template-configuration-hook|vhdl-template-configuration-spec\n\t|vhdl-template-configuration|vhdl-template-constant-hook|vhdl-template-constant|vhdl-template-construct-alist-init|vhdl-template-default-hook\n\t|vhdl-template-default-indent-hook|vhdl-template-default-indent|vhdl-template-default|vhdl-template-directive-synthesis-off|vhdl-template-directive-synthesis-on\n\t|vhdl-template-directive-translate-off|vhdl-template-directive-translate-on|vhdl-template-directive|vhdl-template-disconnect-hook\n\t|vhdl-template-disconnect|vhdl-template-display-comment-hook|vhdl-template-else-hook|vhdl-template-else|vhdl-template-elsif-hook\n\t|vhdl-template-elsif|vhdl-template-entity-hook|vhdl-template-entity|vhdl-template-exit-hook|vhdl-template-exit|vhdl-template-field\n\t|vhdl-template-file-hook|vhdl-template-file|vhdl-template-footer|vhdl-template-for-generate|vhdl-template-for-hook|vhdl-template-for-loop\n\t|vhdl-template-for|vhdl-template-function-body|vhdl-template-function-decl|vhdl-template-function-hook|vhdl-template-function\n\t|vhdl-template-generate-body|vhdl-template-generate|vhdl-template-generic-hook|vhdl-template-generic-list|vhdl-template-generic\n\t|vhdl-template-group-decl|vhdl-template-group-hook|vhdl-template-group-template|vhdl-template-group|vhdl-template-header|vhdl-template-if-generate\n\t|vhdl-template-if-hook|vhdl-template-if-then-use|vhdl-template-if-then|vhdl-template-if-use|vhdl-template-if|vhdl-template-insert-construct\n\t|vhdl-template-insert-date|vhdl-template-insert-directive|vhdl-template-insert-fun|vhdl-template-insert-package|vhdl-template-instance-hook\n\t|vhdl-template-instance|vhdl-template-library-hook|vhdl-template-library|vhdl-template-limit-hook|vhdl-template-limit|vhdl-template-loop\n\t|vhdl-template-map-hook|vhdl-template-map-init|vhdl-template-map|vhdl-template-modify-noerror|vhdl-template-modify|vhdl-template-nand-hook\n\t|vhdl-template-nature-hook|vhdl-template-nature|vhdl-template-next-hook|vhdl-template-next|vhdl-template-nor-hook|vhdl-template-not-hook\n\t|vhdl-template-or-hook|vhdl-template-others-hook|vhdl-template-others|vhdl-template-package-alist-init|vhdl-template-package-body\n\t|vhdl-template-package-decl|vhdl-template-package-electrical-systems|vhdl-template-package-energy-systems|vhdl-template-package-fluidic-systems\n\t|vhdl-template-package-fundamental-constants|vhdl-template-package-hook|vhdl-template-package-material-constants|vhdl-template-package-math-complex\n\t|vhdl-template-package-math-real|vhdl-template-package-mechanical-systems|vhdl-template-package-numeric-bit|vhdl-template-package-numeric-std\n\t|vhdl-template-package-radiant-systems|vhdl-template-package-std-logic-1164|vhdl-template-package-std-logic-arith|vhdl-template-package-std-logic-misc\n\t|vhdl-template-package-std-logic-signed|vhdl-template-package-std-logic-textio|vhdl-template-package-std-logic-unsigned|vhdl-template-package-textio\n\t|vhdl-template-package-thermal-systems|vhdl-template-package|vhdl-template-paired-parens|vhdl-template-port-hook|vhdl-template-port-list\n\t|vhdl-template-port|vhdl-template-procedural-hook|vhdl-template-procedural|vhdl-template-procedure-body|vhdl-template-procedure-decl\n\t|vhdl-template-procedure-hook|vhdl-template-procedure|vhdl-template-process-comb|vhdl-template-process-hook|vhdl-template-process-seq\n\t|vhdl-template-process|vhdl-template-quantity-branch|vhdl-template-quantity-free|vhdl-template-quantity-hook|vhdl-template-quantity-source\n\t|vhdl-template-quantity|vhdl-template-record|vhdl-template-replace-header-keywords|vhdl-template-report-hook|vhdl-template-report\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t vhdl-template-return-hook|vhdl-template-return|vhdl-template-search-prompt|vhdl-template-selected-signal-asst-hook|vhdl-template-selected-signal-asst\n\t|vhdl-template-seq-process|vhdl-template-signal-hook|vhdl-template-signal|vhdl-template-standard-package|vhdl-template-subnature-hook\n\t|vhdl-template-subnature|vhdl-template-subprogram-body|vhdl-template-subprogram-decl|vhdl-template-subtype-hook|vhdl-template-subtype\n\t|vhdl-template-terminal-hook|vhdl-template-terminal|vhdl-template-type-hook|vhdl-template-type|vhdl-template-undo|vhdl-template-use-hook\n\t|vhdl-template-use|vhdl-template-variable-hook|vhdl-template-variable|vhdl-template-wait-hook|vhdl-template-wait|vhdl-template-when-hook\n\t|vhdl-template-when|vhdl-template-while-loop-hook|vhdl-template-while-loop|vhdl-template-with-hook|vhdl-template-with|vhdl-template-xnor-hook\n\t|vhdl-template-xor-hook|vhdl-toggle-project|vhdl-try-expand-abbrev|vhdl-uniquify|vhdl-upcase-list|vhdl-update-file-contents|vhdl-update-hierarchy\n\t|vhdl-update-mode-menu|vhdl-update-progress-info|vhdl-update-sensitivity-list-buffer|vhdl-update-sensitivity-list-process|vhdl-update-sensitivity-list\n\t|vhdl-use-direct-instantiation|vhdl-version|vhdl-visit-file|vhdl-warning-when-idle|vhdl-warning|vhdl-widget-directory-validate\n\t|vhdl-win-bsws|vhdl-win-fsws|vhdl-win-il|vhdl-within-translate-off|vhdl-words-init|vhdl-work-library|vhdl-write-file-hooks-init\n\t|viet-decode-viqr-buffer|viet-decode-viqr-region|viet-encode-viqr-buffer|viet-encode-viqr-region|viet-encode-viscii-char|view--disable\n\t|view--enable|view-buffer-other-frame|view-buffer-other-window|view-buffer|view-echo-area-messages|view-emacs-FAQ|view-emacs-debugging\n\t|view-emacs-news|view-emacs-problems|view-emacs-todo|view-end-message|view-external-packages|view-file-other-frame|view-file-other-window\n\t|view-file|view-hello-file|view-help-file|view-lossage|view-mode-disable|view-mode-enable|view-mode-enter|view-mode-exit|view-mode\n\t|view-order-manuals|view-page-size-default|view-really-at-end|view-recenter|view-return-to-alist-update|view-scroll-lines|view-search-no-match-lines\n\t|view-search|view-set-half-page-size-default|view-todo|view-window-size|viper--lookup-key|viper--tty-ESC-filter|viper-Append|viper-ESC-event-p\n\t|viper-ESC-keyseq-timeout|viper-ESC|viper-Insert|viper-Open-line|viper-P-val|viper-Put-back|viper-R-state-post-command-sentinel\n\t|viper-Region|viper-abbreviate-file-name|viper-abbreviate-string|viper-activate-input-method-action|viper-activate-input-method\n\t|viper-add-keymap|viper-add-local-keys|viper-add-newline-at-eob-if-necessary|viper-adjust-keys-for|viper-adjust-undo|viper-adjust-window\n\t|viper-after-change-sentinel|viper-after-change-undo-hook|viper-alist-to-list|viper-alternate-Meta-key|viper-append-filter-alist\n\t|viper-append-to-register|viper-append|viper-apply-major-mode-modifiers|viper-array-to-string|viper-ask-level|viper-autoindent\n\t|viper-backward-Word|viper-backward-char-carefully|viper-backward-char|viper-backward-indent|viper-backward-paragraph|viper-backward-sentence\n\t|viper-backward-word-kernel|viper-backward-word|viper-before-change-sentinel|viper-beginning-of-field|viper-beginning-of-line\n\t|viper-bind-mouse-insert-key|viper-bind-mouse-search-key|viper-bol-and-skip-white|viper-brac-function|viper-buffer-live-p|viper-buffer-search-enable\n\t|viper-can-release-key|viper-catch-tty-ESC|viper-change-cursor-color|viper-change-state-to-emacs|viper-change-state-to-insert\n\t|viper-change-state-to-replace|viper-change-state-to-vi|viper-change-state|viper-change-subr|viper-change-to-eol|viper-change|viper-char-array-p\n\t|viper-char-array-to-macro|viper-char-at-pos|viper-char-equal|viper-char-symbol-sequence-p|viper-characterp|viper-charlist-to-string\n\t|viper-charpair-command-p|viper-chars-in-region|viper-check-minibuffer-overlay|viper-check-version|viper-cleanup-ring|viper-color-defined-p\n\t|viper-color-display-p|viper-comint-mode-hook|viper-command-argument|viper-common-seq-prefix|viper-complete-filename-or-exit|viper-copy-event\n\t|viper-copy-region-as-kill|viper-current-ring-item|viper-cycle-through-mark-ring|viper-deactivate-input-method-action|viper-deactivate-input-method\n\t|viper-deactivate-mark|viper-debug-keymaps|viper-default-ex-addresses|viper-deflocalvar|viper-del-backward-char-in-insert|viper-del-backward-char-in-replace\n\t|viper-del-forward-char-in-insert|viper-delete-backward-char|viper-delete-backward-word|viper-delete-char|viper-delocalize-var\n\t|viper-describe-arg|viper-describe-kbd-macros|viper-describe-one-macro-elt|viper-describe-one-macro|viper-device-type|viper-digit-argument\n\t|viper-digit-command-p|viper-display-current-destructive-command|viper-display-macro|viper-display-vector-completions|viper-do-sequence-completion\n\t|viper-dotable-command-p|viper-downgrade-to-insert|viper-end-mapping-kbd-macro|viper-end-of-Word|viper-end-of-word-kernel|viper-end-of-word-p\n\t|viper-end-of-word|viper-end-with-a-newline-p|viper-enlarge-region|viper-erase-line|viper-escape-to-emacs|viper-escape-to-state\n\t|viper-escape-to-vi|viper-event-click-count|viper-event-key|viper-event-vector-p|viper-eventify-list-xemacs|viper-events-to-macro\n\t|viper-ex-read-file-name|viper-ex|viper-exchange-point-and-mark|viper-exec-Change|viper-exec-Delete|viper-exec-Yank|viper-exec-bang\n\t|viper-exec-buffer-search|viper-exec-change|viper-exec-delete|viper-exec-dummy|viper-exec-equals|viper-exec-form-in-emacs|viper-exec-form-in-vi\n\t|viper-exec-key-in-emacs|viper-exec-mapped-kbd-macro|viper-exec-shift|viper-exec-yank|viper-execute-com|viper-exit-insert-state\n\t|viper-exit-minibuffer|viper-extract-matching-alist-members|viper-fast-keysequence-p|viper-file-add-suffix|viper-file-checked-in-p\n\t|viper-filter-alist|viper-filter-list|viper-find-best-matching-macro|viper-find-char-backward|viper-find-char-forward|viper-find-char\n\t|viper-finish-R-mode|viper-finish-change|viper-fixup-macro|viper-flash-search-pattern|viper-forward-Word|viper-forward-char-carefully\n\t|viper-forward-char|viper-forward-indent|viper-forward-paragraph|viper-forward-sentence|viper-forward-word-kernel|viper-forward-word\n\t|viper-frame-value|viper-get-cursor-color|viper-get-ex-address-subr|viper-get-ex-address|viper-get-ex-buffer|viper-get-ex-com-subr\n\t|viper-get-ex-count|viper-get-ex-file|viper-get-ex-opt-gc|viper-get-ex-pat|viper-get-ex-token|viper-get-face|viper-get-filenames-from-buffer\n\t|viper-get-saved-cursor-color-in-emacs-mode|viper-get-saved-cursor-color-in-insert-mode|viper-get-saved-cursor-color-in-replace-mode\n\t|viper-get-visible-buffer-window|viper-getCom|viper-getcom|viper-glob-mswindows-files|viper-glob-unix-files|viper-global-execute\n\t|viper-go-away|viper-goto-char-backward|viper-goto-char-forward|viper-goto-col|viper-goto-eol|viper-goto-line|viper-goto-mark-and-skip-white\n\t|viper-goto-mark-subr|viper-goto-mark|viper-handle-!|viper-harness-minor-mode|viper-has-face-support-p|viper-hash-command-p|viper-heading-end\n\t|viper-hide-replace-overlay|viper-hide-search-overlay|viper-iconify|viper-if-string|viper-indent-line|viper-info-on-file|viper-insert-isearch-string\n\t|viper-insert-next-from-insertion-ring|viper-insert-prev-from-insertion-ring|viper-insert-state-post-command-sentinel|viper-insert-state-pre-command-sentinel\n\t|viper-insert-tab|viper-insert|viper-int-to-char|viper-intercept-ESC-key|viper-is-in-minibuffer|viper-isearch-backward|viper-isearch-forward\n\t|viper-join-lines|viper-kbd-buf-alist|viper-kbd-buf-definition|viper-kbd-buf-pair|viper-kbd-global-definition|viper-kbd-global-pair\n\t|viper-kbd-mode-alist|viper-kbd-mode-definition|viper-kbd-mode-pair|viper-ket-function|viper-key-press-events-to-chars|viper-key-to-character\n\t|viper-key-to-emacs-key|viper-keyseq-is-a-possible-macro|viper-kill-buffer|viper-kill-line|viper-last-command-char|viper-leave-region-active\n\t|viper-line-pos|viper-line-to-bottom|viper-line-to-middle|viper-line-to-top|viper-line|viper-list-to-alist|viper-load-custom-file\n\t|viper-looking-at-alpha|viper-looking-at-alphasep|viper-looking-at-separator|viper-looking-back|viper-loop|viper-macro-to-events\n\t|viper-major-mode-change-sentinel|viper-make-overlay|viper-mark-beginning-of-buffer|viper-mark-end-of-buffer|viper-mark-marker\n\t|viper-mark-point|viper-maybe-checkout|viper-memq-char|viper-message-conditions|viper-minibuffer-post-command-hook|viper-minibuffer-real-start\n\t|viper-minibuffer-setup-sentinel|viper-minibuffer-standard-hook|viper-minibuffer-trim-tail|viper-mode|viper-modify-keymap|viper-modify-major-mode\n\t|viper-mouse-catch-frame-switch|viper-mouse-click-frame|viper-mouse-click-get-word|viper-mouse-click-insert-word|viper-mouse-click-posn\n\t|viper-mouse-click-search-word|viper-mouse-click-window-buffer-name|viper-mouse-click-window-buffer|viper-mouse-click-window\n\t|viper-mouse-event-p|viper-move-marker-locally|viper-move-overlay|viper-move-replace-overlay|viper-movement-command-p|viper-multiclick-p\n\t|viper-next-destructive-command|viper-next-heading|viper-next-line-at-bol|viper-next-line-carefully|viper-next-line|viper-nil|viper-non-hook-settings\n\t|viper-normalize-minor-mode-map-alist|viper-open-line-at-point|viper-open-line|viper-over-whitespace-line|viper-overlay-end|viper-overlay-get\n\t|viper-overlay-live-p|viper-overlay-p|viper-overlay-put|viper-overlay-start|viper-overwrite|viper-p-val|viper-paren-match|viper-parse-mouse-key\n\t|viper-pos-within-region|viper-post-command-sentinel|viper-pre-command-sentinel|viper-prefix-arg-com|viper-prefix-arg-value|viper-prefix-command-p\n\t|viper-prefix-subseq-p|viper-preserve-cursor-color|viper-prev-destructive-command|viper-prev-heading|viper-previous-line-at-bol\n\t|viper-previous-line|viper-push-onto-ring|viper-put-back|viper-put-on-search-overlay|viper-put-string-on-kill-ring|viper-query-replace\n\t|viper-quote-region|viper-read-char-exclusive|viper-read-event-convert-to-char|viper-read-event|viper-read-fast-keysequence|viper-read-key-sequence\n\t|viper-read-key|viper-read-string-with-history|viper-record-kbd-macro|viper-refresh-mode-line|viper-region|viper-register-macro\n\t|viper-register-to-point|viper-regsuffix-command-p|viper-remember-current-frame|viper-remove-hooks|viper-repeat-find-opposite\n\t|viper-repeat-find|viper-repeat-from-history|viper-repeat-insert-command|viper-repeat|viper-replace-char-subr|viper-replace-char\n\t|viper-replace-end|viper-replace-mode-spy-after|viper-replace-mode-spy-before|viper-replace-start|viper-replace-state-carriage-return\n\t|viper-replace-state-exit-cmd|viper-replace-state-post-command-sentinel|viper-replace-state-pre-command-sentinel|viper-reset-mouse-insert-key\n\t|viper-reset-mouse-search-key|viper-restore-cursor-color|viper-restore-cursor-type|viper-ring-insert|viper-ring-pop|viper-ring-rotate1\n\t|viper-same-line|viper-save-cursor-color|viper-save-kill-buffer|viper-save-last-insertion|viper-save-setting|viper-save-string-in-file\n\t|viper-scroll-down-one|viper-scroll-down|viper-scroll-screen-back|viper-scroll-screen|viper-scroll-up-one|viper-scroll-up|viper-search-Next\n\t|viper-search-backward|viper-search-forward|viper-search-next|viper-search|viper-separator-skipback-special|viper-seq-last-elt\n\t|viper-set-complex-command-for-undo|viper-set-cursor-color-according-to-state|viper-set-destructive-command|viper-set-emacs-state-searchstyle-macros\n\t|viper-set-expert-level|viper-set-hooks|viper-set-input-method|viper-set-insert-cursor-type|viper-set-iso-accents-mode|viper-set-mark-if-necessary\n\t|viper-set-minibuffer-overlay|viper-set-minibuffer-style|viper-set-mode-vars-for|viper-set-parsing-style-toggling-macro|viper-set-register-macro\n\t|viper-set-replace-overlay-glyphs|viper-set-replace-overlay|viper-set-searchstyle-toggling-macros|viper-set-syntax-preference\n\t|viper-set-unread-command-events|viper-setup-ESC-to-escape|viper-setup-master-buffer|viper-sit-for-short|viper-skip-all-separators-backward\n\t|viper-skip-all-separators-forward|viper-skip-alpha-backward|viper-skip-alpha-forward|viper-skip-nonalphasep-backward|viper-skip-nonalphasep-forward\n\t|viper-skip-nonseparators|viper-skip-separators|viper-skip-syntax|viper-special-prefix-com|viper-special-read-and-insert-char\n\t|viper-special-ring-rotate1|viper-standard-value|viper-start-R-mode|viper-start-replace|viper-string-to-list|viper-submit-report\n\t|viper-subseq|viper-substitute-line|viper-substitute|viper-surrounding-word|viper-switch-to-buffer-other-window|viper-switch-to-buffer\n\t|viper-test-com-defun|viper-this-buffer-macros|viper-tmp-insert-at-eob|viper-toggle-case|viper-toggle-key-action|viper-toggle-parse-sexp-ignore-comments\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t viper-toggle-search-style|viper-translate-all-ESC-keysequences|viper-trim-replace-chars-to-delete-if-necessary|viper-unbind-mouse-insert-key\n\t|viper-unbind-mouse-search-key|viper-uncatch-tty-ESC|viper-undisplayed-files|viper-undo-more|viper-undo-sentinel|viper-undo|viper-unrecord-kbd-macro\n\t|viper-update-syntax-classes|viper-valid-marker|viper-valid-register|viper-version|viper-vi-command-p|viper-wildcard-to-regexp\n\t|viper-window-bottom|viper-window-display-p|viper-window-middle|viper-window-top|viper-yank-defun|viper-yank-last-insertion|viper-yank-line\n\t|viper-yank|viper-zap-local-keys|viper=|viqr-post-read-conversion|viqr-pre-write-conversion|visible-mode|visit-tags-table-buffer\n\t|visit-tags-table|visual-line-mode-set-explicitly|visual-line-mode|vt-keypad-off|vt-keypad-on|vt-narrow|vt-numlock|vt-toggle-screen\n\t|vt-wide|walk-window-subtree|walk-window-tree-1|walk-window-tree|warn-maybe-out-of-memory|warning-numeric-level|warning-suppress-p\n\t|wdired-abort-changes|wdired-capitalize-word|wdired-change-to-dired-mode|wdired-change-to-wdired-mode|wdired-check-kill-buffer\n\t|wdired-customize|wdired-do-perm-changes|wdired-do-renames|wdired-do-symlink-changes|wdired-downcase-word|wdired-exit|wdired-finish-edit\n\t|wdired-flag-for-deletion|wdired-get-filename|wdired-get-previous-link|wdired-isearch-filter-read-only|wdired-mode|wdired-mouse-toggle-bit\n\t|wdired-next-line|wdired-normalize-filename|wdired-perm-allowed-in-pos|wdired-perms-to-number|wdired-preprocess-files|wdired-preprocess-perms\n\t|wdired-preprocess-symlinks|wdired-previous-line|wdired-revert|wdired-search-and-rename|wdired-set-bit|wdired-toggle-bit|wdired-upcase-word\n\t|wdired-xcase-word|webjump-builtin-check-args|webjump-builtin|webjump-choose-mirror|webjump-do-simple-query|webjump-mirror-default\n\t|webjump-null-or-blank-string-p|webjump-read-choice|webjump-read-number|webjump-read-string|webjump-read-url-choice|webjump-to-iwin\n\t|webjump-to-risks|webjump-url-encode|webjump-url-fix-trailing-slash|webjump-url-fix|webjump|what-cursor-position|what-domain|what-line\n\t|what-page|when-let|where-is|which-func-ff-hook|which-func-mode|which-func-update-1|which-func-update-ediff-windows|which-func-update\n\t|which-function-mode|which-function|whitespace-action-when-on|whitespace-buffer-changed|whitespace-char-valid-p|whitespace-cleanup-region\n\t|whitespace-cleanup|whitespace-color-off|whitespace-color-on|whitespace-display-char-off|whitespace-display-char-on|whitespace-display-vector-p\n\t|whitespace-display-window|whitespace-empty-at-bob-regexp|whitespace-empty-at-eob-regexp|whitespace-ensure-local-variables|whitespace-help-off\n\t|whitespace-help-on|whitespace-help-scroll|whitespace-indentation-regexp|whitespace-insert-option-mark|whitespace-insert-value\n\t|whitespace-interactive-char|whitespace-kill-buffer|whitespace-looking-back|whitespace-mark-x|whitespace-mode|whitespace-newline-mode\n\t|whitespace-point--flush-used|whitespace-point--used|whitespace-post-command-hook|whitespace-regexp|whitespace-replace-action\n\t|whitespace-report-region|whitespace-report|whitespace-space-after-tab-regexp|whitespace-style-face-p|whitespace-style-mark-p\n\t|whitespace-toggle-list|whitespace-toggle-options|whitespace-trailing-regexp|whitespace-turn-off|whitespace-turn-on-if-enabled\n\t|whitespace-turn-on|whitespace-unload-function|whitespace-warn-read-only|whitespace-write-file-hook|whois-get-tld|whois-reverse-lookup\n\t|whois|widget-add-change|widget-add-documentation-string-button|widget-after-change|widget-alist-convert-option|widget-alist-convert-widget\n\t|widget-apply-action|widget-apply|widget-at|widget-backward|widget-before-change|widget-beginning-of-line|widget-boolean-prompt-value\n\t|widget-browse-at|widget-browse-other-window|widget-browse|widget-button-click|widget-button-press|widget-button-release-event-p\n\t|widget-checkbox-action|widget-checklist-add-item|widget-checklist-match-find|widget-checklist-match-inline|widget-checklist-match-up\n\t|widget-checklist-match|widget-checklist-validate|widget-checklist-value-create|widget-checklist-value-get|widget-child-validate\n\t|widget-child-value-get|widget-child-value-inline|widget-children-validate|widget-children-value-delete|widget-choice-action|widget-choice-default-get\n\t|widget-choice-match-inline|widget-choice-match|widget-choice-mouse-down-action|widget-choice-prompt-value|widget-choice-validate\n\t|widget-choice-value-create|widget-choose|widget-clear-undo|widget-coding-system-action|widget-coding-system-prompt-value|widget-color--choose-action\n\t|widget-color-action|widget-color-notify|widget-color-sample-face-get|widget-color-value-create|widget-complete|widget-completions-at-point\n\t|widget-cons-match|widget-const-prompt-value|widget-convert-button|widget-convert-text|widget-convert|widget-copy|widget-create-child-and-convert\n\t|widget-create-child-value|widget-create-child|widget-create|widget-default-action|widget-default-active|widget-default-button-face-get\n\t|widget-default-completions|widget-default-create|widget-default-deactivate|widget-default-default-get|widget-default-delete|widget-default-format-handler\n\t|widget-default-get|widget-default-menu-tag-get|widget-default-mouse-face-get|widget-default-notify|widget-default-prompt-value\n\t|widget-default-sample-face-get|widget-default-value-inline|widget-default-value-set|widget-delete-button-action|widget-delete\n\t|widget-docstring|widget-documentation-link-action|widget-documentation-link-add|widget-documentation-string-action|widget-documentation-string-indent-to\n\t|widget-documentation-string-value-create|widget-echo-help|widget-editable-list-delete-at|widget-editable-list-entry-create|widget-editable-list-format-handler\n\t|widget-editable-list-insert-before|widget-editable-list-match-inline|widget-editable-list-match|widget-editable-list-value-create\n\t|widget-editable-list-value-get|widget-emacs-commentary-link-action|widget-emacs-library-link-action|widget-end-of-line|widget-event-point\n\t|widget-face-notify|widget-face-sample-face-get|widget-field-action|widget-field-activate|widget-field-at|widget-field-buffer|widget-field-end\n\t|widget-field-find|widget-field-match|widget-field-prompt-internal|widget-field-prompt-value|widget-field-start|widget-field-text-end\n\t|widget-field-validate|widget-field-value-create|widget-field-value-delete|widget-field-value-get|widget-field-value-set|widget-file-link-action\n\t|widget-file-prompt-value|widget-forward|widget-function-link-action|widget-get-indirect|widget-get-sibling|widget-get|widget-group-default-get\n\t|widget-group-match-inline|widget-group-match|widget-group-value-create|widget-image-find|widget-image-insert|widget-info-link-action\n\t|widget-insert-button-action|widget-insert|widget-item-action|widget-item-match-inline|widget-item-match|widget-item-value-create\n\t|widget-key-sequence-read-event|widget-key-sequence-validate|widget-key-sequence-value-to-external|widget-key-sequence-value-to-internal\n\t|widget-kill-line|widget-leave-text|widget-magic-mouse-down-action|widget-map-buttons|widget-match-inline|widget-member|widget-minor-mode\n\t|widget-mouse-help|widget-move-and-invoke|widget-move|widget-narrow-to-field|widget-overlay-inactive|widget-parent-action|widget-plist-convert-option\n\t|widget-plist-convert-widget|widget-plist-member|widget-princ-to-string|widget-prompt-value|widget-push-button-value-create|widget-put\n\t|widget-radio-action|widget-radio-add-item|widget-radio-button-notify|widget-radio-chosen|widget-radio-validate|widget-radio-value-create\n\t|widget-radio-value-get|widget-radio-value-inline|widget-radio-value-set|widget-regexp-match|widget-regexp-validate|widget-restricted-sexp-match\n\t|widget-setup|widget-sexp-prompt-value|widget-sexp-validate|widget-sexp-value-to-internal|widget-specify-active|widget-specify-button\n\t|widget-specify-doc|widget-specify-field|widget-specify-inactive|widget-specify-insert|widget-specify-sample|widget-specify-secret\n\t|widget-sublist|widget-symbol-prompt-internal|widget-tabable-at|widget-toggle-action|widget-toggle-value-create|widget-type-default-get\n\t|widget-type-match|widget-type-value-create|widget-type|widget-types-convert-widget|widget-types-copy|widget-url-link-action|widget-value-convert-widget\n\t|widget-value-set|widget-value-value-get|widget-value|widget-variable-link-action|widget-vector-match|widget-visibility-value-create\n\t|widgetp|wildcard-to-regexp|windmove-constrain-around-range|windmove-constrain-loc-for-movement|windmove-constrain-to-range|windmove-coord-add\n\t|windmove-default-keybindings|windmove-do-window-select|windmove-down|windmove-find-other-window|windmove-frame-edges|windmove-left\n\t|windmove-other-window-loc|windmove-reference-loc|windmove-right|windmove-up|windmove-wrap-loc-for-movement|window--atom-check-1\n\t|window--atom-check|window--check|window--delete|window--display-buffer|window--dump-frame|window--dump-window|window--even-window-heights\n\t|window--frame-usable-p|window--in-direction-2|window--in-subtree-p|window--major-non-side-window|window--major-side-window|window--max-delta-1\n\t|window--maybe-raise-frame|window--min-delta-1|window--min-size-1|window--min-size-ignore-p|window--pixel-to-total-1|window--pixel-to-total\n\t|window--preservable-size|window--preserve-size|window--resizable-p|window--resizable|window--resize-apply-p|window--resize-child-windows-normal\n\t|window--resize-child-windows-skip-p|window--resize-child-windows|window--resize-mini-window|window--resize-reset-1|window--resize-reset\n\t|window--resize-root-window-vertically|window--resize-root-window|window--resize-siblings|window--resize-this-window|window--sanitize-margin\n\t|window--sanitize-window-sizes|window--side-check|window--side-window-p|window--size-fixed-1|window--size-ignore-p|window--size-to-pixel\n\t|window--state-get-1|window--state-put-1|window--state-put-2|window--subtree|window--try-to-split-window|window-at-side-list|window-at-side-p\n\t|window-atom-root|window-buffer-height|window-child-count|window-combination-p|window-combinations|window-configuration-to-register\n\t|window-deletable-p|window-dot|window-fixed-size-p|window-height|window-last-child|window-left|window-list-1|window-make-atom|window-max-delta\n\t|window-min-delta|window-min-pixel-height|window-min-pixel-size|window-min-pixel-width|window-new-normal|window-new-pixel|window-new-total\n\t|window-normal-size|window-normalize-buffer-to-switch-to|window-normalize-buffer|window-normalize-frame|window-normalize-window\n\t|window-old-point|window-preserve-size|window-preserved-size|window-redisplay-end-trigger|window-resizable-p|window-resize-apply-total\n\t|window-resize-apply|window-resize-no-error|window-right|window-safe-min-pixel-height|window-safe-min-pixel-size|window-safe-min-pixel-width\n\t|window-safe-min-size|window-safely-shrinkable-p|window-screen-lines|window-scroll-bar-height|window-sizable-p|window-sizable|window-size-fixed-p\n\t|window-size|window-splittable-p|window-system-for-display|window-text-height|window-text-width|window-use-time|window-width|window-with-parameter\n\t|winner-active-region|winner-change-fun|winner-conf|winner-configuration|winner-edges|winner-equal|winner-get-point|winner-insert-if-new\n\t|winner-make-point-alist|winner-mode|winner-redo|winner-remember|winner-ring|winner-save-conditionally|winner-save-old-configurations\n\t|winner-save-unconditionally|winner-set-conf|winner-set|winner-sorted-window-list|winner-undo-this|winner-undo|winner-win-data|winner-window-list\n\t|wisent-grammar-mode|wisent-java-default-setup|wisent-javascript-setup-parser|wisent-python-default-setup|with-auto-compression-mode\n\t|with-buffer-modified-unmodified|with-category-table|with-decoded-time-value|with-displayed-buffer-window|with-electric-help|with-file-modes\n\t|with-isearch-suspended|with-js|with-mh-folder-updating|with-mode-local-symbol|with-mode-local|with-parsed-tramp-file-name|with-rcirc-process-buffer\n\t|with-rcirc-server-buffer|with-selected-frame|with-silent-modifications|with-slots|with-timeout-suspend|with-timeout-unsuspend\n\t|with-tramp-connection-property|with-tramp-file-property|with-tramp-progress-reporter|with-vc-properties|with-wrapper-hook|woman-Cyg-to-Win\n\t|woman-bookmark-jump|woman-bookmark-make-record|woman-break-table|woman-cached-data|woman-canonicalize-dir|woman-change-fonts|woman-decode-buffer\n\t|woman-decode-region|woman-default-faces|woman-delete-following-space|woman-delete-line|woman-delete-match|woman-delete-whole-line\n\t|woman-directory-files|woman-dired-define-key-maybe|woman-dired-define-key|woman-dired-define-keys|woman-dired-find-file|woman-display-extended-fonts\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        },
        {
          match:
            '(?x)(?<=[()]|^)(?:\n\t woman-expand-directory-path|woman-expand-locale|woman-file-accessible-directory-p|woman-file-name-all-completions|woman-file-name\n\t|woman-file-readable-p|woman-find-file|woman-find-next-control-line-carefully|woman-find-next-control-line|woman-follow-word|woman-follow\n\t|woman-forward-arg|woman-get-next-char|woman-get-numeric-arg|woman-get-tab-stop|woman-horizontal-escapes|woman-horizontal-line\n\t|woman-if-body|woman-if-ignore|woman-imenu|woman-insert-file-contents|woman-interparagraph-space|woman-interpolate-macro|woman-leave-blank-lines\n\t|woman-make-bufname|woman-man-buffer|woman-manpath-add-locales|woman-mark-horizontal-position|woman-match-name|woman-menu|woman-mini-help\n\t|woman-mode|woman-monochrome-faces|woman-negative-vertical-space|woman-non-underline-faces|woman-not-member|woman-parse-colon-path\n\t|woman-parse-man\\.conf|woman-parse-numeric-arg|woman-parse-numeric-value|woman-pop|woman-pre-process-region|woman-process-buffer\n\t|woman-push|woman-read-directory-cache|woman-really-find-file|woman-reformat-last-file|woman-replace-match|woman-reset-emulation\n\t|woman-reset-nospace|woman-select-symbol-fonts|woman-select|woman-set-arg|woman-set-buffer-display-table|woman-set-face|woman-set-interparagraph-distance\n\t|woman-special-characters|woman-strings|woman-tab-to-tab-stop|woman-tar-extract-file|woman-toggle-fill-frame|woman-toggle-use-extended-font\n\t|woman-toggle-use-symbol-font|woman-topic-all-completions-1|woman-topic-all-completions-merge|woman-topic-all-completions|woman-translate\n\t|woman-unescape|woman-unquote-args|woman-unquote|woman-write-directory-cache|woman|woman0-de|woman0-el|woman0-if|woman0-ig|woman0-macro\n\t|woman0-process-escapes|woman0-rename|woman0-rn|woman0-roff-buffer|woman0-so|woman1-B-or-I|woman1-B|woman1-BI|woman1-BR|woman1-I|woman1-IB\n\t|woman1-IR|woman1-IX|woman1-RB|woman1-RI|woman1-SB|woman1-SM|woman1-TP|woman1-TX|woman1-alt-fonts|woman1-bd|woman1-cs|woman1-hc|woman1-hw\n\t|woman1-hy|woman1-ne|woman1-nh|woman1-ps|woman1-roff-buffer|woman1-ss|woman1-ul|woman1-vs|woman2-DT|woman2-HP|woman2-IP|woman2-LP|woman2-P\n\t|woman2-PD|woman2-PP|woman2-RE|woman2-RS|woman2-SH|woman2-SS|woman2-TE|woman2-TH|woman2-TP|woman2-TS|woman2-ad|woman2-br|woman2-fc|woman2-fi\n\t|woman2-format-paragraphs|woman2-get-prevailing-indent|woman2-in|woman2-ll|woman2-na|woman2-nf|woman2-nr|woman2-ns|woman2-process-escapes-to-eol\n\t|woman2-process-escapes|woman2-roff-buffer|woman2-rs|woman2-sp|woman2-ta|woman2-tagged-paragraph|woman2-ti|woman2-tr|word-at-point\n\t|x-apply-session-resources|x-backspace-delete-keys-p|x-change-window-property|x-clipboard-yank|x-complement-fontset-spec|x-compose-font-name\n\t|x-create-frame-with-faces|x-create-frame|x-cut-buffer-or-selection-value|x-decompose-font-name|x-delete-window-property|x-disown-selection-internal\n\t|x-display-backing-store|x-display-color-cells|x-display-grayscale-p|x-display-mm-height|x-display-mm-width|x-display-monitor-attributes-list\n\t|x-display-pixel-height|x-display-pixel-width|x-display-planes|x-display-save-under|x-display-screens|x-display-visual-class|x-dnd-choose-type\n\t|x-dnd-current-type|x-dnd-default-test-function|x-dnd-drop-data|x-dnd-forget-drop|x-dnd-get-drop-width-height|x-dnd-get-drop-x-y\n\t|x-dnd-get-motif-value|x-dnd-get-state-cons-for-frame|x-dnd-get-state-for-frame|x-dnd-handle-drag-n-drop-event|x-dnd-handle-file-name\n\t|x-dnd-handle-motif|x-dnd-handle-moz-url|x-dnd-handle-old-kde|x-dnd-handle-uri-list|x-dnd-handle-xdnd|x-dnd-init-frame|x-dnd-init-motif-for-frame\n\t|x-dnd-init-xdnd-for-frame|x-dnd-insert-ctext|x-dnd-insert-utf16-text|x-dnd-insert-utf8-text|x-dnd-maybe-call-test-function|x-dnd-more-than-3-from-flags\n\t|x-dnd-motif-value-to-list|x-dnd-save-state|x-dnd-version-from-flags|x-file-dialog|x-focus-frame|x-frame-geometry|x-get-atom-name\n\t|x-get-clipboard|x-get-selection-internal|x-get-selection-value|x-gtk-map-stock|x-handle-args|x-handle-display|x-handle-geometry\n\t|x-handle-iconic|x-handle-initial-switch|x-handle-name-switch|x-handle-named-frame-geometry|x-handle-no-bitmap-icon|x-handle-numeric-switch\n\t|x-handle-parent-id|x-handle-reverse-video|x-handle-smid|x-handle-switch|x-handle-xrm-switch|x-hide-tip|x-initialize-window-system\n\t|x-menu-bar-open-internal|x-menu-bar-open|x-must-resolve-font-name|x-own-selection-internal|x-register-dnd-atom|x-resolve-font-name\n\t|x-select-font|x-select-text|x-selection-exists-p|x-selection-owner-p|x-selection-value|x-selection|x-send-client-message|x-server-max-request-size\n\t|x-show-tip|x-synchronize|x-uses-old-gtk-dialog|x-win-suspend-error|x-window-property|x-wm-set-size-hint|xdb|xml--entity-replacement-text\n\t|xml--parse-buffer|xml-debug-print-internal|xml-debug-print|xml-escape-string|xml-find-file-coding-system|xml-get-attribute-or-nil\n\t|xml-get-attribute|xml-get-children|xml-maybe-do-ns|xml-mode|xml-node-attributes|xml-node-children|xml-node-name|xml-parse-attlist\n\t|xml-parse-dtd|xml-parse-elem-type|xml-parse-file|xml-parse-region|xml-parse-string|xml-parse-tag-1|xml-parse-tag|xml-print|xml-skip-dtd\n\t|xml-substitute-numeric-entities|xml-substitute-special|xmltok-get-declared-encoding-position|xor|xref--alistify|xref--analyze\n\t|xref--display-position|xref--find-definitions|xref--goto-location|xref--insert-propertized|xref--insert-xrefs|xref--location-at-point\n\t|xref--next-line|xref--pop-to-location|xref--read-identifier|xref--search-property|xref--show-location|xref--show-xref-buffer|xref--show-xrefs\n\t|xref--xref-buffer-mode|xref--xref-child-p|xref--xref-description|xref--xref-list-p|xref--xref-location|xref--xref-p|xref--xref\n\t|xref-bogus-location-child-p|xref-bogus-location-list-p|xref-bogus-location-message|xref-bogus-location-p|xref-bogus-location\n\t|xref-buffer-location-child-p|xref-buffer-location-list-p|xref-buffer-location-p|xref-buffer-location|xref-clear-marker-stack\n\t|xref-default-identifier-at-point|xref-elisp-location-child-p|xref-elisp-location-list-p|xref-elisp-location-p|xref-elisp-location\n\t|xref-file-location-child-p|xref-file-location-list-p|xref-file-location-p|xref-file-location|xref-find-apropos|xref-find-definitions-other-frame\n\t|xref-find-definitions-other-window|xref-find-definitions|xref-find-references|xref-goto-xref|xref-location-child-p|xref-location-group\n\t|xref-location-list-p|xref-location-marker|xref-location-p|xref-location|xref-make-bogus-location|xref-make-buffer-location|xref-make-elisp-location\n\t|xref-make-file-location|xref-make|xref-next-line|xref-pop-marker-stack|xref-prev-line|xref-push-marker-stack|xscheme-cd|xscheme-coerce-prompt\n\t|xscheme-debugger-mode-p|xscheme-default-command-line|xscheme-delete-output|xscheme-display-process-buffer|xscheme-enable-control-g\n\t|xscheme-enter-debugger-mode|xscheme-enter-input-wait|xscheme-enter-interaction-mode|xscheme-eval|xscheme-evaluation-commands\n\t|xscheme-exit-input-wait|xscheme-finish-gc|xscheme-goto-output-point|xscheme-guarantee-newlines|xscheme-insert-expression|xscheme-interrupt-commands\n\t|xscheme-message|xscheme-mode-line-initialize|xscheme-output-goto|xscheme-parse-command-line|xscheme-process-buffer-current-p\n\t|xscheme-process-buffer-window|xscheme-process-buffer|xscheme-process-filter-initialize|xscheme-process-filter-output|xscheme-process-filter\n\t|xscheme-process-filter:simple-action|xscheme-process-filter:string-action-noexcursion|xscheme-process-filter:string-action\n\t|xscheme-process-running-p|xscheme-process-sentinel|xscheme-prompt-for-confirmation|xscheme-prompt-for-expression-exit|xscheme-prompt-for-expression\n\t|xscheme-read-command-line|xscheme-region-expression-p|xscheme-rotate-yank-pointer|xscheme-select-process-buffer|xscheme-send-breakpoint-interrupt\n\t|xscheme-send-buffer|xscheme-send-char|xscheme-send-control-g-interrupt|xscheme-send-control-u-interrupt|xscheme-send-control-x-interrupt\n\t|xscheme-send-current-line|xscheme-send-definition|xscheme-send-interrupt|xscheme-send-next-expression|xscheme-send-previous-expression\n\t|xscheme-send-proceed|xscheme-send-region|xscheme-send-string-1|xscheme-send-string-2|xscheme-send-string|xscheme-set-prompt-variable\n\t|xscheme-set-prompt|xscheme-set-runlight|xscheme-start-gc|xscheme-start-process|xscheme-start|xscheme-unsolicited-read-char|xscheme-wait-for-process\n\t|xscheme-write-message-1|xscheme-write-value|xscheme-yank-pop|xscheme-yank-previous-send|xscheme-yank-push|xscheme-yank|xselect--encode-string\n\t|xselect--int-to-cons|xselect--selection-bounds|xselect-convert-to-atom|xselect-convert-to-charpos|xselect-convert-to-class|xselect-convert-to-colno\n\t|xselect-convert-to-delete|xselect-convert-to-filename|xselect-convert-to-host|xselect-convert-to-identity|xselect-convert-to-integer\n\t|xselect-convert-to-length|xselect-convert-to-lineno|xselect-convert-to-name|xselect-convert-to-os|xselect-convert-to-save-targets\n\t|xselect-convert-to-string|xselect-convert-to-targets|xselect-convert-to-user|xterm-mouse--read-event-sequence-1000|xterm-mouse--read-event-sequence-1006\n\t|xterm-mouse--set-click-count|xterm-mouse-event|xterm-mouse-mode|xterm-mouse-position-function|xterm-mouse-translate-1|xterm-mouse-translate-extended\n\t|xterm-mouse-translate|xterm-mouse-truncate-wrap|xw-color-defined-p|xw-color-values|xw-defined-colors|xw-display-color-p|yank-handle-category-property\n\t|yank-handle-font-lock-face-property|yank-menu|yank-rectangle|yenc-decode-region|yenc-extract-filename|zap-to-char|zeroconf-get-domain\n\t|zeroconf-get-host-domain|zeroconf-get-host|zeroconf-get-interface-name|zeroconf-get-interface-number|zeroconf-get-service|zeroconf-init\n\t|zeroconf-list-service-names|zeroconf-list-service-types|zeroconf-list-services|zeroconf-publish-service|zeroconf-register-service-browser\n\t|zeroconf-register-service-resolver|zeroconf-register-service-type-browser|zeroconf-resolve-service|zeroconf-service-add-hook\n\t|zeroconf-service-address|zeroconf-service-aprotocol|zeroconf-service-browser-handler|zeroconf-service-domain|zeroconf-service-flags\n\t|zeroconf-service-host|zeroconf-service-interface|zeroconf-service-name|zeroconf-service-port|zeroconf-service-protocol|zeroconf-service-remove-hook\n\t|zeroconf-service-resolver-handler|zeroconf-service-txt|zeroconf-service-type-browser-handler|zeroconf-service-type|zerop--anon-cmacro\n\t|zone-call|zone-cpos|zone-exploding-remove|zone-fall-through-ws|zone-fill-out-screen|zone-fret|zone-hiding-mode-line|zone-leave-me-alone\n\t|zone-line-specs|zone-mode|zone-orig|zone-park\\/sit-for|zone-pgm-2nd-putz-with-case|zone-pgm-dissolve|zone-pgm-drip-fretfully|zone-pgm-drip\n\t|zone-pgm-explode|zone-pgm-five-oclock-swan-dive|zone-pgm-jitter|zone-pgm-martini-swan-dive|zone-pgm-paragraph-spaz|zone-pgm-putz-with-case\n\t|zone-pgm-random-life|zone-pgm-rat-race|zone-pgm-rotate-LR-lockstep|zone-pgm-rotate-LR-variable|zone-pgm-rotate-RL-lockstep|zone-pgm-rotate-RL-variable\n\t|zone-pgm-rotate|zone-pgm-stress-destress|zone-pgm-stress|zone-pgm-whack-chars|zone-remove-text|zone-replace-char|zone-shift-down\n\t|zone-shift-left|zone-shift-right|zone-shift-up|zone-when-idle|zone|zrgrep\n)(?=[\\s()]|$)',
          name: 'support.function.emacs.lisp'
        }
      ]
    },
    string: {
      begin: '"',
      beginCaptures: {
        0: {name: 'punctuation.definition.string.begin.emacs.lisp'}
      },
      end: '"',
      endCaptures: {0: {name: 'punctuation.definition.string.end.emacs.lisp'}},
      name: 'string.quoted.double.emacs.lisp',
      patterns: [{include: '#string-innards'}]
    },
    'string-innards': {
      patterns: [
        {include: '#eldoc'},
        {
          match: '(\\\\)$\\n?',
          name: 'constant.escape.character.newline.emacs.lisp'
        },
        {
          captures: {1: {name: 'punctuation.escape.backslash.emacs.lisp'}},
          match: '(\\\\).',
          name: 'constant.escape.character.emacs.lisp'
        }
      ]
    },
    symbols: {
      patterns: [
        {
          captures: {0: {name: 'punctuation.definition.symbol.emacs.lisp'}},
          match: '(?<=[\\s()\\[]|^)##',
          name: 'constant.other.interned.blank.symbol.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'punctuation.definition.symbol.emacs.lisp'},
            2: {patterns: [{include: '$self'}]}
          },
          match: "(?<=[\\s()\\[]|^)(#)((?:[-'+=*/\\w~!@$%^&:<>{}?]|\\\\.)+)",
          name: 'constant.other.symbol.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'punctuation.definition.spliced.symbol.emacs.lisp'}
          },
          match: '(,@)([-+=*/\\w~!@$%^&:<>{}?]+)',
          name: 'constant.other.spliced.symbol.emacs.lisp'
        },
        {
          captures: {
            1: {name: 'punctuation.definition.inserted.symbol.emacs.lisp'}
          },
          match: '(,)([-+=*/\\w~!@$%^&:<>{}?]+)',
          name: 'constant.other.inserted.symbol.emacs.lisp'
        }
      ]
    },
    vectors: {
      patterns: [
        {match: '\\[', name: 'punctuation.section.vector.begin.emacs.lisp'},
        {match: '\\]', name: 'punctuation.section.vector.end.emacs.lisp'}
      ]
    }
  },
  scopeName: 'source.emacs.lisp'
}

export default grammar
