// This is a TextMate grammar distributed by `starry-night`.
// This grammar is licensed `mit`.
// See <https://github.com/wooorm/starry-night> for more info.
/** @type {import('../lib/index.js').Grammar} */
const grammar = {
  extensions: ['.mq4', '.mq5', '.mqh', '.mqh'],
  names: ['mql4', 'mql5'],
  patterns: [
    {include: '#comments'},
    {
      begin: '"',
      beginCaptures: {0: {name: 'punctuation.definition.string.begin.mql5'}},
      end: '"',
      endCaptures: {0: {name: 'punctuation.definition.string.end.mql5'}},
      name: 'string.quoted.double.mql5',
      patterns: [{include: '#escape_characters'}]
    },
    {
      begin: "'",
      beginCaptures: {0: {name: 'punctuation.definition.string.begin.mql5'}},
      end: "'",
      endCaptures: {0: {name: 'punctuation.definition.string.end.mql5'}},
      name: 'string.quoted.single.mql5',
      patterns: [{include: '#escape_characters'}]
    },
    {
      begin: '(#include)(?:\\s*)(<)',
      beginCaptures: {
        1: {name: 'keyword.control.preprocessor.mql5'},
        2: {name: 'punctuation.definition.string.begin.mql5'}
      },
      end: '>',
      endCaptures: {0: {name: 'punctuation.definition.string.end.mql5'}},
      name: 'string.other.mql5'
    },
    {
      match:
        '#\\b(include|package|define|undef|import|property|resource|ifdef|ifndef|else|endif)\\b',
      name: 'keyword.control.preprocessor.mql5'
    },
    {
      match: '\\b(const|private|protected|public|virtual|export)\\b',
      name: 'storage.modifier.mql5'
    },
    {match: '\\b(extern|input|static|sinput)\\b', name: 'storage.input.mql5'},
    {
      match:
        '\\b(char|short|int|long|uchar|ushort|uint|ulong|bool|color|datetime|string|double|float|enum|struct|class|interface|void)\\b',
      name: 'storage.type.mql5'
    },
    {
      match:
        '\\b(if|else|switch|case|default|break|continue|for|do|while|new|delete|return)\\b',
      name: 'keyword.control.mql5'
    },
    {
      match:
        '\\b(operator|sizeof|this|template|typename|typedef|trash|defined|unsigned|dynamic_cast|override|final|__DATE__|__DATETIME__|__FILE__|__FUNCSIG__|__FUNCTION__|__LINE__|__PATH__)\\b',
      name: 'keyword.other.mql5'
    },
    {
      match: '=|\\+|\\-|<|>|!|\\*|/|%|&|\\||~|\\^|,|:|\\?|;',
      name: 'punctuation.definition.operator.mql5'
    },
    {match: '\\(|\\)|\\[|\\]', name: 'punctuation.definition.brackets.mql5'},
    {
      match:
        '(?<!\\.)\\b(AccountBalance|AccountCompany|AccountCredit|AccountCurrency|AccountEquity|AccountFreeMargin|AccountFreeMarginCheck|AccountFreeMarginMode|AccountInfoDouble|AccountInfoInteger|AccountInfoString|AccountLeverage|AccountMargin|AccountName|AccountNumber|AccountProfit|AccountServer|AccountStopoutLevel|AccountStopoutMode|acos|Alert|ArrayBsearch|ArrayCompare|ArrayCopy|ArrayCopyRates|ArrayCopySeries|ArrayDimension|ArrayFill|ArrayFree|ArrayGetAsSeries|ArrayInitialize|ArrayIsDynamic|ArrayIsSeries|ArrayMaximum|ArrayMinimum|ArrayRange|ArrayResize|ArraySetAsSeries|ArraySize|ArraySort|asin|atan|Bars|BarsCalculated|BarsPerWindow|ceil|CharArrayToString|ChartApplyTemplate|ChartClose|ChartFirst|ChartGetDouble|ChartGetInteger|ChartGetString|ChartID|ChartIndicatorAdd|ChartIndicatorDelete|ChartIndicatorGet|ChartIndicatorName|ChartIndicatorsTotal|ChartNavigate|ChartNext|ChartOpen|CharToStr|CharToString|ChartPeriod|ChartPriceOnDropped|ChartRedraw|ChartSaveTemplate|ChartScreenShot|ChartSetDouble|ChartSetInteger|ChartSetString|ChartSetSymbolPeriod|ChartSymbol|ChartTimeOnDropped|ChartTimePriceToXY|ChartWindowFind|ChartWindowOnDropped|ChartXOnDropped|ChartXYToTimePrice|ChartYOnDropped|CheckPointer|CLBufferCreate|CLBufferFree|CLBufferRead|CLBufferWrite|CLContextCreate|CLContextFree|CLExecute|CLGetDeviceInfo|CLGetInfoInteger|CLGetInfoString|CLHandleType|ClientTerminalName|CLKernelCreate|CLKernelFree|CLProgramCreate|CLProgramFree|CLSetKernelArg|CLSetKernelArgMem|ColorToARGB|ColorToString|Comment|CompanyName|CopyBuffer|CopyClose|CopyHigh|CopyLow|CopyOpen|CopyRates|CopyRealVolume|CopySpread|CopyTicks|CopyTickVolume|CopyTime|cos|CryptDecode|CryptEncode|CurTime|Day|DayOfWeek|DayOfYear|DebugBreak|Digits|DoubleToStr|DoubleToString|EnumToString|EventChartCustom|EventKillTimer|EventSetMillisecondTimer|EventSetTimer|exp|ExpertRemove|fabs|FileClose|FileCopy|FileDelete|FileFindClose|FileFindFirst|FileFindNext|FileFlush|FileGetInteger|FileIsEnding|FileIsExist|FileIsLineEnding|FileMove|FileOpen|FileOpenHistory|FileReadArray|FileReadBool|FileReadDatetime|FileReadDouble|FileReadFloat|FileReadInteger|FileReadLong|FileReadNumber|FileReadString|FileReadStruct|FileSeek|FileSize|FileTell|FileWrite|FileWriteArray|FileWriteDouble|FileWriteFloat|FileWriteInteger|FileWriteLong|FileWriteString|FileWriteStruct|FirstVisibleBar|floor|fmax|fmin|fmod|FolderClean|FolderCreate|FolderDelete|FrameAdd|FrameFilter|FrameFirst|FrameInputs|FrameNext|GetLastError|GetMicrosecondCount|GetPointer|GetTickCount|GlobalVariableCheck|GlobalVariableDel|GlobalVariableGet|GlobalVariableName|GlobalVariablesDeleteAll|GlobalVariableSet|GlobalVariableSetOnCondition|GlobalVariablesFlush|GlobalVariablesTotal|GlobalVariableTemp|GlobalVariableTime|HideTestIndicators|Highest|HistoryDealGetDouble|HistoryDealGetInteger|HistoryDealGetString|HistoryDealGetTicket|HistoryDealSelect|HistoryDealsTotal|HistoryOrderGetDouble|HistoryOrderGetInteger|HistoryOrderGetString|HistoryOrderGetTicket|HistoryOrderSelect|HistoryOrdersTotal|HistorySelect|HistorySelectByPosition|HistoryTotal|Hour|iBars|iBarShift|iClose|iHigh|iHighest|iLow|iLowest|IndicatorBuffers|IndicatorCounted|IndicatorCreate|IndicatorDigits|IndicatorParameters|IndicatorRelease|IndicatorSetDouble|IndicatorSetInteger|IndicatorSetString|IndicatorShortName|IntegerToString|iOpen|IsConnected|IsDemo|IsDllsAllowed|IsExpertEnabled|IsLibrariesAllowed|IsOptimization|IsStopped|IsTesting|IsTradeAllowed|IsTradeContextBusy|IsVisualMode|iTime|iVolume|LocalTime|log|log10|Lowest|MarketBookAdd|MarketBookGet|MarketBookRelease|MarketInfo|MathAbs|MathArccos|MathArcsin|MathArctan|MathCeil|MathCos|MathExp|MathFloor|MathIsValidNumber|MathLog|MathLog10|MathMax|MathMin|MathMod|MathPow|MathRand|MathRound|MathSin|MathSqrt|MathSrand|MathTan|MessageBox|Minute|Month|MQL5InfoInteger|MQL5InfoString|MQLInfoInteger|MQLInfoString|MQLSetInteger|NormalizeDouble|ObjectCreate|ObjectDelete|ObjectDescription|ObjectFind|ObjectGet|ObjectGetDouble|ObjectGetFiboDescription|ObjectGetInteger|ObjectGetShiftByValue|ObjectGetString|ObjectGetTimeByValue|ObjectGetValueByShift|ObjectGetValueByTime|ObjectMove|ObjectName|ObjectsDeleteAll|ObjectSet|ObjectSetDouble|ObjectSetFiboDescription|ObjectSetInteger|ObjectSetString|ObjectSetText|ObjectsRedraw|ObjectsTotal|ObjectType|OnBookEvent|OnCalculate|OnChartEvent|OnDeinit|OnInit|OnStart|OnTester|OnTesterDeinit|OnTesterInit|OnTesterPass|OnTick|OnTimer|OnTrade|OnTradeTransaction|OrderCalcMargin|OrderCalcProfit|OrderCheck|OrderClose|OrderCloseBy|OrderClosePrice|OrderCloseTime|OrderComment|OrderCommission|OrderDelete|OrderExpiration|OrderGetDouble|OrderGetInteger|OrderGetString|OrderGetTicket|OrderLots|OrderMagicNumber|OrderModify|OrderOpenPrice|OrderOpenTime|OrderPrint|OrderProfit|OrderSelect|OrderSend|OrderSendAsync|OrdersHistoryTotal|OrderStopLoss|OrdersTotal|OrderSwap|OrderSymbol|OrderTakeProfit|OrderTicket|OrderType|ParameterGetRange|ParameterSetRange|Period|PeriodSeconds|PlaySound|PlotIndexGetInteger|PlotIndexSetDouble|PlotIndexSetInteger|PlotIndexSetString|Point|PositionGetDouble|PositionGetInteger|PositionGetString|PositionGetSymbol|PositionGetTicket|PositionSelect|PositionSelectByTicket|PositionsTotal|pow|PriceOnDropped|Print|printf|PrintFormat|rand|RefreshRates|ResetLastError|ResourceCreate|ResourceFree|ResourceReadImage|ResourceSave|round|ScreenShot|Seconds|SendFTP|SendMail|SendNotification|SeriesInfoInteger|ServerAddress|SetIndexArrow|SetIndexBuffer|SetIndexDrawBegin|SetIndexEmptyValue|SetIndexLabel|SetIndexShift|SetIndexStyle|SetLevelStyle|SetLevelValue|SetUserError|ShortArrayToString|ShortToString|SignalBaseGetDouble|SignalBaseGetInteger|SignalBaseGetString|SignalBaseSelect|SignalBaseTotal|SignalInfoGetDouble|SignalInfoGetInteger|SignalInfoGetString|SignalInfoSetDouble|SignalInfoSetInteger|SignalSubscribe|SignalUnsubscribe|sin|Sleep|sqrt|srand|StringAdd|StringBufferLen|StringCompare|StringConcatenate|StringFill|StringFind|StringFormat|StringGetChar|StringGetCharacter|StringInit|StringLen|StringReplace|StringSetChar|StringSetCharacter|StringSplit|StringSubstr|StringToCharArray|StringToColor|StringToDouble|StringToInteger|StringToLower|StringToShortArray|StringToTime|StringToUpper|StringTrimLeft|StringTrimRight|StrToDouble|StrToInteger|StrToTime|StructToTime|Symbol|SymbolInfoDouble|SymbolInfoInteger|SymbolInfoMarginRate|SymbolInfoSessionQuote|SymbolInfoSessionTrade|SymbolInfoString|SymbolInfoTick|SymbolIsSynchronized|SymbolName|SymbolSelect|SymbolsTotal|tan|TerminalClose|TerminalCompany|TerminalInfoDouble|TerminalInfoInteger|TerminalInfoString|TerminalName|TerminalPath|TesterStatistics|TesterWithdrawal|TextGetSize|TextOut|TextSetFont|TimeCurrent|TimeDay|TimeDaylightSavings|TimeDayOfWeek|TimeDayOfYear|TimeGMT|TimeGMTOffset|TimeHour|TimeLocal|TimeMinute|TimeMonth|TimeOnDropped|TimeSeconds|TimeToStr|TimeToString|TimeToStruct|TimeTradeServer|TimeYear|UninitializeReason|WebRequest|WindowBarsPerChart|WindowExpertName|WindowFind|WindowFirstVisibleBar|WindowHandle|WindowIsVisible|WindowOnDropped|WindowPriceMax|WindowPriceMin|WindowPriceOnDropped|WindowRedraw|WindowScreenShot|WindowsTotal|WindowTimeOnDropped|WindowXOnDropped|WindowYOnDropped|Year|ZeroMemory)\\b',
      name: 'entity.name.function.mql5'
    },
    {
      match: '(?<!\\.)\\b(init|deinit|start)\\b',
      name: 'entity.name.function.mql5'
    },
    {
      match:
        '(?<!\\.)\\b(iAC|iAD|iADX|iADXWilder|iAlligator|iAMA|iAO|iATR|iBands|iBandsOnArray|iBearsPower|iBullsPower|iBWMFI|iCCI|iCCIOnArray|iChaikin|iCustom|iDEMA|iDeMarker|iEnvelopes|iEnvelopesOnArray|iForce|iFractals|iFrAMA|iGator|iIchimoku|iMA|iMACD|iMAOnArray|iMFI|iMomentum|iMomentumOnArray|iOBV|iOsMA|iRSI|iRSIOnArray|iRVI|iSAR|iStdDev|iStdDevOnArray|iStochastic|iTEMA|iTriX|iVIDyA|iVolumes|iWPR)\\b',
      name: 'support.function.mql5'
    },
    {
      match:
        '(?<!\\.)\\b(!ZeroString|!ZeroValue|_AppliedTo|_CriticalError|_Digits|_IsX64|_LastError|_Period|_Point|_RandomSeed|_ReturnedDouble|_ReturnedFloat|_ReturnedString|_StopFlag|_Symbol|_UninitReason|Ask|Bars|Bid|Close|Digits|High|Low|MqlBookInfo|MqlDateTime|MqlParam|MqlRates|MqlTick|MqlTradeCheckResult|MqlTradeRequest|MqlTradeResult|MqlTradeTransaction|Open|Point|Time|Volume)\\b',
      name: 'variable.language.predefined.mql5'
    },
    {
      match:
        '\\b(__MQ5BUILD__|__MQL4__|__MQL4BUILD__|__MQL5__|__MQL5BUILD__|__MQL__|__MQLBUILD__|ACCOUNT_ASSETS|ACCOUNT_BALANCE|ACCOUNT_COMMISSION_BLOCKED|ACCOUNT_COMPANY|ACCOUNT_CREDIT|ACCOUNT_CURRENCY|ACCOUNT_EQUITY|ACCOUNT_FREEMARGIN|ACCOUNT_LEVERAGE|ACCOUNT_LIABILITIES|ACCOUNT_LIMIT_ORDERS|ACCOUNT_LOGIN|ACCOUNT_MARGIN|ACCOUNT_MARGIN_FREE|ACCOUNT_MARGIN_INITIAL|ACCOUNT_MARGIN_LEVEL|ACCOUNT_MARGIN_MAINTENANCE|ACCOUNT_MARGIN_MODE|ACCOUNT_MARGIN_MODE_EXCHANGE|ACCOUNT_MARGIN_MODE_RETAIL_HEDGING|ACCOUNT_MARGIN_MODE_RETAIL_NETTING|ACCOUNT_MARGIN_SO_CALL|ACCOUNT_MARGIN_SO_MODE|ACCOUNT_MARGIN_SO_SO|ACCOUNT_NAME|ACCOUNT_PROFIT|ACCOUNT_SERVER|ACCOUNT_STOPOUT_MODE_MONEY|ACCOUNT_STOPOUT_MODE_PERCENT|ACCOUNT_TRADE_ALLOWED|ACCOUNT_TRADE_EXPERT|ACCOUNT_TRADE_MODE|ACCOUNT_TRADE_MODE_CONTEST|ACCOUNT_TRADE_MODE_DEMO|ACCOUNT_TRADE_MODE_REAL|ALIGN_CENTER|ALIGN_LEFT|ALIGN_RIGHT|ANCHOR_BOTTOM|ANCHOR_CENTER|ANCHOR_LEFT|ANCHOR_LEFT_LOWER|ANCHOR_LEFT_UPPER|ANCHOR_LOWER|ANCHOR_RIGHT|ANCHOR_RIGHT_LOWER|ANCHOR_RIGHT_UPPER|ANCHOR_TOP|ANCHOR_UPPER|BASE_LINE|BOOK_TYPE_BUY|BOOK_TYPE_BUY_MARKET|BOOK_TYPE_SELL|BOOK_TYPE_SELL_MARKET|BORDER_FLAT|BORDER_RAISED|BORDER_SUNKEN|CHAR_MAX|CHAR_MIN|CHAR_VALUE|CHART_AUTOSCROLL|CHART_BARS|CHART_BEGIN|CHART_BRING_TO_TOP|CHART_CANDLES|CHART_COLOR_ASK|CHART_COLOR_BACKGROUND|CHART_COLOR_BID|CHART_COLOR_CANDLE_BEAR|CHART_COLOR_CANDLE_BULL|CHART_COLOR_CHART_DOWN|CHART_COLOR_CHART_LINE|CHART_COLOR_CHART_UP|CHART_COLOR_FOREGROUND|CHART_COLOR_GRID|CHART_COLOR_LAST|CHART_COLOR_STOP_LEVEL|CHART_COLOR_VOLUME|CHART_COMMENT|CHART_CURRENT_POS|CHART_DRAG_TRADE_LEVELS|CHART_END|CHART_EVENT_MOUSE_MOVE|CHART_EVENT_OBJECT_CREATE|CHART_EVENT_OBJECT_DELETE|CHART_EXPERT_NAME|CHART_FIRST_VISIBLE_BAR|CHART_FIXED_MAX|CHART_FIXED_MIN|CHART_FIXED_POSITION|CHART_FOREGROUND|CHART_HEIGHT_IN_PIXELS|CHART_IS_OBJECT|CHART_IS_OFFLINE|CHART_LINE|CHART_MODE|CHART_MOUSE_SCROLL|CHART_POINTS_PER_BAR|CHART_PRICE_MAX|CHART_PRICE_MIN|CHART_SCALE|CHART_SCALE_PT_PER_BAR|CHART_SCALEFIX|CHART_SCALEFIX_11|CHART_SCRIPT_NAME|CHART_SHIFT|CHART_SHIFT_SIZE|CHART_SHOW_ASK_LINE|CHART_SHOW_BID_LINE|CHART_SHOW_DATE_SCALE|CHART_SHOW_GRID|CHART_SHOW_LAST_LINE|CHART_SHOW_OBJECT_DESCR|CHART_SHOW_OHLC|CHART_SHOW_ONE_CLICK|CHART_SHOW_PERIOD_SEP|CHART_SHOW_PRICE_SCALE|CHART_SHOW_TRADE_LEVELS|CHART_SHOW_VOLUMES|CHART_VISIBLE_BARS|CHART_VOLUME_HIDE|CHART_VOLUME_REAL|CHART_VOLUME_TICK|CHART_WIDTH_IN_BARS|CHART_WIDTH_IN_PIXELS|CHART_WINDOW_HANDLE|CHART_WINDOW_IS_VISIBLE|CHART_WINDOW_YDISTANCE|CHART_WINDOWS_TOTAL|CHARTEVENT_CHART_CHANGE|CHARTEVENT_CLICK|CHARTEVENT_CUSTOM|CHARTEVENT_CUSTOM_LAST|CHARTEVENT_KEYDOWN|CHARTEVENT_MOUSE_MOVE|CHARTEVENT_OBJECT_CHANGE|CHARTEVENT_OBJECT_CLICK|CHARTEVENT_OBJECT_CREATE|CHARTEVENT_OBJECT_DELETE|CHARTEVENT_OBJECT_DRAG|CHARTEVENT_OBJECT_ENDEDIT|CHARTS_MAX|CHIKOUSPAN_LINE|CL_BUFFER_SIZE|CL_DEVICE_ACCELERATOR|CL_DEVICE_BUILT_IN_KERNELS|CL_DEVICE_COUNT|CL_DEVICE_CPU|CL_DEVICE_CUSTOM|CL_DEVICE_DEFAULT|CL_DEVICE_DOUBLE_FP_CONFIG|CL_DEVICE_EXTENSIONS|CL_DEVICE_GLOBAL_MEM_SIZE|CL_DEVICE_GPU|CL_DEVICE_LOCAL_MEM_SIZE|CL_DEVICE_MAX_CLOCK_FREQUENCY|CL_DEVICE_MAX_COMPUTE_UNITS|CL_DEVICE_MAX_WORK_GROUP_SIZE|CL_DEVICE_MAX_WORK_ITEM_DIMENSIONS|CL_DEVICE_MAX_WORK_ITEM_SIZES|CL_DEVICE_NAME|CL_DEVICE_OPENCL_C_VERSION|CL_DEVICE_PROFILE|CL_DEVICE_TYPE|CL_DEVICE_VENDOR|CL_DEVICE_VENDOR_ID|CL_DEVICE_VERSION|CL_DRIVER_VERSION|CL_MEM_ALLOC_HOST_PTR|CL_MEM_COPY_HOST_PTR|CL_MEM_READ_ONLY|CL_MEM_READ_WRITE|CL_MEM_USE_HOST_PTR|CL_MEM_WRITE_ONLY|CL_PLATFORM_EXTENSIONS|CL_PLATFORM_NAME|CL_PLATFORM_PROFILE|CL_PLATFORM_VENDOR|CL_PLATFORM_VERSION|CL_USE_ANY|CL_USE_CPU_ONLY|CL_USE_GPU_ONLY|clrAliceBlue|clrAntiqueWhite|clrAqua|clrAquamarine|clrAzure|clrBeige|clrBisque|clrBlack|clrBlanchedAlmond|clrBlue|clrBlueViolet|clrBrown|clrBurlyWood|clrCadetBlue|clrChartreuse|clrChocolate|clrCoral|clrCornflowerBlue|clrCornsilk|clrCrimson|clrCyan|clrDarkBlue|clrDarkCyan|clrDarkGoldenrod|clrDarkGray|clrDarkGreen|clrDarkKhaki|clrDarkMagenta|clrDarkOliveGreen|clrDarkOrange|clrDarkOrchid|clrDarkRed|clrDarkSalmon|clrDarkSeaGreen|clrDarkSlateBlue|clrDarkSlateGray|clrDarkTurquoise|clrDarkViolet|clrDeepPink|clrDeepSkyBlue|clrDimGray|clrDodgerBlue|clrFireBrick|clrFloralWhite|clrForestGreen|clrFuchsia|clrGainsboro|clrGhostWhite|clrGold|clrGoldenrod|clrGray|clrGreen|clrGreenYellow|clrHoneydew|clrHotPink|clrIndianRed|clrIndigo|clrIvory|clrKhaki|clrLavender|clrLavenderBlush|clrLawnGreen|clrLemonChiffon|clrLightBlue|clrLightCoral|clrLightCyan|clrLightGoldenrod|clrLightGray|clrLightGreen|clrLightPink|clrLightSalmon|clrLightSeaGreen|clrLightSkyBlue|clrLightSlateGray|clrLightSteelBlue|clrLightYellow|clrLime|clrLimeGreen|clrLinen|clrMagenta|clrMaroon|clrMediumAquamarine|clrMediumBlue|clrMediumOrchid|clrMediumPurple|clrMediumSeaGreen|clrMediumSlateBlue|clrMediumSpringGreen|clrMediumTurquoise|clrMediumVioletRed|clrMidnightBlue|clrMintCream|clrMistyRose|clrMoccasin|clrNavajoWhite|clrNavy|clrNONE|clrOldLace|clrOlive|clrOliveDrab|clrOrange|clrOrangeRed|clrOrchid|clrPaleGoldenrod|clrPaleGreen|clrPaleTurquoise|clrPaleVioletRed|clrPapayaWhip|clrPeachPuff|clrPeru|clrPink|clrPlum|clrPowderBlue|clrPurple|clrRed|clrRosyBrown|clrRoyalBlue|clrSaddleBrown|clrSalmon|clrSandyBrown|clrSeaGreen|clrSeashell|clrSienna|clrSilver|clrSkyBlue|clrSlateBlue|clrSlateGray|clrSnow|clrSpringGreen|clrSteelBlue|clrTan|clrTeal|clrThistle|clrTomato|clrTurquoise|clrViolet|clrWheat|clrWhite|clrWhiteSmoke|clrYellow|clrYellowGreen|COLOR_FORMAT_ARGB_NORMALIZE|COLOR_FORMAT_ARGB_RAW|COLOR_FORMAT_XRGB_NOALPHA|COPY_TICKS_ALL|COPY_TICKS_INFO|COPY_TICKS_TRADE|copyright|CORNER_LEFT_LOWER|CORNER_LEFT_UPPER|CORNER_RIGHT_LOWER|CORNER_RIGHT_UPPER|CP_ACP|CP_MACCP|CP_OEMCP|CP_SYMBOL|CP_THREAD_ACP|CP_UTF7|CP_UTF8|CRYPT_AES128|CRYPT_AES256|CRYPT_ARCH_ZIP|CRYPT_BASE64|CRYPT_DES|CRYPT_HASH_MD5|CRYPT_HASH_SHA1|CRYPT_HASH_SHA256|DBL_DIG|DBL_EPSILON|DBL_MANT_DIG|DBL_MAX|DBL_MAX_10_EXP|DBL_MAX_EXP|DBL_MIN|DBL_MIN_10_EXP|DBL_MIN_EXP|DEAL_COMMENT|DEAL_COMMISSION|DEAL_ENTRY|DEAL_ENTRY_IN|DEAL_ENTRY_INOUT|DEAL_ENTRY_OUT|DEAL_ENTRY_OUT_BY|DEAL_ENTRY_STATE|DEAL_EXTERNAL_ID|DEAL_MAGIC|DEAL_ORDER|DEAL_POSITION_ID|DEAL_PRICE|DEAL_PROFIT|DEAL_SWAP|DEAL_SYMBOL|DEAL_TICKET|DEAL_TIME|DEAL_TIME_MSC|DEAL_TYPE|DEAL_TYPE_BALANCE|DEAL_TYPE_BONUS|DEAL_TYPE_BUY|DEAL_TYPE_BUY_CANCELED|DEAL_TYPE_CHARGE|DEAL_TYPE_COMMISSION|DEAL_TYPE_COMMISSION_AGENT_DAILY|DEAL_TYPE_COMMISSION_AGENT_MONTHLY|DEAL_TYPE_COMMISSION_DAILY|DEAL_TYPE_COMMISSION_MONTHLY|DEAL_TYPE_CORRECTION|DEAL_TYPE_CREDIT|DEAL_TYPE_INTEREST|DEAL_TYPE_SELL|DEAL_TYPE_SELL_CANCELED|DEAL_VOLUME|description|DOUBLE_VALUE|DRAW_ARROW|DRAW_BARS|DRAW_BARSCLOSE|DRAW_BARSHIGH|DRAW_BARSLOW|DRAW_BARSOPEN|DRAW_CANDLES|DRAW_COLOR_ARROW|DRAW_COLOR_BARS|DRAW_COLOR_CANDLES|DRAW_COLOR_HISTOGRAM|DRAW_COLOR_HISTOGRAM2|DRAW_COLOR_LINE|DRAW_COLOR_SECTION|DRAW_COLOR_ZIGZAG|DRAW_COLORCANDLE|DRAW_COLORLINE|DRAW_FILLING|DRAW_HISTOGRAM|DRAW_HISTOGRAM2|DRAW_LINE|DRAW_NONE|DRAW_SECTION|DRAW_ZIGZAG|ELLIOTT_CYCLE|ELLIOTT_GRAND_SUPERCYCLE|ELLIOTT_INTERMEDIATE|ELLIOTT_MINOR|ELLIOTT_MINUETTE|ELLIOTT_MINUTE|ELLIOTT_PRIMARY|ELLIOTT_SUBMINUETTE|ELLIOTT_SUPERCYCLE|EMPTY|EMPTY_VALUE|ENUM_ACCOUNT_INFO_DOUBLE|ENUM_ACCOUNT_INFO_INTEGER|ENUM_ACCOUNT_INFO_STRING|ENUM_ACCOUNT_MARGIN_MODE|ENUM_ACCOUNT_STOPOUT_MODE|ENUM_ACCOUNT_TRADE_MODE|ENUM_ALIGN_MODE|ENUM_ANCHOR_POINT|ENUM_APPLIED_PRICE|ENUM_APPLIED_VOLUME|ENUM_ARROW_ANCHOR|ENUM_BASE_CORNER|ENUM_BOOK_TYPE|ENUM_BORDER_TYPE|ENUM_CHART_EVENT|ENUM_CHART_MODE|ENUM_CHART_POSITION|ENUM_CHART_PROPERTY_DOUBLE|ENUM_CHART_PROPERTY_INTEGER|ENUM_CHART_PROPERTY_STRING|ENUM_CHART_VOLUME_MODE|ENUM_CL_DEVICE_TYPE|ENUM_COLOR_FORMAT|ENUM_CRYPT_METHOD|ENUM_CUSTOMIND_PROPERTY_DOUBLE|ENUM_CUSTOMIND_PROPERTY_INTEGER|ENUM_CUSTOMIND_PROPERTY_STRING|ENUM_DATATYPE|ENUM_DAY_OF_WEEK|ENUM_DEAL_ENTRY|ENUM_DEAL_PROPERTY_DOUBLE|ENUM_DEAL_PROPERTY_INTEGER|ENUM_DEAL_PROPERTY_STRING|ENUM_DEAL_TYPE|ENUM_DRAW_TYPE|ENUM_ELLIOT_WAVE_DEGREE|ENUM_FILE_POSITION|ENUM_FILE_PROPERTY_INTEGER|ENUM_GANN_DIRECTION|ENUM_INDEXBUFFER_TYPE|ENUM_INDICATOR|ENUM_INIT_RETCODE|ENUM_LICENSE_TYPE|ENUM_LINE_STYLE|ENUM_MA_METHOD|ENUM_MARKETINFO|ENUM_MQL5_INFO_INTEGER|ENUM_MQL5_INFO_STRING|ENUM_MQL_INFO_INTEGER|ENUM_MQL_INFO_STRING|ENUM_OBJECT|ENUM_OBJECT_PROPERTY_DOUBLE|ENUM_OBJECT_PROPERTY_INTEGER|ENUM_OBJECT_PROPERTY_STRING|ENUM_OPENCL_HANDLE_TYPE|ENUM_OPENCL_PROPERTY_INTEGER|ENUM_OPENCL_PROPERTY_STRING|ENUM_ORDER_PROPERTY_DOUBLE|ENUM_ORDER_PROPERTY_INTEGER|ENUM_ORDER_PROPERTY_STRING|ENUM_ORDER_STATE|ENUM_ORDER_TYPE|ENUM_ORDER_TYPE_FILLING|ENUM_ORDER_TYPE_TIME|ENUM_PLOT_PROPERTY_DOUBLE|ENUM_PLOT_PROPERTY_INTEGER|ENUM_PLOT_PROPERTY_STRING|ENUM_POINTER_TYPE|ENUM_POSITION_PROPERTY_DOUBLE|ENUM_POSITION_PROPERTY_INTEGER|ENUM_POSITION_PROPERTY_STRING|ENUM_POSITION_TYPE|ENUM_PROGRAM_TYPE|ENUM_SERIES_INFO_INTEGER|ENUM_SERIESMODE|ENUM_SIGNAL_BASE_DOUBLE|ENUM_SIGNAL_BASE_INTEGER|ENUM_SIGNAL_BASE_STRING|ENUM_SIGNAL_INFO_DOUBLE|ENUM_SIGNAL_INFO_INTEGER|ENUM_SIGNAL_INFO_STRING|ENUM_STATISTICS|ENUM_STO_PRICE|ENUM_SYMBOL_CALC_MODE|ENUM_SYMBOL_INFO_DOUBLE|ENUM_SYMBOL_INFO_INTEGER|ENUM_SYMBOL_INFO_STRING|ENUM_SYMBOL_OPTION_MODE|ENUM_SYMBOL_OPTION_RIGHT|ENUM_SYMBOL_SWAP_MODE|ENUM_SYMBOL_TRADE_EXECUTION|ENUM_SYMBOL_TRADE_MODE|ENUM_TERMINAL_INFO_DOUBLE|ENUM_TERMINAL_INFO_INTEGER|ENUM_TERMINAL_INFO_STRING|ENUM_TIMEFRAMES|ENUM_TRADE_REQUEST_ACTIONS|ENUM_TRADE_TRANSACTION_TYPE|ERR_ACCOUNT_DISABLED|ERR_ACCOUNT_WRONG_PROPERTY|ERR_ARRAY_AS_PARAMETER_EXPECTED|ERR_ARRAY_BAD_SIZE|ERR_ARRAY_INDEX_OUT_OF_RANGE|ERR_ARRAY_INVALID|ERR_ARRAY_RESIZE_ERROR|ERR_BOOKS_CANNOT_ADD|ERR_BOOKS_CANNOT_DELETE|ERR_BOOKS_CANNOT_GET|ERR_BOOKS_CANNOT_SUBSCRIBE|ERR_BROKER_BUSY|ERR_BUFFERS_NO_MEMORY|ERR_BUFFERS_WRONG_INDEX|ERR_CANNOT_CALL_FUNCTION|ERR_CANNOT_CLEAN_DIRECTORY|ERR_CANNOT_DELETE_DIRECTORY|ERR_CANNOT_DELETE_FILE|ERR_CANNOT_LOAD_LIBRARY|ERR_CANNOT_OPEN_FILE|ERR_CHAR_ARRAY_ONLY|ERR_CHART_CANNOT_CHANGE|ERR_CHART_CANNOT_CREATE_TIMER|ERR_CHART_CANNOT_OPEN|ERR_CHART_INDICATOR_CANNOT_ADD|ERR_CHART_INDICATOR_CANNOT_DEL|ERR_CHART_INDICATOR_NOT_FOUND|ERR_CHART_NAVIGATE_FAILED|ERR_CHART_NO_EXPERT|ERR_CHART_NO_REPLY|ERR_CHART_NOREPLY|ERR_CHART_NOT_FOUND|ERR_CHART_PROP_INVALID|ERR_CHART_SCREENSHOT_FAILED|ERR_CHART_TEMPLATE_FAILED|ERR_CHART_WINDOW_NOT_FOUND|ERR_CHART_WRONG_ID|ERR_CHART_WRONG_PARAMETER|ERR_CHART_WRONG_PROPERTY|ERR_CHARTINDICATOR_NOT_FOUND|ERR_CHARTWINDOW_NOT_FOUND|ERR_COMMON_ERROR|ERR_CUSTOM_INDICATOR_ERROR|ERR_CUSTOM_WRONG_PROPERTY|ERR_DIRECTORY_NOT_EXIST|ERR_DLL_CALLS_NOT_ALLOWED|ERR_DLLFUNC_CRITICALERROR|ERR_DOUBLE_ARRAY_ONLY|ERR_DOUBLE_PARAMETER_EXPECTED|ERR_END_OF_FILE|ERR_EXTERNAL_CALLS_NOT_ALLOWED|ERR_FILE_ARRAYRESIZE_ERROR|ERR_FILE_BIN_STRINGSIZE|ERR_FILE_BINSTRINGSIZE|ERR_FILE_BUFFER_ALLOCATION_ERROR|ERR_FILE_CACHEBUFFER_ERROR|ERR_FILE_CANNOT_CLEAN_DIRECTORY|ERR_FILE_CANNOT_DELETE|ERR_FILE_CANNOT_DELETE_DIRECTORY|ERR_FILE_CANNOT_OPEN|ERR_FILE_CANNOT_REWRITE|ERR_FILE_DIRECTORY_NOT_EXIST|ERR_FILE_ENDOFFILE|ERR_FILE_INCOMPATIBLE)\\b',
      name: 'support.constant.mql5'
    },
    {
      match:
        '\\b(ERR_FILE_INVALID_HANDLE|ERR_FILE_IS_DIRECTORY|ERR_FILE_ISNOT_DIRECTORY|ERR_FILE_NOT_BIN|ERR_FILE_NOT_CSV|ERR_FILE_NOT_DIRECTORY|ERR_FILE_NOT_EXIST|ERR_FILE_NOT_TOREAD|ERR_FILE_NOT_TOWRITE|ERR_FILE_NOT_TXT|ERR_FILE_NOT_TXTORCSV|ERR_FILE_NOTBIN|ERR_FILE_NOTCSV|ERR_FILE_NOTTOREAD|ERR_FILE_NOTTOWRITE|ERR_FILE_NOTTXT|ERR_FILE_NOTTXTORCSV|ERR_FILE_READ_ERROR|ERR_FILE_READERROR|ERR_FILE_STRINGRESIZE_ERROR|ERR_FILE_STRUCT_WITH_OBJECTS|ERR_FILE_TOO_LONG_FILENAME|ERR_FILE_TOO_MANY_OPENED|ERR_FILE_WRITE_ERROR|ERR_FILE_WRITEERROR|ERR_FILE_WRONG_DIRECTORYNAME|ERR_FILE_WRONG_FILENAME|ERR_FILE_WRONG_HANDLE|ERR_FLOAT_ARRAY_ONLY|ERR_FORMAT_TOO_MANY_FORMATTERS|ERR_FORMAT_TOO_MANY_PARAMETERS|ERR_FTP_CHANGEDIR|ERR_FTP_CLOSED|ERR_FTP_CONNECT_FAILED|ERR_FTP_ERROR|ERR_FTP_FILE_ERROR|ERR_FTP_NOLOGIN|ERR_FTP_NOSERVER|ERR_FTP_SEND_FAILED|ERR_FUNC_NOT_ALLOWED_IN_TESTING|ERR_FUNCTION_NOT_ALLOWED|ERR_FUNCTION_NOT_CONFIRMED|ERR_GLOBAL_VARIABLE_NOT_FOUND|ERR_GLOBAL_VARIABLES_PROCESSING|ERR_GLOBALVARIABLE_EXISTS|ERR_GLOBALVARIABLE_NOT_FOUND|ERR_HISTORY_BARS_LIMIT|ERR_HISTORY_LOAD_ERRORS|ERR_HISTORY_NOT_FOUND|ERR_HISTORY_TIMEOUT|ERR_HISTORY_WILL_UPDATED|ERR_HISTORY_WRONG_PROPERTY|ERR_INCOMPATIBLE_ARRAYS|ERR_INCOMPATIBLE_FILE|ERR_INCOMPATIBLE_FILEACCESS|ERR_INCORRECT_SERIESARRAY_USING|ERR_INDICATOR_CANNOT_ADD|ERR_INDICATOR_CANNOT_APPLY|ERR_INDICATOR_CANNOT_CREATE|ERR_INDICATOR_CANNOT_INIT|ERR_INDICATOR_CANNOT_LOAD|ERR_INDICATOR_CUSTOM_NAME|ERR_INDICATOR_DATA_NOT_FOUND|ERR_INDICATOR_NO_MEMORY|ERR_INDICATOR_PARAMETER_TYPE|ERR_INDICATOR_PARAMETERS_MISSING|ERR_INDICATOR_UNKNOWN_SYMBOL|ERR_INDICATOR_WRONG_HANDLE|ERR_INDICATOR_WRONG_INDEX|ERR_INDICATOR_WRONG_PARAMETERS|ERR_INT_ARRAY_ONLY|ERR_INTEGER_PARAMETER_EXPECTED|ERR_INTERNAL_ERROR|ERR_INVALID_ACCOUNT|ERR_INVALID_ARRAY|ERR_INVALID_DATETIME|ERR_INVALID_FILEHANDLE|ERR_INVALID_FUNCTION_PARAMSCNT|ERR_INVALID_FUNCTION_PARAMVALUE|ERR_INVALID_PARAMETER|ERR_INVALID_POINTER|ERR_INVALID_POINTER_TYPE|ERR_INVALID_PRICE|ERR_INVALID_PRICE_PARAM|ERR_INVALID_STOPS|ERR_INVALID_TICKET|ERR_INVALID_TRADE_PARAMETERS|ERR_INVALID_TRADE_VOLUME|ERR_LONG_ARRAY_ONLY|ERR_LONG_POSITIONS_ONLY_ALLOWED|ERR_LONGS_NOT_ALLOWED|ERR_MAIL_SEND_FAILED|ERR_MALFUNCTIONAL_TRADE|ERR_MARKET_CLOSED|ERR_MARKET_LASTTIME_UNKNOWN|ERR_MARKET_NOT_SELECTED|ERR_MARKET_SELECT_ERROR|ERR_MARKET_UNKNOWN_SYMBOL|ERR_MARKET_WRONG_PROPERTY|ERR_MQL5_WRONG_PROPERTY|ERR_NO_CONNECTION|ERR_NO_ERROR|ERR_NO_HISTORY_DATA|ERR_NO_MEMORY_FOR_ARRAYSTRING|ERR_NO_MEMORY_FOR_CALL_STACK|ERR_NO_MEMORY_FOR_HISTORY|ERR_NO_MEMORY_FOR_INDICATOR|ERR_NO_MEMORY_FOR_PARAM_STRING|ERR_NO_MEMORY_FOR_RETURNED_STR|ERR_NO_MEMORY_FOR_TEMP_STRING|ERR_NO_MQLERROR|ERR_NO_OBJECT_NAME|ERR_NO_ORDER_SELECTED|ERR_NO_RESULT|ERR_NO_SPECIFIED_SUBWINDOW|ERR_NO_STRING_DATE|ERR_NOT_ENOUGH_MEMORY|ERR_NOT_ENOUGH_MONEY|ERR_NOT_ENOUGH_RIGHTS|ERR_NOT_ENOUGH_STACK_FOR_PARAM|ERR_NOT_INITIALIZED_ARRAY|ERR_NOT_INITIALIZED_ARRAYSTRING|ERR_NOT_INITIALIZED_STRING|ERR_NOTIFICATION_ERROR|ERR_NOTIFICATION_PARAMETER|ERR_NOTIFICATION_SEND_FAILED|ERR_NOTIFICATION_SETTINGS|ERR_NOTIFICATION_TOO_FREQUENT|ERR_NOTIFICATION_WRONG_PARAMETER|ERR_NOTIFICATION_WRONG_SETTINGS|ERR_NOTINITIALIZED_STRING|ERR_NUMBER_ARRAYS_ONLY|ERR_OBJECT_ALREADY_EXISTS|ERR_OBJECT_COORDINATES_ERROR|ERR_OBJECT_DOES_NOT_EXIST|ERR_OBJECT_ERROR|ERR_OBJECT_GETDATE_FAILED|ERR_OBJECT_GETVALUE_FAILED|ERR_OBJECT_NOT_FOUND|ERR_OBJECT_WRONG_PROPERTY|ERR_OFF_QUOTES|ERR_OLD_VERSION|ERR_ONEDIM_ARRAYS_ONLY|ERR_OPENCL_BUFFER_CREATE|ERR_OPENCL_CONTEXT_CREATE|ERR_OPENCL_EXECUTE|ERR_OPENCL_INTERNAL|ERR_OPENCL_INVALID_HANDLE|ERR_OPENCL_KERNEL_CREATE|ERR_OPENCL_NOT_SUPPORTED|ERR_OPENCL_PROGRAM_CREATE|ERR_OPENCL_QUEUE_CREATE|ERR_OPENCL_SET_KERNEL_PARAMETER|ERR_OPENCL_TOO_LONG_KERNEL_NAME|ERR_OPENCL_WRONG_BUFFER_OFFSET|ERR_OPENCL_WRONG_BUFFER_SIZE|ERR_ORDER_LOCKED|ERR_OUT_OF_MEMORY|ERR_PLAY_SOUND_FAILED|ERR_PRICE_CHANGED|ERR_RECURSIVE_STACK_OVERFLOW|ERR_REMAINDER_FROM_ZERO_DIVIDE|ERR_REQUOTE|ERR_RESOURCE_DUPLICATED|ERR_RESOURCE_NAME_DUPLICATED|ERR_RESOURCE_NAME_IS_TOO_LONG|ERR_RESOURCE_NOT_FOUND|ERR_RESOURCE_NOT_SUPPORTED|ERR_RESOURCE_UNSUPPOTED_TYPE|ERR_SEND_MAIL_ERROR|ERR_SERIES_ARRAY|ERR_SERVER_BUSY|ERR_SHORT_ARRAY_ONLY|ERR_SHORTS_NOT_ALLOWED|ERR_SMALL_ARRAY|ERR_SMALL_ASSERIES_ARRAY|ERR_SOME_ARRAY_ERROR|ERR_SOME_FILE_ERROR|ERR_SOME_OBJECT_ERROR|ERR_STRING_ARRAY_ONLY|ERR_STRING_FUNCTION_INTERNAL|ERR_STRING_OUT_OF_MEMORY|ERR_STRING_PARAMETER_EXPECTED|ERR_STRING_RESIZE_ERROR|ERR_STRING_SMALL_LEN|ERR_STRING_TIME_ERROR|ERR_STRING_TOO_BIGNUMBER|ERR_STRING_UNKNOWNTYPE|ERR_STRING_ZEROADDED|ERR_STRINGPOS_OUTOFRANGE|ERR_STRUCT_WITHOBJECTS_ORCLASS|ERR_SUCCESS|ERR_SYMBOL_SELECT|ERR_SYSTEM_BUSY|ERR_TERMINAL_WRONG_PROPERTY|ERR_TOO_FREQUENT_REQUESTS|ERR_TOO_LONG_FILENAME|ERR_TOO_LONG_STRING|ERR_TOO_MANY_FILES|ERR_TOO_MANY_FORMATTERS|ERR_TOO_MANY_OPENED_FILES|ERR_TOO_MANY_PARAMETERS|ERR_TOO_MANY_REQUESTS|ERR_TRADE_CALC_FAILED|ERR_TRADE_CONTEXT_BUSY|ERR_TRADE_DEAL_NOT_FOUND|ERR_TRADE_DISABLED|ERR_TRADE_ERROR|ERR_TRADE_EXPERT_DISABLED_BY_SERVER|ERR_TRADE_EXPIRATION_DENIED|ERR_TRADE_HEDGE_PROHIBITED|ERR_TRADE_MODIFY_DENIED|ERR_TRADE_NOT_ALLOWED|ERR_TRADE_ORDER_NOT_FOUND|ERR_TRADE_POSITION_NOT_FOUND|ERR_TRADE_PROHIBITED_BY_FIFO|ERR_TRADE_SEND_FAILED|ERR_TRADE_TIMEOUT|ERR_TRADE_TOO_MANY_ORDERS|ERR_TRADE_WRONG_PROPERTY|ERR_UNKNOWN_COMMAND|ERR_UNKNOWN_OBJECT_PROPERTY|ERR_UNKNOWN_OBJECT_TYPE|ERR_UNKNOWN_SYMBOL|ERR_USER_ERROR_FIRST|ERR_USER_ERROR_LAST|ERR_WEBREQUEST_CONNECT_FAILED|ERR_WEBREQUEST_INVALID_ADDRESS|ERR_WEBREQUEST_REQUEST_FAILED|ERR_WEBREQUEST_TIMEOUT|ERR_WRONG_DIRECTORYNAME|ERR_WRONG_FILE_NAME|ERR_WRONG_FILEHANDLE|ERR_WRONG_FILENAME|ERR_WRONG_FORMATSTRING|ERR_WRONG_FUNCTION_POINTER|ERR_WRONG_INTERNAL_PARAMETER|ERR_WRONG_JUMP|ERR_WRONG_STRING_DATE|ERR_WRONG_STRING_OBJECT|ERR_WRONG_STRING_PARAMETER|ERR_WRONG_STRING_TIME|ERR_ZERO_DIVIDE|ERR_ZEROSIZE_ARRAY|false|False|FALSE|FILE_ACCESS_DATE|FILE_ANSI|FILE_BIN|FILE_COMMON|FILE_CREATE_DATE|FILE_CSV|FILE_END|FILE_EXISTS|FILE_IS_ANSI|FILE_IS_BINARY|FILE_IS_COMMON|FILE_IS_CSV|FILE_IS_READABLE|FILE_IS_TEXT|FILE_IS_WRITABLE|FILE_LINE_END|FILE_MODIFY_DATE|FILE_POSITION|FILE_READ|FILE_REWRITE|FILE_SHARE_READ|FILE_SHARE_WRITE|FILE_SIZE|FILE_TXT|FILE_UNICODE|FILE_WRITE|FLOAT_VALUE|FLT_DIG|FLT_EPSILON|FLT_MANT_DIG|FLT_MAX|FLT_MAX_10_EXP|FLT_MAX_EXP|FLT_MIN|FLT_MIN_10_EXP|FLT_MIN_EXP|FONT_ITALIC|FONT_STRIKEOUT|FONT_UNDERLINE|FRIDAY|FW_BLACK|FW_BOLD|FW_DEMIBOLD|FW_DONTCARE|FW_EXTRABOLD|FW_EXTRALIGHT|FW_HEAVY|FW_LIGHT|FW_MEDIUM|FW_NORMAL|FW_REGULAR|FW_SEMIBOLD|FW_THIN|FW_ULTRABOLD|FW_ULTRALIGHT|GANN_DOWN_TREND|GANN_UP_TREND|GATORJAW_LINE|GATORLIPS_LINE|GATORTEETH_LINE|icon|IDABORT|IDCANCEL|IDCLOSE|IDCONTINUE|IDHELP|IDIGNORE|IDNO|IDOK|IDRETRY|IDTRYAGAIN|IDYES|IND_AC|IND_AD|IND_ADX|IND_ADXW|IND_ALLIGATOR|IND_AMA|IND_AO|IND_ATR|IND_BANDS|IND_BEARS|IND_BULLS|IND_BWMFI|IND_CCI|IND_CHAIKIN|IND_CUSTOM|IND_DEMA|IND_DEMARKER|IND_ENVELOPES|IND_FORCE|IND_FRACTALS|IND_FRAMA|IND_GATOR|IND_ICHIMOKU|IND_MA|IND_MACD|IND_MFI|IND_MOMENTUM|IND_OBV|IND_OSMA|IND_RSI|IND_RVI|IND_SAR|IND_STDDEV|IND_STOCHASTIC|IND_TEMA|IND_TRIX|IND_VIDYA|IND_VOLUMES|IND_WPR|indicator_applied_price|indicator_buffers|INDICATOR_CALCULATIONS|indicator_chart_window|indicator_color|INDICATOR_COLOR_INDEX|INDICATOR_DATA|INDICATOR_DIGITS|indicator_height|INDICATOR_HEIGHT|indicator_label|indicator_level|indicator_levelcolor|INDICATOR_LEVELCOLOR|INDICATOR_LEVELS|indicator_levelstyle|INDICATOR_LEVELSTYLE|INDICATOR_LEVELTEXT|INDICATOR_LEVELVALUE|indicator_levelwidth|INDICATOR_LEVELWIDTH|indicator_maximum|INDICATOR_MAXIMUM|indicator_minimum|INDICATOR_MINIMUM|indicator_plots|indicator_separate_window|INDICATOR_SHORTNAME|indicator_style|indicator_type|indicator_width|INIT_AGENT_NOT_SUITABLE|INIT_FAILED|INIT_PARAMETERS_INCORRECT|INIT_SUCCEEDED|INT_MAX|INT_MIN|INT_VALUE|INVALID_HANDLE|IS_DEBUG_MODE|IS_PROFILE_MODE|KIJUNSEN_LINE|library|LICENSE_DEMO|LICENSE_FREE|LICENSE_FULL|LICENSE_TIME|link|LONG_MAX|LONG_MIN|LONG_VALUE|LOWER_BAND|LOWER_HISTOGRAM|LOWER_LINE|M_1_PI|M_2_PI|M_2_SQRTPI|M_E|M_LN10|M_LN2|M_LOG10E|M_LOG2E|M_PI|M_PI_2|M_PI_4|M_SQRT1_2|M_SQRT2|MAIN_LINE|MB_ABORTRETRYIGNORE|MB_APPLMODAL|MB_CANCELTRYCONTINUE|MB_DEFAULT_DESKTOP_ONLY|MB_DEFBUTTON1|MB_DEFBUTTON2|MB_DEFBUTTON3|MB_DEFBUTTON4|MB_HELP|MB_ICONASTERISK|MB_ICONERROR|MB_ICONEXCLAMATION|MB_ICONHAND|MB_ICONINFORMATION|MB_ICONQUESTION|MB_ICONSTOP|MB_ICONWARNING|MB_NOFOCUS|MB_OK|MB_OKCANCEL|MB_RETRYCANCEL|MB_RIGHT|MB_RTLREADING|MB_SETFOREGROUND|MB_SYSTEMMODAL|MB_TASKMODAL|MB_TOPMOST|MB_USERICON|MB_YESNO|MB_YESNOCANCEL|MINUSDI_LINE|MODE_ASCEND|MODE_ASK|MODE_BASE|MODE_BID|MODE_CHIKOUSPAN|MODE_CLOSE|MODE_CLOSEBY_ALLOWED|MODE_DESCEND|MODE_DIGITS|MODE_EMA|MODE_EXPIRATION|MODE_FREEZELEVEL|MODE_GATORJAW|MODE_GATORLIPS|MODE_GATORTEETH|MODE_HIGH|MODE_HISTORY|MODE_KIJUNSEN|MODE_LOTSIZE|MODE_LOTSTEP|MODE_LOW|MODE_LOWER|MODE_LWMA|MODE_MAIN|MODE_MARGINCALCMODE|MODE_MARGINHEDGED|MODE_MARGININIT|MODE_MARGINMAINTENANCE|MODE_MARGINREQUIRED|MODE_MAXLOT|MODE_MINLOT|MODE_MINUSDI|MODE_OPEN|MODE_PLUSDI|MODE_POINT|MODE_PROFITCALCMODE|MODE_SENKOUSPANA|MODE_SENKOUSPANB|MODE_SIGNAL|MODE_SMA|MODE_SMMA|MODE_SPREAD|MODE_STARTING|MODE_STOPLEVEL|MODE_SWAPLONG|MODE_SWAPSHORT|MODE_SWAPTYPE|MODE_TENKANSEN|MODE_TICKSIZE|MODE_TICKVALUE|MODE_TIME|MODE_TRADEALLOWED|MODE_TRADES|MODE_UPPER|MODE_VOLUME|MONDAY|MQL5_DEBUG|MQL5_DEBUGGING|MQL5_DLLS_ALLOWED|MQL5_FRAME_MODE|MQL5_LICENSE_TYPE|MQL5_MEMORY_LIMIT|MQL5_MEMORY_USED|MQL5_OPTIMIZATION|MQL5_PROFILER|MQL5_PROGRAM_NAME|MQL5_PROGRAM_PATH|MQL5_PROGRAM_TYPE|MQL5_SIGNALS_ALLOWED|MQL5_TESTER|MQL5_TESTING|MQL5_TRADE_ALLOWED|MQL5_VISUAL_MODE|MQL_CODEPAGE|MQL_DEBUG|MQL_DLLS_ALLOWED|MQL_FRAME_MODE|MQL_LICENSE_TYPE|MQL_MEMORY_LIMIT|MQL_MEMORY_USED|MQL_OPTIMIZATION|MQL_PROFILER|MQL_PROGRAM_NAME|MQL_PROGRAM_PATH|MQL_PROGRAM_TYPE|MQL_SIGNALS_ALLOWED|MQL_TESTER|MQL_TRADE_ALLOWED|MQL_VISUAL_MODE|NULL|OBJ_ALL_PERIODS|OBJ_ARROW|OBJ_ARROW_BUY|OBJ_ARROW_CHECK|OBJ_ARROW_DOWN|OBJ_ARROW_LEFT_PRICE|OBJ_ARROW_RIGHT_PRICE|OBJ_ARROW_SELL|OBJ_ARROW_STOP|OBJ_ARROW_THUMB_DOWN|OBJ_ARROW_THUMB_UP|OBJ_ARROW_UP|OBJ_ARROWED_LINE|OBJ_BITMAP|OBJ_BITMAP_LABEL|OBJ_BUTTON|OBJ_CHANNEL|OBJ_CHART|OBJ_CYCLES|OBJ_EDIT|OBJ_ELLIOTWAVE3|OBJ_ELLIOTWAVE5|OBJ_ELLIPSE|OBJ_EVENT|OBJ_EXPANSION|OBJ_FIBO|OBJ_FIBOARC|OBJ_FIBOCHANNEL|OBJ_FIBOFAN|OBJ_FIBOTIMES|OBJ_GANNFAN|OBJ_GANNGRID|OBJ_GANNLINE|OBJ_HLINE|OBJ_LABEL|OBJ_NO_PERIODS|OBJ_PERIOD_D1|OBJ_PERIOD_H1|OBJ_PERIOD_H12|OBJ_PERIOD_H2|OBJ_PERIOD_H3|OBJ_PERIOD_H4|OBJ_PERIOD_H6|OBJ_PERIOD_H8|OBJ_PERIOD_M1|OBJ_PERIOD_M10|OBJ_PERIOD_M12|OBJ_PERIOD_M15|OBJ_PERIOD_M2|OBJ_PERIOD_M20|OBJ_PERIOD_M3|OBJ_PERIOD_M30|OBJ_PERIOD_M4)\\b',
      name: 'support.constant.mql5'
    },
    {
      match:
        '\\b(OBJ_PERIOD_M5|OBJ_PERIOD_M6|OBJ_PERIOD_MN1|OBJ_PERIOD_W1|OBJ_PITCHFORK|OBJ_RECTANGLE|OBJ_RECTANGLE_LABEL|OBJ_REGRESSION|OBJ_STDDEVCHANNEL|OBJ_TEXT|OBJ_TREND|OBJ_TRENDBYANGLE|OBJ_TRIANGLE|OBJ_VLINE|OBJPROP_ALIGN|OBJPROP_ANCHOR|OBJPROP_ANGLE|OBJPROP_ARROWCODE|OBJPROP_BACK|OBJPROP_BGCOLOR|OBJPROP_BMPFILE|OBJPROP_BORDER_COLOR|OBJPROP_BORDER_TYPE|OBJPROP_CHART_ID|OBJPROP_CHART_SCALE|OBJPROP_COLOR|OBJPROP_CORNER|OBJPROP_CREATETIME|OBJPROP_DATE_SCALE|OBJPROP_DEGREE|OBJPROP_DEVIATION|OBJPROP_DIRECTION|OBJPROP_DRAWLINES|OBJPROP_ELLIPSE|OBJPROP_FIBOLEVELS|OBJPROP_FILL|OBJPROP_FIRSTLEVEL|OBJPROP_FONT|OBJPROP_FONTSIZE|OBJPROP_HIDDEN|OBJPROP_LEVELCOLOR|OBJPROP_LEVELS|OBJPROP_LEVELSTYLE|OBJPROP_LEVELTEXT|OBJPROP_LEVELVALUE|OBJPROP_LEVELWIDTH|OBJPROP_NAME|OBJPROP_PERIOD|OBJPROP_PRICE|OBJPROP_PRICE1|OBJPROP_PRICE2|OBJPROP_PRICE3|OBJPROP_PRICE_SCALE|OBJPROP_RAY|OBJPROP_RAY_LEFT|OBJPROP_RAY_RIGHT|OBJPROP_READONLY|OBJPROP_SCALE|OBJPROP_SELECTABLE|OBJPROP_SELECTED|OBJPROP_STATE|OBJPROP_STYLE|OBJPROP_SYMBOL|OBJPROP_TEXT|OBJPROP_TIME|OBJPROP_TIME1|OBJPROP_TIME2|OBJPROP_TIME3|OBJPROP_TIMEFRAMES|OBJPROP_TOOLTIP|OBJPROP_TYPE|OBJPROP_WIDTH|OBJPROP_XDISTANCE|OBJPROP_XOFFSET|OBJPROP_XSIZE|OBJPROP_YDISTANCE|OBJPROP_YOFFSET|OBJPROP_YSIZE|OBJPROP_ZORDER|OP_BUY|OP_BUYLIMIT|OP_BUYSTOP|OP_SELL|OP_SELLLIMIT|OP_SELLSTOP|OPENCL_BUFFER|OPENCL_CONTEXT|OPENCL_INVALID|OPENCL_KERNEL|OPENCL_PROGRAM|ORDER_COMMENT|ORDER_EXTERNAL_ID|ORDER_FILLING_FOK|ORDER_FILLING_IOC|ORDER_FILLING_RETURN|ORDER_MAGIC|ORDER_POSITION_BY_ID|ORDER_POSITION_ID|ORDER_PRICE_CURRENT|ORDER_PRICE_OPEN|ORDER_PRICE_STOPLIMIT|ORDER_SL|ORDER_STATE|ORDER_STATE_CANCELED|ORDER_STATE_EXPIRED|ORDER_STATE_FILLED|ORDER_STATE_PARTIAL|ORDER_STATE_PLACED|ORDER_STATE_REJECTED|ORDER_STATE_REQUEST_ADD|ORDER_STATE_REQUEST_CANCEL|ORDER_STATE_REQUEST_MODIFY|ORDER_STATE_STARTED|ORDER_SYMBOL|ORDER_TICKET|ORDER_TIME_DAY|ORDER_TIME_DONE|ORDER_TIME_DONE_MSC|ORDER_TIME_EXPIRATION|ORDER_TIME_GTC|ORDER_TIME_SETUP|ORDER_TIME_SETUP_MSC|ORDER_TIME_SPECIFIED|ORDER_TIME_SPECIFIED_DAY|ORDER_TP|ORDER_TYPE|ORDER_TYPE_BALANCE|ORDER_TYPE_BUY|ORDER_TYPE_BUY_LIMIT|ORDER_TYPE_BUY_STOP|ORDER_TYPE_BUY_STOP_LIMIT|ORDER_TYPE_CLOSE_BY|ORDER_TYPE_CREDIT|ORDER_TYPE_FILLING|ORDER_TYPE_SELL|ORDER_TYPE_SELL_LIMIT|ORDER_TYPE_SELL_STOP|ORDER_TYPE_SELL_STOP_LIMIT|ORDER_TYPE_TIME|ORDER_VOLUME_CURRENT|ORDER_VOLUME_INITIAL|PERIOD_CURRENT|PERIOD_D1|PERIOD_H1|PERIOD_H12|PERIOD_H2|PERIOD_H3|PERIOD_H4|PERIOD_H6|PERIOD_H8|PERIOD_M1|PERIOD_M10|PERIOD_M12|PERIOD_M15|PERIOD_M2|PERIOD_M20|PERIOD_M3|PERIOD_M30|PERIOD_M4|PERIOD_M5|PERIOD_M6|PERIOD_MN1|PERIOD_W1|PLOT_ARROW|PLOT_ARROW_SHIFT|PLOT_COLOR_INDEXES|PLOT_DRAW_BEGIN|PLOT_DRAW_TYPE|PLOT_EMPTY_VALUE|PLOT_LABEL|PLOT_LINE_COLOR|PLOT_LINE_STYLE|PLOT_LINE_WIDTH|PLOT_SHIFT|PLOT_SHOW_DATA|PLUSDI_LINE|POINTER_AUTOMATIC|POINTER_DYNAMIC|POINTER_INVALID|POSITION_COMMENT|POSITION_COMMISSION|POSITION_IDENTIFIER|POSITION_MAGIC|POSITION_PRICE_CURRENT|POSITION_PRICE_OPEN|POSITION_PROFIT|POSITION_SL|POSITION_SWAP|POSITION_SYMBOL|POSITION_TICKET|POSITION_TIME|POSITION_TIME_MSC|POSITION_TIME_UPDATE|POSITION_TIME_UPDATE_MSC|POSITION_TP|POSITION_TYPE|POSITION_TYPE_BUY|POSITION_TYPE_SELL|POSITION_VOLUME|PRICE_CLOSE|PRICE_HIGH|PRICE_LOW|PRICE_MEDIAN|PRICE_OPEN|PRICE_TYPICAL|PRICE_WEIGHTED|PROGRAM_EXPERT|PROGRAM_INDICATOR|PROGRAM_SCRIPT|REASON_ACCOUNT|REASON_CHARTCHANGE|REASON_CHARTCLOSE|REASON_CLOSE|REASON_INITFAILED|REASON_PARAMETERS|REASON_PROGRAM|REASON_RECOMPILE|REASON_REMOVE|REASON_TEMPLATE|SATURDAY|script_show_confirm|script_show_inputs|SEEK_CUR|SEEK_END|SEEK_SET|SELECT_BY_POS|SELECT_BY_TICKET|SENKOUSPANA_LINE|SENKOUSPANB_LINE|SERIES_BARS_COUNT|SERIES_FIRSTDATE|SERIES_LASTBAR_DATE|SERIES_SERVER_FIRSTDATE|SERIES_SYNCHRONIZED|SERIES_TERMINAL_FIRSTDATE|SHORT_MAX|SHORT_MIN|SHORT_VALUE|show_confirm|show_inputs|SIGNAL_BASE_AUTHOR_LOGIN|SIGNAL_BASE_BALANCE|SIGNAL_BASE_BROKER|SIGNAL_BASE_BROKER_SERVER|SIGNAL_BASE_CURRENCY|SIGNAL_BASE_DATE_PUBLISHED|SIGNAL_BASE_DATE_STARTED|SIGNAL_BASE_DATE_UPDATED|SIGNAL_BASE_EQUITY|SIGNAL_BASE_GAIN|SIGNAL_BASE_ID|SIGNAL_BASE_LEVERAGE|SIGNAL_BASE_MAX_DRAWDOWN|SIGNAL_BASE_NAME|SIGNAL_BASE_PIPS|SIGNAL_BASE_PRICE|SIGNAL_BASE_RATING|SIGNAL_BASE_ROI|SIGNAL_BASE_SUBSCRIBERS|SIGNAL_BASE_TRADE_MODE|SIGNAL_BASE_TRADES|SIGNAL_INFO_CONFIRMATIONS_DISABLED|SIGNAL_INFO_COPY_SLTP|SIGNAL_INFO_DEPOSIT_PERCENT|SIGNAL_INFO_EQUITY_LIMIT|SIGNAL_INFO_ID|SIGNAL_INFO_NAME|SIGNAL_INFO_SLIPPAGE|SIGNAL_INFO_SUBSCRIPTION_ENABLED|SIGNAL_INFO_TERMS_AGREE|SIGNAL_INFO_VOLUME_PERCENT|SIGNAL_LINE|stacksize|STAT_BALANCE_DD|STAT_BALANCE_DD_RELATIVE|STAT_BALANCE_DDREL_PERCENT|STAT_BALANCEDD_PERCENT|STAT_BALANCEMIN|STAT_CONLOSSMAX|STAT_CONLOSSMAX_TRADES|STAT_CONPROFITMAX|STAT_CONPROFITMAX_TRADES|STAT_CUSTOM_ONTESTER|STAT_DEALS|STAT_EQUITY_DD|STAT_EQUITY_DD_RELATIVE|STAT_EQUITY_DDREL_PERCENT|STAT_EQUITYDD_PERCENT|STAT_EQUITYMIN|STAT_EXPECTED_PAYOFF|STAT_GROSS_LOSS|STAT_GROSS_PROFIT|STAT_INITIAL_DEPOSIT|STAT_LONG_TRADES|STAT_LOSS_TRADES|STAT_LOSSTRADES_AVGCON|STAT_MAX_CONLOSS_TRADES|STAT_MAX_CONLOSSES|STAT_MAX_CONPROFIT_TRADES|STAT_MAX_CONWINS|STAT_MAX_LOSSTRADE|STAT_MAX_PROFITTRADE|STAT_MIN_MARGINLEVEL|STAT_PROFIT|STAT_PROFIT_FACTOR|STAT_PROFIT_LONGTRADES|STAT_PROFIT_SHORTTRADES|STAT_PROFIT_TRADES|STAT_PROFITTRADES_AVGCON|STAT_RECOVERY_FACTOR|STAT_SHARPE_RATIO|STAT_SHORT_TRADES|STAT_TRADES|STAT_WITHDRAWAL|STO_CLOSECLOSE|STO_LOWHIGH|strict|STYLE_DASH|STYLE_DASHDOT|STYLE_DASHDOTDOT|STYLE_DOT|STYLE_SOLID|SUNDAY|SYMBOL_ARROWDOWN|SYMBOL_ARROWUP|SYMBOL_ASK|SYMBOL_ASKHIGH|SYMBOL_ASKLOW|SYMBOL_BANK|SYMBOL_BASIS|SYMBOL_BID|SYMBOL_BIDHIGH|SYMBOL_BIDLOW|SYMBOL_CALC_MODE_CFD|SYMBOL_CALC_MODE_CFDINDEX|SYMBOL_CALC_MODE_CFDLEVERAGE|SYMBOL_CALC_MODE_EXCH_FUTURES|SYMBOL_CALC_MODE_EXCH_FUTURES_FORTS|SYMBOL_CALC_MODE_EXCH_OPTIONS|SYMBOL_CALC_MODE_EXCH_OPTIONS_MARGIN|SYMBOL_CALC_MODE_EXCH_STOCKS|SYMBOL_CALC_MODE_FOREX|SYMBOL_CALC_MODE_FUTURES|SYMBOL_CALC_MODE_SERV_COLLATERAL|SYMBOL_CHECKSIGN|SYMBOL_CURRENCY_BASE|SYMBOL_CURRENCY_MARGIN|SYMBOL_CURRENCY_PROFIT|SYMBOL_DESCRIPTION|SYMBOL_DIGITS|SYMBOL_EXPIRATION_DAY|SYMBOL_EXPIRATION_GTC|SYMBOL_EXPIRATION_MODE|SYMBOL_EXPIRATION_SPECIFIED|SYMBOL_EXPIRATION_SPECIFIED_DAY|SYMBOL_EXPIRATION_TIME|SYMBOL_FILLING_FOK|SYMBOL_FILLING_IOC|SYMBOL_FILLING_MODE|SYMBOL_ISIN|SYMBOL_LAST|SYMBOL_LASTHIGH|SYMBOL_LASTLOW|SYMBOL_LEFTPRICE|SYMBOL_MARGIN_HEDGED|SYMBOL_MARGIN_INITIAL|SYMBOL_MARGIN_LIMIT|SYMBOL_MARGIN_LONG|SYMBOL_MARGIN_MAINTENANCE|SYMBOL_MARGIN_SHORT|SYMBOL_MARGIN_STOP|SYMBOL_MARGIN_STOPLIMIT|SYMBOL_OPTION_MODE|SYMBOL_OPTION_MODE_AMERICAN|SYMBOL_OPTION_MODE_EUROPEAN|SYMBOL_OPTION_RIGHT|SYMBOL_OPTION_RIGHT_CALL|SYMBOL_OPTION_RIGHT_PUT|SYMBOL_OPTION_STRIKE|SYMBOL_ORDER_LIMIT|SYMBOL_ORDER_MARKET|SYMBOL_ORDER_MODE|SYMBOL_ORDER_SL|SYMBOL_ORDER_STOP|SYMBOL_ORDER_STOP_LIMIT|SYMBOL_ORDER_TP|SYMBOL_PATH|SYMBOL_POINT|SYMBOL_RIGHTPRICE|SYMBOL_SELECT|SYMBOL_SESSION_AW|SYMBOL_SESSION_BUY_ORDERS|SYMBOL_SESSION_BUY_ORDERS_VOLUME|SYMBOL_SESSION_CLOSE|SYMBOL_SESSION_DEALS|SYMBOL_SESSION_INTEREST|SYMBOL_SESSION_OPEN|SYMBOL_SESSION_PRICE_LIMIT_MAX|SYMBOL_SESSION_PRICE_LIMIT_MIN|SYMBOL_SESSION_PRICE_SETTLEMENT|SYMBOL_SESSION_SELL_ORDERS|SYMBOL_SESSION_SELL_ORDERS_VOLUME|SYMBOL_SESSION_TURNOVER|SYMBOL_SESSION_VOLUME|SYMBOL_SPREAD|SYMBOL_SPREAD_FLOAT|SYMBOL_START_TIME|SYMBOL_STOPSIGN|SYMBOL_SWAP_LONG|SYMBOL_SWAP_MODE|SYMBOL_SWAP_MODE_BY_INTEREST|SYMBOL_SWAP_MODE_BY_MARGIN_CURRENCY|SYMBOL_SWAP_MODE_BY_MONEY|SYMBOL_SWAP_MODE_BY_POINTS|SYMBOL_SWAP_MODE_CURRENCY_DEPOSIT|SYMBOL_SWAP_MODE_CURRENCY_MARGIN|SYMBOL_SWAP_MODE_CURRENCY_SYMBOL|SYMBOL_SWAP_MODE_DISABLED|SYMBOL_SWAP_MODE_INTEREST_CURRENT|SYMBOL_SWAP_MODE_INTEREST_OPEN|SYMBOL_SWAP_MODE_POINTS|SYMBOL_SWAP_MODE_REOPEN_BID|SYMBOL_SWAP_MODE_REOPEN_CURRENT|SYMBOL_SWAP_ROLLOVER3DAYS|SYMBOL_SWAP_SHORT|SYMBOL_THUMBSDOWN|SYMBOL_THUMBSUP|SYMBOL_TICKS_BOOKDEPTH|SYMBOL_TIME|SYMBOL_TRADE_CALC_MODE|SYMBOL_TRADE_CONTRACT_SIZE|SYMBOL_TRADE_EXECUTION_EXCHANGE|SYMBOL_TRADE_EXECUTION_INSTANT|SYMBOL_TRADE_EXECUTION_MARKET|SYMBOL_TRADE_EXECUTION_REQUEST|SYMBOL_TRADE_EXEMODE|SYMBOL_TRADE_FREEZE_LEVEL|SYMBOL_TRADE_MODE|SYMBOL_TRADE_MODE_CLOSEONLY|SYMBOL_TRADE_MODE_DISABLED|SYMBOL_TRADE_MODE_FULL|SYMBOL_TRADE_MODE_LONGONLY|SYMBOL_TRADE_MODE_SHORTONLY|SYMBOL_TRADE_STOPS_LEVEL|SYMBOL_TRADE_TICK_SIZE|SYMBOL_TRADE_TICK_VALUE|SYMBOL_TRADE_TICK_VALUE_LOSS|SYMBOL_TRADE_TICK_VALUE_PROFIT|SYMBOL_VISIBLE|SYMBOL_VOLUME|SYMBOL_VOLUME_LIMIT|SYMBOL_VOLUME_MAX|SYMBOL_VOLUME_MIN|SYMBOL_VOLUME_STEP|SYMBOL_VOLUMEHIGH|SYMBOL_VOLUMELOW|TA_BOTTOM|TA_CENTER|TA_LEFT|TA_RIGHT|TA_TOP|TA_VCENTER|TENKANSEN_LINE|TERMINAL_BUILD|TERMINAL_CODEPAGE|TERMINAL_COMMONDATA_PATH|TERMINAL_COMMUNITY_ACCOUNT|TERMINAL_COMMUNITY_BALANCE|TERMINAL_COMMUNITY_CONNECTION|TERMINAL_COMPANY|TERMINAL_CONNECTED|TERMINAL_CPU_CORES|TERMINAL_DATA_PATH|TERMINAL_DISK_SPACE|TERMINAL_DLLS_ALLOWED|TERMINAL_EMAIL_ENABLED|TERMINAL_FTP_ENABLED|TERMINAL_LANGUAGE|TERMINAL_MAXBARS|TERMINAL_MEMORY_AVAILABLE|TERMINAL_MEMORY_PHYSICAL|TERMINAL_MEMORY_TOTAL|TERMINAL_MEMORY_USED|TERMINAL_MQID|TERMINAL_NAME|TERMINAL_NOTIFICATIONS_ENABLED|TERMINAL_OPENCL_SUPPORT|TERMINAL_PATH|TERMINAL_PING_LAST|TERMINAL_SCREEN_DPI|TERMINAL_TRADE_ALLOWED|TERMINAL_X64|tester_file|tester_indicator|tester_library|THURSDAY|TICK_FLAG_ASK|TICK_FLAG_BID|TICK_FLAG_BUY|TICK_FLAG_LAST|TICK_FLAG_SELL|TICK_FLAG_VOLUME|TIME_DATE|TIME_MINUTES|TIME_SECONDS|TRADE_ACTION_CLOSE_BY|TRADE_ACTION_DEAL|TRADE_ACTION_MODIFY|TRADE_ACTION_PENDING|TRADE_ACTION_REMOVE|TRADE_ACTION_SLTP|TRADE_RETCODE_CANCEL|TRADE_RETCODE_CLIENT_DISABLES_AT|TRADE_RETCODE_CONNECTION|TRADE_RETCODE_DONE|TRADE_RETCODE_DONE_PARTIAL|TRADE_RETCODE_ERROR|TRADE_RETCODE_FROZEN|TRADE_RETCODE_INVALID|TRADE_RETCODE_INVALID_EXPIRATION|TRADE_RETCODE_INVALID_FILL|TRADE_RETCODE_INVALID_ORDER|TRADE_RETCODE_INVALID_PRICE|TRADE_RETCODE_INVALID_STOPS|TRADE_RETCODE_INVALID_VOLUME|TRADE_RETCODE_LIMIT_ORDERS|TRADE_RETCODE_LIMIT_VOLUME|TRADE_RETCODE_LOCKED|TRADE_RETCODE_MARKET_CLOSED|TRADE_RETCODE_NO_CHANGES|TRADE_RETCODE_NO_MONEY|TRADE_RETCODE_ONLY_REAL|TRADE_RETCODE_ORDER_CHANGED|TRADE_RETCODE_PLACED|TRADE_RETCODE_POSITION_CLOSED|TRADE_RETCODE_PRICE_CHANGED|TRADE_RETCODE_PRICE_OFF|TRADE_RETCODE_REJECT|TRADE_RETCODE_REQUOTE|TRADE_RETCODE_SERVER_DISABLES_AT|TRADE_RETCODE_TIMEOUT|TRADE_RETCODE_TOO_MANY_REQUESTS|TRADE_RETCODE_TRADE_DISABLED|TRADE_TRANSACTION_DEAL_ADD|TRADE_TRANSACTION_DEAL_DELETE|TRADE_TRANSACTION_DEAL_UPDATE|TRADE_TRANSACTION_HISTORY_ADD|TRADE_TRANSACTION_HISTORY_DELETE|TRADE_TRANSACTION_HISTORY_UPDATE|TRADE_TRANSACTION_ORDER_ADD|TRADE_TRANSACTION_ORDER_DELETE|TRADE_TRANSACTION_ORDER_UPDATE|TRADE_TRANSACTION_POSITION|TRADE_TRANSACTION_REQUEST|true|True|TRUE|TUESDAY|TYPE_BOOL|TYPE_CHAR|TYPE_COLOR|TYPE_DATETIME|TYPE_DOUBLE|TYPE_FLOAT|TYPE_INT|TYPE_LONG|TYPE_SHORT|TYPE_STRING|TYPE_UCHAR|TYPE_UINT|TYPE_ULONG|TYPE_USHORT|UCHAR_MAX|UINT_MAX|ULONG_MAX|UPPER_BAND|UPPER_HISTOGRAM|UPPER_LINE|USHORT_MAX|version|VOLUME_REAL|VOLUME_TICK|WEDNESDAY|WHOLE_ARRAY|WRONG_VALUE)\\b',
      name: 'support.constant.mql5'
    },
    {
      match: '\\b([0-9]*\\.?[0-9]+([eE][-+]?[0-9]+)?)\\b',
      name: 'constant.numeric.decimal.mql5'
    },
    {match: '\\b(0[xX][0-9a-fA-F]+)\\b', name: 'constant.numeric.hex.mql5'},
    {
      match: "([CDB]\\'[0-9a-fA-FxX\\.\\:\\,\\s]*\\')",
      name: 'constant.numeric.other.mql5'
    },
    {match: '\\b([a-zA-Z_]+[a-zA-Z0-9_]*)\\b', name: 'variable.other.mql5'},
    {
      begin: '(\\{)',
      beginCaptures: {0: {name: 'punctuation.definition.block.begin.mql5'}},
      end: '(\\})',
      endCaptures: {0: {name: 'punctuation.definition.block.end.mql5'}},
      name: 'meta.block.mql5',
      patterns: [{include: '$self'}]
    }
  ],
  repository: {
    comments: {
      patterns: [
        {begin: '/\\*', end: '\\*/', name: 'comment.block.mql5'},
        {begin: '//', end: '$\\n?', name: 'comment.line.double-slash.mql5'}
      ]
    },
    escape_characters: {
      patterns: [{match: '\\\\.', name: 'constant.character.escape.mql5'}]
    }
  },
  scopeName: 'source.mql5'
}

export default grammar
